namespace birch {
#line 59 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
#line 59 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
libbirch::DefaultArray<Integer,1> simulate_dirichlet_multinomial(const Integer& n, const libbirch::DefaultArray<Real,1>& \u03b1) {
  #line 60 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  return simulate_multinomial(n, simulate_dirichlet(\u03b1));
}

#line 72 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
#line 72 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
Real logpdf_dirichlet_multinomial(const libbirch::DefaultArray<Integer,1>& x, const Integer& n, const libbirch::DefaultArray<Real,1>& \u03b1) {
  #line 74 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  assert(length(x) == length(\u03b1));
  auto A = sum(\u03b1);
  auto m = Integer(0);
  auto w = lgamma(n + 1.0) + lgamma(A) - lgamma(n + A);
  #line 78 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  for (auto i = Integer(1); i <= length(\u03b1); ++i) {
    #line 79 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    assert(x(i) >= Integer(0));
    #line 80 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    m = m + x(i);
    #line 81 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    w = w + lgamma(x(i) + \u03b1(i)) - lgamma(x(i) + 1.0) - lgamma(\u03b1(i));
  }
  #line 83 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  if (m == n) {
    #line 84 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    return w;
  } else {
    #line 86 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    return -(std::numeric_limits<double>::infinity());
  }
}

#line 100 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
#line 100 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
DirichletDistribution<libbirch::DefaultArray<Real,1>> update_dirichlet_multinomial(const libbirch::DefaultArray<Integer,1>& x, const Integer& n, const libbirch::DefaultArray<Real,1>& \u03b1) {
  #line 102 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  assert(sum(x) == n);
  #line 103 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  return Dirichlet(\u03b1 + x);
}

}
namespace birch {
#line 54 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
#line 54 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
libbirch::DefaultArray<Integer,1> simulate_multinomial(const Integer& n, const libbirch::DefaultArray<Real,1>& \u03c1) {
  #line 55 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  return simulate_multinomial(n, \u03c1, 1.0);
}

#line 71 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
#line 71 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
libbirch::DefaultArray<Integer,1> simulate_multinomial(const Integer& n, const libbirch::DefaultArray<Real,1>& \u03c1, const Real& Z) {
  #line 72 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  assert(length(\u03c1) > Integer(0));
  #line 73 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  assert(abs(sum(\u03c1) - Z) < 1.0e-6);
  auto D = length(\u03c1);
  auto R = \u03c1(D);
  auto lnMax = 0.0;
  auto j = D;
  auto i = n;
  auto x = vector(Integer(0), D);
  #line 82 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  while (i > Integer(0)) {
    auto u = simulate_uniform(0.0, 1.0);
    #line 84 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    lnMax = lnMax + log(u) / i;
    #line 85 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    u = Z * exp(lnMax);
    #line 86 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    while (u < Z - R) {
      #line 87 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
      j = j - Integer(1);
      #line 88 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
      R = R + \u03c1(j);
    }
    #line 90 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    x(j) = x(j) + Integer(1);
    #line 91 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    i = i - Integer(1);
  }
  #line 93 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  while (j > Integer(1)) {
    #line 94 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    j = j - Integer(1);
    #line 95 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    x(j) = Integer(0);
  }
  #line 97 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  return x;
}

#line 109 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
#line 109 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
Real logpdf_multinomial(const libbirch::DefaultArray<Integer,1>& x, const Integer& n, const libbirch::DefaultArray<Real,1>& \u03c1) {
  #line 110 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  assert(length(x) == length(\u03c1));
  auto m = Integer(0);
  auto w = lgamma(n + 1.0);
  #line 113 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  for (auto i = Integer(1); i <= length(x); ++i) {
    #line 114 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    assert(x(i) >= Integer(0));
    #line 115 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    assert(\u03c1(i) >= 0.0);
    #line 116 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    m = m + x(i);
    #line 117 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    w = w + x(i) * log(\u03c1(i)) - lgamma(x(i) + 1.0);
  }
  #line 119 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  if (m == n) {
    #line 120 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    return w;
  } else {
    #line 122 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    return -(std::numeric_limits<double>::infinity());
  }
}

}
