namespace birch {
#line 88 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 88 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
Integer simulate_beta_binomial(const Integer& n, const Real& \u03b1, const Real& \u03b2) {
  #line 89 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  assert(Integer(0) <= n);
  #line 90 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  assert(0.0 < \u03b1);
  #line 91 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  assert(0.0 < \u03b2);
  #line 92 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return simulate_binomial(n, simulate_beta(\u03b1, \u03b2));
}

#line 135 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 135 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
Real cdf_beta_binomial(const Integer& x, const Integer& n, const Real& \u03b1, const Real& \u03b2) {
  auto P = 0.0;
  #line 137 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  for (auto i = Integer(0); i <= min(n, x); ++i) {
    #line 138 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
    P = P + exp(logpdf_beta_binomial(i, n, \u03b1, \u03b2));
  }
  #line 140 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return P;
}

}
namespace birch {
#line 80 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
#line 80 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
Integer simulate_beta_negative_binomial(const Integer& k, const Real& \u03b1, const Real& \u03b2) {
  #line 81 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  assert(0.0 < \u03b1);
  #line 82 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  assert(0.0 < \u03b2);
  #line 83 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  assert(Integer(0) < k);
  #line 84 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return simulate_negative_binomial(k, simulate_beta(\u03b1, \u03b2));
}

}
#line 1 "src/distribution/scalar_discrete/BinomialDistribution.birch"

#include <boost/math/distributions/binomial.hpp>
namespace birch {
#line 76 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 76 "src/distribution/scalar_discrete/BinomialDistribution.birch"
Integer simulate_binomial(const Integer& n, const Real& \u03c1) {
  #line 77 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  assert(Integer(0) <= n);
  #line 78 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  assert(0.0 <= \u03c1 && \u03c1 <= 1.0);
  #line 79 "src/distribution/scalar_discrete/BinomialDistribution.birch"

  return std::binomial_distribution<Integer>(n, \u03c1)(rng);
  }

#line 106 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 106 "src/distribution/scalar_discrete/BinomialDistribution.birch"
Real cdf_binomial(const Integer& x, const Integer& n, const Real& \u03c1) {
  #line 107 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  assert(Integer(0) <= n);
  #line 108 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  assert(0.0 <= \u03c1 && \u03c1 <= 1.0);
  #line 109 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  if (x < Integer(0)) {
    #line 110 "src/distribution/scalar_discrete/BinomialDistribution.birch"
    return 0.0;
  } else {
    #line 111 "src/distribution/scalar_discrete/BinomialDistribution.birch"
    if (x >= n) {
      #line 112 "src/distribution/scalar_discrete/BinomialDistribution.birch"
      return 1.0;
    } else {
      #line 114 "src/distribution/scalar_discrete/BinomialDistribution.birch"
      return ibeta(n - x, x + 1.0, 1.0 - \u03c1);
    }
  }
}

#line 127 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 127 "src/distribution/scalar_discrete/BinomialDistribution.birch"
Integer quantile_binomial(const Real& P, const Integer& n, const Real& \u03c1) {
  #line 128 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  assert(Integer(0) <= n);
  #line 129 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  assert(0.0 <= \u03c1 && \u03c1 <= 1.0);
  #line 130 "src/distribution/scalar_discrete/BinomialDistribution.birch"

  return boost::math::quantile(boost::math::binomial_distribution<>(n, \u03c1), P);
  }

}
namespace birch {
#line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
#line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
BoundedDiscreteDistribution_::BoundedDiscreteDistribution_() :
    #line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
    base_type_() {
  //
}

#line 5 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
#line 5 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
#line 5 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
Boolean BoundedDiscreteDistribution_::isBoundedDiscrete() {
  #line 6 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  return true;
}

#line 9 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
#line 9 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
#line 9 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
std::optional<BoundedDiscreteDistribution> BoundedDiscreteDistribution_::getBoundedDiscrete() {
  #line 10 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  return this->shared_from_this_();
}

}
namespace birch {
#line 59 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 59 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Integer simulate_categorical(const libbirch::DefaultArray<Real,1>& \u03c1) {
  #line 60 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return simulate_categorical(\u03c1, 1.0);
}

#line 69 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 69 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Integer simulate_categorical(const libbirch::DefaultArray<Real,1>& \u03c1, const Real& Z) {
  #line 70 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  if (length(\u03c1) == Integer(0)) {
    #line 71 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    return Integer(0);
  } else {
    auto u = simulate_uniform(0.0, Z);
    auto x = Integer(1);
    auto P = \u03c1(Integer(1));
    #line 76 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    while (P < u) {
      #line 77 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
      assert(x <= length(\u03c1));
      #line 78 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
      x = x + Integer(1);
      #line 79 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
      P = P + \u03c1(x);
    }
    #line 81 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    return x;
  }
}

#line 93 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 93 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Real logpdf_categorical(const Integer& x, const libbirch::DefaultArray<Real,1>& \u03c1) {
  #line 94 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return logpdf_categorical(x, \u03c1, 1.0);
}

#line 106 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 106 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Real logpdf_categorical(const Integer& x, const libbirch::DefaultArray<Real,1>& \u03c1, const Real& Z) {
  #line 107 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  if (Integer(1) <= x && x <= length(\u03c1)) {
    #line 108 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    return log(element(\u03c1, x)) - log(Z);
  } else {
    #line 110 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    return -(std::numeric_limits<double>::infinity());
  }
}

#line 122 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 122 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Real cdf_categorical(const Integer& x, const libbirch::DefaultArray<Real,1>& \u03c1) {
  #line 123 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return cdf_categorical(x, \u03c1, 1.0);
}

#line 135 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 135 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Real cdf_categorical(const Integer& x, const libbirch::DefaultArray<Real,1>& \u03c1, const Real& Z) {
  #line 136 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  if (x <= Integer(0)) {
    #line 137 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    return 0.0;
  } else {
    #line 138 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    if (Integer(1) <= x && x <= length(\u03c1)) {
      #line 139 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
      return sum(\u03c1(libbirch::make_range(Integer(1), x))) / Z;
    } else {
      #line 141 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
      return 1.0;
    }
  }
}

#line 153 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 153 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Integer quantile_categorical(const Real& P, const libbirch::DefaultArray<Real,1>& \u03c1) {
  #line 154 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return quantile_categorical(P, \u03c1, 1.0);
}

#line 166 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 166 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Integer quantile_categorical(const Real& P, const libbirch::DefaultArray<Real,1>& \u03c1, const Real& Z) {
  #line 167 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  if (length(\u03c1) == Integer(0)) {
    #line 168 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    return Integer(0);
  } else {
    auto i = Integer(1);
    auto R = \u03c1(Integer(1));
    #line 172 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    while (R < P * Z && i < length(\u03c1)) {
      #line 173 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
      i = i + Integer(1);
      #line 174 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
      R = R + \u03c1(i);
    }
    #line 176 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    return i;
  }
}

}
namespace birch {
#line 89 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 89 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
Integer simulate_conway_maxwell_poisson(const Real& \u03bb, const Real& \u03bd, const Integer& n) {
  auto p = p_conway_maxwell_poisson(\u03bb, \u03bd, n);
  #line 92 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return simulate_categorical(p, sum(p)) - Integer(1);
}

#line 120 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 120 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
Real cdf_conway_maxwell_poisson(const Integer& x, const Real& \u03bb, const Real& \u03bd, const Integer& n) {
  auto p = p_conway_maxwell_poisson(\u03bb, \u03bd, n);
  #line 123 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return cdf_categorical(x + Integer(1), p, sum(p));
}

#line 136 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 136 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
Integer quantile_conway_maxwell_poisson(const Real& P, const Real& \u03bb, const Real& \u03bd, const Integer& n) {
  auto p = p_conway_maxwell_poisson(\u03bb, \u03bd, n);
  #line 139 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return quantile_categorical(P, p, sum(p)) - Integer(1);
}

#line 152 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 152 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
libbirch::DefaultArray<Real,1> p_conway_maxwell_poisson(const Real& \u03bb, const Real& \u03bd, const Integer& n) {
  auto log_\u03bb = log(\u03bb);
  auto z = vector_lambda([=](const Integer& x) {
    #line 155 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
    return if_then_else(x == Integer(1), 0.0, (x - Integer(1)) * log_\u03bb - \u03bd * lgamma(x));
  }, n + Integer(1));
  auto mx = max(z);
  auto p = transform(z, [=](const Real& x) {
    #line 158 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
    return exp(x - mx);
  });
  #line 159 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return p / sum(p);
}

}
namespace birch {
#line 42 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 42 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
Distribution<Integer> Categorical(const Random<libbirch::DefaultArray<Real,1>>& \u03c1) {
  #line 43 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  \u03c1->prune();
  #line 44 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  if (\u03c1->hasDirichlet()) {
    auto \u03b1 = \u03c1->getDirichlet();
    auto p = DirichletCategorical(\u03b1);
    #line 47 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
    \u03c1->graft(p);
    #line 48 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
    return p;
  } else {
    #line 49 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
    if (\u03c1->hasRestaurant()) {
      auto n4588_ = \u03c1->getRestaurant();
      auto \u03b1 = std::get<0>(n4588_);
      auto \u03b8 = std::get<1>(n4588_);
      auto n = std::get<2>(n4588_);
      auto N = std::get<3>(n4588_);
      auto p = RestaurantCategorical(\u03b1, \u03b8, n, N);
      #line 52 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
      \u03c1->graft(p);
      #line 53 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
      return p;
    } else {
      #line 55 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
      return construct<CategoricalDistribution<Random<libbirch::DefaultArray<Real,1>>>>(\u03c1);
    }
  }
}

#line 64 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 64 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
Integer simulate_dirichlet_categorical(const libbirch::DefaultArray<Real,1>& \u03b1) {
  #line 65 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  return simulate_categorical(simulate_dirichlet(\u03b1));
}

#line 76 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 76 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
Real logpdf_dirichlet_categorical(const Integer& x, const libbirch::DefaultArray<Real,1>& \u03b1) {
  #line 77 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  if (Integer(1) <= x && x <= length(\u03b1)) {
    auto A = sum(\u03b1);
    #line 79 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
    return lgamma(1.0 + \u03b1(x)) - lgamma(\u03b1(x)) + lgamma(A) - lgamma(1.0 + A);
  } else {
    #line 81 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
    return -(std::numeric_limits<double>::infinity());
  }
}

}
namespace birch {
#line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
#line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
DiscreteDistribution_::DiscreteDistribution_() :
    #line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
    base_type_() {
  //
}

#line 5 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
#line 5 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
#line 5 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
Boolean DiscreteDistribution_::isDiscrete() {
  #line 6 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  return true;
}

#line 9 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
#line 9 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
#line 9 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
std::optional<DiscreteDistribution> DiscreteDistribution_::getDiscrete() {
  #line 10 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  return this->shared_from_this_();
}

}
#line 1 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"

#include <boost/math/distributions.hpp>
namespace birch {
#line 68 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 68 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Distribution<Integer> Poisson(const Random<Real>& \u03bb) {
  #line 69 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  \u03bb->prune();
  #line 70 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  if (\u03bb->hasGamma()) {
    auto n4589_ = \u03bb->getGamma();
    auto k = std::get<0>(n4589_);
    auto \u03b8 = std::get<1>(n4589_);
    auto p = GammaPoisson(k, \u03b8);
    #line 73 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
    \u03bb->graft(p);
    #line 74 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
    return p;
  } else {
    #line 76 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
    return construct<PoissonDistribution<Random<Real>>>(\u03bb);
  }
}

#line 86 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 86 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Integer simulate_gamma_poisson(const Real& k, const Real& \u03b8) {
  #line 87 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  assert(0.0 < k);
  #line 88 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  assert(0.0 < \u03b8);
  #line 92 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return simulate_poisson(simulate_gamma(k, \u03b8));
}

#line 131 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 131 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Real cdf_gamma_poisson(const Integer& x, const Real& k, const Real& \u03b8) {
  #line 132 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  assert(0.0 < k);
  #line 133 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  assert(0.0 < \u03b8);
  #line 134 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return ibeta(k, x + 1.0, 1.0 / (\u03b8 + 1.0));
}

#line 146 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 146 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Integer quantile_gamma_poisson(const Real& P, const Real& k, const Real& \u03b8) {
  #line 147 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  assert(0.0 < k);
  #line 148 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  assert(0.0 < \u03b8);
  auto \u03c1 = 1.0 / (\u03b8 + 1.0);
  #line 150 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"

  return boost::math::quantile(boost::math::negative_binomial_distribution<>(
      k, \u03c1), P);
  }

}
namespace birch {
}
#line 1 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"

#include <boost/math/distributions/negative_binomial.hpp>
namespace birch {
#line 74 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 74 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
Integer simulate_negative_binomial(const Integer& k, const Real& \u03c1) {
  #line 75 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  assert(Integer(0) < k);
  #line 76 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  assert(0.0 <= \u03c1 && \u03c1 <= 1.0);
  #line 77 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"

  return std::negative_binomial_distribution<Integer>(k, \u03c1)(rng);
  }

#line 104 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 104 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
Real cdf_negative_binomial(const Integer& x, const Integer& k, const Real& \u03c1) {
  #line 105 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  assert(Integer(0) < k);
  #line 106 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  assert(0.0 <= \u03c1 && \u03c1 <= 1.0);
  #line 107 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  if (x < Integer(0)) {
    #line 108 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
    return 0.0;
  } else {
    #line 110 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
    return ibeta(k, x + 1.0, \u03c1);
  }
}

#line 123 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 123 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
Integer quantile_negative_binomial(const Real& P, const Integer& k, const Real& \u03c1) {
  #line 124 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  assert(Integer(0) < k);
  #line 125 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  assert(0.0 <= \u03c1 && \u03c1 <= 1.0);
  #line 126 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"

  return boost::math::quantile(boost::math::negative_binomial_distribution<>(
      k, \u03c1), P);
  }

}
#line 1 "src/distribution/scalar_discrete/PoissonDistribution.birch"

#include <boost/math/distributions/poisson.hpp>
namespace birch {
#line 64 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 64 "src/distribution/scalar_discrete/PoissonDistribution.birch"
Integer simulate_poisson(const Real& \u03bb) {
  #line 65 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  assert(0.0 <= \u03bb);
  #line 66 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  if (\u03bb > 0.0) {
    #line 67 "src/distribution/scalar_discrete/PoissonDistribution.birch"

    return std::poisson_distribution<Integer>(\u03bb)(rng);
      } else {
    #line 71 "src/distribution/scalar_discrete/PoissonDistribution.birch"
    return Integer(0);
  }
}

#line 95 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 95 "src/distribution/scalar_discrete/PoissonDistribution.birch"
Real cdf_poisson(const Integer& x, const Real& \u03bb) {
  #line 96 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  assert(0.0 <= \u03bb);
  #line 97 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  if (x < Integer(0)) {
    #line 98 "src/distribution/scalar_discrete/PoissonDistribution.birch"
    return 0.0;
  } else {
    auto k = x + 1.0;
    #line 101 "src/distribution/scalar_discrete/PoissonDistribution.birch"
    return upper_inc_gamma(k, \u03bb);
  }
}

#line 113 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 113 "src/distribution/scalar_discrete/PoissonDistribution.birch"
Integer quantile_poisson(const Real& P, const Real& \u03bb) {
  #line 114 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  assert(0.0 <= \u03bb);
  #line 115 "src/distribution/scalar_discrete/PoissonDistribution.birch"

  return boost::math::quantile(boost::math::poisson_distribution<>(\u03bb), P);
  }

}
namespace birch {
#line 60 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
#line 60 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
Integer simulate_restaurant_categorical(const Real& \u03b1, const Real& \u03b8, const libbirch::DefaultArray<Integer,1>& n, const Integer& N) {
  #line 62 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  assert(N >= Integer(0));
  #line 63 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  assert(sum(n) == N);
  auto k = Integer(0);
  auto K = length(n);
  #line 66 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  if (N == Integer(0)) {
    #line 68 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
    k = Integer(1);
  } else {
    auto u = simulate_uniform(0.0, N + \u03b8);
    auto U = K * \u03b1 + \u03b8;
    #line 72 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
    if (u < U) {
      #line 74 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
      k = K + Integer(1);
    } else {
      #line 77 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
      while (k < K && u > U) {
        #line 78 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
        k = k + Integer(1);
        #line 79 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
        U = U + n(k) - \u03b1;
      }
    }
  }
  #line 83 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  return k;
}

#line 96 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
#line 96 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
Real logpdf_restaurant_categorical(const Integer& k, const Real& \u03b1, const Real& \u03b8, const libbirch::DefaultArray<Integer,1>& n, const Integer& N) {
  auto K = length(n);
  #line 99 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  if (k > K + Integer(1)) {
    #line 100 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
    return -(std::numeric_limits<double>::infinity());
  } else {
    #line 101 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
    if (k == K + Integer(1)) {
      #line 102 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
      return log(K * \u03b1 + \u03b8) - log(N + \u03b8);
    } else {
      #line 104 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
      return log(n(k) - \u03b1) - log(N + \u03b8);
    }
  }
}

}
namespace birch {
#line 101 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 101 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
Distribution<Integer> Poisson(const Multiply<Random<Real>, Random<Real>>& \u03bb) {
  #line 103 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  \u03bb->l->prune();
  #line 104 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  if (\u03bb->l->hasGamma()) {
    auto n4590_ = \u03bb->l->getGamma();
    auto k = std::get<0>(n4590_);
    auto \u03b8 = std::get<1>(n4590_);
    auto p = ScaledGammaPoisson(\u03bb->r, k, \u03b8);
    #line 107 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    \u03bb->l->graft(p);
    #line 108 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    return p;
  } else {
    #line 110 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    \u03bb->r->prune();
    #line 111 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    if (\u03bb->r->hasGamma()) {
      auto n4591_ = \u03bb->r->getGamma();
      auto k = std::get<0>(n4591_);
      auto \u03b8 = std::get<1>(n4591_);
      auto p = ScaledGammaPoisson(\u03bb->l, k, \u03b8);
      #line 114 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
      \u03bb->r->graft(p);
      #line 115 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
      return p;
    } else {
      #line 117 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
      return construct<PoissonDistribution<Multiply<Random<Real>, Random<Real>>>>(\u03bb);
    }
  }
}

}
#line 1 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"

#include <boost/math/distributions/uniform.hpp>
namespace birch {
#line 68 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 68 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Integer simulate_uniform_int(const Integer& l, const Integer& u) {
  #line 69 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  assert(l <= u);
  #line 70 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"

  return std::uniform_int_distribution<Integer>(l, u)(rng);
  }

#line 84 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 84 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Real logpdf_uniform_int(const Integer& x, const Integer& l, const Integer& u) {
  #line 85 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  if (x >= l && x <= u) {
    #line 86 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
    return -(log1p(u - l));
  } else {
    #line 88 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
    return -(std::numeric_limits<double>::infinity());
  }
}

#line 101 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 101 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Real cdf_uniform_int(const Integer& x, const Integer& l, const Integer& u) {
  #line 102 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  if (x < l) {
    #line 103 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
    return 0.0;
  } else {
    #line 104 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
    if (x > u) {
      #line 105 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
      return 1.0;
    } else {
      #line 107 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
      return (x - l + 1.0) / (u - l + 1.0);
    }
  }
}

#line 120 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 120 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Integer quantile_uniform_int(const Real& P, const Integer& l, const Integer& u) {
  #line 121 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  assert(l <= u);
  #line 122 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  return l + scalar<Integer>(P * (u - l));
}

}
