namespace birch {
#line 18 "src/kernel/Kernel.birch"
#line 18 "src/kernel/Kernel.birch"
Kernel_::Kernel_() :
    #line 18 "src/kernel/Kernel.birch"
    base_type_(),
    #line 22 "src/kernel/Kernel.birch"
    nmoves(Integer(1)),
    #line 27 "src/kernel/Kernel.birch"
    nlags(Integer(0)) {
  //
}

#line 39 "src/kernel/Kernel.birch"
#line 39 "src/kernel/Kernel.birch"
#line 39 "src/kernel/Kernel.birch"
void Kernel_::read(const Buffer& buffer) {
  #line 40 "src/kernel/Kernel.birch"
  this->base_type_::read(buffer);
  #line 41 "src/kernel/Kernel.birch"
    libbirch::optional_assign(nmoves, buffer->get<Integer>(String("nmoves")));
;
  #line 42 "src/kernel/Kernel.birch"
    libbirch::optional_assign(nlags, buffer->get<Integer>(String("nlags")));
;
}

#line 45 "src/kernel/Kernel.birch"
#line 45 "src/kernel/Kernel.birch"
#line 45 "src/kernel/Kernel.birch"
void Kernel_::write(const Buffer& buffer) {
  #line 46 "src/kernel/Kernel.birch"
  this->base_type_::write(buffer);
  #line 47 "src/kernel/Kernel.birch"
  buffer->set(String("nmoves"), nmoves);
  #line 48 "src/kernel/Kernel.birch"
  buffer->set(String("nlags"), nlags);
}

}
namespace birch {
#line 1 "src/kernel/LangevinKernel.birch"
#line 1 "src/kernel/LangevinKernel.birch"
LangevinKernel_::LangevinKernel_() :
    #line 1 "src/kernel/LangevinKernel.birch"
    base_type_(),
    #line 15 "src/kernel/LangevinKernel.birch"
    scale(1.0) {
  //
}

#line 17 "src/kernel/LangevinKernel.birch"
#line 17 "src/kernel/LangevinKernel.birch"
#line 17 "src/kernel/LangevinKernel.birch"
Real LangevinKernel_::apply(const Integer& t, const Particle& \u03c0) {
  #line 19 "src/kernel/LangevinKernel.birch"
  \u03c0->label(t);
  #line 20 "src/kernel/LangevinKernel.birch"
  \u03c0->constant(t - nlags);
  #line 22 "src/kernel/LangevinKernel.birch"
  if (nmoves > Integer(0)) {
    auto \u03b4 = scale / pow(min(t, nlags), Integer(3));
    auto x = \u03c0->args();
    auto p = \u03c0->peek();
    auto d = \u03c0->grad();
    auto \u03bc = x + d * \u03b4;
    auto \u03a3 = diagonal(2.0 * \u03b4, length(x));
    auto naccepts = Integer(0);
    auto accept = false;
    #line 36 "src/kernel/LangevinKernel.birch"
    for (auto n = Integer(1); n <= nmoves; ++n) {
      auto x_prime_ = simulate_multivariate_gaussian(\u03bc, \u03a3);
      auto p_prime_ = \u03c0->move(x_prime_);
      auto d_prime_ = \u03c0->grad();
      auto \u03bc_prime_ = x_prime_ + d_prime_ * \u03b4;
      auto q = logpdf_multivariate_gaussian(x, \u03bc_prime_, \u03a3);
      auto q_prime_ = logpdf_multivariate_gaussian(x_prime_, \u03bc, \u03a3);
      #line 48 "src/kernel/LangevinKernel.birch"
      accept = log(simulate_uniform(0.0, 1.0)) <= p_prime_ - p + q - q_prime_;
      #line 49 "src/kernel/LangevinKernel.birch"
      if (accept) {
        #line 50 "src/kernel/LangevinKernel.birch"
        naccepts = naccepts + Integer(1);
        #line 51 "src/kernel/LangevinKernel.birch"
        x = x_prime_;
        #line 52 "src/kernel/LangevinKernel.birch"
        p = p_prime_;
        #line 53 "src/kernel/LangevinKernel.birch"
        d = d_prime_;
        #line 54 "src/kernel/LangevinKernel.birch"
        \u03bc = \u03bc_prime_;
      }
    }
    #line 57 "src/kernel/LangevinKernel.birch"
    if (!(accept)) {
      #line 60 "src/kernel/LangevinKernel.birch"
      \u03c0->move(x);
    }
    #line 62 "src/kernel/LangevinKernel.birch"
    return scalar<Real>(naccepts) / nmoves;
  } else {
    #line 64 "src/kernel/LangevinKernel.birch"
    return 0.0;
  }
}

#line 68 "src/kernel/LangevinKernel.birch"
#line 68 "src/kernel/LangevinKernel.birch"
#line 68 "src/kernel/LangevinKernel.birch"
void LangevinKernel_::read(const Buffer& buffer) {
  #line 69 "src/kernel/LangevinKernel.birch"
  this->base_type_::read(buffer);
  #line 70 "src/kernel/LangevinKernel.birch"
    libbirch::optional_assign(scale, buffer->get<Real>(String("scale")));
;
}

#line 73 "src/kernel/LangevinKernel.birch"
#line 73 "src/kernel/LangevinKernel.birch"
#line 73 "src/kernel/LangevinKernel.birch"
void LangevinKernel_::write(const Buffer& buffer) {
  #line 74 "src/kernel/LangevinKernel.birch"
  this->base_type_::write(buffer);
  #line 75 "src/kernel/LangevinKernel.birch"
  buffer->set(String("scale"), scale);
}

#line 1 "src/kernel/LangevinKernel.birch"
LangevinKernel_* make_LangevinKernel_() {
  #line 1 "src/kernel/LangevinKernel.birch"
  return new LangevinKernel_();
  #line 1 "src/kernel/LangevinKernel.birch"
}

}
