#line 1 "src/system/filesystem.birch"

#if defined(HAVE_FILESYSTEM)
#include <filesystem>
namespace fs = std::filesystem;
#elif defined(HAVE_EXPERIMENTAL_FILESYSTEM)
#include <experimental/filesystem>
namespace fs = std::experimental::filesystem;
#else
#include <boost/filesystem.hpp>
namespace fs = boost::filesystem;
#endif
namespace birch {
#line 14 "src/system/filesystem.birch"
Integer READ = Integer(1);
#line 15 "src/system/filesystem.birch"
Integer WRITE = Integer(2);
#line 16 "src/system/filesystem.birch"
Integer APPEND = Integer(3);
#line 23 "src/system/filesystem.birch"
#line 23 "src/system/filesystem.birch"
void mkdir(const String& path) {
  #line 24 "src/system/filesystem.birch"

  fs::path p = path;
  if (!fs::is_directory(p)) {
    p = p.parent_path();
  }
  fs::create_directories(p);
  }

#line 43 "src/system/filesystem.birch"
#line 43 "src/system/filesystem.birch"
File fopen(const String& path) {
  #line 44 "src/system/filesystem.birch"
  return fopen(path, READ);
}

#line 59 "src/system/filesystem.birch"
#line 59 "src/system/filesystem.birch"
File fopen(const String& path, const Integer& mode) {
  #line 60 "src/system/filesystem.birch"
  assert(mode == READ || mode == WRITE || mode == APPEND);
  String s;
  #line 62 "src/system/filesystem.birch"
  if (mode == READ) {
    #line 63 "src/system/filesystem.birch"
    s = String("r");
  } else {
    #line 64 "src/system/filesystem.birch"
    if (mode == WRITE) {
      #line 65 "src/system/filesystem.birch"
      s = String("w");
      #line 66 "src/system/filesystem.birch"

    fs::path p = path;
    if (!p.parent_path().empty()) {
      fs::create_directories(p.parent_path());
    }
        } else {
      #line 72 "src/system/filesystem.birch"
      if (mode == APPEND) {
        #line 73 "src/system/filesystem.birch"
        s = String("a");
      }
    }
  }
  #line 75 "src/system/filesystem.birch"

  auto f = ::fopen(path.c_str(), s.c_str());
  if (!f) {
    error("could not open file " + path + ".");
  }
  lockf(fileno(f), F_LOCK, 0);
  return f;
  }

#line 88 "src/system/filesystem.birch"
#line 88 "src/system/filesystem.birch"
String extension(const String& path) {
  String ext;
  #line 90 "src/system/filesystem.birch"

  fs::path f(path);
  ext = f.extension().string();
    #line 94 "src/system/filesystem.birch"
  return ext;
}

}
namespace birch {
#line 4 "src/system/stdio.birch"
#line 4 "src/system/stdio.birch"
File getStdIn() {
  #line 5 "src/system/stdio.birch"

  return stdin;
  }

#line 13 "src/system/stdio.birch"
#line 13 "src/system/stdio.birch"
File getStdOut() {
  #line 14 "src/system/stdio.birch"

  return stdout;
  }

#line 22 "src/system/stdio.birch"
#line 22 "src/system/stdio.birch"
File getStdErr() {
  #line 23 "src/system/stdio.birch"

  return stderr;
  }

#line 31 "src/system/stdio.birch"
InputStream stdin_ = make_input_stream(getStdIn());
#line 36 "src/system/stdio.birch"
OutputStream stdout_ = make_output_stream(getStdOut());
#line 41 "src/system/stdio.birch"
OutputStream stderr_ = make_output_stream(getStdErr());
}
namespace birch {
#line 8 "src/system/system.birch"
#line 8 "src/system/system.birch"
Integer system(const String& cmd) {
  #line 9 "src/system/system.birch"

  int code;
  int status = std::system(cmd.c_str());
  if (WIFEXITED(status)) {
    code = WEXITSTATUS(status);
  } else if (WIFSIGNALED(status)) {
    code = WTERMSIG(status);
  } else if (WIFSTOPPED(status)) {
    code = WSTOPSIG(status);
  } else {
    code = status;
  }
  return code;
  }

#line 30 "src/system/system.birch"
#line 30 "src/system/system.birch"
void exit(const Integer& code) {
  #line 31 "src/system/system.birch"

  std::exit(code);
  }

}
