namespace birch {
#line 10 "src/test/distance.birch"
#line 10 "src/test/distance.birch"
Real wasserstein(const libbirch::DefaultArray<Real,1>& x1, const libbirch::DefaultArray<Real,1>& x2) {
  #line 11 "src/test/distance.birch"
  assert(length(x1) == length(x2));
  auto N = length(x1);
  auto y1 = sort<Real>(x1);
  auto y2 = sort<Real>(x2);
  #line 15 "src/test/distance.birch"
  return reduce<Real>(y1 - y2, 0.0, [=](const Real& a, const Real& b) -> Real {
    #line 16 "src/test/distance.birch"
    return abs(a) + abs(b);
  }) / N;
}

}
namespace birch {
#line 9 "src/test/test.birch"
#line 9 "src/test/test.birch"
Boolean pass(const libbirch::DefaultArray<Real,1>& x1, const libbirch::DefaultArray<Real,1>& x2) {
  #line 10 "src/test/test.birch"
  assert(length(x1) == length(x2));
  auto N = length(x1);
  auto \u03b5 = 2.0 / sqrt(N);
  auto mn = min(min(x1), min(x2));
  auto mx = max(max(x1), max(x2));
  auto z1 = (x1 - vector(mn, N)) / (mx - mn);
  auto z2 = (x2 - vector(mn, N)) / (mx - mn);
  auto \u03b4 = wasserstein(z1, z2);
  #line 23 "src/test/test.birch"
  if (Integer(10) < N && !((\u03b4 <= \u03b5))) {
    #line 24 "src/test/test.birch"
    stderr_->print(String("***failed***, ") + \u03b4 + String(" > ") + \u03b5 + String("\n"));
    #line 25 "src/test/test.birch"
    return false;
  }
  #line 27 "src/test/test.birch"
  return true;
}

#line 38 "src/test/test.birch"
#line 38 "src/test/test.birch"
Boolean pass(const libbirch::DefaultArray<Real,2>& X1, const libbirch::DefaultArray<Real,2>& X2) {
  #line 39 "src/test/test.birch"
  assert(rows(X1) == rows(X2));
  #line 40 "src/test/test.birch"
  assert(columns(X1) == columns(X2));
  auto R = rows(X1);
  auto C = columns(X1);
  auto failed = Integer(0);
  auto tests = Integer(0);
  auto \u03b5 = 2.0 / sqrt(R);
  #line 49 "src/test/test.birch"
  for (auto c = Integer(1); c <= C; ++c) {
    auto x1 = X1(libbirch::make_range(Integer(1), R), c);
    auto x2 = X2(libbirch::make_range(Integer(1), R), c);
    auto mn = min(min(x1), min(x2));
    auto mx = max(max(x1), max(x2));
    auto z1 = (x1 - vector(mn, R)) / (mx - mn);
    auto z2 = (x2 - vector(mn, R)) / (mx - mn);
    auto \u03b4 = wasserstein(z1, z2);
    #line 62 "src/test/test.birch"
    if (Integer(10) < R && !((\u03b4 <= \u03b5))) {
      #line 63 "src/test/test.birch"
      failed = failed + Integer(1);
      #line 64 "src/test/test.birch"
      stderr_->print(String("***failed*** on component ") + c + String(", ") + \u03b4 + String(" > ") + \u03b5 + String("\n"));
    }
    #line 66 "src/test/test.birch"
    tests = tests + Integer(1);
  }
  #line 72 "src/test/test.birch"
  for (auto c = Integer(1); c <= C; ++c) {
    auto u = vector_lambda([=](const Integer& i) {
      #line 75 "src/test/test.birch"
      return simulate_gaussian(0.0, 1.0);
    }, C);
    #line 77 "src/test/test.birch"
    u = u / sqrt(dot(u));
    auto x1 = X1 * u;
    auto x2 = X2 * u;
    auto mn = min(min(x1), min(x2));
    auto mx = max(max(x1), max(x2));
    auto z1 = (x1 - vector(mn, R)) / (mx - mn);
    auto z2 = (x2 - vector(mn, R)) / (mx - mn);
    auto \u03b4 = wasserstein(z1, z2);
    #line 89 "src/test/test.birch"
    if (Integer(10) < R && !((\u03b4 <= \u03b5))) {
      #line 90 "src/test/test.birch"
      failed = failed + Integer(1);
      #line 91 "src/test/test.birch"
      stderr_->print(String("***failed*** on random projection, ") + \u03b4 + String(" > ") + \u03b5 + String("\n"));
    }
    #line 93 "src/test/test.birch"
    tests = tests + Integer(1);
  }
  #line 95 "src/test/test.birch"
  if (failed > Integer(0)) {
    #line 96 "src/test/test.birch"
    stderr_->print(String("***failed*** ") + failed + String(" of ") + tests + String(" comparisons\n"));
  }
  #line 98 "src/test/test.birch"
  return failed == Integer(0);
}

}
namespace birch {
#line 7 "src/test/test_cdf.birch"
#line 7 "src/test/test_cdf.birch"
void test_cdf(const Distribution<Real>& q, const Integer& N) {
  auto failed = false;
  auto from = q->lower();
  #line 12 "src/test/test_cdf.birch"
  if (from.has_value()) {
    auto test = q->quantile(0.0);
    #line 15 "src/test/test_cdf.birch"
    if (test.has_value() && abs(from.value() - test.value()) > 1.0 / N) {
      #line 16 "src/test/test_cdf.birch"
      error(String("lower bound and quantile comparison failed, ") + from.value() + String(" vs ") + test.value() + String("."));
    }
  } else {
    #line 20 "src/test/test_cdf.birch"
    from = q->quantile(1.0 / N);
    #line 21 "src/test/test_cdf.birch"
    assert(from.has_value());
  }
  auto to = q->upper();
  #line 26 "src/test/test_cdf.birch"
  if (to.has_value()) {
    auto test = q->quantile(1.0);
    #line 29 "src/test/test_cdf.birch"
    if (test.has_value() && abs(to.value() - test.value()) > 1.0 / N) {
      #line 30 "src/test/test_cdf.birch"
      error(String("upper bound and quantile comparison failed, ") + to.value() + String(" vs ") + test.value() + String("."));
    }
  } else {
    #line 34 "src/test/test_cdf.birch"
    to = q->quantile(1.0 - 1.0 / N);
    #line 35 "src/test/test_cdf.birch"
    if (!(to.has_value())) {
      auto u = 1.0;
      #line 38 "src/test/test_cdf.birch"
      while (q->pdf(u) > 1.0 / N) {
        #line 39 "src/test/test_cdf.birch"
        u = 2.0 * u;
      }
      #line 41 "src/test/test_cdf.birch"
      to = u;
    }
    #line 43 "src/test/test_cdf.birch"
    assert(to.has_value());
  }
  auto P = 0.5 / N;
  #line 48 "src/test/test_cdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    auto x = from.value() + (n - 0.5) * (to.value() - from.value()) / N;
    auto C = q->cdf(x).value();
    #line 51 "src/test/test_cdf.birch"
    P = P + q->pdf(x) * (to.value() - from.value()) / N;
    auto \u03b4 = abs(C - P);
    auto \u03b5 = 2.0 / sqrt(N);
    #line 55 "src/test/test_cdf.birch"
    if (Integer(10) < N && !((\u03b4 <= \u03b5))) {
      #line 56 "src/test/test_cdf.birch"
      failed = true;
      #line 57 "src/test/test_cdf.birch"
      stderr_->print(String("***failed*** on step ") + n + String(", ") + \u03b4 + String(" > ") + \u03b5 + String("\n"));
    }
    #line 59 "src/test/test_cdf.birch"
    if (failed) {
      #line 60 "src/test/test_cdf.birch"
      exit(Integer(1));
    }
  }
}

#line 70 "src/test/test_cdf.birch"
#line 70 "src/test/test_cdf.birch"
void test_cdf(const Distribution<Integer>& q) {
  auto failed = false;
  auto from = q->lower();
  #line 75 "src/test/test_cdf.birch"
  if (from.has_value()) {
    auto test = q->quantile(0.0);
    #line 78 "src/test/test_cdf.birch"
    if (test.has_value() && abs(from.value() - test.value()) > 1.0e-6) {
      #line 79 "src/test/test_cdf.birch"
      error(String("lower bound and quantile comparison failed, ") + from.value() + String(" vs ") + test.value() + String("."));
    }
  } else {
    #line 83 "src/test/test_cdf.birch"
    from = q->quantile(1.0e-6);
    #line 84 "src/test/test_cdf.birch"
    assert(from.has_value());
  }
  auto to = q->upper();
  #line 89 "src/test/test_cdf.birch"
  if (to.has_value()) {
    auto test = q->quantile(1.0);
    #line 92 "src/test/test_cdf.birch"
    if (test.has_value() && abs(to.value() - test.value()) > 1.0e-6) {
      #line 93 "src/test/test_cdf.birch"
      error(String("upper bound and quantile comparison failed, ") + to.value() + String(" vs ") + test.value() + String("."));
    }
  } else {
    #line 97 "src/test/test_cdf.birch"
    to = q->quantile(1.0 - 1.0e-6);
    #line 98 "src/test/test_cdf.birch"
    assert(to.has_value());
  }
  auto P = 0.0;
  #line 103 "src/test/test_cdf.birch"
  for (auto x = from.value(); x <= to.value(); ++x) {
    auto C = q->cdf(x).value();
    #line 105 "src/test/test_cdf.birch"
    P = P + q->pdf(x);
    auto \u03b4 = abs(C - P);
    auto \u03b5 = 2.0e-3;
    #line 109 "src/test/test_cdf.birch"
    if (!((\u03b4 <= \u03b5))) {
      #line 110 "src/test/test_cdf.birch"
      failed = true;
      #line 111 "src/test/test_cdf.birch"
      stderr_->print(String("***failed*** on value ") + x + String(", ") + \u03b4 + String(" > ") + \u03b5 + String("\n"));
    }
    #line 113 "src/test/test_cdf.birch"
    if (failed) {
      #line 114 "src/test/test_cdf.birch"
      exit(Integer(1));
    }
  }
}

}
namespace birch {
#line 8 "src/test/test_conjugacy.birch"
#line 8 "src/test/test_conjugacy.birch"
void test_conjugacy(const TestModel& m, const Integer& N, const Boolean& lazy) {
  auto D = m->size();
  libbirch::DefaultArray<Real,2> X1(libbirch::make_shape(N, D));
  libbirch::DefaultArray<Real,2> X2(libbirch::make_shape(N, D));
  #line 13 "src/test/test_conjugacy.birch"
  #pragma omp parallel
  {
    #pragma omp for schedule(static)
    for (auto n = Integer(1); n <= N; ++n) {
      auto m_prime_ = copy(m);
      #line 15 "src/test/test_conjugacy.birch"
      m_prime_->initialize();
      auto f = copy(m_prime_);
      auto forward = construct<Handler>(false, lazy);
      #line 20 "src/test/test_conjugacy.birch"
      {
        auto handler_ = swap_handler(forward);
        #line 21 "src/test/test_conjugacy.birch"
        f->simulate();
        #line 22 "src/test/test_conjugacy.birch"
        X1(n, libbirch::make_range(Integer(1), D)) = f->forward();
        set_handler(handler_);
      }
      auto b = copy(m_prime_);
      auto backward = construct<Handler>(true, lazy);
      #line 28 "src/test/test_conjugacy.birch"
      {
        auto handler_ = swap_handler(backward);
        #line 29 "src/test/test_conjugacy.birch"
        b->simulate();
        #line 30 "src/test/test_conjugacy.birch"
        X2(n, libbirch::make_range(Integer(1), D)) = b->backward();
        set_handler(handler_);
      }
    }
  }
  #line 35 "src/test/test_conjugacy.birch"
  if (!(pass(X1, X2))) {
    #line 36 "src/test/test_conjugacy.birch"
    exit(Integer(1));
  }
}

}
namespace birch {
#line 9 "src/test/test_grad.birch"
#line 9 "src/test/test_grad.birch"
void test_grad(const TestModel& m, const Integer& N, const Boolean& backward) {
  auto failed = vector(0.0, N);
  auto h = 1.0e-5;
  auto \u03b3 = 1.0e-3;
  auto \u03b5 = 1.0e-2;
  #line 20 "src/test/test_grad.birch"
  #pragma omp parallel
  {
    #pragma omp for schedule(static)
    for (auto n = Integer(1); n <= N; ++n) {
      auto handler = construct<Handler>(backward, true);
      auto m_prime_ = copy(m);
      #line 23 "src/test/test_grad.birch"
      {
        auto handler_ = swap_handler(handler);
        #line 24 "src/test/test_grad.birch"
        m_prime_->initialize();
        #line 25 "src/test/test_grad.birch"
        m_prime_->simulate();
        #line 26 "src/test/test_grad.birch"
        if (backward) {
          #line 27 "src/test/test_grad.birch"
          m_prime_->backwardGrad();
        } else {
          #line 29 "src/test/test_grad.birch"
          m_prime_->forwardGrad();
        }
        set_handler(handler_);
      }
      auto x = handler->args();
      auto d = handler->grad();
      #line 38 "src/test/test_grad.birch"
      for (auto i = Integer(1); i <= rows(x); ++i) {
        auto y = x;
        auto z = x;
        #line 41 "src/test/test_grad.birch"
        y(i) = y(i) - 0.5 * h;
        #line 42 "src/test/test_grad.birch"
        z(i) = z(i) + 0.5 * h;
        auto q = handler->move(y);
        auto r = handler->move(z);
        auto fd = (r - q) / h;
        auto \u03b4 = abs(d(i) - fd);
        #line 47 "src/test/test_grad.birch"
        if (Integer(10) < N && !((\u03b4 <= \u03b5 * abs(fd)))) {
          #line 48 "src/test/test_grad.birch"
          warn(String("on component ") + i + String(", d=") + d(i) + String(", fd=") + fd + String(", ") + \u03b4 + String(" > ") + \u03b5 * abs(fd));
          #line 50 "src/test/test_grad.birch"
          failed(n) = failed(n) + 1.0 / rows(x);
        }
      }
    }
  }
  auto rfail = sum(failed) / N;
  #line 57 "src/test/test_grad.birch"
  if (!((rfail <= \u03b3))) {
    #line 58 "src/test/test_grad.birch"
    stderr_->print(String("***failed*** in aggregate, rate ") + rfail + String(" > ") + \u03b3 + String("\n"));
    #line 60 "src/test/test_grad.birch"
    exit(Integer(1));
  }
}

}
namespace birch {
#line 8 "src/test/test_pdf.birch"
#line 8 "src/test/test_pdf.birch"
void test_pdf(const Distribution<Boolean>& \u03c0, const Integer& N, const Boolean& lazy) {
  auto k = Integer(0);
  #line 11 "src/test/test_pdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    #line 12 "src/test/test_pdf.birch"
    if (\u03c0->simulate()) {
      #line 13 "src/test/test_pdf.birch"
      k = k + Integer(1);
    }
  }
  auto failed = false;
  auto \u03b5 = 5.0 / sqrt(N);
  Real p;
  Real q;
  #line 23 "src/test/test_pdf.birch"
  if (lazy && \u03c0->supportsLazy()) {
    #line 24 "src/test/test_pdf.birch"
    p = exp(\u03c0->logpdfLazy(box(true)).value()->value());
    #line 25 "src/test/test_pdf.birch"
    q = exp(\u03c0->logpdfLazy(box(false)).value()->value());
  } else {
    #line 27 "src/test/test_pdf.birch"
    p = \u03c0->pdf(true);
    #line 28 "src/test/test_pdf.birch"
    q = \u03c0->pdf(false);
  }
  auto \u03b4 = abs(p - scalar<Real>(k) / N);
  #line 31 "src/test/test_pdf.birch"
  if (Integer(10) < N && !((\u03b4 <= \u03b5))) {
    #line 32 "src/test/test_pdf.birch"
    failed = true;
    #line 33 "src/test/test_pdf.birch"
    stderr_->print(String("***failed*** on true, ") + \u03b4 + String(" > ") + \u03b5 + String("\n"));
  }
  #line 35 "src/test/test_pdf.birch"
  \u03b4 = abs(q - scalar<Real>(N - k) / N);
  #line 36 "src/test/test_pdf.birch"
  if (Integer(10) < N && !((\u03b4 <= \u03b5))) {
    #line 37 "src/test/test_pdf.birch"
    failed = true;
    #line 38 "src/test/test_pdf.birch"
    stderr_->print(String("***failed*** on false, ") + \u03b4 + String(" > ") + \u03b5 + String("\n"));
  }
  #line 40 "src/test/test_pdf.birch"
  if (failed) {
    #line 41 "src/test/test_pdf.birch"
    exit(Integer(1));
  }
}

#line 52 "src/test/test_pdf.birch"
#line 52 "src/test/test_pdf.birch"
void test_pdf(const Distribution<Integer>& \u03c0, const Integer& N, const Boolean& lazy) {
  auto from = \u03c0->lower();
  #line 55 "src/test/test_pdf.birch"
  if (!(from.has_value())) {
    #line 56 "src/test/test_pdf.birch"
    from = \u03c0->quantile(1.0e-6);
    #line 57 "src/test/test_pdf.birch"
    assert(from.has_value());
  }
  auto to = \u03c0->upper();
  #line 62 "src/test/test_pdf.birch"
  if (!(to.has_value())) {
    #line 63 "src/test/test_pdf.birch"
    to = \u03c0->quantile(1.0 - 1.0e-6);
    #line 64 "src/test/test_pdf.birch"
    if (!(to.has_value())) {
      auto u = Integer(50);
      auto p = 0.0;
      #line 68 "src/test/test_pdf.birch"
      do {
        #line 69 "src/test/test_pdf.birch"
        u = Integer(2) * u;
        #line 70 "src/test/test_pdf.birch"
        if (lazy && \u03c0->supportsLazy()) {
          #line 71 "src/test/test_pdf.birch"
          p = exp(\u03c0->logpdfLazy(box(u)).value()->value());
        } else {
          #line 73 "src/test/test_pdf.birch"
          p = \u03c0->pdf(u);
        }
      } while (p > 1.0e-6);
      #line 76 "src/test/test_pdf.birch"
      to = u;
    }
  }
  auto count = vector(Integer(0), to.value() - from.value() + Integer(1));
  #line 82 "src/test/test_pdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    Integer j;
    #line 84 "src/test/test_pdf.birch"
    if (lazy && \u03c0->supportsLazy()) {
      #line 85 "src/test/test_pdf.birch"
      j = \u03c0->simulateLazy().value();
    } else {
      #line 87 "src/test/test_pdf.birch"
      j = \u03c0->simulate();
    }
    auto i = j - from.value() + Integer(1);
    #line 90 "src/test/test_pdf.birch"
    if (Integer(1) <= i && i <= length(count)) {
      #line 91 "src/test/test_pdf.birch"
      count(i) = count(i) + Integer(1);
    }
  }
  auto failed = false;
  #line 97 "src/test/test_pdf.birch"
  for (auto x = from.value(); x <= to.value(); ++x) {
    Real p;
    #line 99 "src/test/test_pdf.birch"
    if (lazy && \u03c0->supportsLazy()) {
      #line 100 "src/test/test_pdf.birch"
      p = exp(\u03c0->logpdfLazy(box(x)).value()->value());
    } else {
      #line 102 "src/test/test_pdf.birch"
      p = \u03c0->pdf(x);
    }
    auto \u03b4 = abs(p - scalar<Real>(count(x - from.value() + Integer(1))) / N);
    auto \u03b5 = 5.0 / sqrt(N);
    #line 106 "src/test/test_pdf.birch"
    if (Integer(10) < N && !((\u03b4 <= \u03b5))) {
      #line 107 "src/test/test_pdf.birch"
      failed = true;
      #line 108 "src/test/test_pdf.birch"
      stderr_->print(String("***failed*** on value ") + x + String(", ") + \u03b4 + String(" > ") + \u03b5 + String("\n"));
    }
  }
  #line 111 "src/test/test_pdf.birch"
  if (failed) {
    #line 112 "src/test/test_pdf.birch"
    exit(Integer(1));
  }
}

#line 125 "src/test/test_pdf.birch"
#line 125 "src/test/test_pdf.birch"
void test_pdf(const Distribution<Real>& \u03c0, const Integer& N, const Integer& B, const Integer& S, const Boolean& lazy) {
  libbirch::DefaultArray<Real,1> x1(libbirch::make_shape(N));
  libbirch::DefaultArray<Real,1> x2(libbirch::make_shape(N));
  #line 131 "src/test/test_pdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    #line 132 "src/test/test_pdf.birch"
    x1(n) = \u03c0->simulate();
  }
  auto \u03bc = sum(x1) / N;
  auto \u03c32 = dot(x1) / N - \u03bc * \u03bc;
  auto done = false;
  #line 141 "src/test/test_pdf.birch"
  do {
    auto a = 0.0;
    Real x;
    #line 144 "src/test/test_pdf.birch"
    if (lazy && \u03c0->supportsLazy()) {
      #line 145 "src/test/test_pdf.birch"
      x = \u03c0->simulateLazy().value();
    } else {
      #line 147 "src/test/test_pdf.birch"
      x = \u03c0->simulate();
    }
    Real l;
    #line 150 "src/test/test_pdf.birch"
    if (lazy && \u03c0->supportsLazy()) {
      #line 151 "src/test/test_pdf.birch"
      l = \u03c0->logpdfLazy(box(x)).value()->value();
    } else {
      #line 153 "src/test/test_pdf.birch"
      l = \u03c0->logpdf(x);
    }
    #line 156 "src/test/test_pdf.birch"
    for (auto n = Integer(1); n <= B; ++n) {
      auto x_prime_ = simulate_gaussian(x, \u03c32);
      Real l_prime_;
      #line 159 "src/test/test_pdf.birch"
      if (lazy && \u03c0->supportsLazy()) {
        #line 160 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdfLazy(box(x_prime_)).value()->value();
      } else {
        #line 162 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdf(x_prime_);
      }
      #line 164 "src/test/test_pdf.birch"
      if (log(simulate_uniform(0.0, 1.0)) <= l_prime_ - l) {
        #line 166 "src/test/test_pdf.birch"
        x = x_prime_;
        #line 167 "src/test/test_pdf.birch"
        l = l_prime_;
        #line 168 "src/test/test_pdf.birch"
        a = a + 1.0 / B;
      }
    }
    #line 172 "src/test/test_pdf.birch"
    if (a < 0.4) {
      #line 173 "src/test/test_pdf.birch"
      \u03c32 = 0.5 * \u03c32;
    } else {
      #line 174 "src/test/test_pdf.birch"
      if (a > 0.5) {
        #line 175 "src/test/test_pdf.birch"
        \u03c32 = 1.5 * \u03c32;
      } else {
        #line 177 "src/test/test_pdf.birch"
        done = true;
      }
    }
  } while (!(done));
  auto a = 0.0;
  Real x;
  #line 184 "src/test/test_pdf.birch"
  if (lazy && \u03c0->supportsLazy()) {
    #line 185 "src/test/test_pdf.birch"
    x = \u03c0->simulateLazy().value();
  } else {
    #line 187 "src/test/test_pdf.birch"
    x = \u03c0->simulate();
  }
  Real l;
  #line 190 "src/test/test_pdf.birch"
  if (lazy && \u03c0->supportsLazy()) {
    #line 191 "src/test/test_pdf.birch"
    l = \u03c0->logpdfLazy(box(x)).value()->value();
  } else {
    #line 193 "src/test/test_pdf.birch"
    l = \u03c0->logpdf(x);
  }
  #line 195 "src/test/test_pdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    #line 196 "src/test/test_pdf.birch"
    for (auto s = Integer(1); s <= S; ++s) {
      auto x_prime_ = simulate_gaussian(x, \u03c32);
      Real l_prime_;
      #line 199 "src/test/test_pdf.birch"
      if (lazy && \u03c0->supportsLazy()) {
        #line 200 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdfLazy(box(x_prime_)).value()->value();
      } else {
        #line 202 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdf(x_prime_);
      }
      #line 204 "src/test/test_pdf.birch"
      if (log(simulate_uniform(0.0, 1.0)) <= l_prime_ - l) {
        #line 206 "src/test/test_pdf.birch"
        x = x_prime_;
        #line 207 "src/test/test_pdf.birch"
        l = l_prime_;
        #line 208 "src/test/test_pdf.birch"
        a = a + 1.0 / (N * S);
      }
    }
    #line 211 "src/test/test_pdf.birch"
    x2(n) = x;
  }
  #line 216 "src/test/test_pdf.birch"
  if (!(pass(x1, x2))) {
    #line 217 "src/test/test_pdf.birch"
    exit(Integer(1));
  }
}

#line 230 "src/test/test_pdf.birch"
#line 230 "src/test/test_pdf.birch"
void test_pdf(const Distribution<libbirch::DefaultArray<Real,1>>& \u03c0, const Integer& N, const Integer& B, const Integer& S, const Boolean& lazy) {
  auto D = rows(\u03c0->simulate());
  libbirch::DefaultArray<Real,2> X1(libbirch::make_shape(N, D));
  libbirch::DefaultArray<Real,2> X2(libbirch::make_shape(N, D));
  #line 238 "src/test/test_pdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    #line 239 "src/test/test_pdf.birch"
    X1(n, libbirch::make_range(Integer(1), D)) = \u03c0->simulate();
  }
  auto \u03bc = vector(0.0, D);
  auto \u03a3 = matrix<Real>(0.0, D, D);
  #line 245 "src/test/test_pdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    auto x = X1(n, libbirch::make_range(Integer(1), D));
    #line 247 "src/test/test_pdf.birch"
    \u03bc = \u03bc + x;
    #line 248 "src/test/test_pdf.birch"
    \u03a3 = \u03a3 + outer(x);
  }
  #line 250 "src/test/test_pdf.birch"
  \u03bc = \u03bc / N;
  #line 251 "src/test/test_pdf.birch"
  \u03a3 = \u03a3 / N - outer(\u03bc);
  auto done = false;
  #line 255 "src/test/test_pdf.birch"
  do {
    auto a = 0.0;
    libbirch::DefaultArray<Real,1> x;
    #line 258 "src/test/test_pdf.birch"
    if (lazy && \u03c0->supportsLazy()) {
      #line 259 "src/test/test_pdf.birch"
      x = \u03c0->simulateLazy().value();
    } else {
      #line 261 "src/test/test_pdf.birch"
      x = \u03c0->simulate();
    }
    Real l;
    #line 264 "src/test/test_pdf.birch"
    if (lazy && \u03c0->supportsLazy()) {
      #line 265 "src/test/test_pdf.birch"
      l = \u03c0->logpdfLazy(box(x)).value()->value();
    } else {
      #line 267 "src/test/test_pdf.birch"
      l = \u03c0->logpdf(x);
    }
    #line 270 "src/test/test_pdf.birch"
    for (auto n = Integer(1); n <= B; ++n) {
      auto x_prime_ = simulate_multivariate_gaussian(x, \u03a3);
      Real l_prime_;
      #line 273 "src/test/test_pdf.birch"
      if (lazy && \u03c0->supportsLazy()) {
        #line 274 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdfLazy(box(x_prime_)).value()->value();
      } else {
        #line 276 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdf(x_prime_);
      }
      #line 278 "src/test/test_pdf.birch"
      if (log(simulate_uniform(0.0, 1.0)) <= l_prime_ - l) {
        #line 280 "src/test/test_pdf.birch"
        x = x_prime_;
        #line 281 "src/test/test_pdf.birch"
        l = l_prime_;
        #line 282 "src/test/test_pdf.birch"
        a = a + 1.0 / B;
      }
    }
    #line 286 "src/test/test_pdf.birch"
    if (a < 0.2) {
      #line 287 "src/test/test_pdf.birch"
      \u03a3 = 0.5 * \u03a3;
    } else {
      #line 288 "src/test/test_pdf.birch"
      if (a > 0.25) {
        #line 289 "src/test/test_pdf.birch"
        \u03a3 = 1.5 * \u03a3;
      } else {
        #line 291 "src/test/test_pdf.birch"
        done = true;
      }
    }
  } while (!(done));
  auto a = 0.0;
  libbirch::DefaultArray<Real,1> x;
  #line 298 "src/test/test_pdf.birch"
  if (lazy && \u03c0->supportsLazy()) {
    #line 299 "src/test/test_pdf.birch"
    x = \u03c0->simulateLazy().value();
  } else {
    #line 301 "src/test/test_pdf.birch"
    x = \u03c0->simulate();
  }
  Real l;
  #line 304 "src/test/test_pdf.birch"
  if (lazy && \u03c0->supportsLazy()) {
    #line 305 "src/test/test_pdf.birch"
    l = \u03c0->logpdfLazy(box(x)).value()->value();
  } else {
    #line 307 "src/test/test_pdf.birch"
    l = \u03c0->logpdf(x);
  }
  #line 309 "src/test/test_pdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    #line 310 "src/test/test_pdf.birch"
    for (auto s = Integer(1); s <= S; ++s) {
      auto x_prime_ = simulate_multivariate_gaussian(x, \u03a3);
      Real l_prime_;
      #line 313 "src/test/test_pdf.birch"
      if (lazy && \u03c0->supportsLazy()) {
        #line 314 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdfLazy(box(x_prime_)).value()->value();
      } else {
        #line 316 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdf(x_prime_);
      }
      #line 318 "src/test/test_pdf.birch"
      if (log(simulate_uniform(0.0, 1.0)) <= l_prime_ - l) {
        #line 320 "src/test/test_pdf.birch"
        x = x_prime_;
        #line 321 "src/test/test_pdf.birch"
        l = l_prime_;
        #line 322 "src/test/test_pdf.birch"
        a = a + 1.0 / (N * S);
      }
    }
    #line 325 "src/test/test_pdf.birch"
    X2(n, libbirch::make_range(Integer(1), D)) = x;
  }
  #line 330 "src/test/test_pdf.birch"
  if (!(pass(X1, X2))) {
    #line 331 "src/test/test_pdf.birch"
    exit(Integer(1));
  }
}

#line 344 "src/test/test_pdf.birch"
#line 344 "src/test/test_pdf.birch"
void test_pdf(const Distribution<libbirch::DefaultArray<Real,2>>& \u03c0, const Integer& N, const Integer& B, const Integer& S, const Boolean& lazy) {
  auto R = rows(\u03c0->simulate());
  auto C = columns(\u03c0->simulate());
  libbirch::DefaultArray<Real,2> X1(libbirch::make_shape(N, R * C));
  libbirch::DefaultArray<Real,2> X2(libbirch::make_shape(N, R * C));
  #line 353 "src/test/test_pdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    #line 354 "src/test/test_pdf.birch"
    X1(n, libbirch::make_range(Integer(1), R * C)) = vec(\u03c0->simulate());
  }
  auto \u03bc = vector(0.0, R * C);
  auto \u03a3 = matrix<Real>(0.0, R * C, R * C);
  #line 360 "src/test/test_pdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    auto x = X1(n, libbirch::make_range(Integer(1), R * C));
    #line 362 "src/test/test_pdf.birch"
    \u03bc = \u03bc + x;
    #line 363 "src/test/test_pdf.birch"
    \u03a3 = \u03a3 + outer(x);
  }
  #line 365 "src/test/test_pdf.birch"
  \u03bc = \u03bc / N;
  #line 366 "src/test/test_pdf.birch"
  \u03a3 = \u03a3 / N - outer(\u03bc);
  auto done = false;
  #line 370 "src/test/test_pdf.birch"
  do {
    auto a = 0.0;
    libbirch::DefaultArray<Real,2> x;
    Real l;
    #line 374 "src/test/test_pdf.birch"
    if (lazy && \u03c0->supportsLazy()) {
      #line 375 "src/test/test_pdf.birch"
      x = \u03c0->simulateLazy().value();
      #line 376 "src/test/test_pdf.birch"
      l = \u03c0->logpdfLazy(box(x)).value()->value();
    } else {
      #line 378 "src/test/test_pdf.birch"
      x = \u03c0->simulate();
      #line 379 "src/test/test_pdf.birch"
      l = \u03c0->logpdf(x);
    }
    #line 381 "src/test/test_pdf.birch"
    for (auto n = Integer(1); n <= B; ++n) {
      auto x_prime_ = mat(simulate_multivariate_gaussian(vec(x), \u03a3), C);
      Real l_prime_;
      #line 384 "src/test/test_pdf.birch"
      if (lazy && \u03c0->supportsLazy()) {
        #line 385 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdfLazy(box(x_prime_)).value()->value();
      } else {
        #line 387 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdf(x_prime_);
      }
      #line 389 "src/test/test_pdf.birch"
      if (log(simulate_uniform(0.0, 1.0)) <= l_prime_ - l) {
        #line 391 "src/test/test_pdf.birch"
        x = x_prime_;
        #line 392 "src/test/test_pdf.birch"
        l = l_prime_;
        #line 393 "src/test/test_pdf.birch"
        a = a + 1.0 / B;
      }
    }
    #line 397 "src/test/test_pdf.birch"
    if (a < 0.2) {
      #line 398 "src/test/test_pdf.birch"
      \u03a3 = 0.5 * \u03a3;
    } else {
      #line 399 "src/test/test_pdf.birch"
      if (a > 0.25) {
        #line 400 "src/test/test_pdf.birch"
        \u03a3 = 1.5 * \u03a3;
      } else {
        #line 402 "src/test/test_pdf.birch"
        done = true;
      }
    }
  } while (!(done));
  auto a = 0.0;
  libbirch::DefaultArray<Real,2> x;
  Real l;
  #line 410 "src/test/test_pdf.birch"
  if (lazy && \u03c0->supportsLazy()) {
    #line 411 "src/test/test_pdf.birch"
    x = \u03c0->simulateLazy().value();
    #line 412 "src/test/test_pdf.birch"
    l = \u03c0->logpdfLazy(box(x)).value()->value();
  } else {
    #line 414 "src/test/test_pdf.birch"
    x = \u03c0->simulate();
    #line 415 "src/test/test_pdf.birch"
    l = \u03c0->logpdf(x);
  }
  #line 417 "src/test/test_pdf.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    #line 418 "src/test/test_pdf.birch"
    for (auto s = Integer(1); s <= S; ++s) {
      auto x_prime_ = mat(simulate_multivariate_gaussian(vec(x), \u03a3), C);
      Real l_prime_;
      #line 421 "src/test/test_pdf.birch"
      if (lazy && \u03c0->supportsLazy()) {
        #line 422 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdfLazy(box(x_prime_)).value()->value();
      } else {
        #line 424 "src/test/test_pdf.birch"
        l_prime_ = \u03c0->logpdf(x_prime_);
      }
      #line 426 "src/test/test_pdf.birch"
      if (log(simulate_uniform(0.0, 1.0)) <= l_prime_ - l) {
        #line 428 "src/test/test_pdf.birch"
        x = x_prime_;
        #line 429 "src/test/test_pdf.birch"
        l = l_prime_;
        #line 430 "src/test/test_pdf.birch"
        a = a + 1.0 / (N * S);
      }
    }
    #line 433 "src/test/test_pdf.birch"
    X2(n, libbirch::make_range(Integer(1), R * C)) = vec(x);
  }
  #line 438 "src/test/test_pdf.birch"
  if (!(pass(X1, X2))) {
    #line 439 "src/test/test_pdf.birch"
    exit(Integer(1));
  }
}

}
