#ifndef BIRCH_STANDARD_HPP
#define BIRCH_STANDARD_HPP

#include "libbirch.hpp"

#line 1 "src/external.birch"

#include <random>
#include <string>
#include <sstream>
#include <iomanip>
#include <cmath>

#include <dlfcn.h>
#include <unistd.h>
#include <yaml.h>
#include <getopt.h>

#ifdef __FreeBSD__
#include <sys/wait.h>  /* for WIF.. */
#endif
#line 1 "src/basic/basic.birch"

using Boolean = bool;
using Real = double;
using Integer = std::int64_t;
using String = std::string;
using File = FILE*;
#line 1 "src/math/rng.birch"

/*
 * Pseudorandom number generator for each thread.
 */
extern thread_local std::mt19937_64 rng;
namespace birch {
#line 4 "src/basic/Object.birch"
#line 4 "src/basic/Object.birch"
class Object_;
#line 4 "src/basic/Object.birch"
#line 4 "src/basic/Object.birch"
using Object = libbirch::Shared<Object_>;
#line 6 "src/container/Array.birch"
template<class Type>
#line 6 "src/container/Array.birch"
class Array_;
#line 6 "src/container/Array.birch"
template<class Type>
#line 6 "src/container/Array.birch"
using Array = libbirch::Shared<Array_<Type>>;
#line 30 "src/container/List.birch"
template<class Type>
#line 30 "src/container/List.birch"
class List_;
#line 30 "src/container/List.birch"
template<class Type>
#line 30 "src/container/List.birch"
using List = libbirch::Shared<List_<Type>>;
#line 4 "src/container/ListNode.birch"
template<class Type>
#line 4 "src/container/ListNode.birch"
class ListNode_;
#line 4 "src/container/ListNode.birch"
template<class Type>
#line 4 "src/container/ListNode.birch"
using ListNode = libbirch::Shared<ListNode_<Type>>;
#line 7 "src/container/RaggedArray.birch"
template<class Type>
#line 7 "src/container/RaggedArray.birch"
class RaggedArray_;
#line 7 "src/container/RaggedArray.birch"
template<class Type>
#line 7 "src/container/RaggedArray.birch"
using RaggedArray = libbirch::Shared<RaggedArray_<Type>>;
#line 28 "src/container/Stack.birch"
template<class Type>
#line 28 "src/container/Stack.birch"
class Stack_;
#line 28 "src/container/Stack.birch"
template<class Type>
#line 28 "src/container/Stack.birch"
using Stack = libbirch::Shared<Stack_<Type>>;
#line 1 "src/container/StackNode.birch"
template<class Type>
#line 1 "src/container/StackNode.birch"
class StackNode_;
#line 1 "src/container/StackNode.birch"
template<class Type>
#line 1 "src/container/StackNode.birch"
using StackNode = libbirch::Shared<StackNode_<Type>>;
#line 40 "src/container/Tape.birch"
template<class Type>
#line 40 "src/container/Tape.birch"
class Tape_;
#line 40 "src/container/Tape.birch"
template<class Type>
#line 40 "src/container/Tape.birch"
using Tape = libbirch::Shared<Tape_<Type>>;
#line 1 "src/container/TapeNode.birch"
template<class Type>
#line 1 "src/container/TapeNode.birch"
class TapeNode_;
#line 1 "src/container/TapeNode.birch"
template<class Type>
#line 1 "src/container/TapeNode.birch"
using TapeNode = libbirch::Shared<TapeNode_<Type>>;
#line 1 "src/data/Buffer.birch"
#line 1 "src/data/Buffer.birch"
class Buffer_;
#line 1 "src/data/Buffer.birch"
#line 1 "src/data/Buffer.birch"
using Buffer = libbirch::Shared<Buffer_>;
#line 97 "src/delay/Delay.birch"
#line 97 "src/delay/Delay.birch"
class Delay_;
#line 97 "src/delay/Delay.birch"
#line 97 "src/delay/Delay.birch"
using Delay = libbirch::Shared<Delay_>;
#line 6 "src/distribution/Distribution.birch"
template<class Value>
#line 6 "src/distribution/Distribution.birch"
class Distribution_;
#line 6 "src/distribution/Distribution.birch"
template<class Value>
#line 6 "src/distribution/Distribution.birch"
using Distribution = libbirch::Shared<Distribution_<Value>>;
#line 30 "src/event/Handler.birch"
#line 30 "src/event/Handler.birch"
class Handler_;
#line 30 "src/event/Handler.birch"
#line 30 "src/event/Handler.birch"
using Handler = libbirch::Shared<Handler_>;
#line 1 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 1 "src/expression/BoxedForm.birch"
class BoxedForm_;
#line 1 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 1 "src/expression/BoxedForm.birch"
using BoxedForm = libbirch::Shared<BoxedForm_<Value, Form>>;
#line 4 "src/expression/BoxedValue.birch"
template<class Value>
#line 4 "src/expression/BoxedValue.birch"
class BoxedValue_;
#line 4 "src/expression/BoxedValue.birch"
template<class Value>
#line 4 "src/expression/BoxedValue.birch"
using BoxedValue = libbirch::Shared<BoxedValue_<Value>>;
#line 93 "src/expression/Expression.birch"
template<class Value>
#line 93 "src/expression/Expression.birch"
class Expression_;
#line 93 "src/expression/Expression.birch"
template<class Value>
#line 93 "src/expression/Expression.birch"
using Expression = libbirch::Shared<Expression_<Value>>;
#line 7 "src/expression/MatrixSplitExpression.birch"
template<class Value,class Form>
#line 7 "src/expression/MatrixSplitExpression.birch"
class MatrixSplitExpression_;
#line 7 "src/expression/MatrixSplitExpression.birch"
template<class Value,class Form>
#line 7 "src/expression/MatrixSplitExpression.birch"
using MatrixSplitExpression = libbirch::Shared<MatrixSplitExpression_<Value, Form>>;
#line 13 "src/expression/Random.birch"
template<class Value>
#line 13 "src/expression/Random.birch"
class Random_;
#line 13 "src/expression/Random.birch"
template<class Value>
#line 13 "src/expression/Random.birch"
using Random = libbirch::Shared<Random_<Value>>;
#line 7 "src/expression/VectorSplitExpression.birch"
template<class Value,class Form>
#line 7 "src/expression/VectorSplitExpression.birch"
class VectorSplitExpression_;
#line 7 "src/expression/VectorSplitExpression.birch"
template<class Value,class Form>
#line 7 "src/expression/VectorSplitExpression.birch"
using VectorSplitExpression = libbirch::Shared<VectorSplitExpression_<Value, Form>>;
#line 1 "src/filter/AliveParticleFilter.birch"
#line 1 "src/filter/AliveParticleFilter.birch"
class AliveParticleFilter_;
#line 1 "src/filter/AliveParticleFilter.birch"
#line 1 "src/filter/AliveParticleFilter.birch"
using AliveParticleFilter = libbirch::Shared<AliveParticleFilter_>;
#line 1 "src/filter/Particle.birch"
#line 1 "src/filter/Particle.birch"
class Particle_;
#line 1 "src/filter/Particle.birch"
#line 1 "src/filter/Particle.birch"
using Particle = libbirch::Shared<Particle_>;
#line 1 "src/filter/ParticleFilter.birch"
#line 1 "src/filter/ParticleFilter.birch"
class ParticleFilter_;
#line 1 "src/filter/ParticleFilter.birch"
#line 1 "src/filter/ParticleFilter.birch"
using ParticleFilter = libbirch::Shared<ParticleFilter_>;
#line 4 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 4 "src/form/Binary.birch"
struct Binary_;
#line 4 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 4 "src/form/Binary.birch"
using Binary = libbirch::Inplace<Binary_<Function, Left, Right>>;
#line 4 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 4 "src/form/BinaryMemoized.birch"
struct BinaryMemoized_;
#line 4 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 4 "src/form/BinaryMemoized.birch"
using BinaryMemoized = libbirch::Inplace<BinaryMemoized_<Function, Left, Right>>;
#line 5 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 5 "src/form/Linear.birch"
struct Linear_;
#line 5 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 5 "src/form/Linear.birch"
using Linear = libbirch::Inplace<Linear_<Function, Left, Right>>;
#line 5 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 5 "src/form/LinearMemoized.birch"
struct LinearMemoized_;
#line 5 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 5 "src/form/LinearMemoized.birch"
using LinearMemoized = libbirch::Inplace<LinearMemoized_<Function, Left, Right>>;
#line 4 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 4 "src/form/MatrixElement.birch"
struct MatrixElement_;
#line 4 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 4 "src/form/MatrixElement.birch"
using MatrixElement = libbirch::Inplace<MatrixElement_<Left, Middle, Right>>;
#line 6 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 6 "src/form/MatrixJoinForm.birch"
struct MatrixJoinForm_;
#line 6 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 6 "src/form/MatrixJoinForm.birch"
using MatrixJoinForm = libbirch::Inplace<MatrixJoinForm_<Value>>;
#line 4 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 4 "src/form/Ternary.birch"
struct Ternary_;
#line 4 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 4 "src/form/Ternary.birch"
using Ternary = libbirch::Inplace<Ternary_<Function, Left, Middle, Right>>;
#line 4 "src/form/Unary.birch"
template<class Function,class Middle>
#line 4 "src/form/Unary.birch"
struct Unary_;
#line 4 "src/form/Unary.birch"
template<class Function,class Middle>
#line 4 "src/form/Unary.birch"
using Unary = libbirch::Inplace<Unary_<Function, Middle>>;
#line 4 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 4 "src/form/UnaryMemoized.birch"
struct UnaryMemoized_;
#line 4 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 4 "src/form/UnaryMemoized.birch"
using UnaryMemoized = libbirch::Inplace<UnaryMemoized_<Function, Middle>>;
#line 4 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 4 "src/form/VectorElement.birch"
struct VectorElement_;
#line 4 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 4 "src/form/VectorElement.birch"
using VectorElement = libbirch::Inplace<VectorElement_<Left, Right>>;
#line 6 "src/form/VectorJoinForm.birch"
template<class Value>
#line 6 "src/form/VectorJoinForm.birch"
struct VectorJoinForm_;
#line 6 "src/form/VectorJoinForm.birch"
template<class Value>
#line 6 "src/form/VectorJoinForm.birch"
using VectorJoinForm = libbirch::Inplace<VectorJoinForm_<Value>>;
#line 4 "src/io/InputStream.birch"
#line 4 "src/io/InputStream.birch"
struct InputStream_;
#line 4 "src/io/InputStream.birch"
#line 4 "src/io/InputStream.birch"
using InputStream = libbirch::Inplace<InputStream_>;
#line 1 "src/io/JSONWriter.birch"
#line 1 "src/io/JSONWriter.birch"
class JSONWriter_;
#line 1 "src/io/JSONWriter.birch"
#line 1 "src/io/JSONWriter.birch"
using JSONWriter = libbirch::Shared<JSONWriter_>;
#line 4 "src/io/OutputStream.birch"
#line 4 "src/io/OutputStream.birch"
struct OutputStream_;
#line 4 "src/io/OutputStream.birch"
#line 4 "src/io/OutputStream.birch"
using OutputStream = libbirch::Inplace<OutputStream_>;
#line 38 "src/io/Reader.birch"
#line 38 "src/io/Reader.birch"
class Reader_;
#line 38 "src/io/Reader.birch"
#line 38 "src/io/Reader.birch"
using Reader = libbirch::Shared<Reader_>;
#line 37 "src/io/Writer.birch"
#line 37 "src/io/Writer.birch"
class Writer_;
#line 37 "src/io/Writer.birch"
#line 37 "src/io/Writer.birch"
using Writer = libbirch::Shared<Writer_>;
#line 1 "src/io/YAMLReader.birch"
#line 1 "src/io/YAMLReader.birch"
class YAMLReader_;
#line 1 "src/io/YAMLReader.birch"
#line 1 "src/io/YAMLReader.birch"
using YAMLReader = libbirch::Shared<YAMLReader_>;
#line 1 "src/io/YAMLWriter.birch"
#line 1 "src/io/YAMLWriter.birch"
class YAMLWriter_;
#line 1 "src/io/YAMLWriter.birch"
#line 1 "src/io/YAMLWriter.birch"
using YAMLWriter = libbirch::Shared<YAMLWriter_>;
#line 7 "src/iterator/ArrayBufferIterator.birch"
#line 7 "src/iterator/ArrayBufferIterator.birch"
class ArrayBufferIterator_;
#line 7 "src/iterator/ArrayBufferIterator.birch"
#line 7 "src/iterator/ArrayBufferIterator.birch"
using ArrayBufferIterator = libbirch::Shared<ArrayBufferIterator_>;
#line 6 "src/iterator/ArrayIterator.birch"
template<class Type>
#line 6 "src/iterator/ArrayIterator.birch"
class ArrayIterator_;
#line 6 "src/iterator/ArrayIterator.birch"
template<class Type>
#line 6 "src/iterator/ArrayIterator.birch"
using ArrayIterator = libbirch::Shared<ArrayIterator_<Type>>;
#line 4 "src/iterator/EmptyIterator.birch"
template<class Type>
#line 4 "src/iterator/EmptyIterator.birch"
class EmptyIterator_;
#line 4 "src/iterator/EmptyIterator.birch"
template<class Type>
#line 4 "src/iterator/EmptyIterator.birch"
using EmptyIterator = libbirch::Shared<EmptyIterator_<Type>>;
#line 4 "src/iterator/Iterator.birch"
template<class Type>
#line 4 "src/iterator/Iterator.birch"
class Iterator_;
#line 4 "src/iterator/Iterator.birch"
template<class Type>
#line 4 "src/iterator/Iterator.birch"
using Iterator = libbirch::Shared<Iterator_<Type>>;
#line 6 "src/iterator/ListIterator.birch"
template<class Type>
#line 6 "src/iterator/ListIterator.birch"
class ListIterator_;
#line 6 "src/iterator/ListIterator.birch"
template<class Type>
#line 6 "src/iterator/ListIterator.birch"
using ListIterator = libbirch::Shared<ListIterator_<Type>>;
#line 7 "src/iterator/MatrixBufferIterator.birch"
template<class Type>
#line 7 "src/iterator/MatrixBufferIterator.birch"
class MatrixBufferIterator_;
#line 7 "src/iterator/MatrixBufferIterator.birch"
template<class Type>
#line 7 "src/iterator/MatrixBufferIterator.birch"
using MatrixBufferIterator = libbirch::Shared<MatrixBufferIterator_<Type>>;
#line 8 "src/iterator/ObjectBufferIterator.birch"
#line 8 "src/iterator/ObjectBufferIterator.birch"
class ObjectBufferIterator_;
#line 8 "src/iterator/ObjectBufferIterator.birch"
#line 8 "src/iterator/ObjectBufferIterator.birch"
using ObjectBufferIterator = libbirch::Shared<ObjectBufferIterator_>;
#line 7 "src/iterator/RaggedArrayElementIterator.birch"
template<class Type>
#line 7 "src/iterator/RaggedArrayElementIterator.birch"
class RaggedArrayElementIterator_;
#line 7 "src/iterator/RaggedArrayElementIterator.birch"
template<class Type>
#line 7 "src/iterator/RaggedArrayElementIterator.birch"
using RaggedArrayElementIterator = libbirch::Shared<RaggedArrayElementIterator_<Type>>;
#line 6 "src/iterator/RaggedArrayRowIterator.birch"
template<class Type>
#line 6 "src/iterator/RaggedArrayRowIterator.birch"
class RaggedArrayRowIterator_;
#line 6 "src/iterator/RaggedArrayRowIterator.birch"
template<class Type>
#line 6 "src/iterator/RaggedArrayRowIterator.birch"
using RaggedArrayRowIterator = libbirch::Shared<RaggedArrayRowIterator_<Type>>;
#line 6 "src/iterator/ScalarBufferIterator.birch"
template<class Type>
#line 6 "src/iterator/ScalarBufferIterator.birch"
class ScalarBufferIterator_;
#line 6 "src/iterator/ScalarBufferIterator.birch"
template<class Type>
#line 6 "src/iterator/ScalarBufferIterator.birch"
using ScalarBufferIterator = libbirch::Shared<ScalarBufferIterator_<Type>>;
#line 6 "src/iterator/StackIterator.birch"
template<class Type>
#line 6 "src/iterator/StackIterator.birch"
class StackIterator_;
#line 6 "src/iterator/StackIterator.birch"
template<class Type>
#line 6 "src/iterator/StackIterator.birch"
using StackIterator = libbirch::Shared<StackIterator_<Type>>;
#line 6 "src/iterator/TapeIterator.birch"
template<class Type>
#line 6 "src/iterator/TapeIterator.birch"
class TapeIterator_;
#line 6 "src/iterator/TapeIterator.birch"
template<class Type>
#line 6 "src/iterator/TapeIterator.birch"
using TapeIterator = libbirch::Shared<TapeIterator_<Type>>;
#line 7 "src/iterator/VectorBufferIterator.birch"
template<class Type>
#line 7 "src/iterator/VectorBufferIterator.birch"
class VectorBufferIterator_;
#line 7 "src/iterator/VectorBufferIterator.birch"
template<class Type>
#line 7 "src/iterator/VectorBufferIterator.birch"
using VectorBufferIterator = libbirch::Shared<VectorBufferIterator_<Type>>;
#line 18 "src/kernel/Kernel.birch"
#line 18 "src/kernel/Kernel.birch"
class Kernel_;
#line 18 "src/kernel/Kernel.birch"
#line 18 "src/kernel/Kernel.birch"
using Kernel = libbirch::Shared<Kernel_>;
#line 1 "src/kernel/LangevinKernel.birch"
#line 1 "src/kernel/LangevinKernel.birch"
class LangevinKernel_;
#line 1 "src/kernel/LangevinKernel.birch"
#line 1 "src/kernel/LangevinKernel.birch"
using LangevinKernel = libbirch::Shared<LangevinKernel_>;
#line 4 "src/model/Model.birch"
#line 4 "src/model/Model.birch"
class Model_;
#line 4 "src/model/Model.birch"
#line 4 "src/model/Model.birch"
using Model = libbirch::Shared<Model_>;
#line 1 "src/sampler/ParticleSampler.birch"
#line 1 "src/sampler/ParticleSampler.birch"
class ParticleSampler_;
#line 1 "src/sampler/ParticleSampler.birch"
#line 1 "src/sampler/ParticleSampler.birch"
using ParticleSampler = libbirch::Shared<ParticleSampler_>;
#line 1 "src/utility/ProgressBar.birch"
#line 1 "src/utility/ProgressBar.birch"
class ProgressBar_;
#line 1 "src/utility/ProgressBar.birch"
#line 1 "src/utility/ProgressBar.birch"
using ProgressBar = libbirch::Shared<ProgressBar_>;
#line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
class MatrixGaussianDistribution_;
#line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
using MatrixGaussianDistribution = libbirch::Shared<MatrixGaussianDistribution_<Arg1, Arg2, Arg3>>;
#line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
class LinearMatrixNormalInverseWishartMatrixGaussianDistribution_;
#line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
using LinearMatrixNormalInverseWishartMatrixGaussianDistribution = libbirch::Shared<LinearMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>>;
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
class MatrixNormalInverseWishartDistribution_;
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
using MatrixNormalInverseWishartDistribution = libbirch::Shared<MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>>;
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
class MatrixNormalInverseWishartMatrixGaussianDistribution_;
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
using MatrixNormalInverseWishartMatrixGaussianDistribution = libbirch::Shared<MatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>>;
#line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
class ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_;
#line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
using ScaledMatrixNormalInverseWishartMatrixGaussianDistribution = libbirch::Shared<ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>>;
#line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
class InverseWishartDistribution_;
#line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
using InverseWishartDistribution = libbirch::Shared<InverseWishartDistribution_<Arg1, Arg2>>;
#line 4 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/matrix_other/WishartDistribution.birch"
class WishartDistribution_;
#line 4 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/matrix_other/WishartDistribution.birch"
using WishartDistribution = libbirch::Shared<WishartDistribution_<Arg1, Arg2>>;
#line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
class DirichletMultinomialDistribution_;
#line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
using DirichletMultinomialDistribution = libbirch::Shared<DirichletMultinomialDistribution_<Arg1, Arg2>>;
#line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
class MultinomialDistribution_;
#line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
using MultinomialDistribution = libbirch::Shared<MultinomialDistribution_<Arg1, Arg2>>;
#line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
class LinearMultivariateGaussianMultivariateGaussianDistribution_;
#line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
using LinearMultivariateGaussianMultivariateGaussianDistribution = libbirch::Shared<LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>>;
#line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
class MultivariateGaussianDistribution_;
#line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
using MultivariateGaussianDistribution = libbirch::Shared<MultivariateGaussianDistribution_<Arg1, Arg2>>;
#line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
class MultivariateGaussianMultivariateGaussianDistribution_;
#line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
using MultivariateGaussianMultivariateGaussianDistribution = libbirch::Shared<MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>>;
#line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
class ScaledMultivariateGaussianMultivariateGaussianDistribution_;
#line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
using ScaledMultivariateGaussianMultivariateGaussianDistribution = libbirch::Shared<ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>>;
#line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
class LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_;
#line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
using LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution = libbirch::Shared<LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>>;
#line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
class MultivariateNormalInverseGammaDistribution_;
#line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
using MultivariateNormalInverseGammaDistribution = libbirch::Shared<MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>>;
#line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
class MultivariateNormalInverseGammaMultivariateGaussianDistribution_;
#line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
using MultivariateNormalInverseGammaMultivariateGaussianDistribution = libbirch::Shared<MultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>>;
#line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
class ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_;
#line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
using ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution = libbirch::Shared<ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>>;
#line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
class LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_;
#line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
using LinearMatrixNormalInverseWishartMultivariateGaussianDistribution = libbirch::Shared<LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>>;
#line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
class ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_;
#line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
using ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution = libbirch::Shared<ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>>;
#line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
class DirichletDistribution_;
#line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
using DirichletDistribution = libbirch::Shared<DirichletDistribution_<Arg>>;
#line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
class RestaurantDistribution_;
#line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
using RestaurantDistribution = libbirch::Shared<RestaurantDistribution_<Arg1, Arg2>>;
#line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
class BernoulliDistribution_;
#line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
using BernoulliDistribution = libbirch::Shared<BernoulliDistribution_<Arg>>;
#line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
class BetaBernoulliDistribution_;
#line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
using BetaBernoulliDistribution = libbirch::Shared<BetaBernoulliDistribution_<Arg1, Arg2>>;
#line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
class BetaBinomialDistribution_;
#line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
using BetaBinomialDistribution = libbirch::Shared<BetaBinomialDistribution_<Arg1, Arg2, Arg3>>;
#line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
class BetaNegativeBinomialDistribution_;
#line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
using BetaNegativeBinomialDistribution = libbirch::Shared<BetaNegativeBinomialDistribution_<Arg1, Arg2, Arg3>>;
#line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
class BinomialDistribution_;
#line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
using BinomialDistribution = libbirch::Shared<BinomialDistribution_<Arg1, Arg2>>;
#line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
#line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
class BoundedDiscreteDistribution_;
#line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
#line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
using BoundedDiscreteDistribution = libbirch::Shared<BoundedDiscreteDistribution_>;
#line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
class CategoricalDistribution_;
#line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
using CategoricalDistribution = libbirch::Shared<CategoricalDistribution_<Arg>>;
#line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
class ConwayMaxwellPoissonDistribution_;
#line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
using ConwayMaxwellPoissonDistribution = libbirch::Shared<ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>>;
#line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
class DirichletCategoricalDistribution_;
#line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
using DirichletCategoricalDistribution = libbirch::Shared<DirichletCategoricalDistribution_<Arg>>;
#line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
#line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
class DiscreteDistribution_;
#line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
#line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
using DiscreteDistribution = libbirch::Shared<DiscreteDistribution_>;
#line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
class GammaPoissonDistribution_;
#line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
using GammaPoissonDistribution = libbirch::Shared<GammaPoissonDistribution_<Arg1, Arg2>>;
#line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
class NegativeBinomialDistribution_;
#line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
using NegativeBinomialDistribution = libbirch::Shared<NegativeBinomialDistribution_<Arg1, Arg2>>;
#line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
class PoissonDistribution_;
#line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
using PoissonDistribution = libbirch::Shared<PoissonDistribution_<Arg>>;
#line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
class RestaurantCategoricalDistribution_;
#line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
using RestaurantCategoricalDistribution = libbirch::Shared<RestaurantCategoricalDistribution_<Arg1, Arg2, Arg3, Arg4>>;
#line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
class ScaledGammaPoissonDistribution_;
#line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
using ScaledGammaPoissonDistribution = libbirch::Shared<ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>>;
#line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
class UniformIntegerDistribution_;
#line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
using UniformIntegerDistribution = libbirch::Shared<UniformIntegerDistribution_<Arg1, Arg2>>;
#line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
#line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
class AddDiscreteConstrainedDistribution_;
#line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
#line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
using AddDiscreteConstrainedDistribution = libbirch::Shared<AddDiscreteConstrainedDistribution_>;
#line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
#line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
class AddDiscreteDeltaDistribution_;
#line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
#line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
using AddDiscreteDeltaDistribution = libbirch::Shared<AddDiscreteDeltaDistribution_>;
#line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
class DeltaDistribution_;
#line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
using DeltaDistribution = libbirch::Shared<DeltaDistribution_<Arg>>;
#line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
#line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
class DiscreteDeltaDistribution_;
#line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
#line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
using DiscreteDeltaDistribution = libbirch::Shared<DiscreteDeltaDistribution_>;
#line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
#line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
class NegateDiscreteDeltaDistribution_;
#line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
#line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
using NegateDiscreteDeltaDistribution = libbirch::Shared<NegateDiscreteDeltaDistribution_>;
#line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
#line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
class SubtractDiscreteConstrainedDistribution_;
#line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
#line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
using SubtractDiscreteConstrainedDistribution = libbirch::Shared<SubtractDiscreteConstrainedDistribution_>;
#line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
#line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
class SubtractDiscreteDeltaDistribution_;
#line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
#line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
using SubtractDiscreteDeltaDistribution = libbirch::Shared<SubtractDiscreteDeltaDistribution_>;
#line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
class GaussianDistribution_;
#line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
using GaussianDistribution = libbirch::Shared<GaussianDistribution_<Arg1, Arg2>>;
#line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
class GaussianGaussianDistribution_;
#line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
using GaussianGaussianDistribution = libbirch::Shared<GaussianGaussianDistribution_<Arg1, Arg2, Arg3>>;
#line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
class LinearGaussianGaussianDistribution_;
#line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
using LinearGaussianGaussianDistribution = libbirch::Shared<LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>>;
#line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
class LinearMultivariateGaussianGaussianDistribution_;
#line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
using LinearMultivariateGaussianGaussianDistribution = libbirch::Shared<LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>>;
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
class LinearMultivariateNormalInverseGammaGaussianDistribution_;
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
using LinearMultivariateNormalInverseGammaGaussianDistribution = libbirch::Shared<LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>>;
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
class LinearNormalInverseGammaGaussianDistribution_;
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
using LinearNormalInverseGammaGaussianDistribution = libbirch::Shared<LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>>;
#line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
class NormalInverseGammaDistribution_;
#line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
using NormalInverseGammaDistribution = libbirch::Shared<NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>>;
#line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
class NormalInverseGammaGaussianDistribution_;
#line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
using NormalInverseGammaGaussianDistribution = libbirch::Shared<NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>>;
#line 8 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/BetaDistribution.birch"
class BetaDistribution_;
#line 8 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/BetaDistribution.birch"
using BetaDistribution = libbirch::Shared<BetaDistribution_<Arg1, Arg2>>;
#line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
class ExponentialDistribution_;
#line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
using ExponentialDistribution = libbirch::Shared<ExponentialDistribution_<Arg1>>;
#line 8 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/GammaDistribution.birch"
class GammaDistribution_;
#line 8 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/GammaDistribution.birch"
using GammaDistribution = libbirch::Shared<GammaDistribution_<Arg1, Arg2>>;
#line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
class GammaExponentialDistribution_;
#line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
using GammaExponentialDistribution = libbirch::Shared<GammaExponentialDistribution_<Arg1, Arg2>>;
#line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
class InverseGammaDistribution_;
#line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
using InverseGammaDistribution = libbirch::Shared<InverseGammaDistribution_<Arg1, Arg2>>;
#line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
class InverseGammaGammaDistribution_;
#line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
using InverseGammaGammaDistribution = libbirch::Shared<InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>>;
#line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
class ScaledGammaExponentialDistribution_;
#line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
using ScaledGammaExponentialDistribution = libbirch::Shared<ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>>;
#line 8 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 8 "src/distribution/scalar_other/StudentDistribution.birch"
class StudentDistribution_;
#line 8 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 8 "src/distribution/scalar_other/StudentDistribution.birch"
using StudentDistribution = libbirch::Shared<StudentDistribution_<Arg1, Arg2, Arg3>>;
#line 8 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/UniformDistribution.birch"
class UniformDistribution_;
#line 8 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/UniformDistribution.birch"
using UniformDistribution = libbirch::Shared<UniformDistribution_<Arg1, Arg2>>;
#line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
class WeibullDistribution_;
#line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
using WeibullDistribution = libbirch::Shared<WeibullDistribution_<Arg1, Arg2>>;
#line 1 "src/form/binary_matrix/CholSolve.birch"
#line 1 "src/form/binary_matrix/CholSolve.birch"
struct CholSolveFunction_;
#line 1 "src/form/binary_matrix/CholSolve.birch"
#line 1 "src/form/binary_matrix/CholSolve.birch"
using CholSolveFunction = libbirch::Inplace<CholSolveFunction_>;
#line 1 "src/form/binary_matrix/Diagonal.birch"
#line 1 "src/form/binary_matrix/Diagonal.birch"
struct DiagonalFunction_;
#line 1 "src/form/binary_matrix/Diagonal.birch"
#line 1 "src/form/binary_matrix/Diagonal.birch"
using DiagonalFunction = libbirch::Inplace<DiagonalFunction_>;
#line 1 "src/form/binary_matrix/Dot.birch"
#line 1 "src/form/binary_matrix/Dot.birch"
struct DotFunction_;
#line 1 "src/form/binary_matrix/Dot.birch"
#line 1 "src/form/binary_matrix/Dot.birch"
using DotFunction = libbirch::Inplace<DotFunction_>;
#line 1 "src/form/binary_matrix/Inner.birch"
#line 1 "src/form/binary_matrix/Inner.birch"
struct InnerFunction_;
#line 1 "src/form/binary_matrix/Inner.birch"
#line 1 "src/form/binary_matrix/Inner.birch"
using InnerFunction = libbirch::Inplace<InnerFunction_>;
#line 1 "src/form/binary_matrix/MatrixPack.birch"
#line 1 "src/form/binary_matrix/MatrixPack.birch"
struct MatrixPackFunction_;
#line 1 "src/form/binary_matrix/MatrixPack.birch"
#line 1 "src/form/binary_matrix/MatrixPack.birch"
using MatrixPackFunction = libbirch::Inplace<MatrixPackFunction_>;
#line 1 "src/form/binary_matrix/MatrixStack.birch"
#line 1 "src/form/binary_matrix/MatrixStack.birch"
struct MatrixStackFunction_;
#line 1 "src/form/binary_matrix/MatrixStack.birch"
#line 1 "src/form/binary_matrix/MatrixStack.birch"
using MatrixStackFunction = libbirch::Inplace<MatrixStackFunction_>;
#line 1 "src/form/binary_matrix/Outer.birch"
#line 1 "src/form/binary_matrix/Outer.birch"
struct OuterFunction_;
#line 1 "src/form/binary_matrix/Outer.birch"
#line 1 "src/form/binary_matrix/Outer.birch"
using OuterFunction = libbirch::Inplace<OuterFunction_>;
#line 1 "src/form/binary_matrix/Solve.birch"
#line 1 "src/form/binary_matrix/Solve.birch"
struct SolveFunction_;
#line 1 "src/form/binary_matrix/Solve.birch"
#line 1 "src/form/binary_matrix/Solve.birch"
using SolveFunction = libbirch::Inplace<SolveFunction_>;
#line 1 "src/form/binary_other/Add.birch"
#line 1 "src/form/binary_other/Add.birch"
struct AddFunction_;
#line 1 "src/form/binary_other/Add.birch"
#line 1 "src/form/binary_other/Add.birch"
using AddFunction = libbirch::Inplace<AddFunction_>;
#line 1 "src/form/binary_other/CopySign.birch"
#line 1 "src/form/binary_other/CopySign.birch"
struct CopySignFunction_;
#line 1 "src/form/binary_other/CopySign.birch"
#line 1 "src/form/binary_other/CopySign.birch"
using CopySignFunction = libbirch::Inplace<CopySignFunction_>;
#line 1 "src/form/binary_other/Divide.birch"
#line 1 "src/form/binary_other/Divide.birch"
struct DivideFunction_;
#line 1 "src/form/binary_other/Divide.birch"
#line 1 "src/form/binary_other/Divide.birch"
using DivideFunction = libbirch::Inplace<DivideFunction_>;
#line 1 "src/form/binary_other/LessThan.birch"
#line 1 "src/form/binary_other/LessThan.birch"
struct LessThanFunction_;
#line 1 "src/form/binary_other/LessThan.birch"
#line 1 "src/form/binary_other/LessThan.birch"
using LessThanFunction = libbirch::Inplace<LessThanFunction_>;
#line 1 "src/form/binary_other/LogBeta.birch"
#line 1 "src/form/binary_other/LogBeta.birch"
struct LogBetaFunction_;
#line 1 "src/form/binary_other/LogBeta.birch"
#line 1 "src/form/binary_other/LogBeta.birch"
using LogBetaFunction = libbirch::Inplace<LogBetaFunction_>;
#line 1 "src/form/binary_other/LogChoose.birch"
#line 1 "src/form/binary_other/LogChoose.birch"
struct LogChooseFunction_;
#line 1 "src/form/binary_other/LogChoose.birch"
#line 1 "src/form/binary_other/LogChoose.birch"
using LogChooseFunction = libbirch::Inplace<LogChooseFunction_>;
#line 1 "src/form/binary_other/LogGammaP.birch"
#line 1 "src/form/binary_other/LogGammaP.birch"
struct LogGammaPFunction_;
#line 1 "src/form/binary_other/LogGammaP.birch"
#line 1 "src/form/binary_other/LogGammaP.birch"
using LogGammaPFunction = libbirch::Inplace<LogGammaPFunction_>;
#line 1 "src/form/binary_other/Multiply.birch"
#line 1 "src/form/binary_other/Multiply.birch"
struct MultiplyFunction_;
#line 1 "src/form/binary_other/Multiply.birch"
#line 1 "src/form/binary_other/Multiply.birch"
using MultiplyFunction = libbirch::Inplace<MultiplyFunction_>;
#line 1 "src/form/binary_other/Pow.birch"
#line 1 "src/form/binary_other/Pow.birch"
struct PowFunction_;
#line 1 "src/form/binary_other/Pow.birch"
#line 1 "src/form/binary_other/Pow.birch"
using PowFunction = libbirch::Inplace<PowFunction_>;
#line 1 "src/form/binary_other/Subtract.birch"
#line 1 "src/form/binary_other/Subtract.birch"
struct SubtractFunction_;
#line 1 "src/form/binary_other/Subtract.birch"
#line 1 "src/form/binary_other/Subtract.birch"
using SubtractFunction = libbirch::Inplace<SubtractFunction_>;
#line 1 "src/form/ternary/IfThenElse.birch"
#line 1 "src/form/ternary/IfThenElse.birch"
struct IfThenElseFunction_;
#line 1 "src/form/ternary/IfThenElse.birch"
#line 1 "src/form/ternary/IfThenElse.birch"
using IfThenElseFunction = libbirch::Inplace<IfThenElseFunction_>;
#line 1 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
#line 1 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
struct LogZConwayMaxwellPoissonFunction_;
#line 1 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
#line 1 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
using LogZConwayMaxwellPoissonFunction = libbirch::Inplace<LogZConwayMaxwellPoissonFunction_>;
#line 1 "src/form/unary_matrix/CholInv.birch"
#line 1 "src/form/unary_matrix/CholInv.birch"
struct CholInvFunction_;
#line 1 "src/form/unary_matrix/CholInv.birch"
#line 1 "src/form/unary_matrix/CholInv.birch"
using CholInvFunction = libbirch::Inplace<CholInvFunction_>;
#line 1 "src/form/unary_matrix/Inv.birch"
#line 1 "src/form/unary_matrix/Inv.birch"
struct InvFunction_;
#line 1 "src/form/unary_matrix/Inv.birch"
#line 1 "src/form/unary_matrix/Inv.birch"
using InvFunction = libbirch::Inplace<InvFunction_>;
#line 1 "src/form/unary_matrix/LCholDet.birch"
#line 1 "src/form/unary_matrix/LCholDet.birch"
struct LCholDetFunction_;
#line 1 "src/form/unary_matrix/LCholDet.birch"
#line 1 "src/form/unary_matrix/LCholDet.birch"
using LCholDetFunction = libbirch::Inplace<LCholDetFunction_>;
#line 1 "src/form/unary_matrix/LDet.birch"
#line 1 "src/form/unary_matrix/LDet.birch"
struct LDetFunction_;
#line 1 "src/form/unary_matrix/LDet.birch"
#line 1 "src/form/unary_matrix/LDet.birch"
using LDetFunction = libbirch::Inplace<LDetFunction_>;
#line 1 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 1 "src/form/unary_matrix/Matrix.birch"
struct MatrixFunction_;
#line 1 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 1 "src/form/unary_matrix/Matrix.birch"
using MatrixFunction = libbirch::Inplace<MatrixFunction_<Value>>;
#line 1 "src/form/unary_matrix/Trace.birch"
#line 1 "src/form/unary_matrix/Trace.birch"
struct TraceFunction_;
#line 1 "src/form/unary_matrix/Trace.birch"
#line 1 "src/form/unary_matrix/Trace.birch"
using TraceFunction = libbirch::Inplace<TraceFunction_>;
#line 1 "src/form/unary_matrix/Transpose.birch"
#line 1 "src/form/unary_matrix/Transpose.birch"
struct TransposeFunction_;
#line 1 "src/form/unary_matrix/Transpose.birch"
#line 1 "src/form/unary_matrix/Transpose.birch"
using TransposeFunction = libbirch::Inplace<TransposeFunction_>;
#line 1 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 1 "src/form/unary_matrix/Vector.birch"
struct VectorFunction_;
#line 1 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 1 "src/form/unary_matrix/Vector.birch"
using VectorFunction = libbirch::Inplace<VectorFunction_<Value>>;
#line 1 "src/form/unary_other/Abs.birch"
#line 1 "src/form/unary_other/Abs.birch"
struct AbsFunction_;
#line 1 "src/form/unary_other/Abs.birch"
#line 1 "src/form/unary_other/Abs.birch"
using AbsFunction = libbirch::Inplace<AbsFunction_>;
#line 1 "src/form/unary_other/Acos.birch"
#line 1 "src/form/unary_other/Acos.birch"
struct AcosFunction_;
#line 1 "src/form/unary_other/Acos.birch"
#line 1 "src/form/unary_other/Acos.birch"
using AcosFunction = libbirch::Inplace<AcosFunction_>;
#line 1 "src/form/unary_other/Asin.birch"
#line 1 "src/form/unary_other/Asin.birch"
struct AsinFunction_;
#line 1 "src/form/unary_other/Asin.birch"
#line 1 "src/form/unary_other/Asin.birch"
using AsinFunction = libbirch::Inplace<AsinFunction_>;
#line 1 "src/form/unary_other/Atan.birch"
#line 1 "src/form/unary_other/Atan.birch"
struct AtanFunction_;
#line 1 "src/form/unary_other/Atan.birch"
#line 1 "src/form/unary_other/Atan.birch"
using AtanFunction = libbirch::Inplace<AtanFunction_>;
#line 1 "src/form/unary_other/Cos.birch"
#line 1 "src/form/unary_other/Cos.birch"
struct CosFunction_;
#line 1 "src/form/unary_other/Cos.birch"
#line 1 "src/form/unary_other/Cos.birch"
using CosFunction = libbirch::Inplace<CosFunction_>;
#line 1 "src/form/unary_other/Cosh.birch"
#line 1 "src/form/unary_other/Cosh.birch"
struct CoshFunction_;
#line 1 "src/form/unary_other/Cosh.birch"
#line 1 "src/form/unary_other/Cosh.birch"
using CoshFunction = libbirch::Inplace<CoshFunction_>;
#line 1 "src/form/unary_other/Exp.birch"
#line 1 "src/form/unary_other/Exp.birch"
struct ExpFunction_;
#line 1 "src/form/unary_other/Exp.birch"
#line 1 "src/form/unary_other/Exp.birch"
using ExpFunction = libbirch::Inplace<ExpFunction_>;
#line 1 "src/form/unary_other/Expm1.birch"
#line 1 "src/form/unary_other/Expm1.birch"
struct Expm1Function_;
#line 1 "src/form/unary_other/Expm1.birch"
#line 1 "src/form/unary_other/Expm1.birch"
using Expm1Function = libbirch::Inplace<Expm1Function_>;
#line 1 "src/form/unary_other/Log.birch"
#line 1 "src/form/unary_other/Log.birch"
struct LogFunction_;
#line 1 "src/form/unary_other/Log.birch"
#line 1 "src/form/unary_other/Log.birch"
using LogFunction = libbirch::Inplace<LogFunction_>;
#line 1 "src/form/unary_other/Log1p.birch"
#line 1 "src/form/unary_other/Log1p.birch"
struct Log1pFunction_;
#line 1 "src/form/unary_other/Log1p.birch"
#line 1 "src/form/unary_other/Log1p.birch"
using Log1pFunction = libbirch::Inplace<Log1pFunction_>;
#line 1 "src/form/unary_other/LogGamma.birch"
#line 1 "src/form/unary_other/LogGamma.birch"
struct LogGammaFunction_;
#line 1 "src/form/unary_other/LogGamma.birch"
#line 1 "src/form/unary_other/LogGamma.birch"
using LogGammaFunction = libbirch::Inplace<LogGammaFunction_>;
#line 1 "src/form/unary_other/Negate.birch"
#line 1 "src/form/unary_other/Negate.birch"
struct NegateFunction_;
#line 1 "src/form/unary_other/Negate.birch"
#line 1 "src/form/unary_other/Negate.birch"
using NegateFunction = libbirch::Inplace<NegateFunction_>;
#line 1 "src/form/unary_other/Rectify.birch"
#line 1 "src/form/unary_other/Rectify.birch"
struct RectifyFunction_;
#line 1 "src/form/unary_other/Rectify.birch"
#line 1 "src/form/unary_other/Rectify.birch"
using RectifyFunction = libbirch::Inplace<RectifyFunction_>;
#line 1 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 1 "src/form/unary_other/Scalar.birch"
struct ScalarFunction_;
#line 1 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 1 "src/form/unary_other/Scalar.birch"
using ScalarFunction = libbirch::Inplace<ScalarFunction_<Value>>;
#line 1 "src/form/unary_other/Sin.birch"
#line 1 "src/form/unary_other/Sin.birch"
struct SinFunction_;
#line 1 "src/form/unary_other/Sin.birch"
#line 1 "src/form/unary_other/Sin.birch"
using SinFunction = libbirch::Inplace<SinFunction_>;
#line 1 "src/form/unary_other/Sinh.birch"
#line 1 "src/form/unary_other/Sinh.birch"
struct SinhFunction_;
#line 1 "src/form/unary_other/Sinh.birch"
#line 1 "src/form/unary_other/Sinh.birch"
using SinhFunction = libbirch::Inplace<SinhFunction_>;
#line 1 "src/form/unary_other/Sqrt.birch"
#line 1 "src/form/unary_other/Sqrt.birch"
struct SqrtFunction_;
#line 1 "src/form/unary_other/Sqrt.birch"
#line 1 "src/form/unary_other/Sqrt.birch"
using SqrtFunction = libbirch::Inplace<SqrtFunction_>;
#line 1 "src/form/unary_other/Sum.birch"
#line 1 "src/form/unary_other/Sum.birch"
struct SumFunction_;
#line 1 "src/form/unary_other/Sum.birch"
#line 1 "src/form/unary_other/Sum.birch"
using SumFunction = libbirch::Inplace<SumFunction_>;
#line 1 "src/form/unary_other/Tan.birch"
#line 1 "src/form/unary_other/Tan.birch"
struct TanFunction_;
#line 1 "src/form/unary_other/Tan.birch"
#line 1 "src/form/unary_other/Tan.birch"
using TanFunction = libbirch::Inplace<TanFunction_>;
#line 1 "src/form/unary_other/Tanh.birch"
#line 1 "src/form/unary_other/Tanh.birch"
struct TanhFunction_;
#line 1 "src/form/unary_other/Tanh.birch"
#line 1 "src/form/unary_other/Tanh.birch"
using TanhFunction = libbirch::Inplace<TanhFunction_>;
#line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
#line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
class DeepCloneNode_;
#line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
#line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
using DeepCloneNode = libbirch::Shared<DeepCloneNode_>;
#line 1 "src/test/distribution/TestModel.birch"
#line 1 "src/test/distribution/TestModel.birch"
class TestModel_;
#line 1 "src/test/distribution/TestModel.birch"
#line 1 "src/test/distribution/TestModel.birch"
using TestModel = libbirch::Shared<TestModel_>;
#line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
class TestMatrixGaussian_;
#line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
using TestMatrixGaussian = libbirch::Shared<TestMatrixGaussian_>;
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
class TestFanMatrixNormalInverseWishartMatrixGaussian_;
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
using TestFanMatrixNormalInverseWishartMatrixGaussian = libbirch::Shared<TestFanMatrixNormalInverseWishartMatrixGaussian_>;
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
class TestLinearMatrixNormalInverseWishartMatrixGaussian_;
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
using TestLinearMatrixNormalInverseWishartMatrixGaussian = libbirch::Shared<TestLinearMatrixNormalInverseWishartMatrixGaussian_>;
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
class TestMatrixNormalInverseWishart_;
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
using TestMatrixNormalInverseWishart = libbirch::Shared<TestMatrixNormalInverseWishart_>;
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
class TestMatrixNormalInverseWishartMatrixGaussian_;
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
using TestMatrixNormalInverseWishartMatrixGaussian = libbirch::Shared<TestMatrixNormalInverseWishartMatrixGaussian_>;
#line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
#line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
class TestInverseWishart_;
#line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
#line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
using TestInverseWishart = libbirch::Shared<TestInverseWishart_>;
#line 1 "src/test/distribution/matrix_other/TestWishart.birch"
#line 1 "src/test/distribution/matrix_other/TestWishart.birch"
class TestWishart_;
#line 1 "src/test/distribution/matrix_other/TestWishart.birch"
#line 1 "src/test/distribution/matrix_other/TestWishart.birch"
using TestWishart = libbirch::Shared<TestWishart_>;
#line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
#line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
class TestDirichletMultinomial_;
#line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
#line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
using TestDirichletMultinomial = libbirch::Shared<TestDirichletMultinomial_>;
#line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
class TestChainMultivariateGaussian_;
#line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
using TestChainMultivariateGaussian = libbirch::Shared<TestChainMultivariateGaussian_>;
#line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
class TestLinearMultivariateGaussianMultivariateGaussian_;
#line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
using TestLinearMultivariateGaussianMultivariateGaussian = libbirch::Shared<TestLinearMultivariateGaussianMultivariateGaussian_>;
#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
class TestMultivariateGaussian_;
#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
using TestMultivariateGaussian = libbirch::Shared<TestMultivariateGaussian_>;
#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
class TestMultivariateGaussianMultivariateGaussian_;
#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
using TestMultivariateGaussianMultivariateGaussian = libbirch::Shared<TestMultivariateGaussianMultivariateGaussian_>;
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
class TestFanMultivariateNormalInverseGammaMultivariateGaussian_;
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
using TestFanMultivariateNormalInverseGammaMultivariateGaussian = libbirch::Shared<TestFanMultivariateNormalInverseGammaMultivariateGaussian_>;
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
class TestLinearMultivariateNormalInverseGammaMultivariateGaussian_;
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
using TestLinearMultivariateNormalInverseGammaMultivariateGaussian = libbirch::Shared<TestLinearMultivariateNormalInverseGammaMultivariateGaussian_>;
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
class TestMultivariateNormalInverseGamma_;
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
using TestMultivariateNormalInverseGamma = libbirch::Shared<TestMultivariateNormalInverseGamma_>;
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
class TestMultivariateNormalInverseGammaMultivariateGaussian_;
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
using TestMultivariateNormalInverseGammaMultivariateGaussian = libbirch::Shared<TestMultivariateNormalInverseGammaMultivariateGaussian_>;
#line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
class TestLinearMatrixNormalInverseWishartMultivariateGaussian_;
#line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
using TestLinearMatrixNormalInverseWishartMultivariateGaussian = libbirch::Shared<TestLinearMatrixNormalInverseWishartMultivariateGaussian_>;
#line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
#line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
class TestBernoulli_;
#line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
#line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
using TestBernoulli = libbirch::Shared<TestBernoulli_>;
#line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
#line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
class TestBetaBernoulli_;
#line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
#line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
using TestBetaBernoulli = libbirch::Shared<TestBetaBernoulli_>;
#line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
class TestBetaBinomial_;
#line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
using TestBetaBinomial = libbirch::Shared<TestBetaBinomial_>;
#line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
class TestBetaGeometric_;
#line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
using TestBetaGeometric = libbirch::Shared<TestBetaGeometric_>;
#line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
class TestBetaNegativeBinomial_;
#line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
using TestBetaNegativeBinomial = libbirch::Shared<TestBetaNegativeBinomial_>;
#line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
class TestBinomial_;
#line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
using TestBinomial = libbirch::Shared<TestBinomial_>;
#line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
#line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
class TestCategorical_;
#line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
#line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
using TestCategorical = libbirch::Shared<TestCategorical_>;
#line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
class TestConwayMaxwellPoisson_;
#line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
using TestConwayMaxwellPoisson = libbirch::Shared<TestConwayMaxwellPoisson_>;
#line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
#line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
class TestDirichletCategorical_;
#line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
#line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
using TestDirichletCategorical = libbirch::Shared<TestDirichletCategorical_>;
#line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
class TestGammaPoisson_;
#line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
using TestGammaPoisson = libbirch::Shared<TestGammaPoisson_>;
#line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
#line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
class TestGeometric_;
#line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
#line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
using TestGeometric = libbirch::Shared<TestGeometric_>;
#line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
class TestNegativeBinomial_;
#line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
using TestNegativeBinomial = libbirch::Shared<TestNegativeBinomial_>;
#line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
class TestPoisson_;
#line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
using TestPoisson = libbirch::Shared<TestPoisson_>;
#line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
class TestScaledGammaPoisson_;
#line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
using TestScaledGammaPoisson = libbirch::Shared<TestScaledGammaPoisson_>;
#line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
#line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
class TestUniformInteger_;
#line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
#line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
using TestUniformInteger = libbirch::Shared<TestUniformInteger_>;
#line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
class TestAddDiscreteDelta_;
#line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
using TestAddDiscreteDelta = libbirch::Shared<TestAddDiscreteDelta_>;
#line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
class TestLongDiscreteDelta_;
#line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
using TestLongDiscreteDelta = libbirch::Shared<TestLongDiscreteDelta_>;
#line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
class TestNegateDiscreteDelta_;
#line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
using TestNegateDiscreteDelta = libbirch::Shared<TestNegateDiscreteDelta_>;
#line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
class TestSubtractDiscreteDelta_;
#line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
using TestSubtractDiscreteDelta = libbirch::Shared<TestSubtractDiscreteDelta_>;
#line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
class TestChainGaussian_;
#line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
using TestChainGaussian = libbirch::Shared<TestChainGaussian_>;
#line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
class TestGaussian_;
#line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
using TestGaussian = libbirch::Shared<TestGaussian_>;
#line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
class TestGaussianGaussian_;
#line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
using TestGaussianGaussian = libbirch::Shared<TestGaussianGaussian_>;
#line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
class TestLinearGaussianGaussian_;
#line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
using TestLinearGaussianGaussian = libbirch::Shared<TestLinearGaussianGaussian_>;
#line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
class TestLinearMultivariateGaussianGaussian_;
#line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
using TestLinearMultivariateGaussianGaussian = libbirch::Shared<TestLinearMultivariateGaussianGaussian_>;
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
class TestFanNormalInverseGammaGaussian_;
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
using TestFanNormalInverseGammaGaussian = libbirch::Shared<TestFanNormalInverseGammaGaussian_>;
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
class TestLinearMultivariateNormalInverseGammaGaussian_;
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
using TestLinearMultivariateNormalInverseGammaGaussian = libbirch::Shared<TestLinearMultivariateNormalInverseGammaGaussian_>;
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
class TestLinearNormalInverseGammaGaussian_;
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
using TestLinearNormalInverseGammaGaussian = libbirch::Shared<TestLinearNormalInverseGammaGaussian_>;
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
class TestNormalInverseGamma_;
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
using TestNormalInverseGamma = libbirch::Shared<TestNormalInverseGamma_>;
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
class TestNormalInverseGammaGaussian_;
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
using TestNormalInverseGammaGaussian = libbirch::Shared<TestNormalInverseGammaGaussian_>;
#line 1 "src/test/distribution/scalar_other/TestBeta.birch"
#line 1 "src/test/distribution/scalar_other/TestBeta.birch"
class TestBeta_;
#line 1 "src/test/distribution/scalar_other/TestBeta.birch"
#line 1 "src/test/distribution/scalar_other/TestBeta.birch"
using TestBeta = libbirch::Shared<TestBeta_>;
#line 1 "src/test/distribution/scalar_other/TestExponential.birch"
#line 1 "src/test/distribution/scalar_other/TestExponential.birch"
class TestExponential_;
#line 1 "src/test/distribution/scalar_other/TestExponential.birch"
#line 1 "src/test/distribution/scalar_other/TestExponential.birch"
using TestExponential = libbirch::Shared<TestExponential_>;
#line 1 "src/test/distribution/scalar_other/TestGamma.birch"
#line 1 "src/test/distribution/scalar_other/TestGamma.birch"
class TestGamma_;
#line 1 "src/test/distribution/scalar_other/TestGamma.birch"
#line 1 "src/test/distribution/scalar_other/TestGamma.birch"
using TestGamma = libbirch::Shared<TestGamma_>;
#line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
#line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
class TestGammaExponential_;
#line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
#line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
using TestGammaExponential = libbirch::Shared<TestGammaExponential_>;
#line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
#line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
class TestInverseGamma_;
#line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
#line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
using TestInverseGamma = libbirch::Shared<TestInverseGamma_>;
#line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
#line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
class TestInverseGammaGamma_;
#line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
#line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
using TestInverseGammaGamma = libbirch::Shared<TestInverseGammaGamma_>;
#line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
#line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
class TestScaledGammaExponential_;
#line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
#line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
using TestScaledGammaExponential = libbirch::Shared<TestScaledGammaExponential_>;
#line 1 "src/test/distribution/scalar_other/TestStudent.birch"
#line 1 "src/test/distribution/scalar_other/TestStudent.birch"
class TestStudent_;
#line 1 "src/test/distribution/scalar_other/TestStudent.birch"
#line 1 "src/test/distribution/scalar_other/TestStudent.birch"
using TestStudent = libbirch::Shared<TestStudent_>;
#line 1 "src/test/distribution/scalar_other/TestUniform.birch"
#line 1 "src/test/distribution/scalar_other/TestUniform.birch"
class TestUniform_;
#line 1 "src/test/distribution/scalar_other/TestUniform.birch"
#line 1 "src/test/distribution/scalar_other/TestUniform.birch"
using TestUniform = libbirch::Shared<TestUniform_>;
#line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
#line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
class TestWeibull_;
#line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
#line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
using TestWeibull = libbirch::Shared<TestWeibull_>;


#line 1 "src/io/JSONReader.birch"
#line 1 "src/io/JSONReader.birch"
using JSONReader = YAMLReader;
#line 13 "src/form/binary_matrix/CholSolve.birch"
template<class Left,class Right>
#line 13 "src/form/binary_matrix/CholSolve.birch"
using CholSolve = BinaryMemoized<CholSolveFunction, Left, Right>;
#line 11 "src/form/binary_matrix/Diagonal.birch"
template<class Left,class Right>
#line 11 "src/form/binary_matrix/Diagonal.birch"
using Diagonal = Binary<DiagonalFunction, Left, Right>;
#line 11 "src/form/binary_matrix/Dot.birch"
template<class Left,class Right>
#line 11 "src/form/binary_matrix/Dot.birch"
using Dot = BinaryMemoized<DotFunction, Left, Right>;
#line 11 "src/form/binary_matrix/Inner.birch"
template<class Left,class Right>
#line 11 "src/form/binary_matrix/Inner.birch"
using Inner = BinaryMemoized<InnerFunction, Left, Right>;
#line 24 "src/form/binary_matrix/MatrixPack.birch"
template<class Left,class Right>
#line 24 "src/form/binary_matrix/MatrixPack.birch"
using MatrixPack = BinaryMemoized<MatrixPackFunction, Left, Right>;
#line 34 "src/form/binary_matrix/MatrixStack.birch"
template<class Left,class Right>
#line 34 "src/form/binary_matrix/MatrixStack.birch"
using MatrixStack = BinaryMemoized<MatrixStackFunction, Left, Right>;
#line 11 "src/form/binary_matrix/Outer.birch"
template<class Left,class Right>
#line 11 "src/form/binary_matrix/Outer.birch"
using Outer = BinaryMemoized<OuterFunction, Left, Right>;
#line 13 "src/form/binary_matrix/Solve.birch"
template<class Left,class Right>
#line 13 "src/form/binary_matrix/Solve.birch"
using Solve = BinaryMemoized<SolveFunction, Left, Right>;
#line 10 "src/form/binary_other/Add.birch"
template<class Left,class Right>
#line 10 "src/form/binary_other/Add.birch"
using Add = LinearMemoized<AddFunction, Left, Right>;
#line 11 "src/form/binary_other/CopySign.birch"
template<class Left,class Right>
#line 11 "src/form/binary_other/CopySign.birch"
using CopySign = Binary<CopySignFunction, Left, Right>;
#line 11 "src/form/binary_other/Divide.birch"
template<class Left,class Right>
#line 11 "src/form/binary_other/Divide.birch"
using Divide = BinaryMemoized<DivideFunction, Left, Right>;
#line 10 "src/form/binary_other/LessThan.birch"
template<class Left,class Right>
#line 10 "src/form/binary_other/LessThan.birch"
using LessThan = Linear<LessThanFunction, Left, Right>;
#line 11 "src/form/binary_other/LogBeta.birch"
template<class Left,class Right>
#line 11 "src/form/binary_other/LogBeta.birch"
using LogBeta = Binary<LogBetaFunction, Left, Right>;
#line 11 "src/form/binary_other/LogChoose.birch"
template<class Left,class Right>
#line 11 "src/form/binary_other/LogChoose.birch"
using LogChoose = Binary<LogChooseFunction, Left, Right>;
#line 15 "src/form/binary_other/LogGammaP.birch"
template<class Left,class Right>
#line 15 "src/form/binary_other/LogGammaP.birch"
using LogGammaP = Binary<LogGammaPFunction, Left, Right>;
#line 51 "src/form/binary_other/Multiply.birch"
template<class Left,class Right>
#line 51 "src/form/binary_other/Multiply.birch"
using Multiply = BinaryMemoized<MultiplyFunction, Left, Right>;
#line 16 "src/form/binary_other/Pow.birch"
template<class Left,class Right>
#line 16 "src/form/binary_other/Pow.birch"
using Pow = Binary<PowFunction, Left, Right>;
#line 10 "src/form/binary_other/Subtract.birch"
template<class Left,class Right>
#line 10 "src/form/binary_other/Subtract.birch"
using Subtract = LinearMemoized<SubtractFunction, Left, Right>;
#line 11 "src/form/ternary/IfThenElse.birch"
template<class Left,class Middle,class Right>
#line 11 "src/form/ternary/IfThenElse.birch"
using IfThenElse = Ternary<IfThenElseFunction, Left, Middle, Right>;
#line 22 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
template<class Left,class Middle,class Right>
#line 22 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
using LogZConwayMaxwellPoisson = Ternary<LogZConwayMaxwellPoissonFunction, Left, Middle, Right>;
#line 11 "src/form/unary_matrix/CholInv.birch"
template<class Middle>
#line 11 "src/form/unary_matrix/CholInv.birch"
using CholInv = UnaryMemoized<CholInvFunction, Middle>;
#line 11 "src/form/unary_matrix/Inv.birch"
template<class Middle>
#line 11 "src/form/unary_matrix/Inv.birch"
using Inv = UnaryMemoized<InvFunction, Middle>;
#line 10 "src/form/unary_matrix/LCholDet.birch"
template<class Middle>
#line 10 "src/form/unary_matrix/LCholDet.birch"
using LCholDet = UnaryMemoized<LCholDetFunction, Middle>;
#line 10 "src/form/unary_matrix/LDet.birch"
template<class Middle>
#line 10 "src/form/unary_matrix/LDet.birch"
using LDet = UnaryMemoized<LDetFunction, Middle>;
#line 10 "src/form/unary_matrix/Matrix.birch"
template<class Value,class Middle>
#line 10 "src/form/unary_matrix/Matrix.birch"
using Matrix = Unary<MatrixFunction<Value>, Middle>;
#line 18 "src/form/unary_matrix/Trace.birch"
template<class Middle>
#line 18 "src/form/unary_matrix/Trace.birch"
using Trace = UnaryMemoized<TraceFunction, Middle>;
#line 10 "src/form/unary_matrix/Transpose.birch"
template<class Middle>
#line 10 "src/form/unary_matrix/Transpose.birch"
using Transpose = UnaryMemoized<TransposeFunction, Middle>;
#line 10 "src/form/unary_matrix/Vector.birch"
template<class Value,class Middle>
#line 10 "src/form/unary_matrix/Vector.birch"
using Vector = Unary<VectorFunction<Value>, Middle>;
#line 10 "src/form/unary_other/Abs.birch"
template<class Middle>
#line 10 "src/form/unary_other/Abs.birch"
using Abs = Unary<AbsFunction, Middle>;
#line 10 "src/form/unary_other/Acos.birch"
template<class Middle>
#line 10 "src/form/unary_other/Acos.birch"
using Acos = Unary<AcosFunction, Middle>;
#line 10 "src/form/unary_other/Asin.birch"
template<class Middle>
#line 10 "src/form/unary_other/Asin.birch"
using Asin = Unary<AsinFunction, Middle>;
#line 10 "src/form/unary_other/Atan.birch"
template<class Middle>
#line 10 "src/form/unary_other/Atan.birch"
using Atan = Unary<AtanFunction, Middle>;
#line 10 "src/form/unary_other/Cos.birch"
template<class Middle>
#line 10 "src/form/unary_other/Cos.birch"
using Cos = Unary<CosFunction, Middle>;
#line 10 "src/form/unary_other/Cosh.birch"
template<class Middle>
#line 10 "src/form/unary_other/Cosh.birch"
using Cosh = Unary<CoshFunction, Middle>;
#line 10 "src/form/unary_other/Exp.birch"
template<class Middle>
#line 10 "src/form/unary_other/Exp.birch"
using Exp = Unary<ExpFunction, Middle>;
#line 10 "src/form/unary_other/Expm1.birch"
template<class Middle>
#line 10 "src/form/unary_other/Expm1.birch"
using Expm1 = Unary<Expm1Function, Middle>;
#line 10 "src/form/unary_other/Log.birch"
template<class Middle>
#line 10 "src/form/unary_other/Log.birch"
using Log = Unary<LogFunction, Middle>;
#line 10 "src/form/unary_other/Log1p.birch"
template<class Middle>
#line 10 "src/form/unary_other/Log1p.birch"
using Log1p = Unary<Log1pFunction, Middle>;
#line 10 "src/form/unary_other/LogGamma.birch"
template<class Middle>
#line 10 "src/form/unary_other/LogGamma.birch"
using LogGamma = Unary<LogGammaFunction, Middle>;
#line 10 "src/form/unary_other/Negate.birch"
template<class Middle>
#line 10 "src/form/unary_other/Negate.birch"
using Negate = Unary<NegateFunction, Middle>;
#line 12 "src/form/unary_other/Rectify.birch"
template<class Middle>
#line 12 "src/form/unary_other/Rectify.birch"
using Rectify = Unary<RectifyFunction, Middle>;
#line 10 "src/form/unary_other/Scalar.birch"
template<class Value,class Middle>
#line 10 "src/form/unary_other/Scalar.birch"
using Scalar = Unary<ScalarFunction<Value>, Middle>;
#line 10 "src/form/unary_other/Sin.birch"
template<class Middle>
#line 10 "src/form/unary_other/Sin.birch"
using Sin = Unary<SinFunction, Middle>;
#line 10 "src/form/unary_other/Sinh.birch"
template<class Middle>
#line 10 "src/form/unary_other/Sinh.birch"
using Sinh = Unary<SinhFunction, Middle>;
#line 10 "src/form/unary_other/Sqrt.birch"
template<class Middle>
#line 10 "src/form/unary_other/Sqrt.birch"
using Sqrt = Unary<SqrtFunction, Middle>;
#line 18 "src/form/unary_other/Sum.birch"
template<class Middle>
#line 18 "src/form/unary_other/Sum.birch"
using Sum = Unary<SumFunction, Middle>;
#line 10 "src/form/unary_other/Tan.birch"
template<class Middle>
#line 10 "src/form/unary_other/Tan.birch"
using Tan = Unary<TanFunction, Middle>;
#line 10 "src/form/unary_other/Tanh.birch"
template<class Middle>
#line 10 "src/form/unary_other/Tanh.birch"
using Tanh = Unary<TanhFunction, Middle>;
#line 5 "src/math/constant.birch"
extern Real \u03c0;
#line 7 "src/math/constant.birch"
extern Real MACHEP;
#line 9 "src/math/constant.birch"
extern libbirch::DefaultArray<Real,2> IGAM_ASYMPTOTIC_SERIES_D;
#line 35 "src/math/constant.birch"
extern libbirch::DefaultArray<Real,1> LANCZOS_SUM_EXPG_SCALED_NUM;
#line 51 "src/math/constant.birch"
extern libbirch::DefaultArray<Real,1> LANCZOS_SUM_EXPG_SCALED_DENOM;
#line 14 "src/system/filesystem.birch"
extern Integer READ;
#line 15 "src/system/filesystem.birch"
extern Integer WRITE;
#line 16 "src/system/filesystem.birch"
extern Integer APPEND;
#line 31 "src/system/stdio.birch"
extern InputStream stdin_;
#line 36 "src/system/stdio.birch"
extern OutputStream stdout_;
#line 41 "src/system/stdio.birch"
extern OutputStream stderr_;
#line 41 "src/filter.birch"
extern "C" int filter(int, char**);
#line 44 "src/sample.birch"
extern "C" int sample(int, char**);
#line 4 "src/test/basic/test_basic_array.birch"
extern "C" int test_basic_array(int, char**);
#line 5 "src/test/basic/test_basic_deep_clone_alias.birch"
extern "C" int test_basic_deep_clone_alias(int, char**);
#line 4 "src/test/basic/test_basic_deep_clone_chain.birch"
extern "C" int test_basic_deep_clone_chain(int, char**);
#line 5 "src/test/basic/test_basic_deep_clone_modify_dst.birch"
extern "C" int test_basic_deep_clone_modify_dst(int, char**);
#line 5 "src/test/basic/test_basic_deep_clone_modify_src.birch"
extern "C" int test_basic_deep_clone_modify_src(int, char**);
#line 4 "src/test/basic/test_basic_list.birch"
extern "C" int test_basic_list(int, char**);
#line 4 "src/test/basic/test_basic_ragged_array.birch"
extern "C" int test_basic_ragged_array(int, char**);
#line 4 "src/test/basic/test_basic_stack.birch"
extern "C" int test_basic_stack(int, char**);
#line 4 "src/test/basic/test_basic_tape.birch"
extern "C" int test_basic_tape(int, char**);
#line 41 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
extern "C" int test_pdf_matrix_gaussian(int, char**);
#line 49 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
extern "C" int test_grad_matrix_gaussian(int, char**);
#line 108 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" int test_conjugacy_fan_matrix_normal_inverse_wishart_matrix_gaussian(int, char**);
#line 114 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" int test_pdf_fan_matrix_normal_inverse_wishart_matrix_gaussian(int, char**);
#line 123 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" int test_grad_fan_matrix_normal_inverse_wishart_matrix_gaussian(int, char**);
#line 84 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" int test_conjugacy_linear_matrix_normal_inverse_wishart_matrix_gaussian(int, char**);
#line 90 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" int test_pdf_linear_matrix_normal_inverse_wishart_matrix_gaussian(int, char**);
#line 98 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" int test_grad_linear_matrix_normal_inverse_wishart_matrix_gaussian(int, char**);
#line 68 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
extern "C" int test_conjugacy_matrix_normal_inverse_wishart(int, char**);
#line 74 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
extern "C" int test_pdf_matrix_normal_inverse_wishart(int, char**);
#line 82 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
extern "C" int test_grad_matrix_normal_inverse_wishart(int, char**);
#line 80 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" int test_conjugacy_matrix_normal_inverse_wishart_matrix_gaussian(int, char**);
#line 86 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" int test_pdf_matrix_normal_inverse_wishart_matrix_gaussian(int, char**);
#line 95 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" int test_grad_matrix_normal_inverse_wishart_matrix_gaussian(int, char**);
#line 42 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
extern "C" int test_conjugacy_dirichlet_multinomial(int, char**);
#line 50 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
extern "C" int test_conjugacy_chain_multivariate_gaussian(int, char**);
#line 55 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
extern "C" int test_pdf_chain_multivariate_gaussian(int, char**);
#line 63 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
extern "C" int test_grad_chain_multivariate_gaussian(int, char**);
#line 57 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
extern "C" int test_conjugacy_linear_multivariate_gaussian_multivariate_gaussian(int, char**);
#line 63 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
extern "C" int test_pdf_linear_multivariate_gaussian_multivariate_gaussian(int, char**);
#line 72 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
extern "C" int test_grad_linear_multivariate_gaussian_multivariate_gaussian(int, char**);
#line 37 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
extern "C" int test_conjugacy_multivariate_gaussian(int, char**);
#line 42 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
extern "C" int test_pdf_multivariate_gaussian(int, char**);
#line 50 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
extern "C" int test_grad_multivariate_gaussian(int, char**);
#line 53 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
extern "C" int test_conjugacy_multivariate_gaussian_multivariate_gaussian(int, char**);
#line 58 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
extern "C" int test_pdf_multivariate_gaussian_multivariate_gaussian(int, char**);
#line 67 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
extern "C" int test_grad_multivariate_gaussian_multivariate_gaussian(int, char**);
#line 92 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" int test_conjugacy_fan_multivariate_normal_inverse_gamma_multivariate_gaussian(int, char**);
#line 98 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" int test_pdf_fan_multivariate_normal_inverse_gamma_multivariate_gaussian(int, char**);
#line 107 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" int test_grad_fan_multivariate_normal_inverse_gamma_multivariate_gaussian(int, char**);
#line 61 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" int test_conjugacy_linear_multivariate_normal_inverse_gamma_multivariate_gaussian(int, char**);
#line 67 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" int test_pdf_linear_multivariate_normal_inverse_gamma_multivariate_gaussian(int, char**);
#line 75 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" int test_grad_linear_multivariate_normal_inverse_gamma_multivariate_gaussian(int, char**);
#line 49 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
extern "C" int test_conjugacy_multivariate_normal_inverse_gamma(int, char**);
#line 55 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
extern "C" int test_pdf_linear_multivariate_normal_inverse_gamma(int, char**);
#line 63 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
extern "C" int test_grad_multivariate_normal_inverse_gamma(int, char**);
#line 56 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" int test_conjugacy_multivariate_normal_inverse_gamma_multivariate_gaussian(int, char**);
#line 62 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" int test_pdf_multivariate_normal_inverse_gamma_multivariate_gaussian(int, char**);
#line 70 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" int test_grad_multivariate_normal_inverse_gamma_multivariate_gaussian(int, char**);
#line 90 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
extern "C" int test_conjugacy_linear_matrix_normal_inverse_wishart_multivariate_gaussian(int, char**);
#line 96 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
extern "C" int test_pdf_linear_matrix_normal_inverse_wishart_multivariate_gaussian(int, char**);
#line 105 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
extern "C" int test_grad_linear_matrix_normal_inverse_wishart_multivariate_gaussian(int, char**);
#line 31 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
extern "C" int test_pdf_bernoulli(int, char**);
#line 39 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
extern "C" int test_grad_bernoulli(int, char**);
#line 43 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
extern "C" int test_conjugacy_beta_bernoulli(int, char**);
#line 48 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
extern "C" int test_pdf_beta_bernoulli(int, char**);
#line 56 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
extern "C" int test_grad_beta_bernoulli(int, char**);
#line 44 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
extern "C" int test_conjugacy_beta_binomial(int, char**);
#line 49 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
extern "C" int test_pdf_beta_binomial(int, char**);
#line 57 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
extern "C" int test_cdf_beta_binomial(int, char**);
#line 64 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
extern "C" int test_grad_beta_binomial(int, char**);
#line 42 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
extern "C" int test_conjugacy_beta_geometric(int, char**);
#line 47 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
extern "C" int test_pdf_beta_geometric(int, char**);
#line 55 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
extern "C" int test_grad_beta_geometric(int, char**);
#line 44 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
extern "C" int test_conjugacy_beta_negative_binomial(int, char**);
#line 49 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
extern "C" int test_pdf_beta_negative_binomial(int, char**);
#line 57 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
extern "C" int test_grad_beta_negative_binomial(int, char**);
#line 32 "src/test/distribution/scalar_discrete/TestBinomial.birch"
extern "C" int test_pdf_binomial(int, char**);
#line 40 "src/test/distribution/scalar_discrete/TestBinomial.birch"
extern "C" int test_cdf_binomial(int, char**);
#line 47 "src/test/distribution/scalar_discrete/TestBinomial.birch"
extern "C" int test_grad_binomial(int, char**);
#line 32 "src/test/distribution/scalar_discrete/TestCategorical.birch"
extern "C" int test_pdf_categorical(int, char**);
#line 40 "src/test/distribution/scalar_discrete/TestCategorical.birch"
extern "C" int test_cdf_categorical(int, char**);
#line 35 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
extern "C" int test_pdf_conway_maxwell_poisson(int, char**);
#line 43 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
extern "C" int test_cdf_conway_maxwell_poisson(int, char**);
#line 50 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
extern "C" int test_grad_conway_maxwell_poisson(int, char**);
#line 42 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
extern "C" int test_conjugacy_dirichlet_categorical(int, char**);
#line 47 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
extern "C" int test_pdf_dirichlet_categorical(int, char**);
#line 42 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
extern "C" int test_conjugacy_gamma_poisson(int, char**);
#line 47 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
extern "C" int test_pdf_gamma_poisson(int, char**);
#line 55 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
extern "C" int test_cdf_gamma_poisson(int, char**);
#line 62 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
extern "C" int test_grad_gamma_poisson(int, char**);
#line 30 "src/test/distribution/scalar_discrete/TestGeometric.birch"
extern "C" int test_pdf_geometric(int, char**);
#line 38 "src/test/distribution/scalar_discrete/TestGeometric.birch"
extern "C" int test_cdf_geometric(int, char**);
#line 45 "src/test/distribution/scalar_discrete/TestGeometric.birch"
extern "C" int test_grad_geometric(int, char**);
#line 32 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
extern "C" int test_pdf_negative_binomial(int, char**);
#line 40 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
extern "C" int test_cdf_negative_binomial(int, char**);
#line 47 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
extern "C" int test_grad_negative_binomial(int, char**);
#line 30 "src/test/distribution/scalar_discrete/TestPoisson.birch"
extern "C" int test_pdf_poisson(int, char**);
#line 38 "src/test/distribution/scalar_discrete/TestPoisson.birch"
extern "C" int test_cdf_poisson(int, char**);
#line 45 "src/test/distribution/scalar_discrete/TestPoisson.birch"
extern "C" int test_grad_poisson(int, char**);
#line 45 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
extern "C" int test_conjugacy_scaled_gamma_poisson(int, char**);
#line 50 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
extern "C" int test_pdf_scaled_gamma_poisson(int, char**);
#line 58 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
extern "C" int test_cdf_scaled_gamma_poisson(int, char**);
#line 65 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
extern "C" int test_grad_scaled_gamma_poisson(int, char**);
#line 32 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
extern "C" int test_pdf_uniform_integer(int, char**);
#line 40 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
extern "C" int test_cdf_uniform_integer(int, char**);
#line 43 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
extern "C" int test_conjugacy_add_discrete_delta(int, char**);
#line 49 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
extern "C" int test_pdf_add_discrete_delta(int, char**);
#line 54 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
extern "C" int test_conjugacy_long_discrete_delta(int, char**);
#line 60 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
extern "C" int test_pdf_long_discrete_delta(int, char**);
#line 50 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
extern "C" int test_conjugacy_negate_discrete_delta(int, char**);
#line 56 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
extern "C" int test_pdf_negate_discrete_delta(int, char**);
#line 43 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
extern "C" int test_conjugacy_subtract_discrete_delta(int, char**);
#line 49 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
extern "C" int test_pdf_subtract_discrete_delta(int, char**);
#line 47 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
extern "C" int test_conjugacy_chain_gaussian(int, char**);
#line 52 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
extern "C" int test_pdf_chain_gaussian(int, char**);
#line 60 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
extern "C" int test_cdf_chain_gaussian(int, char**);
#line 67 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
extern "C" int test_grad_chain_gaussian(int, char**);
#line 32 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
extern "C" int test_pdf_gaussian(int, char**);
#line 40 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
extern "C" int test_cdf_gaussian(int, char**);
#line 47 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
extern "C" int test_grad_gaussian(int, char**);
#line 44 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
extern "C" int test_conjugacy_gaussian_gaussian(int, char**);
#line 49 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
extern "C" int test_pdf_gaussian_gaussian(int, char**);
#line 57 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
extern "C" int test_cdf_gaussian_gaussian(int, char**);
#line 64 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
extern "C" int test_grad_gaussian_gaussian(int, char**);
#line 48 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
extern "C" int test_conjugacy_linear_gaussian_gaussian(int, char**);
#line 54 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
extern "C" int test_pdf_linear_gaussian_gaussian(int, char**);
#line 62 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
extern "C" int test_cdf_linear_gaussian_gaussian(int, char**);
#line 69 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
extern "C" int test_grad_linear_gaussian_gaussian(int, char**);
#line 51 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
extern "C" int test_conjugacy_linear_multivariate_gaussian_gaussian(int, char**);
#line 57 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
extern "C" int test_pdf_linear_multivariate_gaussian_gaussian(int, char**);
#line 65 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
extern "C" int test_cdf_linear_multivariate_gaussian_gaussian(int, char**);
#line 72 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
extern "C" int test_grad_linear_multivariate_gaussian_gaussian(int, char**);
#line 70 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
extern "C" int test_conjugacy_fan_normal_inverse_gamma_gaussian(int, char**);
#line 76 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
extern "C" int test_pdf_fan_normal_inverse_gamma_gaussian(int, char**);
#line 84 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
extern "C" int test_cdf_fan_normal_inverse_gamma_gaussian(int, char**);
#line 91 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
extern "C" int test_grad_fan_normal_inverse_gamma_gaussian(int, char**);
#line 60 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
extern "C" int test_conjugacy_linear_multivariate_normal_inverse_gamma_gaussian(int, char**);
#line 66 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
extern "C" int test_pdf_linear_multivariate_normal_inverse_gamma_gaussian(int, char**);
#line 75 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
extern "C" int test_cdf_linear_multivariate_normal_inverse_gamma_gaussian(int, char**);
#line 83 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
extern "C" int test_grad_linear_multivariate_normal_inverse_gamma_gaussian(int, char**);
#line 58 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
extern "C" int test_conjugacy_linear_normal_inverse_gamma_gaussian(int, char**);
#line 64 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
extern "C" int test_pdf_linear_normal_inverse_gamma_gaussian(int, char**);
#line 72 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
extern "C" int test_cdf_linear_normal_inverse_gamma_gaussian(int, char**);
#line 79 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
extern "C" int test_grad_linear_normal_inverse_gamma_gaussian(int, char**);
#line 47 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
extern "C" int test_conjugacy_normal_inverse_gamma(int, char**);
#line 52 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
extern "C" int test_pdf_normal_inverse_gamma(int, char**);
#line 60 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
extern "C" int test_cdf_normal_inverse_gamma(int, char**);
#line 67 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
extern "C" int test_grad_normal_inverse_gamma(int, char**);
#line 52 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
extern "C" int test_conjugacy_normal_inverse_gamma_gaussian(int, char**);
#line 57 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
extern "C" int test_pdf_normal_inverse_gamma_gaussian(int, char**);
#line 65 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
extern "C" int test_cdf_normal_inverse_gamma_gaussian(int, char**);
#line 72 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
extern "C" int test_grad_normal_inverse_gamma_gaussian(int, char**);
#line 32 "src/test/distribution/scalar_other/TestBeta.birch"
extern "C" int test_pdf_beta(int, char**);
#line 40 "src/test/distribution/scalar_other/TestBeta.birch"
extern "C" int test_cdf_beta(int, char**);
#line 47 "src/test/distribution/scalar_other/TestBeta.birch"
extern "C" int test_grad_beta(int, char**);
#line 30 "src/test/distribution/scalar_other/TestExponential.birch"
extern "C" int test_conjugacy_exponential(int, char**);
#line 36 "src/test/distribution/scalar_other/TestExponential.birch"
extern "C" int test_pdf_exponential(int, char**);
#line 44 "src/test/distribution/scalar_other/TestExponential.birch"
extern "C" int test_cdf_exponential(int, char**);
#line 51 "src/test/distribution/scalar_other/TestExponential.birch"
extern "C" int test_grad_exponential(int, char**);
#line 32 "src/test/distribution/scalar_other/TestGamma.birch"
extern "C" int test_pdf_gamma(int, char**);
#line 40 "src/test/distribution/scalar_other/TestGamma.birch"
extern "C" int test_cdf_gamma(int, char**);
#line 47 "src/test/distribution/scalar_other/TestGamma.birch"
extern "C" int test_grad_gamma(int, char**);
#line 42 "src/test/distribution/scalar_other/TestGammaExponential.birch"
extern "C" int test_conjugacy_gamma_exponential(int, char**);
#line 48 "src/test/distribution/scalar_other/TestGammaExponential.birch"
extern "C" int test_pdf_gamma_exponential(int, char**);
#line 56 "src/test/distribution/scalar_other/TestGammaExponential.birch"
extern "C" int test_cdf_gamma_exponential(int, char**);
#line 63 "src/test/distribution/scalar_other/TestGammaExponential.birch"
extern "C" int test_grad_gamma_exponential(int, char**);
#line 32 "src/test/distribution/scalar_other/TestInverseGamma.birch"
extern "C" int test_pdf_inverse_gamma(int, char**);
#line 40 "src/test/distribution/scalar_other/TestInverseGamma.birch"
extern "C" int test_cdf_inverse_gamma(int, char**);
#line 47 "src/test/distribution/scalar_other/TestInverseGamma.birch"
extern "C" int test_grad_inverse_gamma(int, char**);
#line 44 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
extern "C" int test_conjugacy_inverse_gamma_gamma(int, char**);
#line 49 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
extern "C" int test_pdf_inverse_gamma_gamma(int, char**);
#line 57 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
extern "C" int test_cdf_inverse_gamma_gamma(int, char**);
#line 64 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
extern "C" int test_grad_inverse_gamma_gamma(int, char**);
#line 45 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
extern "C" int test_conjugacy_scaled_gamma_exponential(int, char**);
#line 51 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
extern "C" int test_pdf_scaled_gamma_exponential(int, char**);
#line 59 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
extern "C" int test_cdf_scaled_gamma_exponential(int, char**);
#line 66 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
extern "C" int test_grad_scaled_gamma_exponential(int, char**);
#line 34 "src/test/distribution/scalar_other/TestStudent.birch"
extern "C" int test_pdf_student_t(int, char**);
#line 42 "src/test/distribution/scalar_other/TestStudent.birch"
extern "C" int test_cdf_student_t(int, char**);
#line 49 "src/test/distribution/scalar_other/TestStudent.birch"
extern "C" int test_grad_student_t(int, char**);
#line 32 "src/test/distribution/scalar_other/TestUniform.birch"
extern "C" int test_pdf_uniform(int, char**);
#line 40 "src/test/distribution/scalar_other/TestUniform.birch"
extern "C" int test_cdf_uniform(int, char**);
#line 47 "src/test/distribution/scalar_other/TestUniform.birch"
extern "C" int test_grad_uniform(int, char**);
#line 32 "src/test/distribution/scalar_other/TestWeibull.birch"
extern "C" int test_pdf_weibull(int, char**);
#line 40 "src/test/distribution/scalar_other/TestWeibull.birch"
extern "C" int test_cdf_weibull(int, char**);
#line 47 "src/test/distribution/scalar_other/TestWeibull.birch"
extern "C" int test_grad_weibull(int, char**);
#line 18 "src/basic/Boolean.birch"
#line 18 "src/basic/Boolean.birch"
Boolean max(const Boolean& x, const Boolean& y);
#line 25 "src/basic/Boolean.birch"
#line 25 "src/basic/Boolean.birch"
Boolean min(const Boolean& x, const Boolean& y);
#line 29 "src/basic/Boolean.birch"
#line 29 "src/basic/Boolean.birch"
Integer rows(const Boolean& x);
#line 33 "src/basic/Boolean.birch"
#line 33 "src/basic/Boolean.birch"
Integer columns(const Boolean& x);
#line 37 "src/basic/Boolean.birch"
#line 37 "src/basic/Boolean.birch"
Integer length(const Boolean& x);
#line 41 "src/basic/Boolean.birch"
#line 41 "src/basic/Boolean.birch"
Integer size(const Boolean& x);
#line 45 "src/basic/Boolean.birch"
#line 45 "src/basic/Boolean.birch"
Boolean value(const Boolean& x);
#line 49 "src/basic/Boolean.birch"
#line 49 "src/basic/Boolean.birch"
Boolean peek(const Boolean& x);
#line 53 "src/basic/Boolean.birch"
#line 53 "src/basic/Boolean.birch"
Boolean eval(const Boolean& x);
#line 57 "src/basic/Boolean.birch"
#line 57 "src/basic/Boolean.birch"
Boolean reval(const Boolean& x);
#line 61 "src/basic/Boolean.birch"
template<class Gradient>
#line 61 "src/basic/Boolean.birch"
void grad(const Boolean& x, const Gradient& d);
#line 65 "src/basic/Boolean.birch"
#line 65 "src/basic/Boolean.birch"
void label(const Boolean& x, const Integer& gen);
#line 69 "src/basic/Boolean.birch"
#line 69 "src/basic/Boolean.birch"
void constant(const Boolean& x, const Integer& gen);
#line 73 "src/basic/Boolean.birch"
#line 73 "src/basic/Boolean.birch"
void constant(const Boolean& x);
#line 19 "src/basic/Integer.birch"
#line 19 "src/basic/Integer.birch"
Integer mod(const Integer& x, const Integer& y);
#line 28 "src/basic/Integer.birch"
#line 28 "src/basic/Integer.birch"
Integer max(const Integer& x, const Integer& y);
#line 37 "src/basic/Integer.birch"
#line 37 "src/basic/Integer.birch"
Integer min(const Integer& x, const Integer& y);
#line 43 "src/basic/Integer.birch"
#line 43 "src/basic/Integer.birch"
Integer rows(const Integer& x);
#line 47 "src/basic/Integer.birch"
#line 47 "src/basic/Integer.birch"
Integer columns(const Integer& x);
#line 51 "src/basic/Integer.birch"
#line 51 "src/basic/Integer.birch"
Integer length(const Integer& x);
#line 55 "src/basic/Integer.birch"
#line 55 "src/basic/Integer.birch"
Integer size(const Integer& x);
#line 59 "src/basic/Integer.birch"
#line 59 "src/basic/Integer.birch"
Integer value(const Integer& x);
#line 63 "src/basic/Integer.birch"
#line 63 "src/basic/Integer.birch"
Integer peek(const Integer& x);
#line 67 "src/basic/Integer.birch"
#line 67 "src/basic/Integer.birch"
Integer eval(const Integer& x);
#line 71 "src/basic/Integer.birch"
#line 71 "src/basic/Integer.birch"
Integer reval(const Integer& x);
#line 75 "src/basic/Integer.birch"
template<class Gradient>
#line 75 "src/basic/Integer.birch"
void grad(const Integer& x, const Gradient& d);
#line 79 "src/basic/Integer.birch"
#line 79 "src/basic/Integer.birch"
void label(const Integer& x, const Integer& gen);
#line 83 "src/basic/Integer.birch"
#line 83 "src/basic/Integer.birch"
void constant(const Integer& x, const Integer& gen);
#line 87 "src/basic/Integer.birch"
#line 87 "src/basic/Integer.birch"
void constant(const Integer& x);
#line 19 "src/basic/Real.birch"
#line 19 "src/basic/Real.birch"
Real mod(const Real& x, const Real& y);
#line 28 "src/basic/Real.birch"
#line 28 "src/basic/Real.birch"
Real max(const Real& x, const Real& y);
#line 37 "src/basic/Real.birch"
#line 37 "src/basic/Real.birch"
Real min(const Real& x, const Real& y);
#line 46 "src/basic/Real.birch"
#line 46 "src/basic/Real.birch"
Boolean isinf(const Real& x);
#line 55 "src/basic/Real.birch"
#line 55 "src/basic/Real.birch"
Boolean isnan(const Real& x);
#line 64 "src/basic/Real.birch"
#line 64 "src/basic/Real.birch"
Boolean isfinite(const Real& x);
#line 70 "src/basic/Real.birch"
#line 70 "src/basic/Real.birch"
Integer rows(const Real& x);
#line 74 "src/basic/Real.birch"
#line 74 "src/basic/Real.birch"
Integer columns(const Real& x);
#line 78 "src/basic/Real.birch"
#line 78 "src/basic/Real.birch"
Integer length(const Real& x);
#line 82 "src/basic/Real.birch"
#line 82 "src/basic/Real.birch"
Integer size(const Real& x);
#line 86 "src/basic/Real.birch"
#line 86 "src/basic/Real.birch"
Real value(const Real& x);
#line 90 "src/basic/Real.birch"
#line 90 "src/basic/Real.birch"
Real peek(const Real& x);
#line 94 "src/basic/Real.birch"
#line 94 "src/basic/Real.birch"
Real eval(const Real& x);
#line 98 "src/basic/Real.birch"
#line 98 "src/basic/Real.birch"
Real reval(const Real& x);
#line 102 "src/basic/Real.birch"
template<class Gradient>
#line 102 "src/basic/Real.birch"
void grad(const Real& x, const Gradient& d);
#line 106 "src/basic/Real.birch"
#line 106 "src/basic/Real.birch"
void label(const Real& x, const Integer& gen);
#line 110 "src/basic/Real.birch"
#line 110 "src/basic/Real.birch"
void constant(const Real& x, const Integer& gen);
#line 114 "src/basic/Real.birch"
#line 114 "src/basic/Real.birch"
void constant(const Real& x);
#line 9 "src/basic/String.birch"
#line 9 "src/basic/String.birch"
String to_string(const Boolean& x);
#line 20 "src/basic/String.birch"
#line 20 "src/basic/String.birch"
String to_string(const Real& x);
#line 35 "src/basic/String.birch"
#line 35 "src/basic/String.birch"
String to_string(const Integer& x);
#line 44 "src/basic/String.birch"
#line 44 "src/basic/String.birch"
String to_string(const String& x);
#line 51 "src/basic/String.birch"
#line 51 "src/basic/String.birch"
std::optional<String> to_string(const std::optional<Boolean>& x);
#line 62 "src/basic/String.birch"
#line 62 "src/basic/String.birch"
std::optional<String> to_string(const std::optional<Real>& x);
#line 73 "src/basic/String.birch"
#line 73 "src/basic/String.birch"
std::optional<String> to_string(const std::optional<Integer>& x);
#line 84 "src/basic/String.birch"
#line 84 "src/basic/String.birch"
std::optional<String> to_string(const std::optional<String>& x);
#line 91 "src/basic/String.birch"
template<class Type>
#line 91 "src/basic/String.birch"
String to_string(const libbirch::DefaultArray<Type,1>& x);
#line 114 "src/basic/String.birch"
template<class Type>
#line 114 "src/basic/String.birch"
String to_string(const libbirch::DefaultArray<Type,2>& X);
#line 146 "src/basic/String.birch"
template<class Value>
#line 146 "src/basic/String.birch"
Value from_string(const String& s);
#line 348 "src/basic/String.birch"
#line 348 "src/basic/String.birch"
Integer length(const String& x);
#line 357 "src/basic/String.birch"
#line 357 "src/basic/String.birch"
Integer length(const libbirch::DefaultArray<String,1>& x);
#line 4 "src/basic/matrix.birch"
template<class Type>
#line 4 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,2> row(const libbirch::DefaultArray<Type,1>& x);
#line 12 "src/basic/matrix.birch"
template<class Type>
#line 12 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,1> row(const libbirch::DefaultArray<Type,2>& X, const Integer& i);
#line 19 "src/basic/matrix.birch"
template<class Type>
#line 19 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,2> column(const libbirch::DefaultArray<Type,1>& x);
#line 27 "src/basic/matrix.birch"
template<class Type>
#line 27 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,1> column(const libbirch::DefaultArray<Type,2>& X, const Integer& j);
#line 37 "src/basic/matrix.birch"
template<class Type>
#line 37 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,1> vec(const libbirch::DefaultArray<Type,2>& X);
#line 52 "src/basic/matrix.birch"
template<class Type>
#line 52 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,2> mat(const libbirch::DefaultArray<Type,1>& x, const Integer& columns);
#line 66 "src/basic/matrix.birch"
#line 66 "src/basic/matrix.birch"
libbirch::DefaultArray<Real,2> identity(const Integer& length);
#line 83 "src/basic/matrix.birch"
template<class Lambda>
#line 83 "src/basic/matrix.birch"
decltype(auto) matrix_lambda(const Lambda& \u03bb, const Integer& rows, const Integer& columns);
#line 99 "src/basic/matrix.birch"
template<class Type>
#line 99 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,2> matrix(const Type& x, const Integer& rows, const Integer& columns);
#line 106 "src/basic/matrix.birch"
template<class Type>
#line 106 "src/basic/matrix.birch"
Integer rows(const libbirch::DefaultArray<Type,2>& x);
#line 112 "src/basic/matrix.birch"
template<class Type>
#line 112 "src/basic/matrix.birch"
Integer columns(const libbirch::DefaultArray<Type,2>& x);
#line 118 "src/basic/matrix.birch"
template<class Type>
#line 118 "src/basic/matrix.birch"
Integer length(const libbirch::DefaultArray<Type,2>& x);
#line 122 "src/basic/matrix.birch"
template<class Type>
#line 122 "src/basic/matrix.birch"
Integer size(const libbirch::DefaultArray<Type,2>& x);
#line 126 "src/basic/matrix.birch"
template<class Type>
#line 126 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,2> value(const libbirch::DefaultArray<Type,2>& x);
#line 130 "src/basic/matrix.birch"
template<class Type>
#line 130 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,2> peek(const libbirch::DefaultArray<Type,2>& x);
#line 134 "src/basic/matrix.birch"
template<class Type>
#line 134 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,2> eval(const libbirch::DefaultArray<Type,2>& x);
#line 138 "src/basic/matrix.birch"
template<class Type>
#line 138 "src/basic/matrix.birch"
libbirch::DefaultArray<Type,2> reval(const libbirch::DefaultArray<Type,2>& x);
#line 142 "src/basic/matrix.birch"
template<class Type,class Gradient>
#line 142 "src/basic/matrix.birch"
void grad(const libbirch::DefaultArray<Type,2>& x, const Gradient& d);
#line 146 "src/basic/matrix.birch"
template<class Type,class Gradient>
#line 146 "src/basic/matrix.birch"
void grad(const libbirch::DefaultArray<Type,2>& x, const Gradient& d, const Integer& i, const Integer& j);
#line 150 "src/basic/matrix.birch"
template<class Type>
#line 150 "src/basic/matrix.birch"
void label(const libbirch::DefaultArray<Type,2>& x, const Integer& gen);
#line 154 "src/basic/matrix.birch"
template<class Type>
#line 154 "src/basic/matrix.birch"
void constant(const libbirch::DefaultArray<Type,2>& x, const Integer& gen);
#line 158 "src/basic/matrix.birch"
template<class Type>
#line 158 "src/basic/matrix.birch"
void constant(const libbirch::DefaultArray<Type,2>& x);
#line 13 "src/basic/vector.birch"
template<class Lambda>
#line 13 "src/basic/vector.birch"
decltype(auto) vector_lambda(const Lambda& \u03bb, const Integer& length);
#line 25 "src/basic/vector.birch"
template<class Type>
#line 25 "src/basic/vector.birch"
libbirch::DefaultArray<Type,1> vector(const Type& x, const Integer& length);
#line 32 "src/basic/vector.birch"
template<class Type>
#line 32 "src/basic/vector.birch"
Integer rows(const libbirch::DefaultArray<Type,1>& x);
#line 38 "src/basic/vector.birch"
template<class Type>
#line 38 "src/basic/vector.birch"
Integer columns(const libbirch::DefaultArray<Type,1>& x);
#line 42 "src/basic/vector.birch"
template<class Type>
#line 42 "src/basic/vector.birch"
Integer length(const libbirch::DefaultArray<Type,1>& x);
#line 46 "src/basic/vector.birch"
template<class Type>
#line 46 "src/basic/vector.birch"
Integer size(const libbirch::DefaultArray<Type,1>& x);
#line 50 "src/basic/vector.birch"
template<class Type>
#line 50 "src/basic/vector.birch"
libbirch::DefaultArray<Type,1> value(const libbirch::DefaultArray<Type,1>& x);
#line 54 "src/basic/vector.birch"
template<class Type>
#line 54 "src/basic/vector.birch"
libbirch::DefaultArray<Type,1> peek(const libbirch::DefaultArray<Type,1>& x);
#line 58 "src/basic/vector.birch"
template<class Type>
#line 58 "src/basic/vector.birch"
libbirch::DefaultArray<Type,1> eval(const libbirch::DefaultArray<Type,1>& x);
#line 62 "src/basic/vector.birch"
template<class Type>
#line 62 "src/basic/vector.birch"
libbirch::DefaultArray<Type,1> reval(const libbirch::DefaultArray<Type,1>& x);
#line 66 "src/basic/vector.birch"
template<class Type,class Gradient>
#line 66 "src/basic/vector.birch"
void grad(const libbirch::DefaultArray<Type,1>& x, const Gradient& d);
#line 70 "src/basic/vector.birch"
template<class Type,class Gradient>
#line 70 "src/basic/vector.birch"
void grad(const libbirch::DefaultArray<Type,1>& x, const Gradient& d, const Integer& i);
#line 74 "src/basic/vector.birch"
template<class Type>
#line 74 "src/basic/vector.birch"
void label(const libbirch::DefaultArray<Type,1>& x, const Integer& gen);
#line 78 "src/basic/vector.birch"
template<class Type>
#line 78 "src/basic/vector.birch"
void constant(const libbirch::DefaultArray<Type,1>& x, const Integer& gen);
#line 82 "src/basic/vector.birch"
template<class Type>
#line 82 "src/basic/vector.birch"
void constant(const libbirch::DefaultArray<Type,1>& x);
#line 986 "src/data/Buffer.birch"
#line 986 "src/data/Buffer.birch"
Buffer make_buffer();
#line 995 "src/data/Buffer.birch"
template<class Type>
#line 995 "src/data/Buffer.birch"
Buffer make_buffer(const Type& root);
#line 1007 "src/data/Buffer.birch"
template<class Type>
#line 1007 "src/data/Buffer.birch"
Buffer make_buffer(const Integer& t, const Type& root);
#line 1019 "src/data/Buffer.birch"
#line 1019 "src/data/Buffer.birch"
Buffer make_buffer(const libbirch::DefaultArray<String,1>& keys, const libbirch::DefaultArray<Buffer,1>& values);
#line 500 "src/event/Handler.birch"
template<class Value>
#line 500 "src/event/Handler.birch"
Integer argSize(const Tape<Random<Value>>& o);
#line 518 "src/event/Handler.birch"
template<class Value>
#line 518 "src/event/Handler.birch"
Integer argSizeReverse(const Tape<Random<Value>>& o);
#line 11 "src/event/handle.birch"
#line 11 "src/event/handle.birch"
Handler get_handler();
#line 25 "src/event/handle.birch"
#line 25 "src/event/handle.birch"
void set_handler(const Handler& handler);
#line 43 "src/event/handle.birch"
#line 43 "src/event/handle.birch"
Handler swap_handler(const Handler& handler);
#line 65 "src/event/handle.birch"
template<class Left,class Right>
#line 65 "src/event/handle.birch"
Left handle_assume(const Left& x, const Right& p);
#line 83 "src/event/handle.birch"
template<class Right>
#line 83 "src/event/handle.birch"
Random<typename Right::value_type::Value_> handle_assume(const Right& p);
#line 100 "src/event/handle.birch"
template<class Left,class Right>
#line 100 "src/event/handle.birch"
Left handle_simulate(const Left& x, const Right& p);
#line 116 "src/event/handle.birch"
template<class Right>
#line 116 "src/event/handle.birch"
typename Right::value_type::Value_ handle_simulate(const Right& p);
#line 133 "src/event/handle.birch"
template<class Left,class Right>
#line 133 "src/event/handle.birch"
Left handle_observe(const Left& x, const Right& p);
#line 147 "src/event/handle.birch"
template<class Arg>
#line 147 "src/event/handle.birch"
void handle_factor(const Arg& w);
#line 156 "src/event/handle.birch"
template<class Value>
#line 156 "src/event/handle.birch"
void handle_delay_simulate(const Random<Value>& x);
#line 165 "src/event/handle.birch"
template<class Value>
#line 165 "src/event/handle.birch"
void handle_delay_observe(const Random<Value>& x);
#line 177 "src/event/handle.birch"
template<class Left,class Right>
#line 177 "src/event/handle.birch"
Delay handle_prune(const Left& p, const Right& x);
#line 229 "src/expression/Expression.birch"
template<class Type>
#line 229 "src/expression/Expression.birch"
Integer length(const Type& x);
#line 233 "src/expression/Expression.birch"
template<class Type>
#line 233 "src/expression/Expression.birch"
Integer size(const Type& x);
#line 237 "src/expression/Expression.birch"
template<class Type>
#line 237 "src/expression/Expression.birch"
Integer rows(const Type& x);
#line 241 "src/expression/Expression.birch"
template<class Type>
#line 241 "src/expression/Expression.birch"
Integer columns(const Type& x);
#line 245 "src/expression/Expression.birch"
template<class Type>
#line 245 "src/expression/Expression.birch"
decltype(auto) value(const Type& x);
#line 249 "src/expression/Expression.birch"
template<class Type>
#line 249 "src/expression/Expression.birch"
decltype(auto) peek(const Type& x);
#line 253 "src/expression/Expression.birch"
template<class Type>
#line 253 "src/expression/Expression.birch"
decltype(auto) eval(const Type& x);
#line 257 "src/expression/Expression.birch"
template<class Type>
#line 257 "src/expression/Expression.birch"
decltype(auto) reval(const Type& x);
#line 261 "src/expression/Expression.birch"
template<class Type,class Gradient>
#line 261 "src/expression/Expression.birch"
void grad(const Type& x, const Gradient& d);
#line 265 "src/expression/Expression.birch"
template<class Type,class Gradient>
#line 265 "src/expression/Expression.birch"
void grad(const Type& x, const Gradient& d, const Integer& i);
#line 269 "src/expression/Expression.birch"
template<class Type,class Gradient>
#line 269 "src/expression/Expression.birch"
void grad(const Type& x, const Gradient& d, const Integer& i, const Integer& j);
#line 273 "src/expression/Expression.birch"
template<class Type>
#line 273 "src/expression/Expression.birch"
void label(const Type& x, const Integer& gen);
#line 277 "src/expression/Expression.birch"
template<class Type>
#line 277 "src/expression/Expression.birch"
void constant(const Type& x, const Integer& gen);
#line 281 "src/expression/Expression.birch"
template<class Type>
#line 281 "src/expression/Expression.birch"
void constant(const Type& x);
#line 35 "src/expression/MatrixSplitExpression.birch"
template<class Form>
#line 35 "src/expression/MatrixSplitExpression.birch"
decltype(auto) matrix_split(const Form& f);
#line 45 "src/expression/MatrixSplitExpression.birch"
template<class Value,class Form>
#line 45 "src/expression/MatrixSplitExpression.birch"
decltype(auto) matrix_split(const Value& x, const Form& f);
#line 353 "src/expression/Random.birch"
template<class Value>
#line 353 "src/expression/Random.birch"
Random<Value> make_random();
#line 363 "src/expression/Random.birch"
template<class Value>
#line 363 "src/expression/Random.birch"
Random<Value> make_random(const Value& x);
#line 374 "src/expression/Random.birch"
template<class Value>
#line 374 "src/expression/Random.birch"
Random<Value> make_random(const std::optional<Value>& x);
#line 387 "src/expression/Random.birch"
template<class Value>
#line 387 "src/expression/Random.birch"
Random<Value> make_random(const Random<Value>& x);
#line 37 "src/expression/VectorSplitExpression.birch"
template<class Form>
#line 37 "src/expression/VectorSplitExpression.birch"
decltype(auto) vector_split(const Form& f);
#line 47 "src/expression/VectorSplitExpression.birch"
template<class Value,class Form>
#line 47 "src/expression/VectorSplitExpression.birch"
decltype(auto) vector_split(const Value& x, const Form& f);
#line 4 "src/expression/box.birch"
template<class Value>
#line 4 "src/expression/box.birch"
Expression<Value> box(const Expression<Value>& x);
#line 11 "src/expression/box.birch"
template<class Value,class Form>
#line 11 "src/expression/box.birch"
Expression<Value> box(const BoxedForm<Value, Form>& x);
#line 18 "src/expression/box.birch"
template<class Value>
#line 18 "src/expression/box.birch"
Expression<Value> box(const BoxedValue<Value>& x);
#line 25 "src/expression/box.birch"
template<class Value,class Form>
#line 25 "src/expression/box.birch"
Expression<libbirch::DefaultArray<Value,2>> box(const MatrixSplitExpression<libbirch::DefaultArray<Value,2>, Form>& x);
#line 33 "src/expression/box.birch"
template<class Value>
#line 33 "src/expression/box.birch"
Expression<Value> box(const Random<Value>& x);
#line 40 "src/expression/box.birch"
template<class Value,class Form>
#line 40 "src/expression/box.birch"
Expression<libbirch::DefaultArray<Value,1>> box(const VectorSplitExpression<libbirch::DefaultArray<Value,1>, Form>& x);
#line 48 "src/expression/box.birch"
#line 48 "src/expression/box.birch"
Expression<Boolean> box(const Boolean& x);
#line 55 "src/expression/box.birch"
#line 55 "src/expression/box.birch"
Expression<Integer> box(const Integer& x);
#line 62 "src/expression/box.birch"
#line 62 "src/expression/box.birch"
Expression<Real> box(const Real& x);
#line 69 "src/expression/box.birch"
template<class Type>
#line 69 "src/expression/box.birch"
Expression<libbirch::DefaultArray<Type,1>> box(const libbirch::DefaultArray<Type,1>& x);
#line 76 "src/expression/box.birch"
template<class Type>
#line 76 "src/expression/box.birch"
Expression<libbirch::DefaultArray<Type,2>> box(const libbirch::DefaultArray<Type,2>& x);
#line 85 "src/expression/box.birch"
template<class Form>
#line 85 "src/expression/box.birch"
decltype(auto) box(const Form& f);
#line 88 "src/expression/box.birch"
template<class Value,class Form>
#line 88 "src/expression/box.birch"
Expression<Value> box_form(const Value& x, const Form& f);
#line 95 "src/expression/box.birch"
template<class Value1,class Value2>
#line 95 "src/expression/box.birch"
decltype(auto) box(const std::tuple<Value1, Value2>& x);
#line 103 "src/expression/box.birch"
template<class Value1,class Value2,class Value3>
#line 103 "src/expression/box.birch"
decltype(auto) box(const std::tuple<Value1, Value2, Value3>& x);
#line 111 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4>
#line 111 "src/expression/box.birch"
decltype(auto) box(const std::tuple<Value1, Value2, Value3, Value4>& x);
#line 119 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5>
#line 119 "src/expression/box.birch"
decltype(auto) box(const std::tuple<Value1, Value2, Value3, Value4, Value5>& x);
#line 127 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6>
#line 127 "src/expression/box.birch"
decltype(auto) box(const std::tuple<Value1, Value2, Value3, Value4, Value5, Value6>& x);
#line 135 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6,class Value7>
#line 135 "src/expression/box.birch"
decltype(auto) box(const std::tuple<Value1, Value2, Value3, Value4, Value5, Value6, Value7>& x);
#line 143 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6,class Value7,class Value8>
#line 143 "src/expression/box.birch"
decltype(auto) box(const std::tuple<Value1, Value2, Value3, Value4, Value5, Value6, Value7, Value8>& x);
#line 151 "src/expression/box.birch"
template<class Value1,class Value2>
#line 151 "src/expression/box.birch"
decltype(auto) box(const Value1& x1, const Value2& x2);
#line 158 "src/expression/box.birch"
template<class Value1,class Value2,class Value3>
#line 158 "src/expression/box.birch"
decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3);
#line 165 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4>
#line 165 "src/expression/box.birch"
decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3, const Value4& x4);
#line 172 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5>
#line 172 "src/expression/box.birch"
decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3, const Value4& x4, const Value5& x5);
#line 179 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6>
#line 179 "src/expression/box.birch"
decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3, const Value4& x4, const Value5& x5, const Value6& x6);
#line 186 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6,class Value7>
#line 186 "src/expression/box.birch"
decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3, const Value4& x4, const Value5& x5, const Value6& x6, const Value7& x7);
#line 193 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6,class Value7,class Value8>
#line 193 "src/expression/box.birch"
decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3, const Value4& x4, const Value5& x5, const Value6& x6, const Value7& x7, const Value8& x8);
#line 62 "src/form/MatrixElement.birch"
template<class Value,class Form,class Middle,class Right>
#line 62 "src/form/MatrixElement.birch"
decltype(auto) element(const MatrixSplitExpression<Value, Form>& x, const Middle& i, const Right& j);
#line 69 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 69 "src/form/MatrixElement.birch"
decltype(auto) element(const Left& x, const Middle& i, const Right& j);
#line 76 "src/form/MatrixElement.birch"
#line 76 "src/form/MatrixElement.birch"
Boolean element(const libbirch::DefaultArray<Boolean,2>& x, const Integer& i, const Integer& j);
#line 83 "src/form/MatrixElement.birch"
#line 83 "src/form/MatrixElement.birch"
Integer element(const libbirch::DefaultArray<Integer,2>& x, const Integer& i, const Integer& j);
#line 90 "src/form/MatrixElement.birch"
#line 90 "src/form/MatrixElement.birch"
Real element(const libbirch::DefaultArray<Real,2>& x, const Integer& i, const Integer& j);
#line 58 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 58 "src/form/MatrixJoinForm.birch"
decltype(auto) join(const libbirch::DefaultArray<Expression<Value>,2>& x);
#line 54 "src/form/VectorElement.birch"
template<class Value,class Form,class Right>
#line 54 "src/form/VectorElement.birch"
decltype(auto) element(const VectorSplitExpression<Value, Form>& x, const Right& i);
#line 61 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 61 "src/form/VectorElement.birch"
decltype(auto) element(const Left& x, const Right& i);
#line 68 "src/form/VectorElement.birch"
#line 68 "src/form/VectorElement.birch"
Boolean element(const libbirch::DefaultArray<Boolean,1>& x, const Integer& i);
#line 75 "src/form/VectorElement.birch"
#line 75 "src/form/VectorElement.birch"
Integer element(const libbirch::DefaultArray<Integer,1>& x, const Integer& i);
#line 82 "src/form/VectorElement.birch"
#line 82 "src/form/VectorElement.birch"
Real element(const libbirch::DefaultArray<Real,1>& x, const Integer& i);
#line 58 "src/form/VectorJoinForm.birch"
template<class Value>
#line 58 "src/form/VectorJoinForm.birch"
decltype(auto) join(const libbirch::DefaultArray<Expression<Value>,1>& x);
#line 87 "src/io/InputStream.birch"
#line 87 "src/io/InputStream.birch"
InputStream make_input_stream(const File& file);
#line 127 "src/io/OutputStream.birch"
#line 127 "src/io/OutputStream.birch"
OutputStream make_output_stream(const File& file);
#line 69 "src/io/Reader.birch"
#line 69 "src/io/Reader.birch"
Reader make_reader(const String& path);
#line 95 "src/io/Reader.birch"
#line 95 "src/io/Reader.birch"
Buffer slurp(const String& path);
#line 98 "src/io/Writer.birch"
#line 98 "src/io/Writer.birch"
Writer make_writer(const String& path);
#line 123 "src/io/Writer.birch"
#line 123 "src/io/Writer.birch"
void dump(const String& path, const Buffer& buffer);
#line 39 "src/math/eigen.birch"
#line 39 "src/math/eigen.birch"
libbirch::DefaultArray<Real,2> chol(const libbirch::DefaultArray<Real,2>& S);
#line 48 "src/math/eigen.birch"
#line 48 "src/math/eigen.birch"
libbirch::DefaultArray<Real,2> tril(const libbirch::DefaultArray<Real,2>& X);
#line 57 "src/math/eigen.birch"
#line 57 "src/math/eigen.birch"
libbirch::DefaultArray<Real,2> triu(const libbirch::DefaultArray<Real,2>& X);
#line 66 "src/math/eigen.birch"
template<class Type>
#line 66 "src/math/eigen.birch"
libbirch::DefaultArray<Type,1> hadamard(const libbirch::DefaultArray<Type,1>& x, const libbirch::DefaultArray<Type,1>& y);
#line 75 "src/math/eigen.birch"
template<class Type>
#line 75 "src/math/eigen.birch"
libbirch::DefaultArray<Type,2> hadamard(const libbirch::DefaultArray<Type,2>& X, const libbirch::DefaultArray<Type,2>& Y);
#line 17 "src/math/rng.birch"
#line 17 "src/math/rng.birch"
void seed(const Integer& s);
#line 29 "src/math/rng.birch"
#line 29 "src/math/rng.birch"
void seed();
#line 8 "src/math/special.birch"
#line 8 "src/math/special.birch"
Real gamma(const Real& x);
#line 17 "src/math/special.birch"
#line 17 "src/math/special.birch"
Real gamma(const Real& x, const Integer& p);
#line 29 "src/math/special.birch"
#line 29 "src/math/special.birch"
Real beta(const Real& x, const Real& y);
#line 36 "src/math/special.birch"
#line 36 "src/math/special.birch"
Real ibeta(const Real& a, const Real& b, const Real& x);
#line 99 "src/math/special.birch"
#line 99 "src/math/special.birch"
Real digamma(const Real& x);
#line 108 "src/math/special.birch"
#line 108 "src/math/special.birch"
Real choose(const Integer& x, const Integer& y);
#line 124 "src/math/special.birch"
#line 124 "src/math/special.birch"
Real choose(const Real& x, const Real& y);
#line 137 "src/math/special.birch"
#line 137 "src/math/special.birch"
Real lower_inc_gamma(const Real& a, const Real& x);
#line 180 "src/math/special.birch"
#line 180 "src/math/special.birch"
Real upper_inc_gamma(const Real& a, const Real& x);
#line 238 "src/math/special.birch"
#line 238 "src/math/special.birch"
Real igam_fac(const Real& a, const Real& x);
#line 263 "src/math/special.birch"
#line 263 "src/math/special.birch"
Real igamc_continued_fraction(const Real& a, const Real& x);
#line 319 "src/math/special.birch"
#line 319 "src/math/special.birch"
Real igam_series(const Real& a, const Real& x);
#line 343 "src/math/special.birch"
#line 343 "src/math/special.birch"
Real igamc_series(const Real& a, const Real& x);
#line 366 "src/math/special.birch"
#line 366 "src/math/special.birch"
Real asymptotic_series(const Real& a, const Real& x, const Integer& func);
#line 435 "src/math/special.birch"
#line 435 "src/math/special.birch"
Real lanczos_sum_expg_scaled(const Real& x);
#line 442 "src/math/special.birch"
#line 442 "src/math/special.birch"
Real ratevl(const Real& x, const libbirch::DefaultArray<Real,1>& num, const Integer& M, const libbirch::DefaultArray<Real,1>& denom, const Integer& N);
#line 4 "src/math/standard.birch"
#line 4 "src/math/standard.birch"
Real ceil(const Real& x);
#line 13 "src/math/standard.birch"
#line 13 "src/math/standard.birch"
Real floor(const Real& x);
#line 22 "src/math/standard.birch"
#line 22 "src/math/standard.birch"
Real round(const Real& x);
#line 31 "src/math/standard.birch"
#line 31 "src/math/standard.birch"
Real erf(const Real& x);
#line 40 "src/math/standard.birch"
#line 40 "src/math/standard.birch"
Real erfc(const Real& x);
#line 8 "src/primitive/iterator.birch"
template<class Type,class Lambda>
#line 8 "src/primitive/iterator.birch"
Type reduce(const Iterator<Type>& iter, const Type& init, const Lambda& op);
#line 20 "src/primitive/iterator.birch"
template<class Type>
#line 20 "src/primitive/iterator.birch"
Type max(const Iterator<Type>& iter);
#line 30 "src/primitive/iterator.birch"
template<class Type>
#line 30 "src/primitive/iterator.birch"
Type min(const Iterator<Type>& iter);
#line 9 "src/primitive/primitive.birch"
template<class Type,class Lambda>
#line 9 "src/primitive/primitive.birch"
void for_each(const libbirch::DefaultArray<Type,1>& x, const Lambda& f);
#line 23 "src/primitive/primitive.birch"
template<class Type,class Lambda>
#line 23 "src/primitive/primitive.birch"
void for_each(const libbirch::DefaultArray<Type,2>& X, const Lambda& f);
#line 38 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Lambda>
#line 38 "src/primitive/primitive.birch"
void for_each(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const Lambda& f);
#line 52 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Lambda>
#line 52 "src/primitive/primitive.birch"
void for_each(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const Lambda& f);
#line 71 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Lambda>
#line 71 "src/primitive/primitive.birch"
void for_each(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const libbirch::DefaultArray<Input3,1>& z, const Lambda& f);
#line 88 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Lambda>
#line 88 "src/primitive/primitive.birch"
void for_each(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const libbirch::DefaultArray<Input3,2>& Z, const Lambda& f);
#line 107 "src/primitive/primitive.birch"
template<class Input,class Lambda>
#line 107 "src/primitive/primitive.birch"
decltype(auto) transform(const libbirch::DefaultArray<Input,1>& x, const Lambda& f);
#line 117 "src/primitive/primitive.birch"
template<class Input,class Lambda>
#line 117 "src/primitive/primitive.birch"
decltype(auto) transform(const libbirch::DefaultArray<Input,2>& X, const Lambda& f);
#line 129 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Lambda>
#line 129 "src/primitive/primitive.birch"
decltype(auto) transform(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const Lambda& f);
#line 142 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Lambda>
#line 142 "src/primitive/primitive.birch"
decltype(auto) transform(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const Lambda& f);
#line 158 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Lambda>
#line 158 "src/primitive/primitive.birch"
decltype(auto) transform(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const libbirch::DefaultArray<Input3,1>& z, const Lambda& f);
#line 174 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Lambda>
#line 174 "src/primitive/primitive.birch"
decltype(auto) transform(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const libbirch::DefaultArray<Input3,2>& Z, const Lambda& f);
#line 192 "src/primitive/primitive.birch"
template<class Type,class Output,class Lambda>
#line 192 "src/primitive/primitive.birch"
Output reduce(const libbirch::DefaultArray<Type,1>& x, const Output& init, const Lambda& op);
#line 212 "src/primitive/primitive.birch"
template<class Type,class Output,class Lambda>
#line 212 "src/primitive/primitive.birch"
Output reduce(const libbirch::DefaultArray<Type,2>& X, const Output& init, const Lambda& op);
#line 234 "src/primitive/primitive.birch"
template<class Input,class Output,class Lambda1,class Lambda2>
#line 234 "src/primitive/primitive.birch"
Output transform_reduce(const libbirch::DefaultArray<Input,1>& x, const Output& init, const Lambda1& op1, const Lambda2& op2);
#line 251 "src/primitive/primitive.birch"
template<class Input,class Output,class Lambda1,class Lambda2>
#line 251 "src/primitive/primitive.birch"
Output transform_reduce(const libbirch::DefaultArray<Input,2>& X, const Output& init, const Lambda1& op1, const Lambda2& op2);
#line 271 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Output,class Lambda1,class Lambda2>
#line 271 "src/primitive/primitive.birch"
Output transform_reduce(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const Output& init, const Lambda1& op1, const Lambda2& op2);
#line 290 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Output,class Lambda1,class Lambda2>
#line 290 "src/primitive/primitive.birch"
Output transform_reduce(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const Output& init, const Lambda1& op1, const Lambda2& op2);
#line 313 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Output,class Lambda1,class Lambda2>
#line 313 "src/primitive/primitive.birch"
Output transform_reduce(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const libbirch::DefaultArray<Input3,1>& z, const Output& init, const Lambda1& op1, const Lambda2& op2);
#line 335 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Output,class Lambda1,class Lambda2>
#line 335 "src/primitive/primitive.birch"
Output transform_reduce(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const libbirch::DefaultArray<Input3,2>& Z, const Output& init, const Lambda1& op1, const Lambda2& op2);
#line 357 "src/primitive/primitive.birch"
template<class Type,class Lambda>
#line 357 "src/primitive/primitive.birch"
libbirch::DefaultArray<Type,1> inclusive_scan(const libbirch::DefaultArray<Type,1>& x, const Type& init, const Lambda& op);
#line 380 "src/primitive/primitive.birch"
template<class Type>
#line 380 "src/primitive/primitive.birch"
libbirch::DefaultArray<Type,1> inclusive_scan(const libbirch::DefaultArray<Type,1>& x);
#line 391 "src/primitive/primitive.birch"
template<class Type,class Lambda>
#line 391 "src/primitive/primitive.birch"
libbirch::DefaultArray<Type,1> exclusive_scan(const libbirch::DefaultArray<Type,1>& x, const Type& init, const Lambda& op);
#line 414 "src/primitive/primitive.birch"
template<class Type>
#line 414 "src/primitive/primitive.birch"
libbirch::DefaultArray<Type,1> exclusive_scan(const libbirch::DefaultArray<Type,1>& x);
#line 424 "src/primitive/primitive.birch"
template<class Type,class Lambda>
#line 424 "src/primitive/primitive.birch"
libbirch::DefaultArray<Type,1> adjacent_difference(const libbirch::DefaultArray<Type,1>& x, const Lambda& op);
#line 436 "src/primitive/primitive.birch"
template<class Type>
#line 436 "src/primitive/primitive.birch"
Boolean is_sorted(const libbirch::DefaultArray<Type,1>& x);
#line 449 "src/primitive/primitive.birch"
template<class Type>
#line 449 "src/primitive/primitive.birch"
libbirch::DefaultArray<Type,1> sort(const libbirch::DefaultArray<Type,1>& x);
#line 465 "src/primitive/primitive.birch"
template<class Type>
#line 465 "src/primitive/primitive.birch"
libbirch::DefaultArray<Integer,1> sort_index(const libbirch::DefaultArray<Type,1>& x);
#line 483 "src/primitive/primitive.birch"
template<class Type>
#line 483 "src/primitive/primitive.birch"
libbirch::DefaultArray<Type,1> gather(const libbirch::DefaultArray<Integer,1>& a, const libbirch::DefaultArray<Type,1>& x);
#line 497 "src/primitive/primitive.birch"
template<class Type>
#line 497 "src/primitive/primitive.birch"
libbirch::DefaultArray<Type,1> scatter(const libbirch::DefaultArray<Integer,1>& a, const libbirch::DefaultArray<Type,1>& x);
#line 512 "src/primitive/primitive.birch"
template<class Type>
#line 512 "src/primitive/primitive.birch"
libbirch::DefaultArray<Real,1> iota(const Type& x, const Integer& length);
#line 519 "src/primitive/primitive.birch"
template<class Type>
#line 519 "src/primitive/primitive.birch"
Type max(const libbirch::DefaultArray<Type,1>& x);
#line 533 "src/primitive/primitive.birch"
template<class Type>
#line 533 "src/primitive/primitive.birch"
Type min(const libbirch::DefaultArray<Type,1>& x);
#line 10 "src/primitive/resample.birch"
#line 10 "src/primitive/resample.birch"
libbirch::DefaultArray<Integer,1> resample_systematic(const libbirch::DefaultArray<Real,1>& w);
#line 22 "src/primitive/resample.birch"
#line 22 "src/primitive/resample.birch"
libbirch::DefaultArray<Integer,1> resample_multinomial(const libbirch::DefaultArray<Real,1>& w);
#line 30 "src/primitive/resample.birch"
#line 30 "src/primitive/resample.birch"
Real log_sum_exp(const libbirch::DefaultArray<Real,1>& x);
#line 41 "src/primitive/resample.birch"
#line 41 "src/primitive/resample.birch"
Real log_sum(const libbirch::DefaultArray<Real,1>& x);
#line 49 "src/primitive/resample.birch"
#line 49 "src/primitive/resample.birch"
Real nan_exp(const Real& x);
#line 62 "src/primitive/resample.birch"
#line 62 "src/primitive/resample.birch"
libbirch::DefaultArray<Real,1> norm_exp(const libbirch::DefaultArray<Real,1>& x);
#line 74 "src/primitive/resample.birch"
#line 74 "src/primitive/resample.birch"
Integer cumulative_ancestor(const libbirch::DefaultArray<Real,1>& W);
#line 89 "src/primitive/resample.birch"
#line 89 "src/primitive/resample.birch"
Integer ancestor(const libbirch::DefaultArray<Real,1>& w);
#line 102 "src/primitive/resample.birch"
#line 102 "src/primitive/resample.birch"
libbirch::DefaultArray<Integer,1> systematic_cumulative_offspring(const libbirch::DefaultArray<Real,1>& W);
#line 116 "src/primitive/resample.birch"
#line 116 "src/primitive/resample.birch"
libbirch::DefaultArray<Integer,1> offspring_to_ancestors(const libbirch::DefaultArray<Integer,1>& o);
#line 134 "src/primitive/resample.birch"
#line 134 "src/primitive/resample.birch"
libbirch::DefaultArray<Integer,1> cumulative_offspring_to_ancestors(const libbirch::DefaultArray<Integer,1>& O);
#line 154 "src/primitive/resample.birch"
#line 154 "src/primitive/resample.birch"
libbirch::DefaultArray<Integer,1> cumulative_offspring_to_offspring(const libbirch::DefaultArray<Integer,1>& O);
#line 164 "src/primitive/resample.birch"
#line 164 "src/primitive/resample.birch"
libbirch::DefaultArray<Integer,1> permute_ancestors(const libbirch::DefaultArray<Integer,1>& a);
#line 183 "src/primitive/resample.birch"
#line 183 "src/primitive/resample.birch"
libbirch::DefaultArray<Real,1> cumulative_weights(const libbirch::DefaultArray<Real,1>& w);
#line 203 "src/primitive/resample.birch"
#line 203 "src/primitive/resample.birch"
std::tuple<Real, Real> resample_reduce(const libbirch::DefaultArray<Real,1>& w);
#line 23 "src/system/filesystem.birch"
#line 23 "src/system/filesystem.birch"
void mkdir(const String& path);
#line 43 "src/system/filesystem.birch"
#line 43 "src/system/filesystem.birch"
File fopen(const String& path);
#line 59 "src/system/filesystem.birch"
#line 59 "src/system/filesystem.birch"
File fopen(const String& path, const Integer& mode);
#line 88 "src/system/filesystem.birch"
#line 88 "src/system/filesystem.birch"
String extension(const String& path);
#line 4 "src/system/stdio.birch"
#line 4 "src/system/stdio.birch"
File getStdIn();
#line 13 "src/system/stdio.birch"
#line 13 "src/system/stdio.birch"
File getStdOut();
#line 22 "src/system/stdio.birch"
#line 22 "src/system/stdio.birch"
File getStdErr();
#line 8 "src/system/system.birch"
#line 8 "src/system/system.birch"
Integer system(const String& cmd);
#line 30 "src/system/system.birch"
#line 30 "src/system/system.birch"
void exit(const Integer& code);
#line 10 "src/test/distance.birch"
#line 10 "src/test/distance.birch"
Real wasserstein(const libbirch::DefaultArray<Real,1>& x1, const libbirch::DefaultArray<Real,1>& x2);
#line 9 "src/test/test.birch"
#line 9 "src/test/test.birch"
Boolean pass(const libbirch::DefaultArray<Real,1>& x1, const libbirch::DefaultArray<Real,1>& x2);
#line 38 "src/test/test.birch"
#line 38 "src/test/test.birch"
Boolean pass(const libbirch::DefaultArray<Real,2>& X1, const libbirch::DefaultArray<Real,2>& X2);
#line 7 "src/test/test_cdf.birch"
#line 7 "src/test/test_cdf.birch"
void test_cdf(const Distribution<Real>& q, const Integer& N);
#line 70 "src/test/test_cdf.birch"
#line 70 "src/test/test_cdf.birch"
void test_cdf(const Distribution<Integer>& q);
#line 8 "src/test/test_conjugacy.birch"
#line 8 "src/test/test_conjugacy.birch"
void test_conjugacy(const TestModel& m, const Integer& N, const Boolean& lazy);
#line 9 "src/test/test_grad.birch"
#line 9 "src/test/test_grad.birch"
void test_grad(const TestModel& m, const Integer& N, const Boolean& backward);
#line 8 "src/test/test_pdf.birch"
#line 8 "src/test/test_pdf.birch"
void test_pdf(const Distribution<Boolean>& \u03c0, const Integer& N, const Boolean& lazy);
#line 52 "src/test/test_pdf.birch"
#line 52 "src/test/test_pdf.birch"
void test_pdf(const Distribution<Integer>& \u03c0, const Integer& N, const Boolean& lazy);
#line 125 "src/test/test_pdf.birch"
#line 125 "src/test/test_pdf.birch"
void test_pdf(const Distribution<Real>& \u03c0, const Integer& N, const Integer& B, const Integer& S, const Boolean& lazy);
#line 230 "src/test/test_pdf.birch"
#line 230 "src/test/test_pdf.birch"
void test_pdf(const Distribution<libbirch::DefaultArray<Real,1>>& \u03c0, const Integer& N, const Integer& B, const Integer& S, const Boolean& lazy);
#line 344 "src/test/test_pdf.birch"
#line 344 "src/test/test_pdf.birch"
void test_pdf(const Distribution<libbirch::DefaultArray<Real,2>>& \u03c0, const Integer& N, const Integer& B, const Integer& S, const Boolean& lazy);
#line 11 "src/utility/chrono.birch"
#line 11 "src/utility/chrono.birch"
void tic();
#line 20 "src/utility/chrono.birch"
#line 20 "src/utility/chrono.birch"
Real toc();
#line 4 "src/utility/collect.birch"
#line 4 "src/utility/collect.birch"
void collect();
#line 8 "src/utility/construct.birch"
template<class Type>
#line 8 "src/utility/construct.birch"
Type construct();
#line 23 "src/utility/construct.birch"
template<class Type,class Arg1>
#line 23 "src/utility/construct.birch"
Type construct(const Arg1& arg1);
#line 38 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2>
#line 38 "src/utility/construct.birch"
Type construct(const Arg1& arg1, const Arg2& arg2);
#line 53 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3>
#line 53 "src/utility/construct.birch"
Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3);
#line 69 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3,class Arg4>
#line 69 "src/utility/construct.birch"
Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3, const Arg4& arg4);
#line 85 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 85 "src/utility/construct.birch"
Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3, const Arg4& arg4, const Arg5& arg5);
#line 101 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 101 "src/utility/construct.birch"
Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3, const Arg4& arg4, const Arg5& arg5, const Arg6& arg6);
#line 117 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 117 "src/utility/construct.birch"
Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3, const Arg4& arg4, const Arg5& arg5, const Arg6& arg6, const Arg7& arg7);
#line 134 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7,class Arg8>
#line 134 "src/utility/construct.birch"
Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3, const Arg4& arg4, const Arg5& arg5, const Arg6& arg6, const Arg7& arg7, const Arg8& arg8);
#line 4 "src/utility/copy.birch"
template<class Type>
#line 4 "src/utility/copy.birch"
void bridge(const Type& o);
#line 14 "src/utility/copy.birch"
template<class Type>
#line 14 "src/utility/copy.birch"
Type copy(const Type& o);
#line 27 "src/utility/copy.birch"
template<class Type>
#line 27 "src/utility/copy.birch"
libbirch::DefaultArray<Type,1> copy(const Type& o, const Integer& length);
#line 6 "src/utility/error.birch"
#line 6 "src/utility/error.birch"
void error(const String& msg);
#line 16 "src/utility/error.birch"
#line 16 "src/utility/error.birch"
void warn(const String& msg);
#line 12 "src/utility/make.birch"
template<class Type>
#line 12 "src/utility/make.birch"
std::optional<Type> make();
#line 29 "src/utility/make.birch"
template<class Type>
#line 29 "src/utility/make.birch"
std::optional<Type> make(const String& name);
#line 56 "src/utility/make.birch"
#line 56 "src/utility/make.birch"
std::optional<Object> make(const Buffer& buffer);
#line 86 "src/utility/make.birch"
template<class Type>
#line 86 "src/utility/make.birch"
std::optional<Type> make(const Buffer& buffer);
#line 61 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 61 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,2>> MatrixGaussian(const Arg1& M, const Arg2& U, const Arg3& V);
#line 73 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
#line 73 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
libbirch::DefaultArray<Real,2> simulate_matrix_gaussian(const libbirch::DefaultArray<Real,2>& M, const libbirch::DefaultArray<Real,2>& U, const libbirch::DefaultArray<Real,2>& V);
#line 97 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 97 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
decltype(auto) logpdf_matrix_gaussian(const Arg1& X, const Arg2& M, const Arg3& U, const Arg4& V);
#line 81 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 81 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
decltype(auto) LinearMatrixNormalInverseWishartMatrixGaussian(const Arg1& A, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& C, const Arg7& U);
#line 91 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 91 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,2>> MatrixGaussian(const Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,2>>>, Arg2>& M, const Arg3& U, const Random<libbirch::DefaultArray<Real,2>>& V);
#line 77 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 77 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
decltype(auto) MatrixNormalInverseWishart(const Arg1& N, const Arg2& \u039b, const Arg3& \u03a8, const Arg4& k);
#line 86 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 86 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
Distribution<libbirch::DefaultArray<Real,2>> MatrixGaussian(const Arg1& M, const Arg2& U, const Random<libbirch::DefaultArray<Real,2>>& V);
#line 108 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 108 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
libbirch::DefaultArray<Real,2> simulate_matrix_normal_inverse_wishart(const libbirch::DefaultArray<Real,2>& N, const libbirch::DefaultArray<Real,2>& \u039b, const libbirch::DefaultArray<Real,2>& \u03a8, const Real& k);
#line 126 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 126 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
decltype(auto) logpdf_matrix_normal_inverse_wishart(const Arg1& X, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k);
#line 144 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 144 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
decltype(auto) update_matrix_normal_inverse_wishart(const Arg1& X, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k);
#line 66 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 66 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
decltype(auto) MatrixNormalInverseWishartMatrixGaussian(const Arg1& N, const Arg2& \u039b, const Arg3& \u03a8, const Arg4& k, const Arg5& U);
#line 75 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg>
#line 75 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,2>> MatrixGaussian(const Random<libbirch::DefaultArray<Real,2>>& M, const Arg& U, const Random<libbirch::DefaultArray<Real,2>>& V);
#line 109 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 109 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
libbirch::DefaultArray<Real,2> simulate_matrix_normal_inverse_wishart_matrix_gaussian(const libbirch::DefaultArray<Real,2>& N, const libbirch::DefaultArray<Real,2>& \u039b, const libbirch::DefaultArray<Real,2>& \u03a8, const Real& k, const libbirch::DefaultArray<Real,2>& U);
#line 129 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 129 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
decltype(auto) logpdf_matrix_normal_inverse_wishart_matrix_gaussian(const Arg1& X, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& U);
#line 150 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 150 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
decltype(auto) update_matrix_normal_inverse_wishart_matrix_gaussian(const Arg1& X, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& U);
#line 73 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 73 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
decltype(auto) ScaledMatrixNormalInverseWishartMatrixGaussian(const Arg1& A, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& U);
#line 82 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 82 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,2>> MatrixGaussian(const Multiply<Arg1, Random<libbirch::DefaultArray<Real,2>>>& M, const Arg2& U, const Random<libbirch::DefaultArray<Real,2>>& V);
#line 119 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 119 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
libbirch::DefaultArray<Real,2> simulate_scaled_matrix_normal_inverse_wishart_matrix_gaussian(const libbirch::DefaultArray<Real,2>& A, const libbirch::DefaultArray<Real,2>& N, const libbirch::DefaultArray<Real,2>& \u039b, const libbirch::DefaultArray<Real,2>& \u03a8, const Real& k, const libbirch::DefaultArray<Real,2>& U);
#line 142 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 142 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
decltype(auto) logpdf_scaled_matrix_normal_inverse_wishart_matrix_gaussian(const Arg1& X, const Arg2& A, const Arg3& N, const Arg4& \u039b, const Arg5& \u03a8, const Arg6& k, const Arg7& U);
#line 165 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 165 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
decltype(auto) update_scaled_matrix_normal_inverse_wishart_matrix_gaussian(const Arg1& X, const Arg2& A, const Arg3& N, const Arg4& \u039b, const Arg5& \u03a8, const Arg6& k, const Arg7& U);
#line 81 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 81 "src/distribution/matrix_other/InverseWishartDistribution.birch"
decltype(auto) InverseWishart(const Arg1& \u03a8, const Arg2& k);
#line 91 "src/distribution/matrix_other/InverseWishartDistribution.birch"
#line 91 "src/distribution/matrix_other/InverseWishartDistribution.birch"
libbirch::DefaultArray<Real,2> simulate_inverse_wishart(const libbirch::DefaultArray<Real,2>& \u03a8, const Real& k);
#line 104 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 104 "src/distribution/matrix_other/InverseWishartDistribution.birch"
decltype(auto) logpdf_inverse_wishart(const Arg1& X, const Arg2& \u03a8, const Arg3& k);
#line 9 "src/distribution/matrix_other/MatrixTDistribution.birch"
#line 9 "src/distribution/matrix_other/MatrixTDistribution.birch"
libbirch::DefaultArray<Real,2> simulate_matrix_t(const Real& k, const libbirch::DefaultArray<Real,2>& M, const libbirch::DefaultArray<Real,2>& U, const libbirch::DefaultArray<Real,2>& \u03a8);
#line 26 "src/distribution/matrix_other/MatrixTDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 26 "src/distribution/matrix_other/MatrixTDistribution.birch"
decltype(auto) logpdf_matrix_t(const Arg1& X, const Arg2& k, const Arg3& M, const Arg4& U, const Arg5& V);
#line 45 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 45 "src/distribution/matrix_other/WishartDistribution.birch"
decltype(auto) Wishart(const Arg1& \u03a8, const Arg2& k);
#line 55 "src/distribution/matrix_other/WishartDistribution.birch"
#line 55 "src/distribution/matrix_other/WishartDistribution.birch"
libbirch::DefaultArray<Real,2> simulate_wishart(const libbirch::DefaultArray<Real,2>& \u03a8, const Real& k);
#line 82 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 82 "src/distribution/matrix_other/WishartDistribution.birch"
decltype(auto) logpdf_wishart(const Arg1& X, const Arg2& \u03a8, const Arg3& k);
#line 33 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 33 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
decltype(auto) DirichletMultinomial(const Arg1& n, const Arg2& \u03b1);
#line 40 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg>
#line 40 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
Distribution<libbirch::DefaultArray<Integer,1>> Multinomial(const Arg& n, const Random<libbirch::DefaultArray<Real,1>>& \u03c1);
#line 59 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
#line 59 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
libbirch::DefaultArray<Integer,1> simulate_dirichlet_multinomial(const Integer& n, const libbirch::DefaultArray<Real,1>& \u03b1);
#line 72 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
#line 72 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
Real logpdf_dirichlet_multinomial(const libbirch::DefaultArray<Integer,1>& x, const Integer& n, const libbirch::DefaultArray<Real,1>& \u03b1);
#line 100 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
#line 100 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
DirichletDistribution<libbirch::DefaultArray<Real,1>> update_dirichlet_multinomial(const libbirch::DefaultArray<Integer,1>& x, const Integer& n, const libbirch::DefaultArray<Real,1>& \u03b1);
#line 38 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 38 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
decltype(auto) Multinomial(const Arg1& n, const Arg2& \u03c1);
#line 54 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
#line 54 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
libbirch::DefaultArray<Integer,1> simulate_multinomial(const Integer& n, const libbirch::DefaultArray<Real,1>& \u03c1);
#line 71 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
#line 71 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
libbirch::DefaultArray<Integer,1> simulate_multinomial(const Integer& n, const libbirch::DefaultArray<Real,1>& \u03c1, const Real& Z);
#line 109 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
#line 109 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
Real logpdf_multinomial(const libbirch::DefaultArray<Integer,1>& x, const Integer& n, const libbirch::DefaultArray<Real,1>& \u03c1);
#line 76 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 76 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
decltype(auto) LinearMultivariateGaussianMultivariateGaussian(const Arg1& A, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& c, const Arg5& S);
#line 85 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 85 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Arg3& \u03a3);
#line 55 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 55 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
decltype(auto) MultivariateGaussian(const Arg1& \u03bc, const Arg2& \u03a3);
#line 65 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
#line 65 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> simulate_multivariate_gaussian(const libbirch::DefaultArray<Real,1>& \u03bc, const libbirch::DefaultArray<Real,2>& \u03a3);
#line 84 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 84 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
decltype(auto) logpdf_multivariate_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03a3);
#line 64 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 64 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
decltype(auto) MultivariateGaussianMultivariateGaussian(const Arg1& \u03bc, const Arg2& \u03a3, const Arg3& S);
#line 73 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg>
#line 73 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Random<libbirch::DefaultArray<Real,1>>& \u03bc, const Arg& \u03a3);
#line 94 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
#line 94 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> simulate_multivariate_gaussian_multivariate_gaussian(const libbirch::DefaultArray<Real,1>& \u03bc, const libbirch::DefaultArray<Real,2>& \u03a3, const libbirch::DefaultArray<Real,2>& S);
#line 109 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 109 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
decltype(auto) logpdf_multivariate_gaussian_multivariate_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& S);
#line 125 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 125 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
decltype(auto) update_multivariate_gaussian_multivariate_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& S);
#line 70 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 70 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
decltype(auto) ScaledMultivariateGaussianMultivariateGaussian(const Arg1& A, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& S);
#line 79 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 79 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Arg2& \u03a3);
#line 102 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 102 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> simulate_scaled_multivariate_gaussian_multivariate_gaussian(const libbirch::DefaultArray<Real,2>& A, const libbirch::DefaultArray<Real,1>& \u03bc, const libbirch::DefaultArray<Real,2>& \u03a3, const libbirch::DefaultArray<Real,2>& S);
#line 118 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 118 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
decltype(auto) logpdf_scaled_multivariate_gaussian_multivariate_gaussian(const Arg1& x, const Arg2& A, const Arg3& \u03bc, const Arg4& \u03a3, const Arg5& S);
#line 135 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 135 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
decltype(auto) update_scaled_multivariate_gaussian_multivariate_gaussian(const Arg1& x, const Arg2& A, const Arg3& \u03bc, const Arg4& \u03a3, const Arg5& S);
#line 81 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 81 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
decltype(auto) LinearMultivariateNormalInverseGammaMultivariateGaussian(const Arg1& A, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& \u03a3);
#line 91 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 91 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Multiply<Arg3, Random<Real>>& \u03a3);
#line 120 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 120 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Multiply<Random<Real>, Arg3>& \u03a3);
#line 107 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 107 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
decltype(auto) MultivariateNormalInverseGamma(const Arg1& \u03bd, const Arg2& \u039b, const Arg3& \u03b1, const Arg4& \u03b2);
#line 116 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 116 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Arg1& \u03bc, const Multiply<Arg2, Random<Real>>& \u03a3);
#line 134 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 134 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Arg1& \u03bc, const Multiply<Random<Real>, Arg2>& \u03a3);
#line 157 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 157 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
libbirch::DefaultArray<Real,1> simulate_multivariate_normal_inverse_gamma(const libbirch::DefaultArray<Real,1>& \u03bd, const libbirch::DefaultArray<Real,2>& \u039b, const Real& \u03b1, const Real& \u03b2);
#line 174 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 174 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
decltype(auto) logpdf_multivariate_normal_inverse_gamma(const Arg1& x, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2);
#line 192 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 192 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
decltype(auto) update_multivariate_normal_inverse_gamma(const Arg1& x, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2);
#line 67 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 67 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
decltype(auto) MultivariateNormalInverseGammaMultivariateGaussian(const Arg1& \u03bd, const Arg2& \u039b, const Arg3& \u03b1, const Arg4& \u03b2, const Arg5& \u03a3);
#line 76 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg>
#line 76 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Random<libbirch::DefaultArray<Real,1>>& \u03bc, const Multiply<Arg, Random<Real>>& \u03a3);
#line 105 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg>
#line 105 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Random<libbirch::DefaultArray<Real,1>>& \u03bc, const Multiply<Random<Real>, Arg>& \u03a3);
#line 141 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 141 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> simulate_multivariate_normal_inverse_gamma_multivariate_gaussian(const libbirch::DefaultArray<Real,1>& \u03bd, const libbirch::DefaultArray<Real,2>& \u039b, const Real& \u03b1, const Real& \u03b2, const libbirch::DefaultArray<Real,2>& \u03a3);
#line 161 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 161 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
decltype(auto) logpdf_multivariate_normal_inverse_gamma_multivariate_gaussian(const Arg1& x, const Arg2& \u03bd, const Arg3& \u039b, const Arg5& \u03b1, const Arg6& \u03b2, const Arg4& \u03a3);
#line 182 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 182 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
decltype(auto) update_multivariate_normal_inverse_gamma_multivariate_gaussian(const Arg1& x, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& \u03a3);
#line 73 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 73 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
decltype(auto) ScaledMultivariateNormalInverseGammaMultivariateGaussian(const Arg1& A, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& \u03a3);
#line 83 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 83 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Multiply<Arg2, Random<Real>>& \u03a3);
#line 111 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 111 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Multiply<Random<Real>, Arg2>& \u03a3);
#line 147 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 147 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> simulate_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(const libbirch::DefaultArray<Real,2>& A, const libbirch::DefaultArray<Real,1>& \u03bd, const libbirch::DefaultArray<Real,2>& \u039b, const Real& \u03b1, const Real& \u03b2, const libbirch::DefaultArray<Real,2>& \u03a3);
#line 170 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 170 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
decltype(auto) logpdf_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(const Arg1& x, const Arg2& A, const Arg3& \u03bd, const Arg4& \u039b, const Arg5& \u03b1, const Arg6& \u03b2, const Arg7& \u03a3);
#line 193 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 193 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
decltype(auto) update_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(const Arg1& x, const Arg2& A, const Arg3& \u03bd, const Arg4& \u039b, const Arg5& \u03b1, const Arg6& \u03b2, const Arg7& \u03a3);
#line 73 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 73 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
decltype(auto) LinearMatrixNormalInverseWishartMultivariateGaussian(const Arg1& a, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& c);
#line 82 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 82 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,2>>>, Arg2>& \u03bc, const Random<libbirch::DefaultArray<Real,2>>& \u03a3);
#line 68 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 68 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
decltype(auto) ScaledMatrixNormalInverseWishartMultivariateGaussian(const Arg1& a, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k);
#line 77 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg>
#line 77 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Dot<Arg, Random<libbirch::DefaultArray<Real,2>>>& \u03bc, const Random<libbirch::DefaultArray<Real,2>>& \u03a3);
#line 110 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 110 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> simulate_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(const libbirch::DefaultArray<Real,1>& a, const libbirch::DefaultArray<Real,2>& N, const libbirch::DefaultArray<Real,2>& \u039b, const libbirch::DefaultArray<Real,2>& \u03a8, const Real& k);
#line 131 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 131 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
decltype(auto) logpdf_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(const Arg1& x, const Arg2& a, const Arg3& N, const Arg4& \u039b, const Arg5& \u03a8, const Arg6& k);
#line 153 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 153 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
decltype(auto) update_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(const Arg1& x, const Arg2& a, const Arg3& N, const Arg4& \u039b, const Arg5& \u03a8, const Arg6& k);
#line 39 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 39 "src/distribution/multivariate_other/DirichletDistribution.birch"
decltype(auto) Dirichlet(const Arg& \u03b1);
#line 48 "src/distribution/multivariate_other/DirichletDistribution.birch"
#line 48 "src/distribution/multivariate_other/DirichletDistribution.birch"
libbirch::DefaultArray<Real,1> simulate_dirichlet(const libbirch::DefaultArray<Real,1>& \u03b1);
#line 59 "src/distribution/multivariate_other/DirichletDistribution.birch"
#line 59 "src/distribution/multivariate_other/DirichletDistribution.birch"
libbirch::DefaultArray<Real,1> simulate_dirichlet(const Real& \u03b1, const Integer& D);
#line 72 "src/distribution/multivariate_other/DirichletDistribution.birch"
#line 72 "src/distribution/multivariate_other/DirichletDistribution.birch"
Real logpdf_dirichlet(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Real,1>& \u03b1);
#line 8 "src/distribution/multivariate_other/MultivariateTDistribution.birch"
#line 8 "src/distribution/multivariate_other/MultivariateTDistribution.birch"
libbirch::DefaultArray<Real,1> simulate_multivariate_t(const Real& k, const libbirch::DefaultArray<Real,1>& \u03bc, const libbirch::DefaultArray<Real,2>& U);
#line 23 "src/distribution/multivariate_other/MultivariateTDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 23 "src/distribution/multivariate_other/MultivariateTDistribution.birch"
decltype(auto) logpdf_multivariate_t(const Arg1& x, const Arg2& k, const Arg3& \u03bc, const Arg4& U);
#line 73 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 73 "src/distribution/multivariate_other/RestaurantDistribution.birch"
decltype(auto) Restaurant(const Arg1& \u03b1, const Arg2& \u03b8);
#line 39 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 39 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
decltype(auto) Bernoulli(const Arg& \u03c1);
#line 48 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
#line 48 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
Boolean simulate_bernoulli(const Real& \u03c1);
#line 63 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 63 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
decltype(auto) logpdf_bernoulli(const Arg1& x, const Arg2& \u03c1);
#line 44 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 44 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
decltype(auto) BetaBernoulli(const Arg1& \u03b1, const Arg2& \u03b2);
#line 51 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
#line 51 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
Distribution<Boolean> Bernoulli(const Random<Real>& \u03c1);
#line 69 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
#line 69 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
Boolean simulate_beta_bernoulli(const Real& \u03b1, const Real& \u03b2);
#line 84 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 84 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
decltype(auto) logpdf_beta_bernoulli(const Arg1& x, const Arg2& \u03b1, const Arg3& \u03b2);
#line 97 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 97 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
decltype(auto) update_beta_bernoulli(const Arg1& x, const Arg2& \u03b1, const Arg3& \u03b2);
#line 62 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 62 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
decltype(auto) BetaBinomial(const Arg1& n, const Arg2& \u03b1, const Arg3& \u03b2);
#line 69 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1>
#line 69 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
Distribution<Integer> Binomial(const Arg1& n, const Random<Real>& \u03c1);
#line 88 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 88 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
Integer simulate_beta_binomial(const Integer& n, const Real& \u03b1, const Real& \u03b2);
#line 105 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 105 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
decltype(auto) logpdf_beta_binomial(const Arg1& x, const Arg2& n, const Arg3& \u03b1, const Arg4& \u03b2);
#line 120 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 120 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
decltype(auto) update_beta_binomial(const Arg1& x, const Arg2& n, const Arg3& \u03b1, const Arg4& \u03b2);
#line 135 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 135 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
Real cdf_beta_binomial(const Integer& x, const Integer& n, const Real& \u03b1, const Real& \u03b2);
#line 54 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 54 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
decltype(auto) BetaNegativeBinomial(const Arg1& k, const Arg2& \u03b1, const Arg3& \u03b2);
#line 61 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1>
#line 61 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
Distribution<Integer> NegativeBinomial(const Arg1& k, const Random<Real>& \u03c1);
#line 80 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
#line 80 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
Integer simulate_beta_negative_binomial(const Integer& k, const Real& \u03b1, const Real& \u03b2);
#line 97 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 97 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
decltype(auto) logpdf_beta_negative_binomial(const Arg1& x, const Arg2& k, const Arg3& \u03b1, const Arg4& \u03b2);
#line 112 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 112 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
decltype(auto) update_beta_negative_binomial(const Arg1& x, const Arg2& k, const Arg3& \u03b1, const Arg4& \u03b2);
#line 66 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 66 "src/distribution/scalar_discrete/BinomialDistribution.birch"
decltype(auto) Binomial(const Arg1& n, const Arg2& \u03c1);
#line 76 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 76 "src/distribution/scalar_discrete/BinomialDistribution.birch"
Integer simulate_binomial(const Integer& n, const Real& \u03c1);
#line 93 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 93 "src/distribution/scalar_discrete/BinomialDistribution.birch"
decltype(auto) logpdf_binomial(const Arg1& x, const Arg2& n, const Arg3& \u03c1);
#line 106 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 106 "src/distribution/scalar_discrete/BinomialDistribution.birch"
Real cdf_binomial(const Integer& x, const Integer& n, const Real& \u03c1);
#line 127 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 127 "src/distribution/scalar_discrete/BinomialDistribution.birch"
Integer quantile_binomial(const Real& P, const Integer& n, const Real& \u03c1);
#line 50 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 50 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
decltype(auto) Categorical(const Arg& \u03c1);
#line 59 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 59 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Integer simulate_categorical(const libbirch::DefaultArray<Real,1>& \u03c1);
#line 69 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 69 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Integer simulate_categorical(const libbirch::DefaultArray<Real,1>& \u03c1, const Real& Z);
#line 93 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 93 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Real logpdf_categorical(const Integer& x, const libbirch::DefaultArray<Real,1>& \u03c1);
#line 106 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 106 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Real logpdf_categorical(const Integer& x, const libbirch::DefaultArray<Real,1>& \u03c1, const Real& Z);
#line 122 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 122 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Real cdf_categorical(const Integer& x, const libbirch::DefaultArray<Real,1>& \u03c1);
#line 135 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 135 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Real cdf_categorical(const Integer& x, const libbirch::DefaultArray<Real,1>& \u03c1, const Real& Z);
#line 153 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 153 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Integer quantile_categorical(const Real& P, const libbirch::DefaultArray<Real,1>& \u03c1);
#line 166 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 166 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Integer quantile_categorical(const Real& P, const libbirch::DefaultArray<Real,1>& \u03c1, const Real& Z);
#line 78 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 78 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
decltype(auto) ConwayMaxwellPoisson(const Arg1& \u03bb, const Arg2& \u03bd, const Arg3& n);
#line 89 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 89 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
Integer simulate_conway_maxwell_poisson(const Real& \u03bb, const Real& \u03bd, const Integer& n);
#line 105 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 105 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
decltype(auto) logpdf_conway_maxwell_poisson(const Arg1& x, const Arg2& \u03bb, const Arg3& \u03bd, const Arg4& n);
#line 120 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 120 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
Real cdf_conway_maxwell_poisson(const Integer& x, const Real& \u03bb, const Real& \u03bd, const Integer& n);
#line 136 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 136 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
Integer quantile_conway_maxwell_poisson(const Real& P, const Real& \u03bb, const Real& \u03bd, const Integer& n);
#line 152 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 152 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
libbirch::DefaultArray<Real,1> p_conway_maxwell_poisson(const Real& \u03bb, const Real& \u03bd, const Integer& n);
#line 35 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 35 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
decltype(auto) DirichletCategorical(const Arg& \u03b1);
#line 42 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 42 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
Distribution<Integer> Categorical(const Random<libbirch::DefaultArray<Real,1>>& \u03c1);
#line 64 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 64 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
Integer simulate_dirichlet_categorical(const libbirch::DefaultArray<Real,1>& \u03b1);
#line 76 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 76 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
Real logpdf_dirichlet_categorical(const Integer& x, const libbirch::DefaultArray<Real,1>& \u03b1);
#line 94 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg1,class Arg2>
#line 94 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
decltype(auto) update_dirichlet_categorical(const Arg1& x, const Arg2& \u03b1);
#line 61 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 61 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
decltype(auto) GammaPoisson(const Arg1& k, const Arg2& \u03b8);
#line 68 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 68 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Distribution<Integer> Poisson(const Random<Real>& \u03bb);
#line 86 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 86 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Integer simulate_gamma_poisson(const Real& k, const Real& \u03b8);
#line 104 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 104 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
decltype(auto) logpdf_gamma_poisson(const Arg1& x, const Arg2& k, const Arg3& \u03b8);
#line 118 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 118 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
decltype(auto) update_gamma_poisson(const Arg1& x, const Arg2& k, const Arg3& \u03b8);
#line 131 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 131 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Real cdf_gamma_poisson(const Integer& x, const Real& k, const Real& \u03b8);
#line 146 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 146 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Integer quantile_gamma_poisson(const Real& P, const Real& k, const Real& \u03b8);
#line 4 "src/distribution/scalar_discrete/GeometricDistribution.birch"
template<class Arg>
#line 4 "src/distribution/scalar_discrete/GeometricDistribution.birch"
decltype(auto) Geometric(const Arg& \u03c1);
#line 62 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 62 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
decltype(auto) NegativeBinomial(const Arg1& k, const Arg2& \u03c1);
#line 74 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 74 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
Integer simulate_negative_binomial(const Integer& k, const Real& \u03c1);
#line 91 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 91 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
decltype(auto) logpdf_negative_binomial(const Arg1& x, const Arg2& k, const Arg3& \u03c1);
#line 104 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 104 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
Real cdf_negative_binomial(const Integer& x, const Integer& k, const Real& \u03c1);
#line 123 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 123 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
Integer quantile_negative_binomial(const Real& P, const Integer& k, const Real& \u03c1);
#line 55 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 55 "src/distribution/scalar_discrete/PoissonDistribution.birch"
Distribution<Integer> Poisson(const Arg& \u03bb);
#line 64 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 64 "src/distribution/scalar_discrete/PoissonDistribution.birch"
Integer simulate_poisson(const Real& \u03bb);
#line 83 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 83 "src/distribution/scalar_discrete/PoissonDistribution.birch"
decltype(auto) logpdf_poisson(const Arg1& x, const Arg2& \u03bb);
#line 95 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 95 "src/distribution/scalar_discrete/PoissonDistribution.birch"
Real cdf_poisson(const Integer& x, const Real& \u03bb);
#line 113 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 113 "src/distribution/scalar_discrete/PoissonDistribution.birch"
Integer quantile_poisson(const Real& P, const Real& \u03bb);
#line 46 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 46 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
decltype(auto) RestaurantCategorical(const Arg1& \u03b1, const Arg2& \u03b8, const Arg3& n, const Arg4& N);
#line 60 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
#line 60 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
Integer simulate_restaurant_categorical(const Real& \u03b1, const Real& \u03b8, const libbirch::DefaultArray<Integer,1>& n, const Integer& N);
#line 96 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
#line 96 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
Real logpdf_restaurant_categorical(const Integer& k, const Real& \u03b1, const Real& \u03b8, const libbirch::DefaultArray<Integer,1>& n, const Integer& N);
#line 118 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 118 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
decltype(auto) update_restaurant_categorical(const Arg1& x, const Arg2& \u03b1, const Arg3& \u03b8, const Arg4& n, const Arg5& N);
#line 62 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 62 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
decltype(auto) ScaledGammaPoisson(const Arg1& a, const Arg2& k, const Arg3& \u03b8);
#line 69 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg>
#line 69 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
Distribution<Integer> Poisson(const Multiply<Arg, Random<Real>>& \u03bb);
#line 85 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg>
#line 85 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
Distribution<Integer> Poisson(const Multiply<Random<Real>, Arg>& \u03bb);
#line 101 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 101 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
Distribution<Integer> Poisson(const Multiply<Random<Real>, Random<Real>>& \u03bb);
#line 133 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 133 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
decltype(auto) update_scaled_gamma_poisson(const Arg1& x, const Arg2& a, const Arg3& k, const Arg4& \u03b8);
#line 58 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 58 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
decltype(auto) UniformInteger(const Arg1& l, const Arg2& u);
#line 68 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 68 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Integer simulate_uniform_int(const Integer& l, const Integer& u);
#line 84 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 84 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Real logpdf_uniform_int(const Integer& x, const Integer& l, const Integer& u);
#line 101 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 101 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Real cdf_uniform_int(const Integer& x, const Integer& l, const Integer& u);
#line 120 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 120 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Integer quantile_uniform_int(const Real& P, const Integer& l, const Integer& u);
#line 88 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
#line 88 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
Distribution<Integer> Delta(const Add<Random<Integer>, Random<Integer>>& \u03bc);
#line 113 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
template<class Arg>
#line 113 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
decltype(auto) Delta(const Add<Random<Integer>, Arg>& \u03bc);
#line 121 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
template<class Arg>
#line 121 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
decltype(auto) Delta(const Add<Arg, Random<Integer>>& \u03bc);
#line 129 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
template<class Arg1,class Arg2>
#line 129 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
decltype(auto) Delta(const Add<Arg1, Arg2>& \u03bc);
#line 38 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 38 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
Distribution<Integer> Delta(const Arg& \u03bc);
#line 37 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
#line 37 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
Distribution<Integer> Delta(const Random<Integer>& \u03bc);
#line 48 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
#line 48 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
Distribution<Integer> Delta(const Negate<Random<Integer>>& \u03bc);
#line 62 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
template<class Arg>
#line 62 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
decltype(auto) Delta(const Negate<Arg>& \u03bc);
#line 86 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
#line 86 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
Distribution<Integer> Delta(const Subtract<Random<Integer>, Random<Integer>>& \u03bc);
#line 111 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
template<class Arg>
#line 111 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
decltype(auto) Delta(const Subtract<Random<Integer>, Arg>& \u03bc);
#line 119 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
template<class Arg>
#line 119 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
decltype(auto) Delta(const Subtract<Arg, Random<Integer>>& \u03bc);
#line 127 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
template<class Arg1,class Arg2>
#line 127 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
decltype(auto) Delta(const Subtract<Arg1, Arg2>& \u03bc);
#line 65 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 65 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
decltype(auto) Gaussian(const Arg1& \u03bc, const Arg2& \u03c32);
#line 75 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 75 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
Real simulate_gaussian(const Real& \u03bc, const Real& \u03c32);
#line 95 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 95 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
decltype(auto) logpdf_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03c32);
#line 108 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 108 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
Real cdf_gaussian(const Real& x, const Real& \u03bc, const Real& \u03c32);
#line 122 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 122 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
Real quantile_gaussian(const Real& P, const Real& \u03bc, const Real& \u03c32);
#line 66 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 66 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
decltype(auto) GaussianGaussian(const Arg1& \u03bc, const Arg2& \u03c32, const Arg3& s2);
#line 73 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg>
#line 73 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
Distribution<Real> Gaussian(const Random<Real>& \u03bc, const Arg& \u03c32);
#line 93 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 93 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
Real simulate_gaussian_gaussian(const Real& \u03bc, const Real& \u03c32, const Real& s2);
#line 107 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 107 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
decltype(auto) logpdf_gaussian_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03c32, const Arg4& s2);
#line 122 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 122 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
Real cdf_gaussian_gaussian(const Real& x, const Real& \u03bc, const Real& \u03c32, const Real& s2);
#line 136 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 136 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
Real quantile_gaussian_gaussian(const Real& P, const Real& \u03bc, const Real& \u03c32, const Real& s2);
#line 152 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 152 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
decltype(auto) update_gaussian_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03c32, const Arg4& s2);
#line 82 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 82 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
decltype(auto) LinearGaussianGaussian(const Arg1& a, const Arg2& \u03bc, const Arg3& \u03c32, const Arg4& c, const Arg5& s2);
#line 91 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 91 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
Distribution<Real> Gaussian(const Add<Multiply<Arg1, Random<Real>>, Arg2>& \u03bc, const Arg3& \u03c32);
#line 108 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 108 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
Distribution<Real> Gaussian(const Multiply<Arg1, Random<Real>>& \u03bc, const Arg2& \u03c32);
#line 123 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 123 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
Real simulate_linear_gaussian_gaussian(const Real& a, const Real& \u03bc, const Real& \u03c32, const Real& c, const Real& s2);
#line 141 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 141 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
decltype(auto) logpdf_linear_gaussian_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bc, const Arg4& \u03c32, const Arg5& c, const Arg6& s2);
#line 159 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 159 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
Real cdf_linear_gaussian_gaussian(const Real& x, const Real& a, const Real& \u03bc, const Real& \u03c32, const Real& c, const Real& s2);
#line 177 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 177 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
Real quantile_linear_gaussian_gaussian(const Real& P, const Real& a, const Real& \u03bc, const Real& \u03c32, const Real& c, const Real& s2);
#line 195 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 195 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
decltype(auto) update_linear_gaussian_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bc, const Arg4& \u03c32, const Arg5& c, const Arg6& s2);
#line 83 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 83 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
decltype(auto) LinearMultivariateGaussianGaussian(const Arg1& a, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& c, const Arg5& s2);
#line 92 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 92 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
Distribution<Real> Gaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Arg3& \u03c32);
#line 109 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 109 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
Distribution<Real> Gaussian(const Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Arg2& \u03c32);
#line 124 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 124 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
Real simulate_linear_multivariate_gaussian_gaussian(const libbirch::DefaultArray<Real,1>& a, const libbirch::DefaultArray<Real,1>& \u03bc, const libbirch::DefaultArray<Real,2>& \u03a3, const Real& c, const Real& s2);
#line 142 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 142 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
decltype(auto) logpdf_linear_multivariate_gaussian_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bc, const Arg4& \u03a3, const Arg5& c, const Arg6& s2);
#line 160 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 160 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
Real cdf_linear_multivariate_gaussian_gaussian(const Real& x, const libbirch::DefaultArray<Real,1>& a, const libbirch::DefaultArray<Real,1>& \u03bc, const libbirch::DefaultArray<Real,2>& \u03a3, const Real& c, const Real& s2);
#line 178 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 178 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
Real quantile_linear_multivariate_gaussian_gaussian(const Real& P, const libbirch::DefaultArray<Real,1>& a, const libbirch::DefaultArray<Real,1>& \u03bc, const libbirch::DefaultArray<Real,2>& \u03a3, const Real& c, const Real& s2);
#line 197 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 197 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
decltype(auto) update_linear_multivariate_gaussian_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bc, const Arg4& \u03a3, const Arg5& c, const Arg6& s2);
#line 92 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 92 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
decltype(auto) LinearMultivariateNormalInverseGammaGaussian(const Arg1& a, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& s2);
#line 102 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 102 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Random<Real>& \u03c32);
#line 110 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 110 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Multiply<Random<Real>, Arg3>& \u03c32);
#line 118 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 118 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Multiply<Arg3, Random<Real>>& \u03c32);
#line 151 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 151 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32);
#line 199 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 199 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Dot<Arg, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Random<Real>& \u03c32);
#line 207 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 207 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Multiply<Arg2, Random<Real>>& \u03c32);
#line 215 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 215 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Multiply<Random<Real>, Arg2>& \u03c32);
#line 223 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 223 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Dot<Arg, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32);
#line 240 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 240 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Real simulate_linear_multivariate_normal_inverse_gamma_gaussian(const libbirch::DefaultArray<Real,1>& a, const libbirch::DefaultArray<Real,1>& \u03bd, const libbirch::DefaultArray<Real,2>& \u039b, const Real& c, const Real& \u03b1, const Real& \u03b2, const Real& s2);
#line 261 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7,class Arg8>
#line 261 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
decltype(auto) logpdf_linear_multivariate_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bd, const Arg4& \u039b, const Arg5& c, const Arg6& \u03b1, const Arg7& \u03b2, const Arg8& s2);
#line 284 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7,class Arg8>
#line 284 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
decltype(auto) update_linear_multivariate_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bd, const Arg4& \u039b, const Arg5& c, const Arg6& \u03b1, const Arg7& \u03b2, const Arg8& s2);
#line 314 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 314 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Real cdf_linear_multivariate_normal_inverse_gamma_gaussian(const Real& x, const libbirch::DefaultArray<Real,1>& a, const libbirch::DefaultArray<Real,1>& \u03bd, const libbirch::DefaultArray<Real,2>& \u039b, const Real& c, const Real& \u03b1, const Real& \u03b2, const Real& s2);
#line 337 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 337 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Real quantile_linear_multivariate_normal_inverse_gamma_gaussian(const Real& P, const libbirch::DefaultArray<Real,1>& a, const libbirch::DefaultArray<Real,1>& \u03bd, const libbirch::DefaultArray<Real,2>& \u039b, const Real& c, const Real& \u03b1, const Real& \u03b2, const Real& s2);
#line 87 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 87 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
decltype(auto) LinearNormalInverseGammaGaussian(const Arg1& a, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& s2);
#line 96 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 96 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Add<Multiply<Arg1, Random<Real>>, Arg2>& \u03bc, const Random<Real>& \u03c32);
#line 104 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 104 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Add<Multiply<Arg1, Random<Real>>, Arg2>& \u03bc, const Multiply<Random<Real>, Arg3>& \u03c32);
#line 112 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 112 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Add<Multiply<Arg1, Random<Real>>, Arg2>& \u03bc, const Multiply<Arg3, Random<Real>>& \u03c32);
#line 144 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 144 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Add<Multiply<Arg1, Random<Real>>, Arg2>& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32);
#line 189 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 189 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Multiply<Arg, Random<Real>>& \u03bc, const Random<Real>& \u03c32);
#line 197 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 197 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Multiply<Arg1, Random<Real>>& \u03bc, const Multiply<Arg2, Random<Real>>& \u03c32);
#line 205 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 205 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Multiply<Arg1, Random<Real>>& \u03bc, const Multiply<Random<Real>, Arg2>& \u03c32);
#line 213 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 213 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Multiply<Arg, Random<Real>>& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32);
#line 229 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 229 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Real simulate_linear_normal_inverse_gamma_gaussian(const Real& a, const Real& \u03bd, const Real& \u03bb, const Real& \u03b1, const Real& \u03b2, const Real& c, const Real& s2);
#line 249 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7,class Arg8>
#line 249 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
decltype(auto) logpdf_linear_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bd, const Arg4& \u03bb, const Arg5& \u03b1, const Arg6& \u03b2, const Arg7& c, const Arg8& s2);
#line 270 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7,class Arg8>
#line 270 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
decltype(auto) update_linear_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bd, const Arg4& \u03bb, const Arg5& \u03b1, const Arg6& \u03b2, const Arg7& c, const Arg8& s2);
#line 298 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 298 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Real cdf_linear_normal_inverse_gamma_gaussian(const Real& x, const Real& a, const Real& \u03bd, const Real& \u03bb, const Real& \u03b1, const Real& \u03b2, const Real& c, const Real& s2);
#line 318 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 318 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Real quantile_linear_normal_inverse_gamma_gaussian(const Real& P, const Real& a, const Real& \u03bd, const Real& \u03bb, const Real& \u03b1, const Real& \u03b2, const Real& c, const Real& s2);
#line 109 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 109 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
decltype(auto) NormalInverseGamma(const Arg1& \u03bd, const Arg2& \u03bb, const Arg3& \u03b1, const Arg4& \u03b2);
#line 118 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg>
#line 118 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Distribution<Real> Gaussian(const Arg& \u03bc, const Random<Real>& \u03c32);
#line 133 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 133 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Distribution<Real> Gaussian(const Arg1& \u03bc, const Multiply<Arg2, Random<Real>>& \u03c32);
#line 150 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 150 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Distribution<Real> Gaussian(const Arg1& \u03bc, const Multiply<Random<Real>, Arg2>& \u03c32);
#line 167 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg>
#line 167 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Distribution<Real> Gaussian(const Arg& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32);
#line 195 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 195 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Real simulate_normal_inverse_gamma(const Real& \u03bd, const Real& \u03bb, const Real& \u03b1, const Real& \u03b2);
#line 211 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 211 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
decltype(auto) logpdf_normal_inverse_gamma(const Arg1& x, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2);
#line 228 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 228 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
decltype(auto) update_normal_inverse_gamma(const Arg1& x, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2);
#line 244 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 244 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Real cdf_normal_inverse_gamma(const Real& x, const Real& \u03bd, const Real& \u03bb, const Real& \u03b1, const Real& \u03b2);
#line 260 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 260 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Real quantile_normal_inverse_gamma(const Real& P, const Real& \u03bd, const Real& \u03bb, const Real& \u03b1, const Real& \u03b2);
#line 74 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 74 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
decltype(auto) NormalInverseGammaGaussian(const Arg1& \u03bd, const Arg2& \u03bb, const Arg3& \u03b1, const Arg4& \u03b2, const Arg5& s2);
#line 83 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 83 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Random<Real>& \u03bc, const Random<Real>& \u03c32);
#line 90 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 90 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Random<Real>& \u03bc, const Multiply<Random<Real>, Arg>& \u03c32);
#line 98 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 98 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Random<Real>& \u03bc, const Multiply<Arg, Random<Real>>& \u03c32);
#line 130 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 130 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
Distribution<Real> Gaussian(const Random<Real>& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32);
#line 181 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 181 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
Real simulate_normal_inverse_gamma_gaussian(const Real& \u03bd, const Real& \u03bb, const Real& \u03b1, const Real& \u03b2, const Real& s2);
#line 198 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 198 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
decltype(auto) logpdf_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& s2);
#line 216 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 216 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
decltype(auto) update_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& s2);
#line 239 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 239 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
Real cdf_normal_inverse_gamma_gaussian(const Real& x, const Real& \u03bd, const Real& \u03bb, const Real& \u03b1, const Real& \u03b2, const Real& s2);
#line 256 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 256 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
Real quantile_normal_inverse_gamma_gaussian(const Real& P, const Real& \u03bd, const Real& \u03bb, const Real& \u03b1, const Real& \u03b2, const Real& s2);
#line 73 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 73 "src/distribution/scalar_other/BetaDistribution.birch"
decltype(auto) Beta(const Arg1& \u03b1, const Arg2& \u03b2);
#line 83 "src/distribution/scalar_other/BetaDistribution.birch"
#line 83 "src/distribution/scalar_other/BetaDistribution.birch"
Real simulate_beta(const Real& \u03b1, const Real& \u03b2);
#line 100 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 100 "src/distribution/scalar_other/BetaDistribution.birch"
decltype(auto) logpdf_beta(const Arg1& x, const Arg2& \u03b1, const Arg3& \u03b2);
#line 113 "src/distribution/scalar_other/BetaDistribution.birch"
#line 113 "src/distribution/scalar_other/BetaDistribution.birch"
Real cdf_beta(const Real& x, const Real& \u03b1, const Real& \u03b2);
#line 135 "src/distribution/scalar_other/BetaDistribution.birch"
#line 135 "src/distribution/scalar_other/BetaDistribution.birch"
Real quantile_beta(const Real& P, const Real& \u03b1, const Real& \u03b2);
#line 10 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
#line 10 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
Real simulate_chi_squared(const Real& \u03bd);
#line 25 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
template<class Arg1,class Arg2>
#line 25 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
decltype(auto) logpdf_chi_squared(const Arg1& x, const Arg2& \u03bd);
#line 38 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
#line 38 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
Real cdf_chi_squared(const Real& x, const Real& \u03bd);
#line 51 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
#line 51 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
Real quantile_chi_squared(const Real& P, const Real& \u03bd);
#line 55 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg>
#line 55 "src/distribution/scalar_other/ExponentialDistribution.birch"
decltype(auto) Exponential(const Arg& \u03bb);
#line 64 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 64 "src/distribution/scalar_other/ExponentialDistribution.birch"
Real simulate_exponential(const Real& \u03bb);
#line 79 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 79 "src/distribution/scalar_other/ExponentialDistribution.birch"
decltype(auto) logpdf_exponential(const Arg1& x, const Arg2& \u03bb);
#line 91 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 91 "src/distribution/scalar_other/ExponentialDistribution.birch"
Real cdf_exponential(const Real& x, const Real& \u03bb);
#line 108 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 108 "src/distribution/scalar_other/ExponentialDistribution.birch"
Real quantile_exponential(const Real& P, const Real& \u03bb);
#line 69 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 69 "src/distribution/scalar_other/GammaDistribution.birch"
decltype(auto) Gamma(const Arg1& k, const Arg2& \u03b8);
#line 79 "src/distribution/scalar_other/GammaDistribution.birch"
#line 79 "src/distribution/scalar_other/GammaDistribution.birch"
Real simulate_gamma(const Real& k, const Real& \u03b8);
#line 96 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 96 "src/distribution/scalar_other/GammaDistribution.birch"
decltype(auto) logpdf_gamma(const Arg1& x, const Arg2& k, const Arg3& \u03b8);
#line 110 "src/distribution/scalar_other/GammaDistribution.birch"
#line 110 "src/distribution/scalar_other/GammaDistribution.birch"
Real cdf_gamma(const Real& x, const Real& k, const Real& \u03b8);
#line 130 "src/distribution/scalar_other/GammaDistribution.birch"
#line 130 "src/distribution/scalar_other/GammaDistribution.birch"
Real quantile_gamma(const Real& P, const Real& k, const Real& \u03b8);
#line 60 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 60 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
decltype(auto) GammaExponential(const Arg1& k, const Arg2& \u03b8);
#line 67 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 67 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
Distribution<Real> Exponential(const Random<Real>& \u03bb);
#line 85 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 85 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
Real simulate_lomax(const Real& \u03bb, const Real& \u03b1);
#line 101 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 101 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
decltype(auto) logpdf_lomax(const Arg1& x, const Arg2& \u03bb, const Arg3& \u03b1);
#line 116 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 116 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
decltype(auto) update_gamma_exponential(const Arg1& x, const Arg2& k, const Arg3& \u03b8);
#line 129 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 129 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
Real cdf_lomax(const Real& x, const Real& \u03bb, const Real& \u03b1);
#line 148 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 148 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
Real quantile_lomax(const Real& P, const Real& \u03bb, const Real& \u03b1);
#line 69 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 69 "src/distribution/scalar_other/InverseGammaDistribution.birch"
decltype(auto) InverseGamma(const Arg1& \u03b1, const Arg2& \u03b2);
#line 79 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 79 "src/distribution/scalar_other/InverseGammaDistribution.birch"
Real simulate_inverse_gamma(const Real& \u03b1, const Real& \u03b2);
#line 92 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 92 "src/distribution/scalar_other/InverseGammaDistribution.birch"
decltype(auto) logpdf_inverse_gamma(const Arg1& x, const Arg2& \u03b1, const Arg3& \u03b2);
#line 106 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 106 "src/distribution/scalar_other/InverseGammaDistribution.birch"
Real cdf_inverse_gamma(const Real& x, const Real& \u03b1, const Real& \u03b2);
#line 126 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 126 "src/distribution/scalar_other/InverseGammaDistribution.birch"
Real quantile_inverse_gamma(const Real& P, const Real& \u03b1, const Real& \u03b2);
#line 58 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 58 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
decltype(auto) InverseGammaGamma(const Arg1& k, const Arg2& \u03b1, const Arg3& \u03b2);
#line 65 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg>
#line 65 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
Distribution<Real> Gamma(const Arg& k, const Random<Real>& \u03b8);
#line 84 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 84 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
Real simulate_inverse_gamma_gamma(const Real& k, const Real& \u03b1, const Real& \u03b2);
#line 98 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 98 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
decltype(auto) logpdf_inverse_gamma_gamma(const Arg1& x, const Arg2& k, const Arg3& \u03b1, const Arg4& \u03b2);
#line 114 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 114 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
decltype(auto) update_inverse_gamma_gamma(const Arg1& x, const Arg2& k, const Arg3& \u03b1, const Arg4& \u03b2);
#line 129 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 129 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
Real cdf_inverse_gamma_gamma(const Real& x, const Real& k, const Real& \u03b1, const Real& \u03b2);
#line 62 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 62 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
decltype(auto) ScaledGammaExponential(const Arg1& a, const Arg2& k, const Arg3& \u03b8);
#line 69 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg>
#line 69 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
Distribution<Real> Exponential(const Multiply<Arg, Random<Real>>& \u03bb);
#line 85 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg>
#line 85 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
Distribution<Real> Exponential(const Multiply<Random<Real>, Arg>& \u03bb);
#line 101 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 101 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
Distribution<Real> Exponential(const Multiply<Random<Real>, Random<Real>>& \u03bb);
#line 133 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 133 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
decltype(auto) update_scaled_gamma_exponential(const Arg1& x, const Arg2& a, const Arg3& k, const Arg4& \u03b8);
#line 64 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 64 "src/distribution/scalar_other/StudentDistribution.birch"
decltype(auto) Student(const Arg1& \u03bd, const Arg2& \u03bc, const Arg3& \u03c32);
#line 71 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg>
#line 71 "src/distribution/scalar_other/StudentDistribution.birch"
decltype(auto) Student(const Arg& \u03bd);
#line 80 "src/distribution/scalar_other/StudentDistribution.birch"
#line 80 "src/distribution/scalar_other/StudentDistribution.birch"
Real simulate_student_t(const Real& k);
#line 94 "src/distribution/scalar_other/StudentDistribution.birch"
#line 94 "src/distribution/scalar_other/StudentDistribution.birch"
Real simulate_student_t(const Real& k, const Real& \u03bc, const Real& v);
#line 106 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2>
#line 106 "src/distribution/scalar_other/StudentDistribution.birch"
decltype(auto) logpdf_student_t(const Arg1& x, const Arg2& k);
#line 122 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 122 "src/distribution/scalar_other/StudentDistribution.birch"
decltype(auto) logpdf_student_t(const Arg1& x, const Arg2& k, const Arg3& \u03bc, const Arg4& v);
#line 138 "src/distribution/scalar_other/StudentDistribution.birch"
#line 138 "src/distribution/scalar_other/StudentDistribution.birch"
Real cdf_student_t(const Real& x, const Real& \u03bd);
#line 155 "src/distribution/scalar_other/StudentDistribution.birch"
#line 155 "src/distribution/scalar_other/StudentDistribution.birch"
Real cdf_student_t(const Real& x, const Real& k, const Real& \u03bc, const Real& \u03c32);
#line 168 "src/distribution/scalar_other/StudentDistribution.birch"
#line 168 "src/distribution/scalar_other/StudentDistribution.birch"
Real quantile_student_t(const Real& P, const Real& \u03bd);
#line 185 "src/distribution/scalar_other/StudentDistribution.birch"
#line 185 "src/distribution/scalar_other/StudentDistribution.birch"
Real quantile_student_t(const Real& P, const Real& \u03bd, const Real& \u03bc, const Real& \u03c32);
#line 57 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 57 "src/distribution/scalar_other/UniformDistribution.birch"
decltype(auto) Uniform(const Arg1& l, const Arg2& u);
#line 67 "src/distribution/scalar_other/UniformDistribution.birch"
#line 67 "src/distribution/scalar_other/UniformDistribution.birch"
Real simulate_uniform(const Real& l, const Real& u);
#line 83 "src/distribution/scalar_other/UniformDistribution.birch"
#line 83 "src/distribution/scalar_other/UniformDistribution.birch"
Real logpdf_uniform(const Real& x, const Real& l, const Real& u);
#line 101 "src/distribution/scalar_other/UniformDistribution.birch"
#line 101 "src/distribution/scalar_other/UniformDistribution.birch"
Real cdf_uniform(const Real& x, const Real& l, const Real& u);
#line 121 "src/distribution/scalar_other/UniformDistribution.birch"
#line 121 "src/distribution/scalar_other/UniformDistribution.birch"
Real quantile_uniform(const Real& P, const Real& l, const Real& u);
#line 61 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 61 "src/distribution/scalar_other/WeibullDistribution.birch"
decltype(auto) Weibull(const Arg1& k, const Arg2& \u03bb);
#line 71 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 71 "src/distribution/scalar_other/WeibullDistribution.birch"
Real simulate_weibull(const Real& k, const Real& \u03bb);
#line 88 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 88 "src/distribution/scalar_other/WeibullDistribution.birch"
decltype(auto) logpdf_weibull(const Arg1& x, const Arg2& k, const Arg3& \u03bb);
#line 102 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 102 "src/distribution/scalar_other/WeibullDistribution.birch"
Real cdf_weibull(const Real& x, const Real& k, const Real& \u03bb);
#line 121 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 121 "src/distribution/scalar_other/WeibullDistribution.birch"
Real quantile_weibull(const Real& P, const Real& k, const Real& \u03bb);
#line 19 "src/form/binary_matrix/CholSolve.birch"
template<class Left,class Right>
#line 19 "src/form/binary_matrix/CholSolve.birch"
decltype(auto) cholsolve(const Left& l, const Right& r);
#line 27 "src/form/binary_matrix/CholSolve.birch"
#line 27 "src/form/binary_matrix/CholSolve.birch"
libbirch::DefaultArray<Real,1> cholsolve(const libbirch::DefaultArray<Real,2>& l, const libbirch::DefaultArray<Real,1>& r);
#line 37 "src/form/binary_matrix/CholSolve.birch"
#line 37 "src/form/binary_matrix/CholSolve.birch"
libbirch::DefaultArray<Real,2> cholsolve(const libbirch::DefaultArray<Real,2>& l, const libbirch::DefaultArray<Real,2>& r);
#line 19 "src/form/binary_matrix/Diagonal.birch"
template<class Left,class Right>
#line 19 "src/form/binary_matrix/Diagonal.birch"
decltype(auto) diagonal(const Left& l, const Right& r);
#line 29 "src/form/binary_matrix/Diagonal.birch"
#line 29 "src/form/binary_matrix/Diagonal.birch"
libbirch::DefaultArray<Boolean,2> diagonal(const Boolean& x, const Integer& length);
#line 41 "src/form/binary_matrix/Diagonal.birch"
#line 41 "src/form/binary_matrix/Diagonal.birch"
libbirch::DefaultArray<Integer,2> diagonal(const Integer& x, const Integer& length);
#line 53 "src/form/binary_matrix/Diagonal.birch"
#line 53 "src/form/binary_matrix/Diagonal.birch"
libbirch::DefaultArray<Real,2> diagonal(const Real& x, const Integer& length);
#line 16 "src/form/binary_matrix/Dot.birch"
template<class Left,class Right>
#line 16 "src/form/binary_matrix/Dot.birch"
decltype(auto) dot(const Left& l, const Right& r);
#line 23 "src/form/binary_matrix/Dot.birch"
template<class Middle>
#line 23 "src/form/binary_matrix/Dot.birch"
decltype(auto) dot(const Middle& m);
#line 30 "src/form/binary_matrix/Dot.birch"
#line 30 "src/form/binary_matrix/Dot.birch"
Integer dot(const libbirch::DefaultArray<Integer,1>& x);
#line 39 "src/form/binary_matrix/Dot.birch"
#line 39 "src/form/binary_matrix/Dot.birch"
Real dot(const libbirch::DefaultArray<Real,1>& x);
#line 48 "src/form/binary_matrix/Dot.birch"
#line 48 "src/form/binary_matrix/Dot.birch"
Integer dot(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 57 "src/form/binary_matrix/Dot.birch"
#line 57 "src/form/binary_matrix/Dot.birch"
Real dot(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 66 "src/form/binary_matrix/Dot.birch"
#line 66 "src/form/binary_matrix/Dot.birch"
Real dot(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 75 "src/form/binary_matrix/Dot.birch"
#line 75 "src/form/binary_matrix/Dot.birch"
Real dot(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 85 "src/form/binary_matrix/Dot.birch"
#line 85 "src/form/binary_matrix/Dot.birch"
libbirch::DefaultArray<Integer,1> dot(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Integer,2>& Y);
#line 95 "src/form/binary_matrix/Dot.birch"
#line 95 "src/form/binary_matrix/Dot.birch"
libbirch::DefaultArray<Real,1> dot(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Real,2>& Y);
#line 105 "src/form/binary_matrix/Dot.birch"
#line 105 "src/form/binary_matrix/Dot.birch"
libbirch::DefaultArray<Real,1> dot(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Integer,2>& Y);
#line 115 "src/form/binary_matrix/Dot.birch"
#line 115 "src/form/binary_matrix/Dot.birch"
libbirch::DefaultArray<Real,1> dot(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Real,2>& Y);
#line 16 "src/form/binary_matrix/Inner.birch"
template<class Left,class Right>
#line 16 "src/form/binary_matrix/Inner.birch"
decltype(auto) inner(const Left& l, const Right& r);
#line 23 "src/form/binary_matrix/Inner.birch"
template<class Middle>
#line 23 "src/form/binary_matrix/Inner.birch"
decltype(auto) inner(const Middle& m);
#line 30 "src/form/binary_matrix/Inner.birch"
#line 30 "src/form/binary_matrix/Inner.birch"
Integer inner(const Integer& x);
#line 37 "src/form/binary_matrix/Inner.birch"
#line 37 "src/form/binary_matrix/Inner.birch"
Real inner(const Real& x);
#line 44 "src/form/binary_matrix/Inner.birch"
#line 44 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Integer,1> inner(const libbirch::DefaultArray<Integer,1>& x);
#line 51 "src/form/binary_matrix/Inner.birch"
#line 51 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,1> inner(const libbirch::DefaultArray<Real,1>& x);
#line 58 "src/form/binary_matrix/Inner.birch"
#line 58 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Integer,2> inner(const libbirch::DefaultArray<Integer,2>& X);
#line 65 "src/form/binary_matrix/Inner.birch"
#line 65 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Real,2>& X);
#line 72 "src/form/binary_matrix/Inner.birch"
#line 72 "src/form/binary_matrix/Inner.birch"
Integer inner(const Integer& x, const Integer& y);
#line 79 "src/form/binary_matrix/Inner.birch"
#line 79 "src/form/binary_matrix/Inner.birch"
Real inner(const Integer& x, const Real& y);
#line 86 "src/form/binary_matrix/Inner.birch"
#line 86 "src/form/binary_matrix/Inner.birch"
Real inner(const Real& x, const Integer& y);
#line 93 "src/form/binary_matrix/Inner.birch"
#line 93 "src/form/binary_matrix/Inner.birch"
Real inner(const Real& x, const Real& y);
#line 100 "src/form/binary_matrix/Inner.birch"
#line 100 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Integer,1> inner(const Integer& x, const libbirch::DefaultArray<Integer,1>& y);
#line 109 "src/form/binary_matrix/Inner.birch"
#line 109 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,1> inner(const Integer& x, const libbirch::DefaultArray<Real,1>& y);
#line 118 "src/form/binary_matrix/Inner.birch"
#line 118 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,1> inner(const Real& x, const libbirch::DefaultArray<Integer,1>& y);
#line 127 "src/form/binary_matrix/Inner.birch"
#line 127 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,1> inner(const Real& x, const libbirch::DefaultArray<Real,1>& y);
#line 136 "src/form/binary_matrix/Inner.birch"
#line 136 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Integer,2> inner(const libbirch::DefaultArray<Integer,1>& x, const Integer& y);
#line 145 "src/form/binary_matrix/Inner.birch"
#line 145 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Integer,1>& x, const Real& y);
#line 154 "src/form/binary_matrix/Inner.birch"
#line 154 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Real,1>& x, const Integer& y);
#line 163 "src/form/binary_matrix/Inner.birch"
#line 163 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Real,1>& x, const Real& y);
#line 172 "src/form/binary_matrix/Inner.birch"
#line 172 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Integer,1> inner(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 181 "src/form/binary_matrix/Inner.birch"
#line 181 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,1> inner(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 190 "src/form/binary_matrix/Inner.birch"
#line 190 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,1> inner(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 199 "src/form/binary_matrix/Inner.birch"
#line 199 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,1> inner(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 208 "src/form/binary_matrix/Inner.birch"
#line 208 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Integer,2> inner(const Integer& x, const libbirch::DefaultArray<Integer,2>& y);
#line 217 "src/form/binary_matrix/Inner.birch"
#line 217 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const Integer& x, const libbirch::DefaultArray<Real,2>& y);
#line 226 "src/form/binary_matrix/Inner.birch"
#line 226 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const Real& x, const libbirch::DefaultArray<Integer,2>& y);
#line 235 "src/form/binary_matrix/Inner.birch"
#line 235 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const Real& x, const libbirch::DefaultArray<Real,2>& y);
#line 244 "src/form/binary_matrix/Inner.birch"
#line 244 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Integer,2> inner(const libbirch::DefaultArray<Integer,2>& x, const Integer& y);
#line 253 "src/form/binary_matrix/Inner.birch"
#line 253 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Integer,2>& x, const Real& y);
#line 262 "src/form/binary_matrix/Inner.birch"
#line 262 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Real,2>& x, const Integer& y);
#line 271 "src/form/binary_matrix/Inner.birch"
#line 271 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Real,2>& x, const Real& y);
#line 280 "src/form/binary_matrix/Inner.birch"
#line 280 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Integer,1> inner(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Integer,1>& y);
#line 289 "src/form/binary_matrix/Inner.birch"
#line 289 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,1> inner(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Real,1>& y);
#line 298 "src/form/binary_matrix/Inner.birch"
#line 298 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,1> inner(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Integer,1>& y);
#line 307 "src/form/binary_matrix/Inner.birch"
#line 307 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,1> inner(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Real,1>& y);
#line 316 "src/form/binary_matrix/Inner.birch"
#line 316 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Integer,2> inner(const libbirch::DefaultArray<Integer,1>& X, const libbirch::DefaultArray<Integer,2>& y);
#line 325 "src/form/binary_matrix/Inner.birch"
#line 325 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Integer,1>& X, const libbirch::DefaultArray<Real,2>& y);
#line 334 "src/form/binary_matrix/Inner.birch"
#line 334 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Real,1>& X, const libbirch::DefaultArray<Integer,2>& y);
#line 343 "src/form/binary_matrix/Inner.birch"
#line 343 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Real,1>& X, const libbirch::DefaultArray<Real,2>& y);
#line 352 "src/form/binary_matrix/Inner.birch"
#line 352 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Integer,2> inner(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Integer,2>& Y);
#line 361 "src/form/binary_matrix/Inner.birch"
#line 361 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Real,2>& Y);
#line 370 "src/form/binary_matrix/Inner.birch"
#line 370 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Integer,2>& Y);
#line 379 "src/form/binary_matrix/Inner.birch"
#line 379 "src/form/binary_matrix/Inner.birch"
libbirch::DefaultArray<Real,2> inner(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Real,2>& Y);
#line 30 "src/form/binary_matrix/MatrixPack.birch"
template<class Left,class Right>
#line 30 "src/form/binary_matrix/MatrixPack.birch"
decltype(auto) pack(const Left& l, const Right& r);
#line 37 "src/form/binary_matrix/MatrixPack.birch"
#line 37 "src/form/binary_matrix/MatrixPack.birch"
libbirch::DefaultArray<Real,2> pack(const Real& x, const Real& y);
#line 44 "src/form/binary_matrix/MatrixPack.birch"
#line 44 "src/form/binary_matrix/MatrixPack.birch"
libbirch::DefaultArray<Integer,2> pack(const Integer& x, const Integer& y);
#line 51 "src/form/binary_matrix/MatrixPack.birch"
#line 51 "src/form/binary_matrix/MatrixPack.birch"
libbirch::DefaultArray<Boolean,2> pack(const Boolean& x, const Boolean& y);
#line 58 "src/form/binary_matrix/MatrixPack.birch"
template<class Type>
#line 58 "src/form/binary_matrix/MatrixPack.birch"
libbirch::DefaultArray<Type,2> pack(const libbirch::DefaultArray<Type,1>& x, const libbirch::DefaultArray<Type,1>& y);
#line 73 "src/form/binary_matrix/MatrixPack.birch"
template<class Type>
#line 73 "src/form/binary_matrix/MatrixPack.birch"
libbirch::DefaultArray<Type,2> pack(const libbirch::DefaultArray<Type,2>& l, const libbirch::DefaultArray<Type,2>& r);
#line 40 "src/form/binary_matrix/MatrixStack.birch"
template<class Left,class Right>
#line 40 "src/form/binary_matrix/MatrixStack.birch"
decltype(auto) stack(const Left& l, const Right& r);
#line 48 "src/form/binary_matrix/MatrixStack.birch"
#line 48 "src/form/binary_matrix/MatrixStack.birch"
libbirch::DefaultArray<Real,1> stack(const Real& x, const Real& y);
#line 56 "src/form/binary_matrix/MatrixStack.birch"
#line 56 "src/form/binary_matrix/MatrixStack.birch"
libbirch::DefaultArray<Integer,1> stack(const Integer& x, const Integer& y);
#line 64 "src/form/binary_matrix/MatrixStack.birch"
#line 64 "src/form/binary_matrix/MatrixStack.birch"
libbirch::DefaultArray<Boolean,1> stack(const Boolean& x, const Boolean& y);
#line 72 "src/form/binary_matrix/MatrixStack.birch"
template<class Type>
#line 72 "src/form/binary_matrix/MatrixStack.birch"
libbirch::DefaultArray<Type,1> stack(const libbirch::DefaultArray<Type,1>& x, const libbirch::DefaultArray<Type,1>& y);
#line 88 "src/form/binary_matrix/MatrixStack.birch"
template<class Type>
#line 88 "src/form/binary_matrix/MatrixStack.birch"
libbirch::DefaultArray<Type,1> stack(const libbirch::DefaultArray<Type,1>& x, const Type& y);
#line 103 "src/form/binary_matrix/MatrixStack.birch"
template<class Type>
#line 103 "src/form/binary_matrix/MatrixStack.birch"
libbirch::DefaultArray<Type,1> stack(const Type& x, const libbirch::DefaultArray<Type,1>& y);
#line 118 "src/form/binary_matrix/MatrixStack.birch"
template<class Type>
#line 118 "src/form/binary_matrix/MatrixStack.birch"
libbirch::DefaultArray<Type,2> stack(const libbirch::DefaultArray<Type,2>& X, const libbirch::DefaultArray<Type,2>& Y);
#line 16 "src/form/binary_matrix/Outer.birch"
template<class Left,class Right>
#line 16 "src/form/binary_matrix/Outer.birch"
decltype(auto) outer(const Left& l, const Right& r);
#line 23 "src/form/binary_matrix/Outer.birch"
template<class Middle>
#line 23 "src/form/binary_matrix/Outer.birch"
decltype(auto) outer(const Middle& m);
#line 30 "src/form/binary_matrix/Outer.birch"
#line 30 "src/form/binary_matrix/Outer.birch"
Integer outer(const Integer& x);
#line 37 "src/form/binary_matrix/Outer.birch"
#line 37 "src/form/binary_matrix/Outer.birch"
Real outer(const Real& x);
#line 44 "src/form/binary_matrix/Outer.birch"
#line 44 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Integer,2> outer(const libbirch::DefaultArray<Integer,1>& x);
#line 51 "src/form/binary_matrix/Outer.birch"
#line 51 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,1>& x);
#line 58 "src/form/binary_matrix/Outer.birch"
#line 58 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Integer,2> outer(const libbirch::DefaultArray<Integer,2>& X);
#line 65 "src/form/binary_matrix/Outer.birch"
#line 65 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,2>& X);
#line 72 "src/form/binary_matrix/Outer.birch"
#line 72 "src/form/binary_matrix/Outer.birch"
Integer outer(const Integer& x, const Integer& y);
#line 79 "src/form/binary_matrix/Outer.birch"
#line 79 "src/form/binary_matrix/Outer.birch"
Real outer(const Integer& x, const Real& y);
#line 86 "src/form/binary_matrix/Outer.birch"
#line 86 "src/form/binary_matrix/Outer.birch"
Real outer(const Real& x, const Integer& y);
#line 93 "src/form/binary_matrix/Outer.birch"
#line 93 "src/form/binary_matrix/Outer.birch"
Real outer(const Real& x, const Real& y);
#line 100 "src/form/binary_matrix/Outer.birch"
#line 100 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Integer,2> outer(const Integer& x, const libbirch::DefaultArray<Integer,1>& y);
#line 109 "src/form/binary_matrix/Outer.birch"
#line 109 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const Integer& x, const libbirch::DefaultArray<Real,1>& y);
#line 118 "src/form/binary_matrix/Outer.birch"
#line 118 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const Real& x, const libbirch::DefaultArray<Integer,1>& y);
#line 127 "src/form/binary_matrix/Outer.birch"
#line 127 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const Real& x, const libbirch::DefaultArray<Real,1>& y);
#line 136 "src/form/binary_matrix/Outer.birch"
#line 136 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Integer,1> outer(const libbirch::DefaultArray<Integer,1>& x, const Integer& y);
#line 145 "src/form/binary_matrix/Outer.birch"
#line 145 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,1> outer(const libbirch::DefaultArray<Integer,1>& x, const Real& y);
#line 154 "src/form/binary_matrix/Outer.birch"
#line 154 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,1> outer(const libbirch::DefaultArray<Real,1>& x, const Integer& y);
#line 163 "src/form/binary_matrix/Outer.birch"
#line 163 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,1> outer(const libbirch::DefaultArray<Real,1>& x, const Real& y);
#line 172 "src/form/binary_matrix/Outer.birch"
#line 172 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Integer,2> outer(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 181 "src/form/binary_matrix/Outer.birch"
#line 181 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 190 "src/form/binary_matrix/Outer.birch"
#line 190 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 199 "src/form/binary_matrix/Outer.birch"
#line 199 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 208 "src/form/binary_matrix/Outer.birch"
#line 208 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Integer,2> outer(const Integer& x, const libbirch::DefaultArray<Integer,2>& y);
#line 217 "src/form/binary_matrix/Outer.birch"
#line 217 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const Integer& x, const libbirch::DefaultArray<Real,2>& y);
#line 226 "src/form/binary_matrix/Outer.birch"
#line 226 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const Real& x, const libbirch::DefaultArray<Integer,2>& y);
#line 235 "src/form/binary_matrix/Outer.birch"
#line 235 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const Real& x, const libbirch::DefaultArray<Real,2>& y);
#line 244 "src/form/binary_matrix/Outer.birch"
#line 244 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Integer,2> outer(const libbirch::DefaultArray<Integer,2>& x, const Integer& y);
#line 253 "src/form/binary_matrix/Outer.birch"
#line 253 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Integer,2>& x, const Real& y);
#line 262 "src/form/binary_matrix/Outer.birch"
#line 262 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,2>& x, const Integer& y);
#line 271 "src/form/binary_matrix/Outer.birch"
#line 271 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,2>& x, const Real& y);
#line 280 "src/form/binary_matrix/Outer.birch"
#line 280 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Integer,2> outer(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Integer,1>& y);
#line 289 "src/form/binary_matrix/Outer.birch"
#line 289 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Real,1>& y);
#line 298 "src/form/binary_matrix/Outer.birch"
#line 298 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Integer,1>& y);
#line 307 "src/form/binary_matrix/Outer.birch"
#line 307 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Real,1>& y);
#line 316 "src/form/binary_matrix/Outer.birch"
#line 316 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Integer,2> outer(const libbirch::DefaultArray<Integer,1>& X, const libbirch::DefaultArray<Integer,2>& y);
#line 325 "src/form/binary_matrix/Outer.birch"
#line 325 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Integer,1>& X, const libbirch::DefaultArray<Real,2>& y);
#line 334 "src/form/binary_matrix/Outer.birch"
#line 334 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,1>& X, const libbirch::DefaultArray<Integer,2>& y);
#line 343 "src/form/binary_matrix/Outer.birch"
#line 343 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,1>& X, const libbirch::DefaultArray<Real,2>& y);
#line 352 "src/form/binary_matrix/Outer.birch"
#line 352 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Integer,2> outer(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Integer,2>& Y);
#line 361 "src/form/binary_matrix/Outer.birch"
#line 361 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Real,2>& Y);
#line 370 "src/form/binary_matrix/Outer.birch"
#line 370 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Integer,2>& Y);
#line 379 "src/form/binary_matrix/Outer.birch"
#line 379 "src/form/binary_matrix/Outer.birch"
libbirch::DefaultArray<Real,2> outer(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Real,2>& Y);
#line 18 "src/form/binary_matrix/Solve.birch"
template<class Left,class Right>
#line 18 "src/form/binary_matrix/Solve.birch"
decltype(auto) solve(const Left& l, const Right& r);
#line 25 "src/form/binary_matrix/Solve.birch"
#line 25 "src/form/binary_matrix/Solve.birch"
libbirch::DefaultArray<Real,1> solve(const libbirch::DefaultArray<Real,2>& l, const libbirch::DefaultArray<Real,1>& r);
#line 34 "src/form/binary_matrix/Solve.birch"
#line 34 "src/form/binary_matrix/Solve.birch"
libbirch::DefaultArray<Real,2> solve(const libbirch::DefaultArray<Real,2>& l, const libbirch::DefaultArray<Real,2>& r);
#line 16 "src/form/binary_other/CopySign.birch"
template<class Left,class Right>
#line 16 "src/form/binary_other/CopySign.birch"
decltype(auto) copysign(const Left& l, const Right& r);
#line 23 "src/form/binary_other/CopySign.birch"
#line 23 "src/form/binary_other/CopySign.birch"
Integer copysign(const Integer& l, const Integer& r);
#line 30 "src/form/binary_other/CopySign.birch"
#line 30 "src/form/binary_other/CopySign.birch"
Real copysign(const Real& l, const Real& r);
#line 39 "src/form/binary_other/CopySign.birch"
#line 39 "src/form/binary_other/CopySign.birch"
Real copysign(const Integer& l, const Real& r);
#line 46 "src/form/binary_other/CopySign.birch"
#line 46 "src/form/binary_other/CopySign.birch"
Real copysign(const Real& l, const Integer& r);
#line 16 "src/form/binary_other/LogBeta.birch"
template<class Left,class Right>
#line 16 "src/form/binary_other/LogBeta.birch"
decltype(auto) lbeta(const Left& l, const Right& r);
#line 23 "src/form/binary_other/LogBeta.birch"
#line 23 "src/form/binary_other/LogBeta.birch"
Real lbeta(const Integer& l, const Integer& r);
#line 30 "src/form/binary_other/LogBeta.birch"
#line 30 "src/form/binary_other/LogBeta.birch"
Real lbeta(const Real& l, const Real& r);
#line 37 "src/form/binary_other/LogBeta.birch"
#line 37 "src/form/binary_other/LogBeta.birch"
Real lbeta(const Integer& l, const Real& r);
#line 44 "src/form/binary_other/LogBeta.birch"
#line 44 "src/form/binary_other/LogBeta.birch"
Real lbeta(const Real& l, const Integer& r);
#line 16 "src/form/binary_other/LogChoose.birch"
template<class Left,class Right>
#line 16 "src/form/binary_other/LogChoose.birch"
decltype(auto) lchoose(const Left& l, const Right& r);
#line 23 "src/form/binary_other/LogChoose.birch"
#line 23 "src/form/binary_other/LogChoose.birch"
Real lchoose(const Integer& x, const Integer& y);
#line 30 "src/form/binary_other/LogChoose.birch"
#line 30 "src/form/binary_other/LogChoose.birch"
Real lchoose(const Real& x, const Real& y);
#line 42 "src/form/binary_other/LogChoose.birch"
#line 42 "src/form/binary_other/LogChoose.birch"
Real lchoose(const Integer& x, const Real& y);
#line 49 "src/form/binary_other/LogChoose.birch"
#line 49 "src/form/binary_other/LogChoose.birch"
Real lchoose(const Real& x, const Integer& y);
#line 20 "src/form/binary_other/LogGammaP.birch"
template<class Left,class Right>
#line 20 "src/form/binary_other/LogGammaP.birch"
decltype(auto) lgamma(const Left& l, const Right& r);
#line 27 "src/form/binary_other/LogGammaP.birch"
#line 27 "src/form/binary_other/LogGammaP.birch"
Real lgamma(const Integer& x, const Integer& p);
#line 34 "src/form/binary_other/LogGammaP.birch"
#line 34 "src/form/binary_other/LogGammaP.birch"
Real lgamma(const Real& x, const Integer& p);
#line 21 "src/form/binary_other/Pow.birch"
template<class Left,class Right>
#line 21 "src/form/binary_other/Pow.birch"
decltype(auto) pow(const Left& l, const Right& r);
#line 28 "src/form/binary_other/Pow.birch"
#line 28 "src/form/binary_other/Pow.birch"
Integer pow(const Integer& x, const Integer& y);
#line 37 "src/form/binary_other/Pow.birch"
#line 37 "src/form/binary_other/Pow.birch"
Real pow(const Real& x, const Real& y);
#line 46 "src/form/binary_other/Pow.birch"
#line 46 "src/form/binary_other/Pow.birch"
Real pow(const Integer& x, const Real& y);
#line 53 "src/form/binary_other/Pow.birch"
#line 53 "src/form/binary_other/Pow.birch"
Real pow(const Real& x, const Integer& y);
#line 16 "src/form/ternary/IfThenElse.birch"
template<class Left,class Middle,class Right>
#line 16 "src/form/ternary/IfThenElse.birch"
decltype(auto) if_then_else(const Left& l, const Middle& m, const Right& r);
#line 23 "src/form/ternary/IfThenElse.birch"
template<class Type>
#line 23 "src/form/ternary/IfThenElse.birch"
Type if_then_else(const Boolean& cond, const Type& x, const Type& y);
#line 29 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
template<class Arg1,class Arg2,class Arg3>
#line 29 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
decltype(auto) logz_conway_maxwell_poisson(const Arg1& \u03bb, const Arg2& \u03bd, const Arg3& n);
#line 43 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
#line 43 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
Real logz_conway_maxwell_poisson(const Real& \u03bb, const Real& \u03bd, const Integer& n);
#line 17 "src/form/unary_matrix/CholInv.birch"
template<class Middle>
#line 17 "src/form/unary_matrix/CholInv.birch"
decltype(auto) cholinv(const Middle& m);
#line 25 "src/form/unary_matrix/CholInv.birch"
#line 25 "src/form/unary_matrix/CholInv.birch"
libbirch::DefaultArray<Real,2> cholinv(const libbirch::DefaultArray<Real,2>& S);
#line 16 "src/form/unary_matrix/Inv.birch"
template<class Middle>
#line 16 "src/form/unary_matrix/Inv.birch"
decltype(auto) inv(const Middle& m);
#line 23 "src/form/unary_matrix/Inv.birch"
#line 23 "src/form/unary_matrix/Inv.birch"
libbirch::DefaultArray<Real,2> inv(const libbirch::DefaultArray<Real,2>& X);
#line 16 "src/form/unary_matrix/LCholDet.birch"
template<class Middle>
#line 16 "src/form/unary_matrix/LCholDet.birch"
decltype(auto) lcholdet(const Middle& m);
#line 23 "src/form/unary_matrix/LCholDet.birch"
#line 23 "src/form/unary_matrix/LCholDet.birch"
Real lcholdet(const libbirch::DefaultArray<Real,2>& S);
#line 15 "src/form/unary_matrix/LDet.birch"
template<class Middle>
#line 15 "src/form/unary_matrix/LDet.birch"
decltype(auto) ldet(const Middle& m);
#line 22 "src/form/unary_matrix/LDet.birch"
#line 22 "src/form/unary_matrix/LDet.birch"
Real ldet(const libbirch::DefaultArray<Real,2>& X);
#line 15 "src/form/unary_matrix/Matrix.birch"
template<class Value,class Middle>
#line 15 "src/form/unary_matrix/Matrix.birch"
decltype(auto) matrix(const Middle& m);
#line 22 "src/form/unary_matrix/Matrix.birch"
template<class Value,class Type>
#line 22 "src/form/unary_matrix/Matrix.birch"
libbirch::DefaultArray<Value,2> matrix(const libbirch::DefaultArray<Type,1>& m);
#line 29 "src/form/unary_matrix/Matrix.birch"
template<class Value,class Type>
#line 29 "src/form/unary_matrix/Matrix.birch"
libbirch::DefaultArray<Value,2> matrix(const libbirch::DefaultArray<Type,2>& m);
#line 36 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 36 "src/form/unary_matrix/Matrix.birch"
libbirch::DefaultArray<Value,2> matrix(const Boolean& m);
#line 43 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 43 "src/form/unary_matrix/Matrix.birch"
libbirch::DefaultArray<Value,2> matrix(const Integer& m);
#line 50 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 50 "src/form/unary_matrix/Matrix.birch"
libbirch::DefaultArray<Value,2> matrix(const Real& m);
#line 57 "src/form/unary_matrix/Matrix.birch"
template<class Gradient,class Value>
#line 57 "src/form/unary_matrix/Matrix.birch"
decltype(auto) grad_matrix(const libbirch::DefaultArray<Gradient,2>& d, const Value& m);
#line 64 "src/form/unary_matrix/Matrix.birch"
template<class Gradient,class Value>
#line 64 "src/form/unary_matrix/Matrix.birch"
decltype(auto) grad_matrix(const libbirch::DefaultArray<Gradient,2>& d, const libbirch::DefaultArray<Value,1>& m);
#line 71 "src/form/unary_matrix/Matrix.birch"
template<class Gradient,class Value>
#line 71 "src/form/unary_matrix/Matrix.birch"
decltype(auto) grad_matrix(const libbirch::DefaultArray<Gradient,2>& d, const libbirch::DefaultArray<Value,2>& m);
#line 23 "src/form/unary_matrix/Trace.birch"
template<class Middle>
#line 23 "src/form/unary_matrix/Trace.birch"
decltype(auto) trace(const Middle& m);
#line 30 "src/form/unary_matrix/Trace.birch"
#line 30 "src/form/unary_matrix/Trace.birch"
Integer trace(const Integer& m);
#line 37 "src/form/unary_matrix/Trace.birch"
#line 37 "src/form/unary_matrix/Trace.birch"
Real trace(const Real& m);
#line 44 "src/form/unary_matrix/Trace.birch"
#line 44 "src/form/unary_matrix/Trace.birch"
Integer trace(const libbirch::DefaultArray<Integer,1>& m);
#line 53 "src/form/unary_matrix/Trace.birch"
#line 53 "src/form/unary_matrix/Trace.birch"
Real trace(const libbirch::DefaultArray<Real,1>& m);
#line 62 "src/form/unary_matrix/Trace.birch"
#line 62 "src/form/unary_matrix/Trace.birch"
Integer trace(const libbirch::DefaultArray<Integer,2>& m);
#line 71 "src/form/unary_matrix/Trace.birch"
#line 71 "src/form/unary_matrix/Trace.birch"
Real trace(const libbirch::DefaultArray<Real,2>& m);
#line 15 "src/form/unary_matrix/Transpose.birch"
template<class Middle>
#line 15 "src/form/unary_matrix/Transpose.birch"
decltype(auto) transpose(const Middle& m);
#line 22 "src/form/unary_matrix/Transpose.birch"
#line 22 "src/form/unary_matrix/Transpose.birch"
Boolean transpose(const Boolean& m);
#line 29 "src/form/unary_matrix/Transpose.birch"
#line 29 "src/form/unary_matrix/Transpose.birch"
Integer transpose(const Integer& m);
#line 36 "src/form/unary_matrix/Transpose.birch"
#line 36 "src/form/unary_matrix/Transpose.birch"
Real transpose(const Real& m);
#line 43 "src/form/unary_matrix/Transpose.birch"
#line 43 "src/form/unary_matrix/Transpose.birch"
libbirch::DefaultArray<Boolean,2> transpose(const libbirch::DefaultArray<Boolean,1>& x);
#line 52 "src/form/unary_matrix/Transpose.birch"
#line 52 "src/form/unary_matrix/Transpose.birch"
libbirch::DefaultArray<Integer,2> transpose(const libbirch::DefaultArray<Integer,1>& x);
#line 61 "src/form/unary_matrix/Transpose.birch"
#line 61 "src/form/unary_matrix/Transpose.birch"
libbirch::DefaultArray<Real,2> transpose(const libbirch::DefaultArray<Real,1>& x);
#line 70 "src/form/unary_matrix/Transpose.birch"
#line 70 "src/form/unary_matrix/Transpose.birch"
libbirch::DefaultArray<Boolean,2> transpose(const libbirch::DefaultArray<Boolean,2>& X);
#line 79 "src/form/unary_matrix/Transpose.birch"
#line 79 "src/form/unary_matrix/Transpose.birch"
libbirch::DefaultArray<Integer,2> transpose(const libbirch::DefaultArray<Integer,2>& X);
#line 88 "src/form/unary_matrix/Transpose.birch"
#line 88 "src/form/unary_matrix/Transpose.birch"
libbirch::DefaultArray<Real,2> transpose(const libbirch::DefaultArray<Real,2>& X);
#line 15 "src/form/unary_matrix/Vector.birch"
template<class Value,class Middle>
#line 15 "src/form/unary_matrix/Vector.birch"
decltype(auto) vector(const Middle& m);
#line 22 "src/form/unary_matrix/Vector.birch"
template<class Value,class Type>
#line 22 "src/form/unary_matrix/Vector.birch"
libbirch::DefaultArray<Value,1> vector(const libbirch::DefaultArray<Type,1>& m);
#line 29 "src/form/unary_matrix/Vector.birch"
template<class Value,class Type>
#line 29 "src/form/unary_matrix/Vector.birch"
libbirch::DefaultArray<Value,1> vector(const libbirch::DefaultArray<Type,2>& m);
#line 37 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 37 "src/form/unary_matrix/Vector.birch"
libbirch::DefaultArray<Value,1> vector(const Boolean& m);
#line 44 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 44 "src/form/unary_matrix/Vector.birch"
libbirch::DefaultArray<Value,1> vector(const Integer& m);
#line 51 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 51 "src/form/unary_matrix/Vector.birch"
libbirch::DefaultArray<Value,1> vector(const Real& m);
#line 58 "src/form/unary_matrix/Vector.birch"
template<class Gradient,class Value>
#line 58 "src/form/unary_matrix/Vector.birch"
decltype(auto) grad_vector(const libbirch::DefaultArray<Gradient,1>& d, const Value& m);
#line 65 "src/form/unary_matrix/Vector.birch"
template<class Gradient,class Value>
#line 65 "src/form/unary_matrix/Vector.birch"
decltype(auto) grad_vector(const libbirch::DefaultArray<Gradient,1>& d, const libbirch::DefaultArray<Value,1>& m);
#line 72 "src/form/unary_matrix/Vector.birch"
template<class Gradient,class Value>
#line 72 "src/form/unary_matrix/Vector.birch"
decltype(auto) grad_vector(const libbirch::DefaultArray<Gradient,1>& d, const libbirch::DefaultArray<Value,2>& m);
#line 15 "src/form/unary_other/Abs.birch"
template<class Middle>
#line 15 "src/form/unary_other/Abs.birch"
decltype(auto) abs(const Middle& m);
#line 22 "src/form/unary_other/Abs.birch"
#line 22 "src/form/unary_other/Abs.birch"
Integer abs(const Integer& x);
#line 31 "src/form/unary_other/Abs.birch"
#line 31 "src/form/unary_other/Abs.birch"
Real abs(const Real& x);
#line 15 "src/form/unary_other/Acos.birch"
template<class Middle>
#line 15 "src/form/unary_other/Acos.birch"
decltype(auto) acos(const Middle& m);
#line 22 "src/form/unary_other/Acos.birch"
#line 22 "src/form/unary_other/Acos.birch"
Real acos(const Integer& m);
#line 29 "src/form/unary_other/Acos.birch"
#line 29 "src/form/unary_other/Acos.birch"
Real acos(const Real& m);
#line 15 "src/form/unary_other/Asin.birch"
template<class Middle>
#line 15 "src/form/unary_other/Asin.birch"
decltype(auto) asin(const Middle& m);
#line 22 "src/form/unary_other/Asin.birch"
#line 22 "src/form/unary_other/Asin.birch"
Real asin(const Integer& m);
#line 29 "src/form/unary_other/Asin.birch"
#line 29 "src/form/unary_other/Asin.birch"
Real asin(const Real& m);
#line 15 "src/form/unary_other/Atan.birch"
template<class Middle>
#line 15 "src/form/unary_other/Atan.birch"
decltype(auto) atan(const Middle& m);
#line 22 "src/form/unary_other/Atan.birch"
#line 22 "src/form/unary_other/Atan.birch"
Real atan(const Integer& m);
#line 29 "src/form/unary_other/Atan.birch"
#line 29 "src/form/unary_other/Atan.birch"
Real atan(const Real& m);
#line 16 "src/form/unary_other/Cos.birch"
template<class Middle>
#line 16 "src/form/unary_other/Cos.birch"
decltype(auto) cos(const Middle& m);
#line 23 "src/form/unary_other/Cos.birch"
#line 23 "src/form/unary_other/Cos.birch"
Real cos(const Integer& m);
#line 30 "src/form/unary_other/Cos.birch"
#line 30 "src/form/unary_other/Cos.birch"
Real cos(const Real& m);
#line 15 "src/form/unary_other/Cosh.birch"
template<class Middle>
#line 15 "src/form/unary_other/Cosh.birch"
decltype(auto) cosh(const Middle& m);
#line 22 "src/form/unary_other/Cosh.birch"
#line 22 "src/form/unary_other/Cosh.birch"
Real cosh(const Integer& m);
#line 29 "src/form/unary_other/Cosh.birch"
#line 29 "src/form/unary_other/Cosh.birch"
Real cosh(const Real& m);
#line 15 "src/form/unary_other/Exp.birch"
template<class Middle>
#line 15 "src/form/unary_other/Exp.birch"
decltype(auto) exp(const Middle& m);
#line 22 "src/form/unary_other/Exp.birch"
#line 22 "src/form/unary_other/Exp.birch"
Real exp(const Integer& m);
#line 29 "src/form/unary_other/Exp.birch"
#line 29 "src/form/unary_other/Exp.birch"
Real exp(const Real& m);
#line 15 "src/form/unary_other/Expm1.birch"
template<class Middle>
#line 15 "src/form/unary_other/Expm1.birch"
decltype(auto) expm1(const Middle& m);
#line 22 "src/form/unary_other/Expm1.birch"
#line 22 "src/form/unary_other/Expm1.birch"
Real expm1(const Integer& m);
#line 29 "src/form/unary_other/Expm1.birch"
#line 29 "src/form/unary_other/Expm1.birch"
Real expm1(const Real& m);
#line 15 "src/form/unary_other/Log.birch"
template<class Middle>
#line 15 "src/form/unary_other/Log.birch"
decltype(auto) log(const Middle& m);
#line 22 "src/form/unary_other/Log.birch"
#line 22 "src/form/unary_other/Log.birch"
Real log(const Integer& m);
#line 29 "src/form/unary_other/Log.birch"
#line 29 "src/form/unary_other/Log.birch"
Real log(const Real& m);
#line 15 "src/form/unary_other/Log1p.birch"
template<class Middle>
#line 15 "src/form/unary_other/Log1p.birch"
decltype(auto) log1p(const Middle& m);
#line 22 "src/form/unary_other/Log1p.birch"
#line 22 "src/form/unary_other/Log1p.birch"
Real log1p(const Integer& m);
#line 29 "src/form/unary_other/Log1p.birch"
#line 29 "src/form/unary_other/Log1p.birch"
Real log1p(const Real& m);
#line 15 "src/form/unary_other/LogGamma.birch"
template<class Middle>
#line 15 "src/form/unary_other/LogGamma.birch"
decltype(auto) lgamma(const Middle& m);
#line 22 "src/form/unary_other/LogGamma.birch"
#line 22 "src/form/unary_other/LogGamma.birch"
Real lgamma(const Integer& x);
#line 29 "src/form/unary_other/LogGamma.birch"
#line 29 "src/form/unary_other/LogGamma.birch"
Real lgamma(const Real& x);
#line 17 "src/form/unary_other/Rectify.birch"
template<class Middle>
#line 17 "src/form/unary_other/Rectify.birch"
decltype(auto) rectify(const Middle& m);
#line 24 "src/form/unary_other/Rectify.birch"
#line 24 "src/form/unary_other/Rectify.birch"
Integer rectify(const Integer& m);
#line 31 "src/form/unary_other/Rectify.birch"
#line 31 "src/form/unary_other/Rectify.birch"
Real rectify(const Real& m);
#line 38 "src/form/unary_other/Rectify.birch"
#line 38 "src/form/unary_other/Rectify.birch"
libbirch::DefaultArray<Integer,1> rectify(const libbirch::DefaultArray<Integer,1>& m);
#line 45 "src/form/unary_other/Rectify.birch"
#line 45 "src/form/unary_other/Rectify.birch"
libbirch::DefaultArray<Real,1> rectify(const libbirch::DefaultArray<Real,1>& m);
#line 52 "src/form/unary_other/Rectify.birch"
#line 52 "src/form/unary_other/Rectify.birch"
libbirch::DefaultArray<Integer,2> rectify(const libbirch::DefaultArray<Integer,2>& m);
#line 59 "src/form/unary_other/Rectify.birch"
#line 59 "src/form/unary_other/Rectify.birch"
libbirch::DefaultArray<Real,2> rectify(const libbirch::DefaultArray<Real,2>& m);
#line 15 "src/form/unary_other/Scalar.birch"
template<class Value,class Middle>
#line 15 "src/form/unary_other/Scalar.birch"
decltype(auto) scalar(const Middle& m);
#line 22 "src/form/unary_other/Scalar.birch"
template<class Value,class Type>
#line 22 "src/form/unary_other/Scalar.birch"
Value scalar(const libbirch::DefaultArray<Type,1>& m);
#line 30 "src/form/unary_other/Scalar.birch"
template<class Value,class Type>
#line 30 "src/form/unary_other/Scalar.birch"
Value scalar(const libbirch::DefaultArray<Type,2>& m);
#line 38 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 38 "src/form/unary_other/Scalar.birch"
Value scalar(const Boolean& m);
#line 47 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 47 "src/form/unary_other/Scalar.birch"
Value scalar(const Integer& m);
#line 56 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 56 "src/form/unary_other/Scalar.birch"
Value scalar(const Real& m);
#line 65 "src/form/unary_other/Scalar.birch"
template<class Gradient,class Value>
#line 65 "src/form/unary_other/Scalar.birch"
decltype(auto) grad_scalar(const Gradient& d, const Value& m);
#line 72 "src/form/unary_other/Scalar.birch"
template<class Gradient,class Value>
#line 72 "src/form/unary_other/Scalar.birch"
decltype(auto) grad_scalar(const Gradient& d, const libbirch::DefaultArray<Value,1>& m);
#line 79 "src/form/unary_other/Scalar.birch"
template<class Gradient,class Value>
#line 79 "src/form/unary_other/Scalar.birch"
decltype(auto) grad_scalar(const Gradient& d, const libbirch::DefaultArray<Value,2>& m);
#line 15 "src/form/unary_other/Sin.birch"
template<class Middle>
#line 15 "src/form/unary_other/Sin.birch"
decltype(auto) sin(const Middle& m);
#line 22 "src/form/unary_other/Sin.birch"
#line 22 "src/form/unary_other/Sin.birch"
Real sin(const Integer& m);
#line 29 "src/form/unary_other/Sin.birch"
#line 29 "src/form/unary_other/Sin.birch"
Real sin(const Real& m);
#line 15 "src/form/unary_other/Sinh.birch"
template<class Middle>
#line 15 "src/form/unary_other/Sinh.birch"
decltype(auto) sinh(const Middle& m);
#line 22 "src/form/unary_other/Sinh.birch"
#line 22 "src/form/unary_other/Sinh.birch"
Real sinh(const Integer& m);
#line 29 "src/form/unary_other/Sinh.birch"
#line 29 "src/form/unary_other/Sinh.birch"
Real sinh(const Real& m);
#line 15 "src/form/unary_other/Sqrt.birch"
template<class Middle>
#line 15 "src/form/unary_other/Sqrt.birch"
decltype(auto) sqrt(const Middle& m);
#line 22 "src/form/unary_other/Sqrt.birch"
#line 22 "src/form/unary_other/Sqrt.birch"
Real sqrt(const Integer& m);
#line 29 "src/form/unary_other/Sqrt.birch"
#line 29 "src/form/unary_other/Sqrt.birch"
Real sqrt(const Real& x);
#line 23 "src/form/unary_other/Sum.birch"
template<class Middle>
#line 23 "src/form/unary_other/Sum.birch"
decltype(auto) sum(const Middle& m);
#line 30 "src/form/unary_other/Sum.birch"
#line 30 "src/form/unary_other/Sum.birch"
Integer sum(const Integer& m);
#line 37 "src/form/unary_other/Sum.birch"
#line 37 "src/form/unary_other/Sum.birch"
Real sum(const Real& m);
#line 44 "src/form/unary_other/Sum.birch"
#line 44 "src/form/unary_other/Sum.birch"
Integer sum(const libbirch::DefaultArray<Integer,1>& m);
#line 53 "src/form/unary_other/Sum.birch"
#line 53 "src/form/unary_other/Sum.birch"
Real sum(const libbirch::DefaultArray<Real,1>& m);
#line 62 "src/form/unary_other/Sum.birch"
#line 62 "src/form/unary_other/Sum.birch"
Integer sum(const libbirch::DefaultArray<Integer,2>& m);
#line 71 "src/form/unary_other/Sum.birch"
#line 71 "src/form/unary_other/Sum.birch"
Real sum(const libbirch::DefaultArray<Real,2>& m);
#line 15 "src/form/unary_other/Tan.birch"
template<class Middle>
#line 15 "src/form/unary_other/Tan.birch"
decltype(auto) tan(const Middle& m);
#line 22 "src/form/unary_other/Tan.birch"
#line 22 "src/form/unary_other/Tan.birch"
Real tan(const Integer& m);
#line 29 "src/form/unary_other/Tan.birch"
#line 29 "src/form/unary_other/Tan.birch"
Real tan(const Real& m);
#line 15 "src/form/unary_other/Tanh.birch"
template<class Middle>
#line 15 "src/form/unary_other/Tanh.birch"
decltype(auto) tanh(const Middle& m);
#line 22 "src/form/unary_other/Tanh.birch"
#line 22 "src/form/unary_other/Tanh.birch"
Real tanh(const Integer& m);
#line 29 "src/form/unary_other/Tanh.birch"
#line 29 "src/form/unary_other/Tanh.birch"
Real tanh(const Real& m);
#line 40 "src/test/basic/test_basic_array.birch"
template<class Container>
#line 40 "src/test/basic/test_basic_array.birch"
Boolean check_container(const Container& o, const libbirch::DefaultArray<Integer,1>& values);
#line 31 "src/test/basic/test_basic_ragged_array.birch"
#line 31 "src/test/basic/test_basic_ragged_array.birch"
Boolean check_ragged_array(const RaggedArray<Integer>& o, const libbirch::DefaultArray<Integer,1>& sizes, const libbirch::DefaultArray<Integer,1>& values);
#line 36 "src/test/basic/test_basic_stack.birch"
template<class Container>
#line 36 "src/test/basic/test_basic_stack.birch"
Boolean check_stack(const Container& o, const libbirch::DefaultArray<Integer,1>& values);
#line 47 "src/basic/Object.birch"
#line 47 "src/basic/Object.birch"
Boolean operator==(const Object& x, const Object& y);
#line 56 "src/basic/Object.birch"
#line 56 "src/basic/Object.birch"
Boolean operator!=(const Object& x, const Object& y);
#line 163 "src/basic/String.birch"
#line 163 "src/basic/String.birch"
Boolean operator>(const String& x, const String& y);
#line 172 "src/basic/String.birch"
#line 172 "src/basic/String.birch"
Boolean operator<(const String& x, const String& y);
#line 181 "src/basic/String.birch"
#line 181 "src/basic/String.birch"
Boolean operator>=(const String& x, const String& y);
#line 190 "src/basic/String.birch"
#line 190 "src/basic/String.birch"
Boolean operator<=(const String& x, const String& y);
#line 199 "src/basic/String.birch"
#line 199 "src/basic/String.birch"
Boolean operator==(const String& x, const String& y);
#line 208 "src/basic/String.birch"
#line 208 "src/basic/String.birch"
Boolean operator!=(const String& x, const String& y);
#line 222 "src/basic/String.birch"
#line 222 "src/basic/String.birch"
String operator+(const String& x, const Boolean& y);
#line 229 "src/basic/String.birch"
#line 229 "src/basic/String.birch"
String operator+(const String& x, const Real& y);
#line 236 "src/basic/String.birch"
#line 236 "src/basic/String.birch"
String operator+(const String& x, const Integer& y);
#line 243 "src/basic/String.birch"
#line 243 "src/basic/String.birch"
String operator+(const String& x, const libbirch::DefaultArray<Boolean,1>& y);
#line 250 "src/basic/String.birch"
#line 250 "src/basic/String.birch"
String operator+(const String& x, const libbirch::DefaultArray<Real,1>& y);
#line 257 "src/basic/String.birch"
#line 257 "src/basic/String.birch"
String operator+(const String& x, const libbirch::DefaultArray<Integer,1>& y);
#line 264 "src/basic/String.birch"
#line 264 "src/basic/String.birch"
String operator+(const String& x, const libbirch::DefaultArray<Boolean,2>& y);
#line 271 "src/basic/String.birch"
#line 271 "src/basic/String.birch"
String operator+(const String& x, const libbirch::DefaultArray<Real,2>& y);
#line 278 "src/basic/String.birch"
#line 278 "src/basic/String.birch"
String operator+(const String& x, const libbirch::DefaultArray<Integer,2>& y);
#line 285 "src/basic/String.birch"
#line 285 "src/basic/String.birch"
String operator+(const Boolean& x, const String& y);
#line 292 "src/basic/String.birch"
#line 292 "src/basic/String.birch"
String operator+(const Real& x, const String& y);
#line 299 "src/basic/String.birch"
#line 299 "src/basic/String.birch"
String operator+(const Integer& x, const String& y);
#line 306 "src/basic/String.birch"
#line 306 "src/basic/String.birch"
String operator+(const libbirch::DefaultArray<Boolean,1>& x, const String& y);
#line 313 "src/basic/String.birch"
#line 313 "src/basic/String.birch"
String operator+(const libbirch::DefaultArray<Real,1>& x, const String& y);
#line 320 "src/basic/String.birch"
#line 320 "src/basic/String.birch"
String operator+(const libbirch::DefaultArray<Integer,1>& x, const String& y);
#line 327 "src/basic/String.birch"
#line 327 "src/basic/String.birch"
String operator+(const libbirch::DefaultArray<Boolean,2>& x, const String& y);
#line 334 "src/basic/String.birch"
#line 334 "src/basic/String.birch"
String operator+(const libbirch::DefaultArray<Real,2>& x, const String& y);
#line 341 "src/basic/String.birch"
#line 341 "src/basic/String.birch"
String operator+(const libbirch::DefaultArray<Integer,2>& x, const String& y);
#line 15 "src/form/binary_other/Add.birch"
template<class Left,class Right>
#line 15 "src/form/binary_other/Add.birch"
Add<Left, Right> operator+(const Left& l, const Right& r);
#line 42 "src/form/binary_other/Add.birch"
#line 42 "src/form/binary_other/Add.birch"
libbirch::DefaultArray<Integer,1> operator+(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 51 "src/form/binary_other/Add.birch"
#line 51 "src/form/binary_other/Add.birch"
libbirch::DefaultArray<Real,1> operator+(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 60 "src/form/binary_other/Add.birch"
#line 60 "src/form/binary_other/Add.birch"
libbirch::DefaultArray<Real,1> operator+(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 69 "src/form/binary_other/Add.birch"
#line 69 "src/form/binary_other/Add.birch"
libbirch::DefaultArray<Real,1> operator+(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 78 "src/form/binary_other/Add.birch"
#line 78 "src/form/binary_other/Add.birch"
libbirch::DefaultArray<Integer,2> operator+(const libbirch::DefaultArray<Integer,2>& x, const libbirch::DefaultArray<Integer,2>& y);
#line 87 "src/form/binary_other/Add.birch"
#line 87 "src/form/binary_other/Add.birch"
libbirch::DefaultArray<Real,2> operator+(const libbirch::DefaultArray<Integer,2>& x, const libbirch::DefaultArray<Real,2>& y);
#line 96 "src/form/binary_other/Add.birch"
#line 96 "src/form/binary_other/Add.birch"
libbirch::DefaultArray<Real,2> operator+(const libbirch::DefaultArray<Real,2>& x, const libbirch::DefaultArray<Integer,2>& y);
#line 105 "src/form/binary_other/Add.birch"
#line 105 "src/form/binary_other/Add.birch"
libbirch::DefaultArray<Real,2> operator+(const libbirch::DefaultArray<Real,2>& x, const libbirch::DefaultArray<Real,2>& y);
#line 16 "src/form/binary_other/Divide.birch"
template<class Left,class Right>
#line 16 "src/form/binary_other/Divide.birch"
Divide<Left, Right> operator/(const Left& l, const Right& r);
#line 43 "src/form/binary_other/Divide.birch"
#line 43 "src/form/binary_other/Divide.birch"
libbirch::DefaultArray<Integer,1> operator/(const libbirch::DefaultArray<Integer,1>& x, const Integer& y);
#line 52 "src/form/binary_other/Divide.birch"
#line 52 "src/form/binary_other/Divide.birch"
libbirch::DefaultArray<Real,1> operator/(const libbirch::DefaultArray<Integer,1>& x, const Real& y);
#line 61 "src/form/binary_other/Divide.birch"
#line 61 "src/form/binary_other/Divide.birch"
libbirch::DefaultArray<Real,1> operator/(const libbirch::DefaultArray<Real,1>& x, const Integer& y);
#line 68 "src/form/binary_other/Divide.birch"
#line 68 "src/form/binary_other/Divide.birch"
libbirch::DefaultArray<Real,1> operator/(const libbirch::DefaultArray<Real,1>& x, const Real& y);
#line 77 "src/form/binary_other/Divide.birch"
#line 77 "src/form/binary_other/Divide.birch"
libbirch::DefaultArray<Integer,2> operator/(const libbirch::DefaultArray<Integer,2>& x, const Integer& y);
#line 86 "src/form/binary_other/Divide.birch"
#line 86 "src/form/binary_other/Divide.birch"
libbirch::DefaultArray<Real,2> operator/(const libbirch::DefaultArray<Integer,2>& x, const Real& y);
#line 95 "src/form/binary_other/Divide.birch"
#line 95 "src/form/binary_other/Divide.birch"
libbirch::DefaultArray<Real,2> operator/(const libbirch::DefaultArray<Real,2>& x, const Integer& y);
#line 102 "src/form/binary_other/Divide.birch"
#line 102 "src/form/binary_other/Divide.birch"
libbirch::DefaultArray<Real,2> operator/(const libbirch::DefaultArray<Real,2>& x, const Real& y);
#line 15 "src/form/binary_other/LessThan.birch"
template<class Left,class Right>
#line 15 "src/form/binary_other/LessThan.birch"
decltype(auto) operator<(const Left& l, const Right& r);
#line 56 "src/form/binary_other/Multiply.birch"
template<class Left,class Right>
#line 56 "src/form/binary_other/Multiply.birch"
Multiply<Left, Right> operator*(const Left& l, const Right& r);
#line 83 "src/form/binary_other/Multiply.birch"
#line 83 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Integer,1> operator*(const Integer& x, const libbirch::DefaultArray<Integer,1>& y);
#line 92 "src/form/binary_other/Multiply.birch"
#line 92 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const Integer& x, const libbirch::DefaultArray<Real,1>& y);
#line 99 "src/form/binary_other/Multiply.birch"
#line 99 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const Real& x, const libbirch::DefaultArray<Integer,1>& y);
#line 108 "src/form/binary_other/Multiply.birch"
#line 108 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const Real& x, const libbirch::DefaultArray<Real,1>& y);
#line 117 "src/form/binary_other/Multiply.birch"
#line 117 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Integer,1> operator*(const libbirch::DefaultArray<Integer,1>& x, const Integer& y);
#line 126 "src/form/binary_other/Multiply.birch"
#line 126 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const libbirch::DefaultArray<Integer,1>& x, const Real& y);
#line 135 "src/form/binary_other/Multiply.birch"
#line 135 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const libbirch::DefaultArray<Real,1>& x, const Integer& y);
#line 142 "src/form/binary_other/Multiply.birch"
#line 142 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const libbirch::DefaultArray<Real,1>& x, const Real& y);
#line 151 "src/form/binary_other/Multiply.birch"
#line 151 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Integer,1> operator*(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 160 "src/form/binary_other/Multiply.birch"
#line 160 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 169 "src/form/binary_other/Multiply.birch"
#line 169 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 178 "src/form/binary_other/Multiply.birch"
#line 178 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 187 "src/form/binary_other/Multiply.birch"
#line 187 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Integer,2> operator*(const Integer& x, const libbirch::DefaultArray<Integer,2>& y);
#line 196 "src/form/binary_other/Multiply.birch"
#line 196 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const Integer& x, const libbirch::DefaultArray<Real,2>& y);
#line 203 "src/form/binary_other/Multiply.birch"
#line 203 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const Real& x, const libbirch::DefaultArray<Integer,2>& y);
#line 212 "src/form/binary_other/Multiply.birch"
#line 212 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const Real& x, const libbirch::DefaultArray<Real,2>& y);
#line 221 "src/form/binary_other/Multiply.birch"
#line 221 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Integer,2> operator*(const libbirch::DefaultArray<Integer,2>& x, const Integer& y);
#line 230 "src/form/binary_other/Multiply.birch"
#line 230 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const libbirch::DefaultArray<Integer,2>& x, const Real& y);
#line 239 "src/form/binary_other/Multiply.birch"
#line 239 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const libbirch::DefaultArray<Real,2>& x, const Integer& y);
#line 246 "src/form/binary_other/Multiply.birch"
#line 246 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const libbirch::DefaultArray<Real,2>& x, const Real& y);
#line 255 "src/form/binary_other/Multiply.birch"
#line 255 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Integer,1> operator*(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Integer,1>& y);
#line 264 "src/form/binary_other/Multiply.birch"
#line 264 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Real,1>& y);
#line 273 "src/form/binary_other/Multiply.birch"
#line 273 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Integer,1>& y);
#line 282 "src/form/binary_other/Multiply.birch"
#line 282 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,1> operator*(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Real,1>& y);
#line 291 "src/form/binary_other/Multiply.birch"
#line 291 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Integer,2> operator*(const libbirch::DefaultArray<Integer,1>& X, const libbirch::DefaultArray<Integer,2>& y);
#line 300 "src/form/binary_other/Multiply.birch"
#line 300 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const libbirch::DefaultArray<Integer,1>& X, const libbirch::DefaultArray<Real,2>& y);
#line 309 "src/form/binary_other/Multiply.birch"
#line 309 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const libbirch::DefaultArray<Real,1>& X, const libbirch::DefaultArray<Integer,2>& y);
#line 318 "src/form/binary_other/Multiply.birch"
#line 318 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const libbirch::DefaultArray<Real,1>& X, const libbirch::DefaultArray<Real,2>& y);
#line 327 "src/form/binary_other/Multiply.birch"
#line 327 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Integer,2> operator*(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Integer,2>& Y);
#line 336 "src/form/binary_other/Multiply.birch"
#line 336 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const libbirch::DefaultArray<Integer,2>& X, const libbirch::DefaultArray<Real,2>& Y);
#line 345 "src/form/binary_other/Multiply.birch"
#line 345 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Integer,2>& Y);
#line 354 "src/form/binary_other/Multiply.birch"
#line 354 "src/form/binary_other/Multiply.birch"
libbirch::DefaultArray<Real,2> operator*(const libbirch::DefaultArray<Real,2>& X, const libbirch::DefaultArray<Real,2>& Y);
#line 15 "src/form/binary_other/Subtract.birch"
template<class Left,class Right>
#line 15 "src/form/binary_other/Subtract.birch"
Subtract<Left, Right> operator-(const Left& l, const Right& r);
#line 42 "src/form/binary_other/Subtract.birch"
#line 42 "src/form/binary_other/Subtract.birch"
libbirch::DefaultArray<Integer,1> operator-(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 51 "src/form/binary_other/Subtract.birch"
#line 51 "src/form/binary_other/Subtract.birch"
libbirch::DefaultArray<Real,1> operator-(const libbirch::DefaultArray<Integer,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 60 "src/form/binary_other/Subtract.birch"
#line 60 "src/form/binary_other/Subtract.birch"
libbirch::DefaultArray<Real,1> operator-(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Integer,1>& y);
#line 69 "src/form/binary_other/Subtract.birch"
#line 69 "src/form/binary_other/Subtract.birch"
libbirch::DefaultArray<Real,1> operator-(const libbirch::DefaultArray<Real,1>& x, const libbirch::DefaultArray<Real,1>& y);
#line 78 "src/form/binary_other/Subtract.birch"
#line 78 "src/form/binary_other/Subtract.birch"
libbirch::DefaultArray<Integer,2> operator-(const libbirch::DefaultArray<Integer,2>& x, const libbirch::DefaultArray<Integer,2>& y);
#line 87 "src/form/binary_other/Subtract.birch"
#line 87 "src/form/binary_other/Subtract.birch"
libbirch::DefaultArray<Real,2> operator-(const libbirch::DefaultArray<Integer,2>& x, const libbirch::DefaultArray<Real,2>& y);
#line 96 "src/form/binary_other/Subtract.birch"
#line 96 "src/form/binary_other/Subtract.birch"
libbirch::DefaultArray<Real,2> operator-(const libbirch::DefaultArray<Real,2>& x, const libbirch::DefaultArray<Integer,2>& y);
#line 105 "src/form/binary_other/Subtract.birch"
#line 105 "src/form/binary_other/Subtract.birch"
libbirch::DefaultArray<Real,2> operator-(const libbirch::DefaultArray<Real,2>& x, const libbirch::DefaultArray<Real,2>& y);
#line 24 "src/math/eigen.birch"
template<class Type>
#line 24 "src/math/eigen.birch"
libbirch::DefaultArray<Type,1> operator+(const libbirch::DefaultArray<Type,1>& x);
#line 31 "src/math/eigen.birch"
template<class Type>
#line 31 "src/math/eigen.birch"
libbirch::DefaultArray<Type,2> operator+(const libbirch::DefaultArray<Type,2>& X);
#line 15 "src/form/unary_other/Negate.birch"
template<class Middle>
#line 15 "src/form/unary_other/Negate.birch"
decltype(auto) operator-(const Middle& m);
#line 32 "src/form/unary_other/Negate.birch"
#line 32 "src/form/unary_other/Negate.birch"
libbirch::DefaultArray<Integer,1> operator-(const libbirch::DefaultArray<Integer,1>& x);
#line 41 "src/form/unary_other/Negate.birch"
#line 41 "src/form/unary_other/Negate.birch"
libbirch::DefaultArray<Real,1> operator-(const libbirch::DefaultArray<Real,1>& x);
#line 50 "src/form/unary_other/Negate.birch"
#line 50 "src/form/unary_other/Negate.birch"
libbirch::DefaultArray<Integer,2> operator-(const libbirch::DefaultArray<Integer,2>& X);
#line 59 "src/form/unary_other/Negate.birch"
#line 59 "src/form/unary_other/Negate.birch"
libbirch::DefaultArray<Real,2> operator-(const libbirch::DefaultArray<Real,2>& X);
#line 4 "src/basic/Object.birch"
#line 4 "src/basic/Object.birch"
class Object_ : public libbirch::Any {
public:
  #line 4 "src/basic/Object.birch"
  LIBBIRCH_ABSTRACT_CLASS(Object_, libbirch::Any)
  #line 4 "src/basic/Object.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 4 "src/basic/Object.birch"
  using base_type_::operator=;
  
  #line 4 "src/basic/Object.birch"
  Object_();

  #line 10 "src/basic/Object.birch"
  #line 10 "src/basic/Object.birch"
  virtual void read(const Buffer& buffer);
  #line 19 "src/basic/Object.birch"
  #line 19 "src/basic/Object.birch"
  virtual void write(const Buffer& buffer);
  #line 29 "src/basic/Object.birch"
  #line 29 "src/basic/Object.birch"
  virtual void read(const Integer& t, const Buffer& buffer);
  #line 39 "src/basic/Object.birch"
  #line 39 "src/basic/Object.birch"
  virtual void write(const Integer& t, const Buffer& buffer);
};

#line 6 "src/container/Array.birch"
template<class Type>
#line 6 "src/container/Array.birch"
class Array_ final : public Object_ {
public:
  #line 6 "src/container/Array.birch"
  using Type_ = Type;
  #line 6 "src/container/Array.birch"
  LIBBIRCH_CLASS(Array_, Object_)
  #line 6 "src/container/Array.birch"
  LIBBIRCH_CLASS_MEMBERS(values)
  #line 6 "src/container/Array.birch"
  using base_type_::operator=;
  #line 6 "src/container/Array.birch"
  using base_type_::read;
  #line 6 "src/container/Array.birch"
  using base_type_::write;
  
  #line 6 "src/container/Array.birch"
  Array_();

  libbirch::DefaultArray<Type,1> values;
  #line 15 "src/container/Array.birch"
  Type& operator()(const Integer& i);
  #line 22 "src/container/Array.birch"
  #line 22 "src/container/Array.birch"
  Integer size();
  #line 29 "src/container/Array.birch"
  #line 29 "src/container/Array.birch"
  Boolean empty();
  #line 36 "src/container/Array.birch"
  #line 36 "src/container/Array.birch"
  void clear();
  #line 46 "src/container/Array.birch"
  #line 46 "src/container/Array.birch"
  Type get(const Integer& i);
  #line 56 "src/container/Array.birch"
  #line 56 "src/container/Array.birch"
  void set(const Integer& i, const Type& x);
  #line 63 "src/container/Array.birch"
  #line 63 "src/container/Array.birch"
  Type front();
  #line 71 "src/container/Array.birch"
  #line 71 "src/container/Array.birch"
  Type back();
  #line 81 "src/container/Array.birch"
  #line 81 "src/container/Array.birch"
  void pushFront(const Type& x);
  #line 88 "src/container/Array.birch"
  #line 88 "src/container/Array.birch"
  Type pushFront();
  #line 103 "src/container/Array.birch"
  #line 103 "src/container/Array.birch"
  void pushBack(const Type& x);
  #line 110 "src/container/Array.birch"
  #line 110 "src/container/Array.birch"
  Type pushBack();
  #line 123 "src/container/Array.birch"
  #line 123 "src/container/Array.birch"
  void popFront();
  #line 130 "src/container/Array.birch"
  #line 130 "src/container/Array.birch"
  void popBack();
  #line 144 "src/container/Array.birch"
  #line 144 "src/container/Array.birch"
  void insert(const Integer& i, const Type& x);
  #line 158 "src/container/Array.birch"
  #line 158 "src/container/Array.birch"
  void erase(const Integer& i);
  #line 173 "src/container/Array.birch"
  #line 173 "src/container/Array.birch"
  void erase(const Integer& i, const Integer& n);
  #line 186 "src/container/Array.birch"
  #line 186 "src/container/Array.birch"
  Iterator<Type> walk();
  #line 193 "src/container/Array.birch"
  #line 193 "src/container/Array.birch"
  libbirch::DefaultArray<Type,1> toArray();
  #line 200 "src/container/Array.birch"
  #line 200 "src/container/Array.birch"
  void fromArray(const libbirch::DefaultArray<Type,1>& x);
  #line 204 "src/container/Array.birch"
  #line 204 "src/container/Array.birch"
  void read(const Buffer& buffer) override;
  #line 209 "src/container/Array.birch"
  #line 209 "src/container/Array.birch"
  void write(const Buffer& buffer) override;
};

#line 30 "src/container/List.birch"
template<class Type>
#line 30 "src/container/List.birch"
class List_ final : public Object_ {
public:
  #line 30 "src/container/List.birch"
  using Type_ = Type;
  #line 30 "src/container/List.birch"
  LIBBIRCH_CLASS(List_, Object_)
  #line 30 "src/container/List.birch"
  LIBBIRCH_CLASS_MEMBERS(head, tail, count)
  #line 30 "src/container/List.birch"
  using base_type_::operator=;
  #line 30 "src/container/List.birch"
  using base_type_::read;
  #line 30 "src/container/List.birch"
  using base_type_::write;
  
  #line 30 "src/container/List.birch"
  List_();

  std::optional<ListNode<Type>> head;
  std::optional<ListNode<Type>> tail;
  Integer count;
  #line 38 "src/container/List.birch"
  Type& operator()(const Integer& i);
  #line 45 "src/container/List.birch"
  #line 45 "src/container/List.birch"
  Integer size();
  #line 52 "src/container/List.birch"
  #line 52 "src/container/List.birch"
  Boolean empty();
  #line 59 "src/container/List.birch"
  #line 59 "src/container/List.birch"
  void clear();
  #line 68 "src/container/List.birch"
  #line 68 "src/container/List.birch"
  Type front();
  #line 76 "src/container/List.birch"
  #line 76 "src/container/List.birch"
  Type back();
  #line 87 "src/container/List.birch"
  #line 87 "src/container/List.birch"
  Type get(const Integer& i);
  #line 97 "src/container/List.birch"
  #line 97 "src/container/List.birch"
  void set(const Integer& i, const Type& x);
  #line 106 "src/container/List.birch"
  #line 106 "src/container/List.birch"
  void pushFront(const Type& x);
  #line 124 "src/container/List.birch"
  #line 124 "src/container/List.birch"
  Type pushFront();
  #line 139 "src/container/List.birch"
  #line 139 "src/container/List.birch"
  void pushBack(const Type& x);
  #line 158 "src/container/List.birch"
  #line 158 "src/container/List.birch"
  Type pushBack();
  #line 171 "src/container/List.birch"
  #line 171 "src/container/List.birch"
  void popFront();
  #line 187 "src/container/List.birch"
  #line 187 "src/container/List.birch"
  void popBack();
  #line 210 "src/container/List.birch"
  #line 210 "src/container/List.birch"
  void insert(const Integer& i, const Type& x);
  #line 231 "src/container/List.birch"
  #line 231 "src/container/List.birch"
  void erase(const Integer& i);
  #line 252 "src/container/List.birch"
  #line 252 "src/container/List.birch"
  Iterator<Type> walk();
  #line 261 "src/container/List.birch"
  #line 261 "src/container/List.birch"
  ListNode<Type> getNode(const Integer& i);
  #line 285 "src/container/List.birch"
  #line 285 "src/container/List.birch"
  void read(const Buffer& buffer) override;
  #line 296 "src/container/List.birch"
  #line 296 "src/container/List.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/container/ListNode.birch"
template<class Type>
#line 4 "src/container/ListNode.birch"
class ListNode_ final : public Object_ {
public:
  #line 4 "src/container/ListNode.birch"
  using Type_ = Type;
  #line 4 "src/container/ListNode.birch"
  LIBBIRCH_CLASS(ListNode_, Object_)
  #line 4 "src/container/ListNode.birch"
  LIBBIRCH_CLASS_MEMBERS(prev, next, x)
  #line 4 "src/container/ListNode.birch"
  using base_type_::operator=;
  
  #line 4 "src/container/ListNode.birch"
  ListNode_(const Type& x);

  std::optional<ListNode<Type>> prev;
  std::optional<ListNode<Type>> next;
  Type x;
  #line 14 "src/container/ListNode.birch"
  #line 14 "src/container/ListNode.birch"
  void insert(const ListNode<Type>& node);
  #line 25 "src/container/ListNode.birch"
  #line 25 "src/container/ListNode.birch"
  void erase();
  #line 39 "src/container/ListNode.birch"
  #line 39 "src/container/ListNode.birch"
  std::optional<ListNode<Type>> popFront();
  #line 54 "src/container/ListNode.birch"
  #line 54 "src/container/ListNode.birch"
  std::optional<ListNode<Type>> popBack();
};

#line 7 "src/container/RaggedArray.birch"
template<class Type>
#line 7 "src/container/RaggedArray.birch"
class RaggedArray_ final : public Object_ {
public:
  #line 7 "src/container/RaggedArray.birch"
  using Type_ = Type;
  #line 7 "src/container/RaggedArray.birch"
  LIBBIRCH_CLASS(RaggedArray_, Object_)
  #line 7 "src/container/RaggedArray.birch"
  LIBBIRCH_CLASS_MEMBERS(values, offsets, sizes)
  #line 7 "src/container/RaggedArray.birch"
  using base_type_::operator=;
  #line 7 "src/container/RaggedArray.birch"
  using base_type_::read;
  #line 7 "src/container/RaggedArray.birch"
  using base_type_::write;
  
  #line 7 "src/container/RaggedArray.birch"
  RaggedArray_();

  libbirch::DefaultArray<Type,1> values;
  libbirch::DefaultArray<Integer,1> offsets;
  libbirch::DefaultArray<Integer,1> sizes;
  #line 29 "src/container/RaggedArray.birch"
  Type& operator()(const Integer& i, const Integer& j);
  #line 38 "src/container/RaggedArray.birch"
  #line 38 "src/container/RaggedArray.birch"
  Boolean empty();
  #line 45 "src/container/RaggedArray.birch"
  #line 45 "src/container/RaggedArray.birch"
  void clear();
  #line 58 "src/container/RaggedArray.birch"
  #line 58 "src/container/RaggedArray.birch"
  Integer count();
  #line 65 "src/container/RaggedArray.birch"
  #line 65 "src/container/RaggedArray.birch"
  Integer size();
  #line 74 "src/container/RaggedArray.birch"
  #line 74 "src/container/RaggedArray.birch"
  Integer size(const Integer& i);
  #line 84 "src/container/RaggedArray.birch"
  #line 84 "src/container/RaggedArray.birch"
  Type get(const Integer& i, const Integer& j);
  #line 93 "src/container/RaggedArray.birch"
  #line 93 "src/container/RaggedArray.birch"
  libbirch::DefaultArray<Type,1> get(const Integer& i);
  #line 104 "src/container/RaggedArray.birch"
  #line 104 "src/container/RaggedArray.birch"
  void set(const Integer& i, const Integer& j, const Type& x);
  #line 117 "src/container/RaggedArray.birch"
  #line 117 "src/container/RaggedArray.birch"
  void set(const Integer& i, const libbirch::DefaultArray<Type,1>& x);
  #line 124 "src/container/RaggedArray.birch"
  #line 124 "src/container/RaggedArray.birch"
  void popFront();
  #line 150 "src/container/RaggedArray.birch"
  #line 150 "src/container/RaggedArray.birch"
  void popFront(const Integer& i);
  #line 165 "src/container/RaggedArray.birch"
  #line 165 "src/container/RaggedArray.birch"
  void pushBack();
  #line 179 "src/container/RaggedArray.birch"
  #line 179 "src/container/RaggedArray.birch"
  void pushBack(const Integer& i, const Type& x);
  #line 194 "src/container/RaggedArray.birch"
  #line 194 "src/container/RaggedArray.birch"
  Type pushBack(const Integer& i);
  #line 209 "src/container/RaggedArray.birch"
  #line 209 "src/container/RaggedArray.birch"
  Iterator<libbirch::DefaultArray<Type,1>> walk();
  #line 221 "src/container/RaggedArray.birch"
  #line 221 "src/container/RaggedArray.birch"
  Iterator<Type> walk(const Integer& i);
  #line 230 "src/container/RaggedArray.birch"
  #line 230 "src/container/RaggedArray.birch"
  Integer from(const Integer& i);
  #line 241 "src/container/RaggedArray.birch"
  #line 241 "src/container/RaggedArray.birch"
  Integer to(const Integer& i);
  #line 253 "src/container/RaggedArray.birch"
  #line 253 "src/container/RaggedArray.birch"
  Integer serial(const Integer& i, const Integer& j);
  #line 259 "src/container/RaggedArray.birch"
  #line 259 "src/container/RaggedArray.birch"
  void read(const Buffer& buffer) override;
  #line 274 "src/container/RaggedArray.birch"
  #line 274 "src/container/RaggedArray.birch"
  void write(const Buffer& buffer) override;
};

#line 28 "src/container/Stack.birch"
template<class Type>
#line 28 "src/container/Stack.birch"
class Stack_ final : public Object_ {
public:
  #line 28 "src/container/Stack.birch"
  using Type_ = Type;
  #line 28 "src/container/Stack.birch"
  LIBBIRCH_CLASS(Stack_, Object_)
  #line 28 "src/container/Stack.birch"
  LIBBIRCH_CLASS_MEMBERS(head, count)
  #line 28 "src/container/Stack.birch"
  using base_type_::operator=;
  
  #line 28 "src/container/Stack.birch"
  Stack_();

  std::optional<StackNode<Type>> head;
  Integer count;
  #line 35 "src/container/Stack.birch"
  #line 35 "src/container/Stack.birch"
  Integer size();
  #line 42 "src/container/Stack.birch"
  #line 42 "src/container/Stack.birch"
  Boolean empty();
  #line 49 "src/container/Stack.birch"
  #line 49 "src/container/Stack.birch"
  void clear();
  #line 57 "src/container/Stack.birch"
  #line 57 "src/container/Stack.birch"
  Type top();
  #line 67 "src/container/Stack.birch"
  #line 67 "src/container/Stack.birch"
  void push(const Type& x);
  #line 77 "src/container/Stack.birch"
  #line 77 "src/container/Stack.birch"
  void push();
  #line 90 "src/container/Stack.birch"
  #line 90 "src/container/Stack.birch"
  void pop();
  #line 101 "src/container/Stack.birch"
  #line 101 "src/container/Stack.birch"
  Iterator<Type> walk();
};

#line 1 "src/container/StackNode.birch"
template<class Type>
#line 1 "src/container/StackNode.birch"
class StackNode_ : public Object_ {
public:
  #line 1 "src/container/StackNode.birch"
  using Type_ = Type;
  #line 1 "src/container/StackNode.birch"
  LIBBIRCH_CLASS(StackNode_, Object_)
  #line 1 "src/container/StackNode.birch"
  LIBBIRCH_CLASS_MEMBERS(next, x)
  #line 1 "src/container/StackNode.birch"
  using base_type_::operator=;
  
  #line 1 "src/container/StackNode.birch"
  StackNode_(const Type& x);

  std::optional<StackNode<Type>> next;
  Type x;
};

#line 40 "src/container/Tape.birch"
template<class Type>
#line 40 "src/container/Tape.birch"
class Tape_ final : public Object_ {
public:
  #line 40 "src/container/Tape.birch"
  using Type_ = Type;
  #line 40 "src/container/Tape.birch"
  LIBBIRCH_CLASS(Tape_, Object_)
  #line 40 "src/container/Tape.birch"
  LIBBIRCH_CLASS_MEMBERS(ahead, behind, aheadCount, behindCount)
  #line 40 "src/container/Tape.birch"
  using base_type_::operator=;
  #line 40 "src/container/Tape.birch"
  using base_type_::read;
  #line 40 "src/container/Tape.birch"
  using base_type_::write;
  
  #line 40 "src/container/Tape.birch"
  Tape_();

  std::optional<TapeNode<Type>> ahead;
  std::optional<TapeNode<Type>> behind;
  Integer aheadCount;
  Integer behindCount;
  #line 65 "src/container/Tape.birch"
  Type& operator()(const Integer& i);
  #line 74 "src/container/Tape.birch"
  #line 74 "src/container/Tape.birch"
  Integer size();
  #line 81 "src/container/Tape.birch"
  #line 81 "src/container/Tape.birch"
  Boolean empty();
  #line 88 "src/container/Tape.birch"
  #line 88 "src/container/Tape.birch"
  void clear();
  #line 98 "src/container/Tape.birch"
  #line 98 "src/container/Tape.birch"
  Type front();
  #line 109 "src/container/Tape.birch"
  #line 109 "src/container/Tape.birch"
  Type back();
  #line 122 "src/container/Tape.birch"
  #line 122 "src/container/Tape.birch"
  Type get(const Integer& i);
  #line 134 "src/container/Tape.birch"
  #line 134 "src/container/Tape.birch"
  void set(const Integer& i, const Type& x);
  #line 145 "src/container/Tape.birch"
  #line 145 "src/container/Tape.birch"
  void pushFront(const Type& x);
  #line 154 "src/container/Tape.birch"
  #line 154 "src/container/Tape.birch"
  Type pushFront();
  #line 166 "src/container/Tape.birch"
  #line 166 "src/container/Tape.birch"
  void pushBack(const Type& x);
  #line 175 "src/container/Tape.birch"
  #line 175 "src/container/Tape.birch"
  Type pushBack();
  #line 185 "src/container/Tape.birch"
  #line 185 "src/container/Tape.birch"
  void popFront();
  #line 195 "src/container/Tape.birch"
  #line 195 "src/container/Tape.birch"
  void popBack();
  #line 212 "src/container/Tape.birch"
  #line 212 "src/container/Tape.birch"
  void insert(const Integer& i, const Type& x);
  #line 226 "src/container/Tape.birch"
  #line 226 "src/container/Tape.birch"
  void erase(const Integer& i);
  #line 238 "src/container/Tape.birch"
  #line 238 "src/container/Tape.birch"
  Iterator<Type> walk();
  #line 246 "src/container/Tape.birch"
  #line 246 "src/container/Tape.birch"
  void forward();
  #line 258 "src/container/Tape.birch"
  #line 258 "src/container/Tape.birch"
  void backward();
  #line 270 "src/container/Tape.birch"
  #line 270 "src/container/Tape.birch"
  void rewind();
  #line 279 "src/container/Tape.birch"
  #line 279 "src/container/Tape.birch"
  void fastForward();
  #line 293 "src/container/Tape.birch"
  #line 293 "src/container/Tape.birch"
  void seek(const Integer& k);
  #line 306 "src/container/Tape.birch"
  #line 306 "src/container/Tape.birch"
  void read(const Buffer& buffer) override;
  #line 317 "src/container/Tape.birch"
  #line 317 "src/container/Tape.birch"
  void write(const Buffer& buffer) override;
};

#line 1 "src/container/TapeNode.birch"
template<class Type>
#line 1 "src/container/TapeNode.birch"
class TapeNode_ : public Object_ {
public:
  #line 1 "src/container/TapeNode.birch"
  using Type_ = Type;
  #line 1 "src/container/TapeNode.birch"
  LIBBIRCH_CLASS(TapeNode_, Object_)
  #line 1 "src/container/TapeNode.birch"
  LIBBIRCH_CLASS_MEMBERS(next, x)
  #line 1 "src/container/TapeNode.birch"
  using base_type_::operator=;
  
  #line 1 "src/container/TapeNode.birch"
  TapeNode_(const Type& x);

  std::optional<TapeNode<Type>> next;
  Type x;
};

#line 1 "src/data/Buffer.birch"
#line 1 "src/data/Buffer.birch"
class Buffer_ : public Object_ {
public:
  #line 1 "src/data/Buffer.birch"
  LIBBIRCH_CLASS(Buffer_, Object_)
  #line 1 "src/data/Buffer.birch"
  LIBBIRCH_CLASS_MEMBERS(keys, values, scalarString, scalarReal, scalarInteger, scalarBoolean, vectorReal, vectorInteger, vectorBoolean, matrixReal, matrixInteger, matrixBoolean)
  #line 1 "src/data/Buffer.birch"
  using base_type_::operator=;
  
  #line 1 "src/data/Buffer.birch"
  Buffer_();

  std::optional<libbirch::DefaultArray<String,1>> keys;
  std::optional<libbirch::DefaultArray<Buffer,1>> values;
  std::optional<String> scalarString;
  std::optional<Real> scalarReal;
  std::optional<Integer> scalarInteger;
  std::optional<Boolean> scalarBoolean;
  std::optional<libbirch::DefaultArray<Real,1>> vectorReal;
  std::optional<libbirch::DefaultArray<Integer,1>> vectorInteger;
  std::optional<libbirch::DefaultArray<Boolean,1>> vectorBoolean;
  std::optional<libbirch::DefaultArray<Real,2>> matrixReal;
  std::optional<libbirch::DefaultArray<Integer,2>> matrixInteger;
  std::optional<libbirch::DefaultArray<Boolean,2>> matrixBoolean;
  #line 56 "src/data/Buffer.birch"
  #line 56 "src/data/Buffer.birch"
  virtual Boolean isNil();
  #line 66 "src/data/Buffer.birch"
  #line 66 "src/data/Buffer.birch"
  virtual Boolean isNil(const String& key);
  #line 78 "src/data/Buffer.birch"
  #line 78 "src/data/Buffer.birch"
  virtual void setNil();
  #line 96 "src/data/Buffer.birch"
  #line 96 "src/data/Buffer.birch"
  virtual void setNil(const String& key);
  #line 106 "src/data/Buffer.birch"
  #line 106 "src/data/Buffer.birch"
  virtual Boolean isEmpty();
  #line 114 "src/data/Buffer.birch"
  #line 114 "src/data/Buffer.birch"
  virtual Boolean isEmpty(const String& key);
  #line 126 "src/data/Buffer.birch"
  #line 126 "src/data/Buffer.birch"
  virtual void setEmptyObject();
  #line 137 "src/data/Buffer.birch"
  #line 137 "src/data/Buffer.birch"
  virtual void setEmptyObject(const String& key);
  #line 146 "src/data/Buffer.birch"
  #line 146 "src/data/Buffer.birch"
  virtual void setEmptyArray();
  #line 155 "src/data/Buffer.birch"
  #line 155 "src/data/Buffer.birch"
  virtual void setEmptyArray(const String& key);
  #line 165 "src/data/Buffer.birch"
  #line 165 "src/data/Buffer.birch"
  virtual void split();
  #line 181 "src/data/Buffer.birch"
  #line 181 "src/data/Buffer.birch"
  virtual Integer size();
  #line 211 "src/data/Buffer.birch"
  #line 211 "src/data/Buffer.birch"
  virtual Integer size(const String& key);
  #line 228 "src/data/Buffer.birch"
  template<class Type>
  #line 228 "src/data/Buffer.birch"
  std::optional<Type> get();
  #line 243 "src/data/Buffer.birch"
  template<class Type>
  #line 243 "src/data/Buffer.birch"
  std::optional<Type> get(const String& key);
  #line 263 "src/data/Buffer.birch"
  template<class Type>
  #line 263 "src/data/Buffer.birch"
  void get(const Type& o);
  #line 279 "src/data/Buffer.birch"
  template<class Type>
  #line 279 "src/data/Buffer.birch"
  void get(const String& key, const Type& o);
  #line 293 "src/data/Buffer.birch"
  #line 293 "src/data/Buffer.birch"
  virtual std::optional<Buffer> get(const String& key);
  #line 310 "src/data/Buffer.birch"
  #line 310 "src/data/Buffer.birch"
  virtual void set(const libbirch::DefaultArray<String,1>& keys, const libbirch::DefaultArray<Buffer,1>& values);
  #line 322 "src/data/Buffer.birch"
  template<class Type>
  #line 322 "src/data/Buffer.birch"
  void set(const Type& x);
  #line 332 "src/data/Buffer.birch"
  template<class Type>
  #line 332 "src/data/Buffer.birch"
  void set(const Integer& t, const Type& x);
  #line 346 "src/data/Buffer.birch"
  #line 346 "src/data/Buffer.birch"
  virtual void set(const String& key, const libbirch::DefaultArray<String,1>& keys, const libbirch::DefaultArray<Buffer,1>& values);
  #line 359 "src/data/Buffer.birch"
  template<class Type>
  #line 359 "src/data/Buffer.birch"
  void set(const String& key, const Type& x);
  #line 373 "src/data/Buffer.birch"
  template<class Type>
  #line 373 "src/data/Buffer.birch"
  void set(const String& key, const Integer& t, const Type& x);
  #line 386 "src/data/Buffer.birch"
  #line 386 "src/data/Buffer.birch"
  virtual void set(const String& key, const Buffer& x);
  #line 404 "src/data/Buffer.birch"
  #line 404 "src/data/Buffer.birch"
  virtual Iterator<Buffer> walk();
  #line 442 "src/data/Buffer.birch"
  #line 442 "src/data/Buffer.birch"
  virtual Iterator<Buffer> walk(const String& key);
  #line 461 "src/data/Buffer.birch"
  template<class Type>
  #line 461 "src/data/Buffer.birch"
  void push(const Type& x);
  #line 471 "src/data/Buffer.birch"
  template<class Type>
  #line 471 "src/data/Buffer.birch"
  void push(const Integer& t, const Type& x);
  #line 480 "src/data/Buffer.birch"
  #line 480 "src/data/Buffer.birch"
  virtual void push(const Buffer& x);
  #line 501 "src/data/Buffer.birch"
  template<class Type>
  #line 501 "src/data/Buffer.birch"
  void push(const String& key, const Type& x);
  #line 517 "src/data/Buffer.birch"
  template<class Type>
  #line 517 "src/data/Buffer.birch"
  void push(const String& key, const Integer& t, const Type& x);
  #line 532 "src/data/Buffer.birch"
  #line 532 "src/data/Buffer.birch"
  virtual void push(const String& key, const Buffer& x);
  #line 544 "src/data/Buffer.birch"
  #line 544 "src/data/Buffer.birch"
  virtual void pushNil();
  #line 553 "src/data/Buffer.birch"
  #line 553 "src/data/Buffer.birch"
  virtual void pushNil(const String& key);
  #line 557 "src/data/Buffer.birch"
  #line 557 "src/data/Buffer.birch"
  virtual void accept(const Writer& writer);
  #line 587 "src/data/Buffer.birch"
  #line 587 "src/data/Buffer.birch"
  virtual std::optional<Boolean> doGet(const std::optional<Boolean>& x);
  #line 601 "src/data/Buffer.birch"
  #line 601 "src/data/Buffer.birch"
  virtual std::optional<Integer> doGet(const std::optional<Integer>& x);
  #line 615 "src/data/Buffer.birch"
  #line 615 "src/data/Buffer.birch"
  virtual std::optional<Real> doGet(const std::optional<Real>& x);
  #line 629 "src/data/Buffer.birch"
  #line 629 "src/data/Buffer.birch"
  virtual std::optional<String> doGet(const std::optional<String>& x);
  #line 643 "src/data/Buffer.birch"
  template<class Type>
  #line 643 "src/data/Buffer.birch"
  std::optional<Type> doGet(const std::optional<Type>& x);
  #line 647 "src/data/Buffer.birch"
  #line 647 "src/data/Buffer.birch"
  virtual std::optional<libbirch::DefaultArray<Boolean,1>> doGet(const std::optional<libbirch::DefaultArray<Boolean,1>>& x);
  #line 659 "src/data/Buffer.birch"
  #line 659 "src/data/Buffer.birch"
  virtual std::optional<libbirch::DefaultArray<Integer,1>> doGet(const std::optional<libbirch::DefaultArray<Integer,1>>& x);
  #line 671 "src/data/Buffer.birch"
  #line 671 "src/data/Buffer.birch"
  virtual std::optional<libbirch::DefaultArray<Real,1>> doGet(const std::optional<libbirch::DefaultArray<Real,1>>& x);
  #line 683 "src/data/Buffer.birch"
  template<class Type>
  #line 683 "src/data/Buffer.birch"
  std::optional<libbirch::DefaultArray<Type,1>> doGet(const std::optional<libbirch::DefaultArray<Type,1>>& x);
  #line 687 "src/data/Buffer.birch"
  template<class Type>
  #line 687 "src/data/Buffer.birch"
  std::optional<libbirch::DefaultArray<Type,1>> doGetVector();
  #line 707 "src/data/Buffer.birch"
  #line 707 "src/data/Buffer.birch"
  virtual std::optional<libbirch::DefaultArray<Boolean,2>> doGet(const std::optional<libbirch::DefaultArray<Boolean,2>>& x);
  #line 719 "src/data/Buffer.birch"
  #line 719 "src/data/Buffer.birch"
  virtual std::optional<libbirch::DefaultArray<Integer,2>> doGet(const std::optional<libbirch::DefaultArray<Integer,2>>& x);
  #line 731 "src/data/Buffer.birch"
  #line 731 "src/data/Buffer.birch"
  virtual std::optional<libbirch::DefaultArray<Real,2>> doGet(const std::optional<libbirch::DefaultArray<Real,2>>& x);
  #line 743 "src/data/Buffer.birch"
  template<class Type>
  #line 743 "src/data/Buffer.birch"
  std::optional<libbirch::DefaultArray<Type,2>> doGet(const std::optional<libbirch::DefaultArray<Type,2>>& x);
  #line 747 "src/data/Buffer.birch"
  template<class Type>
  #line 747 "src/data/Buffer.birch"
  std::optional<libbirch::DefaultArray<Type,2>> doGetMatrix();
  #line 776 "src/data/Buffer.birch"
  #line 776 "src/data/Buffer.birch"
  virtual void doSet(const Boolean& x);
  #line 781 "src/data/Buffer.birch"
  #line 781 "src/data/Buffer.birch"
  virtual void doSet(const Integer& x);
  #line 786 "src/data/Buffer.birch"
  #line 786 "src/data/Buffer.birch"
  virtual void doSet(const Real& x);
  #line 791 "src/data/Buffer.birch"
  #line 791 "src/data/Buffer.birch"
  virtual void doSet(const String& x);
  #line 796 "src/data/Buffer.birch"
  template<class Type>
  #line 796 "src/data/Buffer.birch"
  void doSet(const Type& x);
  #line 801 "src/data/Buffer.birch"
  template<class Type>
  #line 801 "src/data/Buffer.birch"
  void doSet(const Integer& t, const Type& x);
  #line 806 "src/data/Buffer.birch"
  #line 806 "src/data/Buffer.birch"
  virtual void doSet(const libbirch::DefaultArray<Boolean,1>& x);
  #line 811 "src/data/Buffer.birch"
  #line 811 "src/data/Buffer.birch"
  virtual void doSet(const libbirch::DefaultArray<Integer,1>& x);
  #line 816 "src/data/Buffer.birch"
  #line 816 "src/data/Buffer.birch"
  virtual void doSet(const libbirch::DefaultArray<Real,1>& x);
  #line 821 "src/data/Buffer.birch"
  template<class Type>
  #line 821 "src/data/Buffer.birch"
  void doSet(const libbirch::DefaultArray<Type,1>& x);
  #line 828 "src/data/Buffer.birch"
  template<class Type>
  #line 828 "src/data/Buffer.birch"
  void doSet(const Integer& t, const libbirch::DefaultArray<Type,1>& x);
  #line 835 "src/data/Buffer.birch"
  #line 835 "src/data/Buffer.birch"
  virtual void doSet(const libbirch::DefaultArray<Boolean,2>& x);
  #line 840 "src/data/Buffer.birch"
  #line 840 "src/data/Buffer.birch"
  virtual void doSet(const libbirch::DefaultArray<Integer,2>& x);
  #line 845 "src/data/Buffer.birch"
  #line 845 "src/data/Buffer.birch"
  virtual void doSet(const libbirch::DefaultArray<Real,2>& x);
  #line 850 "src/data/Buffer.birch"
  #line 850 "src/data/Buffer.birch"
  virtual void doPush(const Boolean& x);
  #line 872 "src/data/Buffer.birch"
  #line 872 "src/data/Buffer.birch"
  virtual void doPush(const Integer& x);
  #line 894 "src/data/Buffer.birch"
  #line 894 "src/data/Buffer.birch"
  virtual void doPush(const Real& x);
  #line 916 "src/data/Buffer.birch"
  #line 916 "src/data/Buffer.birch"
  virtual void doPush(const String& x);
  #line 920 "src/data/Buffer.birch"
  template<class Type>
  #line 920 "src/data/Buffer.birch"
  void doPush(const Type& x);
  #line 924 "src/data/Buffer.birch"
  template<class Type>
  #line 924 "src/data/Buffer.birch"
  void doPush(const Integer& t, const Type& x);
  #line 928 "src/data/Buffer.birch"
  #line 928 "src/data/Buffer.birch"
  virtual void doPush(const libbirch::DefaultArray<Boolean,1>& x);
  #line 944 "src/data/Buffer.birch"
  #line 944 "src/data/Buffer.birch"
  virtual void doPush(const libbirch::DefaultArray<Integer,1>& x);
  #line 962 "src/data/Buffer.birch"
  #line 962 "src/data/Buffer.birch"
  virtual void doPush(const libbirch::DefaultArray<Real,1>& x);
};

#line 1 "src/data/Buffer.birch"
extern "C" Buffer_* make_Buffer_();

#line 97 "src/delay/Delay.birch"
#line 97 "src/delay/Delay.birch"
class Delay_ : public Object_ {
public:
  #line 97 "src/delay/Delay.birch"
  LIBBIRCH_ABSTRACT_CLASS(Delay_, Object_)
  #line 97 "src/delay/Delay.birch"
  LIBBIRCH_CLASS_MEMBERS(child, coparent, n)
  #line 97 "src/delay/Delay.birch"
  using base_type_::operator=;
  
  #line 97 "src/delay/Delay.birch"
  Delay_();

  std::optional<Delay> child;
  std::optional<Delay> coparent;
  Integer n;
  #line 120 "src/delay/Delay.birch"
  #line 120 "src/delay/Delay.birch"
  virtual void graft(const Delay& node);
  #line 132 "src/delay/Delay.birch"
  #line 132 "src/delay/Delay.birch"
  virtual void graft(const Delay& node, const Integer& n);
  #line 143 "src/delay/Delay.birch"
  #line 143 "src/delay/Delay.birch"
  virtual void graftReplace(const Delay& node);
  #line 155 "src/delay/Delay.birch"
  #line 155 "src/delay/Delay.birch"
  virtual void graftReplace(const Delay& node, const Integer& n);
  #line 168 "src/delay/Delay.birch"
  #line 168 "src/delay/Delay.birch"
  virtual Delay prune();
  #line 180 "src/delay/Delay.birch"
  #line 180 "src/delay/Delay.birch"
  virtual Delay prune(const Integer& n);
  #line 188 "src/delay/Delay.birch"
  #line 188 "src/delay/Delay.birch"
  virtual Boolean isGrandChildOf(const Delay& node);
  #line 197 "src/delay/Delay.birch"
  #line 197 "src/delay/Delay.birch"
  virtual void join(const Delay& node);
};

#line 6 "src/distribution/Distribution.birch"
template<class Value>
#line 6 "src/distribution/Distribution.birch"
class Distribution_ : public libbirch::unwrap_pointer<Delay>::type {
public:
  #line 6 "src/distribution/Distribution.birch"
  using Value_ = Value;
  #line 6 "src/distribution/Distribution.birch"
  LIBBIRCH_ABSTRACT_CLASS(Distribution_, typename libbirch::unwrap_pointer<Delay>::type)
  #line 6 "src/distribution/Distribution.birch"
  LIBBIRCH_CLASS_MEMBERS(childValue)
  #line 6 "src/distribution/Distribution.birch"
  using base_type_::operator=;
  #line 6 "src/distribution/Distribution.birch"
  using base_type_::graft;
  #line 6 "src/distribution/Distribution.birch"
  using base_type_::graftReplace;
  #line 6 "src/distribution/Distribution.birch"
  using base_type_::prune;
  
  #line 6 "src/distribution/Distribution.birch"
  Distribution_();

  std::optional<Value> childValue;
  #line 19 "src/distribution/Distribution.birch"
  #line 19 "src/distribution/Distribution.birch"
  virtual Boolean supportsLazy();
  #line 28 "src/distribution/Distribution.birch"
  #line 28 "src/distribution/Distribution.birch"
  virtual Value simulate() = 0;
  #line 35 "src/distribution/Distribution.birch"
  #line 35 "src/distribution/Distribution.birch"
  virtual std::optional<Value> simulateLazy();
  #line 46 "src/distribution/Distribution.birch"
  #line 46 "src/distribution/Distribution.birch"
  virtual Real logpdf(const Value& x) = 0;
  #line 56 "src/distribution/Distribution.birch"
  #line 56 "src/distribution/Distribution.birch"
  virtual std::optional<Expression<Real>> logpdfLazy(const Expression<Value>& x);
  #line 65 "src/distribution/Distribution.birch"
  #line 65 "src/distribution/Distribution.birch"
  virtual std::optional<Delay> update(const Value& x);
  #line 74 "src/distribution/Distribution.birch"
  #line 74 "src/distribution/Distribution.birch"
  virtual std::optional<Delay> updateLazy(const Expression<Value>& x);
  #line 85 "src/distribution/Distribution.birch"
  #line 85 "src/distribution/Distribution.birch"
  virtual Real pdf(const Value& x);
  #line 96 "src/distribution/Distribution.birch"
  #line 96 "src/distribution/Distribution.birch"
  virtual std::optional<Real> cdf(const Value& x);
  #line 107 "src/distribution/Distribution.birch"
  #line 107 "src/distribution/Distribution.birch"
  virtual std::optional<Value> quantile(const Real& P);
  #line 114 "src/distribution/Distribution.birch"
  #line 114 "src/distribution/Distribution.birch"
  virtual std::optional<Value> lower();
  #line 121 "src/distribution/Distribution.birch"
  #line 121 "src/distribution/Distribution.birch"
  virtual std::optional<Value> upper();
  #line 125 "src/distribution/Distribution.birch"
  #line 125 "src/distribution/Distribution.birch"
  virtual void graft(const Delay& node) override;
  #line 130 "src/distribution/Distribution.birch"
  #line 130 "src/distribution/Distribution.birch"
  virtual void graftReplace(const Delay& node) override;
  #line 139 "src/distribution/Distribution.birch"
  #line 139 "src/distribution/Distribution.birch"
  virtual void graft(const Value& childValue);
  #line 145 "src/distribution/Distribution.birch"
  #line 145 "src/distribution/Distribution.birch"
  virtual Delay prune() override;
  #line 160 "src/distribution/Distribution.birch"
  #line 160 "src/distribution/Distribution.birch"
  virtual Boolean isGaussian();
  #line 164 "src/distribution/Distribution.birch"
  #line 164 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<Real>, Expression<Real>>> getGaussian();
  #line 168 "src/distribution/Distribution.birch"
  #line 168 "src/distribution/Distribution.birch"
  virtual Boolean isBeta();
  #line 172 "src/distribution/Distribution.birch"
  #line 172 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<Real>, Expression<Real>>> getBeta();
  #line 176 "src/distribution/Distribution.birch"
  #line 176 "src/distribution/Distribution.birch"
  virtual Boolean isGamma();
  #line 180 "src/distribution/Distribution.birch"
  #line 180 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<Real>, Expression<Real>>> getGamma();
  #line 184 "src/distribution/Distribution.birch"
  #line 184 "src/distribution/Distribution.birch"
  virtual Boolean isInverseGamma();
  #line 188 "src/distribution/Distribution.birch"
  #line 188 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<Real>, Expression<Real>>> getInverseGamma();
  #line 192 "src/distribution/Distribution.birch"
  #line 192 "src/distribution/Distribution.birch"
  virtual Boolean isInverseWishart();
  #line 196 "src/distribution/Distribution.birch"
  #line 196 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>>> getInverseWishart();
  #line 200 "src/distribution/Distribution.birch"
  #line 200 "src/distribution/Distribution.birch"
  virtual Boolean isNormalInverseGamma();
  #line 204 "src/distribution/Distribution.birch"
  #line 204 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<Real>, Expression<Real>, Expression<Real>, Expression<Real>>> getNormalInverseGamma();
  #line 209 "src/distribution/Distribution.birch"
  #line 209 "src/distribution/Distribution.birch"
  virtual Boolean isDirichlet();
  #line 213 "src/distribution/Distribution.birch"
  #line 213 "src/distribution/Distribution.birch"
  virtual std::optional<Expression<libbirch::DefaultArray<Real,1>>> getDirichlet();
  #line 217 "src/distribution/Distribution.birch"
  #line 217 "src/distribution/Distribution.birch"
  virtual Boolean isRestaurant();
  #line 221 "src/distribution/Distribution.birch"
  #line 221 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<Real>, Expression<Real>, Expression<libbirch::DefaultArray<Integer,1>>, Expression<Integer>>> getRestaurant();
  #line 226 "src/distribution/Distribution.birch"
  #line 226 "src/distribution/Distribution.birch"
  virtual Boolean isMultivariateGaussian();
  #line 230 "src/distribution/Distribution.birch"
  #line 230 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>>> getMultivariateGaussian();
  #line 235 "src/distribution/Distribution.birch"
  #line 235 "src/distribution/Distribution.birch"
  virtual Boolean isMultivariateNormalInverseGamma();
  #line 239 "src/distribution/Distribution.birch"
  #line 239 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>, Expression<Real>>> getMultivariateNormalInverseGamma();
  #line 244 "src/distribution/Distribution.birch"
  #line 244 "src/distribution/Distribution.birch"
  virtual Boolean isMatrixGaussian();
  #line 248 "src/distribution/Distribution.birch"
  #line 248 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>>> getMatrixGaussian();
  #line 253 "src/distribution/Distribution.birch"
  #line 253 "src/distribution/Distribution.birch"
  virtual Boolean isMatrixNormalInverseWishart();
  #line 257 "src/distribution/Distribution.birch"
  #line 257 "src/distribution/Distribution.birch"
  virtual std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>>> getMatrixNormalInverseWishart();
  #line 262 "src/distribution/Distribution.birch"
  #line 262 "src/distribution/Distribution.birch"
  virtual Boolean isDiscrete();
  #line 266 "src/distribution/Distribution.birch"
  #line 266 "src/distribution/Distribution.birch"
  virtual std::optional<DiscreteDistribution> getDiscrete();
  #line 270 "src/distribution/Distribution.birch"
  #line 270 "src/distribution/Distribution.birch"
  virtual Boolean isBoundedDiscrete();
  #line 274 "src/distribution/Distribution.birch"
  #line 274 "src/distribution/Distribution.birch"
  virtual std::optional<BoundedDiscreteDistribution> getBoundedDiscrete();
};

#line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
class MatrixGaussianDistribution_ : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type {
public:
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  LIBBIRCH_CLASS(MatrixGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type)
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(M, U, V)
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using base_type_::operator=;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using base_type_::getMatrixGaussian;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using base_type_::isMatrixGaussian;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using base_type_::simulate;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  using base_type_::write;
  
  #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  MatrixGaussianDistribution_(const Arg1& M, const Arg2& U, const Arg3& V);

  Arg1 M;
  Arg2 U;
  Arg3 V;
  #line 21 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  #line 21 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  virtual Boolean supportsLazy() override;
  #line 25 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  #line 25 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  virtual libbirch::DefaultArray<Real,2> simulate() override;
  #line 29 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  #line 29 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  virtual std::optional<libbirch::DefaultArray<Real,2>> simulateLazy() override;
  #line 33 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  #line 33 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  virtual Real logpdf(const libbirch::DefaultArray<Real,2>& X) override;
  #line 37 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  #line 37 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  virtual std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
  #line 41 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  #line 41 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  virtual Boolean isMatrixGaussian() override;
  #line 45 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  #line 45 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  virtual std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>>> getMatrixGaussian() override;
  #line 50 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  #line 50 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  virtual void write(const Buffer& buffer) override;
};

#line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
class LinearMatrixNormalInverseWishartMatrixGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type {
public:
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg6_ = Arg6;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg7_ = Arg7;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  LIBBIRCH_CLASS(LinearMatrixNormalInverseWishartMatrixGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type)
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(A, N, \u039b, \u03a8, k, C, U)
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::operator=;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::simulate;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::update;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  LinearMatrixNormalInverseWishartMatrixGaussianDistribution_(const Arg1& A, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& C, const Arg7& U);

  Arg1 A;
  Arg2 N;
  Arg3 \u039b;
  Arg4 \u03a8;
  Arg5 k;
  Arg6 C;
  Arg7 U;
  #line 43 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 43 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 47 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 47 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  libbirch::DefaultArray<Real,2> simulate() override;
  #line 53 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 53 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,2>> simulateLazy() override;
  #line 58 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 58 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,2>& X) override;
  #line 64 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 64 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
  #line 69 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 69 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,2>& X) override;
  #line 75 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 75 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
};

#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
class MatrixNormalInverseWishartDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type {
public:
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  LIBBIRCH_CLASS(MatrixNormalInverseWishartDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type)
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(N, \u039b, \u03a8, k)
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::getMatrixNormalInverseWishart;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::isMatrixNormalInverseWishart;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::updateLazy;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  using base_type_::write;
  
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  MatrixNormalInverseWishartDistribution_(const Arg1& N, const Arg2& \u039b, const Arg3& \u03a8, const Arg4& k);

  Arg1 N;
  Arg2 \u039b;
  Arg3 \u03a8;
  Arg4 k;
  #line 26 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  #line 26 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  Boolean supportsLazy() override;
  #line 30 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  #line 30 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  libbirch::DefaultArray<Real,2> simulate() override;
  #line 35 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  #line 35 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,2>> simulateLazy() override;
  #line 40 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  #line 40 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,2>& X) override;
  #line 45 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  #line 45 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
  #line 49 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  #line 49 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,2>& X) override;
  #line 54 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  #line 54 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
  #line 59 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  #line 59 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  Boolean isMatrixNormalInverseWishart() override;
  #line 63 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  #line 63 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>>> getMatrixNormalInverseWishart() override;
  #line 68 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  #line 68 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
class MatrixNormalInverseWishartMatrixGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type {
public:
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  LIBBIRCH_CLASS(MatrixNormalInverseWishartMatrixGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type)
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(N, \u039b, \u03a8, k, U)
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  MatrixNormalInverseWishartMatrixGaussianDistribution_(const Arg1& N, const Arg2& \u039b, const Arg3& \u03a8, const Arg4& k, const Arg5& U);

  Arg1 N;
  Arg2 \u039b;
  Arg3 \u03a8;
  Arg4 k;
  Arg5 U;
  #line 31 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 31 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 35 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 35 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  libbirch::DefaultArray<Real,2> simulate() override;
  #line 40 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 40 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,2>> simulateLazy() override;
  #line 45 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 45 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,2>& X) override;
  #line 50 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 50 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
  #line 55 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 55 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,2>& X) override;
  #line 60 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 60 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
};

#line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
class ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type {
public:
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using Arg6_ = Arg6;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  LIBBIRCH_CLASS(ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type)
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(A, N, \u039b, \u03a8, k, U)
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::operator=;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::simulate;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::update;
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_(const Arg1& A, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& U);

  Arg1 A;
  Arg2 N;
  Arg3 \u039b;
  Arg4 \u03a8;
  Arg5 k;
  Arg6 U;
  #line 38 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 38 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 42 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 42 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  libbirch::DefaultArray<Real,2> simulate() override;
  #line 47 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 47 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,2>> simulateLazy() override;
  #line 52 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 52 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,2>& X) override;
  #line 57 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 57 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
  #line 62 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 62 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,2>& X) override;
  #line 67 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  #line 67 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
};

#line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
class InverseWishartDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type {
public:
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using Arg1_ = Arg1;
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using Arg2_ = Arg2;
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  LIBBIRCH_CLASS(InverseWishartDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type)
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03a8, k)
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using base_type_::operator=;
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using base_type_::getInverseWishart;
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using base_type_::isInverseWishart;
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using base_type_::logpdf;
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using base_type_::logpdfLazy;
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using base_type_::simulate;
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using base_type_::simulateLazy;
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using base_type_::supportsLazy;
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  using base_type_::write;
  
  #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  InverseWishartDistribution_(const Arg1& \u03a8, const Arg2& k);

  Arg1 \u03a8;
  Arg2 k;
  #line 42 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  #line 42 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  Boolean supportsLazy() override;
  #line 46 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  #line 46 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  libbirch::DefaultArray<Real,2> simulate() override;
  #line 50 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  #line 50 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,2>> simulateLazy() override;
  #line 54 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  #line 54 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,2>& X) override;
  #line 58 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  #line 58 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
  #line 62 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  #line 62 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  Boolean isInverseWishart() override;
  #line 66 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  #line 66 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>>> getInverseWishart() override;
  #line 71 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  #line 71 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/matrix_other/WishartDistribution.birch"
class WishartDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type {
public:
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  LIBBIRCH_CLASS(WishartDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,2>>>::type)
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03a8, k)
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  using base_type_::write;
  
  #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
  WishartDistribution_(const Arg1& \u03a8, const Arg2& k);

  Arg1 \u03a8;
  Arg2 k;
  #line 15 "src/distribution/matrix_other/WishartDistribution.birch"
  #line 15 "src/distribution/matrix_other/WishartDistribution.birch"
  Boolean supportsLazy() override;
  #line 19 "src/distribution/matrix_other/WishartDistribution.birch"
  #line 19 "src/distribution/matrix_other/WishartDistribution.birch"
  libbirch::DefaultArray<Real,2> simulate() override;
  #line 23 "src/distribution/matrix_other/WishartDistribution.birch"
  #line 23 "src/distribution/matrix_other/WishartDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,2>> simulateLazy() override;
  #line 27 "src/distribution/matrix_other/WishartDistribution.birch"
  #line 27 "src/distribution/matrix_other/WishartDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,2>& X) override;
  #line 31 "src/distribution/matrix_other/WishartDistribution.birch"
  #line 31 "src/distribution/matrix_other/WishartDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) override;
  #line 35 "src/distribution/matrix_other/WishartDistribution.birch"
  #line 35 "src/distribution/matrix_other/WishartDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
class DirichletMultinomialDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Integer,1>>>::type {
public:
  #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  LIBBIRCH_CLASS(DirichletMultinomialDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Integer,1>>>::type)
  #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(n, \u03b1)
  #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  using base_type_::update;
  
  #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  DirichletMultinomialDistribution_(const Arg1& n, const Arg2& \u03b1);

  Arg1 n;
  Arg2 \u03b1;
  #line 16 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  #line 16 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  Boolean supportsLazy() override;
  #line 20 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  #line 20 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  libbirch::DefaultArray<Integer,1> simulate() override;
  #line 24 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  #line 24 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Integer,1>& x) override;
  #line 28 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  #line 28 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Integer,1>& x) override;
};

#line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
class MultinomialDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Integer,1>>>::type {
public:
  #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  LIBBIRCH_CLASS(MultinomialDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Integer,1>>>::type)
  #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(n, \u03c1)
  #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  using base_type_::write;
  
  #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  MultinomialDistribution_(const Arg1& n, const Arg2& \u03c1);

  Arg1 n;
  Arg2 \u03c1;
  #line 16 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  #line 16 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  Boolean supportsLazy() override;
  #line 20 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  #line 20 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  libbirch::DefaultArray<Integer,1> simulate() override;
  #line 24 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  #line 24 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Integer,1>& x) override;
  #line 28 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  #line 28 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
class LinearMultivariateGaussianMultivariateGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS(LinearMultivariateGaussianMultivariateGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(A, \u03bc, \u03a3, c, S)
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::getMultivariateGaussian;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::isMultivariateGaussian;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  LinearMultivariateGaussianMultivariateGaussianDistribution_(const Arg1& A, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& c, const Arg5& S);

  Arg1 A;
  Arg2 \u03bc;
  Arg3 \u03a3;
  Arg4 c;
  Arg5 S;
  #line 32 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 32 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 36 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 36 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 41 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 41 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 46 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 46 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 51 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 51 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 56 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 56 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,1>& x) override;
  #line 61 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 61 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 66 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 66 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  Boolean isMultivariateGaussian() override;
  #line 70 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 70 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>>> getMultivariateGaussian() override;
};

#line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
class MultivariateGaussianDistribution_ : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS(MultivariateGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bc, \u03a3)
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using base_type_::operator=;
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using base_type_::getMultivariateGaussian;
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using base_type_::isMultivariateGaussian;
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using base_type_::simulate;
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  using base_type_::write;
  
  #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  MultivariateGaussianDistribution_(const Arg1& \u03bc, const Arg2& \u03a3);

  Arg1 \u03bc;
  Arg2 \u03a3;
  #line 16 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  #line 16 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  virtual Boolean supportsLazy() override;
  #line 20 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  #line 20 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  virtual libbirch::DefaultArray<Real,1> simulate() override;
  #line 24 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  #line 24 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  virtual std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 28 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  #line 28 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  virtual Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 32 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  #line 32 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  virtual std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 36 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  #line 36 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  virtual Boolean isMultivariateGaussian() override;
  #line 40 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  #line 40 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  virtual std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>>> getMultivariateGaussian() override;
  #line 45 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  #line 45 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  virtual void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
class MultivariateGaussianMultivariateGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS(MultivariateGaussianMultivariateGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bc, \u03a3, S)
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::getMultivariateGaussian;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::isMultivariateGaussian;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  MultivariateGaussianMultivariateGaussianDistribution_(const Arg1& \u03bc, const Arg2& \u03a3, const Arg3& S);

  Arg1 \u03bc;
  Arg2 \u03a3;
  Arg3 S;
  #line 21 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 21 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 25 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 25 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 30 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 30 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 35 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 35 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 40 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 40 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 45 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 45 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,1>& x) override;
  #line 50 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 50 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 54 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 54 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  Boolean isMultivariateGaussian() override;
  #line 58 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 58 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>>> getMultivariateGaussian() override;
};

#line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
class ScaledMultivariateGaussianMultivariateGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS(ScaledMultivariateGaussianMultivariateGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(A, \u03bc, \u03a3, S)
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::getMultivariateGaussian;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::isMultivariateGaussian;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  ScaledMultivariateGaussianMultivariateGaussianDistribution_(const Arg1& A, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& S);

  Arg1 A;
  Arg2 \u03bc;
  Arg3 \u03a3;
  Arg4 S;
  #line 26 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 26 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 30 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 30 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 35 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 35 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 40 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 40 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 45 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 45 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 50 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 50 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,1>& x) override;
  #line 55 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 55 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 60 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 60 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  Boolean isMultivariateGaussian() override;
  #line 64 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  #line 64 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>>> getMultivariateGaussian() override;
};

#line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
class LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg6_ = Arg6;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg7_ = Arg7;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS(LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(A, \u03bd, \u039b, \u03b1, \u03b2, c, \u03a3)
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::operator=;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::simulate;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::update;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_(const Arg1& A, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& \u03a3);

  Arg1 A;
  Arg2 \u03bd;
  Arg3 \u039b;
  Arg4 \u03b1;
  Arg5 \u03b2;
  Arg6 c;
  Arg7 \u03a3;
  #line 43 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 43 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 47 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 47 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 53 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 53 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 58 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 58 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 64 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 64 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 69 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 69 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,1>& x) override;
  #line 75 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 75 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
};

#line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
class MultivariateNormalInverseGammaDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using Arg1_ = Arg1;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using Arg2_ = Arg2;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using Arg3_ = Arg3;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using Arg4_ = Arg4;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  LIBBIRCH_CLASS(MultivariateNormalInverseGammaDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bd, \u039b, \u03b1, \u03b2)
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::operator=;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::getMultivariateNormalInverseGamma;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::isMultivariateNormalInverseGamma;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::logpdf;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::logpdfLazy;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::simulate;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::simulateLazy;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::supportsLazy;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::update;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::updateLazy;
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  using base_type_::write;
  
  #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  MultivariateNormalInverseGammaDistribution_(const Arg1& \u03bd, const Arg2& \u039b, const Arg3& \u03b1, const Arg4& \u03b2);

  Arg1 \u03bd;
  Arg2 \u039b;
  Arg3 \u03b1;
  Arg4 \u03b2;
  #line 57 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  #line 57 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  Boolean supportsLazy() override;
  #line 61 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  #line 61 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 66 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  #line 66 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 71 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  #line 71 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 76 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  #line 76 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 80 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  #line 80 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,1>& x) override;
  #line 85 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  #line 85 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 89 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  #line 89 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  Boolean isMultivariateNormalInverseGamma() override;
  #line 93 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  #line 93 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>, Expression<Real>>> getMultivariateNormalInverseGamma() override;
  #line 98 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  #line 98 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
class MultivariateNormalInverseGammaMultivariateGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS(MultivariateNormalInverseGammaMultivariateGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bd, \u039b, \u03b1, \u03b2, \u03a3)
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  MultivariateNormalInverseGammaMultivariateGaussianDistribution_(const Arg1& \u03bd, const Arg2& \u039b, const Arg3& \u03b1, const Arg4& \u03b2, const Arg5& \u03a3);

  Arg1 \u03bd;
  Arg2 \u039b;
  Arg3 \u03b1;
  Arg4 \u03b2;
  Arg5 \u03a3;
  #line 32 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 32 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 36 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 36 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 41 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 41 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 46 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 46 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 51 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 51 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 56 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 56 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,1>& x) override;
  #line 61 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 61 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
};

#line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
class ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using Arg6_ = Arg6;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS(ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(A, \u03bd, \u039b, \u03b1, \u03b2, \u03a3)
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::operator=;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::simulate;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::update;
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_(const Arg1& A, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& \u03a3);

  Arg1 A;
  Arg2 \u03bd;
  Arg3 \u039b;
  Arg4 \u03b1;
  Arg5 \u03b2;
  Arg6 \u03a3;
  #line 38 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 38 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 42 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 42 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 47 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 47 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 52 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 52 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 57 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 57 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 62 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 62 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,1>& x) override;
  #line 67 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  #line 67 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
};

#line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
class LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg6_ = Arg6;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS(LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(a, N, \u039b, \u03a8, k, c)
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::operator=;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::simulate;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::update;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_(const Arg1& a, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& c);

  Arg1 a;
  Arg2 N;
  Arg3 \u039b;
  Arg4 \u03a8;
  Arg5 k;
  Arg6 c;
  #line 38 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 38 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 42 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 42 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 47 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 47 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 52 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 52 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 57 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 57 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 62 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 62 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,1>& x) override;
  #line 67 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 67 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
};

#line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
class ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS(ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(a, N, \u039b, \u03a8, k)
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::operator=;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::simulate;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::update;
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_(const Arg1& a, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k);

  Arg1 a;
  Arg2 N;
  Arg3 \u039b;
  Arg4 \u03a8;
  Arg5 k;
  #line 33 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 33 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 37 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 37 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 42 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 42 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 47 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 47 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 52 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 52 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 57 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 57 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  std::optional<Delay> update(const libbirch::DefaultArray<Real,1>& x) override;
  #line 62 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  #line 62 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
};

#line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
class DirichletDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  using Arg_ = Arg;
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  LIBBIRCH_CLASS(DirichletDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03b1)
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  using base_type_::getDirichlet;
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  using base_type_::isDirichlet;
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  using base_type_::write;
  
  #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
  DirichletDistribution_(const Arg& \u03b1);

  Arg \u03b1;
  #line 10 "src/distribution/multivariate_other/DirichletDistribution.birch"
  #line 10 "src/distribution/multivariate_other/DirichletDistribution.birch"
  Boolean supportsLazy() override;
  #line 14 "src/distribution/multivariate_other/DirichletDistribution.birch"
  #line 14 "src/distribution/multivariate_other/DirichletDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 18 "src/distribution/multivariate_other/DirichletDistribution.birch"
  #line 18 "src/distribution/multivariate_other/DirichletDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 22 "src/distribution/multivariate_other/DirichletDistribution.birch"
  #line 22 "src/distribution/multivariate_other/DirichletDistribution.birch"
  Boolean isDirichlet() override;
  #line 26 "src/distribution/multivariate_other/DirichletDistribution.birch"
  #line 26 "src/distribution/multivariate_other/DirichletDistribution.birch"
  std::optional<Expression<libbirch::DefaultArray<Real,1>>> getDirichlet() override;
  #line 30 "src/distribution/multivariate_other/DirichletDistribution.birch"
  #line 30 "src/distribution/multivariate_other/DirichletDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
class RestaurantDistribution_ final : public libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type {
public:
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  LIBBIRCH_CLASS(RestaurantDistribution_, typename libbirch::unwrap_pointer<Distribution<libbirch::DefaultArray<Real,1>>>::type)
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03b1, \u03b8, n, N)
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using base_type_::getRestaurant;
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using base_type_::isRestaurant;
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  RestaurantDistribution_(const Arg1& \u03b1, const Arg2& \u03b8);

  Arg1 \u03b1;
  Arg2 \u03b8;
  libbirch::DefaultArray<Integer,1> n;
  Integer N;
  #line 29 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  #line 29 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  Boolean supportsLazy() override;
  #line 33 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  #line 33 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  libbirch::DefaultArray<Real,1> simulate() override;
  #line 38 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  #line 38 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  std::optional<libbirch::DefaultArray<Real,1>> simulateLazy() override;
  #line 43 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  #line 43 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  Real logpdf(const libbirch::DefaultArray<Real,1>& x) override;
  #line 48 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  #line 48 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) override;
  #line 53 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  #line 53 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  Boolean isRestaurant() override;
  #line 57 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  #line 57 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  std::optional<std::tuple<Expression<Real>, Expression<Real>, Expression<libbirch::DefaultArray<Integer,1>>, Expression<Integer>>> getRestaurant() override;
  #line 62 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  #line 62 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
class BernoulliDistribution_ final : public libbirch::unwrap_pointer<Distribution<Boolean>>::type {
public:
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  using Arg_ = Arg;
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  LIBBIRCH_CLASS(BernoulliDistribution_, typename libbirch::unwrap_pointer<Distribution<Boolean>>::type)
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c1)
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  using base_type_::write;
  
  #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  BernoulliDistribution_(const Arg& \u03c1);

  Arg \u03c1;
  #line 10 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  #line 10 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  Boolean supportsLazy() override;
  #line 14 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  #line 14 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  Boolean simulate() override;
  #line 18 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  #line 18 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  std::optional<Boolean> simulateLazy() override;
  #line 22 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  #line 22 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  Real logpdf(const Boolean& x) override;
  #line 26 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  #line 26 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Boolean>& x) override;
  #line 30 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  #line 30 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
class BetaBernoulliDistribution_ final : public libbirch::unwrap_pointer<Distribution<Boolean>>::type {
public:
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  LIBBIRCH_CLASS(BetaBernoulliDistribution_, typename libbirch::unwrap_pointer<Distribution<Boolean>>::type)
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03b1, \u03b2)
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  BetaBernoulliDistribution_(const Arg1& \u03b1, const Arg2& \u03b2);

  Arg1 \u03b1;
  Arg2 \u03b2;
  #line 15 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  #line 15 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  Boolean supportsLazy() override;
  #line 19 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  #line 19 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  Boolean simulate() override;
  #line 23 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  #line 23 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  std::optional<Boolean> simulateLazy() override;
  #line 27 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  #line 27 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  Real logpdf(const Boolean& x) override;
  #line 31 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  #line 31 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Boolean>& x) override;
  #line 35 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  #line 35 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  std::optional<Delay> update(const Boolean& x) override;
  #line 39 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  #line 39 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Boolean>& x) override;
};

#line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
class DirichletCategoricalDistribution_ final : public libbirch::unwrap_pointer<Distribution<Integer>>::type {
public:
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  using Arg_ = Arg;
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  LIBBIRCH_CLASS(DirichletCategoricalDistribution_, typename libbirch::unwrap_pointer<Distribution<Integer>>::type)
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03b1)
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  using base_type_::lower;
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  using base_type_::upper;
  
  #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  DirichletCategoricalDistribution_(const Arg& \u03b1);

  Arg \u03b1;
  #line 10 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  #line 10 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  Boolean supportsLazy() override;
  #line 14 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  #line 14 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  Integer simulate() override;
  #line 18 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  #line 18 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 22 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  #line 22 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 26 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  #line 26 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  std::optional<Integer> lower() override;
  #line 30 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  #line 30 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  std::optional<Integer> upper() override;
};

#line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
#line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
class DiscreteDistribution_ : public libbirch::unwrap_pointer<Distribution<Integer>>::type {
public:
  #line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  LIBBIRCH_ABSTRACT_CLASS(DiscreteDistribution_, typename libbirch::unwrap_pointer<Distribution<Integer>>::type)
  #line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  using base_type_::getDiscrete;
  #line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  using base_type_::isDiscrete;
  
  #line 4 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  DiscreteDistribution_();

  #line 5 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  #line 5 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  virtual Boolean isDiscrete() override;
  #line 9 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  #line 9 "src/distribution/scalar_discrete/DiscreteDistribution.birch"
  virtual std::optional<DiscreteDistribution> getDiscrete() override;
};

#line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
#line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
class BoundedDiscreteDistribution_ : public libbirch::unwrap_pointer<DiscreteDistribution>::type {
public:
  #line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  LIBBIRCH_ABSTRACT_CLASS(BoundedDiscreteDistribution_, typename libbirch::unwrap_pointer<DiscreteDistribution>::type)
  #line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  using base_type_::getBoundedDiscrete;
  #line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  using base_type_::isBoundedDiscrete;
  
  #line 4 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  BoundedDiscreteDistribution_();

  #line 5 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  #line 5 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  virtual Boolean isBoundedDiscrete() override;
  #line 9 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  #line 9 "src/distribution/scalar_discrete/BoundedDiscreteDistribution.birch"
  virtual std::optional<BoundedDiscreteDistribution> getBoundedDiscrete() override;
};

#line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
class BinomialDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  LIBBIRCH_CLASS(BinomialDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(n, \u03c1)
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::upper;
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  BinomialDistribution_(const Arg1& n, const Arg2& \u03c1);

  Arg1 n;
  Arg2 \u03c1;
  #line 20 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  #line 20 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  Boolean supportsLazy() override;
  #line 24 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  #line 24 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  Integer simulate() override;
  #line 28 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  #line 28 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  std::optional<Integer> simulateLazy() override;
  #line 32 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  #line 32 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 36 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  #line 36 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Integer>& x) override;
  #line 40 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  #line 40 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  std::optional<Real> cdf(const Integer& x) override;
  #line 44 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  #line 44 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  std::optional<Integer> quantile(const Real& P) override;
  #line 48 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  #line 48 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  std::optional<Integer> lower() override;
  #line 52 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  #line 52 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  std::optional<Integer> upper() override;
  #line 56 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  #line 56 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
class BetaBinomialDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  LIBBIRCH_CLASS(BetaBinomialDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(n, \u03b1, \u03b2)
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::cdf;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::lower;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::updateLazy;
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  using base_type_::upper;
  
  #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  BetaBinomialDistribution_(const Arg1& n, const Arg2& \u03b1, const Arg3& \u03b2);

  Arg1 n;
  Arg2 \u03b1;
  Arg3 \u03b2;
  #line 21 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  #line 21 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  Boolean supportsLazy() override;
  #line 25 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  #line 25 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  Integer simulate() override;
  #line 29 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  #line 29 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  std::optional<Integer> simulateLazy() override;
  #line 33 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  #line 33 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 37 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  #line 37 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Integer>& x) override;
  #line 41 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  #line 41 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 45 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  #line 45 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Integer>& x) override;
  #line 49 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  #line 49 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  std::optional<Real> cdf(const Integer& x) override;
  #line 53 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  #line 53 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  std::optional<Integer> lower() override;
  #line 57 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  #line 57 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  std::optional<Integer> upper() override;
};

#line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
class CategoricalDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  using Arg_ = Arg;
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  LIBBIRCH_CLASS(CategoricalDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c1)
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  using base_type_::operator=;
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  using base_type_::cdf;
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  using base_type_::logpdf;
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  using base_type_::lower;
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  using base_type_::quantile;
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  using base_type_::simulate;
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  using base_type_::supportsLazy;
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  using base_type_::upper;
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  using base_type_::write;
  
  #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  CategoricalDistribution_(const Arg& \u03c1);

  Arg \u03c1;
  #line 13 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  #line 13 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  Boolean supportsLazy() override;
  #line 17 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  #line 17 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  Integer simulate() override;
  #line 21 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  #line 21 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 25 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  #line 25 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  std::optional<Real> cdf(const Integer& x) override;
  #line 29 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  #line 29 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  std::optional<Integer> quantile(const Real& P) override;
  #line 33 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  #line 33 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  std::optional<Integer> lower() override;
  #line 37 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  #line 37 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  std::optional<Integer> upper() override;
  #line 41 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  #line 41 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
class ConwayMaxwellPoissonDistribution_ : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using Arg1_ = Arg1;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using Arg2_ = Arg2;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using Arg3_ = Arg3;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  LIBBIRCH_CLASS(ConwayMaxwellPoissonDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bb, \u03bd, n)
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::operator=;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::cdf;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::logpdf;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::logpdfLazy;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::lower;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::quantile;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::simulate;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::simulateLazy;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::supportsLazy;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::upper;
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  using base_type_::write;
  
  #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  ConwayMaxwellPoissonDistribution_(const Arg1& \u03bb, const Arg2& \u03bd, const Arg3& n);

  Arg1 \u03bb;
  Arg2 \u03bd;
  Arg3 n;
  #line 31 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  #line 31 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  virtual Boolean supportsLazy() override;
  #line 35 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  #line 35 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  virtual Integer simulate() override;
  #line 39 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  #line 39 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  virtual std::optional<Integer> simulateLazy() override;
  #line 43 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  #line 43 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  virtual Real logpdf(const Integer& x) override;
  #line 47 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  #line 47 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  virtual std::optional<Expression<Real>> logpdfLazy(const Expression<Integer>& x) override;
  #line 51 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  #line 51 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  virtual std::optional<Real> cdf(const Integer& x) override;
  #line 55 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  #line 55 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  virtual std::optional<Integer> quantile(const Real& P) override;
  #line 59 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  #line 59 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  virtual std::optional<Integer> lower() override;
  #line 63 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  #line 63 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  virtual std::optional<Integer> upper() override;
  #line 67 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  #line 67 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  virtual void write(const Buffer& buffer) override;
};

#line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
class UniformIntegerDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  LIBBIRCH_CLASS(UniformIntegerDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(l, u)
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using base_type_::upper;
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  UniformIntegerDistribution_(const Arg1& l, const Arg2& u);

  Arg1 l;
  Arg2 u;
  #line 20 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  #line 20 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  Boolean supportsLazy() override;
  #line 24 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  #line 24 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  Integer simulate() override;
  #line 28 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  #line 28 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 32 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  #line 32 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  std::optional<Real> cdf(const Integer& x) override;
  #line 36 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  #line 36 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  std::optional<Integer> quantile(const Real& P) override;
  #line 40 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  #line 40 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  std::optional<Integer> lower() override;
  #line 44 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  #line 44 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  std::optional<Integer> upper() override;
  #line 48 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  #line 48 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
#line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
class AddDiscreteConstrainedDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  LIBBIRCH_CLASS(AddDiscreteConstrainedDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(p, l, y)
  #line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  using base_type_::operator=;
  #line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  using base_type_::logpdf;
  #line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  using base_type_::lower;
  #line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  using base_type_::simulate;
  #line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  using base_type_::update;
  #line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  using base_type_::upper;
  
  #line 7 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  AddDiscreteConstrainedDistribution_(const libbirch::DefaultArray<Real,1>& p, const Integer& l, const Integer& y);

  libbirch::DefaultArray<Real,1> p;
  Integer l;
  Integer y;
  #line 24 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  #line 24 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  Integer simulate() override;
  #line 28 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  #line 28 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 32 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  #line 32 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 36 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  #line 36 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  std::optional<Integer> lower() override;
  #line 40 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  #line 40 "src/distribution/scalar_enumerate/AddDiscreteConstrainedDistribution.birch"
  std::optional<Integer> upper() override;
};

#line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
#line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
class AddDiscreteDeltaDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  LIBBIRCH_CLASS(AddDiscreteDeltaDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(p, q, x, l, u, z, Z)
  #line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  using base_type_::lower;
  #line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  using base_type_::upper;
  
  #line 4 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  AddDiscreteDeltaDistribution_(const BoundedDiscreteDistribution& p, const BoundedDiscreteDistribution& q);

  BoundedDiscreteDistribution p;
  BoundedDiscreteDistribution q;
  std::optional<Integer> x;
  Integer l;
  Integer u;
  libbirch::DefaultArray<Real,1> z;
  Real Z;
  #line 42 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  #line 42 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  void enumerate(const Integer& x);
  #line 62 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  #line 62 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  Integer simulate() override;
  #line 66 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  #line 66 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 71 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  #line 71 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 76 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  #line 76 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  std::optional<Integer> lower() override;
  #line 80 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  #line 80 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  std::optional<Integer> upper() override;
};

#line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
class DeltaDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  using Arg_ = Arg;
  #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  LIBBIRCH_CLASS(DeltaDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bc)
  #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  using base_type_::operator=;
  #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  using base_type_::logpdf;
  #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  using base_type_::lower;
  #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  using base_type_::simulate;
  #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  using base_type_::upper;
  #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  using base_type_::write;
  
  #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  DeltaDistribution_(const Arg& \u03bc);

  Arg \u03bc;
  #line 13 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  #line 13 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  Integer simulate() override;
  #line 17 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  #line 17 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 21 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  #line 21 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  std::optional<Integer> lower() override;
  #line 25 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  #line 25 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  std::optional<Integer> upper() override;
  #line 29 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  #line 29 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
#line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
class DiscreteDeltaDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  LIBBIRCH_CLASS(DiscreteDeltaDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(p)
  #line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  using base_type_::operator=;
  #line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  using base_type_::logpdf;
  #line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  using base_type_::lower;
  #line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  using base_type_::simulate;
  #line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  using base_type_::update;
  #line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  using base_type_::upper;
  
  #line 6 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  DiscreteDeltaDistribution_(const BoundedDiscreteDistribution& p);

  BoundedDiscreteDistribution p;
  #line 13 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  #line 13 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  Integer simulate() override;
  #line 17 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  #line 17 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 21 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  #line 21 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 25 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  #line 25 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  std::optional<Integer> lower() override;
  #line 29 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  #line 29 "src/distribution/scalar_enumerate/DiscreteDeltaDistribution.birch"
  std::optional<Integer> upper() override;
};

#line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
#line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
class NegateDiscreteDeltaDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  LIBBIRCH_CLASS(NegateDiscreteDeltaDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(p)
  #line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  using base_type_::operator=;
  #line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  using base_type_::logpdf;
  #line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  using base_type_::lower;
  #line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  using base_type_::simulate;
  #line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  using base_type_::update;
  #line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  using base_type_::upper;
  
  #line 7 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  NegateDiscreteDeltaDistribution_(const BoundedDiscreteDistribution& p);

  BoundedDiscreteDistribution p;
  #line 14 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  #line 14 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  Integer simulate() override;
  #line 18 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  #line 18 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 22 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  #line 22 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 26 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  #line 26 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  std::optional<Integer> lower() override;
  #line 35 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  #line 35 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  std::optional<Integer> upper() override;
};

#line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
#line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
class SubtractDiscreteConstrainedDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  LIBBIRCH_CLASS(SubtractDiscreteConstrainedDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(p, l, y)
  #line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  using base_type_::update;
  #line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  using base_type_::upper;
  
  #line 8 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  SubtractDiscreteConstrainedDistribution_(const libbirch::DefaultArray<Real,1>& p, const Integer& l, const Integer& y);

  libbirch::DefaultArray<Real,1> p;
  Integer l;
  Integer y;
  #line 25 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  #line 25 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  Integer simulate() override;
  #line 29 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  #line 29 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 33 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  #line 33 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 37 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  #line 37 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  std::optional<Integer> lower() override;
  #line 41 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  #line 41 "src/distribution/scalar_enumerate/SubtractDiscreteConstrainedDistribution.birch"
  std::optional<Integer> upper() override;
};

#line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
#line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
class SubtractDiscreteDeltaDistribution_ final : public libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type {
public:
  #line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  LIBBIRCH_CLASS(SubtractDiscreteDeltaDistribution_, typename libbirch::unwrap_pointer<BoundedDiscreteDistribution>::type)
  #line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(p, q, x, l, u, z, Z)
  #line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  using base_type_::lower;
  #line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  using base_type_::upper;
  
  #line 4 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  SubtractDiscreteDeltaDistribution_(const BoundedDiscreteDistribution& p, const BoundedDiscreteDistribution& q);

  BoundedDiscreteDistribution p;
  BoundedDiscreteDistribution q;
  std::optional<Integer> x;
  Integer l;
  Integer u;
  libbirch::DefaultArray<Real,1> z;
  Real Z;
  #line 42 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  #line 42 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  void enumerate(const Integer& x);
  #line 60 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  #line 60 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  Integer simulate() override;
  #line 64 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  #line 64 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 69 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  #line 69 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 74 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  #line 74 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  std::optional<Integer> lower() override;
  #line 78 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  #line 78 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  std::optional<Integer> upper() override;
};

#line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
class BetaNegativeBinomialDistribution_ final : public libbirch::unwrap_pointer<DiscreteDistribution>::type {
public:
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  LIBBIRCH_CLASS(BetaNegativeBinomialDistribution_, typename libbirch::unwrap_pointer<DiscreteDistribution>::type)
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(k, \u03b1, \u03b2)
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using base_type_::lower;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  BetaNegativeBinomialDistribution_(const Arg1& k, const Arg2& \u03b1, const Arg3& \u03b2);

  Arg1 k;
  Arg2 \u03b1;
  Arg3 \u03b2;
  #line 21 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  #line 21 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  Boolean supportsLazy() override;
  #line 25 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  #line 25 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  Integer simulate() override;
  #line 29 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  #line 29 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  std::optional<Integer> simulateLazy() override;
  #line 33 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  #line 33 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 37 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  #line 37 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Integer>& x) override;
  #line 41 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  #line 41 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 45 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  #line 45 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Integer>& x) override;
  #line 49 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  #line 49 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  std::optional<Integer> lower() override;
};

#line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
class GammaPoissonDistribution_ final : public libbirch::unwrap_pointer<DiscreteDistribution>::type {
public:
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  LIBBIRCH_CLASS(GammaPoissonDistribution_, typename libbirch::unwrap_pointer<DiscreteDistribution>::type)
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(k, \u03b8)
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::update;
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  using base_type_::updateLazy;
  
  #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  GammaPoissonDistribution_(const Arg1& k, const Arg2& \u03b8);

  Arg1 k;
  Arg2 \u03b8;
  #line 20 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  #line 20 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  Boolean supportsLazy() override;
  #line 24 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  #line 24 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  Integer simulate() override;
  #line 28 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  #line 28 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  std::optional<Integer> simulateLazy() override;
  #line 32 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  #line 32 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 36 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  #line 36 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Integer>& x) override;
  #line 40 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  #line 40 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 44 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  #line 44 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Integer>& x) override;
  #line 48 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  #line 48 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  std::optional<Real> cdf(const Integer& x) override;
  #line 52 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  #line 52 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  std::optional<Integer> quantile(const Real& P) override;
  #line 56 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  #line 56 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  std::optional<Integer> lower() override;
};

#line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
class NegativeBinomialDistribution_ final : public libbirch::unwrap_pointer<DiscreteDistribution>::type {
public:
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  LIBBIRCH_CLASS(NegativeBinomialDistribution_, typename libbirch::unwrap_pointer<DiscreteDistribution>::type)
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(k, \u03c1)
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  NegativeBinomialDistribution_(const Arg1& k, const Arg2& \u03c1);

  Arg1 k;
  Arg2 \u03c1;
  #line 20 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  #line 20 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  Boolean supportsLazy() override;
  #line 24 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  #line 24 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  Integer simulate() override;
  #line 28 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  #line 28 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  std::optional<Integer> simulateLazy() override;
  #line 32 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  #line 32 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 36 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  #line 36 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Integer>& x) override;
  #line 40 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  #line 40 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  std::optional<Real> cdf(const Integer& x) override;
  #line 44 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  #line 44 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  std::optional<Integer> quantile(const Real& P) override;
  #line 48 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  #line 48 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  std::optional<Integer> lower() override;
  #line 52 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  #line 52 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
class PoissonDistribution_ final : public libbirch::unwrap_pointer<DiscreteDistribution>::type {
public:
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using Arg_ = Arg;
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  LIBBIRCH_CLASS(PoissonDistribution_, typename libbirch::unwrap_pointer<DiscreteDistribution>::type)
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bb)
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  PoissonDistribution_(const Arg& \u03bb);

  Arg \u03bb;
  #line 14 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  #line 14 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  Boolean supportsLazy() override;
  #line 18 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  #line 18 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  Integer simulate() override;
  #line 22 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  #line 22 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  std::optional<Integer> simulateLazy() override;
  #line 26 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  #line 26 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 30 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  #line 30 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Integer>& x) override;
  #line 34 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  #line 34 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  std::optional<Real> cdf(const Integer& x) override;
  #line 38 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  #line 38 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  std::optional<Integer> quantile(const Real& P) override;
  #line 42 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  #line 42 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  std::optional<Integer> lower() override;
  #line 46 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  #line 46 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
class ScaledGammaPoissonDistribution_ final : public libbirch::unwrap_pointer<DiscreteDistribution>::type {
public:
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  LIBBIRCH_CLASS(ScaledGammaPoissonDistribution_, typename libbirch::unwrap_pointer<DiscreteDistribution>::type)
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(a, k, \u03b8)
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::cdf;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::lower;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::quantile;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  ScaledGammaPoissonDistribution_(const Arg1& a, const Arg2& k, const Arg3& \u03b8);

  Arg1 a;
  Arg2 k;
  Arg3 \u03b8;
  #line 21 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  #line 21 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  Boolean supportsLazy() override;
  #line 25 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  #line 25 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  Integer simulate() override;
  #line 29 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  #line 29 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  std::optional<Integer> simulateLazy() override;
  #line 33 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  #line 33 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 37 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  #line 37 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Integer>& x) override;
  #line 41 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  #line 41 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
  #line 45 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  #line 45 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Integer>& x) override;
  #line 49 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  #line 49 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  std::optional<Real> cdf(const Integer& x) override;
  #line 53 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  #line 53 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  std::optional<Integer> quantile(const Real& P) override;
  #line 57 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  #line 57 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  std::optional<Integer> lower() override;
};

#line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
class RestaurantCategoricalDistribution_ final : public libbirch::unwrap_pointer<Distribution<Integer>>::type {
public:
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  LIBBIRCH_CLASS(RestaurantCategoricalDistribution_, typename libbirch::unwrap_pointer<Distribution<Integer>>::type)
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03b1, \u03b8, n, N)
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  using base_type_::update;
  
  #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  RestaurantCategoricalDistribution_(const Arg1& \u03b1, const Arg2& \u03b8, const Arg3& n, const Arg4& N);

  Arg1 \u03b1;
  Arg2 \u03b8;
  Arg3 n;
  Arg4 N;
  #line 26 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  #line 26 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  Boolean supportsLazy() override;
  #line 30 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  #line 30 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  Integer simulate() override;
  #line 35 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  #line 35 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  Real logpdf(const Integer& x) override;
  #line 40 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  #line 40 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  std::optional<Delay> update(const Integer& x) override;
};

#line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
class GaussianDistribution_ : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  LIBBIRCH_CLASS(GaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bc, \u03c32)
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::operator=;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::cdf;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::getGaussian;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::isGaussian;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::logpdf;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::quantile;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::simulate;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  using base_type_::write;
  
  #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  GaussianDistribution_(const Arg1& \u03bc, const Arg2& \u03c32);

  Arg1 \u03bc;
  Arg2 \u03c32;
  #line 19 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  #line 19 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  virtual Boolean supportsLazy() override;
  #line 23 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  #line 23 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  virtual Real simulate() override;
  #line 27 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  #line 27 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  virtual std::optional<Real> simulateLazy() override;
  #line 31 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  #line 31 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  virtual Real logpdf(const Real& x) override;
  #line 35 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  #line 35 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  virtual std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 39 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  #line 39 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  virtual std::optional<Real> cdf(const Real& x) override;
  #line 43 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  #line 43 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  virtual std::optional<Real> quantile(const Real& P) override;
  #line 47 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  #line 47 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  virtual Boolean isGaussian() override;
  #line 51 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  #line 51 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  virtual std::optional<std::tuple<Expression<Real>, Expression<Real>>> getGaussian() override;
  #line 55 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  #line 55 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  virtual void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
class GaussianGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  LIBBIRCH_CLASS(GaussianGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bc, \u03c32, s2)
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::cdf;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::getGaussian;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::isGaussian;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::quantile;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  GaussianGaussianDistribution_(const Arg1& \u03bc, const Arg2& \u03c32, const Arg3& s2);

  Arg1 \u03bc;
  Arg2 \u03c32;
  Arg3 s2;
  #line 21 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 21 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 25 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 25 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  Real simulate() override;
  #line 29 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 29 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 33 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 33 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 37 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 37 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 41 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 41 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  std::optional<Delay> update(const Real& x) override;
  #line 45 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 45 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Real>& x) override;
  #line 49 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 49 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 53 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 53 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 57 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 57 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  Boolean isGaussian() override;
  #line 61 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  #line 61 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  std::optional<std::tuple<Expression<Real>, Expression<Real>>> getGaussian() override;
};

#line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
class LinearGaussianGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  LIBBIRCH_CLASS(LinearGaussianGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(a, \u03bc, \u03c32, c, s2)
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::cdf;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::getGaussian;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::isGaussian;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::quantile;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  LinearGaussianGaussianDistribution_(const Arg1& a, const Arg2& \u03bc, const Arg3& \u03c32, const Arg4& c, const Arg5& s2);

  Arg1 a;
  Arg2 \u03bc;
  Arg3 \u03c32;
  Arg4 c;
  Arg5 s2;
  #line 31 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 31 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 35 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 35 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  Real simulate() override;
  #line 40 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 40 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 45 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 45 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 50 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 50 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 54 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 54 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  std::optional<Delay> update(const Real& x) override;
  #line 59 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 59 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Real>& x) override;
  #line 63 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 63 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 68 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 68 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 73 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 73 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  Boolean isGaussian() override;
  #line 77 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  #line 77 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  std::optional<std::tuple<Expression<Real>, Expression<Real>>> getGaussian() override;
};

#line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
class LinearMultivariateGaussianGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  LIBBIRCH_CLASS(LinearMultivariateGaussianGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(a, \u03bc, \u03a3, c, s2)
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::cdf;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::getGaussian;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::isGaussian;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::quantile;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  LinearMultivariateGaussianGaussianDistribution_(const Arg1& a, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& c, const Arg5& s2);

  Arg1 a;
  Arg2 \u03bc;
  Arg3 \u03a3;
  Arg4 c;
  Arg5 s2;
  #line 31 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 31 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 35 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 35 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  Real simulate() override;
  #line 40 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 40 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 45 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 45 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 50 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 50 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 55 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 55 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  std::optional<Delay> update(const Real& x) override;
  #line 60 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 60 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Real>& x) override;
  #line 64 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 64 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 69 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 69 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 74 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 74 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  Boolean isGaussian() override;
  #line 78 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  #line 78 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  std::optional<std::tuple<Expression<Real>, Expression<Real>>> getGaussian() override;
};

#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
class LinearMultivariateNormalInverseGammaGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using Arg6_ = Arg6;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using Arg7_ = Arg7;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  LIBBIRCH_CLASS(LinearMultivariateNormalInverseGammaGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(a, \u03bd, \u039b, \u03b1, \u03b2, c, s2)
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using base_type_::cdf;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using base_type_::quantile;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  LinearMultivariateNormalInverseGammaGaussianDistribution_(const Arg1& a, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& s2);

  Arg1 a;
  Arg2 \u03bd;
  Arg3 \u039b;
  Arg4 \u03b1;
  Arg5 \u03b2;
  Arg6 c;
  Arg7 s2;
  #line 42 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  #line 42 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 46 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  #line 46 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  Real simulate() override;
  #line 52 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  #line 52 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 58 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  #line 58 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 64 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  #line 64 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 69 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  #line 69 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  std::optional<Delay> update(const Real& x) override;
  #line 75 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  #line 75 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Real>& x) override;
  #line 80 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  #line 80 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 85 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  #line 85 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
};

#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
class LinearNormalInverseGammaGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using Arg6_ = Arg6;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using Arg7_ = Arg7;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  LIBBIRCH_CLASS(LinearNormalInverseGammaGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(a, \u03bd, \u03bb, \u03b1, \u03b2, c, s2)
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using base_type_::cdf;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using base_type_::quantile;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  LinearNormalInverseGammaGaussianDistribution_(const Arg1& a, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& s2);

  Arg1 a;
  Arg2 \u03bd;
  Arg3 \u03bb;
  Arg4 \u03b1;
  Arg5 \u03b2;
  Arg6 c;
  Arg7 s2;
  #line 42 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  #line 42 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 46 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  #line 46 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  Real simulate() override;
  #line 51 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  #line 51 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 56 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  #line 56 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 61 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  #line 61 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 66 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  #line 66 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  std::optional<Delay> update(const Real& x) override;
  #line 71 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  #line 71 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Real>& x) override;
  #line 76 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  #line 76 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 81 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  #line 81 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
};

#line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
class NormalInverseGammaDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using Arg1_ = Arg1;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using Arg2_ = Arg2;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using Arg3_ = Arg3;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using Arg4_ = Arg4;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  LIBBIRCH_CLASS(NormalInverseGammaDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bd, \u03bb, \u03b1, \u03b2)
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::operator=;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::cdf;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::getNormalInverseGamma;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::isNormalInverseGamma;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::logpdf;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::logpdfLazy;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::quantile;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::simulate;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::simulateLazy;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::supportsLazy;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::update;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::updateLazy;
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  using base_type_::write;
  
  #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  NormalInverseGammaDistribution_(const Arg1& \u03bd, const Arg2& \u03bb, const Arg3& \u03b1, const Arg4& \u03b2);

  Arg1 \u03bd;
  Arg2 \u03bb;
  Arg3 \u03b1;
  Arg4 \u03b2;
  #line 50 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 50 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  Boolean supportsLazy() override;
  #line 54 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 54 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  Real simulate() override;
  #line 59 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 59 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 63 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 63 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 68 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 68 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 72 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 72 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  std::optional<Delay> update(const Real& x) override;
  #line 77 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 77 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Real>& x) override;
  #line 81 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 81 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 86 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 86 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 91 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 91 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  Boolean isNormalInverseGamma() override;
  #line 95 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 95 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  std::optional<std::tuple<Expression<Real>, Expression<Real>, Expression<Real>, Expression<Real>>> getNormalInverseGamma() override;
  #line 100 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  #line 100 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
class NormalInverseGammaGaussianDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using Arg4_ = Arg4;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using Arg5_ = Arg5;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  LIBBIRCH_CLASS(NormalInverseGammaGaussianDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bd, \u03bb, \u03b1, \u03b2, s2)
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using base_type_::cdf;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using base_type_::quantile;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  NormalInverseGammaGaussianDistribution_(const Arg1& \u03bd, const Arg2& \u03bb, const Arg3& \u03b1, const Arg4& \u03b2, const Arg5& s2);

  Arg1 \u03bd;
  Arg2 \u03bb;
  Arg3 \u03b1;
  Arg4 \u03b2;
  Arg5 s2;
  #line 31 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  #line 31 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  Boolean supportsLazy() override;
  #line 35 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  #line 35 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  Real simulate() override;
  #line 40 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  #line 40 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 45 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  #line 45 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 50 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  #line 50 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 54 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  #line 54 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  std::optional<Delay> update(const Real& x) override;
  #line 59 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  #line 59 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Real>& x) override;
  #line 63 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  #line 63 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 68 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  #line 68 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
};

#line 8 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/BetaDistribution.birch"
class BetaDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  LIBBIRCH_CLASS(BetaDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03b1, \u03b2)
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::getBeta;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::isBeta;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::upper;
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
  BetaDistribution_(const Arg1& \u03b1, const Arg2& \u03b2);

  Arg1 \u03b1;
  Arg2 \u03b2;
  #line 19 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 19 "src/distribution/scalar_other/BetaDistribution.birch"
  Boolean supportsLazy() override;
  #line 23 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 23 "src/distribution/scalar_other/BetaDistribution.birch"
  Real simulate() override;
  #line 27 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 27 "src/distribution/scalar_other/BetaDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 31 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 31 "src/distribution/scalar_other/BetaDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 35 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 35 "src/distribution/scalar_other/BetaDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 39 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 39 "src/distribution/scalar_other/BetaDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 43 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 43 "src/distribution/scalar_other/BetaDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 47 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 47 "src/distribution/scalar_other/BetaDistribution.birch"
  std::optional<Real> lower() override;
  #line 51 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 51 "src/distribution/scalar_other/BetaDistribution.birch"
  std::optional<Real> upper() override;
  #line 55 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 55 "src/distribution/scalar_other/BetaDistribution.birch"
  Boolean isBeta() override;
  #line 59 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 59 "src/distribution/scalar_other/BetaDistribution.birch"
  std::optional<std::tuple<Expression<Real>, Expression<Real>>> getBeta() override;
  #line 63 "src/distribution/scalar_other/BetaDistribution.birch"
  #line 63 "src/distribution/scalar_other/BetaDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
class ExponentialDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  LIBBIRCH_CLASS(ExponentialDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bb)
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
  ExponentialDistribution_(const Arg1& \u03bb);

  Arg1 \u03bb;
  #line 14 "src/distribution/scalar_other/ExponentialDistribution.birch"
  #line 14 "src/distribution/scalar_other/ExponentialDistribution.birch"
  Boolean supportsLazy() override;
  #line 18 "src/distribution/scalar_other/ExponentialDistribution.birch"
  #line 18 "src/distribution/scalar_other/ExponentialDistribution.birch"
  Real simulate() override;
  #line 22 "src/distribution/scalar_other/ExponentialDistribution.birch"
  #line 22 "src/distribution/scalar_other/ExponentialDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 26 "src/distribution/scalar_other/ExponentialDistribution.birch"
  #line 26 "src/distribution/scalar_other/ExponentialDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 30 "src/distribution/scalar_other/ExponentialDistribution.birch"
  #line 30 "src/distribution/scalar_other/ExponentialDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 34 "src/distribution/scalar_other/ExponentialDistribution.birch"
  #line 34 "src/distribution/scalar_other/ExponentialDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 38 "src/distribution/scalar_other/ExponentialDistribution.birch"
  #line 38 "src/distribution/scalar_other/ExponentialDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 42 "src/distribution/scalar_other/ExponentialDistribution.birch"
  #line 42 "src/distribution/scalar_other/ExponentialDistribution.birch"
  std::optional<Real> lower() override;
  #line 46 "src/distribution/scalar_other/ExponentialDistribution.birch"
  #line 46 "src/distribution/scalar_other/ExponentialDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 8 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/GammaDistribution.birch"
class GammaDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  LIBBIRCH_CLASS(GammaDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(k, \u03b8)
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::getGamma;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::isGamma;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
  GammaDistribution_(const Arg1& k, const Arg2& \u03b8);

  Arg1 k;
  Arg2 \u03b8;
  #line 19 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 19 "src/distribution/scalar_other/GammaDistribution.birch"
  Boolean supportsLazy() override;
  #line 23 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 23 "src/distribution/scalar_other/GammaDistribution.birch"
  Real simulate() override;
  #line 27 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 27 "src/distribution/scalar_other/GammaDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 31 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 31 "src/distribution/scalar_other/GammaDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 35 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 35 "src/distribution/scalar_other/GammaDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 39 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 39 "src/distribution/scalar_other/GammaDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 43 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 43 "src/distribution/scalar_other/GammaDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 47 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 47 "src/distribution/scalar_other/GammaDistribution.birch"
  std::optional<Real> lower() override;
  #line 51 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 51 "src/distribution/scalar_other/GammaDistribution.birch"
  Boolean isGamma() override;
  #line 55 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 55 "src/distribution/scalar_other/GammaDistribution.birch"
  std::optional<std::tuple<Expression<Real>, Expression<Real>>> getGamma() override;
  #line 59 "src/distribution/scalar_other/GammaDistribution.birch"
  #line 59 "src/distribution/scalar_other/GammaDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
class GammaExponentialDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  LIBBIRCH_CLASS(GammaExponentialDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(k, \u03b8)
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::update;
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  using base_type_::updateLazy;
  
  #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  GammaExponentialDistribution_(const Arg1& k, const Arg2& \u03b8);

  Arg1 k;
  Arg2 \u03b8;
  #line 19 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  #line 19 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  Boolean supportsLazy() override;
  #line 23 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  #line 23 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  Real simulate() override;
  #line 27 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  #line 27 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 31 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  #line 31 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 35 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  #line 35 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 39 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  #line 39 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  std::optional<Delay> update(const Real& x) override;
  #line 43 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  #line 43 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Real>& x) override;
  #line 47 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  #line 47 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 51 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  #line 51 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 55 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  #line 55 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  std::optional<Real> lower() override;
};

#line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
class InverseGammaDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  LIBBIRCH_CLASS(InverseGammaDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03b1, \u03b2)
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::getInverseGamma;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::isInverseGamma;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  InverseGammaDistribution_(const Arg1& \u03b1, const Arg2& \u03b2);

  Arg1 \u03b1;
  Arg2 \u03b2;
  #line 19 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 19 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  Boolean supportsLazy() override;
  #line 23 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 23 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  Real simulate() override;
  #line 27 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 27 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 31 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 31 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 35 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 35 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 39 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 39 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 43 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 43 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 47 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 47 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  std::optional<Real> lower() override;
  #line 51 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 51 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  Boolean isInverseGamma() override;
  #line 55 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 55 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  std::optional<std::tuple<Expression<Real>, Expression<Real>>> getInverseGamma() override;
  #line 59 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  #line 59 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
class InverseGammaGammaDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  LIBBIRCH_CLASS(InverseGammaGammaDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(k, \u03b1, \u03b2)
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using base_type_::cdf;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using base_type_::lower;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  InverseGammaGammaDistribution_(const Arg1& k, const Arg2& \u03b1, const Arg3& \u03b2);

  Arg1 k;
  Arg2 \u03b1;
  Arg3 \u03b2;
  #line 21 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  #line 21 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  Boolean supportsLazy() override;
  #line 25 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  #line 25 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  Real simulate() override;
  #line 29 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  #line 29 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 33 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  #line 33 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 37 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  #line 37 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 41 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  #line 41 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  std::optional<Delay> update(const Real& x) override;
  #line 45 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  #line 45 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Real>& x) override;
  #line 49 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  #line 49 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 53 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  #line 53 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  std::optional<Real> lower() override;
};

#line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
class ScaledGammaExponentialDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using Arg1_ = Arg1;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using Arg2_ = Arg2;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using Arg3_ = Arg3;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  LIBBIRCH_CLASS(ScaledGammaExponentialDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(a, k, \u03b8)
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::operator=;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::cdf;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::logpdf;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::logpdfLazy;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::lower;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::quantile;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::simulate;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::simulateLazy;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::supportsLazy;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::update;
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  using base_type_::updateLazy;
  
  #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  ScaledGammaExponentialDistribution_(const Arg1& a, const Arg2& k, const Arg3& \u03b8);

  Arg1 a;
  Arg2 k;
  Arg3 \u03b8;
  #line 21 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  #line 21 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  Boolean supportsLazy() override;
  #line 25 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  #line 25 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  Real simulate() override;
  #line 29 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  #line 29 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 33 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  #line 33 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 37 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  #line 37 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 41 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  #line 41 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  std::optional<Delay> update(const Real& x) override;
  #line 45 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  #line 45 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  std::optional<Delay> updateLazy(const Expression<Real>& x) override;
  #line 49 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  #line 49 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 53 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  #line 53 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 57 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  #line 57 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  std::optional<Real> lower() override;
};

#line 8 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 8 "src/distribution/scalar_other/StudentDistribution.birch"
class StudentDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using Arg3_ = Arg3;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  LIBBIRCH_CLASS(StudentDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bd, \u03bc, \u03c32)
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
  StudentDistribution_(const Arg1& \u03bd, const Arg2& \u03bc, const Arg3& \u03c32);

  Arg1 \u03bd;
  Arg2 \u03bc;
  Arg3 \u03c32;
  #line 25 "src/distribution/scalar_other/StudentDistribution.birch"
  #line 25 "src/distribution/scalar_other/StudentDistribution.birch"
  Boolean supportsLazy() override;
  #line 29 "src/distribution/scalar_other/StudentDistribution.birch"
  #line 29 "src/distribution/scalar_other/StudentDistribution.birch"
  Real simulate() override;
  #line 33 "src/distribution/scalar_other/StudentDistribution.birch"
  #line 33 "src/distribution/scalar_other/StudentDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 37 "src/distribution/scalar_other/StudentDistribution.birch"
  #line 37 "src/distribution/scalar_other/StudentDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 41 "src/distribution/scalar_other/StudentDistribution.birch"
  #line 41 "src/distribution/scalar_other/StudentDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 45 "src/distribution/scalar_other/StudentDistribution.birch"
  #line 45 "src/distribution/scalar_other/StudentDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 49 "src/distribution/scalar_other/StudentDistribution.birch"
  #line 49 "src/distribution/scalar_other/StudentDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 53 "src/distribution/scalar_other/StudentDistribution.birch"
  #line 53 "src/distribution/scalar_other/StudentDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 8 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/UniformDistribution.birch"
class UniformDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  LIBBIRCH_CLASS(UniformDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(l, u)
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using base_type_::upper;
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
  UniformDistribution_(const Arg1& l, const Arg2& u);

  Arg1 l;
  Arg2 u;
  #line 19 "src/distribution/scalar_other/UniformDistribution.birch"
  #line 19 "src/distribution/scalar_other/UniformDistribution.birch"
  Boolean supportsLazy() override;
  #line 23 "src/distribution/scalar_other/UniformDistribution.birch"
  #line 23 "src/distribution/scalar_other/UniformDistribution.birch"
  Real simulate() override;
  #line 27 "src/distribution/scalar_other/UniformDistribution.birch"
  #line 27 "src/distribution/scalar_other/UniformDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 31 "src/distribution/scalar_other/UniformDistribution.birch"
  #line 31 "src/distribution/scalar_other/UniformDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 35 "src/distribution/scalar_other/UniformDistribution.birch"
  #line 35 "src/distribution/scalar_other/UniformDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 39 "src/distribution/scalar_other/UniformDistribution.birch"
  #line 39 "src/distribution/scalar_other/UniformDistribution.birch"
  std::optional<Real> lower() override;
  #line 43 "src/distribution/scalar_other/UniformDistribution.birch"
  #line 43 "src/distribution/scalar_other/UniformDistribution.birch"
  std::optional<Real> upper() override;
  #line 47 "src/distribution/scalar_other/UniformDistribution.birch"
  #line 47 "src/distribution/scalar_other/UniformDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
class WeibullDistribution_ final : public libbirch::unwrap_pointer<Distribution<Real>>::type {
public:
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using Arg1_ = Arg1;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using Arg2_ = Arg2;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  LIBBIRCH_CLASS(WeibullDistribution_, typename libbirch::unwrap_pointer<Distribution<Real>>::type)
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  LIBBIRCH_CLASS_MEMBERS(k, \u03bb)
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using base_type_::operator=;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using base_type_::cdf;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using base_type_::logpdf;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using base_type_::logpdfLazy;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using base_type_::lower;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using base_type_::quantile;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using base_type_::simulate;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using base_type_::simulateLazy;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using base_type_::supportsLazy;
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  using base_type_::write;
  
  #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
  WeibullDistribution_(const Arg1& k, const Arg2& \u03bb);

  Arg1 k;
  Arg2 \u03bb;
  #line 19 "src/distribution/scalar_other/WeibullDistribution.birch"
  #line 19 "src/distribution/scalar_other/WeibullDistribution.birch"
  Boolean supportsLazy() override;
  #line 23 "src/distribution/scalar_other/WeibullDistribution.birch"
  #line 23 "src/distribution/scalar_other/WeibullDistribution.birch"
  Real simulate() override;
  #line 27 "src/distribution/scalar_other/WeibullDistribution.birch"
  #line 27 "src/distribution/scalar_other/WeibullDistribution.birch"
  std::optional<Real> simulateLazy() override;
  #line 31 "src/distribution/scalar_other/WeibullDistribution.birch"
  #line 31 "src/distribution/scalar_other/WeibullDistribution.birch"
  Real logpdf(const Real& x) override;
  #line 35 "src/distribution/scalar_other/WeibullDistribution.birch"
  #line 35 "src/distribution/scalar_other/WeibullDistribution.birch"
  std::optional<Expression<Real>> logpdfLazy(const Expression<Real>& x) override;
  #line 39 "src/distribution/scalar_other/WeibullDistribution.birch"
  #line 39 "src/distribution/scalar_other/WeibullDistribution.birch"
  std::optional<Real> cdf(const Real& x) override;
  #line 43 "src/distribution/scalar_other/WeibullDistribution.birch"
  #line 43 "src/distribution/scalar_other/WeibullDistribution.birch"
  std::optional<Real> quantile(const Real& P) override;
  #line 47 "src/distribution/scalar_other/WeibullDistribution.birch"
  #line 47 "src/distribution/scalar_other/WeibullDistribution.birch"
  std::optional<Real> lower() override;
  #line 51 "src/distribution/scalar_other/WeibullDistribution.birch"
  #line 51 "src/distribution/scalar_other/WeibullDistribution.birch"
  void write(const Buffer& buffer) override;
};

#line 93 "src/expression/Expression.birch"
template<class Value>
#line 93 "src/expression/Expression.birch"
class Expression_ : public libbirch::unwrap_pointer<Delay>::type {
public:
  #line 93 "src/expression/Expression.birch"
  using Value_ = Value;
  #line 93 "src/expression/Expression.birch"
  LIBBIRCH_ABSTRACT_CLASS(Expression_, typename libbirch::unwrap_pointer<Delay>::type)
  #line 93 "src/expression/Expression.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 93 "src/expression/Expression.birch"
  using base_type_::operator=;
  #line 93 "src/expression/Expression.birch"
  using base_type_::write;
  
  #line 93 "src/expression/Expression.birch"
  Expression_();

  #line 97 "src/expression/Expression.birch"
  #line 97 "src/expression/Expression.birch"
  virtual Boolean isRandom() = 0;
  #line 102 "src/expression/Expression.birch"
  #line 102 "src/expression/Expression.birch"
  virtual Boolean isConstant() = 0;
  #line 107 "src/expression/Expression.birch"
  #line 107 "src/expression/Expression.birch"
  virtual Integer rows() = 0;
  #line 112 "src/expression/Expression.birch"
  #line 112 "src/expression/Expression.birch"
  virtual Integer columns() = 0;
  #line 117 "src/expression/Expression.birch"
  #line 117 "src/expression/Expression.birch"
  virtual Integer length() final;
  #line 124 "src/expression/Expression.birch"
  #line 124 "src/expression/Expression.birch"
  virtual Integer size() final;
  #line 133 "src/expression/Expression.birch"
  #line 133 "src/expression/Expression.birch"
  virtual Value value() = 0;
  #line 140 "src/expression/Expression.birch"
  #line 140 "src/expression/Expression.birch"
  virtual Value peek() = 0;
  #line 145 "src/expression/Expression.birch"
  #line 145 "src/expression/Expression.birch"
  virtual Value eval() = 0;
  #line 150 "src/expression/Expression.birch"
  #line 150 "src/expression/Expression.birch"
  virtual Value reval() = 0;
  #line 197 "src/expression/Expression.birch"
  #line 197 "src/expression/Expression.birch"
  virtual void grad(const Value& d) = 0;
  #line 207 "src/expression/Expression.birch"
  #line 207 "src/expression/Expression.birch"
  virtual void label(const Integer& gen) = 0;
  #line 217 "src/expression/Expression.birch"
  #line 217 "src/expression/Expression.birch"
  virtual void constant(const Integer& gen) = 0;
  #line 222 "src/expression/Expression.birch"
  #line 222 "src/expression/Expression.birch"
  virtual void constant() = 0;
  #line 224 "src/expression/Expression.birch"
  #line 224 "src/expression/Expression.birch"
  virtual void write(const Buffer& buffer) override;
};

#line 4 "src/expression/BoxedValue.birch"
template<class Value>
#line 4 "src/expression/BoxedValue.birch"
class BoxedValue_ final : public libbirch::unwrap_pointer<Expression<Value>>::type {
public:
  #line 4 "src/expression/BoxedValue.birch"
  using Value_ = Value;
  #line 4 "src/expression/BoxedValue.birch"
  LIBBIRCH_CLASS(BoxedValue_, typename libbirch::unwrap_pointer<Expression<Value>>::type)
  #line 4 "src/expression/BoxedValue.birch"
  LIBBIRCH_CLASS_MEMBERS(x)
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::operator=;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::columns;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::constant;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::eval;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::grad;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::isConstant;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::isRandom;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::label;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::peek;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::reval;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::rows;
  #line 4 "src/expression/BoxedValue.birch"
  using base_type_::value;
  
  #line 4 "src/expression/BoxedValue.birch"
  BoxedValue_(const Value& x);

  Value x;
  #line 10 "src/expression/BoxedValue.birch"
  #line 10 "src/expression/BoxedValue.birch"
  Boolean isRandom() override;
  #line 14 "src/expression/BoxedValue.birch"
  #line 14 "src/expression/BoxedValue.birch"
  Boolean isConstant() override;
  #line 18 "src/expression/BoxedValue.birch"
  #line 18 "src/expression/BoxedValue.birch"
  Integer rows() override;
  #line 22 "src/expression/BoxedValue.birch"
  #line 22 "src/expression/BoxedValue.birch"
  Integer columns() override;
  #line 26 "src/expression/BoxedValue.birch"
  #line 26 "src/expression/BoxedValue.birch"
  Value value() override;
  #line 30 "src/expression/BoxedValue.birch"
  #line 30 "src/expression/BoxedValue.birch"
  Value peek() override;
  #line 34 "src/expression/BoxedValue.birch"
  #line 34 "src/expression/BoxedValue.birch"
  Value eval() override;
  #line 38 "src/expression/BoxedValue.birch"
  #line 38 "src/expression/BoxedValue.birch"
  Value reval() override;
  #line 42 "src/expression/BoxedValue.birch"
  #line 42 "src/expression/BoxedValue.birch"
  void grad(const Value& d) override;
  #line 46 "src/expression/BoxedValue.birch"
  #line 46 "src/expression/BoxedValue.birch"
  void label(const Integer& gen) override;
  #line 50 "src/expression/BoxedValue.birch"
  #line 50 "src/expression/BoxedValue.birch"
  void constant(const Integer& gen) override;
  #line 54 "src/expression/BoxedValue.birch"
  #line 54 "src/expression/BoxedValue.birch"
  void constant() override;
};

#line 1 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 1 "src/expression/BoxedForm.birch"
class BoxedForm_ : public libbirch::unwrap_pointer<Expression<Value>>::type {
public:
  #line 1 "src/expression/BoxedForm.birch"
  using Value_ = Value;
  #line 1 "src/expression/BoxedForm.birch"
  using Form_ = Form;
  #line 1 "src/expression/BoxedForm.birch"
  LIBBIRCH_CLASS(BoxedForm_, typename libbirch::unwrap_pointer<Expression<Value>>::type)
  #line 1 "src/expression/BoxedForm.birch"
  LIBBIRCH_CLASS_MEMBERS(x, f, d, gen, evalCount, visitCount)
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::operator=;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::columns;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::constant;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::eval;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::grad;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::isConstant;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::isRandom;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::label;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::peek;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::reval;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::rows;
  #line 1 "src/expression/BoxedForm.birch"
  using base_type_::value;
  
  #line 1 "src/expression/BoxedForm.birch"
  BoxedForm_(const Value& x, const Form& f);

  Value x;
  std::optional<Form> f;
  std::optional<Value> d;
  Integer gen;
  Integer evalCount;
  Integer visitCount;
  #line 39 "src/expression/BoxedForm.birch"
  #line 39 "src/expression/BoxedForm.birch"
  virtual Boolean isRandom() final;
  #line 43 "src/expression/BoxedForm.birch"
  #line 43 "src/expression/BoxedForm.birch"
  virtual Boolean isConstant() final;
  #line 47 "src/expression/BoxedForm.birch"
  #line 47 "src/expression/BoxedForm.birch"
  virtual Integer rows() final;
  #line 51 "src/expression/BoxedForm.birch"
  #line 51 "src/expression/BoxedForm.birch"
  virtual Integer columns() final;
  #line 55 "src/expression/BoxedForm.birch"
  #line 55 "src/expression/BoxedForm.birch"
  virtual Value value() final;
  #line 60 "src/expression/BoxedForm.birch"
  #line 60 "src/expression/BoxedForm.birch"
  virtual Value peek() final;
  #line 64 "src/expression/BoxedForm.birch"
  #line 64 "src/expression/BoxedForm.birch"
  virtual Value eval() final;
  #line 71 "src/expression/BoxedForm.birch"
  #line 71 "src/expression/BoxedForm.birch"
  virtual Value reval() final;
  #line 90 "src/expression/BoxedForm.birch"
  #line 90 "src/expression/BoxedForm.birch"
  virtual void grad(const Value& d) final;
  #line 113 "src/expression/BoxedForm.birch"
  #line 113 "src/expression/BoxedForm.birch"
  virtual void label(const Integer& gen) final;
  #line 120 "src/expression/BoxedForm.birch"
  #line 120 "src/expression/BoxedForm.birch"
  virtual void constant(const Integer& gen) final;
  #line 130 "src/expression/BoxedForm.birch"
  #line 130 "src/expression/BoxedForm.birch"
  virtual void constant() final;
};

#line 7 "src/expression/MatrixSplitExpression.birch"
template<class Value,class Form>
#line 7 "src/expression/MatrixSplitExpression.birch"
class MatrixSplitExpression_ final : public libbirch::unwrap_pointer<BoxedForm<Value, Form>>::type {
public:
  #line 7 "src/expression/MatrixSplitExpression.birch"
  using Value_ = Value;
  #line 7 "src/expression/MatrixSplitExpression.birch"
  using Form_ = Form;
  #line 7 "src/expression/MatrixSplitExpression.birch"
  LIBBIRCH_CLASS(MatrixSplitExpression_, typename libbirch::unwrap_pointer<BoxedForm<Value, Form>>::type)
  #line 7 "src/expression/MatrixSplitExpression.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 7 "src/expression/MatrixSplitExpression.birch"
  using base_type_::operator=;
  
  #line 7 "src/expression/MatrixSplitExpression.birch"
  MatrixSplitExpression_(const Value& x, const Form& f);

  #line 9 "src/expression/MatrixSplitExpression.birch"
  #line 9 "src/expression/MatrixSplitExpression.birch"
  void grad(const Real& d, const Integer& i, const Integer& j);
};

#line 7 "src/expression/VectorSplitExpression.birch"
template<class Value,class Form>
#line 7 "src/expression/VectorSplitExpression.birch"
class VectorSplitExpression_ final : public libbirch::unwrap_pointer<BoxedForm<Value, Form>>::type {
public:
  #line 7 "src/expression/VectorSplitExpression.birch"
  using Value_ = Value;
  #line 7 "src/expression/VectorSplitExpression.birch"
  using Form_ = Form;
  #line 7 "src/expression/VectorSplitExpression.birch"
  LIBBIRCH_CLASS(VectorSplitExpression_, typename libbirch::unwrap_pointer<BoxedForm<Value, Form>>::type)
  #line 7 "src/expression/VectorSplitExpression.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 7 "src/expression/VectorSplitExpression.birch"
  using base_type_::operator=;
  
  #line 7 "src/expression/VectorSplitExpression.birch"
  VectorSplitExpression_(const Value& x, const Form& f);

  #line 9 "src/expression/VectorSplitExpression.birch"
  #line 9 "src/expression/VectorSplitExpression.birch"
  void grad(const Real& d, const Integer& i);
};

#line 13 "src/expression/Random.birch"
template<class Value>
#line 13 "src/expression/Random.birch"
class Random_ final : public libbirch::unwrap_pointer<Expression<Value>>::type {
public:
  #line 13 "src/expression/Random.birch"
  using Value_ = Value;
  #line 13 "src/expression/Random.birch"
  LIBBIRCH_CLASS(Random_, typename libbirch::unwrap_pointer<Expression<Value>>::type)
  #line 13 "src/expression/Random.birch"
  LIBBIRCH_CLASS_MEMBERS(x, p, d, flagConstant, gen)
  #line 13 "src/expression/Random.birch"
  using base_type_::operator=;
  #line 13 "src/expression/Random.birch"
  using base_type_::columns;
  #line 13 "src/expression/Random.birch"
  using base_type_::constant;
  #line 13 "src/expression/Random.birch"
  using base_type_::eval;
  #line 13 "src/expression/Random.birch"
  using base_type_::grad;
  #line 13 "src/expression/Random.birch"
  using base_type_::graft;
  #line 13 "src/expression/Random.birch"
  using base_type_::graftReplace;
  #line 13 "src/expression/Random.birch"
  using base_type_::isConstant;
  #line 13 "src/expression/Random.birch"
  using base_type_::isRandom;
  #line 13 "src/expression/Random.birch"
  using base_type_::label;
  #line 13 "src/expression/Random.birch"
  using base_type_::peek;
  #line 13 "src/expression/Random.birch"
  using base_type_::prune;
  #line 13 "src/expression/Random.birch"
  using base_type_::read;
  #line 13 "src/expression/Random.birch"
  using base_type_::reval;
  #line 13 "src/expression/Random.birch"
  using base_type_::rows;
  #line 13 "src/expression/Random.birch"
  using base_type_::value;
  #line 13 "src/expression/Random.birch"
  using base_type_::write;
  
  #line 13 "src/expression/Random.birch"
  Random_();

  std::optional<Value> x;
  std::optional<Distribution<Value>> p;
  std::optional<Value> d;
  Boolean flagConstant;
  Integer gen;
  Random_<Value>& operator=(const Value& x);
  #line 52 "src/expression/Random.birch"
  #line 52 "src/expression/Random.birch"
  Boolean isRandom() override;
  #line 56 "src/expression/Random.birch"
  #line 56 "src/expression/Random.birch"
  Boolean isConstant() override;
  #line 63 "src/expression/Random.birch"
  #line 63 "src/expression/Random.birch"
  Boolean hasValue();
  #line 67 "src/expression/Random.birch"
  #line 67 "src/expression/Random.birch"
  Integer rows() override;
  #line 72 "src/expression/Random.birch"
  #line 72 "src/expression/Random.birch"
  Integer columns() override;
  #line 77 "src/expression/Random.birch"
  #line 77 "src/expression/Random.birch"
  Value value() override;
  #line 83 "src/expression/Random.birch"
  #line 83 "src/expression/Random.birch"
  Value peek() override;
  #line 87 "src/expression/Random.birch"
  #line 87 "src/expression/Random.birch"
  Value eval() override;
  #line 95 "src/expression/Random.birch"
  #line 95 "src/expression/Random.birch"
  Value reval() override;
  #line 101 "src/expression/Random.birch"
  #line 101 "src/expression/Random.birch"
  void grad(const Value& d) override;
  #line 109 "src/expression/Random.birch"
  #line 109 "src/expression/Random.birch"
  void label(const Integer& gen) override;
  #line 115 "src/expression/Random.birch"
  #line 115 "src/expression/Random.birch"
  void constant(const Integer& gen) override;
  #line 121 "src/expression/Random.birch"
  #line 121 "src/expression/Random.birch"
  void constant() override;
  #line 132 "src/expression/Random.birch"
  #line 132 "src/expression/Random.birch"
  void move(const Value& x);
  #line 143 "src/expression/Random.birch"
  #line 143 "src/expression/Random.birch"
  void assume(const Distribution<Value>& p);
  #line 150 "src/expression/Random.birch"
  #line 150 "src/expression/Random.birch"
  void graft(const Delay& node) override;
  #line 155 "src/expression/Random.birch"
  #line 155 "src/expression/Random.birch"
  void graftReplace(const Delay& node) override;
  #line 160 "src/expression/Random.birch"
  #line 160 "src/expression/Random.birch"
  Delay prune() override;
  #line 189 "src/expression/Random.birch"
  #line 189 "src/expression/Random.birch"
  Boolean hasGradient();
  #line 197 "src/expression/Random.birch"
  #line 197 "src/expression/Random.birch"
  Value getGradient();
  #line 205 "src/expression/Random.birch"
  #line 205 "src/expression/Random.birch"
  Boolean hasDistribution();
  #line 212 "src/expression/Random.birch"
  #line 212 "src/expression/Random.birch"
  Distribution<Value> getDistribution();
  #line 219 "src/expression/Random.birch"
  #line 219 "src/expression/Random.birch"
  Boolean hasGaussian();
  #line 223 "src/expression/Random.birch"
  #line 223 "src/expression/Random.birch"
  std::tuple<Expression<Real>, Expression<Real>> getGaussian();
  #line 227 "src/expression/Random.birch"
  #line 227 "src/expression/Random.birch"
  Boolean hasBeta();
  #line 231 "src/expression/Random.birch"
  #line 231 "src/expression/Random.birch"
  std::tuple<Expression<Real>, Expression<Real>> getBeta();
  #line 235 "src/expression/Random.birch"
  #line 235 "src/expression/Random.birch"
  Boolean hasGamma();
  #line 239 "src/expression/Random.birch"
  #line 239 "src/expression/Random.birch"
  std::tuple<Expression<Real>, Expression<Real>> getGamma();
  #line 243 "src/expression/Random.birch"
  #line 243 "src/expression/Random.birch"
  Boolean hasInverseGamma();
  #line 247 "src/expression/Random.birch"
  #line 247 "src/expression/Random.birch"
  std::tuple<Expression<Real>, Expression<Real>> getInverseGamma();
  #line 251 "src/expression/Random.birch"
  #line 251 "src/expression/Random.birch"
  Boolean hasInverseWishart();
  #line 255 "src/expression/Random.birch"
  #line 255 "src/expression/Random.birch"
  std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>> getInverseWishart();
  #line 259 "src/expression/Random.birch"
  #line 259 "src/expression/Random.birch"
  Boolean hasNormalInverseGamma();
  #line 263 "src/expression/Random.birch"
  #line 263 "src/expression/Random.birch"
  std::tuple<Expression<Real>, Expression<Real>, Expression<Real>, Expression<Real>> getNormalInverseGamma();
  #line 268 "src/expression/Random.birch"
  #line 268 "src/expression/Random.birch"
  Boolean hasDirichlet();
  #line 272 "src/expression/Random.birch"
  #line 272 "src/expression/Random.birch"
  Expression<libbirch::DefaultArray<Real,1>> getDirichlet();
  #line 276 "src/expression/Random.birch"
  #line 276 "src/expression/Random.birch"
  Boolean hasRestaurant();
  #line 280 "src/expression/Random.birch"
  #line 280 "src/expression/Random.birch"
  std::tuple<Expression<Real>, Expression<Real>, Expression<libbirch::DefaultArray<Integer,1>>, Expression<Integer>> getRestaurant();
  #line 285 "src/expression/Random.birch"
  #line 285 "src/expression/Random.birch"
  Boolean hasMultivariateGaussian();
  #line 289 "src/expression/Random.birch"
  #line 289 "src/expression/Random.birch"
  std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>> getMultivariateGaussian();
  #line 294 "src/expression/Random.birch"
  #line 294 "src/expression/Random.birch"
  Boolean hasMultivariateNormalInverseGamma();
  #line 298 "src/expression/Random.birch"
  #line 298 "src/expression/Random.birch"
  std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>, Expression<Real>> getMultivariateNormalInverseGamma();
  #line 303 "src/expression/Random.birch"
  #line 303 "src/expression/Random.birch"
  Boolean hasMatrixGaussian();
  #line 307 "src/expression/Random.birch"
  #line 307 "src/expression/Random.birch"
  std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>> getMatrixGaussian();
  #line 312 "src/expression/Random.birch"
  #line 312 "src/expression/Random.birch"
  Boolean hasMatrixNormalInverseWishart();
  #line 316 "src/expression/Random.birch"
  #line 316 "src/expression/Random.birch"
  std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>> getMatrixNormalInverseWishart();
  #line 321 "src/expression/Random.birch"
  #line 321 "src/expression/Random.birch"
  Boolean hasDiscrete();
  #line 325 "src/expression/Random.birch"
  #line 325 "src/expression/Random.birch"
  DiscreteDistribution getDiscrete();
  #line 329 "src/expression/Random.birch"
  #line 329 "src/expression/Random.birch"
  Boolean hasBoundedDiscrete();
  #line 333 "src/expression/Random.birch"
  #line 333 "src/expression/Random.birch"
  BoundedDiscreteDistribution getBoundedDiscrete();
  #line 337 "src/expression/Random.birch"
  #line 337 "src/expression/Random.birch"
  void read(const Buffer& buffer) override;
  #line 341 "src/expression/Random.birch"
  #line 341 "src/expression/Random.birch"
  void write(const Buffer& buffer) override;
};

#line 30 "src/event/Handler.birch"
#line 30 "src/event/Handler.birch"
class Handler_ final : public Object_ {
public:
  #line 30 "src/event/Handler.birch"
  LIBBIRCH_CLASS(Handler_, Object_)
  #line 30 "src/event/Handler.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c0, b1, i1, i2, r1, r2, r3, n, w, delaySampling, delayExpressions)
  #line 30 "src/event/Handler.birch"
  using base_type_::operator=;
  
  #line 30 "src/event/Handler.birch"
  Handler_(const Boolean& delaySampling, const Boolean& delayExpressions);

  std::optional<Expression<Real>> \u03c0;
  std::optional<Tape<Random<Boolean>>> b1;
  std::optional<Tape<Random<Integer>>> i1;
  std::optional<Tape<Random<libbirch::DefaultArray<Integer,1>>>> i2;
  std::optional<Tape<Random<Real>>> r1;
  std::optional<Tape<Random<libbirch::DefaultArray<Real,1>>>> r2;
  std::optional<Tape<Random<libbirch::DefaultArray<Real,2>>>> r3;
  Integer n;
  Real w;
  Boolean delaySampling;
  Boolean delayExpressions;
  #line 74 "src/event/Handler.birch"
  template<class Value>
  #line 74 "src/event/Handler.birch"
  Random<Value> handleAssume(const Random<Value>& x, const Distribution<Value>& p);
  #line 94 "src/event/Handler.birch"
  template<class Value>
  #line 94 "src/event/Handler.birch"
  Random<Value> handleSimulate(const Random<Value>& x, const Distribution<Value>& p);
  #line 108 "src/event/Handler.birch"
  template<class Value>
  #line 108 "src/event/Handler.birch"
  Value handleSimulate(const Distribution<Value>& p);
  #line 128 "src/event/Handler.birch"
  template<class Value>
  #line 128 "src/event/Handler.birch"
  Random<Value> handleObserve(const Random<Value>& x, const Distribution<Value>& p);
  #line 143 "src/event/Handler.birch"
  template<class Value>
  #line 143 "src/event/Handler.birch"
  Value handleObserve(const Value& x, const Distribution<Value>& p);
  #line 160 "src/event/Handler.birch"
  template<class Arg>
  #line 160 "src/event/Handler.birch"
  void handleFactor(const Arg& w);
  #line 178 "src/event/Handler.birch"
  template<class Value>
  #line 178 "src/event/Handler.birch"
  void handleDelaySimulate(const Random<Value>& x);
  #line 194 "src/event/Handler.birch"
  template<class Value>
  #line 194 "src/event/Handler.birch"
  void handleDelayObserve(const Random<Value>& x);
  #line 212 "src/event/Handler.birch"
  template<class Value>
  #line 212 "src/event/Handler.birch"
  Delay handleDelayPrune(const Distribution<Value>& p, const Random<Value>& x);
  #line 229 "src/event/Handler.birch"
  template<class Value>
  #line 229 "src/event/Handler.birch"
  Delay handleDelayPrune(const Distribution<Value>& p, const Value& x);
  #line 240 "src/event/Handler.birch"
  #line 240 "src/event/Handler.birch"
  Real peek();
  #line 251 "src/event/Handler.birch"
  #line 251 "src/event/Handler.birch"
  libbirch::DefaultArray<Real,1> args();
  #line 300 "src/event/Handler.birch"
  #line 300 "src/event/Handler.birch"
  libbirch::DefaultArray<Real,1> grad();
  #line 342 "src/event/Handler.birch"
  #line 342 "src/event/Handler.birch"
  Real move(const libbirch::DefaultArray<Real,1>& x);
  #line 386 "src/event/Handler.birch"
  #line 386 "src/event/Handler.birch"
  void label(const Integer& gen);
  #line 397 "src/event/Handler.birch"
  #line 397 "src/event/Handler.birch"
  void constant(const Integer& gen);
  #line 403 "src/event/Handler.birch"
  #line 403 "src/event/Handler.birch"
  void arg(const Random<Boolean>& x);
  #line 418 "src/event/Handler.birch"
  #line 418 "src/event/Handler.birch"
  void arg(const Random<Integer>& x);
  #line 433 "src/event/Handler.birch"
  #line 433 "src/event/Handler.birch"
  void arg(const Random<libbirch::DefaultArray<Integer,1>>& x);
  #line 448 "src/event/Handler.birch"
  #line 448 "src/event/Handler.birch"
  void arg(const Random<Real>& x);
  #line 463 "src/event/Handler.birch"
  #line 463 "src/event/Handler.birch"
  void arg(const Random<libbirch::DefaultArray<Real,1>>& x);
  #line 478 "src/event/Handler.birch"
  #line 478 "src/event/Handler.birch"
  void arg(const Random<libbirch::DefaultArray<Real,2>>& x);
};

#line 1 "src/filter/Particle.birch"
#line 1 "src/filter/Particle.birch"
class Particle_ : public Object_ {
public:
  #line 1 "src/filter/Particle.birch"
  LIBBIRCH_CLASS(Particle_, Object_)
  #line 1 "src/filter/Particle.birch"
  LIBBIRCH_CLASS_MEMBERS(m, h)
  #line 1 "src/filter/Particle.birch"
  using base_type_::operator=;
  #line 1 "src/filter/Particle.birch"
  using base_type_::read;
  #line 1 "src/filter/Particle.birch"
  using base_type_::write;
  
  #line 1 "src/filter/Particle.birch"
  Particle_(const Model& m, const Boolean& delaySampling, const Boolean& delayExpressions);

  Model m;
  Handler h;
  #line 22 "src/filter/Particle.birch"
  #line 22 "src/filter/Particle.birch"
  virtual Real peek();
  #line 29 "src/filter/Particle.birch"
  #line 29 "src/filter/Particle.birch"
  virtual libbirch::DefaultArray<Real,1> args();
  #line 36 "src/filter/Particle.birch"
  #line 36 "src/filter/Particle.birch"
  virtual libbirch::DefaultArray<Real,1> grad();
  #line 43 "src/filter/Particle.birch"
  #line 43 "src/filter/Particle.birch"
  virtual Real move(const libbirch::DefaultArray<Real,1>& x);
  #line 52 "src/filter/Particle.birch"
  #line 52 "src/filter/Particle.birch"
  virtual void label(const Integer& gen);
  #line 61 "src/filter/Particle.birch"
  #line 61 "src/filter/Particle.birch"
  virtual void constant(const Integer& gen);
  #line 65 "src/filter/Particle.birch"
  #line 65 "src/filter/Particle.birch"
  virtual void read(const Buffer& buffer) override;
  #line 71 "src/filter/Particle.birch"
  #line 71 "src/filter/Particle.birch"
  virtual void write(const Buffer& buffer) override;
  #line 77 "src/filter/Particle.birch"
  #line 77 "src/filter/Particle.birch"
  virtual void read(const Integer& t, const Buffer& buffer) override;
  #line 83 "src/filter/Particle.birch"
  #line 83 "src/filter/Particle.birch"
  virtual void write(const Integer& t, const Buffer& buffer) override;
};

#line 1 "src/filter/ParticleFilter.birch"
#line 1 "src/filter/ParticleFilter.birch"
class ParticleFilter_ : public Object_ {
public:
  #line 1 "src/filter/ParticleFilter.birch"
  LIBBIRCH_CLASS(ParticleFilter_, Object_)
  #line 1 "src/filter/ParticleFilter.birch"
  LIBBIRCH_CLASS_MEMBERS(x, w, a, b, lsum, ess, lnormalize, npropagations, raccepts, nparticles, trigger, delayed, autodiff)
  #line 1 "src/filter/ParticleFilter.birch"
  using base_type_::operator=;
  #line 1 "src/filter/ParticleFilter.birch"
  using base_type_::read;
  
  #line 1 "src/filter/ParticleFilter.birch"
  ParticleFilter_();

  libbirch::DefaultArray<Particle,1> x;
  libbirch::DefaultArray<Real,1> w;
  libbirch::DefaultArray<Integer,1> a;
  Integer b;
  Real lsum;
  Real ess;
  Real lnormalize;
  Integer npropagations;
  Real raccepts;
  Integer nparticles;
  Real trigger;
  Boolean delayed;
  Boolean autodiff;
  #line 86 "src/filter/ParticleFilter.birch"
  #line 86 "src/filter/ParticleFilter.birch"
  virtual Particle particle(const Model& model);
  #line 96 "src/filter/ParticleFilter.birch"
  #line 96 "src/filter/ParticleFilter.birch"
  virtual void filter(const Model& model, const Buffer& input);
  #line 115 "src/filter/ParticleFilter.birch"
  #line 115 "src/filter/ParticleFilter.birch"
  virtual void filter(const Integer& t, const Buffer& input);
  #line 128 "src/filter/ParticleFilter.birch"
  #line 128 "src/filter/ParticleFilter.birch"
  virtual void filter(const Integer& t, const Buffer& input, const Kernel& \u03ba);
  #line 138 "src/filter/ParticleFilter.birch"
  #line 138 "src/filter/ParticleFilter.birch"
  virtual void simulate(const Buffer& input);
  #line 154 "src/filter/ParticleFilter.birch"
  #line 154 "src/filter/ParticleFilter.birch"
  virtual void simulate(const Integer& t, const Buffer& input);
  #line 171 "src/filter/ParticleFilter.birch"
  #line 171 "src/filter/ParticleFilter.birch"
  virtual void reduce();
  #line 185 "src/filter/ParticleFilter.birch"
  #line 185 "src/filter/ParticleFilter.birch"
  virtual void resample(const Integer& t);
  #line 201 "src/filter/ParticleFilter.birch"
  #line 201 "src/filter/ParticleFilter.birch"
  virtual void copy();
  #line 228 "src/filter/ParticleFilter.birch"
  #line 228 "src/filter/ParticleFilter.birch"
  virtual void move(const Integer& t, const Kernel& \u03ba);
  #line 236 "src/filter/ParticleFilter.birch"
  #line 236 "src/filter/ParticleFilter.birch"
  virtual void read(const Buffer& buffer) override;
};

#line 1 "src/filter/ParticleFilter.birch"
extern "C" ParticleFilter_* make_ParticleFilter_();

#line 1 "src/filter/AliveParticleFilter.birch"
#line 1 "src/filter/AliveParticleFilter.birch"
class AliveParticleFilter_ : public libbirch::unwrap_pointer<ParticleFilter>::type {
public:
  #line 1 "src/filter/AliveParticleFilter.birch"
  LIBBIRCH_CLASS(AliveParticleFilter_, typename libbirch::unwrap_pointer<ParticleFilter>::type)
  #line 1 "src/filter/AliveParticleFilter.birch"
  LIBBIRCH_CLASS_MEMBERS(p)
  #line 1 "src/filter/AliveParticleFilter.birch"
  using base_type_::operator=;
  #line 1 "src/filter/AliveParticleFilter.birch"
  using base_type_::reduce;
  #line 1 "src/filter/AliveParticleFilter.birch"
  using base_type_::resample;
  #line 1 "src/filter/AliveParticleFilter.birch"
  using base_type_::simulate;
  
  #line 1 "src/filter/AliveParticleFilter.birch"
  AliveParticleFilter_();

  libbirch::DefaultArray<Integer,1> p;
  #line 21 "src/filter/AliveParticleFilter.birch"
  #line 21 "src/filter/AliveParticleFilter.birch"
  virtual void simulate(const Integer& t, const Buffer& input) override;
  #line 60 "src/filter/AliveParticleFilter.birch"
  #line 60 "src/filter/AliveParticleFilter.birch"
  virtual void resample(const Integer& t) override;
  #line 67 "src/filter/AliveParticleFilter.birch"
  #line 67 "src/filter/AliveParticleFilter.birch"
  virtual void reduce() override;
};

#line 1 "src/filter/AliveParticleFilter.birch"
extern "C" AliveParticleFilter_* make_AliveParticleFilter_();

#line 4 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 4 "src/form/Binary.birch"
struct Binary_ final {
public:
  #line 4 "src/form/Binary.birch"
  using Function_ = Function;
  #line 4 "src/form/Binary.birch"
  using Left_ = Left;
  #line 4 "src/form/Binary.birch"
  using Right_ = Right;
  #line 4 "src/form/Binary.birch"
  LIBBIRCH_STRUCT(Binary_)
  #line 4 "src/form/Binary.birch"
  LIBBIRCH_STRUCT_MEMBERS(f, l, r)
  #line 4 "src/form/Binary.birch"
  Binary_(const Left& l, const Right& r);

  Function f;
  Left l;
  Right r;
  #line 20 "src/form/Binary.birch"
  #line 20 "src/form/Binary.birch"
  decltype(auto) value();
  #line 24 "src/form/Binary.birch"
  #line 24 "src/form/Binary.birch"
  decltype(auto) peek();
  #line 28 "src/form/Binary.birch"
  #line 28 "src/form/Binary.birch"
  decltype(auto) eval();
  #line 32 "src/form/Binary.birch"
  #line 32 "src/form/Binary.birch"
  decltype(auto) reval();
  #line 36 "src/form/Binary.birch"
  template<class Gradient>
  #line 36 "src/form/Binary.birch"
  void grad(const Gradient& d);
  #line 42 "src/form/Binary.birch"
  #line 42 "src/form/Binary.birch"
  void label(const Integer& gen);
  #line 47 "src/form/Binary.birch"
  #line 47 "src/form/Binary.birch"
  void constant(const Integer& gen);
  #line 52 "src/form/Binary.birch"
  #line 52 "src/form/Binary.birch"
  void constant();
};

#line 4 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 4 "src/form/BinaryMemoized.birch"
struct BinaryMemoized_ final {
public:
  #line 4 "src/form/BinaryMemoized.birch"
  using Function_ = Function;
  #line 4 "src/form/BinaryMemoized.birch"
  using Left_ = Left;
  #line 4 "src/form/BinaryMemoized.birch"
  using Right_ = Right;
  #line 4 "src/form/BinaryMemoized.birch"
  LIBBIRCH_STRUCT(BinaryMemoized_)
  #line 4 "src/form/BinaryMemoized.birch"
  LIBBIRCH_STRUCT_MEMBERS(f, l, r)
  #line 4 "src/form/BinaryMemoized.birch"
  BinaryMemoized_(const Left& l, const Right& r);

  Function f;
  Left l;
  Right r;
  #line 20 "src/form/BinaryMemoized.birch"

  /**
   * Memoized result. This is declared with C++ as Birch does not provide an
   * equivalent to the decltype() operator, and we do not wish to evaluate the
   * form in order to determine the result type at the time of construction;
   * doing so could, for example, force the realization of Random objects that
   * could otherwise be marginalized out, e.g. in `A*x + c`, where `x` is
   * Random.
   */
  std::optional<decltype(f->compute(birch::value(l), birch::value(r)))> x;
    #line 32 "src/form/BinaryMemoized.birch"
  #line 32 "src/form/BinaryMemoized.birch"
  decltype(auto) value();
  #line 39 "src/form/BinaryMemoized.birch"
  #line 39 "src/form/BinaryMemoized.birch"
  decltype(auto) peek();
  #line 46 "src/form/BinaryMemoized.birch"
  #line 46 "src/form/BinaryMemoized.birch"
  decltype(auto) eval();
  #line 52 "src/form/BinaryMemoized.birch"
  #line 52 "src/form/BinaryMemoized.birch"
  decltype(auto) reval();
  #line 58 "src/form/BinaryMemoized.birch"
  template<class Gradient>
  #line 58 "src/form/BinaryMemoized.birch"
  void grad(const Gradient& d);
  #line 64 "src/form/BinaryMemoized.birch"
  #line 64 "src/form/BinaryMemoized.birch"
  void label(const Integer& gen);
  #line 69 "src/form/BinaryMemoized.birch"
  #line 69 "src/form/BinaryMemoized.birch"
  void constant(const Integer& gen);
  #line 74 "src/form/BinaryMemoized.birch"
  #line 74 "src/form/BinaryMemoized.birch"
  void constant();
};

#line 5 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 5 "src/form/Linear.birch"
struct Linear_ final {
public:
  #line 5 "src/form/Linear.birch"
  using Function_ = Function;
  #line 5 "src/form/Linear.birch"
  using Left_ = Left;
  #line 5 "src/form/Linear.birch"
  using Right_ = Right;
  #line 5 "src/form/Linear.birch"
  LIBBIRCH_STRUCT(Linear_)
  #line 5 "src/form/Linear.birch"
  LIBBIRCH_STRUCT_MEMBERS(f, l, r)
  #line 5 "src/form/Linear.birch"
  Linear_(const Left& l, const Right& r);

  Function f;
  Left l;
  Right r;
  #line 21 "src/form/Linear.birch"
  #line 21 "src/form/Linear.birch"
  decltype(auto) value();
  #line 25 "src/form/Linear.birch"
  #line 25 "src/form/Linear.birch"
  decltype(auto) peek();
  #line 29 "src/form/Linear.birch"
  #line 29 "src/form/Linear.birch"
  decltype(auto) eval();
  #line 33 "src/form/Linear.birch"
  #line 33 "src/form/Linear.birch"
  decltype(auto) reval();
  #line 37 "src/form/Linear.birch"
  template<class Gradient>
  #line 37 "src/form/Linear.birch"
  void grad(const Gradient& d);
  #line 43 "src/form/Linear.birch"
  #line 43 "src/form/Linear.birch"
  void label(const Integer& gen);
  #line 48 "src/form/Linear.birch"
  #line 48 "src/form/Linear.birch"
  void constant(const Integer& gen);
  #line 53 "src/form/Linear.birch"
  #line 53 "src/form/Linear.birch"
  void constant();
};

#line 5 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 5 "src/form/LinearMemoized.birch"
struct LinearMemoized_ final {
public:
  #line 5 "src/form/LinearMemoized.birch"
  using Function_ = Function;
  #line 5 "src/form/LinearMemoized.birch"
  using Left_ = Left;
  #line 5 "src/form/LinearMemoized.birch"
  using Right_ = Right;
  #line 5 "src/form/LinearMemoized.birch"
  LIBBIRCH_STRUCT(LinearMemoized_)
  #line 5 "src/form/LinearMemoized.birch"
  LIBBIRCH_STRUCT_MEMBERS(f, l, r)
  #line 5 "src/form/LinearMemoized.birch"
  LinearMemoized_(const Left& l, const Right& r);

  Function f;
  Left l;
  Right r;
  #line 21 "src/form/LinearMemoized.birch"

  /**
   * Memoized result. This is declared with C++ as Birch does not provide an
   * equivalent to the decltype() operator, and we do not wish to evaluate the
   * form in order to determine the result type at the time of construction;
   * doing so could, for example, force the realization of Random objects that
   * could otherwise be marginalized out, e.g. in `A*x + c`, where `x` is
   * Random.
   */
  std::optional<decltype(f->compute(birch::value(l), birch::value(r)))> x;
    #line 33 "src/form/LinearMemoized.birch"
  #line 33 "src/form/LinearMemoized.birch"
  decltype(auto) value();
  #line 40 "src/form/LinearMemoized.birch"
  #line 40 "src/form/LinearMemoized.birch"
  decltype(auto) peek();
  #line 47 "src/form/LinearMemoized.birch"
  #line 47 "src/form/LinearMemoized.birch"
  decltype(auto) eval();
  #line 53 "src/form/LinearMemoized.birch"
  #line 53 "src/form/LinearMemoized.birch"
  decltype(auto) reval();
  #line 59 "src/form/LinearMemoized.birch"
  template<class Gradient>
  #line 59 "src/form/LinearMemoized.birch"
  void grad(const Gradient& d);
  #line 65 "src/form/LinearMemoized.birch"
  #line 65 "src/form/LinearMemoized.birch"
  void label(const Integer& gen);
  #line 70 "src/form/LinearMemoized.birch"
  #line 70 "src/form/LinearMemoized.birch"
  void constant(const Integer& gen);
  #line 75 "src/form/LinearMemoized.birch"
  #line 75 "src/form/LinearMemoized.birch"
  void constant();
};

#line 4 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 4 "src/form/MatrixElement.birch"
struct MatrixElement_ final {
public:
  #line 4 "src/form/MatrixElement.birch"
  using Left_ = Left;
  #line 4 "src/form/MatrixElement.birch"
  using Middle_ = Middle;
  #line 4 "src/form/MatrixElement.birch"
  using Right_ = Right;
  #line 4 "src/form/MatrixElement.birch"
  LIBBIRCH_STRUCT(MatrixElement_)
  #line 4 "src/form/MatrixElement.birch"
  LIBBIRCH_STRUCT_MEMBERS(l, m, r)
  #line 4 "src/form/MatrixElement.birch"
  MatrixElement_(const Left& l, const Middle& m, const Right& r);

  Left l;
  Middle m;
  Right r;
  #line 20 "src/form/MatrixElement.birch"
  #line 20 "src/form/MatrixElement.birch"
  decltype(auto) value();
  #line 24 "src/form/MatrixElement.birch"
  #line 24 "src/form/MatrixElement.birch"
  decltype(auto) peek();
  #line 28 "src/form/MatrixElement.birch"
  #line 28 "src/form/MatrixElement.birch"
  decltype(auto) eval();
  #line 32 "src/form/MatrixElement.birch"
  #line 32 "src/form/MatrixElement.birch"
  decltype(auto) reval();
  #line 36 "src/form/MatrixElement.birch"
  template<class Gradient>
  #line 36 "src/form/MatrixElement.birch"
  void grad(const Gradient& d);
  #line 40 "src/form/MatrixElement.birch"
  #line 40 "src/form/MatrixElement.birch"
  void label(const Integer& gen);
  #line 46 "src/form/MatrixElement.birch"
  #line 46 "src/form/MatrixElement.birch"
  void constant(const Integer& gen);
  #line 52 "src/form/MatrixElement.birch"
  #line 52 "src/form/MatrixElement.birch"
  void constant();
};

#line 6 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 6 "src/form/MatrixJoinForm.birch"
struct MatrixJoinForm_ final {
public:
  #line 6 "src/form/MatrixJoinForm.birch"
  using Value_ = Value;
  #line 6 "src/form/MatrixJoinForm.birch"
  LIBBIRCH_STRUCT(MatrixJoinForm_)
  #line 6 "src/form/MatrixJoinForm.birch"
  LIBBIRCH_STRUCT_MEMBERS(x)
  #line 6 "src/form/MatrixJoinForm.birch"
  MatrixJoinForm_(const libbirch::DefaultArray<Expression<Value>,2>& x);

  std::optional<libbirch::DefaultArray<Expression<Value>,2>> x;
  #line 12 "src/form/MatrixJoinForm.birch"
  #line 12 "src/form/MatrixJoinForm.birch"
  libbirch::DefaultArray<Value,2> value();
  #line 18 "src/form/MatrixJoinForm.birch"
  #line 18 "src/form/MatrixJoinForm.birch"
  libbirch::DefaultArray<Value,2> peek();
  #line 24 "src/form/MatrixJoinForm.birch"
  #line 24 "src/form/MatrixJoinForm.birch"
  libbirch::DefaultArray<Value,2> eval();
  #line 30 "src/form/MatrixJoinForm.birch"
  #line 30 "src/form/MatrixJoinForm.birch"
  libbirch::DefaultArray<Value,2> reval();
  #line 36 "src/form/MatrixJoinForm.birch"
  #line 36 "src/form/MatrixJoinForm.birch"
  void grad(const libbirch::DefaultArray<Value,2>& d);
  #line 42 "src/form/MatrixJoinForm.birch"
  #line 42 "src/form/MatrixJoinForm.birch"
  void label(const Integer& gen);
  #line 46 "src/form/MatrixJoinForm.birch"
  #line 46 "src/form/MatrixJoinForm.birch"
  void constant(const Integer& gen);
  #line 50 "src/form/MatrixJoinForm.birch"
  #line 50 "src/form/MatrixJoinForm.birch"
  void constant();
};

#line 4 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 4 "src/form/Ternary.birch"
struct Ternary_ final {
public:
  #line 4 "src/form/Ternary.birch"
  using Function_ = Function;
  #line 4 "src/form/Ternary.birch"
  using Left_ = Left;
  #line 4 "src/form/Ternary.birch"
  using Middle_ = Middle;
  #line 4 "src/form/Ternary.birch"
  using Right_ = Right;
  #line 4 "src/form/Ternary.birch"
  LIBBIRCH_STRUCT(Ternary_)
  #line 4 "src/form/Ternary.birch"
  LIBBIRCH_STRUCT_MEMBERS(f, l, m, r)
  #line 4 "src/form/Ternary.birch"
  Ternary_(const Left& l, const Middle& m, const Right& r);

  Function f;
  Left l;
  Middle m;
  Right r;
  #line 25 "src/form/Ternary.birch"
  #line 25 "src/form/Ternary.birch"
  decltype(auto) value();
  #line 29 "src/form/Ternary.birch"
  #line 29 "src/form/Ternary.birch"
  decltype(auto) peek();
  #line 33 "src/form/Ternary.birch"
  #line 33 "src/form/Ternary.birch"
  decltype(auto) eval();
  #line 37 "src/form/Ternary.birch"
  #line 37 "src/form/Ternary.birch"
  decltype(auto) reval();
  #line 41 "src/form/Ternary.birch"
  template<class Gradient>
  #line 41 "src/form/Ternary.birch"
  void grad(const Gradient& d);
  #line 49 "src/form/Ternary.birch"
  #line 49 "src/form/Ternary.birch"
  void label(const Integer& gen);
  #line 55 "src/form/Ternary.birch"
  #line 55 "src/form/Ternary.birch"
  void constant(const Integer& gen);
  #line 61 "src/form/Ternary.birch"
  #line 61 "src/form/Ternary.birch"
  void constant();
};

#line 4 "src/form/Unary.birch"
template<class Function,class Middle>
#line 4 "src/form/Unary.birch"
struct Unary_ final {
public:
  #line 4 "src/form/Unary.birch"
  using Function_ = Function;
  #line 4 "src/form/Unary.birch"
  using Middle_ = Middle;
  #line 4 "src/form/Unary.birch"
  LIBBIRCH_STRUCT(Unary_)
  #line 4 "src/form/Unary.birch"
  LIBBIRCH_STRUCT_MEMBERS(f, m)
  #line 4 "src/form/Unary.birch"
  Unary_(const Middle& m);

  Function f;
  Middle m;
  #line 15 "src/form/Unary.birch"
  #line 15 "src/form/Unary.birch"
  decltype(auto) value();
  #line 19 "src/form/Unary.birch"
  #line 19 "src/form/Unary.birch"
  decltype(auto) peek();
  #line 23 "src/form/Unary.birch"
  #line 23 "src/form/Unary.birch"
  decltype(auto) eval();
  #line 27 "src/form/Unary.birch"
  #line 27 "src/form/Unary.birch"
  decltype(auto) reval();
  #line 31 "src/form/Unary.birch"
  template<class Gradient>
  #line 31 "src/form/Unary.birch"
  void grad(const Gradient& d);
  #line 35 "src/form/Unary.birch"
  #line 35 "src/form/Unary.birch"
  void label(const Integer& gen);
  #line 39 "src/form/Unary.birch"
  #line 39 "src/form/Unary.birch"
  void constant(const Integer& gen);
  #line 43 "src/form/Unary.birch"
  #line 43 "src/form/Unary.birch"
  void constant();
};

#line 4 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 4 "src/form/UnaryMemoized.birch"
struct UnaryMemoized_ final {
public:
  #line 4 "src/form/UnaryMemoized.birch"
  using Function_ = Function;
  #line 4 "src/form/UnaryMemoized.birch"
  using Middle_ = Middle;
  #line 4 "src/form/UnaryMemoized.birch"
  LIBBIRCH_STRUCT(UnaryMemoized_)
  #line 4 "src/form/UnaryMemoized.birch"
  LIBBIRCH_STRUCT_MEMBERS(f, m)
  #line 4 "src/form/UnaryMemoized.birch"
  UnaryMemoized_(const Middle& m);

  Function f;
  Middle m;
  #line 15 "src/form/UnaryMemoized.birch"

  /**
   * Memoized result. This is declared with C++ as Birch does not provide an
   * equivalent to the decltype() operator, and we do not wish to evaluate the
   * form in order to determine the result type at the time of construction;
   * doing so could, for example, force the realization of Random objects that
   * could otherwise be marginalized out, e.g. in `A*x + c`, where `x` is
   * Random.
   */
  std::optional<decltype(f->compute(birch::value(m)))> x;
    #line 27 "src/form/UnaryMemoized.birch"
  #line 27 "src/form/UnaryMemoized.birch"
  decltype(auto) value();
  #line 34 "src/form/UnaryMemoized.birch"
  #line 34 "src/form/UnaryMemoized.birch"
  decltype(auto) peek();
  #line 41 "src/form/UnaryMemoized.birch"
  #line 41 "src/form/UnaryMemoized.birch"
  decltype(auto) eval();
  #line 47 "src/form/UnaryMemoized.birch"
  #line 47 "src/form/UnaryMemoized.birch"
  decltype(auto) reval();
  #line 53 "src/form/UnaryMemoized.birch"
  template<class Gradient>
  #line 53 "src/form/UnaryMemoized.birch"
  void grad(const Gradient& d);
  #line 57 "src/form/UnaryMemoized.birch"
  #line 57 "src/form/UnaryMemoized.birch"
  void label(const Integer& gen);
  #line 61 "src/form/UnaryMemoized.birch"
  #line 61 "src/form/UnaryMemoized.birch"
  void constant(const Integer& gen);
  #line 65 "src/form/UnaryMemoized.birch"
  #line 65 "src/form/UnaryMemoized.birch"
  void constant();
};

#line 4 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 4 "src/form/VectorElement.birch"
struct VectorElement_ final {
public:
  #line 4 "src/form/VectorElement.birch"
  using Left_ = Left;
  #line 4 "src/form/VectorElement.birch"
  using Right_ = Right;
  #line 4 "src/form/VectorElement.birch"
  LIBBIRCH_STRUCT(VectorElement_)
  #line 4 "src/form/VectorElement.birch"
  LIBBIRCH_STRUCT_MEMBERS(l, r)
  #line 4 "src/form/VectorElement.birch"
  VectorElement_(const Left& l, const Right& r);

  Left l;
  Right r;
  #line 15 "src/form/VectorElement.birch"
  #line 15 "src/form/VectorElement.birch"
  decltype(auto) value();
  #line 19 "src/form/VectorElement.birch"
  #line 19 "src/form/VectorElement.birch"
  decltype(auto) peek();
  #line 23 "src/form/VectorElement.birch"
  #line 23 "src/form/VectorElement.birch"
  decltype(auto) eval();
  #line 27 "src/form/VectorElement.birch"
  #line 27 "src/form/VectorElement.birch"
  decltype(auto) reval();
  #line 31 "src/form/VectorElement.birch"
  template<class Gradient>
  #line 31 "src/form/VectorElement.birch"
  void grad(const Gradient& d);
  #line 35 "src/form/VectorElement.birch"
  #line 35 "src/form/VectorElement.birch"
  void label(const Integer& gen);
  #line 40 "src/form/VectorElement.birch"
  #line 40 "src/form/VectorElement.birch"
  void constant(const Integer& gen);
  #line 45 "src/form/VectorElement.birch"
  #line 45 "src/form/VectorElement.birch"
  void constant();
};

#line 6 "src/form/VectorJoinForm.birch"
template<class Value>
#line 6 "src/form/VectorJoinForm.birch"
struct VectorJoinForm_ final {
public:
  #line 6 "src/form/VectorJoinForm.birch"
  using Value_ = Value;
  #line 6 "src/form/VectorJoinForm.birch"
  LIBBIRCH_STRUCT(VectorJoinForm_)
  #line 6 "src/form/VectorJoinForm.birch"
  LIBBIRCH_STRUCT_MEMBERS(x)
  #line 6 "src/form/VectorJoinForm.birch"
  VectorJoinForm_(const libbirch::DefaultArray<Expression<Value>,1>& x);

  libbirch::DefaultArray<Expression<Value>,1> x;
  #line 12 "src/form/VectorJoinForm.birch"
  #line 12 "src/form/VectorJoinForm.birch"
  libbirch::DefaultArray<Value,1> value();
  #line 18 "src/form/VectorJoinForm.birch"
  #line 18 "src/form/VectorJoinForm.birch"
  libbirch::DefaultArray<Value,1> peek();
  #line 24 "src/form/VectorJoinForm.birch"
  #line 24 "src/form/VectorJoinForm.birch"
  libbirch::DefaultArray<Value,1> eval();
  #line 30 "src/form/VectorJoinForm.birch"
  #line 30 "src/form/VectorJoinForm.birch"
  libbirch::DefaultArray<Value,1> reval();
  #line 36 "src/form/VectorJoinForm.birch"
  #line 36 "src/form/VectorJoinForm.birch"
  void grad(const libbirch::DefaultArray<Value,1>& d);
  #line 42 "src/form/VectorJoinForm.birch"
  #line 42 "src/form/VectorJoinForm.birch"
  void label(const Integer& gen);
  #line 46 "src/form/VectorJoinForm.birch"
  #line 46 "src/form/VectorJoinForm.birch"
  void constant(const Integer& gen);
  #line 50 "src/form/VectorJoinForm.birch"
  #line 50 "src/form/VectorJoinForm.birch"
  void constant();
};

#line 4 "src/io/InputStream.birch"
#line 4 "src/io/InputStream.birch"
struct InputStream_ final {
public:
  #line 4 "src/io/InputStream.birch"
  LIBBIRCH_STRUCT(InputStream_)
  #line 4 "src/io/InputStream.birch"
  LIBBIRCH_STRUCT_MEMBERS(file)
  #line 4 "src/io/InputStream.birch"
  InputStream_();

  std::optional<File> file;
  #line 16 "src/io/InputStream.birch"
  #line 16 "src/io/InputStream.birch"
  void open(const String& path, const Integer& mode);
  #line 26 "src/io/InputStream.birch"
  #line 26 "src/io/InputStream.birch"
  void open(const String& path);
  #line 33 "src/io/InputStream.birch"
  #line 33 "src/io/InputStream.birch"
  void close();
  #line 44 "src/io/InputStream.birch"
  #line 44 "src/io/InputStream.birch"
  Boolean eof();
  #line 54 "src/io/InputStream.birch"
  #line 54 "src/io/InputStream.birch"
  std::optional<Integer> scanInteger();
  #line 70 "src/io/InputStream.birch"
  #line 70 "src/io/InputStream.birch"
  std::optional<Real> scanReal();
};

#line 4 "src/io/OutputStream.birch"
#line 4 "src/io/OutputStream.birch"
struct OutputStream_ final {
public:
  #line 4 "src/io/OutputStream.birch"
  LIBBIRCH_STRUCT(OutputStream_)
  #line 4 "src/io/OutputStream.birch"
  LIBBIRCH_STRUCT_MEMBERS(file)
  #line 4 "src/io/OutputStream.birch"
  OutputStream_();

  std::optional<File> file;
  #line 16 "src/io/OutputStream.birch"
  #line 16 "src/io/OutputStream.birch"
  void open(const String& path, const Integer& mode);
  #line 25 "src/io/OutputStream.birch"
  #line 25 "src/io/OutputStream.birch"
  void open(const String& path);
  #line 32 "src/io/OutputStream.birch"
  #line 32 "src/io/OutputStream.birch"
  void close();
  #line 43 "src/io/OutputStream.birch"
  #line 43 "src/io/OutputStream.birch"
  void flush();
  #line 53 "src/io/OutputStream.birch"
  #line 53 "src/io/OutputStream.birch"
  void print(const String& value);
  #line 63 "src/io/OutputStream.birch"
  #line 63 "src/io/OutputStream.birch"
  void print(const Boolean& value);
  #line 70 "src/io/OutputStream.birch"
  #line 70 "src/io/OutputStream.birch"
  void print(const Integer& value);
  #line 77 "src/io/OutputStream.birch"
  #line 77 "src/io/OutputStream.birch"
  void print(const Real& value);
  #line 84 "src/io/OutputStream.birch"
  #line 84 "src/io/OutputStream.birch"
  void print(const libbirch::DefaultArray<Boolean,1>& value);
  #line 91 "src/io/OutputStream.birch"
  #line 91 "src/io/OutputStream.birch"
  void print(const libbirch::DefaultArray<Integer,1>& value);
  #line 98 "src/io/OutputStream.birch"
  #line 98 "src/io/OutputStream.birch"
  void print(const libbirch::DefaultArray<Real,1>& value);
  #line 105 "src/io/OutputStream.birch"
  #line 105 "src/io/OutputStream.birch"
  void print(const libbirch::DefaultArray<Boolean,2>& value);
  #line 112 "src/io/OutputStream.birch"
  #line 112 "src/io/OutputStream.birch"
  void print(const libbirch::DefaultArray<Integer,2>& value);
  #line 119 "src/io/OutputStream.birch"
  #line 119 "src/io/OutputStream.birch"
  void print(const libbirch::DefaultArray<Real,2>& value);
};

#line 37 "src/io/Writer.birch"
#line 37 "src/io/Writer.birch"
class Writer_ : public Object_ {
public:
  #line 37 "src/io/Writer.birch"
  LIBBIRCH_ABSTRACT_CLASS(Writer_, Object_)
  #line 37 "src/io/Writer.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 37 "src/io/Writer.birch"
  using base_type_::operator=;
  
  #line 37 "src/io/Writer.birch"
  Writer_();

  #line 43 "src/io/Writer.birch"
  #line 43 "src/io/Writer.birch"
  virtual void open(const String& path) = 0;
  #line 50 "src/io/Writer.birch"
  #line 50 "src/io/Writer.birch"
  virtual void dump(const Buffer& buffer) = 0;
  #line 59 "src/io/Writer.birch"
  #line 59 "src/io/Writer.birch"
  virtual void push(const Buffer& buffer) = 0;
  #line 64 "src/io/Writer.birch"
  #line 64 "src/io/Writer.birch"
  virtual void flush() = 0;
  #line 69 "src/io/Writer.birch"
  #line 69 "src/io/Writer.birch"
  virtual void close() = 0;
  #line 71 "src/io/Writer.birch"
  #line 71 "src/io/Writer.birch"
  virtual void visit(const libbirch::DefaultArray<String,1>& keys, const libbirch::DefaultArray<Buffer,1>& values) = 0;
  #line 72 "src/io/Writer.birch"
  #line 72 "src/io/Writer.birch"
  virtual void visit(const libbirch::DefaultArray<Buffer,1>& values) = 0;
  #line 73 "src/io/Writer.birch"
  #line 73 "src/io/Writer.birch"
  virtual void visit(const String& value) = 0;
  #line 74 "src/io/Writer.birch"
  #line 74 "src/io/Writer.birch"
  virtual void visit(const Real& value) = 0;
  #line 75 "src/io/Writer.birch"
  #line 75 "src/io/Writer.birch"
  virtual void visit(const Integer& value) = 0;
  #line 76 "src/io/Writer.birch"
  #line 76 "src/io/Writer.birch"
  virtual void visit(const Boolean& value) = 0;
  #line 77 "src/io/Writer.birch"
  #line 77 "src/io/Writer.birch"
  virtual void visit(const libbirch::DefaultArray<String,1>& value) = 0;
  #line 78 "src/io/Writer.birch"
  #line 78 "src/io/Writer.birch"
  virtual void visit(const libbirch::DefaultArray<Real,1>& value) = 0;
  #line 79 "src/io/Writer.birch"
  #line 79 "src/io/Writer.birch"
  virtual void visit(const libbirch::DefaultArray<Integer,1>& value) = 0;
  #line 80 "src/io/Writer.birch"
  #line 80 "src/io/Writer.birch"
  virtual void visit(const libbirch::DefaultArray<Boolean,1>& value) = 0;
  #line 81 "src/io/Writer.birch"
  #line 81 "src/io/Writer.birch"
  virtual void visit(const libbirch::DefaultArray<String,2>& value) = 0;
  #line 82 "src/io/Writer.birch"
  #line 82 "src/io/Writer.birch"
  virtual void visit(const libbirch::DefaultArray<Real,2>& value) = 0;
  #line 83 "src/io/Writer.birch"
  #line 83 "src/io/Writer.birch"
  virtual void visit(const libbirch::DefaultArray<Integer,2>& value) = 0;
  #line 84 "src/io/Writer.birch"
  #line 84 "src/io/Writer.birch"
  virtual void visit(const libbirch::DefaultArray<Boolean,2>& value) = 0;
  #line 85 "src/io/Writer.birch"
  #line 85 "src/io/Writer.birch"
  virtual void visitNil() = 0;
};

#line 1 "src/io/YAMLWriter.birch"
#line 1 "src/io/YAMLWriter.birch"
class YAMLWriter_ : public libbirch::unwrap_pointer<Writer>::type {
public:
  #line 1 "src/io/YAMLWriter.birch"
  LIBBIRCH_CLASS(YAMLWriter_, typename libbirch::unwrap_pointer<Writer>::type)
  #line 1 "src/io/YAMLWriter.birch"
  LIBBIRCH_CLASS_MEMBERS(file, sequential)
  #line 1 "src/io/YAMLWriter.birch"
  using base_type_::operator=;
  #line 1 "src/io/YAMLWriter.birch"
  using base_type_::close;
  #line 1 "src/io/YAMLWriter.birch"
  using base_type_::dump;
  #line 1 "src/io/YAMLWriter.birch"
  using base_type_::flush;
  #line 1 "src/io/YAMLWriter.birch"
  using base_type_::open;
  #line 1 "src/io/YAMLWriter.birch"
  using base_type_::push;
  #line 1 "src/io/YAMLWriter.birch"
  using base_type_::visit;
  #line 1 "src/io/YAMLWriter.birch"
  using base_type_::visitNil;
  
  #line 1 "src/io/YAMLWriter.birch"
  YAMLWriter_();

  File file;
  Boolean sequential;
  #line 24 "src/io/YAMLWriter.birch"

  yaml_emitter_t emitter;
  yaml_event_t event;
    #line 29 "src/io/YAMLWriter.birch"
  #line 29 "src/io/YAMLWriter.birch"
  virtual void open(const String& path) override;
  #line 42 "src/io/YAMLWriter.birch"
  #line 42 "src/io/YAMLWriter.birch"
  virtual void dump(const Buffer& buffer) override;
  #line 46 "src/io/YAMLWriter.birch"
  #line 46 "src/io/YAMLWriter.birch"
  virtual void push(const Buffer& buffer) override;
  #line 54 "src/io/YAMLWriter.birch"
  #line 54 "src/io/YAMLWriter.birch"
  virtual void flush() override;
  #line 61 "src/io/YAMLWriter.birch"
  #line 61 "src/io/YAMLWriter.birch"
  virtual void close() override;
  #line 75 "src/io/YAMLWriter.birch"
  #line 75 "src/io/YAMLWriter.birch"
  virtual void visit(const libbirch::DefaultArray<String,1>& keys, const libbirch::DefaultArray<Buffer,1>& values) override;
  #line 85 "src/io/YAMLWriter.birch"
  #line 85 "src/io/YAMLWriter.birch"
  virtual void visit(const libbirch::DefaultArray<Buffer,1>& values) override;
  #line 93 "src/io/YAMLWriter.birch"
  #line 93 "src/io/YAMLWriter.birch"
  virtual void visit(const Boolean& value) override;
  #line 103 "src/io/YAMLWriter.birch"
  #line 103 "src/io/YAMLWriter.birch"
  virtual void visit(const Integer& value) override;
  #line 113 "src/io/YAMLWriter.birch"
  #line 113 "src/io/YAMLWriter.birch"
  virtual void visit(const Real& value) override;
  #line 135 "src/io/YAMLWriter.birch"
  #line 135 "src/io/YAMLWriter.birch"
  virtual void visit(const String& value) override;
  #line 144 "src/io/YAMLWriter.birch"
  #line 144 "src/io/YAMLWriter.birch"
  virtual void visit(const libbirch::DefaultArray<Boolean,1>& value) override;
  #line 152 "src/io/YAMLWriter.birch"
  #line 152 "src/io/YAMLWriter.birch"
  virtual void visit(const libbirch::DefaultArray<Integer,1>& value) override;
  #line 160 "src/io/YAMLWriter.birch"
  #line 160 "src/io/YAMLWriter.birch"
  virtual void visit(const libbirch::DefaultArray<Real,1>& value) override;
  #line 168 "src/io/YAMLWriter.birch"
  #line 168 "src/io/YAMLWriter.birch"
  virtual void visit(const libbirch::DefaultArray<String,1>& value) override;
  #line 176 "src/io/YAMLWriter.birch"
  #line 176 "src/io/YAMLWriter.birch"
  virtual void visit(const libbirch::DefaultArray<Boolean,2>& value) override;
  #line 184 "src/io/YAMLWriter.birch"
  #line 184 "src/io/YAMLWriter.birch"
  virtual void visit(const libbirch::DefaultArray<Integer,2>& value) override;
  #line 192 "src/io/YAMLWriter.birch"
  #line 192 "src/io/YAMLWriter.birch"
  virtual void visit(const libbirch::DefaultArray<Real,2>& value) override;
  #line 200 "src/io/YAMLWriter.birch"
  #line 200 "src/io/YAMLWriter.birch"
  virtual void visit(const libbirch::DefaultArray<String,2>& value) override;
  #line 208 "src/io/YAMLWriter.birch"
  #line 208 "src/io/YAMLWriter.birch"
  virtual void visitNil() override;
  #line 217 "src/io/YAMLWriter.birch"
  #line 217 "src/io/YAMLWriter.birch"
  virtual void startMapping();
  #line 225 "src/io/YAMLWriter.birch"
  #line 225 "src/io/YAMLWriter.birch"
  virtual void endMapping();
  #line 232 "src/io/YAMLWriter.birch"
  #line 232 "src/io/YAMLWriter.birch"
  virtual void startSequence();
  #line 240 "src/io/YAMLWriter.birch"
  #line 240 "src/io/YAMLWriter.birch"
  virtual void endSequence();
};

#line 1 "src/io/YAMLWriter.birch"
extern "C" YAMLWriter_* make_YAMLWriter_();

#line 1 "src/io/JSONWriter.birch"
#line 1 "src/io/JSONWriter.birch"
class JSONWriter_ : public libbirch::unwrap_pointer<YAMLWriter>::type {
public:
  #line 1 "src/io/JSONWriter.birch"
  LIBBIRCH_CLASS(JSONWriter_, typename libbirch::unwrap_pointer<YAMLWriter>::type)
  #line 1 "src/io/JSONWriter.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 1 "src/io/JSONWriter.birch"
  using base_type_::operator=;
  #line 1 "src/io/JSONWriter.birch"
  using base_type_::startMapping;
  #line 1 "src/io/JSONWriter.birch"
  using base_type_::startSequence;
  #line 1 "src/io/JSONWriter.birch"
  using base_type_::visit;
  
  #line 1 "src/io/JSONWriter.birch"
  JSONWriter_();

  #line 14 "src/io/JSONWriter.birch"
  #line 14 "src/io/JSONWriter.birch"
  virtual void visit(const String& value) override;
  #line 23 "src/io/JSONWriter.birch"
  #line 23 "src/io/JSONWriter.birch"
  virtual void startMapping() override;
  #line 31 "src/io/JSONWriter.birch"
  #line 31 "src/io/JSONWriter.birch"
  virtual void startSequence() override;
};

#line 1 "src/io/JSONWriter.birch"
extern "C" JSONWriter_* make_JSONWriter_();

#line 4 "src/iterator/Iterator.birch"
template<class Type>
#line 4 "src/iterator/Iterator.birch"
class Iterator_ : public Object_ {
public:
  #line 4 "src/iterator/Iterator.birch"
  using Type_ = Type;
  #line 4 "src/iterator/Iterator.birch"
  LIBBIRCH_ABSTRACT_CLASS(Iterator_, Object_)
  #line 4 "src/iterator/Iterator.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 4 "src/iterator/Iterator.birch"
  using base_type_::operator=;
  
  #line 4 "src/iterator/Iterator.birch"
  Iterator_();

  #line 8 "src/iterator/Iterator.birch"
  #line 8 "src/iterator/Iterator.birch"
  virtual Boolean hasNext() = 0;
  #line 13 "src/iterator/Iterator.birch"
  #line 13 "src/iterator/Iterator.birch"
  virtual Type next() = 0;
};

#line 4 "src/iterator/EmptyIterator.birch"
template<class Type>
#line 4 "src/iterator/EmptyIterator.birch"
class EmptyIterator_ final : public libbirch::unwrap_pointer<Iterator<Type>>::type {
public:
  #line 4 "src/iterator/EmptyIterator.birch"
  using Type_ = Type;
  #line 4 "src/iterator/EmptyIterator.birch"
  LIBBIRCH_CLASS(EmptyIterator_, typename libbirch::unwrap_pointer<Iterator<Type>>::type)
  #line 4 "src/iterator/EmptyIterator.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 4 "src/iterator/EmptyIterator.birch"
  using base_type_::operator=;
  #line 4 "src/iterator/EmptyIterator.birch"
  using base_type_::hasNext;
  #line 4 "src/iterator/EmptyIterator.birch"
  using base_type_::next;
  
  #line 4 "src/iterator/EmptyIterator.birch"
  EmptyIterator_();

  #line 8 "src/iterator/EmptyIterator.birch"
  #line 8 "src/iterator/EmptyIterator.birch"
  Boolean hasNext() override;
  #line 15 "src/iterator/EmptyIterator.birch"
  #line 15 "src/iterator/EmptyIterator.birch"
  Type next() override;
};

#line 6 "src/iterator/ArrayIterator.birch"
template<class Type>
#line 6 "src/iterator/ArrayIterator.birch"
class ArrayIterator_ final : public libbirch::unwrap_pointer<Iterator<Type>>::type {
public:
  #line 6 "src/iterator/ArrayIterator.birch"
  using Type_ = Type;
  #line 6 "src/iterator/ArrayIterator.birch"
  LIBBIRCH_CLASS(ArrayIterator_, typename libbirch::unwrap_pointer<Iterator<Type>>::type)
  #line 6 "src/iterator/ArrayIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(o, i)
  #line 6 "src/iterator/ArrayIterator.birch"
  using base_type_::operator=;
  #line 6 "src/iterator/ArrayIterator.birch"
  using base_type_::hasNext;
  #line 6 "src/iterator/ArrayIterator.birch"
  using base_type_::next;
  
  #line 6 "src/iterator/ArrayIterator.birch"
  ArrayIterator_(const Array<Type>& o);

  Array<Type> o;
  Integer i;
  #line 20 "src/iterator/ArrayIterator.birch"
  #line 20 "src/iterator/ArrayIterator.birch"
  Boolean hasNext() override;
  #line 27 "src/iterator/ArrayIterator.birch"
  #line 27 "src/iterator/ArrayIterator.birch"
  Type next() override;
};

#line 7 "src/iterator/ArrayBufferIterator.birch"
#line 7 "src/iterator/ArrayBufferIterator.birch"
class ArrayBufferIterator_ final : public libbirch::unwrap_pointer<Iterator<Buffer>>::type {
public:
  #line 7 "src/iterator/ArrayBufferIterator.birch"
  LIBBIRCH_CLASS(ArrayBufferIterator_, typename libbirch::unwrap_pointer<Iterator<Buffer>>::type)
  #line 7 "src/iterator/ArrayBufferIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(values, i)
  #line 7 "src/iterator/ArrayBufferIterator.birch"
  using base_type_::operator=;
  #line 7 "src/iterator/ArrayBufferIterator.birch"
  using base_type_::hasNext;
  #line 7 "src/iterator/ArrayBufferIterator.birch"
  using base_type_::next;
  
  #line 7 "src/iterator/ArrayBufferIterator.birch"
  ArrayBufferIterator_(const libbirch::DefaultArray<Buffer,1>& values);

  libbirch::DefaultArray<Buffer,1> values;
  Integer i;
  #line 21 "src/iterator/ArrayBufferIterator.birch"
  #line 21 "src/iterator/ArrayBufferIterator.birch"
  Boolean hasNext() override;
  #line 28 "src/iterator/ArrayBufferIterator.birch"
  #line 28 "src/iterator/ArrayBufferIterator.birch"
  Buffer next() override;
};

#line 38 "src/io/Reader.birch"
#line 38 "src/io/Reader.birch"
class Reader_ : public libbirch::unwrap_pointer<Iterator<Buffer>>::type {
public:
  #line 38 "src/io/Reader.birch"
  LIBBIRCH_ABSTRACT_CLASS(Reader_, typename libbirch::unwrap_pointer<Iterator<Buffer>>::type)
  #line 38 "src/io/Reader.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 38 "src/io/Reader.birch"
  using base_type_::operator=;
  
  #line 38 "src/io/Reader.birch"
  Reader_();

  #line 44 "src/io/Reader.birch"
  #line 44 "src/io/Reader.birch"
  virtual void open(const String& path) = 0;
  #line 51 "src/io/Reader.birch"
  #line 51 "src/io/Reader.birch"
  virtual Buffer slurp() = 0;
  #line 56 "src/io/Reader.birch"
  #line 56 "src/io/Reader.birch"
  virtual void close() = 0;
};

#line 1 "src/io/YAMLReader.birch"
#line 1 "src/io/YAMLReader.birch"
class YAMLReader_ : public libbirch::unwrap_pointer<Reader>::type {
public:
  #line 1 "src/io/YAMLReader.birch"
  LIBBIRCH_CLASS(YAMLReader_, typename libbirch::unwrap_pointer<Reader>::type)
  #line 1 "src/io/YAMLReader.birch"
  LIBBIRCH_CLASS_MEMBERS(file, sequential)
  #line 1 "src/io/YAMLReader.birch"
  using base_type_::operator=;
  #line 1 "src/io/YAMLReader.birch"
  using base_type_::close;
  #line 1 "src/io/YAMLReader.birch"
  using base_type_::hasNext;
  #line 1 "src/io/YAMLReader.birch"
  using base_type_::next;
  #line 1 "src/io/YAMLReader.birch"
  using base_type_::open;
  #line 1 "src/io/YAMLReader.birch"
  using base_type_::slurp;
  
  #line 1 "src/io/YAMLReader.birch"
  YAMLReader_();

  File file;
  Boolean sequential;
  #line 31 "src/io/YAMLReader.birch"

  yaml_parser_t parser;
  yaml_event_t event;
    #line 36 "src/io/YAMLReader.birch"
  #line 36 "src/io/YAMLReader.birch"
  virtual void open(const String& path) override;
  #line 47 "src/io/YAMLReader.birch"
  #line 47 "src/io/YAMLReader.birch"
  virtual void close() override;
  #line 55 "src/io/YAMLReader.birch"
  #line 55 "src/io/YAMLReader.birch"
  virtual Buffer slurp() override;
  #line 73 "src/io/YAMLReader.birch"
  #line 73 "src/io/YAMLReader.birch"
  virtual Boolean hasNext() override;
  #line 98 "src/io/YAMLReader.birch"
  #line 98 "src/io/YAMLReader.birch"
  virtual Buffer next() override;
  #line 122 "src/io/YAMLReader.birch"
  #line 122 "src/io/YAMLReader.birch"
  virtual void parseMapping(const Buffer& buffer);
  #line 155 "src/io/YAMLReader.birch"
  #line 155 "src/io/YAMLReader.birch"
  virtual void parseSequence(const Buffer& buffer);
  #line 189 "src/io/YAMLReader.birch"
  #line 189 "src/io/YAMLReader.birch"
  virtual void parseValue(const Buffer& buffer);
  #line 223 "src/io/YAMLReader.birch"
  #line 223 "src/io/YAMLReader.birch"
  virtual void parseElement(const Buffer& buffer);
  #line 257 "src/io/YAMLReader.birch"
  #line 257 "src/io/YAMLReader.birch"
  virtual void nextEvent();
};

#line 1 "src/io/YAMLReader.birch"
extern "C" YAMLReader_* make_YAMLReader_();

#line 6 "src/iterator/ListIterator.birch"
template<class Type>
#line 6 "src/iterator/ListIterator.birch"
class ListIterator_ final : public libbirch::unwrap_pointer<Iterator<Type>>::type {
public:
  #line 6 "src/iterator/ListIterator.birch"
  using Type_ = Type;
  #line 6 "src/iterator/ListIterator.birch"
  LIBBIRCH_CLASS(ListIterator_, typename libbirch::unwrap_pointer<Iterator<Type>>::type)
  #line 6 "src/iterator/ListIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(node)
  #line 6 "src/iterator/ListIterator.birch"
  using base_type_::operator=;
  #line 6 "src/iterator/ListIterator.birch"
  using base_type_::hasNext;
  #line 6 "src/iterator/ListIterator.birch"
  using base_type_::next;
  
  #line 6 "src/iterator/ListIterator.birch"
  ListIterator_(const std::optional<ListNode<Type>>& head);

  std::optional<ListNode<Type>> node;
  #line 15 "src/iterator/ListIterator.birch"
  #line 15 "src/iterator/ListIterator.birch"
  Boolean hasNext() override;
  #line 22 "src/iterator/ListIterator.birch"
  #line 22 "src/iterator/ListIterator.birch"
  Type next() override;
};

#line 7 "src/iterator/MatrixBufferIterator.birch"
template<class Type>
#line 7 "src/iterator/MatrixBufferIterator.birch"
class MatrixBufferIterator_ final : public libbirch::unwrap_pointer<Iterator<Buffer>>::type {
public:
  #line 7 "src/iterator/MatrixBufferIterator.birch"
  using Type_ = Type;
  #line 7 "src/iterator/MatrixBufferIterator.birch"
  LIBBIRCH_CLASS(MatrixBufferIterator_, typename libbirch::unwrap_pointer<Iterator<Buffer>>::type)
  #line 7 "src/iterator/MatrixBufferIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(values, i)
  #line 7 "src/iterator/MatrixBufferIterator.birch"
  using base_type_::operator=;
  #line 7 "src/iterator/MatrixBufferIterator.birch"
  using base_type_::hasNext;
  #line 7 "src/iterator/MatrixBufferIterator.birch"
  using base_type_::next;
  
  #line 7 "src/iterator/MatrixBufferIterator.birch"
  MatrixBufferIterator_(const libbirch::DefaultArray<Type,2>& values);

  libbirch::DefaultArray<Type,2> values;
  Integer i;
  #line 21 "src/iterator/MatrixBufferIterator.birch"
  #line 21 "src/iterator/MatrixBufferIterator.birch"
  Boolean hasNext() override;
  #line 28 "src/iterator/MatrixBufferIterator.birch"
  #line 28 "src/iterator/MatrixBufferIterator.birch"
  Buffer next() override;
};

#line 8 "src/iterator/ObjectBufferIterator.birch"
#line 8 "src/iterator/ObjectBufferIterator.birch"
class ObjectBufferIterator_ final : public libbirch::unwrap_pointer<Iterator<Buffer>>::type {
public:
  #line 8 "src/iterator/ObjectBufferIterator.birch"
  LIBBIRCH_CLASS(ObjectBufferIterator_, typename libbirch::unwrap_pointer<Iterator<Buffer>>::type)
  #line 8 "src/iterator/ObjectBufferIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(keys, values)
  #line 8 "src/iterator/ObjectBufferIterator.birch"
  using base_type_::operator=;
  #line 8 "src/iterator/ObjectBufferIterator.birch"
  using base_type_::hasNext;
  #line 8 "src/iterator/ObjectBufferIterator.birch"
  using base_type_::next;
  
  #line 8 "src/iterator/ObjectBufferIterator.birch"
  ObjectBufferIterator_(const libbirch::DefaultArray<String,1>& keys, const libbirch::DefaultArray<Buffer,1>& values);

  std::optional<libbirch::DefaultArray<String,1>> keys;
  std::optional<libbirch::DefaultArray<Buffer,1>> values;
  #line 23 "src/iterator/ObjectBufferIterator.birch"
  #line 23 "src/iterator/ObjectBufferIterator.birch"
  Boolean hasNext() override;
  #line 30 "src/iterator/ObjectBufferIterator.birch"
  #line 30 "src/iterator/ObjectBufferIterator.birch"
  Buffer next() override;
};

#line 7 "src/iterator/RaggedArrayElementIterator.birch"
template<class Type>
#line 7 "src/iterator/RaggedArrayElementIterator.birch"
class RaggedArrayElementIterator_ final : public libbirch::unwrap_pointer<Iterator<Type>>::type {
public:
  #line 7 "src/iterator/RaggedArrayElementIterator.birch"
  using Type_ = Type;
  #line 7 "src/iterator/RaggedArrayElementIterator.birch"
  LIBBIRCH_CLASS(RaggedArrayElementIterator_, typename libbirch::unwrap_pointer<Iterator<Type>>::type)
  #line 7 "src/iterator/RaggedArrayElementIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(o, i, j)
  #line 7 "src/iterator/RaggedArrayElementIterator.birch"
  using base_type_::operator=;
  #line 7 "src/iterator/RaggedArrayElementIterator.birch"
  using base_type_::hasNext;
  #line 7 "src/iterator/RaggedArrayElementIterator.birch"
  using base_type_::next;
  
  #line 7 "src/iterator/RaggedArrayElementIterator.birch"
  RaggedArrayElementIterator_(const RaggedArray<Type>& o, const Integer& i);

  RaggedArray<Type> o;
  Integer i;
  Integer j;
  #line 27 "src/iterator/RaggedArrayElementIterator.birch"
  #line 27 "src/iterator/RaggedArrayElementIterator.birch"
  Boolean hasNext() override;
  #line 34 "src/iterator/RaggedArrayElementIterator.birch"
  #line 34 "src/iterator/RaggedArrayElementIterator.birch"
  Type next() override;
};

#line 6 "src/iterator/RaggedArrayRowIterator.birch"
template<class Type>
#line 6 "src/iterator/RaggedArrayRowIterator.birch"
class RaggedArrayRowIterator_ final : public libbirch::unwrap_pointer<Iterator<libbirch::DefaultArray<Type,1>>>::type {
public:
  #line 6 "src/iterator/RaggedArrayRowIterator.birch"
  using Type_ = Type;
  #line 6 "src/iterator/RaggedArrayRowIterator.birch"
  LIBBIRCH_CLASS(RaggedArrayRowIterator_, typename libbirch::unwrap_pointer<Iterator<libbirch::DefaultArray<Type,1>>>::type)
  #line 6 "src/iterator/RaggedArrayRowIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(o, i)
  #line 6 "src/iterator/RaggedArrayRowIterator.birch"
  using base_type_::operator=;
  #line 6 "src/iterator/RaggedArrayRowIterator.birch"
  using base_type_::hasNext;
  #line 6 "src/iterator/RaggedArrayRowIterator.birch"
  using base_type_::next;
  
  #line 6 "src/iterator/RaggedArrayRowIterator.birch"
  RaggedArrayRowIterator_(const RaggedArray<Type>& o);

  RaggedArray<Type> o;
  Integer i;
  #line 21 "src/iterator/RaggedArrayRowIterator.birch"
  #line 21 "src/iterator/RaggedArrayRowIterator.birch"
  Boolean hasNext() override;
  #line 28 "src/iterator/RaggedArrayRowIterator.birch"
  #line 28 "src/iterator/RaggedArrayRowIterator.birch"
  libbirch::DefaultArray<Type,1> next() override;
};

#line 6 "src/iterator/ScalarBufferIterator.birch"
template<class Type>
#line 6 "src/iterator/ScalarBufferIterator.birch"
class ScalarBufferIterator_ final : public libbirch::unwrap_pointer<Iterator<Buffer>>::type {
public:
  #line 6 "src/iterator/ScalarBufferIterator.birch"
  using Type_ = Type;
  #line 6 "src/iterator/ScalarBufferIterator.birch"
  LIBBIRCH_CLASS(ScalarBufferIterator_, typename libbirch::unwrap_pointer<Iterator<Buffer>>::type)
  #line 6 "src/iterator/ScalarBufferIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(x)
  #line 6 "src/iterator/ScalarBufferIterator.birch"
  using base_type_::operator=;
  #line 6 "src/iterator/ScalarBufferIterator.birch"
  using base_type_::hasNext;
  #line 6 "src/iterator/ScalarBufferIterator.birch"
  using base_type_::next;
  
  #line 6 "src/iterator/ScalarBufferIterator.birch"
  ScalarBufferIterator_(const Type& x);

  std::optional<Type> x;
  #line 12 "src/iterator/ScalarBufferIterator.birch"
  #line 12 "src/iterator/ScalarBufferIterator.birch"
  Boolean hasNext() override;
  #line 16 "src/iterator/ScalarBufferIterator.birch"
  #line 16 "src/iterator/ScalarBufferIterator.birch"
  Buffer next() override;
};

#line 6 "src/iterator/StackIterator.birch"
template<class Type>
#line 6 "src/iterator/StackIterator.birch"
class StackIterator_ final : public libbirch::unwrap_pointer<Iterator<Type>>::type {
public:
  #line 6 "src/iterator/StackIterator.birch"
  using Type_ = Type;
  #line 6 "src/iterator/StackIterator.birch"
  LIBBIRCH_CLASS(StackIterator_, typename libbirch::unwrap_pointer<Iterator<Type>>::type)
  #line 6 "src/iterator/StackIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(node)
  #line 6 "src/iterator/StackIterator.birch"
  using base_type_::operator=;
  #line 6 "src/iterator/StackIterator.birch"
  using base_type_::hasNext;
  #line 6 "src/iterator/StackIterator.birch"
  using base_type_::next;
  
  #line 6 "src/iterator/StackIterator.birch"
  StackIterator_(const std::optional<StackNode<Type>>& head);

  std::optional<StackNode<Type>> node;
  #line 15 "src/iterator/StackIterator.birch"
  #line 15 "src/iterator/StackIterator.birch"
  Boolean hasNext() override;
  #line 22 "src/iterator/StackIterator.birch"
  #line 22 "src/iterator/StackIterator.birch"
  Type next() override;
};

#line 6 "src/iterator/TapeIterator.birch"
template<class Type>
#line 6 "src/iterator/TapeIterator.birch"
class TapeIterator_ final : public libbirch::unwrap_pointer<Iterator<Type>>::type {
public:
  #line 6 "src/iterator/TapeIterator.birch"
  using Type_ = Type;
  #line 6 "src/iterator/TapeIterator.birch"
  LIBBIRCH_CLASS(TapeIterator_, typename libbirch::unwrap_pointer<Iterator<Type>>::type)
  #line 6 "src/iterator/TapeIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(node)
  #line 6 "src/iterator/TapeIterator.birch"
  using base_type_::operator=;
  #line 6 "src/iterator/TapeIterator.birch"
  using base_type_::hasNext;
  #line 6 "src/iterator/TapeIterator.birch"
  using base_type_::next;
  
  #line 6 "src/iterator/TapeIterator.birch"
  TapeIterator_(const std::optional<TapeNode<Type>>& head);

  std::optional<TapeNode<Type>> node;
  #line 15 "src/iterator/TapeIterator.birch"
  #line 15 "src/iterator/TapeIterator.birch"
  Boolean hasNext() override;
  #line 22 "src/iterator/TapeIterator.birch"
  #line 22 "src/iterator/TapeIterator.birch"
  Type next() override;
};

#line 7 "src/iterator/VectorBufferIterator.birch"
template<class Type>
#line 7 "src/iterator/VectorBufferIterator.birch"
class VectorBufferIterator_ final : public libbirch::unwrap_pointer<Iterator<Buffer>>::type {
public:
  #line 7 "src/iterator/VectorBufferIterator.birch"
  using Type_ = Type;
  #line 7 "src/iterator/VectorBufferIterator.birch"
  LIBBIRCH_CLASS(VectorBufferIterator_, typename libbirch::unwrap_pointer<Iterator<Buffer>>::type)
  #line 7 "src/iterator/VectorBufferIterator.birch"
  LIBBIRCH_CLASS_MEMBERS(values, i)
  #line 7 "src/iterator/VectorBufferIterator.birch"
  using base_type_::operator=;
  #line 7 "src/iterator/VectorBufferIterator.birch"
  using base_type_::hasNext;
  #line 7 "src/iterator/VectorBufferIterator.birch"
  using base_type_::next;
  
  #line 7 "src/iterator/VectorBufferIterator.birch"
  VectorBufferIterator_(const libbirch::DefaultArray<Type,1>& values);

  libbirch::DefaultArray<Type,1> values;
  Integer i;
  #line 21 "src/iterator/VectorBufferIterator.birch"
  #line 21 "src/iterator/VectorBufferIterator.birch"
  Boolean hasNext() override;
  #line 28 "src/iterator/VectorBufferIterator.birch"
  #line 28 "src/iterator/VectorBufferIterator.birch"
  Buffer next() override;
};

#line 18 "src/kernel/Kernel.birch"
#line 18 "src/kernel/Kernel.birch"
class Kernel_ : public Object_ {
public:
  #line 18 "src/kernel/Kernel.birch"
  LIBBIRCH_ABSTRACT_CLASS(Kernel_, Object_)
  #line 18 "src/kernel/Kernel.birch"
  LIBBIRCH_CLASS_MEMBERS(nmoves, nlags)
  #line 18 "src/kernel/Kernel.birch"
  using base_type_::operator=;
  #line 18 "src/kernel/Kernel.birch"
  using base_type_::read;
  #line 18 "src/kernel/Kernel.birch"
  using base_type_::write;
  
  #line 18 "src/kernel/Kernel.birch"
  Kernel_();

  Integer nmoves;
  Integer nlags;
  #line 37 "src/kernel/Kernel.birch"
  #line 37 "src/kernel/Kernel.birch"
  virtual Real apply(const Integer& t, const Particle& \u03c0) = 0;
  #line 39 "src/kernel/Kernel.birch"
  #line 39 "src/kernel/Kernel.birch"
  virtual void read(const Buffer& buffer) override;
  #line 45 "src/kernel/Kernel.birch"
  #line 45 "src/kernel/Kernel.birch"
  virtual void write(const Buffer& buffer) override;
};

#line 1 "src/kernel/LangevinKernel.birch"
#line 1 "src/kernel/LangevinKernel.birch"
class LangevinKernel_ : public libbirch::unwrap_pointer<Kernel>::type {
public:
  #line 1 "src/kernel/LangevinKernel.birch"
  LIBBIRCH_CLASS(LangevinKernel_, typename libbirch::unwrap_pointer<Kernel>::type)
  #line 1 "src/kernel/LangevinKernel.birch"
  LIBBIRCH_CLASS_MEMBERS(scale)
  #line 1 "src/kernel/LangevinKernel.birch"
  using base_type_::operator=;
  #line 1 "src/kernel/LangevinKernel.birch"
  using base_type_::apply;
  #line 1 "src/kernel/LangevinKernel.birch"
  using base_type_::read;
  #line 1 "src/kernel/LangevinKernel.birch"
  using base_type_::write;
  
  #line 1 "src/kernel/LangevinKernel.birch"
  LangevinKernel_();

  Real scale;
  #line 17 "src/kernel/LangevinKernel.birch"
  #line 17 "src/kernel/LangevinKernel.birch"
  virtual Real apply(const Integer& t, const Particle& \u03c0) override;
  #line 68 "src/kernel/LangevinKernel.birch"
  #line 68 "src/kernel/LangevinKernel.birch"
  virtual void read(const Buffer& buffer) override;
  #line 73 "src/kernel/LangevinKernel.birch"
  #line 73 "src/kernel/LangevinKernel.birch"
  virtual void write(const Buffer& buffer) override;
};

#line 1 "src/kernel/LangevinKernel.birch"
extern "C" LangevinKernel_* make_LangevinKernel_();

#line 4 "src/model/Model.birch"
#line 4 "src/model/Model.birch"
class Model_ : public Object_ {
public:
  #line 4 "src/model/Model.birch"
  LIBBIRCH_ABSTRACT_CLASS(Model_, Object_)
  #line 4 "src/model/Model.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 4 "src/model/Model.birch"
  using base_type_::operator=;
  #line 4 "src/model/Model.birch"
  using base_type_::read;
  #line 4 "src/model/Model.birch"
  using base_type_::write;
  
  #line 4 "src/model/Model.birch"
  Model_();

  #line 8 "src/model/Model.birch"
  #line 8 "src/model/Model.birch"
  virtual void simulate();
  #line 17 "src/model/Model.birch"
  #line 17 "src/model/Model.birch"
  virtual void simulate(const Integer& t);
  #line 24 "src/model/Model.birch"
  #line 24 "src/model/Model.birch"
  virtual void read(const Buffer& buffer) override;
  #line 31 "src/model/Model.birch"
  #line 31 "src/model/Model.birch"
  virtual void read(const Integer& t, const Buffer& buffer) override;
  #line 38 "src/model/Model.birch"
  #line 38 "src/model/Model.birch"
  virtual void write(const Buffer& buffer) override;
  #line 45 "src/model/Model.birch"
  #line 45 "src/model/Model.birch"
  virtual void write(const Integer& t, const Buffer& buffer) override;
};

#line 1 "src/test/distribution/TestModel.birch"
#line 1 "src/test/distribution/TestModel.birch"
class TestModel_ : public libbirch::unwrap_pointer<Model>::type {
public:
  #line 1 "src/test/distribution/TestModel.birch"
  LIBBIRCH_ABSTRACT_CLASS(TestModel_, typename libbirch::unwrap_pointer<Model>::type)
  #line 1 "src/test/distribution/TestModel.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 1 "src/test/distribution/TestModel.birch"
  using base_type_::operator=;
  
  #line 1 "src/test/distribution/TestModel.birch"
  TestModel_();

  #line 2 "src/test/distribution/TestModel.birch"
  #line 2 "src/test/distribution/TestModel.birch"
  virtual void initialize() = 0;
  #line 3 "src/test/distribution/TestModel.birch"
  #line 3 "src/test/distribution/TestModel.birch"
  virtual libbirch::DefaultArray<Real,1> forward() = 0;
  #line 4 "src/test/distribution/TestModel.birch"
  #line 4 "src/test/distribution/TestModel.birch"
  virtual libbirch::DefaultArray<Real,1> backward() = 0;
  #line 9 "src/test/distribution/TestModel.birch"
  #line 9 "src/test/distribution/TestModel.birch"
  virtual void forwardGrad();
  #line 16 "src/test/distribution/TestModel.birch"
  #line 16 "src/test/distribution/TestModel.birch"
  virtual void backwardGrad();
  #line 20 "src/test/distribution/TestModel.birch"
  #line 20 "src/test/distribution/TestModel.birch"
  virtual Integer size() = 0;
};

#line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
class TestMatrixGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  LIBBIRCH_CLASS(TestMatrixGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(X, n, p, M, U, V)
  #line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  TestMatrixGaussian_();

  Random<libbirch::DefaultArray<Real,2>> X;
  Integer n;
  Integer p;
  libbirch::DefaultArray<Real,2> M;
  libbirch::DefaultArray<Real,2> U;
  libbirch::DefaultArray<Real,2> V;
  #line 11 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  #line 11 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  virtual void initialize() override;
  #line 20 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  #line 20 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  virtual void simulate() override;
  #line 24 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  #line 24 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 28 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  #line 28 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 32 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  #line 32 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,2>> marginal();
  #line 36 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  #line 36 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/matrix_gaussian/TestMatrixGaussian.birch"
extern "C" TestMatrixGaussian_* make_TestMatrixGaussian_();

#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
class TestFanMatrixNormalInverseWishartMatrixGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  LIBBIRCH_CLASS(TestFanMatrixNormalInverseWishartMatrixGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03a3, X, Y1, Y2, Y3, Y4, Y5, n, p, k, \u03a8, M, U, V)
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::backwardGrad;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  TestFanMatrixNormalInverseWishartMatrixGaussian_();

  Random<libbirch::DefaultArray<Real,2>> \u03a3;
  Random<libbirch::DefaultArray<Real,2>> X;
  Random<libbirch::DefaultArray<Real,2>> Y1;
  Random<libbirch::DefaultArray<Real,2>> Y2;
  Random<libbirch::DefaultArray<Real,2>> Y3;
  Random<libbirch::DefaultArray<Real,2>> Y4;
  Random<libbirch::DefaultArray<Real,2>> Y5;
  Integer n;
  Integer p;
  Real k;
  libbirch::DefaultArray<Real,2> \u03a8;
  libbirch::DefaultArray<Real,2> M;
  libbirch::DefaultArray<Real,2> U;
  libbirch::DefaultArray<Real,2> V;
  #line 19 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 19 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual void initialize() override;
  #line 31 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 31 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual void simulate() override;
  #line 41 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 41 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 52 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 52 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 66 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 66 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual void backwardGrad() override;
  #line 80 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 80 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,2>> marginal();
  #line 84 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 84 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> vectorize();
  #line 103 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 103 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestFanMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" TestFanMatrixNormalInverseWishartMatrixGaussian_* make_TestFanMatrixNormalInverseWishartMatrixGaussian_();

#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
class TestLinearMatrixNormalInverseWishartMatrixGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  LIBBIRCH_CLASS(TestLinearMatrixNormalInverseWishartMatrixGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03a3, X, Y, n, p, k, \u03a8, A, M, C, U, V)
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::backwardGrad;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  TestLinearMatrixNormalInverseWishartMatrixGaussian_();

  Random<libbirch::DefaultArray<Real,2>> \u03a3;
  Random<libbirch::DefaultArray<Real,2>> X;
  Random<libbirch::DefaultArray<Real,2>> Y;
  Integer n;
  Integer p;
  Real k;
  libbirch::DefaultArray<Real,2> \u03a8;
  libbirch::DefaultArray<Real,2> A;
  libbirch::DefaultArray<Real,2> M;
  libbirch::DefaultArray<Real,2> C;
  libbirch::DefaultArray<Real,2> U;
  libbirch::DefaultArray<Real,2> V;
  #line 17 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 17 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual void initialize() override;
  #line 31 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 31 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual void simulate() override;
  #line 37 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 37 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 44 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 44 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 54 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 54 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual void backwardGrad() override;
  #line 64 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 64 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,2>> marginal();
  #line 68 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 68 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> vectorize();
  #line 79 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 79 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" TestLinearMatrixNormalInverseWishartMatrixGaussian_* make_TestLinearMatrixNormalInverseWishartMatrixGaussian_();

#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
class TestMatrixNormalInverseWishart_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  LIBBIRCH_CLASS(TestMatrixNormalInverseWishart_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03a3, X, n, p, \u03a8, k, M, U)
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  using base_type_::backwardGrad;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  TestMatrixNormalInverseWishart_();

  Random<libbirch::DefaultArray<Real,2>> \u03a3;
  Random<libbirch::DefaultArray<Real,2>> X;
  Integer n;
  Integer p;
  libbirch::DefaultArray<Real,2> \u03a8;
  Real k;
  libbirch::DefaultArray<Real,2> M;
  libbirch::DefaultArray<Real,2> U;
  #line 13 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  #line 13 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  virtual void initialize() override;
  #line 23 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  #line 23 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  virtual void simulate() override;
  #line 28 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  #line 28 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 34 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  #line 34 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 42 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  #line 42 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  virtual void backwardGrad() override;
  #line 50 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  #line 50 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  virtual Distribution<libbirch::DefaultArray<Real,2>> marginal();
  #line 54 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  #line 54 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  virtual libbirch::DefaultArray<Real,1> vectorize();
  #line 63 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  #line 63 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishart.birch"
extern "C" TestMatrixNormalInverseWishart_* make_TestMatrixNormalInverseWishart_();

#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
class TestMatrixNormalInverseWishartMatrixGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  LIBBIRCH_CLASS(TestMatrixNormalInverseWishartMatrixGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03a3, X, Y, n, p, k, \u03a8, M, U, V)
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::backwardGrad;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  TestMatrixNormalInverseWishartMatrixGaussian_();

  Random<libbirch::DefaultArray<Real,2>> \u03a3;
  Random<libbirch::DefaultArray<Real,2>> X;
  Random<libbirch::DefaultArray<Real,2>> Y;
  Integer n;
  Integer p;
  Real k;
  libbirch::DefaultArray<Real,2> \u03a8;
  libbirch::DefaultArray<Real,2> M;
  libbirch::DefaultArray<Real,2> U;
  libbirch::DefaultArray<Real,2> V;
  #line 15 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 15 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual void initialize() override;
  #line 27 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 27 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual void simulate() override;
  #line 33 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 33 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 40 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 40 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 50 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 50 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual void backwardGrad() override;
  #line 60 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 60 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,2>> marginal();
  #line 64 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 64 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> vectorize();
  #line 75 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  #line 75 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/matrix_normal_inverse_wishart/TestMatrixNormalInverseWishartMatrixGaussian.birch"
extern "C" TestMatrixNormalInverseWishartMatrixGaussian_* make_TestMatrixNormalInverseWishartMatrixGaussian_();

#line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
#line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
class TestInverseWishart_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  LIBBIRCH_CLASS(TestInverseWishart_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03a3, p, k, \u03a8)
  #line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  TestInverseWishart_();

  Random<libbirch::DefaultArray<Real,2>> \u03a3;
  Integer p;
  Real k;
  libbirch::DefaultArray<Real,2> \u03a8;
  #line 7 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  #line 7 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  virtual void initialize() override;
  #line 14 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  #line 14 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  virtual void simulate() override;
  #line 18 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  #line 18 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 23 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  #line 23 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 28 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  #line 28 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  virtual Distribution<libbirch::DefaultArray<Real,2>> marginal();
  #line 32 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  #line 32 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  virtual libbirch::DefaultArray<Real,1> vectorize();
  #line 36 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  #line 36 "src/test/distribution/matrix_other/TestInverseWishart.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/matrix_other/TestInverseWishart.birch"
extern "C" TestInverseWishart_* make_TestInverseWishart_();

#line 1 "src/test/distribution/matrix_other/TestWishart.birch"
#line 1 "src/test/distribution/matrix_other/TestWishart.birch"
class TestWishart_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/matrix_other/TestWishart.birch"
  LIBBIRCH_CLASS(TestWishart_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/matrix_other/TestWishart.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03a3, p, k, \u03a8)
  #line 1 "src/test/distribution/matrix_other/TestWishart.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/matrix_other/TestWishart.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/matrix_other/TestWishart.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/matrix_other/TestWishart.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/matrix_other/TestWishart.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/matrix_other/TestWishart.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/matrix_other/TestWishart.birch"
  TestWishart_();

  Random<libbirch::DefaultArray<Real,2>> \u03a3;
  Integer p;
  Real k;
  libbirch::DefaultArray<Real,2> \u03a8;
  #line 7 "src/test/distribution/matrix_other/TestWishart.birch"
  #line 7 "src/test/distribution/matrix_other/TestWishart.birch"
  virtual void initialize() override;
  #line 14 "src/test/distribution/matrix_other/TestWishart.birch"
  #line 14 "src/test/distribution/matrix_other/TestWishart.birch"
  virtual void simulate() override;
  #line 18 "src/test/distribution/matrix_other/TestWishart.birch"
  #line 18 "src/test/distribution/matrix_other/TestWishart.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 23 "src/test/distribution/matrix_other/TestWishart.birch"
  #line 23 "src/test/distribution/matrix_other/TestWishart.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 28 "src/test/distribution/matrix_other/TestWishart.birch"
  #line 28 "src/test/distribution/matrix_other/TestWishart.birch"
  virtual Distribution<libbirch::DefaultArray<Real,2>> marginal();
  #line 32 "src/test/distribution/matrix_other/TestWishart.birch"
  #line 32 "src/test/distribution/matrix_other/TestWishart.birch"
  virtual libbirch::DefaultArray<Real,1> vectorize();
  #line 36 "src/test/distribution/matrix_other/TestWishart.birch"
  #line 36 "src/test/distribution/matrix_other/TestWishart.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/matrix_other/TestWishart.birch"
extern "C" TestWishart_* make_TestWishart_();

#line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
#line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
class TestDirichletMultinomial_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  LIBBIRCH_CLASS(TestDirichletMultinomial_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  LIBBIRCH_CLASS_MEMBERS(n, \u03b1, \u03c1, x)
  #line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  TestDirichletMultinomial_();

  Integer n;
  libbirch::DefaultArray<Real,1> \u03b1;
  Random<libbirch::DefaultArray<Real,1>> \u03c1;
  Random<libbirch::DefaultArray<Integer,1>> x;
  #line 7 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  #line 7 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  virtual void initialize() override;
  #line 14 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  #line 14 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  virtual void simulate() override;
  #line 19 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  #line 19 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 27 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  #line 27 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 37 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  #line 37 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/multivariate_discrete/TestDirichletMultinomial.birch"
extern "C" TestDirichletMultinomial_* make_TestDirichletMultinomial_();

#line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
class TestChainMultivariateGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  LIBBIRCH_CLASS(TestChainMultivariateGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03bc, \u03a3)
  #line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  TestChainMultivariateGaussian_();

  libbirch::DefaultArray<Random<libbirch::DefaultArray<Real,1>>,1> x;
  libbirch::DefaultArray<Real,1> \u03bc;
  libbirch::DefaultArray<Real,2> \u03a3;
  #line 6 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  #line 6 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  virtual void initialize() override;
  #line 16 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  #line 16 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  virtual void simulate() override;
  #line 24 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  #line 24 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 32 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  #line 32 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 41 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  #line 41 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,1>> marginal();
  #line 45 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  #line 45 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/multivariate_gaussian/TestChainMultivariateGaussian.birch"
extern "C" TestChainMultivariateGaussian_* make_TestChainMultivariateGaussian_();

#line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
class TestLinearMultivariateGaussianMultivariateGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  LIBBIRCH_CLASS(TestLinearMultivariateGaussianMultivariateGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bc, x, n, A, \u03bc_0, \u03a3_0, c, \u03a3_1)
  #line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  TestLinearMultivariateGaussianMultivariateGaussian_();

  Random<libbirch::DefaultArray<Real,1>> \u03bc;
  Random<libbirch::DefaultArray<Real,1>> x;
  Integer n;
  libbirch::DefaultArray<Real,2> A;
  libbirch::DefaultArray<Real,1> \u03bc_0;
  libbirch::DefaultArray<Real,2> \u03a3_0;
  libbirch::DefaultArray<Real,1> c;
  libbirch::DefaultArray<Real,2> \u03a3_1;
  #line 13 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  #line 13 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  virtual void initialize() override;
  #line 27 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  #line 27 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  virtual void simulate() override;
  #line 32 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  #line 32 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 39 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  #line 39 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 48 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  #line 48 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,1>> marginal();
  #line 52 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  #line 52 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/multivariate_gaussian/TestLinearMultivariateGaussianMultivariateGaussian.birch"
extern "C" TestLinearMultivariateGaussianMultivariateGaussian_* make_TestLinearMultivariateGaussianMultivariateGaussian_();

#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
class TestMultivariateGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  LIBBIRCH_CLASS(TestMultivariateGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03bc, \u03a3)
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  TestMultivariateGaussian_();

  Random<libbirch::DefaultArray<Real,1>> x;
  libbirch::DefaultArray<Real,1> \u03bc;
  libbirch::DefaultArray<Real,2> \u03a3;
  #line 6 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  #line 6 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  virtual void initialize() override;
  #line 16 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  #line 16 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  virtual void simulate() override;
  #line 20 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  #line 20 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 24 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  #line 24 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 28 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  #line 28 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,1>> marginal();
  #line 32 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  #line 32 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussian.birch"
extern "C" TestMultivariateGaussian_* make_TestMultivariateGaussian_();

#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
class TestMultivariateGaussianMultivariateGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  LIBBIRCH_CLASS(TestMultivariateGaussianMultivariateGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bc_1, x, n, \u03bc_0, \u03a3_0, \u03a3_1)
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  TestMultivariateGaussianMultivariateGaussian_();

  Random<libbirch::DefaultArray<Real,1>> \u03bc_1;
  Random<libbirch::DefaultArray<Real,1>> x;
  Integer n;
  libbirch::DefaultArray<Real,1> \u03bc_0;
  libbirch::DefaultArray<Real,2> \u03a3_0;
  libbirch::DefaultArray<Real,2> \u03a3_1;
  #line 11 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  #line 11 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  virtual void initialize() override;
  #line 23 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  #line 23 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  virtual void simulate() override;
  #line 28 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  #line 28 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 35 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  #line 35 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 44 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  #line 44 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,1>> marginal();
  #line 48 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  #line 48 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/multivariate_gaussian/TestMultivariateGaussianMultivariateGaussian.birch"
extern "C" TestMultivariateGaussianMultivariateGaussian_* make_TestMultivariateGaussianMultivariateGaussian_();

#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
class TestFanMultivariateNormalInverseGammaMultivariateGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  LIBBIRCH_CLASS(TestFanMultivariateNormalInverseGammaMultivariateGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c32, \u03bc, x1, x2, x3, x4, x5, n, \u03b1, \u03b2, \u03bc_0, \u03a3, \u03a8)
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  TestFanMultivariateNormalInverseGammaMultivariateGaussian_();

  Random<Real> \u03c32;
  Random<libbirch::DefaultArray<Real,1>> \u03bc;
  Random<libbirch::DefaultArray<Real,1>> x1;
  Random<libbirch::DefaultArray<Real,1>> x2;
  Random<libbirch::DefaultArray<Real,1>> x3;
  Random<libbirch::DefaultArray<Real,1>> x4;
  Random<libbirch::DefaultArray<Real,1>> x5;
  Integer n;
  Real \u03b1;
  Real \u03b2;
  libbirch::DefaultArray<Real,1> \u03bc_0;
  libbirch::DefaultArray<Real,2> \u03a3;
  libbirch::DefaultArray<Real,2> \u03a8;
  #line 18 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 18 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual void initialize() override;
  #line 29 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 29 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual void simulate() override;
  #line 39 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 39 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 50 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 50 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 64 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 64 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,1>> marginal();
  #line 68 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 68 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> vectorize();
  #line 87 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 87 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestFanMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" TestFanMultivariateNormalInverseGammaMultivariateGaussian_* make_TestFanMultivariateNormalInverseGammaMultivariateGaussian_();

#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
class TestLinearMultivariateNormalInverseGammaMultivariateGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  LIBBIRCH_CLASS(TestLinearMultivariateNormalInverseGammaMultivariateGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c32, \u03bc, x, n, p, \u03b1, \u03b2, A, m, S, c, \u03a8)
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  TestLinearMultivariateNormalInverseGammaMultivariateGaussian_();

  Random<Real> \u03c32;
  Random<libbirch::DefaultArray<Real,1>> \u03bc;
  Random<libbirch::DefaultArray<Real,1>> x;
  Integer n;
  Integer p;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  Random<libbirch::DefaultArray<Real,2>> A;
  Random<libbirch::DefaultArray<Real,1>> m;
  Random<libbirch::DefaultArray<Real,2>> S;
  Random<libbirch::DefaultArray<Real,1>> c;
  Random<libbirch::DefaultArray<Real,2>> \u03a8;
  #line 17 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 17 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual void initialize() override;
  #line 27 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 27 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual void simulate() override;
  #line 33 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 33 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 41 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 41 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 52 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 52 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,1>> marginal();
  #line 56 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 56 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" TestLinearMultivariateNormalInverseGammaMultivariateGaussian_* make_TestLinearMultivariateNormalInverseGammaMultivariateGaussian_();

#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
class TestMultivariateNormalInverseGamma_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  LIBBIRCH_CLASS(TestMultivariateNormalInverseGamma_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c32, \u03bc, n, \u03b1, \u03b2, m, S)
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  TestMultivariateNormalInverseGamma_();

  Random<Real> \u03c32;
  Random<libbirch::DefaultArray<Real,1>> \u03bc;
  Integer n;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  Random<libbirch::DefaultArray<Real,1>> m;
  Random<libbirch::DefaultArray<Real,2>> S;
  #line 12 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  #line 12 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  virtual void initialize() override;
  #line 19 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  #line 19 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  virtual void simulate() override;
  #line 24 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  #line 24 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 31 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  #line 31 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 40 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  #line 40 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  virtual Distribution<libbirch::DefaultArray<Real,1>> marginal();
  #line 44 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  #line 44 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGamma.birch"
extern "C" TestMultivariateNormalInverseGamma_* make_TestMultivariateNormalInverseGamma_();

#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
class TestMultivariateNormalInverseGammaMultivariateGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  LIBBIRCH_CLASS(TestMultivariateNormalInverseGammaMultivariateGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c32, \u03bc, x, n, \u03b1, \u03b2, m, S, \u03a8)
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  TestMultivariateNormalInverseGammaMultivariateGaussian_();

  Random<Real> \u03c32;
  Random<libbirch::DefaultArray<Real,1>> \u03bc;
  Random<libbirch::DefaultArray<Real,1>> x;
  Integer n;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  Random<libbirch::DefaultArray<Real,1>> m;
  Random<libbirch::DefaultArray<Real,2>> S;
  Random<libbirch::DefaultArray<Real,2>> \u03a8;
  #line 14 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 14 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual void initialize() override;
  #line 22 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 22 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual void simulate() override;
  #line 28 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 28 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 36 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 36 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 47 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 47 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,1>> marginal();
  #line 51 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  #line 51 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/multivariate_normal_inverse_gamma/TestMultivariateNormalInverseGammaMultivariateGaussian.birch"
extern "C" TestMultivariateNormalInverseGammaMultivariateGaussian_* make_TestMultivariateNormalInverseGammaMultivariateGaussian_();

#line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
#line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
class TestLinearMatrixNormalInverseWishartMultivariateGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  LIBBIRCH_CLASS(TestLinearMatrixNormalInverseWishartMultivariateGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(V, X, y, n, p, a, M, U, c, k, \u03a8)
  #line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  using base_type_::backwardGrad;
  #line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  TestLinearMatrixNormalInverseWishartMultivariateGaussian_();

  Random<libbirch::DefaultArray<Real,2>> V;
  Random<libbirch::DefaultArray<Real,2>> X;
  Random<libbirch::DefaultArray<Real,1>> y;
  Integer n;
  Integer p;
  libbirch::DefaultArray<Real,1> a;
  libbirch::DefaultArray<Real,2> M;
  libbirch::DefaultArray<Real,2> U;
  libbirch::DefaultArray<Real,1> c;
  Real k;
  libbirch::DefaultArray<Real,2> \u03a8;
  #line 16 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  #line 16 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  virtual void initialize() override;
  #line 37 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  #line 37 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  virtual void simulate() override;
  #line 43 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  #line 43 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 50 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  #line 50 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 60 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  #line 60 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  virtual void backwardGrad() override;
  #line 70 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  #line 70 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  virtual Distribution<libbirch::DefaultArray<Real,1>> marginal();
  #line 74 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  #line 74 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> vectorize();
  #line 85 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  #line 85 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/multivariate_normal_inverse_wishart/TestLinearMatrixNormalInverseWishartMultivariateGaussian.birch"
extern "C" TestLinearMatrixNormalInverseWishartMultivariateGaussian_* make_TestLinearMatrixNormalInverseWishartMultivariateGaussian_();

#line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
#line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
class TestBernoulli_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  LIBBIRCH_CLASS(TestBernoulli_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03c1)
  #line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  TestBernoulli_();

  Random<Boolean> x;
  Random<Real> \u03c1;
  #line 5 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  #line 5 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  virtual void initialize() override;
  #line 9 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  #line 9 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  virtual void simulate() override;
  #line 14 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  #line 14 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 18 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  #line 18 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 22 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  #line 22 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  virtual Distribution<Boolean> marginal();
  #line 26 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  #line 26 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_boolean/TestBernoulli.birch"
extern "C" TestBernoulli_* make_TestBernoulli_();

#line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
#line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
class TestBetaBernoulli_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  LIBBIRCH_CLASS(TestBetaBernoulli_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03b1, \u03b2, \u03c1, x)
  #line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  TestBetaBernoulli_();

  Random<Real> \u03b1;
  Random<Real> \u03b2;
  Random<Real> \u03c1;
  Random<Boolean> x;
  #line 7 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  #line 7 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  #line 11 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  virtual void simulate() override;
  #line 18 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  #line 18 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 25 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  #line 25 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 34 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  #line 34 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  virtual Distribution<Boolean> marginal();
  #line 38 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  #line 38 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_boolean/TestBetaBernoulli.birch"
extern "C" TestBetaBernoulli_* make_TestBetaBernoulli_();

#line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
class TestBetaBinomial_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  LIBBIRCH_CLASS(TestBetaBinomial_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c1, x, n, \u03b1, \u03b2)
  #line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  TestBetaBinomial_();

  Random<Real> \u03c1;
  Random<Integer> x;
  Random<Integer> n;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  #line 8 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  #line 8 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  virtual void initialize() override;
  #line 14 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  #line 14 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  virtual void simulate() override;
  #line 19 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  #line 19 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 26 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  #line 26 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 35 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  #line 35 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  virtual Distribution<Integer> marginal();
  #line 39 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  #line 39 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestBetaBinomial.birch"
extern "C" TestBetaBinomial_* make_TestBetaBinomial_();

#line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
class TestBetaGeometric_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  LIBBIRCH_CLASS(TestBetaGeometric_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c1, x, \u03b1, \u03b2)
  #line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  TestBetaGeometric_();

  Random<Real> \u03c1;
  Random<Integer> x;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  #line 7 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  #line 7 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  virtual void initialize() override;
  #line 12 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  #line 12 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  virtual void simulate() override;
  #line 17 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  #line 17 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 24 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  #line 24 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 33 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  #line 33 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  virtual Distribution<Integer> marginal();
  #line 37 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  #line 37 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestBetaGeometric.birch"
extern "C" TestBetaGeometric_* make_TestBetaGeometric_();

#line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
class TestBetaNegativeBinomial_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  LIBBIRCH_CLASS(TestBetaNegativeBinomial_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  LIBBIRCH_CLASS_MEMBERS(k, \u03b1, \u03b2, \u03c1, x)
  #line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  TestBetaNegativeBinomial_();

  Random<Integer> k;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  Random<Real> \u03c1;
  Random<Integer> x;
  #line 8 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  #line 8 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  virtual void initialize() override;
  #line 14 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  #line 14 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  virtual void simulate() override;
  #line 19 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  #line 19 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 26 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  #line 26 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 35 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  #line 35 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  virtual Distribution<Integer> marginal();
  #line 39 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  #line 39 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestBetaNegativeBinomial.birch"
extern "C" TestBetaNegativeBinomial_* make_TestBetaNegativeBinomial_();

#line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
class TestBinomial_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  LIBBIRCH_CLASS(TestBinomial_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  LIBBIRCH_CLASS_MEMBERS(n, \u03c1, x)
  #line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  TestBinomial_();

  Random<Integer> n;
  Random<Real> \u03c1;
  Random<Integer> x;
  #line 6 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  #line 6 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  #line 11 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  virtual void simulate() override;
  #line 15 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  #line 15 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 19 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  #line 19 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 23 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  #line 23 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  virtual Distribution<Integer> marginal();
  #line 27 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  #line 27 "src/test/distribution/scalar_discrete/TestBinomial.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestBinomial.birch"
extern "C" TestBinomial_* make_TestBinomial_();

#line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
#line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
class TestCategorical_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  LIBBIRCH_CLASS(TestCategorical_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03b1, \u03c1)
  #line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  TestCategorical_();

  Random<Integer> x;
  Real \u03b1;
  libbirch::DefaultArray<Real,1> \u03c1;
  #line 6 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  #line 6 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  #line 11 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  virtual void simulate() override;
  #line 15 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  #line 15 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 19 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  #line 19 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 23 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  #line 23 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  virtual Distribution<Integer> marginal();
  #line 27 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  #line 27 "src/test/distribution/scalar_discrete/TestCategorical.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestCategorical.birch"
extern "C" TestCategorical_* make_TestCategorical_();

#line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
class TestConwayMaxwellPoisson_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  LIBBIRCH_CLASS(TestConwayMaxwellPoisson_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03bb, \u03bd, n)
  #line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  TestConwayMaxwellPoisson_();

  Random<Integer> x;
  Random<Real> \u03bb;
  Random<Real> \u03bd;
  Integer n;
  #line 7 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  #line 7 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  virtual void initialize() override;
  #line 14 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  #line 14 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  virtual void simulate() override;
  #line 18 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  #line 18 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 22 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  #line 22 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 26 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  #line 26 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  virtual Distribution<Integer> marginal();
  #line 30 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  #line 30 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestConwayMaxwellPoisson.birch"
extern "C" TestConwayMaxwellPoisson_* make_TestConwayMaxwellPoisson_();

#line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
#line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
class TestDirichletCategorical_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  LIBBIRCH_CLASS(TestDirichletCategorical_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c1, x, \u03b1)
  #line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  TestDirichletCategorical_();

  Random<libbirch::DefaultArray<Real,1>> \u03c1;
  Random<Integer> x;
  libbirch::DefaultArray<Real,1> \u03b1;
  #line 6 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  #line 6 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  virtual void initialize() override;
  #line 12 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  #line 12 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  virtual void simulate() override;
  #line 17 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  #line 17 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 24 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  #line 24 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 33 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  #line 33 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  virtual Distribution<Integer> marginal();
  #line 37 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  #line 37 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestDirichletCategorical.birch"
extern "C" TestDirichletCategorical_* make_TestDirichletCategorical_();

#line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
class TestGammaPoisson_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  LIBBIRCH_CLASS(TestGammaPoisson_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bb, x, k, \u03b8)
  #line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  TestGammaPoisson_();

  Random<Real> \u03bb;
  Random<Integer> x;
  Random<Real> k;
  Random<Real> \u03b8;
  #line 7 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  #line 7 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  virtual void initialize() override;
  #line 12 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  #line 12 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  virtual void simulate() override;
  #line 17 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  #line 17 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 24 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  #line 24 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 33 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  #line 33 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  virtual Distribution<Integer> marginal();
  #line 37 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  #line 37 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestGammaPoisson.birch"
extern "C" TestGammaPoisson_* make_TestGammaPoisson_();

#line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
#line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
class TestGeometric_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  LIBBIRCH_CLASS(TestGeometric_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03c1)
  #line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  TestGeometric_();

  Random<Integer> x;
  Random<Real> \u03c1;
  #line 5 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  #line 5 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  virtual void initialize() override;
  #line 9 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  #line 9 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  virtual void simulate() override;
  #line 13 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  #line 13 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 17 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  #line 17 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 21 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  #line 21 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  virtual Distribution<Integer> marginal();
  #line 25 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  #line 25 "src/test/distribution/scalar_discrete/TestGeometric.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestGeometric.birch"
extern "C" TestGeometric_* make_TestGeometric_();

#line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
#line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
class TestNegativeBinomial_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  LIBBIRCH_CLASS(TestNegativeBinomial_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  LIBBIRCH_CLASS_MEMBERS(x, k, \u03c1)
  #line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  TestNegativeBinomial_();

  Random<Integer> x;
  Random<Integer> k;
  Random<Real> \u03c1;
  #line 6 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  #line 6 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  #line 11 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  virtual void simulate() override;
  #line 15 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  #line 15 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 19 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  #line 19 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 23 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  #line 23 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  virtual Distribution<Integer> marginal();
  #line 27 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  #line 27 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestNegativeBinomial.birch"
extern "C" TestNegativeBinomial_* make_TestNegativeBinomial_();

#line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
class TestPoisson_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  LIBBIRCH_CLASS(TestPoisson_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03bb)
  #line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  TestPoisson_();

  Random<Integer> x;
  Random<Real> \u03bb;
  #line 5 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  #line 5 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  virtual void initialize() override;
  #line 9 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  #line 9 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  virtual void simulate() override;
  #line 13 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  #line 13 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 17 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  #line 17 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 21 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  #line 21 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  virtual Distribution<Integer> marginal();
  #line 25 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  #line 25 "src/test/distribution/scalar_discrete/TestPoisson.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestPoisson.birch"
extern "C" TestPoisson_* make_TestPoisson_();

#line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
#line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
class TestScaledGammaPoisson_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  LIBBIRCH_CLASS(TestScaledGammaPoisson_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bb, x, a, k, \u03b8)
  #line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  TestScaledGammaPoisson_();

  Random<Real> \u03bb;
  Random<Integer> x;
  Random<Real> a;
  Random<Real> k;
  Random<Real> \u03b8;
  #line 9 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  #line 9 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  virtual void initialize() override;
  #line 15 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  #line 15 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  virtual void simulate() override;
  #line 20 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  #line 20 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 27 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  #line 27 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 36 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  #line 36 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  virtual Distribution<Integer> marginal();
  #line 40 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  #line 40 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestScaledGammaPoisson.birch"
extern "C" TestScaledGammaPoisson_* make_TestScaledGammaPoisson_();

#line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
#line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
class TestUniformInteger_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  LIBBIRCH_CLASS(TestUniformInteger_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  LIBBIRCH_CLASS_MEMBERS(x, l, u)
  #line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  TestUniformInteger_();

  Random<Integer> x;
  Integer l;
  Integer u;
  #line 6 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  #line 6 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  #line 11 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  virtual void simulate() override;
  #line 15 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  #line 15 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 19 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  #line 19 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 23 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  #line 23 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  virtual Distribution<Integer> marginal();
  #line 27 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  #line 27 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_discrete/TestUniformInteger.birch"
extern "C" TestUniformInteger_* make_TestUniformInteger_();

#line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
class TestAddDiscreteDelta_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  LIBBIRCH_CLASS(TestAddDiscreteDelta_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  LIBBIRCH_CLASS_MEMBERS(x1, x2, s)
  #line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  TestAddDiscreteDelta_();

  Random<Integer> x1;
  Random<Integer> x2;
  Random<Integer> s;
  #line 6 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  #line 6 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  virtual void initialize() override;
  #line 10 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  #line 10 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  virtual void simulate() override;
  #line 16 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  #line 16 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 24 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  #line 24 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 34 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  #line 34 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  virtual Distribution<Integer> marginal();
  #line 38 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  #line 38 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_enumerate/TestAddDiscreteDelta.birch"
extern "C" TestAddDiscreteDelta_* make_TestAddDiscreteDelta_();

#line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
class TestLongDiscreteDelta_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  LIBBIRCH_CLASS(TestLongDiscreteDelta_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  LIBBIRCH_CLASS_MEMBERS(x, s)
  #line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  TestLongDiscreteDelta_();

  libbirch::DefaultArray<Random<Integer>,1> x;
  Random<Integer> s;
  #line 5 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  #line 5 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  virtual void initialize() override;
  #line 9 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  #line 9 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  virtual void simulate() override;
  #line 18 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  #line 18 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 29 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  #line 29 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 45 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  #line 45 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  virtual Distribution<Integer> marginal();
  #line 49 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  #line 49 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_enumerate/TestLongDiscreteDelta.birch"
extern "C" TestLongDiscreteDelta_* make_TestLongDiscreteDelta_();

#line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
class TestNegateDiscreteDelta_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  LIBBIRCH_CLASS(TestNegateDiscreteDelta_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c1, x, y, \u03b1, \u03b2, n)
  #line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  TestNegateDiscreteDelta_();

  Random<Real> \u03c1;
  Random<Integer> x;
  Random<Integer> y;
  Real \u03b1;
  Real \u03b2;
  Integer n;
  #line 10 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  #line 10 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  virtual void initialize() override;
  #line 16 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  #line 16 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  virtual void simulate() override;
  #line 22 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  #line 22 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 30 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  #line 30 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 41 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  #line 41 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  virtual Distribution<Integer> marginal();
  #line 45 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  #line 45 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_enumerate/TestNegateDiscreteDelta.birch"
extern "C" TestNegateDiscreteDelta_* make_TestNegateDiscreteDelta_();

#line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
#line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
class TestSubtractDiscreteDelta_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  LIBBIRCH_CLASS(TestSubtractDiscreteDelta_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  LIBBIRCH_CLASS_MEMBERS(x1, x2, s)
  #line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  TestSubtractDiscreteDelta_();

  Random<Integer> x1;
  Random<Integer> x2;
  Random<Integer> s;
  #line 6 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  #line 6 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  virtual void initialize() override;
  #line 10 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  #line 10 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  virtual void simulate() override;
  #line 16 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  #line 16 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 24 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  #line 24 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 34 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  #line 34 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  virtual Distribution<Integer> marginal();
  #line 38 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  #line 38 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_enumerate/TestSubtractDiscreteDelta.birch"
extern "C" TestSubtractDiscreteDelta_* make_TestSubtractDiscreteDelta_();

#line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
class TestChainGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  LIBBIRCH_CLASS(TestChainGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03bc, \u03c3)
  #line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  TestChainGaussian_();

  libbirch::DefaultArray<Random<Real>,1> x;
  Random<Real> \u03bc;
  libbirch::DefaultArray<Random<Real>,1> \u03c3;
  #line 6 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  #line 6 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  virtual void initialize() override;
  #line 13 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  #line 13 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  virtual void simulate() override;
  #line 21 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  #line 21 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 29 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  #line 29 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 38 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  #line 38 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  virtual Distribution<Real> marginal();
  #line 42 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  #line 42 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_gaussian/TestChainGaussian.birch"
extern "C" TestChainGaussian_* make_TestChainGaussian_();

#line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
class TestGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  LIBBIRCH_CLASS(TestGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03bc, \u03c32)
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  TestGaussian_();

  Random<Real> x;
  Random<Real> \u03bc;
  Random<Real> \u03c32;
  #line 6 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  #line 6 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  #line 11 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  virtual void simulate() override;
  #line 15 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  #line 15 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 19 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  #line 19 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 23 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  #line 23 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  virtual Distribution<Real> marginal();
  #line 27 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  #line 27 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_gaussian/TestGaussian.birch"
extern "C" TestGaussian_* make_TestGaussian_();

#line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
class TestGaussianGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  LIBBIRCH_CLASS(TestGaussianGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bc_1, x, \u03bc_0, \u03c32_0, \u03c32_1)
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  TestGaussianGaussian_();

  Random<Real> \u03bc_1;
  Random<Real> x;
  Random<Real> \u03bc_0;
  Random<Real> \u03c32_0;
  Random<Real> \u03c32_1;
  #line 8 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  #line 8 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  virtual void initialize() override;
  #line 14 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  #line 14 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  virtual void simulate() override;
  #line 19 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  #line 19 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 26 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  #line 26 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 35 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  #line 35 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  virtual Distribution<Real> marginal();
  #line 39 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  #line 39 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_gaussian/TestGaussianGaussian.birch"
extern "C" TestGaussianGaussian_* make_TestGaussianGaussian_();

#line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
class TestLinearGaussianGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  LIBBIRCH_CLASS(TestLinearGaussianGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bc_1, x, a, c, \u03bc_0, \u03c32_0, \u03c32_1)
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  TestLinearGaussianGaussian_();

  Random<Real> \u03bc_1;
  Random<Real> x;
  Random<Real> a;
  Random<Real> c;
  Random<Real> \u03bc_0;
  Random<Real> \u03c32_0;
  Random<Real> \u03c32_1;
  #line 10 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  #line 10 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  virtual void initialize() override;
  #line 18 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  #line 18 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  virtual void simulate() override;
  #line 23 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  #line 23 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 30 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  #line 30 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 39 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  #line 39 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  virtual Distribution<Real> marginal();
  #line 43 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  #line 43 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_gaussian/TestLinearGaussianGaussian.birch"
extern "C" TestLinearGaussianGaussian_* make_TestLinearGaussianGaussian_();

#line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
#line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
class TestLinearMultivariateGaussianGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  LIBBIRCH_CLASS(TestLinearMultivariateGaussianGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bc, x, n, a, m, S, c, s2)
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  TestLinearMultivariateGaussianGaussian_();

  Random<libbirch::DefaultArray<Real,1>> \u03bc;
  Random<Real> x;
  Integer n;
  Random<libbirch::DefaultArray<Real,1>> a;
  Random<libbirch::DefaultArray<Real,1>> m;
  Random<libbirch::DefaultArray<Real,2>> S;
  Random<Real> c;
  Random<Real> s2;
  #line 13 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  #line 13 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  virtual void initialize() override;
  #line 21 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  #line 21 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  virtual void simulate() override;
  #line 26 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  #line 26 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 33 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  #line 33 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 42 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  #line 42 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  virtual Distribution<Real> marginal();
  #line 46 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  #line 46 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_gaussian/TestLinearMultivariateGaussianGaussian.birch"
extern "C" TestLinearMultivariateGaussianGaussian_* make_TestLinearMultivariateGaussianGaussian_();

#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
class TestFanNormalInverseGammaGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  LIBBIRCH_CLASS(TestFanNormalInverseGammaGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c32, \u03bc, x1, x2, x3, x4, x5, \u03bc_0, a2, \u03b1, \u03b2, s2)
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  TestFanNormalInverseGammaGaussian_();

  Random<Real> \u03c32;
  Random<Real> \u03bc;
  Random<Real> x1;
  Random<Real> x2;
  Random<Real> x3;
  Random<Real> x4;
  Random<Real> x5;
  Random<Real> \u03bc_0;
  Random<Real> a2;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  Random<Real> s2;
  #line 16 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  #line 16 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  virtual void initialize() override;
  #line 24 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  #line 24 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  virtual void simulate() override;
  #line 34 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  #line 34 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 46 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  #line 46 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 61 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  #line 61 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  virtual Distribution<Real> marginal();
  #line 65 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  #line 65 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestFanNormalInverseGamma.birch"
extern "C" TestFanNormalInverseGammaGaussian_* make_TestFanNormalInverseGammaGaussian_();

#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
class TestLinearMultivariateNormalInverseGammaGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  LIBBIRCH_CLASS(TestLinearMultivariateNormalInverseGammaGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c32, \u03bc, x, n, a, m, S, \u03b1, \u03b2, c, s2)
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  TestLinearMultivariateNormalInverseGammaGaussian_();

  Random<Real> \u03c32;
  Random<libbirch::DefaultArray<Real,1>> \u03bc;
  Random<Real> x;
  Integer n;
  Random<libbirch::DefaultArray<Real,1>> a;
  Random<libbirch::DefaultArray<Real,1>> m;
  Random<libbirch::DefaultArray<Real,2>> S;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  Random<Real> c;
  Random<Real> s2;
  #line 16 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  #line 16 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  virtual void initialize() override;
  #line 26 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  #line 26 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  virtual void simulate() override;
  #line 32 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  #line 32 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 40 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  #line 40 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 51 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  #line 51 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  virtual Distribution<Real> marginal();
  #line 55 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  #line 55 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearMultivariateNormalInverseGammaGaussian.birch"
extern "C" TestLinearMultivariateNormalInverseGammaGaussian_* make_TestLinearMultivariateNormalInverseGammaGaussian_();

#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
class TestLinearNormalInverseGammaGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  LIBBIRCH_CLASS(TestLinearNormalInverseGammaGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c32, \u03bc, x, a, \u03bc_0, a2, \u03b1, \u03b2, c, s2)
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  TestLinearNormalInverseGammaGaussian_();

  Random<Real> \u03c32;
  Random<Real> \u03bc;
  Random<Real> x;
  Random<Real> a;
  Random<Real> \u03bc_0;
  Random<Real> a2;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  Random<Real> c;
  Random<Real> s2;
  #line 14 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  #line 14 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  virtual void initialize() override;
  #line 24 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  #line 24 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  virtual void simulate() override;
  #line 30 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  #line 30 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 38 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  #line 38 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 49 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  #line 49 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  virtual Distribution<Real> marginal();
  #line 53 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  #line 53 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestLinearNormalInverseGammaGaussian.birch"
extern "C" TestLinearNormalInverseGammaGaussian_* make_TestLinearNormalInverseGammaGaussian_();

#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
class TestNormalInverseGamma_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  LIBBIRCH_CLASS(TestNormalInverseGamma_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c32, x, \u03bc, a2, \u03b1, \u03b2)
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  TestNormalInverseGamma_();

  Random<Real> \u03c32;
  Random<Real> x;
  Random<Real> \u03bc;
  Random<Real> a2;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  #line 10 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  #line 10 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  virtual void initialize() override;
  #line 17 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  #line 17 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  virtual void simulate() override;
  #line 22 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  #line 22 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 29 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  #line 29 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 38 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  #line 38 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  virtual Distribution<Real> marginal();
  #line 42 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  #line 42 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGamma.birch"
extern "C" TestNormalInverseGamma_* make_TestNormalInverseGamma_();

#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
class TestNormalInverseGammaGaussian_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  LIBBIRCH_CLASS(TestNormalInverseGammaGaussian_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c32, \u03bc, x, \u03bc_0, a2, \u03b1, \u03b2)
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  TestNormalInverseGammaGaussian_();

  Random<Real> \u03c32;
  Random<Real> \u03bc;
  Random<Real> x;
  Random<Real> \u03bc_0;
  Random<Real> a2;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  #line 11 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  #line 11 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  virtual void initialize() override;
  #line 18 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  #line 18 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  virtual void simulate() override;
  #line 24 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  #line 24 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 32 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  #line 32 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 43 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  #line 43 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  virtual Distribution<Real> marginal();
  #line 47 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  #line 47 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_normal_inverse_gamma/TestNormalInverseGammaGaussian.birch"
extern "C" TestNormalInverseGammaGaussian_* make_TestNormalInverseGammaGaussian_();

#line 1 "src/test/distribution/scalar_other/TestBeta.birch"
#line 1 "src/test/distribution/scalar_other/TestBeta.birch"
class TestBeta_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_other/TestBeta.birch"
  LIBBIRCH_CLASS(TestBeta_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_other/TestBeta.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03b1, \u03b2)
  #line 1 "src/test/distribution/scalar_other/TestBeta.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_other/TestBeta.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_other/TestBeta.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_other/TestBeta.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_other/TestBeta.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_other/TestBeta.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_other/TestBeta.birch"
  TestBeta_();

  Random<Real> x;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  #line 6 "src/test/distribution/scalar_other/TestBeta.birch"
  #line 6 "src/test/distribution/scalar_other/TestBeta.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_other/TestBeta.birch"
  #line 11 "src/test/distribution/scalar_other/TestBeta.birch"
  virtual void simulate() override;
  #line 15 "src/test/distribution/scalar_other/TestBeta.birch"
  #line 15 "src/test/distribution/scalar_other/TestBeta.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 19 "src/test/distribution/scalar_other/TestBeta.birch"
  #line 19 "src/test/distribution/scalar_other/TestBeta.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 23 "src/test/distribution/scalar_other/TestBeta.birch"
  #line 23 "src/test/distribution/scalar_other/TestBeta.birch"
  virtual Distribution<Real> marginal();
  #line 27 "src/test/distribution/scalar_other/TestBeta.birch"
  #line 27 "src/test/distribution/scalar_other/TestBeta.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_other/TestBeta.birch"
extern "C" TestBeta_* make_TestBeta_();

#line 1 "src/test/distribution/scalar_other/TestExponential.birch"
#line 1 "src/test/distribution/scalar_other/TestExponential.birch"
class TestExponential_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_other/TestExponential.birch"
  LIBBIRCH_CLASS(TestExponential_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_other/TestExponential.birch"
  LIBBIRCH_CLASS_MEMBERS(x, \u03bb)
  #line 1 "src/test/distribution/scalar_other/TestExponential.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_other/TestExponential.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_other/TestExponential.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_other/TestExponential.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_other/TestExponential.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_other/TestExponential.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_other/TestExponential.birch"
  TestExponential_();

  Random<Real> x;
  Random<Real> \u03bb;
  #line 5 "src/test/distribution/scalar_other/TestExponential.birch"
  #line 5 "src/test/distribution/scalar_other/TestExponential.birch"
  virtual void initialize() override;
  #line 9 "src/test/distribution/scalar_other/TestExponential.birch"
  #line 9 "src/test/distribution/scalar_other/TestExponential.birch"
  virtual void simulate() override;
  #line 13 "src/test/distribution/scalar_other/TestExponential.birch"
  #line 13 "src/test/distribution/scalar_other/TestExponential.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 17 "src/test/distribution/scalar_other/TestExponential.birch"
  #line 17 "src/test/distribution/scalar_other/TestExponential.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 21 "src/test/distribution/scalar_other/TestExponential.birch"
  #line 21 "src/test/distribution/scalar_other/TestExponential.birch"
  virtual Distribution<Real> marginal();
  #line 25 "src/test/distribution/scalar_other/TestExponential.birch"
  #line 25 "src/test/distribution/scalar_other/TestExponential.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_other/TestExponential.birch"
extern "C" TestExponential_* make_TestExponential_();

#line 1 "src/test/distribution/scalar_other/TestGamma.birch"
#line 1 "src/test/distribution/scalar_other/TestGamma.birch"
class TestGamma_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_other/TestGamma.birch"
  LIBBIRCH_CLASS(TestGamma_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_other/TestGamma.birch"
  LIBBIRCH_CLASS_MEMBERS(x, k, \u03b8)
  #line 1 "src/test/distribution/scalar_other/TestGamma.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_other/TestGamma.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_other/TestGamma.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_other/TestGamma.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_other/TestGamma.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_other/TestGamma.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_other/TestGamma.birch"
  TestGamma_();

  Random<Real> x;
  Random<Real> k;
  Random<Real> \u03b8;
  #line 6 "src/test/distribution/scalar_other/TestGamma.birch"
  #line 6 "src/test/distribution/scalar_other/TestGamma.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_other/TestGamma.birch"
  #line 11 "src/test/distribution/scalar_other/TestGamma.birch"
  virtual void simulate() override;
  #line 15 "src/test/distribution/scalar_other/TestGamma.birch"
  #line 15 "src/test/distribution/scalar_other/TestGamma.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 19 "src/test/distribution/scalar_other/TestGamma.birch"
  #line 19 "src/test/distribution/scalar_other/TestGamma.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 23 "src/test/distribution/scalar_other/TestGamma.birch"
  #line 23 "src/test/distribution/scalar_other/TestGamma.birch"
  virtual Distribution<Real> marginal();
  #line 27 "src/test/distribution/scalar_other/TestGamma.birch"
  #line 27 "src/test/distribution/scalar_other/TestGamma.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_other/TestGamma.birch"
extern "C" TestGamma_* make_TestGamma_();

#line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
#line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
class TestGammaExponential_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  LIBBIRCH_CLASS(TestGammaExponential_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  LIBBIRCH_CLASS_MEMBERS(k, \u03b8, \u03bb, x)
  #line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  TestGammaExponential_();

  Random<Real> k;
  Random<Real> \u03b8;
  Random<Real> \u03bb;
  Random<Real> x;
  #line 7 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  #line 7 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  virtual void initialize() override;
  #line 12 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  #line 12 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  virtual void simulate() override;
  #line 17 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  #line 17 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 24 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  #line 24 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 33 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  #line 33 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  virtual Distribution<Real> marginal();
  #line 37 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  #line 37 "src/test/distribution/scalar_other/TestGammaExponential.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_other/TestGammaExponential.birch"
extern "C" TestGammaExponential_* make_TestGammaExponential_();

#line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
#line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
class TestInverseGamma_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  LIBBIRCH_CLASS(TestInverseGamma_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03c32, \u03b1, \u03b2)
  #line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  TestInverseGamma_();

  Random<Real> \u03c32;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  #line 6 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  #line 6 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  #line 11 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  virtual void simulate() override;
  #line 15 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  #line 15 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 19 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  #line 19 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 23 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  #line 23 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  virtual Distribution<Real> marginal();
  #line 27 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  #line 27 "src/test/distribution/scalar_other/TestInverseGamma.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_other/TestInverseGamma.birch"
extern "C" TestInverseGamma_* make_TestInverseGamma_();

#line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
#line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
class TestInverseGammaGamma_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  LIBBIRCH_CLASS(TestInverseGammaGamma_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03b8, x, k, \u03b1, \u03b2)
  #line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  TestInverseGammaGamma_();

  Random<Real> \u03b8;
  Random<Real> x;
  Random<Real> k;
  Random<Real> \u03b1;
  Random<Real> \u03b2;
  #line 8 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  #line 8 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  virtual void initialize() override;
  #line 14 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  #line 14 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  virtual void simulate() override;
  #line 19 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  #line 19 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 26 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  #line 26 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 35 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  #line 35 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  virtual Distribution<Real> marginal();
  #line 39 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  #line 39 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_other/TestInverseGammaGamma.birch"
extern "C" TestInverseGammaGamma_* make_TestInverseGammaGamma_();

#line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
#line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
class TestScaledGammaExponential_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  LIBBIRCH_CLASS(TestScaledGammaExponential_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  LIBBIRCH_CLASS_MEMBERS(\u03bb, x, a, k, \u03b8)
  #line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  TestScaledGammaExponential_();

  Random<Real> \u03bb;
  Random<Real> x;
  Random<Real> a;
  Random<Real> k;
  Random<Real> \u03b8;
  #line 9 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  #line 9 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  virtual void initialize() override;
  #line 15 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  #line 15 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  virtual void simulate() override;
  #line 20 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  #line 20 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 27 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  #line 27 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 36 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  #line 36 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  virtual Distribution<Real> marginal();
  #line 40 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  #line 40 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_other/TestScaledGammaExponential.birch"
extern "C" TestScaledGammaExponential_* make_TestScaledGammaExponential_();

#line 1 "src/test/distribution/scalar_other/TestStudent.birch"
#line 1 "src/test/distribution/scalar_other/TestStudent.birch"
class TestStudent_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_other/TestStudent.birch"
  LIBBIRCH_CLASS(TestStudent_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_other/TestStudent.birch"
  LIBBIRCH_CLASS_MEMBERS(x, k, \u03bc, \u03c32)
  #line 1 "src/test/distribution/scalar_other/TestStudent.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_other/TestStudent.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_other/TestStudent.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_other/TestStudent.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_other/TestStudent.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_other/TestStudent.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_other/TestStudent.birch"
  TestStudent_();

  Random<Real> x;
  Random<Real> k;
  Random<Real> \u03bc;
  Random<Real> \u03c32;
  #line 7 "src/test/distribution/scalar_other/TestStudent.birch"
  #line 7 "src/test/distribution/scalar_other/TestStudent.birch"
  virtual void initialize() override;
  #line 13 "src/test/distribution/scalar_other/TestStudent.birch"
  #line 13 "src/test/distribution/scalar_other/TestStudent.birch"
  virtual void simulate() override;
  #line 17 "src/test/distribution/scalar_other/TestStudent.birch"
  #line 17 "src/test/distribution/scalar_other/TestStudent.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 21 "src/test/distribution/scalar_other/TestStudent.birch"
  #line 21 "src/test/distribution/scalar_other/TestStudent.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 25 "src/test/distribution/scalar_other/TestStudent.birch"
  #line 25 "src/test/distribution/scalar_other/TestStudent.birch"
  virtual Distribution<Real> marginal();
  #line 29 "src/test/distribution/scalar_other/TestStudent.birch"
  #line 29 "src/test/distribution/scalar_other/TestStudent.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_other/TestStudent.birch"
extern "C" TestStudent_* make_TestStudent_();

#line 1 "src/test/distribution/scalar_other/TestUniform.birch"
#line 1 "src/test/distribution/scalar_other/TestUniform.birch"
class TestUniform_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_other/TestUniform.birch"
  LIBBIRCH_CLASS(TestUniform_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_other/TestUniform.birch"
  LIBBIRCH_CLASS_MEMBERS(x, l, u)
  #line 1 "src/test/distribution/scalar_other/TestUniform.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_other/TestUniform.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_other/TestUniform.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_other/TestUniform.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_other/TestUniform.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_other/TestUniform.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_other/TestUniform.birch"
  TestUniform_();

  Random<Real> x;
  Random<Real> l;
  Random<Real> u;
  #line 6 "src/test/distribution/scalar_other/TestUniform.birch"
  #line 6 "src/test/distribution/scalar_other/TestUniform.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_other/TestUniform.birch"
  #line 11 "src/test/distribution/scalar_other/TestUniform.birch"
  virtual void simulate() override;
  #line 15 "src/test/distribution/scalar_other/TestUniform.birch"
  #line 15 "src/test/distribution/scalar_other/TestUniform.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 19 "src/test/distribution/scalar_other/TestUniform.birch"
  #line 19 "src/test/distribution/scalar_other/TestUniform.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 23 "src/test/distribution/scalar_other/TestUniform.birch"
  #line 23 "src/test/distribution/scalar_other/TestUniform.birch"
  virtual Distribution<Real> marginal();
  #line 27 "src/test/distribution/scalar_other/TestUniform.birch"
  #line 27 "src/test/distribution/scalar_other/TestUniform.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_other/TestUniform.birch"
extern "C" TestUniform_* make_TestUniform_();

#line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
#line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
class TestWeibull_ : public libbirch::unwrap_pointer<TestModel>::type {
public:
  #line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
  LIBBIRCH_CLASS(TestWeibull_, typename libbirch::unwrap_pointer<TestModel>::type)
  #line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
  LIBBIRCH_CLASS_MEMBERS(x, k, \u03bb)
  #line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
  using base_type_::operator=;
  #line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
  using base_type_::backward;
  #line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
  using base_type_::forward;
  #line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
  using base_type_::initialize;
  #line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
  using base_type_::simulate;
  #line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
  using base_type_::size;
  
  #line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
  TestWeibull_();

  Random<Real> x;
  Random<Real> k;
  Random<Real> \u03bb;
  #line 6 "src/test/distribution/scalar_other/TestWeibull.birch"
  #line 6 "src/test/distribution/scalar_other/TestWeibull.birch"
  virtual void initialize() override;
  #line 11 "src/test/distribution/scalar_other/TestWeibull.birch"
  #line 11 "src/test/distribution/scalar_other/TestWeibull.birch"
  virtual void simulate() override;
  #line 15 "src/test/distribution/scalar_other/TestWeibull.birch"
  #line 15 "src/test/distribution/scalar_other/TestWeibull.birch"
  virtual libbirch::DefaultArray<Real,1> forward() override;
  #line 19 "src/test/distribution/scalar_other/TestWeibull.birch"
  #line 19 "src/test/distribution/scalar_other/TestWeibull.birch"
  virtual libbirch::DefaultArray<Real,1> backward() override;
  #line 23 "src/test/distribution/scalar_other/TestWeibull.birch"
  #line 23 "src/test/distribution/scalar_other/TestWeibull.birch"
  virtual Distribution<Real> marginal();
  #line 27 "src/test/distribution/scalar_other/TestWeibull.birch"
  #line 27 "src/test/distribution/scalar_other/TestWeibull.birch"
  virtual Integer size() override;
};

#line 1 "src/test/distribution/scalar_other/TestWeibull.birch"
extern "C" TestWeibull_* make_TestWeibull_();

#line 1 "src/sampler/ParticleSampler.birch"
#line 1 "src/sampler/ParticleSampler.birch"
class ParticleSampler_ : public Object_ {
public:
  #line 1 "src/sampler/ParticleSampler.birch"
  LIBBIRCH_CLASS(ParticleSampler_, Object_)
  #line 1 "src/sampler/ParticleSampler.birch"
  LIBBIRCH_CLASS_NO_MEMBERS()
  #line 1 "src/sampler/ParticleSampler.birch"
  using base_type_::operator=;
  
  #line 1 "src/sampler/ParticleSampler.birch"
  ParticleSampler_();

  #line 12 "src/sampler/ParticleSampler.birch"
  #line 12 "src/sampler/ParticleSampler.birch"
  virtual void sample(const ParticleFilter& filter, const Model& model, const Buffer& input);
  #line 23 "src/sampler/ParticleSampler.birch"
  #line 23 "src/sampler/ParticleSampler.birch"
  virtual void sample(const ParticleFilter& filter, const Integer& t, const Buffer& input);
  #line 35 "src/sampler/ParticleSampler.birch"
  #line 35 "src/sampler/ParticleSampler.birch"
  virtual void sample(const ParticleFilter& filter, const Integer& t, const Buffer& input, const Kernel& \u03ba);
  #line 42 "src/sampler/ParticleSampler.birch"
  #line 42 "src/sampler/ParticleSampler.birch"
  virtual std::tuple<Model, Real> draw(const ParticleFilter& filter);
};

#line 1 "src/sampler/ParticleSampler.birch"
extern "C" ParticleSampler_* make_ParticleSampler_();

#line 1 "src/utility/ProgressBar.birch"
#line 1 "src/utility/ProgressBar.birch"
class ProgressBar_ : public Object_ {
public:
  #line 1 "src/utility/ProgressBar.birch"
  LIBBIRCH_CLASS(ProgressBar_, Object_)
  #line 1 "src/utility/ProgressBar.birch"
  LIBBIRCH_CLASS_MEMBERS(out, current, maximum)
  #line 1 "src/utility/ProgressBar.birch"
  using base_type_::operator=;
  
  #line 1 "src/utility/ProgressBar.birch"
  ProgressBar_();

  OutputStream out;
  Integer current;
  Integer maximum;
  #line 30 "src/utility/ProgressBar.birch"
  #line 30 "src/utility/ProgressBar.birch"
  virtual void update(const Real& progress);
};

#line 1 "src/utility/ProgressBar.birch"
extern "C" ProgressBar_* make_ProgressBar_();

#line 1 "src/form/binary_matrix/CholSolve.birch"
#line 1 "src/form/binary_matrix/CholSolve.birch"
struct CholSolveFunction_ final {
public:
  #line 1 "src/form/binary_matrix/CholSolve.birch"
  LIBBIRCH_STRUCT(CholSolveFunction_)
  #line 1 "src/form/binary_matrix/CholSolve.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_matrix/CholSolve.birch"
  CholSolveFunction_();

  #line 2 "src/form/binary_matrix/CholSolve.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_matrix/CholSolve.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_matrix/CholSolve.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_matrix/CholSolve.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_matrix/Diagonal.birch"
#line 1 "src/form/binary_matrix/Diagonal.birch"
struct DiagonalFunction_ final {
public:
  #line 1 "src/form/binary_matrix/Diagonal.birch"
  LIBBIRCH_STRUCT(DiagonalFunction_)
  #line 1 "src/form/binary_matrix/Diagonal.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_matrix/Diagonal.birch"
  DiagonalFunction_();

  #line 2 "src/form/binary_matrix/Diagonal.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_matrix/Diagonal.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_matrix/Diagonal.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_matrix/Diagonal.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_matrix/Dot.birch"
#line 1 "src/form/binary_matrix/Dot.birch"
struct DotFunction_ final {
public:
  #line 1 "src/form/binary_matrix/Dot.birch"
  LIBBIRCH_STRUCT(DotFunction_)
  #line 1 "src/form/binary_matrix/Dot.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_matrix/Dot.birch"
  DotFunction_();

  #line 2 "src/form/binary_matrix/Dot.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_matrix/Dot.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_matrix/Dot.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_matrix/Dot.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_matrix/Inner.birch"
#line 1 "src/form/binary_matrix/Inner.birch"
struct InnerFunction_ final {
public:
  #line 1 "src/form/binary_matrix/Inner.birch"
  LIBBIRCH_STRUCT(InnerFunction_)
  #line 1 "src/form/binary_matrix/Inner.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_matrix/Inner.birch"
  InnerFunction_();

  #line 2 "src/form/binary_matrix/Inner.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_matrix/Inner.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_matrix/Inner.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_matrix/Inner.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_matrix/MatrixPack.birch"
#line 1 "src/form/binary_matrix/MatrixPack.birch"
struct MatrixPackFunction_ final {
public:
  #line 1 "src/form/binary_matrix/MatrixPack.birch"
  LIBBIRCH_STRUCT(MatrixPackFunction_)
  #line 1 "src/form/binary_matrix/MatrixPack.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_matrix/MatrixPack.birch"
  MatrixPackFunction_();

  #line 2 "src/form/binary_matrix/MatrixPack.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_matrix/MatrixPack.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_matrix/MatrixPack.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_matrix/MatrixPack.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
  #line 11 "src/form/binary_matrix/MatrixPack.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 11 "src/form/binary_matrix/MatrixPack.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const libbirch::DefaultArray<RightValue,1>& r);
  #line 16 "src/form/binary_matrix/MatrixPack.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 16 "src/form/binary_matrix/MatrixPack.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,2>& l, const libbirch::DefaultArray<RightValue,2>& r);
};

#line 1 "src/form/binary_matrix/MatrixStack.birch"
#line 1 "src/form/binary_matrix/MatrixStack.birch"
struct MatrixStackFunction_ final {
public:
  #line 1 "src/form/binary_matrix/MatrixStack.birch"
  LIBBIRCH_STRUCT(MatrixStackFunction_)
  #line 1 "src/form/binary_matrix/MatrixStack.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_matrix/MatrixStack.birch"
  MatrixStackFunction_();

  #line 2 "src/form/binary_matrix/MatrixStack.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_matrix/MatrixStack.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_matrix/MatrixStack.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_matrix/MatrixStack.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
  #line 11 "src/form/binary_matrix/MatrixStack.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 11 "src/form/binary_matrix/MatrixStack.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const RightValue& r);
  #line 16 "src/form/binary_matrix/MatrixStack.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 16 "src/form/binary_matrix/MatrixStack.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const libbirch::DefaultArray<RightValue,1>& r);
  #line 21 "src/form/binary_matrix/MatrixStack.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 21 "src/form/binary_matrix/MatrixStack.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const libbirch::DefaultArray<RightValue,1>& r);
  #line 26 "src/form/binary_matrix/MatrixStack.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 26 "src/form/binary_matrix/MatrixStack.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,2>& l, const libbirch::DefaultArray<RightValue,2>& r);
};

#line 1 "src/form/binary_matrix/Outer.birch"
#line 1 "src/form/binary_matrix/Outer.birch"
struct OuterFunction_ final {
public:
  #line 1 "src/form/binary_matrix/Outer.birch"
  LIBBIRCH_STRUCT(OuterFunction_)
  #line 1 "src/form/binary_matrix/Outer.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_matrix/Outer.birch"
  OuterFunction_();

  #line 2 "src/form/binary_matrix/Outer.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_matrix/Outer.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_matrix/Outer.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_matrix/Outer.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_matrix/Solve.birch"
#line 1 "src/form/binary_matrix/Solve.birch"
struct SolveFunction_ final {
public:
  #line 1 "src/form/binary_matrix/Solve.birch"
  LIBBIRCH_STRUCT(SolveFunction_)
  #line 1 "src/form/binary_matrix/Solve.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_matrix/Solve.birch"
  SolveFunction_();

  #line 2 "src/form/binary_matrix/Solve.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_matrix/Solve.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_matrix/Solve.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_matrix/Solve.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_other/Add.birch"
#line 1 "src/form/binary_other/Add.birch"
struct AddFunction_ final {
public:
  #line 1 "src/form/binary_other/Add.birch"
  LIBBIRCH_STRUCT(AddFunction_)
  #line 1 "src/form/binary_other/Add.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_other/Add.birch"
  AddFunction_();

  #line 2 "src/form/binary_other/Add.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_other/Add.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_other/Add.birch"
  template<class Gradient>
  #line 6 "src/form/binary_other/Add.birch"
  decltype(auto) grad(const Gradient& d);
};

#line 1 "src/form/binary_other/CopySign.birch"
#line 1 "src/form/binary_other/CopySign.birch"
struct CopySignFunction_ final {
public:
  #line 1 "src/form/binary_other/CopySign.birch"
  LIBBIRCH_STRUCT(CopySignFunction_)
  #line 1 "src/form/binary_other/CopySign.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_other/CopySign.birch"
  CopySignFunction_();

  #line 2 "src/form/binary_other/CopySign.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_other/CopySign.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_other/CopySign.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_other/CopySign.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_other/Divide.birch"
#line 1 "src/form/binary_other/Divide.birch"
struct DivideFunction_ final {
public:
  #line 1 "src/form/binary_other/Divide.birch"
  LIBBIRCH_STRUCT(DivideFunction_)
  #line 1 "src/form/binary_other/Divide.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_other/Divide.birch"
  DivideFunction_();

  #line 2 "src/form/binary_other/Divide.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_other/Divide.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_other/Divide.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_other/Divide.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_other/LessThan.birch"
#line 1 "src/form/binary_other/LessThan.birch"
struct LessThanFunction_ final {
public:
  #line 1 "src/form/binary_other/LessThan.birch"
  LIBBIRCH_STRUCT(LessThanFunction_)
  #line 1 "src/form/binary_other/LessThan.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_other/LessThan.birch"
  LessThanFunction_();

  #line 2 "src/form/binary_other/LessThan.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_other/LessThan.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_other/LessThan.birch"
  template<class Gradient>
  #line 6 "src/form/binary_other/LessThan.birch"
  decltype(auto) grad(const Gradient& d);
};

#line 1 "src/form/binary_other/LogBeta.birch"
#line 1 "src/form/binary_other/LogBeta.birch"
struct LogBetaFunction_ final {
public:
  #line 1 "src/form/binary_other/LogBeta.birch"
  LIBBIRCH_STRUCT(LogBetaFunction_)
  #line 1 "src/form/binary_other/LogBeta.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_other/LogBeta.birch"
  LogBetaFunction_();

  #line 2 "src/form/binary_other/LogBeta.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_other/LogBeta.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_other/LogBeta.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_other/LogBeta.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_other/LogChoose.birch"
#line 1 "src/form/binary_other/LogChoose.birch"
struct LogChooseFunction_ final {
public:
  #line 1 "src/form/binary_other/LogChoose.birch"
  LIBBIRCH_STRUCT(LogChooseFunction_)
  #line 1 "src/form/binary_other/LogChoose.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_other/LogChoose.birch"
  LogChooseFunction_();

  #line 2 "src/form/binary_other/LogChoose.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_other/LogChoose.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_other/LogChoose.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_other/LogChoose.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_other/LogGammaP.birch"
#line 1 "src/form/binary_other/LogGammaP.birch"
struct LogGammaPFunction_ final {
public:
  #line 1 "src/form/binary_other/LogGammaP.birch"
  LIBBIRCH_STRUCT(LogGammaPFunction_)
  #line 1 "src/form/binary_other/LogGammaP.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_other/LogGammaP.birch"
  LogGammaPFunction_();

  #line 2 "src/form/binary_other/LogGammaP.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_other/LogGammaP.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_other/LogGammaP.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_other/LogGammaP.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_other/Multiply.birch"
#line 1 "src/form/binary_other/Multiply.birch"
struct MultiplyFunction_ final {
public:
  #line 1 "src/form/binary_other/Multiply.birch"
  LIBBIRCH_STRUCT(MultiplyFunction_)
  #line 1 "src/form/binary_other/Multiply.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_other/Multiply.birch"
  MultiplyFunction_();

  #line 2 "src/form/binary_other/Multiply.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_other/Multiply.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_other/Multiply.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_other/Multiply.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
  #line 11 "src/form/binary_other/Multiply.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 11 "src/form/binary_other/Multiply.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const libbirch::DefaultArray<RightValue,1>& r);
  #line 16 "src/form/binary_other/Multiply.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 16 "src/form/binary_other/Multiply.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const libbirch::DefaultArray<RightValue,2>& r);
  #line 21 "src/form/binary_other/Multiply.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 21 "src/form/binary_other/Multiply.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const RightValue& r);
  #line 26 "src/form/binary_other/Multiply.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 26 "src/form/binary_other/Multiply.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const libbirch::DefaultArray<RightValue,1>& r);
  #line 31 "src/form/binary_other/Multiply.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 31 "src/form/binary_other/Multiply.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const libbirch::DefaultArray<RightValue,2>& r);
  #line 36 "src/form/binary_other/Multiply.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 36 "src/form/binary_other/Multiply.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,2>& l, const RightValue& r);
  #line 41 "src/form/binary_other/Multiply.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 41 "src/form/binary_other/Multiply.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,2>& l, const libbirch::DefaultArray<RightValue,1>& r);
  #line 46 "src/form/binary_other/Multiply.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 46 "src/form/binary_other/Multiply.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,2>& l, const libbirch::DefaultArray<RightValue,2>& r);
};

#line 1 "src/form/binary_other/Pow.birch"
#line 1 "src/form/binary_other/Pow.birch"
struct PowFunction_ final {
public:
  #line 1 "src/form/binary_other/Pow.birch"
  LIBBIRCH_STRUCT(PowFunction_)
  #line 1 "src/form/binary_other/Pow.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_other/Pow.birch"
  PowFunction_();

  #line 2 "src/form/binary_other/Pow.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_other/Pow.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_other/Pow.birch"
  template<class Gradient,class LeftValue,class RightValue>
  #line 6 "src/form/binary_other/Pow.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const RightValue& r);
};

#line 1 "src/form/binary_other/Subtract.birch"
#line 1 "src/form/binary_other/Subtract.birch"
struct SubtractFunction_ final {
public:
  #line 1 "src/form/binary_other/Subtract.birch"
  LIBBIRCH_STRUCT(SubtractFunction_)
  #line 1 "src/form/binary_other/Subtract.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/binary_other/Subtract.birch"
  SubtractFunction_();

  #line 2 "src/form/binary_other/Subtract.birch"
  template<class LeftValue,class RightValue>
  #line 2 "src/form/binary_other/Subtract.birch"
  decltype(auto) compute(const LeftValue& l, const RightValue& r);
  #line 6 "src/form/binary_other/Subtract.birch"
  template<class Gradient>
  #line 6 "src/form/binary_other/Subtract.birch"
  decltype(auto) grad(const Gradient& d);
};

#line 1 "src/form/ternary/IfThenElse.birch"
#line 1 "src/form/ternary/IfThenElse.birch"
struct IfThenElseFunction_ final {
public:
  #line 1 "src/form/ternary/IfThenElse.birch"
  LIBBIRCH_STRUCT(IfThenElseFunction_)
  #line 1 "src/form/ternary/IfThenElse.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/ternary/IfThenElse.birch"
  IfThenElseFunction_();

  #line 2 "src/form/ternary/IfThenElse.birch"
  template<class LeftValue,class MiddleValue,class RightValue>
  #line 2 "src/form/ternary/IfThenElse.birch"
  decltype(auto) compute(const LeftValue& l, const MiddleValue& m, const RightValue& r);
  #line 6 "src/form/ternary/IfThenElse.birch"
  template<class Gradient,class LeftValue,class MiddleValue,class RightValue>
  #line 6 "src/form/ternary/IfThenElse.birch"
  decltype(auto) grad(const Gradient& d, const LeftValue& l, const MiddleValue& m, const RightValue& r);
};

#line 1 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
#line 1 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
struct LogZConwayMaxwellPoissonFunction_ final {
public:
  #line 1 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  LIBBIRCH_STRUCT(LogZConwayMaxwellPoissonFunction_)
  #line 1 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  LogZConwayMaxwellPoissonFunction_();

  #line 2 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  template<class Arg1,class Arg2,class Arg3>
  #line 2 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  decltype(auto) compute(const Arg1& \u03bb, const Arg2& \u03bd, const Arg3& n);
  #line 6 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  template<class Gradient,class Arg1,class Arg2,class Arg3>
  #line 6 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  decltype(auto) grad(const Gradient& d, const Arg1& \u03bb, const Arg2& \u03bd, const Arg3& n);
};

#line 1 "src/form/unary_matrix/CholInv.birch"
#line 1 "src/form/unary_matrix/CholInv.birch"
struct CholInvFunction_ final {
public:
  #line 1 "src/form/unary_matrix/CholInv.birch"
  LIBBIRCH_STRUCT(CholInvFunction_)
  #line 1 "src/form/unary_matrix/CholInv.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_matrix/CholInv.birch"
  CholInvFunction_();

  #line 2 "src/form/unary_matrix/CholInv.birch"
  template<class Value>
  #line 2 "src/form/unary_matrix/CholInv.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_matrix/CholInv.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_matrix/CholInv.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_matrix/Inv.birch"
#line 1 "src/form/unary_matrix/Inv.birch"
struct InvFunction_ final {
public:
  #line 1 "src/form/unary_matrix/Inv.birch"
  LIBBIRCH_STRUCT(InvFunction_)
  #line 1 "src/form/unary_matrix/Inv.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_matrix/Inv.birch"
  InvFunction_();

  #line 2 "src/form/unary_matrix/Inv.birch"
  template<class Value>
  #line 2 "src/form/unary_matrix/Inv.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_matrix/Inv.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_matrix/Inv.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_matrix/LCholDet.birch"
#line 1 "src/form/unary_matrix/LCholDet.birch"
struct LCholDetFunction_ final {
public:
  #line 1 "src/form/unary_matrix/LCholDet.birch"
  LIBBIRCH_STRUCT(LCholDetFunction_)
  #line 1 "src/form/unary_matrix/LCholDet.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_matrix/LCholDet.birch"
  LCholDetFunction_();

  #line 2 "src/form/unary_matrix/LCholDet.birch"
  template<class Value>
  #line 2 "src/form/unary_matrix/LCholDet.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_matrix/LCholDet.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_matrix/LCholDet.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_matrix/LDet.birch"
#line 1 "src/form/unary_matrix/LDet.birch"
struct LDetFunction_ final {
public:
  #line 1 "src/form/unary_matrix/LDet.birch"
  LIBBIRCH_STRUCT(LDetFunction_)
  #line 1 "src/form/unary_matrix/LDet.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_matrix/LDet.birch"
  LDetFunction_();

  #line 2 "src/form/unary_matrix/LDet.birch"
  template<class Value>
  #line 2 "src/form/unary_matrix/LDet.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_matrix/LDet.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_matrix/LDet.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 1 "src/form/unary_matrix/Matrix.birch"
struct MatrixFunction_ final {
public:
  #line 1 "src/form/unary_matrix/Matrix.birch"
  using Value_ = Value;
  #line 1 "src/form/unary_matrix/Matrix.birch"
  LIBBIRCH_STRUCT(MatrixFunction_)
  #line 1 "src/form/unary_matrix/Matrix.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_matrix/Matrix.birch"
  MatrixFunction_();

  #line 2 "src/form/unary_matrix/Matrix.birch"
  template<class MiddleValue>
  #line 2 "src/form/unary_matrix/Matrix.birch"
  decltype(auto) compute(const MiddleValue& m);
  #line 6 "src/form/unary_matrix/Matrix.birch"
  template<class Gradient,class MiddleValue>
  #line 6 "src/form/unary_matrix/Matrix.birch"
  decltype(auto) grad(const Gradient& d, const MiddleValue& m);
};

#line 1 "src/form/unary_matrix/Trace.birch"
#line 1 "src/form/unary_matrix/Trace.birch"
struct TraceFunction_ final {
public:
  #line 1 "src/form/unary_matrix/Trace.birch"
  LIBBIRCH_STRUCT(TraceFunction_)
  #line 1 "src/form/unary_matrix/Trace.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_matrix/Trace.birch"
  TraceFunction_();

  #line 2 "src/form/unary_matrix/Trace.birch"
  template<class Value>
  #line 2 "src/form/unary_matrix/Trace.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_matrix/Trace.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_matrix/Trace.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
  #line 10 "src/form/unary_matrix/Trace.birch"
  template<class Gradient,class Value>
  #line 10 "src/form/unary_matrix/Trace.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<Value,1>& m);
  #line 14 "src/form/unary_matrix/Trace.birch"
  template<class Gradient,class Value>
  #line 14 "src/form/unary_matrix/Trace.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<Value,2>& m);
};

#line 1 "src/form/unary_matrix/Transpose.birch"
#line 1 "src/form/unary_matrix/Transpose.birch"
struct TransposeFunction_ final {
public:
  #line 1 "src/form/unary_matrix/Transpose.birch"
  LIBBIRCH_STRUCT(TransposeFunction_)
  #line 1 "src/form/unary_matrix/Transpose.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_matrix/Transpose.birch"
  TransposeFunction_();

  #line 2 "src/form/unary_matrix/Transpose.birch"
  template<class Value>
  #line 2 "src/form/unary_matrix/Transpose.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_matrix/Transpose.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_matrix/Transpose.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 1 "src/form/unary_matrix/Vector.birch"
struct VectorFunction_ final {
public:
  #line 1 "src/form/unary_matrix/Vector.birch"
  using Value_ = Value;
  #line 1 "src/form/unary_matrix/Vector.birch"
  LIBBIRCH_STRUCT(VectorFunction_)
  #line 1 "src/form/unary_matrix/Vector.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_matrix/Vector.birch"
  VectorFunction_();

  #line 2 "src/form/unary_matrix/Vector.birch"
  template<class MiddleValue>
  #line 2 "src/form/unary_matrix/Vector.birch"
  decltype(auto) compute(const MiddleValue& m);
  #line 6 "src/form/unary_matrix/Vector.birch"
  template<class Gradient,class MiddleValue>
  #line 6 "src/form/unary_matrix/Vector.birch"
  decltype(auto) grad(const Gradient& d, const MiddleValue& m);
};

#line 1 "src/form/unary_other/Abs.birch"
#line 1 "src/form/unary_other/Abs.birch"
struct AbsFunction_ final {
public:
  #line 1 "src/form/unary_other/Abs.birch"
  LIBBIRCH_STRUCT(AbsFunction_)
  #line 1 "src/form/unary_other/Abs.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Abs.birch"
  AbsFunction_();

  #line 2 "src/form/unary_other/Abs.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Abs.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Abs.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Abs.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Acos.birch"
#line 1 "src/form/unary_other/Acos.birch"
struct AcosFunction_ final {
public:
  #line 1 "src/form/unary_other/Acos.birch"
  LIBBIRCH_STRUCT(AcosFunction_)
  #line 1 "src/form/unary_other/Acos.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Acos.birch"
  AcosFunction_();

  #line 2 "src/form/unary_other/Acos.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Acos.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Acos.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Acos.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Asin.birch"
#line 1 "src/form/unary_other/Asin.birch"
struct AsinFunction_ final {
public:
  #line 1 "src/form/unary_other/Asin.birch"
  LIBBIRCH_STRUCT(AsinFunction_)
  #line 1 "src/form/unary_other/Asin.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Asin.birch"
  AsinFunction_();

  #line 2 "src/form/unary_other/Asin.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Asin.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Asin.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Asin.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Atan.birch"
#line 1 "src/form/unary_other/Atan.birch"
struct AtanFunction_ final {
public:
  #line 1 "src/form/unary_other/Atan.birch"
  LIBBIRCH_STRUCT(AtanFunction_)
  #line 1 "src/form/unary_other/Atan.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Atan.birch"
  AtanFunction_();

  #line 2 "src/form/unary_other/Atan.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Atan.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Atan.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Atan.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Cos.birch"
#line 1 "src/form/unary_other/Cos.birch"
struct CosFunction_ final {
public:
  #line 1 "src/form/unary_other/Cos.birch"
  LIBBIRCH_STRUCT(CosFunction_)
  #line 1 "src/form/unary_other/Cos.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Cos.birch"
  CosFunction_();

  #line 2 "src/form/unary_other/Cos.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Cos.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Cos.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Cos.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Cosh.birch"
#line 1 "src/form/unary_other/Cosh.birch"
struct CoshFunction_ final {
public:
  #line 1 "src/form/unary_other/Cosh.birch"
  LIBBIRCH_STRUCT(CoshFunction_)
  #line 1 "src/form/unary_other/Cosh.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Cosh.birch"
  CoshFunction_();

  #line 2 "src/form/unary_other/Cosh.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Cosh.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Cosh.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Cosh.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Exp.birch"
#line 1 "src/form/unary_other/Exp.birch"
struct ExpFunction_ final {
public:
  #line 1 "src/form/unary_other/Exp.birch"
  LIBBIRCH_STRUCT(ExpFunction_)
  #line 1 "src/form/unary_other/Exp.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Exp.birch"
  ExpFunction_();

  #line 2 "src/form/unary_other/Exp.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Exp.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Exp.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Exp.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Expm1.birch"
#line 1 "src/form/unary_other/Expm1.birch"
struct Expm1Function_ final {
public:
  #line 1 "src/form/unary_other/Expm1.birch"
  LIBBIRCH_STRUCT(Expm1Function_)
  #line 1 "src/form/unary_other/Expm1.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Expm1.birch"
  Expm1Function_();

  #line 2 "src/form/unary_other/Expm1.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Expm1.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Expm1.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Expm1.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Log.birch"
#line 1 "src/form/unary_other/Log.birch"
struct LogFunction_ final {
public:
  #line 1 "src/form/unary_other/Log.birch"
  LIBBIRCH_STRUCT(LogFunction_)
  #line 1 "src/form/unary_other/Log.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Log.birch"
  LogFunction_();

  #line 2 "src/form/unary_other/Log.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Log.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Log.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Log.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Log1p.birch"
#line 1 "src/form/unary_other/Log1p.birch"
struct Log1pFunction_ final {
public:
  #line 1 "src/form/unary_other/Log1p.birch"
  LIBBIRCH_STRUCT(Log1pFunction_)
  #line 1 "src/form/unary_other/Log1p.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Log1p.birch"
  Log1pFunction_();

  #line 2 "src/form/unary_other/Log1p.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Log1p.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Log1p.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Log1p.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/LogGamma.birch"
#line 1 "src/form/unary_other/LogGamma.birch"
struct LogGammaFunction_ final {
public:
  #line 1 "src/form/unary_other/LogGamma.birch"
  LIBBIRCH_STRUCT(LogGammaFunction_)
  #line 1 "src/form/unary_other/LogGamma.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/LogGamma.birch"
  LogGammaFunction_();

  #line 2 "src/form/unary_other/LogGamma.birch"
  template<class Value>
  #line 2 "src/form/unary_other/LogGamma.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/LogGamma.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/LogGamma.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Negate.birch"
#line 1 "src/form/unary_other/Negate.birch"
struct NegateFunction_ final {
public:
  #line 1 "src/form/unary_other/Negate.birch"
  LIBBIRCH_STRUCT(NegateFunction_)
  #line 1 "src/form/unary_other/Negate.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Negate.birch"
  NegateFunction_();

  #line 2 "src/form/unary_other/Negate.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Negate.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Negate.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Negate.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Rectify.birch"
#line 1 "src/form/unary_other/Rectify.birch"
struct RectifyFunction_ final {
public:
  #line 1 "src/form/unary_other/Rectify.birch"
  LIBBIRCH_STRUCT(RectifyFunction_)
  #line 1 "src/form/unary_other/Rectify.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Rectify.birch"
  RectifyFunction_();

  #line 2 "src/form/unary_other/Rectify.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Rectify.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Rectify.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Rectify.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 1 "src/form/unary_other/Scalar.birch"
struct ScalarFunction_ final {
public:
  #line 1 "src/form/unary_other/Scalar.birch"
  using Value_ = Value;
  #line 1 "src/form/unary_other/Scalar.birch"
  LIBBIRCH_STRUCT(ScalarFunction_)
  #line 1 "src/form/unary_other/Scalar.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Scalar.birch"
  ScalarFunction_();

  #line 2 "src/form/unary_other/Scalar.birch"
  template<class MiddleValue>
  #line 2 "src/form/unary_other/Scalar.birch"
  decltype(auto) compute(const MiddleValue& m);
  #line 6 "src/form/unary_other/Scalar.birch"
  template<class Gradient,class MiddleValue>
  #line 6 "src/form/unary_other/Scalar.birch"
  decltype(auto) grad(const Gradient& d, const MiddleValue& m);
};

#line 1 "src/form/unary_other/Sin.birch"
#line 1 "src/form/unary_other/Sin.birch"
struct SinFunction_ final {
public:
  #line 1 "src/form/unary_other/Sin.birch"
  LIBBIRCH_STRUCT(SinFunction_)
  #line 1 "src/form/unary_other/Sin.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Sin.birch"
  SinFunction_();

  #line 2 "src/form/unary_other/Sin.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Sin.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Sin.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Sin.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Sinh.birch"
#line 1 "src/form/unary_other/Sinh.birch"
struct SinhFunction_ final {
public:
  #line 1 "src/form/unary_other/Sinh.birch"
  LIBBIRCH_STRUCT(SinhFunction_)
  #line 1 "src/form/unary_other/Sinh.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Sinh.birch"
  SinhFunction_();

  #line 2 "src/form/unary_other/Sinh.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Sinh.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Sinh.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Sinh.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Sqrt.birch"
#line 1 "src/form/unary_other/Sqrt.birch"
struct SqrtFunction_ final {
public:
  #line 1 "src/form/unary_other/Sqrt.birch"
  LIBBIRCH_STRUCT(SqrtFunction_)
  #line 1 "src/form/unary_other/Sqrt.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Sqrt.birch"
  SqrtFunction_();

  #line 2 "src/form/unary_other/Sqrt.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Sqrt.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Sqrt.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Sqrt.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Sum.birch"
#line 1 "src/form/unary_other/Sum.birch"
struct SumFunction_ final {
public:
  #line 1 "src/form/unary_other/Sum.birch"
  LIBBIRCH_STRUCT(SumFunction_)
  #line 1 "src/form/unary_other/Sum.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Sum.birch"
  SumFunction_();

  #line 2 "src/form/unary_other/Sum.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Sum.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Sum.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Sum.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
  #line 10 "src/form/unary_other/Sum.birch"
  template<class Gradient,class Value>
  #line 10 "src/form/unary_other/Sum.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<Value,1>& m);
  #line 14 "src/form/unary_other/Sum.birch"
  template<class Gradient,class Value>
  #line 14 "src/form/unary_other/Sum.birch"
  decltype(auto) grad(const Gradient& d, const libbirch::DefaultArray<Value,2>& m);
};

#line 1 "src/form/unary_other/Tan.birch"
#line 1 "src/form/unary_other/Tan.birch"
struct TanFunction_ final {
public:
  #line 1 "src/form/unary_other/Tan.birch"
  LIBBIRCH_STRUCT(TanFunction_)
  #line 1 "src/form/unary_other/Tan.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Tan.birch"
  TanFunction_();

  #line 2 "src/form/unary_other/Tan.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Tan.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Tan.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Tan.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 1 "src/form/unary_other/Tanh.birch"
#line 1 "src/form/unary_other/Tanh.birch"
struct TanhFunction_ final {
public:
  #line 1 "src/form/unary_other/Tanh.birch"
  LIBBIRCH_STRUCT(TanhFunction_)
  #line 1 "src/form/unary_other/Tanh.birch"
  LIBBIRCH_STRUCT_NO_MEMBERS()
  #line 1 "src/form/unary_other/Tanh.birch"
  TanhFunction_();

  #line 2 "src/form/unary_other/Tanh.birch"
  template<class Value>
  #line 2 "src/form/unary_other/Tanh.birch"
  decltype(auto) compute(const Value& m);
  #line 6 "src/form/unary_other/Tanh.birch"
  template<class Gradient,class Value>
  #line 6 "src/form/unary_other/Tanh.birch"
  decltype(auto) grad(const Gradient& d, const Value& m);
};

#line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
#line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
class DeepCloneNode_ : public Object_ {
public:
  #line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
  LIBBIRCH_CLASS(DeepCloneNode_, Object_)
  #line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
  LIBBIRCH_CLASS_MEMBERS(a)
  #line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
  using base_type_::operator=;
  
  #line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
  DeepCloneNode_();

  Integer a;
};

#line 23 "src/test/basic/test_basic_deep_clone_chain.birch"
extern "C" DeepCloneNode_* make_DeepCloneNode_();

#line 6 "src/container/Array.birch"
template<class Type>
#line 6 "src/container/Array.birch"
Array_<Type>::Array_() :
    #line 6 "src/container/Array.birch"
    base_type_(),
    #line 10 "src/container/Array.birch"
    values() {
  //
}

#line 15 "src/container/Array.birch"
template<class Type>
#line 15 "src/container/Array.birch"
Type& Array_<Type>::operator()(const Integer& i) {
  #line 16 "src/container/Array.birch"
  return values(i);
  #line 15 "src/container/Array.birch"
}

#line 22 "src/container/Array.birch"
template<class Type>
#line 22 "src/container/Array.birch"
#line 22 "src/container/Array.birch"
Integer Array_<Type>::size() {
  #line 23 "src/container/Array.birch"
  return length(values);
}

#line 29 "src/container/Array.birch"
template<class Type>
#line 29 "src/container/Array.birch"
#line 29 "src/container/Array.birch"
Boolean Array_<Type>::empty() {
  #line 30 "src/container/Array.birch"
  return size() == Integer(0);
}

#line 36 "src/container/Array.birch"
template<class Type>
#line 36 "src/container/Array.birch"
#line 36 "src/container/Array.birch"
void Array_<Type>::clear() {
  libbirch::DefaultArray<Type,1> values;
  #line 38 "src/container/Array.birch"
  this->values = values;
}

#line 46 "src/container/Array.birch"
template<class Type>
#line 46 "src/container/Array.birch"
#line 46 "src/container/Array.birch"
Type Array_<Type>::get(const Integer& i) {
  #line 47 "src/container/Array.birch"
  return values(i);
}

#line 56 "src/container/Array.birch"
template<class Type>
#line 56 "src/container/Array.birch"
#line 56 "src/container/Array.birch"
void Array_<Type>::set(const Integer& i, const Type& x) {
  #line 57 "src/container/Array.birch"
  values(i) = x;
}

#line 63 "src/container/Array.birch"
template<class Type>
#line 63 "src/container/Array.birch"
#line 63 "src/container/Array.birch"
Type Array_<Type>::front() {
  #line 64 "src/container/Array.birch"
  assert(size() > Integer(0));
  #line 65 "src/container/Array.birch"
  return values(Integer(1));
}

#line 71 "src/container/Array.birch"
template<class Type>
#line 71 "src/container/Array.birch"
#line 71 "src/container/Array.birch"
Type Array_<Type>::back() {
  #line 72 "src/container/Array.birch"
  assert(size() > Integer(0));
  #line 73 "src/container/Array.birch"
  return values(size());
}

#line 81 "src/container/Array.birch"
template<class Type>
#line 81 "src/container/Array.birch"
#line 81 "src/container/Array.birch"
void Array_<Type>::pushFront(const Type& x) {
  #line 82 "src/container/Array.birch"
  insert(Integer(1), x);
}

#line 88 "src/container/Array.birch"
template<class Type>
#line 88 "src/container/Array.birch"
#line 88 "src/container/Array.birch"
Type Array_<Type>::pushFront() {
  auto x = make<Type>();
  #line 90 "src/container/Array.birch"
  if (x.has_value()) {
    #line 91 "src/container/Array.birch"
    pushFront(x.value());
  } else {
    #line 93 "src/container/Array.birch"
    error(String("not default constructible"));
  }
  #line 95 "src/container/Array.birch"
  return x.value();
}

#line 103 "src/container/Array.birch"
template<class Type>
#line 103 "src/container/Array.birch"
#line 103 "src/container/Array.birch"
void Array_<Type>::pushBack(const Type& x) {
  #line 104 "src/container/Array.birch"
  insert(size() + Integer(1), x);
}

#line 110 "src/container/Array.birch"
template<class Type>
#line 110 "src/container/Array.birch"
#line 110 "src/container/Array.birch"
Type Array_<Type>::pushBack() {
  auto x = make<Type>();
  #line 112 "src/container/Array.birch"
  if (x.has_value()) {
    #line 113 "src/container/Array.birch"
    pushBack(x.value());
  } else {
    #line 115 "src/container/Array.birch"
    error(String("not default constructible"));
  }
  #line 117 "src/container/Array.birch"
  return x.value();
}

#line 123 "src/container/Array.birch"
template<class Type>
#line 123 "src/container/Array.birch"
#line 123 "src/container/Array.birch"
void Array_<Type>::popFront() {
  #line 124 "src/container/Array.birch"
  erase(Integer(1));
}

#line 130 "src/container/Array.birch"
template<class Type>
#line 130 "src/container/Array.birch"
#line 130 "src/container/Array.birch"
void Array_<Type>::popBack() {
  #line 131 "src/container/Array.birch"
  erase(size());
}

#line 144 "src/container/Array.birch"
template<class Type>
#line 144 "src/container/Array.birch"
#line 144 "src/container/Array.birch"
void Array_<Type>::insert(const Integer& i, const Type& x) {
  #line 145 "src/container/Array.birch"
  assert(Integer(1) <= i && i <= size() + Integer(1));
  #line 146 "src/container/Array.birch"

    this->values.insert(i - 1, x);
    }

#line 158 "src/container/Array.birch"
template<class Type>
#line 158 "src/container/Array.birch"
#line 158 "src/container/Array.birch"
void Array_<Type>::erase(const Integer& i) {
  #line 159 "src/container/Array.birch"
  assert(Integer(1) <= i && i <= size());
  #line 160 "src/container/Array.birch"

    this->values.erase(i - 1);
    }

#line 173 "src/container/Array.birch"
template<class Type>
#line 173 "src/container/Array.birch"
#line 173 "src/container/Array.birch"
void Array_<Type>::erase(const Integer& i, const Integer& n) {
  #line 174 "src/container/Array.birch"
  assert(Integer(1) <= i && i <= size());
  #line 175 "src/container/Array.birch"
  assert(Integer(1) <= n && n <= size() - i + Integer(1));
  #line 176 "src/container/Array.birch"

    this->values.erase(i - 1, n);
    }

#line 186 "src/container/Array.birch"
template<class Type>
#line 186 "src/container/Array.birch"
#line 186 "src/container/Array.birch"
Iterator<Type> Array_<Type>::walk() {
  #line 187 "src/container/Array.birch"
  return construct<ArrayIterator<Type>>(this->shared_from_this_());
}

#line 193 "src/container/Array.birch"
template<class Type>
#line 193 "src/container/Array.birch"
#line 193 "src/container/Array.birch"
libbirch::DefaultArray<Type,1> Array_<Type>::toArray() {
  #line 194 "src/container/Array.birch"
  return values;
}

#line 200 "src/container/Array.birch"
template<class Type>
#line 200 "src/container/Array.birch"
#line 200 "src/container/Array.birch"
void Array_<Type>::fromArray(const libbirch::DefaultArray<Type,1>& x) {
  #line 201 "src/container/Array.birch"
  values = x;
}

#line 204 "src/container/Array.birch"
template<class Type>
#line 204 "src/container/Array.birch"
#line 204 "src/container/Array.birch"
void Array_<Type>::read(const Buffer& buffer) {
  #line 205 "src/container/Array.birch"
  clear();
  #line 206 "src/container/Array.birch"
    libbirch::optional_assign(values, buffer->get<libbirch::DefaultArray<Type,1>>());
;
}

#line 209 "src/container/Array.birch"
template<class Type>
#line 209 "src/container/Array.birch"
#line 209 "src/container/Array.birch"
void Array_<Type>::write(const Buffer& buffer) {
  #line 210 "src/container/Array.birch"
  buffer->setEmptyArray();
  auto f = walk();
  #line 212 "src/container/Array.birch"
  while (f->hasNext()) {
    #line 213 "src/container/Array.birch"
    buffer->push(f->next());
  }
}

#line 30 "src/container/List.birch"
template<class Type>
#line 30 "src/container/List.birch"
List_<Type>::List_() :
    #line 30 "src/container/List.birch"
    base_type_(),
    #line 31 "src/container/List.birch"
    head(),
    #line 32 "src/container/List.birch"
    tail(),
    #line 33 "src/container/List.birch"
    count(Integer(0)) {
  //
}

#line 38 "src/container/List.birch"
template<class Type>
#line 38 "src/container/List.birch"
Type& List_<Type>::operator()(const Integer& i) {
  #line 39 "src/container/List.birch"
  return getNode(i)->x;
  #line 38 "src/container/List.birch"
}

#line 45 "src/container/List.birch"
template<class Type>
#line 45 "src/container/List.birch"
#line 45 "src/container/List.birch"
Integer List_<Type>::size() {
  #line 46 "src/container/List.birch"
  return count;
}

#line 52 "src/container/List.birch"
template<class Type>
#line 52 "src/container/List.birch"
#line 52 "src/container/List.birch"
Boolean List_<Type>::empty() {
  #line 53 "src/container/List.birch"
  return count == Integer(0);
}

#line 59 "src/container/List.birch"
template<class Type>
#line 59 "src/container/List.birch"
#line 59 "src/container/List.birch"
void List_<Type>::clear() {
  #line 60 "src/container/List.birch"
  head = std::nullopt;
  #line 61 "src/container/List.birch"
  tail = std::nullopt;
  #line 62 "src/container/List.birch"
  count = Integer(0);
}

#line 68 "src/container/List.birch"
template<class Type>
#line 68 "src/container/List.birch"
#line 68 "src/container/List.birch"
Type List_<Type>::front() {
  #line 69 "src/container/List.birch"
  assert(head.has_value());
  #line 70 "src/container/List.birch"
  return head.value()->x;
}

#line 76 "src/container/List.birch"
template<class Type>
#line 76 "src/container/List.birch"
#line 76 "src/container/List.birch"
Type List_<Type>::back() {
  #line 77 "src/container/List.birch"
  assert(this->tail.has_value());
  auto tail = this->tail.value();
  #line 79 "src/container/List.birch"
  return tail->x;
}

#line 87 "src/container/List.birch"
template<class Type>
#line 87 "src/container/List.birch"
#line 87 "src/container/List.birch"
Type List_<Type>::get(const Integer& i) {
  #line 88 "src/container/List.birch"
  return getNode(i)->x;
}

#line 97 "src/container/List.birch"
template<class Type>
#line 97 "src/container/List.birch"
#line 97 "src/container/List.birch"
void List_<Type>::set(const Integer& i, const Type& x) {
  #line 98 "src/container/List.birch"
  getNode(i)->x = x;
}

#line 106 "src/container/List.birch"
template<class Type>
#line 106 "src/container/List.birch"
#line 106 "src/container/List.birch"
void List_<Type>::pushFront(const Type& x) {
  ListNode<Type> node(std::in_place, x);
  #line 108 "src/container/List.birch"
  if (empty()) {
    #line 109 "src/container/List.birch"
    tail = node;
  } else {
    #line 111 "src/container/List.birch"
    assert(head.has_value());
    #line 112 "src/container/List.birch"
    head.value()->prev = node;
    #line 113 "src/container/List.birch"
    node->next = head;
  }
  #line 115 "src/container/List.birch"
  head = node;
  #line 116 "src/container/List.birch"
  count = count + Integer(1);
  #line 118 "src/container/List.birch"
  assert(this->head.has_value() && this->tail.has_value());
}

#line 124 "src/container/List.birch"
template<class Type>
#line 124 "src/container/List.birch"
#line 124 "src/container/List.birch"
Type List_<Type>::pushFront() {
  auto x = make<Type>();
  #line 126 "src/container/List.birch"
  if (x.has_value()) {
    #line 127 "src/container/List.birch"
    pushFront(x.value());
  } else {
    #line 129 "src/container/List.birch"
    error(String("not default constructible"));
  }
  #line 131 "src/container/List.birch"
  return x.value();
}

#line 139 "src/container/List.birch"
template<class Type>
#line 139 "src/container/List.birch"
#line 139 "src/container/List.birch"
void List_<Type>::pushBack(const Type& x) {
  ListNode<Type> node(std::in_place, x);
  #line 141 "src/container/List.birch"
  if (empty()) {
    #line 142 "src/container/List.birch"
    head = node;
  } else {
    #line 144 "src/container/List.birch"
    assert(this->tail.has_value());
    auto tail = this->tail.value();
    #line 146 "src/container/List.birch"
    tail->next = node;
    #line 147 "src/container/List.birch"
    node->prev = tail;
  }
  #line 149 "src/container/List.birch"
  this->tail = node;
  #line 150 "src/container/List.birch"
  count = count + Integer(1);
  #line 152 "src/container/List.birch"
  assert(this->head.has_value() && this->tail.has_value());
}

#line 158 "src/container/List.birch"
template<class Type>
#line 158 "src/container/List.birch"
#line 158 "src/container/List.birch"
Type List_<Type>::pushBack() {
  auto x = make<Type>();
  #line 160 "src/container/List.birch"
  if (x.has_value()) {
    #line 161 "src/container/List.birch"
    pushBack(x.value());
  } else {
    #line 163 "src/container/List.birch"
    error(String("not default constructible"));
  }
  #line 165 "src/container/List.birch"
  return x.value();
}

#line 171 "src/container/List.birch"
template<class Type>
#line 171 "src/container/List.birch"
#line 171 "src/container/List.birch"
void List_<Type>::popFront() {
  #line 172 "src/container/List.birch"
  assert(!(empty()));
  #line 173 "src/container/List.birch"
  assert(head.has_value());
  #line 174 "src/container/List.birch"
  head = head.value()->popFront();
  #line 175 "src/container/List.birch"
  count = count - Integer(1);
  #line 176 "src/container/List.birch"
  if (count <= Integer(1)) {
    #line 177 "src/container/List.birch"
    tail = head;
  }
  #line 180 "src/container/List.birch"
  assert(count == Integer(0) || (head.has_value() && tail.has_value()));
  #line 181 "src/container/List.birch"
  assert(count > Integer(0) || (!(head.has_value()) && !(tail.has_value())));
}

#line 187 "src/container/List.birch"
template<class Type>
#line 187 "src/container/List.birch"
#line 187 "src/container/List.birch"
void List_<Type>::popBack() {
  #line 188 "src/container/List.birch"
  assert(!(empty()));
  #line 189 "src/container/List.birch"
  assert(tail.has_value());
  #line 190 "src/container/List.birch"
  tail = tail.value()->popBack();
  #line 191 "src/container/List.birch"
  count = count - Integer(1);
  #line 192 "src/container/List.birch"
  if (count <= Integer(1)) {
    #line 193 "src/container/List.birch"
    head = tail;
  }
  #line 196 "src/container/List.birch"
  assert(count == Integer(0) || (head.has_value() && tail.has_value()));
  #line 197 "src/container/List.birch"
  assert(count > Integer(0) || (!(head.has_value()) && !(tail.has_value())));
}

#line 210 "src/container/List.birch"
template<class Type>
#line 210 "src/container/List.birch"
#line 210 "src/container/List.birch"
void List_<Type>::insert(const Integer& i, const Type& x) {
  #line 211 "src/container/List.birch"
  assert(Integer(1) <= i && i <= count + Integer(1));
  #line 213 "src/container/List.birch"
  if (i == Integer(1)) {
    #line 214 "src/container/List.birch"
    pushFront(x);
  } else {
    #line 215 "src/container/List.birch"
    if (i == count + Integer(1)) {
      #line 216 "src/container/List.birch"
      pushBack(x);
    } else {
      ListNode<Type> node(std::in_place, x);
      #line 219 "src/container/List.birch"
      getNode(i)->insert(node);
      #line 220 "src/container/List.birch"
      count = count + Integer(1);
    }
  }
  #line 223 "src/container/List.birch"
  assert(this->head.has_value() && this->tail.has_value());
}

#line 231 "src/container/List.birch"
template<class Type>
#line 231 "src/container/List.birch"
#line 231 "src/container/List.birch"
void List_<Type>::erase(const Integer& i) {
  #line 232 "src/container/List.birch"
  assert(Integer(1) <= i && i <= count);
  #line 234 "src/container/List.birch"
  if (i == Integer(1)) {
    #line 235 "src/container/List.birch"
    popFront();
  } else {
    #line 236 "src/container/List.birch"
    if (i == count) {
      #line 237 "src/container/List.birch"
      popBack();
    } else {
      #line 239 "src/container/List.birch"
      getNode(i)->erase();
      #line 240 "src/container/List.birch"
      count = count - Integer(1);
    }
  }
  #line 243 "src/container/List.birch"
  assert(this->count == Integer(0) || (this->head.has_value() && this->tail.has_value()));
  #line 244 "src/container/List.birch"
  assert(this->count > Integer(0) || (!(this->head.has_value()) && !(this->tail.has_value())));
}

#line 252 "src/container/List.birch"
template<class Type>
#line 252 "src/container/List.birch"
#line 252 "src/container/List.birch"
Iterator<Type> List_<Type>::walk() {
  #line 253 "src/container/List.birch"
  return construct<ListIterator<Type>>(head);
}

#line 261 "src/container/List.birch"
template<class Type>
#line 261 "src/container/List.birch"
#line 261 "src/container/List.birch"
ListNode<Type> List_<Type>::getNode(const Integer& i) {
  #line 262 "src/container/List.birch"
  assert(Integer(1) <= i && i <= count);
  std::optional<ListNode<Type>> node;
  #line 265 "src/container/List.birch"
  if (Integer(2) * i <= count) {
    #line 267 "src/container/List.birch"
    node = head;
    #line 268 "src/container/List.birch"
    for (auto j = Integer(1); j <= i - Integer(1); ++j) {
      #line 269 "src/container/List.birch"
      assert(node.has_value());
      #line 270 "src/container/List.birch"
      node = node.value()->next;
    }
  } else {
    #line 274 "src/container/List.birch"
    node = tail;
    #line 275 "src/container/List.birch"
    for (auto j = Integer(1); j <= count - i; ++j) {
      #line 276 "src/container/List.birch"
      assert(node.has_value());
      #line 277 "src/container/List.birch"
      node = node.value()->prev;
    }
  }
  #line 281 "src/container/List.birch"
  assert(node.has_value());
  #line 282 "src/container/List.birch"
  return node.value();
}

#line 285 "src/container/List.birch"
template<class Type>
#line 285 "src/container/List.birch"
#line 285 "src/container/List.birch"
void List_<Type>::read(const Buffer& buffer) {
  #line 286 "src/container/List.birch"
  clear();
  auto f = buffer->walk();
  #line 288 "src/container/List.birch"
  while (f->hasNext()) {
    auto x = f->next()->get<Type>();
    #line 290 "src/container/List.birch"
    if (x.has_value()) {
      #line 291 "src/container/List.birch"
      pushBack(x.value());
    }
  }
}

#line 296 "src/container/List.birch"
template<class Type>
#line 296 "src/container/List.birch"
#line 296 "src/container/List.birch"
void List_<Type>::write(const Buffer& buffer) {
  #line 297 "src/container/List.birch"
  buffer->setEmptyArray();
  auto f = walk();
  #line 299 "src/container/List.birch"
  while (f->hasNext()) {
    #line 300 "src/container/List.birch"
    buffer->push(f->next());
  }
}

#line 4 "src/container/ListNode.birch"
template<class Type>
#line 4 "src/container/ListNode.birch"
ListNode_<Type>::ListNode_(const Type& x) :
    #line 4 "src/container/ListNode.birch"
    base_type_(),
    #line 5 "src/container/ListNode.birch"
    prev(),
    #line 6 "src/container/ListNode.birch"
    next(),
    #line 7 "src/container/ListNode.birch"
    x(x) {
  //
}

#line 14 "src/container/ListNode.birch"
template<class Type>
#line 14 "src/container/ListNode.birch"
#line 14 "src/container/ListNode.birch"
void ListNode_<Type>::insert(const ListNode<Type>& node) {
  #line 15 "src/container/ListNode.birch"
  assert(prev.has_value());
  #line 16 "src/container/ListNode.birch"
  node->prev = prev;
  #line 17 "src/container/ListNode.birch"
  node->next = this->shared_from_this_();
  #line 18 "src/container/ListNode.birch"
  prev.value()->next = node;
  #line 19 "src/container/ListNode.birch"
  this->prev = node;
}

#line 25 "src/container/ListNode.birch"
template<class Type>
#line 25 "src/container/ListNode.birch"
#line 25 "src/container/ListNode.birch"
void ListNode_<Type>::erase() {
  #line 26 "src/container/ListNode.birch"
  assert(prev.has_value());
  #line 27 "src/container/ListNode.birch"
  assert(next.has_value());
  #line 28 "src/container/ListNode.birch"
  next.value()->prev = prev;
  #line 29 "src/container/ListNode.birch"
  prev.value()->next = next;
  #line 30 "src/container/ListNode.birch"
  this->next = std::nullopt;
  #line 31 "src/container/ListNode.birch"
  this->prev = std::nullopt;
}

#line 39 "src/container/ListNode.birch"
template<class Type>
#line 39 "src/container/ListNode.birch"
#line 39 "src/container/ListNode.birch"
std::optional<ListNode<Type>> ListNode_<Type>::popFront() {
  auto node = this->next;
  #line 41 "src/container/ListNode.birch"
  if (node.has_value()) {
    #line 42 "src/container/ListNode.birch"
    node.value()->prev = std::nullopt;
  }
  #line 44 "src/container/ListNode.birch"
  this->next = std::nullopt;
  #line 45 "src/container/ListNode.birch"
  this->prev = std::nullopt;
  #line 46 "src/container/ListNode.birch"
  return node;
}

#line 54 "src/container/ListNode.birch"
template<class Type>
#line 54 "src/container/ListNode.birch"
#line 54 "src/container/ListNode.birch"
std::optional<ListNode<Type>> ListNode_<Type>::popBack() {
  auto node = this->prev;
  #line 56 "src/container/ListNode.birch"
  if (node.has_value()) {
    #line 57 "src/container/ListNode.birch"
    node.value()->next = std::nullopt;
  }
  #line 59 "src/container/ListNode.birch"
  this->next = std::nullopt;
  #line 60 "src/container/ListNode.birch"
  this->prev = std::nullopt;
  #line 61 "src/container/ListNode.birch"
  return node;
}

#line 7 "src/container/RaggedArray.birch"
template<class Type>
#line 7 "src/container/RaggedArray.birch"
RaggedArray_<Type>::RaggedArray_() :
    #line 7 "src/container/RaggedArray.birch"
    base_type_(),
    #line 11 "src/container/RaggedArray.birch"
    values(),
    #line 16 "src/container/RaggedArray.birch"
    offsets(),
    #line 21 "src/container/RaggedArray.birch"
    sizes() {
  //
}

#line 29 "src/container/RaggedArray.birch"
template<class Type>
#line 29 "src/container/RaggedArray.birch"
Type& RaggedArray_<Type>::operator()(const Integer& i, const Integer& j) {
  #line 30 "src/container/RaggedArray.birch"
  return values(serial(i, j));
  #line 29 "src/container/RaggedArray.birch"
}

#line 38 "src/container/RaggedArray.birch"
template<class Type>
#line 38 "src/container/RaggedArray.birch"
#line 38 "src/container/RaggedArray.birch"
Boolean RaggedArray_<Type>::empty() {
  #line 39 "src/container/RaggedArray.birch"
  return length(offsets) == Integer(0);
}

#line 45 "src/container/RaggedArray.birch"
template<class Type>
#line 45 "src/container/RaggedArray.birch"
#line 45 "src/container/RaggedArray.birch"
void RaggedArray_<Type>::clear() {
  libbirch::DefaultArray<Type,1> values;
  libbirch::DefaultArray<Integer,1> offsets;
  libbirch::DefaultArray<Integer,1> sizes;
  #line 50 "src/container/RaggedArray.birch"
  this->values = values;
  #line 51 "src/container/RaggedArray.birch"
  this->offsets = offsets;
  #line 52 "src/container/RaggedArray.birch"
  this->sizes = sizes;
}

#line 58 "src/container/RaggedArray.birch"
template<class Type>
#line 58 "src/container/RaggedArray.birch"
#line 58 "src/container/RaggedArray.birch"
Integer RaggedArray_<Type>::count() {
  #line 59 "src/container/RaggedArray.birch"
  return length(values);
}

#line 65 "src/container/RaggedArray.birch"
template<class Type>
#line 65 "src/container/RaggedArray.birch"
#line 65 "src/container/RaggedArray.birch"
Integer RaggedArray_<Type>::size() {
  #line 66 "src/container/RaggedArray.birch"
  return length(offsets);
}

#line 74 "src/container/RaggedArray.birch"
template<class Type>
#line 74 "src/container/RaggedArray.birch"
#line 74 "src/container/RaggedArray.birch"
Integer RaggedArray_<Type>::size(const Integer& i) {
  #line 75 "src/container/RaggedArray.birch"
  return sizes(i);
}

#line 84 "src/container/RaggedArray.birch"
template<class Type>
#line 84 "src/container/RaggedArray.birch"
#line 84 "src/container/RaggedArray.birch"
Type RaggedArray_<Type>::get(const Integer& i, const Integer& j) {
  #line 85 "src/container/RaggedArray.birch"
  return values(serial(i, j));
}

#line 93 "src/container/RaggedArray.birch"
template<class Type>
#line 93 "src/container/RaggedArray.birch"
#line 93 "src/container/RaggedArray.birch"
libbirch::DefaultArray<Type,1> RaggedArray_<Type>::get(const Integer& i) {
  #line 94 "src/container/RaggedArray.birch"
  return values(libbirch::make_range(from(i), to(i)));
}

#line 104 "src/container/RaggedArray.birch"
template<class Type>
#line 104 "src/container/RaggedArray.birch"
#line 104 "src/container/RaggedArray.birch"
void RaggedArray_<Type>::set(const Integer& i, const Integer& j, const Type& x) {
  #line 105 "src/container/RaggedArray.birch"
  values(serial(i, j)) = x;
}

#line 117 "src/container/RaggedArray.birch"
template<class Type>
#line 117 "src/container/RaggedArray.birch"
#line 117 "src/container/RaggedArray.birch"
void RaggedArray_<Type>::set(const Integer& i, const libbirch::DefaultArray<Type,1>& x) {
  #line 118 "src/container/RaggedArray.birch"
  values(libbirch::make_range(from(i), to(i))) = x;
}

#line 124 "src/container/RaggedArray.birch"
template<class Type>
#line 124 "src/container/RaggedArray.birch"
#line 124 "src/container/RaggedArray.birch"
void RaggedArray_<Type>::popFront() {
  #line 125 "src/container/RaggedArray.birch"
  assert(size() > Integer(0));
  #line 126 "src/container/RaggedArray.birch"
  if (size() == Integer(1)) {
    #line 127 "src/container/RaggedArray.birch"
    clear();
  } else {
    auto ncols = sizes(Integer(1));
    #line 130 "src/container/RaggedArray.birch"

      this->offsets.erase(0);
      this->sizes.erase(0);
          #line 134 "src/container/RaggedArray.birch"
    if (ncols > Integer(0)) {
      #line 135 "src/container/RaggedArray.birch"

        this->values.erase(0, ncols);
              #line 138 "src/container/RaggedArray.birch"
      for (auto k = Integer(1); k <= length(offsets); ++k) {
        #line 139 "src/container/RaggedArray.birch"
        offsets(k) = offsets(k) - ncols;
      }
    }
  }
}

#line 150 "src/container/RaggedArray.birch"
template<class Type>
#line 150 "src/container/RaggedArray.birch"
#line 150 "src/container/RaggedArray.birch"
void RaggedArray_<Type>::popFront(const Integer& i) {
  #line 151 "src/container/RaggedArray.birch"
  assert(size(i) > Integer(0));
  auto j = offsets(i);
  #line 153 "src/container/RaggedArray.birch"

    this->values.erase(j - 1);
      #line 156 "src/container/RaggedArray.birch"
  for (auto k = (i + Integer(1)); k <= length(offsets); ++k) {
    #line 157 "src/container/RaggedArray.birch"
    offsets(k) = offsets(k) - Integer(1);
  }
  #line 159 "src/container/RaggedArray.birch"
  sizes(i) = sizes(i) - Integer(1);
}

#line 165 "src/container/RaggedArray.birch"
template<class Type>
#line 165 "src/container/RaggedArray.birch"
#line 165 "src/container/RaggedArray.birch"
void RaggedArray_<Type>::pushBack() {
  auto nvalues = length(values);
  #line 167 "src/container/RaggedArray.birch"
    
    this->offsets.push(nvalues + 1);
    this->sizes.push(0);
    }

#line 179 "src/container/RaggedArray.birch"
template<class Type>
#line 179 "src/container/RaggedArray.birch"
#line 179 "src/container/RaggedArray.birch"
void RaggedArray_<Type>::pushBack(const Integer& i, const Type& x) {
  auto j = offsets(i) + sizes(i);
  #line 181 "src/container/RaggedArray.birch"

    this->values.insert(j - 1, x);
      #line 184 "src/container/RaggedArray.birch"
  for (auto k = (i + Integer(1)); k <= size(); ++k) {
    #line 185 "src/container/RaggedArray.birch"
    offsets(k) = offsets(k) + Integer(1);
  }
  #line 187 "src/container/RaggedArray.birch"
  sizes(i) = sizes(i) + Integer(1);
}

#line 194 "src/container/RaggedArray.birch"
template<class Type>
#line 194 "src/container/RaggedArray.birch"
#line 194 "src/container/RaggedArray.birch"
Type RaggedArray_<Type>::pushBack(const Integer& i) {
  auto x = make<Type>();
  #line 196 "src/container/RaggedArray.birch"
  if (x.has_value()) {
    #line 197 "src/container/RaggedArray.birch"
    pushBack(i, x.value());
  } else {
    #line 199 "src/container/RaggedArray.birch"
    error(String("not default constructible"));
  }
  #line 201 "src/container/RaggedArray.birch"
  return x.value();
}

#line 209 "src/container/RaggedArray.birch"
template<class Type>
#line 209 "src/container/RaggedArray.birch"
#line 209 "src/container/RaggedArray.birch"
Iterator<libbirch::DefaultArray<Type,1>> RaggedArray_<Type>::walk() {
  #line 210 "src/container/RaggedArray.birch"
  return construct<RaggedArrayRowIterator<Type>>(this->shared_from_this_());
}

#line 221 "src/container/RaggedArray.birch"
template<class Type>
#line 221 "src/container/RaggedArray.birch"
#line 221 "src/container/RaggedArray.birch"
Iterator<Type> RaggedArray_<Type>::walk(const Integer& i) {
  #line 222 "src/container/RaggedArray.birch"
  return construct<RaggedArrayElementIterator<Type>>(this->shared_from_this_(), i);
}

#line 230 "src/container/RaggedArray.birch"
template<class Type>
#line 230 "src/container/RaggedArray.birch"
#line 230 "src/container/RaggedArray.birch"
Integer RaggedArray_<Type>::from(const Integer& i) {
  #line 231 "src/container/RaggedArray.birch"
  assert(Integer(1) <= i && i <= size());
  #line 232 "src/container/RaggedArray.birch"
  assert(offsets(i) != Integer(0));
  #line 233 "src/container/RaggedArray.birch"
  return offsets(i);
}

#line 241 "src/container/RaggedArray.birch"
template<class Type>
#line 241 "src/container/RaggedArray.birch"
#line 241 "src/container/RaggedArray.birch"
Integer RaggedArray_<Type>::to(const Integer& i) {
  #line 242 "src/container/RaggedArray.birch"
  assert(Integer(1) <= i && i <= size());
  #line 243 "src/container/RaggedArray.birch"
  assert(offsets(i) != Integer(0));
  #line 244 "src/container/RaggedArray.birch"
  return offsets(i) + sizes(i) - Integer(1);
}

#line 253 "src/container/RaggedArray.birch"
template<class Type>
#line 253 "src/container/RaggedArray.birch"
#line 253 "src/container/RaggedArray.birch"
Integer RaggedArray_<Type>::serial(const Integer& i, const Integer& j) {
  #line 254 "src/container/RaggedArray.birch"
  assert(Integer(1) <= i && i <= size());
  #line 255 "src/container/RaggedArray.birch"
  assert(Integer(1) <= j && j <= sizes(i));
  #line 256 "src/container/RaggedArray.birch"
  return from(i) + j - Integer(1);
}

#line 259 "src/container/RaggedArray.birch"
template<class Type>
#line 259 "src/container/RaggedArray.birch"
#line 259 "src/container/RaggedArray.birch"
void RaggedArray_<Type>::read(const Buffer& buffer) {
  #line 260 "src/container/RaggedArray.birch"
  clear();
  auto row = buffer->walk();
  #line 262 "src/container/RaggedArray.birch"
  while (row->hasNext()) {
    #line 263 "src/container/RaggedArray.birch"
    pushBack();
    auto col = row->next()->walk();
    #line 265 "src/container/RaggedArray.birch"
    while (col->hasNext()) {
      auto x = col->next()->get<Type>();
      #line 267 "src/container/RaggedArray.birch"
      if (x.has_value()) {
        #line 268 "src/container/RaggedArray.birch"
        pushBack(size(), x.value());
      }
    }
  }
}

#line 274 "src/container/RaggedArray.birch"
template<class Type>
#line 274 "src/container/RaggedArray.birch"
#line 274 "src/container/RaggedArray.birch"
void RaggedArray_<Type>::write(const Buffer& buffer) {
  #line 275 "src/container/RaggedArray.birch"
  buffer->setEmptyArray();
  auto f = walk();
  #line 277 "src/container/RaggedArray.birch"
  while (f->hasNext()) {
    #line 278 "src/container/RaggedArray.birch"
    buffer->push(f->next());
  }
}

#line 28 "src/container/Stack.birch"
template<class Type>
#line 28 "src/container/Stack.birch"
Stack_<Type>::Stack_() :
    #line 28 "src/container/Stack.birch"
    base_type_(),
    #line 29 "src/container/Stack.birch"
    head(),
    #line 30 "src/container/Stack.birch"
    count(Integer(0)) {
  //
}

#line 35 "src/container/Stack.birch"
template<class Type>
#line 35 "src/container/Stack.birch"
#line 35 "src/container/Stack.birch"
Integer Stack_<Type>::size() {
  #line 36 "src/container/Stack.birch"
  return count;
}

#line 42 "src/container/Stack.birch"
template<class Type>
#line 42 "src/container/Stack.birch"
#line 42 "src/container/Stack.birch"
Boolean Stack_<Type>::empty() {
  #line 43 "src/container/Stack.birch"
  return count == Integer(0);
}

#line 49 "src/container/Stack.birch"
template<class Type>
#line 49 "src/container/Stack.birch"
#line 49 "src/container/Stack.birch"
void Stack_<Type>::clear() {
  #line 50 "src/container/Stack.birch"
  head = std::nullopt;
  #line 51 "src/container/Stack.birch"
  count = Integer(0);
}

#line 57 "src/container/Stack.birch"
template<class Type>
#line 57 "src/container/Stack.birch"
#line 57 "src/container/Stack.birch"
Type Stack_<Type>::top() {
  #line 58 "src/container/Stack.birch"
  assert(head.has_value());
  #line 59 "src/container/Stack.birch"
  return head.value()->x;
}

#line 67 "src/container/Stack.birch"
template<class Type>
#line 67 "src/container/Stack.birch"
#line 67 "src/container/Stack.birch"
void Stack_<Type>::push(const Type& x) {
  StackNode<Type> node(std::in_place, x);
  #line 69 "src/container/Stack.birch"
  node->next = head;
  #line 70 "src/container/Stack.birch"
  head = node;
  #line 71 "src/container/Stack.birch"
  count = count + Integer(1);
}

#line 77 "src/container/Stack.birch"
template<class Type>
#line 77 "src/container/Stack.birch"
#line 77 "src/container/Stack.birch"
void Stack_<Type>::push() {
  auto x = make<Type>();
  #line 79 "src/container/Stack.birch"
  if (x.has_value()) {
    #line 80 "src/container/Stack.birch"
    push(x.value());
  } else {
    #line 82 "src/container/Stack.birch"
    error(String("not default constructible"));
  }
  #line 84 "src/container/Stack.birch"
  return x.value();
}

#line 90 "src/container/Stack.birch"
template<class Type>
#line 90 "src/container/Stack.birch"
#line 90 "src/container/Stack.birch"
void Stack_<Type>::pop() {
  #line 91 "src/container/Stack.birch"
  assert(head.has_value());
  #line 92 "src/container/Stack.birch"
  head = head.value()->next;
  #line 93 "src/container/Stack.birch"
  count = count - Integer(1);
}

#line 101 "src/container/Stack.birch"
template<class Type>
#line 101 "src/container/Stack.birch"
#line 101 "src/container/Stack.birch"
Iterator<Type> Stack_<Type>::walk() {
  #line 102 "src/container/Stack.birch"
  return construct<StackIterator<Type>>(head);
}

#line 1 "src/container/StackNode.birch"
template<class Type>
#line 1 "src/container/StackNode.birch"
StackNode_<Type>::StackNode_(const Type& x) :
    #line 1 "src/container/StackNode.birch"
    base_type_(),
    #line 5 "src/container/StackNode.birch"
    next(),
    #line 6 "src/container/StackNode.birch"
    x(x) {
  //
}

#line 40 "src/container/Tape.birch"
template<class Type>
#line 40 "src/container/Tape.birch"
Tape_<Type>::Tape_() :
    #line 40 "src/container/Tape.birch"
    base_type_(),
    #line 44 "src/container/Tape.birch"
    ahead(),
    #line 49 "src/container/Tape.birch"
    behind(),
    #line 54 "src/container/Tape.birch"
    aheadCount(Integer(0)),
    #line 60 "src/container/Tape.birch"
    behindCount(Integer(0)) {
  //
}

#line 65 "src/container/Tape.birch"
template<class Type>
#line 65 "src/container/Tape.birch"
Type& Tape_<Type>::operator()(const Integer& i) {
  #line 66 "src/container/Tape.birch"
  assert(Integer(0) < i);
  #line 67 "src/container/Tape.birch"
  seek(i);
  #line 68 "src/container/Tape.birch"
  return ahead.value()->x;
  #line 65 "src/container/Tape.birch"
}

#line 74 "src/container/Tape.birch"
template<class Type>
#line 74 "src/container/Tape.birch"
#line 74 "src/container/Tape.birch"
Integer Tape_<Type>::size() {
  #line 75 "src/container/Tape.birch"
  return aheadCount + behindCount;
}

#line 81 "src/container/Tape.birch"
template<class Type>
#line 81 "src/container/Tape.birch"
#line 81 "src/container/Tape.birch"
Boolean Tape_<Type>::empty() {
  #line 82 "src/container/Tape.birch"
  return aheadCount == Integer(0) && behindCount == Integer(0);
}

#line 88 "src/container/Tape.birch"
template<class Type>
#line 88 "src/container/Tape.birch"
#line 88 "src/container/Tape.birch"
void Tape_<Type>::clear() {
  #line 89 "src/container/Tape.birch"
  ahead = std::nullopt;
  #line 90 "src/container/Tape.birch"
  behind = std::nullopt;
  #line 91 "src/container/Tape.birch"
  aheadCount = Integer(0);
  #line 92 "src/container/Tape.birch"
  behindCount = Integer(0);
}

#line 98 "src/container/Tape.birch"
template<class Type>
#line 98 "src/container/Tape.birch"
#line 98 "src/container/Tape.birch"
Type Tape_<Type>::front() {
  #line 99 "src/container/Tape.birch"
  if (empty()) {
    #line 100 "src/container/Tape.birch"
    pushFront();
  }
  #line 102 "src/container/Tape.birch"
  rewind();
  #line 103 "src/container/Tape.birch"
  return ahead.value()->x;
}

#line 109 "src/container/Tape.birch"
template<class Type>
#line 109 "src/container/Tape.birch"
#line 109 "src/container/Tape.birch"
Type Tape_<Type>::back() {
  #line 110 "src/container/Tape.birch"
  if (empty()) {
    #line 111 "src/container/Tape.birch"
    pushBack();
  }
  #line 113 "src/container/Tape.birch"
  fastForward();
  #line 114 "src/container/Tape.birch"
  return behind.value()->x;
}

#line 122 "src/container/Tape.birch"
template<class Type>
#line 122 "src/container/Tape.birch"
#line 122 "src/container/Tape.birch"
Type Tape_<Type>::get(const Integer& i) {
  #line 123 "src/container/Tape.birch"
  assert(Integer(0) < i);
  #line 124 "src/container/Tape.birch"
  seek(i);
  #line 125 "src/container/Tape.birch"
  return ahead.value()->x;
}

#line 134 "src/container/Tape.birch"
template<class Type>
#line 134 "src/container/Tape.birch"
#line 134 "src/container/Tape.birch"
void Tape_<Type>::set(const Integer& i, const Type& x) {
  #line 135 "src/container/Tape.birch"
  assert(Integer(0) < i);
  #line 136 "src/container/Tape.birch"
  seek(i);
  #line 137 "src/container/Tape.birch"
  ahead.value()->x = x;
}

#line 145 "src/container/Tape.birch"
template<class Type>
#line 145 "src/container/Tape.birch"
#line 145 "src/container/Tape.birch"
void Tape_<Type>::pushFront(const Type& x) {
  #line 146 "src/container/Tape.birch"
  rewind();
  #line 147 "src/container/Tape.birch"
  behind = construct<TapeNode<Type>>(x);
  #line 148 "src/container/Tape.birch"
  behindCount = behindCount + Integer(1);
}

#line 154 "src/container/Tape.birch"
template<class Type>
#line 154 "src/container/Tape.birch"
#line 154 "src/container/Tape.birch"
Type Tape_<Type>::pushFront() {
  #line 155 "src/container/Tape.birch"
  rewind();
  #line 156 "src/container/Tape.birch"
  behind = construct<TapeNode<Type>>(construct<Type>());
  #line 157 "src/container/Tape.birch"
  behindCount = behindCount + Integer(1);
  #line 158 "src/container/Tape.birch"
  return behind.value()->x;
}

#line 166 "src/container/Tape.birch"
template<class Type>
#line 166 "src/container/Tape.birch"
#line 166 "src/container/Tape.birch"
void Tape_<Type>::pushBack(const Type& x) {
  #line 167 "src/container/Tape.birch"
  fastForward();
  #line 168 "src/container/Tape.birch"
  ahead = construct<TapeNode<Type>>(x);
  #line 169 "src/container/Tape.birch"
  aheadCount = aheadCount + Integer(1);
}

#line 175 "src/container/Tape.birch"
template<class Type>
#line 175 "src/container/Tape.birch"
#line 175 "src/container/Tape.birch"
Type Tape_<Type>::pushBack() {
  #line 176 "src/container/Tape.birch"
  fastForward();
  #line 177 "src/container/Tape.birch"
  ahead = construct<TapeNode<Type>>(construct<Type>());
  #line 178 "src/container/Tape.birch"
  aheadCount = aheadCount + Integer(1);
  #line 179 "src/container/Tape.birch"
  return ahead.value()->x;
}

#line 185 "src/container/Tape.birch"
template<class Type>
#line 185 "src/container/Tape.birch"
#line 185 "src/container/Tape.birch"
void Tape_<Type>::popFront() {
  #line 186 "src/container/Tape.birch"
  assert(!(empty()));
  #line 187 "src/container/Tape.birch"
  rewind();
  #line 188 "src/container/Tape.birch"
  ahead = ahead.value()->next;
  #line 189 "src/container/Tape.birch"
  aheadCount = aheadCount - Integer(1);
}

#line 195 "src/container/Tape.birch"
template<class Type>
#line 195 "src/container/Tape.birch"
#line 195 "src/container/Tape.birch"
void Tape_<Type>::popBack() {
  #line 196 "src/container/Tape.birch"
  assert(!(empty()));
  #line 197 "src/container/Tape.birch"
  fastForward();
  #line 198 "src/container/Tape.birch"
  behind = behind.value()->next;
  #line 199 "src/container/Tape.birch"
  behindCount = behindCount - Integer(1);
}

#line 212 "src/container/Tape.birch"
template<class Type>
#line 212 "src/container/Tape.birch"
#line 212 "src/container/Tape.birch"
void Tape_<Type>::insert(const Integer& i, const Type& x) {
  #line 213 "src/container/Tape.birch"
  assert(Integer(0) < i);
  #line 214 "src/container/Tape.birch"
  seek(i);
  auto node = construct<TapeNode<Type>>(x);
  #line 216 "src/container/Tape.birch"
  node->next = ahead;
  #line 217 "src/container/Tape.birch"
  ahead = node;
  #line 218 "src/container/Tape.birch"
  aheadCount = aheadCount + Integer(1);
}

#line 226 "src/container/Tape.birch"
template<class Type>
#line 226 "src/container/Tape.birch"
#line 226 "src/container/Tape.birch"
void Tape_<Type>::erase(const Integer& i) {
  #line 227 "src/container/Tape.birch"
  assert(Integer(0) < i);
  #line 228 "src/container/Tape.birch"
  seek(i);
  #line 229 "src/container/Tape.birch"
  ahead = ahead.value()->next;
  #line 230 "src/container/Tape.birch"
  aheadCount = aheadCount - Integer(1);
}

#line 238 "src/container/Tape.birch"
template<class Type>
#line 238 "src/container/Tape.birch"
#line 238 "src/container/Tape.birch"
Iterator<Type> Tape_<Type>::walk() {
  #line 239 "src/container/Tape.birch"
  rewind();
  #line 240 "src/container/Tape.birch"
  return construct<TapeIterator<Type>>(ahead);
}

#line 246 "src/container/Tape.birch"
template<class Type>
#line 246 "src/container/Tape.birch"
#line 246 "src/container/Tape.birch"
void Tape_<Type>::forward() {
  auto node = ahead.value();
  #line 248 "src/container/Tape.birch"
  ahead = node->next;
  #line 249 "src/container/Tape.birch"
  node->next = behind;
  #line 250 "src/container/Tape.birch"
  behind = node;
  #line 251 "src/container/Tape.birch"
  aheadCount = aheadCount - Integer(1);
  #line 252 "src/container/Tape.birch"
  behindCount = behindCount + Integer(1);
}

#line 258 "src/container/Tape.birch"
template<class Type>
#line 258 "src/container/Tape.birch"
#line 258 "src/container/Tape.birch"
void Tape_<Type>::backward() {
  auto node = behind.value();
  #line 260 "src/container/Tape.birch"
  behind = node->next;
  #line 261 "src/container/Tape.birch"
  node->next = ahead;
  #line 262 "src/container/Tape.birch"
  ahead = node;
  #line 263 "src/container/Tape.birch"
  aheadCount = aheadCount + Integer(1);
  #line 264 "src/container/Tape.birch"
  behindCount = behindCount - Integer(1);
}

#line 270 "src/container/Tape.birch"
template<class Type>
#line 270 "src/container/Tape.birch"
#line 270 "src/container/Tape.birch"
void Tape_<Type>::rewind() {
  #line 271 "src/container/Tape.birch"
  while (behind.has_value()) {
    #line 272 "src/container/Tape.birch"
    backward();
  }
}

#line 279 "src/container/Tape.birch"
template<class Type>
#line 279 "src/container/Tape.birch"
#line 279 "src/container/Tape.birch"
void Tape_<Type>::fastForward() {
  #line 280 "src/container/Tape.birch"
  while (ahead.has_value()) {
    #line 281 "src/container/Tape.birch"
    forward();
  }
}

#line 293 "src/container/Tape.birch"
template<class Type>
#line 293 "src/container/Tape.birch"
#line 293 "src/container/Tape.birch"
void Tape_<Type>::seek(const Integer& k) {
  #line 294 "src/container/Tape.birch"
  assert(k > Integer(0));
  #line 295 "src/container/Tape.birch"
  while (k > size()) {
    #line 296 "src/container/Tape.birch"
    pushBack();
  }
  #line 298 "src/container/Tape.birch"
  while (behindCount < k - Integer(1)) {
    #line 299 "src/container/Tape.birch"
    forward();
  }
  #line 301 "src/container/Tape.birch"
  while (behindCount > k - Integer(1)) {
    #line 302 "src/container/Tape.birch"
    backward();
  }
}

#line 306 "src/container/Tape.birch"
template<class Type>
#line 306 "src/container/Tape.birch"
#line 306 "src/container/Tape.birch"
void Tape_<Type>::read(const Buffer& buffer) {
  #line 307 "src/container/Tape.birch"
  clear();
  auto f = buffer->walk();
  #line 309 "src/container/Tape.birch"
  while (f->hasNext()) {
    auto x = f->next()->get<Type>();
    #line 311 "src/container/Tape.birch"
    if (x.has_value()) {
      #line 312 "src/container/Tape.birch"
      pushBack(x.value());
    }
  }
}

#line 317 "src/container/Tape.birch"
template<class Type>
#line 317 "src/container/Tape.birch"
#line 317 "src/container/Tape.birch"
void Tape_<Type>::write(const Buffer& buffer) {
  #line 318 "src/container/Tape.birch"
  buffer->setEmptyArray();
  auto f = walk();
  #line 320 "src/container/Tape.birch"
  while (f->hasNext()) {
    #line 321 "src/container/Tape.birch"
    buffer->push(f->next());
  }
}

#line 1 "src/container/TapeNode.birch"
template<class Type>
#line 1 "src/container/TapeNode.birch"
TapeNode_<Type>::TapeNode_(const Type& x) :
    #line 1 "src/container/TapeNode.birch"
    base_type_(),
    #line 5 "src/container/TapeNode.birch"
    next(),
    #line 6 "src/container/TapeNode.birch"
    x(x) {
  //
}

#line 228 "src/data/Buffer.birch"
#line 228 "src/data/Buffer.birch"
template<class Type>
#line 228 "src/data/Buffer.birch"
std::optional<Type> Buffer_::get() {
  std::optional<Type> x;
  #line 230 "src/data/Buffer.birch"
  return libbirch::cast<Type>(doGet(x));
}

#line 243 "src/data/Buffer.birch"
#line 243 "src/data/Buffer.birch"
template<class Type>
#line 243 "src/data/Buffer.birch"
std::optional<Type> Buffer_::get(const String& key) {
  auto buffer = get(key);
  #line 245 "src/data/Buffer.birch"
  if (buffer.has_value()) {
    #line 246 "src/data/Buffer.birch"
    return buffer.value()->get<Type>();
  } else {
    #line 248 "src/data/Buffer.birch"
    return std::nullopt;
  }
}

#line 263 "src/data/Buffer.birch"
#line 263 "src/data/Buffer.birch"
template<class Type>
#line 263 "src/data/Buffer.birch"
void Buffer_::get(const Type& o) {
  #line 264 "src/data/Buffer.birch"
  o->read(this->shared_from_this_());
}

#line 279 "src/data/Buffer.birch"
#line 279 "src/data/Buffer.birch"
template<class Type>
#line 279 "src/data/Buffer.birch"
void Buffer_::get(const String& key, const Type& o) {
  auto buffer = get(key);
  #line 281 "src/data/Buffer.birch"
  if (buffer.has_value()) {
    #line 282 "src/data/Buffer.birch"
    buffer.value()->get<Type>(o);
  }
}

#line 322 "src/data/Buffer.birch"
#line 322 "src/data/Buffer.birch"
template<class Type>
#line 322 "src/data/Buffer.birch"
void Buffer_::set(const Type& x) {
  #line 323 "src/data/Buffer.birch"
  doSet(x);
}

#line 332 "src/data/Buffer.birch"
#line 332 "src/data/Buffer.birch"
template<class Type>
#line 332 "src/data/Buffer.birch"
void Buffer_::set(const Integer& t, const Type& x) {
  #line 333 "src/data/Buffer.birch"
  doSet(t, x);
}

#line 359 "src/data/Buffer.birch"
#line 359 "src/data/Buffer.birch"
template<class Type>
#line 359 "src/data/Buffer.birch"
void Buffer_::set(const String& key, const Type& x) {
  #line 360 "src/data/Buffer.birch"
  set(key, make_buffer(x));
}

#line 373 "src/data/Buffer.birch"
#line 373 "src/data/Buffer.birch"
template<class Type>
#line 373 "src/data/Buffer.birch"
void Buffer_::set(const String& key, const Integer& t, const Type& x) {
  #line 374 "src/data/Buffer.birch"
  set(key, make_buffer(t, x));
}

#line 461 "src/data/Buffer.birch"
#line 461 "src/data/Buffer.birch"
template<class Type>
#line 461 "src/data/Buffer.birch"
void Buffer_::push(const Type& x) {
  #line 462 "src/data/Buffer.birch"
  doPush(x);
}

#line 471 "src/data/Buffer.birch"
#line 471 "src/data/Buffer.birch"
template<class Type>
#line 471 "src/data/Buffer.birch"
void Buffer_::push(const Integer& t, const Type& x) {
  #line 472 "src/data/Buffer.birch"
  doPush(t, x);
}

#line 501 "src/data/Buffer.birch"
#line 501 "src/data/Buffer.birch"
template<class Type>
#line 501 "src/data/Buffer.birch"
void Buffer_::push(const String& key, const Type& x) {
  auto buffer = get(key);
  #line 503 "src/data/Buffer.birch"
  if (!(buffer.has_value())) {
    #line 504 "src/data/Buffer.birch"
    buffer = make_buffer();
    #line 505 "src/data/Buffer.birch"
    set(key, buffer.value());
  }
  #line 507 "src/data/Buffer.birch"
  buffer.value()->push(x);
}

#line 517 "src/data/Buffer.birch"
#line 517 "src/data/Buffer.birch"
template<class Type>
#line 517 "src/data/Buffer.birch"
void Buffer_::push(const String& key, const Integer& t, const Type& x) {
  auto buffer = get(key);
  #line 519 "src/data/Buffer.birch"
  if (!(buffer.has_value())) {
    #line 520 "src/data/Buffer.birch"
    buffer = make_buffer();
    #line 521 "src/data/Buffer.birch"
    set(key, buffer.value());
  }
  #line 523 "src/data/Buffer.birch"
  buffer.value()->push(t, x);
}

#line 643 "src/data/Buffer.birch"
#line 643 "src/data/Buffer.birch"
template<class Type>
#line 643 "src/data/Buffer.birch"
std::optional<Type> Buffer_::doGet(const std::optional<Type>& x) {
  #line 644 "src/data/Buffer.birch"
  return make<Type>(this->shared_from_this_());
}

#line 683 "src/data/Buffer.birch"
#line 683 "src/data/Buffer.birch"
template<class Type>
#line 683 "src/data/Buffer.birch"
std::optional<libbirch::DefaultArray<Type,1>> Buffer_::doGet(const std::optional<libbirch::DefaultArray<Type,1>>& x) {
  #line 684 "src/data/Buffer.birch"
  return doGetVector<Type>();
}

#line 687 "src/data/Buffer.birch"
#line 687 "src/data/Buffer.birch"
template<class Type>
#line 687 "src/data/Buffer.birch"
std::optional<libbirch::DefaultArray<Type,1>> Buffer_::doGetVector() {
  #line 688 "src/data/Buffer.birch"
  if (isNil()) {
    #line 689 "src/data/Buffer.birch"
    return std::nullopt;
  } else {
    libbirch::DefaultArray<Type,1> y;
    auto iter = walk();
    #line 693 "src/data/Buffer.birch"
    while (iter->hasNext()) {
      auto z = iter->next()->get<Type>();
      #line 695 "src/data/Buffer.birch"
      if (z.has_value()) {
        #line 696 "src/data/Buffer.birch"

          y.push(z.value());
                } else {
        #line 700 "src/data/Buffer.birch"
        return std::nullopt;
      }
    }
    #line 703 "src/data/Buffer.birch"
    return y;
  }
}

#line 743 "src/data/Buffer.birch"
#line 743 "src/data/Buffer.birch"
template<class Type>
#line 743 "src/data/Buffer.birch"
std::optional<libbirch::DefaultArray<Type,2>> Buffer_::doGet(const std::optional<libbirch::DefaultArray<Type,2>>& x) {
  #line 744 "src/data/Buffer.birch"
  return doGetMatrix<Type>();
}

#line 747 "src/data/Buffer.birch"
#line 747 "src/data/Buffer.birch"
template<class Type>
#line 747 "src/data/Buffer.birch"
std::optional<libbirch::DefaultArray<Type,2>> Buffer_::doGetMatrix() {
  #line 748 "src/data/Buffer.birch"
  if (isNil()) {
    #line 749 "src/data/Buffer.birch"
    return std::nullopt;
  } else {
    libbirch::DefaultArray<Type,1> y;
    auto nrows = Integer(0);
    auto ncols = Integer(0);
    auto iter = walk();
    #line 755 "src/data/Buffer.birch"
    while (iter->hasNext()) {
      auto z = iter->next()->get<libbirch::DefaultArray<Type,1>>();
      #line 757 "src/data/Buffer.birch"
      if (z.has_value() && (nrows == Integer(0) || ncols == length(z.value()))) {
        #line 758 "src/data/Buffer.birch"
        nrows = nrows + Integer(1);
        #line 759 "src/data/Buffer.birch"
        ncols = length(z.value());
        #line 760 "src/data/Buffer.birch"
        for (auto i = Integer(1); i <= ncols; ++i) {
          auto element = z.value()(i);
          #line 762 "src/data/Buffer.birch"

            y.push(element);
                    }
      } else {
        #line 767 "src/data/Buffer.birch"
        return std::nullopt;
      }
    }
    #line 770 "src/data/Buffer.birch"
    return matrix_lambda([=](const Integer& i, const Integer& j) -> Type {
      #line 771 "src/data/Buffer.birch"
      return y((i - Integer(1)) * ncols + j);
    }, nrows, ncols);
  }
}

#line 796 "src/data/Buffer.birch"
#line 796 "src/data/Buffer.birch"
template<class Type>
#line 796 "src/data/Buffer.birch"
void Buffer_::doSet(const Type& x) {
  #line 797 "src/data/Buffer.birch"
  setNil();
  #line 798 "src/data/Buffer.birch"
  x->write(this->shared_from_this_());
}

#line 801 "src/data/Buffer.birch"
#line 801 "src/data/Buffer.birch"
template<class Type>
#line 801 "src/data/Buffer.birch"
void Buffer_::doSet(const Integer& t, const Type& x) {
  #line 802 "src/data/Buffer.birch"
  setNil();
  #line 803 "src/data/Buffer.birch"
  x->write(t, this->shared_from_this_());
}

#line 821 "src/data/Buffer.birch"
#line 821 "src/data/Buffer.birch"
template<class Type>
#line 821 "src/data/Buffer.birch"
void Buffer_::doSet(const libbirch::DefaultArray<Type,1>& x) {
  #line 822 "src/data/Buffer.birch"
  setEmptyArray();
  #line 823 "src/data/Buffer.birch"
  for (auto i = Integer(1); i <= length(x); ++i) {
    #line 824 "src/data/Buffer.birch"
    push(make_buffer(x(i)));
  }
}

#line 828 "src/data/Buffer.birch"
#line 828 "src/data/Buffer.birch"
template<class Type>
#line 828 "src/data/Buffer.birch"
void Buffer_::doSet(const Integer& t, const libbirch::DefaultArray<Type,1>& x) {
  #line 829 "src/data/Buffer.birch"
  setEmptyArray();
  #line 830 "src/data/Buffer.birch"
  for (auto i = Integer(1); i <= length(x); ++i) {
    #line 831 "src/data/Buffer.birch"
    push(make_buffer(t, x(i)));
  }
}

#line 920 "src/data/Buffer.birch"
#line 920 "src/data/Buffer.birch"
template<class Type>
#line 920 "src/data/Buffer.birch"
void Buffer_::doPush(const Type& x) {
  #line 921 "src/data/Buffer.birch"
  push(make_buffer(x));
}

#line 924 "src/data/Buffer.birch"
#line 924 "src/data/Buffer.birch"
template<class Type>
#line 924 "src/data/Buffer.birch"
void Buffer_::doPush(const Integer& t, const Type& x) {
  #line 925 "src/data/Buffer.birch"
  push(make_buffer(t, x));
}

#line 6 "src/distribution/Distribution.birch"
template<class Value>
#line 6 "src/distribution/Distribution.birch"
Distribution_<Value>::Distribution_() :
    #line 6 "src/distribution/Distribution.birch"
    base_type_(),
    #line 14 "src/distribution/Distribution.birch"
    childValue() {
  //
}

#line 19 "src/distribution/Distribution.birch"
template<class Value>
#line 19 "src/distribution/Distribution.birch"
#line 19 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::supportsLazy() {
  #line 20 "src/distribution/Distribution.birch"
  return false;
}

#line 35 "src/distribution/Distribution.birch"
template<class Value>
#line 35 "src/distribution/Distribution.birch"
#line 35 "src/distribution/Distribution.birch"
std::optional<Value> Distribution_<Value>::simulateLazy() {
  #line 36 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 56 "src/distribution/Distribution.birch"
template<class Value>
#line 56 "src/distribution/Distribution.birch"
#line 56 "src/distribution/Distribution.birch"
std::optional<Expression<Real>> Distribution_<Value>::logpdfLazy(const Expression<Value>& x) {
  #line 57 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 65 "src/distribution/Distribution.birch"
template<class Value>
#line 65 "src/distribution/Distribution.birch"
#line 65 "src/distribution/Distribution.birch"
std::optional<Delay> Distribution_<Value>::update(const Value& x) {
  #line 66 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 74 "src/distribution/Distribution.birch"
template<class Value>
#line 74 "src/distribution/Distribution.birch"
#line 74 "src/distribution/Distribution.birch"
std::optional<Delay> Distribution_<Value>::updateLazy(const Expression<Value>& x) {
  #line 75 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 85 "src/distribution/Distribution.birch"
template<class Value>
#line 85 "src/distribution/Distribution.birch"
#line 85 "src/distribution/Distribution.birch"
Real Distribution_<Value>::pdf(const Value& x) {
  #line 86 "src/distribution/Distribution.birch"
  return exp(logpdf(x));
}

#line 96 "src/distribution/Distribution.birch"
template<class Value>
#line 96 "src/distribution/Distribution.birch"
#line 96 "src/distribution/Distribution.birch"
std::optional<Real> Distribution_<Value>::cdf(const Value& x) {
  #line 97 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 107 "src/distribution/Distribution.birch"
template<class Value>
#line 107 "src/distribution/Distribution.birch"
#line 107 "src/distribution/Distribution.birch"
std::optional<Value> Distribution_<Value>::quantile(const Real& P) {
  #line 108 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 114 "src/distribution/Distribution.birch"
template<class Value>
#line 114 "src/distribution/Distribution.birch"
#line 114 "src/distribution/Distribution.birch"
std::optional<Value> Distribution_<Value>::lower() {
  #line 115 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 121 "src/distribution/Distribution.birch"
template<class Value>
#line 121 "src/distribution/Distribution.birch"
#line 121 "src/distribution/Distribution.birch"
std::optional<Value> Distribution_<Value>::upper() {
  #line 122 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 125 "src/distribution/Distribution.birch"
template<class Value>
#line 125 "src/distribution/Distribution.birch"
#line 125 "src/distribution/Distribution.birch"
void Distribution_<Value>::graft(const Delay& node) {
  #line 126 "src/distribution/Distribution.birch"
  assert(!(this->childValue.has_value()));
  #line 127 "src/distribution/Distribution.birch"
  this->base_type_::graft(node);
}

#line 130 "src/distribution/Distribution.birch"
template<class Value>
#line 130 "src/distribution/Distribution.birch"
#line 130 "src/distribution/Distribution.birch"
void Distribution_<Value>::graftReplace(const Delay& node) {
  #line 131 "src/distribution/Distribution.birch"
  assert(!(this->childValue.has_value()));
  #line 132 "src/distribution/Distribution.birch"
  this->base_type_::graftReplace(node);
}

#line 139 "src/distribution/Distribution.birch"
template<class Value>
#line 139 "src/distribution/Distribution.birch"
#line 139 "src/distribution/Distribution.birch"
void Distribution_<Value>::graft(const Value& childValue) {
  #line 140 "src/distribution/Distribution.birch"
  assert(!(this->child.has_value()));
  #line 141 "src/distribution/Distribution.birch"
  assert(!(this->childValue.has_value()));
  #line 142 "src/distribution/Distribution.birch"
  this->childValue = childValue;
}

#line 145 "src/distribution/Distribution.birch"
template<class Value>
#line 145 "src/distribution/Distribution.birch"
#line 145 "src/distribution/Distribution.birch"
Delay Distribution_<Value>::prune() {
  #line 146 "src/distribution/Distribution.birch"
  if (child.has_value()) {
    auto x = libbirch::cast<Random<Value>>(child.value()->prune());
    #line 148 "src/distribution/Distribution.birch"
    assert(x.has_value());
    #line 149 "src/distribution/Distribution.birch"
    child = std::nullopt;
    #line 150 "src/distribution/Distribution.birch"
    return handle_prune(this->shared_from_this_(), x.value());
  } else {
    #line 151 "src/distribution/Distribution.birch"
    if (childValue.has_value()) {
      auto x = childValue.value();
      #line 153 "src/distribution/Distribution.birch"
      childValue = std::nullopt;
      #line 154 "src/distribution/Distribution.birch"
      return handle_prune(this->shared_from_this_(), x);
    } else {
      #line 156 "src/distribution/Distribution.birch"
      return this->shared_from_this_();
    }
  }
}

#line 160 "src/distribution/Distribution.birch"
template<class Value>
#line 160 "src/distribution/Distribution.birch"
#line 160 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isGaussian() {
  #line 161 "src/distribution/Distribution.birch"
  return false;
}

#line 164 "src/distribution/Distribution.birch"
template<class Value>
#line 164 "src/distribution/Distribution.birch"
#line 164 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> Distribution_<Value>::getGaussian() {
  #line 165 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 168 "src/distribution/Distribution.birch"
template<class Value>
#line 168 "src/distribution/Distribution.birch"
#line 168 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isBeta() {
  #line 169 "src/distribution/Distribution.birch"
  return false;
}

#line 172 "src/distribution/Distribution.birch"
template<class Value>
#line 172 "src/distribution/Distribution.birch"
#line 172 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> Distribution_<Value>::getBeta() {
  #line 173 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 176 "src/distribution/Distribution.birch"
template<class Value>
#line 176 "src/distribution/Distribution.birch"
#line 176 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isGamma() {
  #line 177 "src/distribution/Distribution.birch"
  return false;
}

#line 180 "src/distribution/Distribution.birch"
template<class Value>
#line 180 "src/distribution/Distribution.birch"
#line 180 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> Distribution_<Value>::getGamma() {
  #line 181 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 184 "src/distribution/Distribution.birch"
template<class Value>
#line 184 "src/distribution/Distribution.birch"
#line 184 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isInverseGamma() {
  #line 185 "src/distribution/Distribution.birch"
  return false;
}

#line 188 "src/distribution/Distribution.birch"
template<class Value>
#line 188 "src/distribution/Distribution.birch"
#line 188 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> Distribution_<Value>::getInverseGamma() {
  #line 189 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 192 "src/distribution/Distribution.birch"
template<class Value>
#line 192 "src/distribution/Distribution.birch"
#line 192 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isInverseWishart() {
  #line 193 "src/distribution/Distribution.birch"
  return false;
}

#line 196 "src/distribution/Distribution.birch"
template<class Value>
#line 196 "src/distribution/Distribution.birch"
#line 196 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>>> Distribution_<Value>::getInverseWishart() {
  #line 197 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 200 "src/distribution/Distribution.birch"
template<class Value>
#line 200 "src/distribution/Distribution.birch"
#line 200 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isNormalInverseGamma() {
  #line 201 "src/distribution/Distribution.birch"
  return false;
}

#line 204 "src/distribution/Distribution.birch"
template<class Value>
#line 204 "src/distribution/Distribution.birch"
#line 204 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>, Expression<Real>, Expression<Real>>> Distribution_<Value>::getNormalInverseGamma() {
  #line 206 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 209 "src/distribution/Distribution.birch"
template<class Value>
#line 209 "src/distribution/Distribution.birch"
#line 209 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isDirichlet() {
  #line 210 "src/distribution/Distribution.birch"
  return false;
}

#line 213 "src/distribution/Distribution.birch"
template<class Value>
#line 213 "src/distribution/Distribution.birch"
#line 213 "src/distribution/Distribution.birch"
std::optional<Expression<libbirch::DefaultArray<Real,1>>> Distribution_<Value>::getDirichlet() {
  #line 214 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 217 "src/distribution/Distribution.birch"
template<class Value>
#line 217 "src/distribution/Distribution.birch"
#line 217 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isRestaurant() {
  #line 218 "src/distribution/Distribution.birch"
  return false;
}

#line 221 "src/distribution/Distribution.birch"
template<class Value>
#line 221 "src/distribution/Distribution.birch"
#line 221 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>, Expression<libbirch::DefaultArray<Integer,1>>, Expression<Integer>>> Distribution_<Value>::getRestaurant() {
  #line 223 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 226 "src/distribution/Distribution.birch"
template<class Value>
#line 226 "src/distribution/Distribution.birch"
#line 226 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isMultivariateGaussian() {
  #line 227 "src/distribution/Distribution.birch"
  return false;
}

#line 230 "src/distribution/Distribution.birch"
template<class Value>
#line 230 "src/distribution/Distribution.birch"
#line 230 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>>> Distribution_<Value>::getMultivariateGaussian() {
  #line 232 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 235 "src/distribution/Distribution.birch"
template<class Value>
#line 235 "src/distribution/Distribution.birch"
#line 235 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isMultivariateNormalInverseGamma() {
  #line 236 "src/distribution/Distribution.birch"
  return false;
}

#line 239 "src/distribution/Distribution.birch"
template<class Value>
#line 239 "src/distribution/Distribution.birch"
#line 239 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>, Expression<Real>>> Distribution_<Value>::getMultivariateNormalInverseGamma() {
  #line 241 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 244 "src/distribution/Distribution.birch"
template<class Value>
#line 244 "src/distribution/Distribution.birch"
#line 244 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isMatrixGaussian() {
  #line 245 "src/distribution/Distribution.birch"
  return false;
}

#line 248 "src/distribution/Distribution.birch"
template<class Value>
#line 248 "src/distribution/Distribution.birch"
#line 248 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>>> Distribution_<Value>::getMatrixGaussian() {
  #line 250 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 253 "src/distribution/Distribution.birch"
template<class Value>
#line 253 "src/distribution/Distribution.birch"
#line 253 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isMatrixNormalInverseWishart() {
  #line 254 "src/distribution/Distribution.birch"
  return false;
}

#line 257 "src/distribution/Distribution.birch"
template<class Value>
#line 257 "src/distribution/Distribution.birch"
#line 257 "src/distribution/Distribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>>> Distribution_<Value>::getMatrixNormalInverseWishart() {
  #line 259 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 262 "src/distribution/Distribution.birch"
template<class Value>
#line 262 "src/distribution/Distribution.birch"
#line 262 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isDiscrete() {
  #line 263 "src/distribution/Distribution.birch"
  return false;
}

#line 266 "src/distribution/Distribution.birch"
template<class Value>
#line 266 "src/distribution/Distribution.birch"
#line 266 "src/distribution/Distribution.birch"
std::optional<DiscreteDistribution> Distribution_<Value>::getDiscrete() {
  #line 267 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 270 "src/distribution/Distribution.birch"
template<class Value>
#line 270 "src/distribution/Distribution.birch"
#line 270 "src/distribution/Distribution.birch"
Boolean Distribution_<Value>::isBoundedDiscrete() {
  #line 271 "src/distribution/Distribution.birch"
  return false;
}

#line 274 "src/distribution/Distribution.birch"
template<class Value>
#line 274 "src/distribution/Distribution.birch"
#line 274 "src/distribution/Distribution.birch"
std::optional<BoundedDiscreteDistribution> Distribution_<Value>::getBoundedDiscrete() {
  #line 275 "src/distribution/Distribution.birch"
  return std::nullopt;
}

#line 74 "src/event/Handler.birch"
#line 74 "src/event/Handler.birch"
template<class Value>
#line 74 "src/event/Handler.birch"
Random<Value> Handler_::handleAssume(const Random<Value>& x, const Distribution<Value>& p) {
  #line 76 "src/event/Handler.birch"
  if (x->hasValue()) {
    #line 77 "src/event/Handler.birch"
    handleObserve(x, p);
  } else {
    #line 78 "src/event/Handler.birch"
    if (!(delaySampling)) {
      #line 79 "src/event/Handler.birch"
      handleSimulate(x, p);
    } else {
      #line 81 "src/event/Handler.birch"
      x->assume(p);
    }
  }
  #line 83 "src/event/Handler.birch"
  return x;
}

#line 94 "src/event/Handler.birch"
#line 94 "src/event/Handler.birch"
template<class Value>
#line 94 "src/event/Handler.birch"
Random<Value> Handler_::handleSimulate(const Random<Value>& x, const Distribution<Value>& p) {
  #line 96 "src/event/Handler.birch"
  x->assume(p);
  #line 97 "src/event/Handler.birch"
  handleDelaySimulate(x);
  #line 98 "src/event/Handler.birch"
  return x;
}

#line 108 "src/event/Handler.birch"
#line 108 "src/event/Handler.birch"
template<class Value>
#line 108 "src/event/Handler.birch"
Value Handler_::handleSimulate(const Distribution<Value>& p) {
  #line 109 "src/event/Handler.birch"
  if (delayExpressions && p->supportsLazy()) {
    auto x = p->simulateLazy().value();
    #line 111 "src/event/Handler.birch"
    p->graft(x);
    #line 112 "src/event/Handler.birch"
    return x;
  } else {
    auto x = p->simulate();
    #line 115 "src/event/Handler.birch"
    p->graft(x);
    #line 116 "src/event/Handler.birch"
    return x;
  }
}

#line 128 "src/event/Handler.birch"
#line 128 "src/event/Handler.birch"
template<class Value>
#line 128 "src/event/Handler.birch"
Random<Value> Handler_::handleObserve(const Random<Value>& x, const Distribution<Value>& p) {
  #line 130 "src/event/Handler.birch"
  x->assume(p);
  #line 131 "src/event/Handler.birch"
  handleDelayObserve(x);
  #line 132 "src/event/Handler.birch"
  return x;
}

#line 143 "src/event/Handler.birch"
#line 143 "src/event/Handler.birch"
template<class Value>
#line 143 "src/event/Handler.birch"
Value Handler_::handleObserve(const Value& x, const Distribution<Value>& p) {
  #line 144 "src/event/Handler.birch"
  if (delayExpressions && p->supportsLazy()) {
    #line 145 "src/event/Handler.birch"
    handleFactor(p->logpdfLazy(box(x)).value());
  } else {
    #line 147 "src/event/Handler.birch"
    handleFactor(p->logpdf(x));
  }
  #line 149 "src/event/Handler.birch"
  p->graft(x);
  #line 150 "src/event/Handler.birch"
  return x;
}

#line 160 "src/event/Handler.birch"
#line 160 "src/event/Handler.birch"
template<class Arg>
#line 160 "src/event/Handler.birch"
void Handler_::handleFactor(const Arg& w) {
  #line 161 "src/event/Handler.birch"
  if (delayExpressions) {
    #line 162 "src/event/Handler.birch"
    this->w = this->w + birch::peek(w);
    #line 163 "src/event/Handler.birch"
    if (\u03c0.has_value()) {
      #line 164 "src/event/Handler.birch"
      \u03c0 = box(\u03c0.value() + w);
    } else {
      #line 166 "src/event/Handler.birch"
      \u03c0 = box(w);
    }
  } else {
    #line 169 "src/event/Handler.birch"
    this->w = this->w + birch::value(w);
  }
}

#line 178 "src/event/Handler.birch"
#line 178 "src/event/Handler.birch"
template<class Value>
#line 178 "src/event/Handler.birch"
void Handler_::handleDelaySimulate(const Random<Value>& x) {
  auto p = x->getDistribution();
  #line 180 "src/event/Handler.birch"
  if (delayExpressions && p->supportsLazy()) {
    #line 181 "src/event/Handler.birch"
    x->move(p->simulateLazy().value());
    #line 182 "src/event/Handler.birch"
    arg(x);
  } else {
    #line 184 "src/event/Handler.birch"
    x->move(p->simulate());
  }
  #line 186 "src/event/Handler.birch"
  x->p = std::nullopt;
}

#line 194 "src/event/Handler.birch"
#line 194 "src/event/Handler.birch"
template<class Value>
#line 194 "src/event/Handler.birch"
void Handler_::handleDelayObserve(const Random<Value>& x) {
  auto p = x->getDistribution();
  #line 196 "src/event/Handler.birch"
  if (delayExpressions && p->supportsLazy()) {
    #line 197 "src/event/Handler.birch"
    handleFactor(p->logpdfLazy(x).value());
  } else {
    #line 199 "src/event/Handler.birch"
    handleFactor(p->logpdf(x->value()));
  }
  #line 201 "src/event/Handler.birch"
  x->p = std::nullopt;
}

#line 212 "src/event/Handler.birch"
#line 212 "src/event/Handler.birch"
template<class Value>
#line 212 "src/event/Handler.birch"
Delay Handler_::handleDelayPrune(const Distribution<Value>& p, const Random<Value>& x) {
  #line 214 "src/event/Handler.birch"
  if (delayExpressions && p->supportsLazy()) {
    #line 215 "src/event/Handler.birch"
    return p->updateLazy(x).value();
  } else {
    #line 217 "src/event/Handler.birch"
    return p->update(x->value()).value();
  }
}

#line 229 "src/event/Handler.birch"
#line 229 "src/event/Handler.birch"
template<class Value>
#line 229 "src/event/Handler.birch"
Delay Handler_::handleDelayPrune(const Distribution<Value>& p, const Value& x) {
  #line 230 "src/event/Handler.birch"
  if (delayExpressions && p->supportsLazy()) {
    #line 231 "src/event/Handler.birch"
    return p->updateLazy(box(x)).value();
  } else {
    #line 233 "src/event/Handler.birch"
    return p->update(x).value();
  }
}

#line 1 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 1 "src/expression/BoxedForm.birch"
BoxedForm_<Value, Form>::BoxedForm_(const Value& x, const Form& f) :
    #line 1 "src/expression/BoxedForm.birch"
    base_type_(),
    #line 11 "src/expression/BoxedForm.birch"
    x(x),
    #line 16 "src/expression/BoxedForm.birch"
    f(f),
    #line 21 "src/expression/BoxedForm.birch"
    d(),
    #line 26 "src/expression/BoxedForm.birch"
    gen(Integer(0)),
    #line 31 "src/expression/BoxedForm.birch"
    evalCount(Integer(0)),
    #line 37 "src/expression/BoxedForm.birch"
    visitCount(Integer(0)) {
  //
}

#line 39 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 39 "src/expression/BoxedForm.birch"
#line 39 "src/expression/BoxedForm.birch"
Boolean BoxedForm_<Value, Form>::isRandom() {
  #line 40 "src/expression/BoxedForm.birch"
  return false;
}

#line 43 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 43 "src/expression/BoxedForm.birch"
#line 43 "src/expression/BoxedForm.birch"
Boolean BoxedForm_<Value, Form>::isConstant() {
  #line 44 "src/expression/BoxedForm.birch"
  return !(f.has_value());
}

#line 47 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 47 "src/expression/BoxedForm.birch"
#line 47 "src/expression/BoxedForm.birch"
Integer BoxedForm_<Value, Form>::rows() {
  #line 48 "src/expression/BoxedForm.birch"
  return birch::rows(x);
}

#line 51 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 51 "src/expression/BoxedForm.birch"
#line 51 "src/expression/BoxedForm.birch"
Integer BoxedForm_<Value, Form>::columns() {
  #line 52 "src/expression/BoxedForm.birch"
  return birch::columns(x);
}

#line 55 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 55 "src/expression/BoxedForm.birch"
#line 55 "src/expression/BoxedForm.birch"
Value BoxedForm_<Value, Form>::value() {
  #line 56 "src/expression/BoxedForm.birch"
  constant();
  #line 57 "src/expression/BoxedForm.birch"
  return x;
}

#line 60 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 60 "src/expression/BoxedForm.birch"
#line 60 "src/expression/BoxedForm.birch"
Value BoxedForm_<Value, Form>::peek() {
  #line 61 "src/expression/BoxedForm.birch"
  return x;
}

#line 64 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 64 "src/expression/BoxedForm.birch"
#line 64 "src/expression/BoxedForm.birch"
Value BoxedForm_<Value, Form>::eval() {
  #line 65 "src/expression/BoxedForm.birch"
  if (f.has_value()) {
    #line 66 "src/expression/BoxedForm.birch"
    evalCount = evalCount + Integer(1);
  }
  #line 68 "src/expression/BoxedForm.birch"
  return x;
}

#line 71 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 71 "src/expression/BoxedForm.birch"
#line 71 "src/expression/BoxedForm.birch"
Value BoxedForm_<Value, Form>::reval() {
  #line 72 "src/expression/BoxedForm.birch"
  if (f.has_value()) {
    #line 73 "src/expression/BoxedForm.birch"
    if (visitCount == Integer(0)) {
      #line 74 "src/expression/BoxedForm.birch"
      assert(!(d.has_value()));
      #line 75 "src/expression/BoxedForm.birch"
      x = birch::reval(f.value());
    }
    #line 77 "src/expression/BoxedForm.birch"
    visitCount = visitCount + Integer(1);
    #line 78 "src/expression/BoxedForm.birch"
    if (visitCount >= evalCount) {
      #line 79 "src/expression/BoxedForm.birch"
      assert(visitCount == evalCount || evalCount == Integer(0));
      #line 84 "src/expression/BoxedForm.birch"
      visitCount = Integer(0);
    }
  }
  #line 87 "src/expression/BoxedForm.birch"
  return x;
}

#line 90 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 90 "src/expression/BoxedForm.birch"
#line 90 "src/expression/BoxedForm.birch"
void BoxedForm_<Value, Form>::grad(const Value& d) {
  #line 91 "src/expression/BoxedForm.birch"
  if (f.has_value()) {
    #line 92 "src/expression/BoxedForm.birch"
    if (visitCount == Integer(0)) {
      #line 93 "src/expression/BoxedForm.birch"
      assert(!(this->d.has_value()));
      #line 94 "src/expression/BoxedForm.birch"
      this->d = d;
    } else {
      #line 96 "src/expression/BoxedForm.birch"
      assert(this->d.has_value());
      #line 97 "src/expression/BoxedForm.birch"
      this->d = this->d.value() + d;
    }
    #line 99 "src/expression/BoxedForm.birch"
    visitCount = visitCount + Integer(1);
    #line 100 "src/expression/BoxedForm.birch"
    if (visitCount >= evalCount) {
      #line 101 "src/expression/BoxedForm.birch"
      assert(visitCount == evalCount || evalCount == Integer(0));
      #line 106 "src/expression/BoxedForm.birch"
      birch::grad(f.value(), this->d.value());
      #line 107 "src/expression/BoxedForm.birch"
      this->d = std::nullopt;
      #line 108 "src/expression/BoxedForm.birch"
      visitCount = Integer(0);
    }
  }
}

#line 113 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 113 "src/expression/BoxedForm.birch"
#line 113 "src/expression/BoxedForm.birch"
void BoxedForm_<Value, Form>::label(const Integer& gen) {
  #line 114 "src/expression/BoxedForm.birch"
  if (f.has_value() && this->gen == Integer(0)) {
    #line 115 "src/expression/BoxedForm.birch"
    this->gen = gen;
    #line 116 "src/expression/BoxedForm.birch"
    birch::label(f.value(), gen);
  }
}

#line 120 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 120 "src/expression/BoxedForm.birch"
#line 120 "src/expression/BoxedForm.birch"
void BoxedForm_<Value, Form>::constant(const Integer& gen) {
  #line 121 "src/expression/BoxedForm.birch"
  if (f.has_value()) {
    #line 122 "src/expression/BoxedForm.birch"
    if (this->gen < gen) {
      #line 123 "src/expression/BoxedForm.birch"
      constant();
    } else {
      #line 125 "src/expression/BoxedForm.birch"
      birch::constant(f.value(), gen);
    }
  }
}

#line 130 "src/expression/BoxedForm.birch"
template<class Value,class Form>
#line 130 "src/expression/BoxedForm.birch"
#line 130 "src/expression/BoxedForm.birch"
void BoxedForm_<Value, Form>::constant() {
  #line 131 "src/expression/BoxedForm.birch"
  if (f.has_value()) {
    #line 132 "src/expression/BoxedForm.birch"
    birch::constant(f.value());
    #line 133 "src/expression/BoxedForm.birch"
    f = std::nullopt;
    #line 134 "src/expression/BoxedForm.birch"
    d = std::nullopt;
    #line 135 "src/expression/BoxedForm.birch"
    gen = Integer(0);
    #line 136 "src/expression/BoxedForm.birch"
    evalCount = Integer(0);
    #line 137 "src/expression/BoxedForm.birch"
    visitCount = Integer(0);
  }
}

#line 4 "src/expression/BoxedValue.birch"
template<class Value>
#line 4 "src/expression/BoxedValue.birch"
BoxedValue_<Value>::BoxedValue_(const Value& x) :
    #line 4 "src/expression/BoxedValue.birch"
    base_type_(),
    #line 8 "src/expression/BoxedValue.birch"
    x(x) {
  this->libbirch::Any::acyclic_();
}

#line 10 "src/expression/BoxedValue.birch"
template<class Value>
#line 10 "src/expression/BoxedValue.birch"
#line 10 "src/expression/BoxedValue.birch"
Boolean BoxedValue_<Value>::isRandom() {
  #line 11 "src/expression/BoxedValue.birch"
  return false;
}

#line 14 "src/expression/BoxedValue.birch"
template<class Value>
#line 14 "src/expression/BoxedValue.birch"
#line 14 "src/expression/BoxedValue.birch"
Boolean BoxedValue_<Value>::isConstant() {
  #line 15 "src/expression/BoxedValue.birch"
  return true;
}

#line 18 "src/expression/BoxedValue.birch"
template<class Value>
#line 18 "src/expression/BoxedValue.birch"
#line 18 "src/expression/BoxedValue.birch"
Integer BoxedValue_<Value>::rows() {
  #line 19 "src/expression/BoxedValue.birch"
  return birch::rows(x);
}

#line 22 "src/expression/BoxedValue.birch"
template<class Value>
#line 22 "src/expression/BoxedValue.birch"
#line 22 "src/expression/BoxedValue.birch"
Integer BoxedValue_<Value>::columns() {
  #line 23 "src/expression/BoxedValue.birch"
  return birch::columns(x);
}

#line 26 "src/expression/BoxedValue.birch"
template<class Value>
#line 26 "src/expression/BoxedValue.birch"
#line 26 "src/expression/BoxedValue.birch"
Value BoxedValue_<Value>::value() {
  #line 27 "src/expression/BoxedValue.birch"
  return x;
}

#line 30 "src/expression/BoxedValue.birch"
template<class Value>
#line 30 "src/expression/BoxedValue.birch"
#line 30 "src/expression/BoxedValue.birch"
Value BoxedValue_<Value>::peek() {
  #line 31 "src/expression/BoxedValue.birch"
  return x;
}

#line 34 "src/expression/BoxedValue.birch"
template<class Value>
#line 34 "src/expression/BoxedValue.birch"
#line 34 "src/expression/BoxedValue.birch"
Value BoxedValue_<Value>::eval() {
  #line 35 "src/expression/BoxedValue.birch"
  return x;
}

#line 38 "src/expression/BoxedValue.birch"
template<class Value>
#line 38 "src/expression/BoxedValue.birch"
#line 38 "src/expression/BoxedValue.birch"
Value BoxedValue_<Value>::reval() {
  #line 39 "src/expression/BoxedValue.birch"
  return x;
}

#line 42 "src/expression/BoxedValue.birch"
template<class Value>
#line 42 "src/expression/BoxedValue.birch"
#line 42 "src/expression/BoxedValue.birch"
void BoxedValue_<Value>::grad(const Value& d) {
}

#line 46 "src/expression/BoxedValue.birch"
template<class Value>
#line 46 "src/expression/BoxedValue.birch"
#line 46 "src/expression/BoxedValue.birch"
void BoxedValue_<Value>::label(const Integer& gen) {
}

#line 50 "src/expression/BoxedValue.birch"
template<class Value>
#line 50 "src/expression/BoxedValue.birch"
#line 50 "src/expression/BoxedValue.birch"
void BoxedValue_<Value>::constant(const Integer& gen) {
}

#line 54 "src/expression/BoxedValue.birch"
template<class Value>
#line 54 "src/expression/BoxedValue.birch"
#line 54 "src/expression/BoxedValue.birch"
void BoxedValue_<Value>::constant() {
}

#line 93 "src/expression/Expression.birch"
template<class Value>
#line 93 "src/expression/Expression.birch"
Expression_<Value>::Expression_() :
    #line 93 "src/expression/Expression.birch"
    base_type_() {
  //
}

#line 117 "src/expression/Expression.birch"
template<class Value>
#line 117 "src/expression/Expression.birch"
#line 117 "src/expression/Expression.birch"
Integer Expression_<Value>::length() {
  #line 118 "src/expression/Expression.birch"
  return rows();
}

#line 124 "src/expression/Expression.birch"
template<class Value>
#line 124 "src/expression/Expression.birch"
#line 124 "src/expression/Expression.birch"
Integer Expression_<Value>::size() {
  #line 125 "src/expression/Expression.birch"
  return rows() * columns();
}

#line 224 "src/expression/Expression.birch"
template<class Value>
#line 224 "src/expression/Expression.birch"
#line 224 "src/expression/Expression.birch"
void Expression_<Value>::write(const Buffer& buffer) {
  #line 225 "src/expression/Expression.birch"
  buffer->set(value());
}

#line 7 "src/expression/MatrixSplitExpression.birch"
template<class Value,class Form>
#line 7 "src/expression/MatrixSplitExpression.birch"
MatrixSplitExpression_<Value, Form>::MatrixSplitExpression_(const Value& x, const Form& f) :
    #line 7 "src/expression/MatrixSplitExpression.birch"
    base_type_(x, f) {
  //
}

#line 9 "src/expression/MatrixSplitExpression.birch"
template<class Value,class Form>
#line 9 "src/expression/MatrixSplitExpression.birch"
#line 9 "src/expression/MatrixSplitExpression.birch"
void MatrixSplitExpression_<Value, Form>::grad(const Real& d, const Integer& i, const Integer& j) {
  #line 10 "src/expression/MatrixSplitExpression.birch"
  if (this->f.has_value()) {
    #line 11 "src/expression/MatrixSplitExpression.birch"
    if (this->visitCount == Integer(0)) {
      #line 12 "src/expression/MatrixSplitExpression.birch"
      this->d = matrix(0.0, this->rows(), this->columns());
    }
    #line 14 "src/expression/MatrixSplitExpression.birch"
    this->d.value()(i, j) = this->d.value()(i, j) + d;
    #line 15 "src/expression/MatrixSplitExpression.birch"
    this->visitCount = this->visitCount + Integer(1);
    #line 16 "src/expression/MatrixSplitExpression.birch"
    if (this->visitCount >= this->evalCount) {
      #line 17 "src/expression/MatrixSplitExpression.birch"
      assert(this->visitCount == this->evalCount || this->evalCount == Integer(0));
      #line 22 "src/expression/MatrixSplitExpression.birch"
      birch::grad(this->f.value(), this->d.value());
      #line 23 "src/expression/MatrixSplitExpression.birch"
      this->d = std::nullopt;
      #line 24 "src/expression/MatrixSplitExpression.birch"
      this->visitCount = Integer(0);
    }
  }
}

#line 13 "src/expression/Random.birch"
template<class Value>
#line 13 "src/expression/Random.birch"
Random_<Value>::Random_() :
    #line 13 "src/expression/Random.birch"
    base_type_(),
    #line 17 "src/expression/Random.birch"
    x(),
    #line 22 "src/expression/Random.birch"
    p(),
    #line 27 "src/expression/Random.birch"
    d(),
    #line 32 "src/expression/Random.birch"
    flagConstant(false),
    #line 37 "src/expression/Random.birch"
    gen(Integer(0)) {
  //
}

#line 42 "src/expression/Random.birch"
template<class Value>
#line 42 "src/expression/Random.birch"
Random_<Value>& Random_<Value>::operator=(const Value& x) {
  #line 43 "src/expression/Random.birch"
  assert(!(this->x.has_value()));
  #line 44 "src/expression/Random.birch"
  prune();
  #line 45 "src/expression/Random.birch"
  this->x = x;
  #line 46 "src/expression/Random.birch"
  if (this->p.has_value()) {
    #line 47 "src/expression/Random.birch"
    handle_delay_observe(this->shared_from_this_());
  }
  #line 49 "src/expression/Random.birch"
  constant();
  #line 42 "src/expression/Random.birch"
  return *this;
}

#line 52 "src/expression/Random.birch"
template<class Value>
#line 52 "src/expression/Random.birch"
#line 52 "src/expression/Random.birch"
Boolean Random_<Value>::isRandom() {
  #line 53 "src/expression/Random.birch"
  return true;
}

#line 56 "src/expression/Random.birch"
template<class Value>
#line 56 "src/expression/Random.birch"
#line 56 "src/expression/Random.birch"
Boolean Random_<Value>::isConstant() {
  #line 57 "src/expression/Random.birch"
  return flagConstant;
}

#line 63 "src/expression/Random.birch"
template<class Value>
#line 63 "src/expression/Random.birch"
#line 63 "src/expression/Random.birch"
Boolean Random_<Value>::hasValue() {
  #line 64 "src/expression/Random.birch"
  return x.has_value();
}

#line 67 "src/expression/Random.birch"
template<class Value>
#line 67 "src/expression/Random.birch"
#line 67 "src/expression/Random.birch"
Integer Random_<Value>::rows() {
  #line 68 "src/expression/Random.birch"
  eval();
  #line 69 "src/expression/Random.birch"
  return birch::rows(x.value());
}

#line 72 "src/expression/Random.birch"
template<class Value>
#line 72 "src/expression/Random.birch"
#line 72 "src/expression/Random.birch"
Integer Random_<Value>::columns() {
  #line 73 "src/expression/Random.birch"
  eval();
  #line 74 "src/expression/Random.birch"
  return birch::columns(x.value());
}

#line 77 "src/expression/Random.birch"
template<class Value>
#line 77 "src/expression/Random.birch"
#line 77 "src/expression/Random.birch"
Value Random_<Value>::value() {
  #line 78 "src/expression/Random.birch"
  eval();
  #line 79 "src/expression/Random.birch"
  constant();
  #line 80 "src/expression/Random.birch"
  return x.value();
}

#line 83 "src/expression/Random.birch"
template<class Value>
#line 83 "src/expression/Random.birch"
#line 83 "src/expression/Random.birch"
Value Random_<Value>::peek() {
  #line 84 "src/expression/Random.birch"
  return eval();
}

#line 87 "src/expression/Random.birch"
template<class Value>
#line 87 "src/expression/Random.birch"
#line 87 "src/expression/Random.birch"
Value Random_<Value>::eval() {
  #line 88 "src/expression/Random.birch"
  if (!(x.has_value())) {
    #line 89 "src/expression/Random.birch"
    prune();
    #line 90 "src/expression/Random.birch"
    handle_delay_simulate(this->shared_from_this_());
  }
  #line 92 "src/expression/Random.birch"
  return x.value();
}

#line 95 "src/expression/Random.birch"
template<class Value>
#line 95 "src/expression/Random.birch"
#line 95 "src/expression/Random.birch"
Value Random_<Value>::reval() {
  #line 98 "src/expression/Random.birch"
  return x.value();
}

#line 101 "src/expression/Random.birch"
template<class Value>
#line 101 "src/expression/Random.birch"
#line 101 "src/expression/Random.birch"
void Random_<Value>::grad(const Value& d) {
  #line 102 "src/expression/Random.birch"
  if (this->d.has_value()) {
    #line 103 "src/expression/Random.birch"
    this->d = this->d.value() + d;
  } else {
    #line 105 "src/expression/Random.birch"
    this->d = d;
  }
}

#line 109 "src/expression/Random.birch"
template<class Value>
#line 109 "src/expression/Random.birch"
#line 109 "src/expression/Random.birch"
void Random_<Value>::label(const Integer& gen) {
  #line 110 "src/expression/Random.birch"
  if (!(flagConstant) && this->gen == Integer(0)) {
    #line 111 "src/expression/Random.birch"
    this->gen = gen;
  }
}

#line 115 "src/expression/Random.birch"
template<class Value>
#line 115 "src/expression/Random.birch"
#line 115 "src/expression/Random.birch"
void Random_<Value>::constant(const Integer& gen) {
  #line 116 "src/expression/Random.birch"
  if (!(flagConstant) && this->gen < gen) {
    #line 117 "src/expression/Random.birch"
    constant();
  }
}

#line 121 "src/expression/Random.birch"
template<class Value>
#line 121 "src/expression/Random.birch"
#line 121 "src/expression/Random.birch"
void Random_<Value>::constant() {
  #line 122 "src/expression/Random.birch"
  if (!(flagConstant)) {
    #line 123 "src/expression/Random.birch"
    p = std::nullopt;
    #line 124 "src/expression/Random.birch"
    d = std::nullopt;
    #line 125 "src/expression/Random.birch"
    flagConstant = true;
  }
}

#line 132 "src/expression/Random.birch"
template<class Value>
#line 132 "src/expression/Random.birch"
#line 132 "src/expression/Random.birch"
void Random_<Value>::move(const Value& x) {
  #line 133 "src/expression/Random.birch"
  this->x = x;
}

#line 143 "src/expression/Random.birch"
template<class Value>
#line 143 "src/expression/Random.birch"
#line 143 "src/expression/Random.birch"
void Random_<Value>::assume(const Distribution<Value>& p) {
  #line 144 "src/expression/Random.birch"
  assert(!(this->child.has_value()));
  #line 145 "src/expression/Random.birch"
  assert(!(this->p.has_value()));
  #line 146 "src/expression/Random.birch"
  p->graft(this->shared_from_this_());
  #line 147 "src/expression/Random.birch"
  this->p = p;
}

#line 150 "src/expression/Random.birch"
template<class Value>
#line 150 "src/expression/Random.birch"
#line 150 "src/expression/Random.birch"
void Random_<Value>::graft(const Delay& node) {
  #line 151 "src/expression/Random.birch"
  this->base_type_::graft(node);
  #line 152 "src/expression/Random.birch"
  p = std::nullopt;
}

#line 155 "src/expression/Random.birch"
template<class Value>
#line 155 "src/expression/Random.birch"
#line 155 "src/expression/Random.birch"
void Random_<Value>::graftReplace(const Delay& node) {
  #line 156 "src/expression/Random.birch"
  this->base_type_::graftReplace(node);
  #line 157 "src/expression/Random.birch"
  p = std::nullopt;
}

#line 160 "src/expression/Random.birch"
template<class Value>
#line 160 "src/expression/Random.birch"
#line 160 "src/expression/Random.birch"
Delay Random_<Value>::prune() {
  #line 161 "src/expression/Random.birch"
  if (this->child.has_value()) {
    #line 162 "src/expression/Random.birch"
    if (this->coparent.has_value() && this->n != Integer(1)) {
      #line 164 "src/expression/Random.birch"
      this->coparent.value()->prune();
    }
    #line 168 "src/expression/Random.birch"
    this->p = libbirch::cast<Distribution<Value>>(this->child.value()->prune());
    #line 169 "src/expression/Random.birch"
    assert(this->p.has_value());
    #line 170 "src/expression/Random.birch"
    this->child = std::nullopt;
    #line 172 "src/expression/Random.birch"
    if (this->coparent.has_value() && this->n == Integer(1)) {
      #line 174 "src/expression/Random.birch"
      this->coparent.value()->graftReplace(this->p.value());
      #line 175 "src/expression/Random.birch"
      this->p.value()->graft(this->shared_from_this_());
      #line 176 "src/expression/Random.birch"
      this->coparent.value()->coparent = std::nullopt;
      #line 177 "src/expression/Random.birch"
      this->coparent = std::nullopt;
    }
    #line 179 "src/expression/Random.birch"
    assert(this->p.has_value());
  }
  #line 181 "src/expression/Random.birch"
  assert(!(this->coparent.has_value()));
  #line 182 "src/expression/Random.birch"
  assert(!(this->child.has_value()));
  #line 183 "src/expression/Random.birch"
  return this->shared_from_this_();
}

#line 189 "src/expression/Random.birch"
template<class Value>
#line 189 "src/expression/Random.birch"
#line 189 "src/expression/Random.birch"
Boolean Random_<Value>::hasGradient() {
  #line 190 "src/expression/Random.birch"
  return this->d.has_value();
}

#line 197 "src/expression/Random.birch"
template<class Value>
#line 197 "src/expression/Random.birch"
#line 197 "src/expression/Random.birch"
Value Random_<Value>::getGradient() {
  #line 198 "src/expression/Random.birch"
  assert(this->d.has_value());
  #line 199 "src/expression/Random.birch"
  return this->d.value();
}

#line 205 "src/expression/Random.birch"
template<class Value>
#line 205 "src/expression/Random.birch"
#line 205 "src/expression/Random.birch"
Boolean Random_<Value>::hasDistribution() {
  #line 206 "src/expression/Random.birch"
  return this->p.has_value() || this->child.has_value();
}

#line 212 "src/expression/Random.birch"
template<class Value>
#line 212 "src/expression/Random.birch"
#line 212 "src/expression/Random.birch"
Distribution<Value> Random_<Value>::getDistribution() {
  #line 213 "src/expression/Random.birch"
  assert(this->p.has_value() || this->child.has_value());
  #line 214 "src/expression/Random.birch"
  prune();
  #line 215 "src/expression/Random.birch"
  assert(this->p.has_value());
  #line 216 "src/expression/Random.birch"
  return this->p.value();
}

#line 219 "src/expression/Random.birch"
template<class Value>
#line 219 "src/expression/Random.birch"
#line 219 "src/expression/Random.birch"
Boolean Random_<Value>::hasGaussian() {
  #line 220 "src/expression/Random.birch"
  return !(this->x.has_value()) && this->p.has_value() && this->p.value()->isGaussian();
}

#line 223 "src/expression/Random.birch"
template<class Value>
#line 223 "src/expression/Random.birch"
#line 223 "src/expression/Random.birch"
std::tuple<Expression<Real>, Expression<Real>> Random_<Value>::getGaussian() {
  #line 224 "src/expression/Random.birch"
  return this->p.value()->getGaussian().value();
}

#line 227 "src/expression/Random.birch"
template<class Value>
#line 227 "src/expression/Random.birch"
#line 227 "src/expression/Random.birch"
Boolean Random_<Value>::hasBeta() {
  #line 228 "src/expression/Random.birch"
  return !(this->x.has_value()) && this->p.has_value() && this->p.value()->isBeta();
}

#line 231 "src/expression/Random.birch"
template<class Value>
#line 231 "src/expression/Random.birch"
#line 231 "src/expression/Random.birch"
std::tuple<Expression<Real>, Expression<Real>> Random_<Value>::getBeta() {
  #line 232 "src/expression/Random.birch"
  return this->p.value()->getBeta().value();
}

#line 235 "src/expression/Random.birch"
template<class Value>
#line 235 "src/expression/Random.birch"
#line 235 "src/expression/Random.birch"
Boolean Random_<Value>::hasGamma() {
  #line 236 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isGamma();
}

#line 239 "src/expression/Random.birch"
template<class Value>
#line 239 "src/expression/Random.birch"
#line 239 "src/expression/Random.birch"
std::tuple<Expression<Real>, Expression<Real>> Random_<Value>::getGamma() {
  #line 240 "src/expression/Random.birch"
  return this->p.value()->getGamma().value();
}

#line 243 "src/expression/Random.birch"
template<class Value>
#line 243 "src/expression/Random.birch"
#line 243 "src/expression/Random.birch"
Boolean Random_<Value>::hasInverseGamma() {
  #line 244 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isInverseGamma();
}

#line 247 "src/expression/Random.birch"
template<class Value>
#line 247 "src/expression/Random.birch"
#line 247 "src/expression/Random.birch"
std::tuple<Expression<Real>, Expression<Real>> Random_<Value>::getInverseGamma() {
  #line 248 "src/expression/Random.birch"
  return this->p.value()->getInverseGamma().value();
}

#line 251 "src/expression/Random.birch"
template<class Value>
#line 251 "src/expression/Random.birch"
#line 251 "src/expression/Random.birch"
Boolean Random_<Value>::hasInverseWishart() {
  #line 252 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isInverseWishart();
}

#line 255 "src/expression/Random.birch"
template<class Value>
#line 255 "src/expression/Random.birch"
#line 255 "src/expression/Random.birch"
std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>> Random_<Value>::getInverseWishart() {
  #line 256 "src/expression/Random.birch"
  return this->p.value()->getInverseWishart().value();
}

#line 259 "src/expression/Random.birch"
template<class Value>
#line 259 "src/expression/Random.birch"
#line 259 "src/expression/Random.birch"
Boolean Random_<Value>::hasNormalInverseGamma() {
  #line 260 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isNormalInverseGamma();
}

#line 263 "src/expression/Random.birch"
template<class Value>
#line 263 "src/expression/Random.birch"
#line 263 "src/expression/Random.birch"
std::tuple<Expression<Real>, Expression<Real>, Expression<Real>, Expression<Real>> Random_<Value>::getNormalInverseGamma() {
  #line 265 "src/expression/Random.birch"
  return this->p.value()->getNormalInverseGamma().value();
}

#line 268 "src/expression/Random.birch"
template<class Value>
#line 268 "src/expression/Random.birch"
#line 268 "src/expression/Random.birch"
Boolean Random_<Value>::hasDirichlet() {
  #line 269 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isDirichlet();
}

#line 272 "src/expression/Random.birch"
template<class Value>
#line 272 "src/expression/Random.birch"
#line 272 "src/expression/Random.birch"
Expression<libbirch::DefaultArray<Real,1>> Random_<Value>::getDirichlet() {
  #line 273 "src/expression/Random.birch"
  return this->p.value()->getDirichlet().value();
}

#line 276 "src/expression/Random.birch"
template<class Value>
#line 276 "src/expression/Random.birch"
#line 276 "src/expression/Random.birch"
Boolean Random_<Value>::hasRestaurant() {
  #line 277 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isRestaurant();
}

#line 280 "src/expression/Random.birch"
template<class Value>
#line 280 "src/expression/Random.birch"
#line 280 "src/expression/Random.birch"
std::tuple<Expression<Real>, Expression<Real>, Expression<libbirch::DefaultArray<Integer,1>>, Expression<Integer>> Random_<Value>::getRestaurant() {
  #line 282 "src/expression/Random.birch"
  return this->p.value()->getRestaurant().value();
}

#line 285 "src/expression/Random.birch"
template<class Value>
#line 285 "src/expression/Random.birch"
#line 285 "src/expression/Random.birch"
Boolean Random_<Value>::hasMultivariateGaussian() {
  #line 286 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isMultivariateGaussian();
}

#line 289 "src/expression/Random.birch"
template<class Value>
#line 289 "src/expression/Random.birch"
#line 289 "src/expression/Random.birch"
std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>> Random_<Value>::getMultivariateGaussian() {
  #line 291 "src/expression/Random.birch"
  return this->p.value()->getMultivariateGaussian().value();
}

#line 294 "src/expression/Random.birch"
template<class Value>
#line 294 "src/expression/Random.birch"
#line 294 "src/expression/Random.birch"
Boolean Random_<Value>::hasMultivariateNormalInverseGamma() {
  #line 295 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isMultivariateNormalInverseGamma();
}

#line 298 "src/expression/Random.birch"
template<class Value>
#line 298 "src/expression/Random.birch"
#line 298 "src/expression/Random.birch"
std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>, Expression<Real>> Random_<Value>::getMultivariateNormalInverseGamma() {
  #line 300 "src/expression/Random.birch"
  return this->p.value()->getMultivariateNormalInverseGamma().value();
}

#line 303 "src/expression/Random.birch"
template<class Value>
#line 303 "src/expression/Random.birch"
#line 303 "src/expression/Random.birch"
Boolean Random_<Value>::hasMatrixGaussian() {
  #line 304 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isMatrixGaussian();
}

#line 307 "src/expression/Random.birch"
template<class Value>
#line 307 "src/expression/Random.birch"
#line 307 "src/expression/Random.birch"
std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>> Random_<Value>::getMatrixGaussian() {
  #line 309 "src/expression/Random.birch"
  return this->p.value()->getMatrixGaussian().value();
}

#line 312 "src/expression/Random.birch"
template<class Value>
#line 312 "src/expression/Random.birch"
#line 312 "src/expression/Random.birch"
Boolean Random_<Value>::hasMatrixNormalInverseWishart() {
  #line 313 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isMatrixNormalInverseWishart();
}

#line 316 "src/expression/Random.birch"
template<class Value>
#line 316 "src/expression/Random.birch"
#line 316 "src/expression/Random.birch"
std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>> Random_<Value>::getMatrixNormalInverseWishart() {
  #line 318 "src/expression/Random.birch"
  return this->p.value()->getMatrixNormalInverseWishart().value();
}

#line 321 "src/expression/Random.birch"
template<class Value>
#line 321 "src/expression/Random.birch"
#line 321 "src/expression/Random.birch"
Boolean Random_<Value>::hasDiscrete() {
  #line 322 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isDiscrete();
}

#line 325 "src/expression/Random.birch"
template<class Value>
#line 325 "src/expression/Random.birch"
#line 325 "src/expression/Random.birch"
DiscreteDistribution Random_<Value>::getDiscrete() {
  #line 326 "src/expression/Random.birch"
  return this->p.value()->getDiscrete().value();
}

#line 329 "src/expression/Random.birch"
template<class Value>
#line 329 "src/expression/Random.birch"
#line 329 "src/expression/Random.birch"
Boolean Random_<Value>::hasBoundedDiscrete() {
  #line 330 "src/expression/Random.birch"
  return !(x.has_value()) && this->p.has_value() && this->p.value()->isBoundedDiscrete();
}

#line 333 "src/expression/Random.birch"
template<class Value>
#line 333 "src/expression/Random.birch"
#line 333 "src/expression/Random.birch"
BoundedDiscreteDistribution Random_<Value>::getBoundedDiscrete() {
  #line 334 "src/expression/Random.birch"
  return this->p.value()->getBoundedDiscrete().value();
}

#line 337 "src/expression/Random.birch"
template<class Value>
#line 337 "src/expression/Random.birch"
#line 337 "src/expression/Random.birch"
void Random_<Value>::read(const Buffer& buffer) {
  #line 338 "src/expression/Random.birch"
    libbirch::optional_assign(this->shared_from_this_(), buffer->get<Value>());
;
}

#line 341 "src/expression/Random.birch"
template<class Value>
#line 341 "src/expression/Random.birch"
#line 341 "src/expression/Random.birch"
void Random_<Value>::write(const Buffer& buffer) {
  #line 342 "src/expression/Random.birch"
  if (hasValue() || hasDistribution()) {
    #line 343 "src/expression/Random.birch"
    buffer->set(value());
  } else {
    #line 345 "src/expression/Random.birch"
    buffer->setNil();
  }
}

#line 7 "src/expression/VectorSplitExpression.birch"
template<class Value,class Form>
#line 7 "src/expression/VectorSplitExpression.birch"
VectorSplitExpression_<Value, Form>::VectorSplitExpression_(const Value& x, const Form& f) :
    #line 7 "src/expression/VectorSplitExpression.birch"
    base_type_(x, f) {
  //
}

#line 9 "src/expression/VectorSplitExpression.birch"
template<class Value,class Form>
#line 9 "src/expression/VectorSplitExpression.birch"
#line 9 "src/expression/VectorSplitExpression.birch"
void VectorSplitExpression_<Value, Form>::grad(const Real& d, const Integer& i) {
  #line 10 "src/expression/VectorSplitExpression.birch"
  if (this->f.has_value()) {
    #line 11 "src/expression/VectorSplitExpression.birch"
    if (this->visitCount == Integer(0)) {
      #line 12 "src/expression/VectorSplitExpression.birch"
      assert(!(this->d.has_value()));
      #line 13 "src/expression/VectorSplitExpression.birch"
      this->d = vector(0.0, this->length());
    }
    #line 15 "src/expression/VectorSplitExpression.birch"
    assert(this->d.has_value());
    #line 16 "src/expression/VectorSplitExpression.birch"
    this->d.value()(i) = this->d.value()(i) + d;
    #line 17 "src/expression/VectorSplitExpression.birch"
    this->visitCount = this->visitCount + Integer(1);
    #line 18 "src/expression/VectorSplitExpression.birch"
    if (this->visitCount >= this->evalCount) {
      #line 19 "src/expression/VectorSplitExpression.birch"
      assert(this->visitCount == this->evalCount || this->evalCount == Integer(0));
      #line 24 "src/expression/VectorSplitExpression.birch"
      birch::grad(this->f.value(), this->d.value());
      #line 25 "src/expression/VectorSplitExpression.birch"
      this->d = std::nullopt;
      #line 26 "src/expression/VectorSplitExpression.birch"
      this->visitCount = Integer(0);
    }
  }
}

#line 4 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 4 "src/form/Binary.birch"
Binary_<Function, Left, Right>::Binary_(const Left& l, const Right& r) :
    #line 8 "src/form/Binary.birch"
    f(),
    #line 13 "src/form/Binary.birch"
    l(l),
    #line 18 "src/form/Binary.birch"
    r(r) {
  //
}

#line 20 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 20 "src/form/Binary.birch"
#line 20 "src/form/Binary.birch"
decltype(auto) Binary_<Function, Left, Right>::value() {
  #line 21 "src/form/Binary.birch"
  return f->compute(birch::value(l), birch::value(r));
}

#line 24 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 24 "src/form/Binary.birch"
#line 24 "src/form/Binary.birch"
decltype(auto) Binary_<Function, Left, Right>::peek() {
  #line 25 "src/form/Binary.birch"
  return f->compute(birch::peek(l), birch::peek(r));
}

#line 28 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 28 "src/form/Binary.birch"
#line 28 "src/form/Binary.birch"
decltype(auto) Binary_<Function, Left, Right>::eval() {
  #line 29 "src/form/Binary.birch"
  return f->compute(birch::eval(l), birch::eval(r));
}

#line 32 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 32 "src/form/Binary.birch"
#line 32 "src/form/Binary.birch"
decltype(auto) Binary_<Function, Left, Right>::reval() {
  #line 33 "src/form/Binary.birch"
  return f->compute(birch::reval(l), birch::reval(r));
}

#line 36 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 36 "src/form/Binary.birch"
template<class Gradient>
#line 36 "src/form/Binary.birch"
void Binary_<Function, Left, Right>::grad(const Gradient& d) {
  auto n4514_ = f->grad(d, birch::peek(l), birch::peek(r));
  auto dl = std::get<0>(n4514_);
  auto dr = std::get<1>(n4514_);
  #line 38 "src/form/Binary.birch"
  birch::grad(l, dl);
  #line 39 "src/form/Binary.birch"
  birch::grad(r, dr);
}

#line 42 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 42 "src/form/Binary.birch"
#line 42 "src/form/Binary.birch"
void Binary_<Function, Left, Right>::label(const Integer& gen) {
  #line 43 "src/form/Binary.birch"
  birch::label(l, gen);
  #line 44 "src/form/Binary.birch"
  birch::label(r, gen);
}

#line 47 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 47 "src/form/Binary.birch"
#line 47 "src/form/Binary.birch"
void Binary_<Function, Left, Right>::constant(const Integer& gen) {
  #line 48 "src/form/Binary.birch"
  birch::constant(l, gen);
  #line 49 "src/form/Binary.birch"
  birch::constant(r, gen);
}

#line 52 "src/form/Binary.birch"
template<class Function,class Left,class Right>
#line 52 "src/form/Binary.birch"
#line 52 "src/form/Binary.birch"
void Binary_<Function, Left, Right>::constant() {
  #line 53 "src/form/Binary.birch"
  birch::constant(l);
  #line 54 "src/form/Binary.birch"
  birch::constant(r);
}

#line 4 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 4 "src/form/BinaryMemoized.birch"
BinaryMemoized_<Function, Left, Right>::BinaryMemoized_(const Left& l, const Right& r) :
    #line 8 "src/form/BinaryMemoized.birch"
    f(),
    #line 13 "src/form/BinaryMemoized.birch"
    l(l),
    #line 18 "src/form/BinaryMemoized.birch"
    r(r) {
  //
}

#line 32 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 32 "src/form/BinaryMemoized.birch"
#line 32 "src/form/BinaryMemoized.birch"
decltype(auto) BinaryMemoized_<Function, Left, Right>::value() {
  #line 33 "src/form/BinaryMemoized.birch"
  if (!(x.has_value())) {
    #line 34 "src/form/BinaryMemoized.birch"
    x = f->compute(birch::value(l), birch::value(r));
  }
  #line 36 "src/form/BinaryMemoized.birch"
  return x.value();
}

#line 39 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 39 "src/form/BinaryMemoized.birch"
#line 39 "src/form/BinaryMemoized.birch"
decltype(auto) BinaryMemoized_<Function, Left, Right>::peek() {
  #line 40 "src/form/BinaryMemoized.birch"
  if (!(x.has_value())) {
    #line 41 "src/form/BinaryMemoized.birch"
    x = f->compute(birch::peek(l), birch::peek(r));
  }
  #line 43 "src/form/BinaryMemoized.birch"
  return x.value();
}

#line 46 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 46 "src/form/BinaryMemoized.birch"
#line 46 "src/form/BinaryMemoized.birch"
decltype(auto) BinaryMemoized_<Function, Left, Right>::eval() {
  #line 47 "src/form/BinaryMemoized.birch"
  x = std::nullopt;
  #line 48 "src/form/BinaryMemoized.birch"
  x = f->compute(birch::eval(l), birch::eval(r));
  #line 49 "src/form/BinaryMemoized.birch"
  return x.value();
}

#line 52 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 52 "src/form/BinaryMemoized.birch"
#line 52 "src/form/BinaryMemoized.birch"
decltype(auto) BinaryMemoized_<Function, Left, Right>::reval() {
  #line 53 "src/form/BinaryMemoized.birch"
  x = std::nullopt;
  #line 54 "src/form/BinaryMemoized.birch"
  x = f->compute(birch::reval(l), birch::reval(r));
  #line 55 "src/form/BinaryMemoized.birch"
  return x.value();
}

#line 58 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 58 "src/form/BinaryMemoized.birch"
template<class Gradient>
#line 58 "src/form/BinaryMemoized.birch"
void BinaryMemoized_<Function, Left, Right>::grad(const Gradient& d) {
  auto n4515_ = f->grad(d, birch::peek(l), birch::peek(r));
  auto dl = std::get<0>(n4515_);
  auto dr = std::get<1>(n4515_);
  #line 60 "src/form/BinaryMemoized.birch"
  birch::grad(l, dl);
  #line 61 "src/form/BinaryMemoized.birch"
  birch::grad(r, dr);
}

#line 64 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 64 "src/form/BinaryMemoized.birch"
#line 64 "src/form/BinaryMemoized.birch"
void BinaryMemoized_<Function, Left, Right>::label(const Integer& gen) {
  #line 65 "src/form/BinaryMemoized.birch"
  birch::label(l, gen);
  #line 66 "src/form/BinaryMemoized.birch"
  birch::label(r, gen);
}

#line 69 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 69 "src/form/BinaryMemoized.birch"
#line 69 "src/form/BinaryMemoized.birch"
void BinaryMemoized_<Function, Left, Right>::constant(const Integer& gen) {
  #line 70 "src/form/BinaryMemoized.birch"
  birch::constant(l, gen);
  #line 71 "src/form/BinaryMemoized.birch"
  birch::constant(r, gen);
}

#line 74 "src/form/BinaryMemoized.birch"
template<class Function,class Left,class Right>
#line 74 "src/form/BinaryMemoized.birch"
#line 74 "src/form/BinaryMemoized.birch"
void BinaryMemoized_<Function, Left, Right>::constant() {
  #line 75 "src/form/BinaryMemoized.birch"
  x = std::nullopt;
  #line 76 "src/form/BinaryMemoized.birch"
  birch::constant(l);
  #line 77 "src/form/BinaryMemoized.birch"
  birch::constant(r);
}

#line 5 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 5 "src/form/Linear.birch"
Linear_<Function, Left, Right>::Linear_(const Left& l, const Right& r) :
    #line 9 "src/form/Linear.birch"
    f(),
    #line 14 "src/form/Linear.birch"
    l(l),
    #line 19 "src/form/Linear.birch"
    r(r) {
  //
}

#line 21 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 21 "src/form/Linear.birch"
#line 21 "src/form/Linear.birch"
decltype(auto) Linear_<Function, Left, Right>::value() {
  #line 22 "src/form/Linear.birch"
  return f->compute(birch::value(l), birch::value(r));
}

#line 25 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 25 "src/form/Linear.birch"
#line 25 "src/form/Linear.birch"
decltype(auto) Linear_<Function, Left, Right>::peek() {
  #line 26 "src/form/Linear.birch"
  return f->compute(birch::peek(l), birch::peek(r));
}

#line 29 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 29 "src/form/Linear.birch"
#line 29 "src/form/Linear.birch"
decltype(auto) Linear_<Function, Left, Right>::eval() {
  #line 30 "src/form/Linear.birch"
  return f->compute(birch::eval(l), birch::eval(r));
}

#line 33 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 33 "src/form/Linear.birch"
#line 33 "src/form/Linear.birch"
decltype(auto) Linear_<Function, Left, Right>::reval() {
  #line 34 "src/form/Linear.birch"
  return f->compute(birch::reval(l), birch::reval(r));
}

#line 37 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 37 "src/form/Linear.birch"
template<class Gradient>
#line 37 "src/form/Linear.birch"
void Linear_<Function, Left, Right>::grad(const Gradient& d) {
  auto n4516_ = f->grad(d);
  auto dl = std::get<0>(n4516_);
  auto dr = std::get<1>(n4516_);
  #line 39 "src/form/Linear.birch"
  birch::grad(l, dl);
  #line 40 "src/form/Linear.birch"
  birch::grad(r, dr);
}

#line 43 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 43 "src/form/Linear.birch"
#line 43 "src/form/Linear.birch"
void Linear_<Function, Left, Right>::label(const Integer& gen) {
  #line 44 "src/form/Linear.birch"
  birch::label(l, gen);
  #line 45 "src/form/Linear.birch"
  birch::label(r, gen);
}

#line 48 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 48 "src/form/Linear.birch"
#line 48 "src/form/Linear.birch"
void Linear_<Function, Left, Right>::constant(const Integer& gen) {
  #line 49 "src/form/Linear.birch"
  birch::constant(l, gen);
  #line 50 "src/form/Linear.birch"
  birch::constant(r, gen);
}

#line 53 "src/form/Linear.birch"
template<class Function,class Left,class Right>
#line 53 "src/form/Linear.birch"
#line 53 "src/form/Linear.birch"
void Linear_<Function, Left, Right>::constant() {
  #line 54 "src/form/Linear.birch"
  birch::constant(l);
  #line 55 "src/form/Linear.birch"
  birch::constant(r);
}

#line 5 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 5 "src/form/LinearMemoized.birch"
LinearMemoized_<Function, Left, Right>::LinearMemoized_(const Left& l, const Right& r) :
    #line 9 "src/form/LinearMemoized.birch"
    f(),
    #line 14 "src/form/LinearMemoized.birch"
    l(l),
    #line 19 "src/form/LinearMemoized.birch"
    r(r) {
  //
}

#line 33 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 33 "src/form/LinearMemoized.birch"
#line 33 "src/form/LinearMemoized.birch"
decltype(auto) LinearMemoized_<Function, Left, Right>::value() {
  #line 34 "src/form/LinearMemoized.birch"
  if (!(x.has_value())) {
    #line 35 "src/form/LinearMemoized.birch"
    x = f->compute(birch::value(l), birch::value(r));
  }
  #line 37 "src/form/LinearMemoized.birch"
  return x.value();
}

#line 40 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 40 "src/form/LinearMemoized.birch"
#line 40 "src/form/LinearMemoized.birch"
decltype(auto) LinearMemoized_<Function, Left, Right>::peek() {
  #line 41 "src/form/LinearMemoized.birch"
  if (!(x.has_value())) {
    #line 42 "src/form/LinearMemoized.birch"
    x = f->compute(birch::peek(l), birch::peek(r));
  }
  #line 44 "src/form/LinearMemoized.birch"
  return x.value();
}

#line 47 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 47 "src/form/LinearMemoized.birch"
#line 47 "src/form/LinearMemoized.birch"
decltype(auto) LinearMemoized_<Function, Left, Right>::eval() {
  #line 48 "src/form/LinearMemoized.birch"
  x = std::nullopt;
  #line 49 "src/form/LinearMemoized.birch"
  x = f->compute(birch::eval(l), birch::eval(r));
  #line 50 "src/form/LinearMemoized.birch"
  return x.value();
}

#line 53 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 53 "src/form/LinearMemoized.birch"
#line 53 "src/form/LinearMemoized.birch"
decltype(auto) LinearMemoized_<Function, Left, Right>::reval() {
  #line 54 "src/form/LinearMemoized.birch"
  x = std::nullopt;
  #line 55 "src/form/LinearMemoized.birch"
  x = f->compute(birch::reval(l), birch::reval(r));
  #line 56 "src/form/LinearMemoized.birch"
  return x.value();
}

#line 59 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 59 "src/form/LinearMemoized.birch"
template<class Gradient>
#line 59 "src/form/LinearMemoized.birch"
void LinearMemoized_<Function, Left, Right>::grad(const Gradient& d) {
  auto n4517_ = f->grad(d);
  auto dl = std::get<0>(n4517_);
  auto dr = std::get<1>(n4517_);
  #line 61 "src/form/LinearMemoized.birch"
  birch::grad(l, dl);
  #line 62 "src/form/LinearMemoized.birch"
  birch::grad(r, dr);
}

#line 65 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 65 "src/form/LinearMemoized.birch"
#line 65 "src/form/LinearMemoized.birch"
void LinearMemoized_<Function, Left, Right>::label(const Integer& gen) {
  #line 66 "src/form/LinearMemoized.birch"
  birch::label(l, gen);
  #line 67 "src/form/LinearMemoized.birch"
  birch::label(r, gen);
}

#line 70 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 70 "src/form/LinearMemoized.birch"
#line 70 "src/form/LinearMemoized.birch"
void LinearMemoized_<Function, Left, Right>::constant(const Integer& gen) {
  #line 71 "src/form/LinearMemoized.birch"
  birch::constant(l, gen);
  #line 72 "src/form/LinearMemoized.birch"
  birch::constant(r, gen);
}

#line 75 "src/form/LinearMemoized.birch"
template<class Function,class Left,class Right>
#line 75 "src/form/LinearMemoized.birch"
#line 75 "src/form/LinearMemoized.birch"
void LinearMemoized_<Function, Left, Right>::constant() {
  #line 76 "src/form/LinearMemoized.birch"
  x = std::nullopt;
  #line 77 "src/form/LinearMemoized.birch"
  birch::constant(l);
  #line 78 "src/form/LinearMemoized.birch"
  birch::constant(r);
}

#line 4 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 4 "src/form/MatrixElement.birch"
MatrixElement_<Left, Middle, Right>::MatrixElement_(const Left& l, const Middle& m, const Right& r) :
    #line 8 "src/form/MatrixElement.birch"
    l(l),
    #line 13 "src/form/MatrixElement.birch"
    m(m),
    #line 18 "src/form/MatrixElement.birch"
    r(r) {
  //
}

#line 20 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 20 "src/form/MatrixElement.birch"
#line 20 "src/form/MatrixElement.birch"
decltype(auto) MatrixElement_<Left, Middle, Right>::value() {
  #line 21 "src/form/MatrixElement.birch"
  return birch::value(l)(birch::value(m), birch::value(r));
}

#line 24 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 24 "src/form/MatrixElement.birch"
#line 24 "src/form/MatrixElement.birch"
decltype(auto) MatrixElement_<Left, Middle, Right>::peek() {
  #line 25 "src/form/MatrixElement.birch"
  return birch::peek(l)(birch::peek(m), birch::peek(r));
}

#line 28 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 28 "src/form/MatrixElement.birch"
#line 28 "src/form/MatrixElement.birch"
decltype(auto) MatrixElement_<Left, Middle, Right>::eval() {
  #line 29 "src/form/MatrixElement.birch"
  return birch::eval(l)(birch::eval(m), birch::eval(r));
}

#line 32 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 32 "src/form/MatrixElement.birch"
#line 32 "src/form/MatrixElement.birch"
decltype(auto) MatrixElement_<Left, Middle, Right>::reval() {
  #line 33 "src/form/MatrixElement.birch"
  return birch::reval(l)(birch::reval(m), birch::reval(r));
}

#line 36 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 36 "src/form/MatrixElement.birch"
template<class Gradient>
#line 36 "src/form/MatrixElement.birch"
void MatrixElement_<Left, Middle, Right>::grad(const Gradient& d) {
  #line 37 "src/form/MatrixElement.birch"
  birch::grad(l, d, birch::peek(m), birch::peek(r));
}

#line 40 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 40 "src/form/MatrixElement.birch"
#line 40 "src/form/MatrixElement.birch"
void MatrixElement_<Left, Middle, Right>::label(const Integer& gen) {
  #line 41 "src/form/MatrixElement.birch"
  birch::label(l, gen);
  #line 42 "src/form/MatrixElement.birch"
  birch::label(m, gen);
  #line 43 "src/form/MatrixElement.birch"
  birch::label(r, gen);
}

#line 46 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 46 "src/form/MatrixElement.birch"
#line 46 "src/form/MatrixElement.birch"
void MatrixElement_<Left, Middle, Right>::constant(const Integer& gen) {
  #line 47 "src/form/MatrixElement.birch"
  birch::constant(l, gen);
  #line 48 "src/form/MatrixElement.birch"
  birch::constant(m, gen);
  #line 49 "src/form/MatrixElement.birch"
  birch::constant(r, gen);
}

#line 52 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 52 "src/form/MatrixElement.birch"
#line 52 "src/form/MatrixElement.birch"
void MatrixElement_<Left, Middle, Right>::constant() {
  #line 53 "src/form/MatrixElement.birch"
  birch::constant(l);
  #line 54 "src/form/MatrixElement.birch"
  birch::constant(m);
  #line 55 "src/form/MatrixElement.birch"
  birch::constant(r);
}

#line 6 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 6 "src/form/MatrixJoinForm.birch"
MatrixJoinForm_<Value>::MatrixJoinForm_(const libbirch::DefaultArray<Expression<Value>,2>& x) :
    #line 10 "src/form/MatrixJoinForm.birch"
    x(x) {
  //
}

#line 12 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 12 "src/form/MatrixJoinForm.birch"
#line 12 "src/form/MatrixJoinForm.birch"
libbirch::DefaultArray<Value,2> MatrixJoinForm_<Value>::value() {
  #line 13 "src/form/MatrixJoinForm.birch"
  return transform(x, [=](const Expression<Value>& x_prime_) {
    #line 14 "src/form/MatrixJoinForm.birch"
    return birch::value(x_prime_);
  });
}

#line 18 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 18 "src/form/MatrixJoinForm.birch"
#line 18 "src/form/MatrixJoinForm.birch"
libbirch::DefaultArray<Value,2> MatrixJoinForm_<Value>::peek() {
  #line 19 "src/form/MatrixJoinForm.birch"
  return transform(x, [=](const Expression<Value>& x_prime_) {
    #line 20 "src/form/MatrixJoinForm.birch"
    return birch::peek(x_prime_);
  });
}

#line 24 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 24 "src/form/MatrixJoinForm.birch"
#line 24 "src/form/MatrixJoinForm.birch"
libbirch::DefaultArray<Value,2> MatrixJoinForm_<Value>::eval() {
  #line 25 "src/form/MatrixJoinForm.birch"
  return transform(x, [=](const Expression<Value>& x_prime_) {
    #line 26 "src/form/MatrixJoinForm.birch"
    return birch::eval(x_prime_);
  });
}

#line 30 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 30 "src/form/MatrixJoinForm.birch"
#line 30 "src/form/MatrixJoinForm.birch"
libbirch::DefaultArray<Value,2> MatrixJoinForm_<Value>::reval() {
  #line 31 "src/form/MatrixJoinForm.birch"
  return transform(x, [=](const Expression<Value>& x_prime_) {
    #line 32 "src/form/MatrixJoinForm.birch"
    return birch::reval(x_prime_);
  });
}

#line 36 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 36 "src/form/MatrixJoinForm.birch"
#line 36 "src/form/MatrixJoinForm.birch"
void MatrixJoinForm_<Value>::grad(const libbirch::DefaultArray<Value,2>& d) {
  #line 37 "src/form/MatrixJoinForm.birch"
  for_each(x, d, [=](const Expression<Value>& x_prime_, const Value& d_prime_) {
    #line 38 "src/form/MatrixJoinForm.birch"
    birch::grad(x_prime_, d_prime_);
  });
}

#line 42 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 42 "src/form/MatrixJoinForm.birch"
#line 42 "src/form/MatrixJoinForm.birch"
void MatrixJoinForm_<Value>::label(const Integer& gen) {
  #line 43 "src/form/MatrixJoinForm.birch"
  for_each(x, [=](const Expression<Value>& x_prime_) {
    #line 43 "src/form/MatrixJoinForm.birch"
    birch::label(x_prime_, gen);
  });
}

#line 46 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 46 "src/form/MatrixJoinForm.birch"
#line 46 "src/form/MatrixJoinForm.birch"
void MatrixJoinForm_<Value>::constant(const Integer& gen) {
  #line 47 "src/form/MatrixJoinForm.birch"
  for_each(x, [=](const Expression<Value>& x_prime_) {
    #line 47 "src/form/MatrixJoinForm.birch"
    birch::constant(x_prime_, gen);
  });
}

#line 50 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 50 "src/form/MatrixJoinForm.birch"
#line 50 "src/form/MatrixJoinForm.birch"
void MatrixJoinForm_<Value>::constant() {
  #line 51 "src/form/MatrixJoinForm.birch"
  for_each(x, [=](const Expression<Value>& x_prime_) {
    #line 51 "src/form/MatrixJoinForm.birch"
    birch::constant(x_prime_);
  });
}

#line 4 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 4 "src/form/Ternary.birch"
Ternary_<Function, Left, Middle, Right>::Ternary_(const Left& l, const Middle& m, const Right& r) :
    #line 8 "src/form/Ternary.birch"
    f(),
    #line 13 "src/form/Ternary.birch"
    l(l),
    #line 18 "src/form/Ternary.birch"
    m(m),
    #line 23 "src/form/Ternary.birch"
    r(r) {
  //
}

#line 25 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 25 "src/form/Ternary.birch"
#line 25 "src/form/Ternary.birch"
decltype(auto) Ternary_<Function, Left, Middle, Right>::value() {
  #line 26 "src/form/Ternary.birch"
  return f->compute(birch::value(l), birch::value(m), birch::value(r));
}

#line 29 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 29 "src/form/Ternary.birch"
#line 29 "src/form/Ternary.birch"
decltype(auto) Ternary_<Function, Left, Middle, Right>::peek() {
  #line 30 "src/form/Ternary.birch"
  return f->compute(birch::peek(l), birch::peek(m), birch::peek(r));
}

#line 33 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 33 "src/form/Ternary.birch"
#line 33 "src/form/Ternary.birch"
decltype(auto) Ternary_<Function, Left, Middle, Right>::eval() {
  #line 34 "src/form/Ternary.birch"
  return f->compute(birch::eval(l), birch::eval(m), birch::eval(r));
}

#line 37 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 37 "src/form/Ternary.birch"
#line 37 "src/form/Ternary.birch"
decltype(auto) Ternary_<Function, Left, Middle, Right>::reval() {
  #line 38 "src/form/Ternary.birch"
  return f->compute(birch::reval(l), birch::reval(m), birch::reval(r));
}

#line 41 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 41 "src/form/Ternary.birch"
template<class Gradient>
#line 41 "src/form/Ternary.birch"
void Ternary_<Function, Left, Middle, Right>::grad(const Gradient& d) {
  auto n4518_ = f->grad(d, birch::peek(l), birch::peek(m), birch::peek(r));
  auto dl = std::get<0>(n4518_);
  auto dm = std::get<1>(n4518_);
  auto dr = std::get<2>(n4518_);
  #line 44 "src/form/Ternary.birch"
  birch::grad(l, dl);
  #line 45 "src/form/Ternary.birch"
  birch::grad(m, dm);
  #line 46 "src/form/Ternary.birch"
  birch::grad(r, dr);
}

#line 49 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 49 "src/form/Ternary.birch"
#line 49 "src/form/Ternary.birch"
void Ternary_<Function, Left, Middle, Right>::label(const Integer& gen) {
  #line 50 "src/form/Ternary.birch"
  birch::label(l, gen);
  #line 51 "src/form/Ternary.birch"
  birch::label(m, gen);
  #line 52 "src/form/Ternary.birch"
  birch::label(r, gen);
}

#line 55 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 55 "src/form/Ternary.birch"
#line 55 "src/form/Ternary.birch"
void Ternary_<Function, Left, Middle, Right>::constant(const Integer& gen) {
  #line 56 "src/form/Ternary.birch"
  birch::constant(l, gen);
  #line 57 "src/form/Ternary.birch"
  birch::constant(m, gen);
  #line 58 "src/form/Ternary.birch"
  birch::constant(r, gen);
}

#line 61 "src/form/Ternary.birch"
template<class Function,class Left,class Middle,class Right>
#line 61 "src/form/Ternary.birch"
#line 61 "src/form/Ternary.birch"
void Ternary_<Function, Left, Middle, Right>::constant() {
  #line 62 "src/form/Ternary.birch"
  birch::constant(l);
  #line 63 "src/form/Ternary.birch"
  birch::constant(m);
  #line 64 "src/form/Ternary.birch"
  birch::constant(r);
}

#line 4 "src/form/Unary.birch"
template<class Function,class Middle>
#line 4 "src/form/Unary.birch"
Unary_<Function, Middle>::Unary_(const Middle& m) :
    #line 8 "src/form/Unary.birch"
    f(),
    #line 13 "src/form/Unary.birch"
    m(m) {
  //
}

#line 15 "src/form/Unary.birch"
template<class Function,class Middle>
#line 15 "src/form/Unary.birch"
#line 15 "src/form/Unary.birch"
decltype(auto) Unary_<Function, Middle>::value() {
  #line 16 "src/form/Unary.birch"
  return f->compute(birch::value(m));
}

#line 19 "src/form/Unary.birch"
template<class Function,class Middle>
#line 19 "src/form/Unary.birch"
#line 19 "src/form/Unary.birch"
decltype(auto) Unary_<Function, Middle>::peek() {
  #line 20 "src/form/Unary.birch"
  return f->compute(birch::peek(m));
}

#line 23 "src/form/Unary.birch"
template<class Function,class Middle>
#line 23 "src/form/Unary.birch"
#line 23 "src/form/Unary.birch"
decltype(auto) Unary_<Function, Middle>::eval() {
  #line 24 "src/form/Unary.birch"
  return f->compute(birch::eval(m));
}

#line 27 "src/form/Unary.birch"
template<class Function,class Middle>
#line 27 "src/form/Unary.birch"
#line 27 "src/form/Unary.birch"
decltype(auto) Unary_<Function, Middle>::reval() {
  #line 28 "src/form/Unary.birch"
  return f->compute(birch::reval(m));
}

#line 31 "src/form/Unary.birch"
template<class Function,class Middle>
#line 31 "src/form/Unary.birch"
template<class Gradient>
#line 31 "src/form/Unary.birch"
void Unary_<Function, Middle>::grad(const Gradient& d) {
  #line 32 "src/form/Unary.birch"
  birch::grad(m, f->grad(d, birch::peek(m)));
}

#line 35 "src/form/Unary.birch"
template<class Function,class Middle>
#line 35 "src/form/Unary.birch"
#line 35 "src/form/Unary.birch"
void Unary_<Function, Middle>::label(const Integer& gen) {
  #line 36 "src/form/Unary.birch"
  birch::label(m, gen);
}

#line 39 "src/form/Unary.birch"
template<class Function,class Middle>
#line 39 "src/form/Unary.birch"
#line 39 "src/form/Unary.birch"
void Unary_<Function, Middle>::constant(const Integer& gen) {
  #line 40 "src/form/Unary.birch"
  birch::constant(m, gen);
}

#line 43 "src/form/Unary.birch"
template<class Function,class Middle>
#line 43 "src/form/Unary.birch"
#line 43 "src/form/Unary.birch"
void Unary_<Function, Middle>::constant() {
  #line 44 "src/form/Unary.birch"
  birch::constant(m);
}

#line 4 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 4 "src/form/UnaryMemoized.birch"
UnaryMemoized_<Function, Middle>::UnaryMemoized_(const Middle& m) :
    #line 8 "src/form/UnaryMemoized.birch"
    f(),
    #line 13 "src/form/UnaryMemoized.birch"
    m(m) {
  //
}

#line 27 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 27 "src/form/UnaryMemoized.birch"
#line 27 "src/form/UnaryMemoized.birch"
decltype(auto) UnaryMemoized_<Function, Middle>::value() {
  #line 28 "src/form/UnaryMemoized.birch"
  if (!(x.has_value())) {
    #line 29 "src/form/UnaryMemoized.birch"
    x = f->compute(birch::value(m));
  }
  #line 31 "src/form/UnaryMemoized.birch"
  return x.value();
}

#line 34 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 34 "src/form/UnaryMemoized.birch"
#line 34 "src/form/UnaryMemoized.birch"
decltype(auto) UnaryMemoized_<Function, Middle>::peek() {
  #line 35 "src/form/UnaryMemoized.birch"
  if (!(x.has_value())) {
    #line 36 "src/form/UnaryMemoized.birch"
    x = f->compute(birch::peek(m));
  }
  #line 38 "src/form/UnaryMemoized.birch"
  return x.value();
}

#line 41 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 41 "src/form/UnaryMemoized.birch"
#line 41 "src/form/UnaryMemoized.birch"
decltype(auto) UnaryMemoized_<Function, Middle>::eval() {
  #line 42 "src/form/UnaryMemoized.birch"
  x = std::nullopt;
  #line 43 "src/form/UnaryMemoized.birch"
  x = f->compute(birch::eval(m));
  #line 44 "src/form/UnaryMemoized.birch"
  return x.value();
}

#line 47 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 47 "src/form/UnaryMemoized.birch"
#line 47 "src/form/UnaryMemoized.birch"
decltype(auto) UnaryMemoized_<Function, Middle>::reval() {
  #line 48 "src/form/UnaryMemoized.birch"
  x = std::nullopt;
  #line 49 "src/form/UnaryMemoized.birch"
  x = f->compute(birch::reval(m));
  #line 50 "src/form/UnaryMemoized.birch"
  return x.value();
}

#line 53 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 53 "src/form/UnaryMemoized.birch"
template<class Gradient>
#line 53 "src/form/UnaryMemoized.birch"
void UnaryMemoized_<Function, Middle>::grad(const Gradient& d) {
  #line 54 "src/form/UnaryMemoized.birch"
  birch::grad(m, f->grad(d, birch::peek(m)));
}

#line 57 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 57 "src/form/UnaryMemoized.birch"
#line 57 "src/form/UnaryMemoized.birch"
void UnaryMemoized_<Function, Middle>::label(const Integer& gen) {
  #line 58 "src/form/UnaryMemoized.birch"
  birch::label(m, gen);
}

#line 61 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 61 "src/form/UnaryMemoized.birch"
#line 61 "src/form/UnaryMemoized.birch"
void UnaryMemoized_<Function, Middle>::constant(const Integer& gen) {
  #line 62 "src/form/UnaryMemoized.birch"
  birch::constant(m, gen);
}

#line 65 "src/form/UnaryMemoized.birch"
template<class Function,class Middle>
#line 65 "src/form/UnaryMemoized.birch"
#line 65 "src/form/UnaryMemoized.birch"
void UnaryMemoized_<Function, Middle>::constant() {
  #line 66 "src/form/UnaryMemoized.birch"
  x = std::nullopt;
  #line 67 "src/form/UnaryMemoized.birch"
  birch::constant(m);
}

#line 4 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 4 "src/form/VectorElement.birch"
VectorElement_<Left, Right>::VectorElement_(const Left& l, const Right& r) :
    #line 8 "src/form/VectorElement.birch"
    l(l),
    #line 13 "src/form/VectorElement.birch"
    r(r) {
  //
}

#line 15 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 15 "src/form/VectorElement.birch"
#line 15 "src/form/VectorElement.birch"
decltype(auto) VectorElement_<Left, Right>::value() {
  #line 16 "src/form/VectorElement.birch"
  return birch::value(l)(birch::value(r));
}

#line 19 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 19 "src/form/VectorElement.birch"
#line 19 "src/form/VectorElement.birch"
decltype(auto) VectorElement_<Left, Right>::peek() {
  #line 20 "src/form/VectorElement.birch"
  return birch::peek(l)(birch::peek(r));
}

#line 23 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 23 "src/form/VectorElement.birch"
#line 23 "src/form/VectorElement.birch"
decltype(auto) VectorElement_<Left, Right>::eval() {
  #line 24 "src/form/VectorElement.birch"
  return birch::eval(l)(birch::eval(r));
}

#line 27 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 27 "src/form/VectorElement.birch"
#line 27 "src/form/VectorElement.birch"
decltype(auto) VectorElement_<Left, Right>::reval() {
  #line 28 "src/form/VectorElement.birch"
  return birch::reval(l)(birch::reval(r));
}

#line 31 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 31 "src/form/VectorElement.birch"
template<class Gradient>
#line 31 "src/form/VectorElement.birch"
void VectorElement_<Left, Right>::grad(const Gradient& d) {
  #line 32 "src/form/VectorElement.birch"
  birch::grad(l, d, birch::peek(r));
}

#line 35 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 35 "src/form/VectorElement.birch"
#line 35 "src/form/VectorElement.birch"
void VectorElement_<Left, Right>::label(const Integer& gen) {
  #line 36 "src/form/VectorElement.birch"
  birch::label(l, gen);
  #line 37 "src/form/VectorElement.birch"
  birch::label(r, gen);
}

#line 40 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 40 "src/form/VectorElement.birch"
#line 40 "src/form/VectorElement.birch"
void VectorElement_<Left, Right>::constant(const Integer& gen) {
  #line 41 "src/form/VectorElement.birch"
  birch::constant(l, gen);
  #line 42 "src/form/VectorElement.birch"
  birch::constant(r, gen);
}

#line 45 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 45 "src/form/VectorElement.birch"
#line 45 "src/form/VectorElement.birch"
void VectorElement_<Left, Right>::constant() {
  #line 46 "src/form/VectorElement.birch"
  birch::constant(l);
  #line 47 "src/form/VectorElement.birch"
  birch::constant(r);
}

#line 6 "src/form/VectorJoinForm.birch"
template<class Value>
#line 6 "src/form/VectorJoinForm.birch"
VectorJoinForm_<Value>::VectorJoinForm_(const libbirch::DefaultArray<Expression<Value>,1>& x) :
    #line 10 "src/form/VectorJoinForm.birch"
    x(x) {
  //
}

#line 12 "src/form/VectorJoinForm.birch"
template<class Value>
#line 12 "src/form/VectorJoinForm.birch"
#line 12 "src/form/VectorJoinForm.birch"
libbirch::DefaultArray<Value,1> VectorJoinForm_<Value>::value() {
  #line 13 "src/form/VectorJoinForm.birch"
  return transform(x, [=](const Expression<Value>& x_prime_) {
    #line 14 "src/form/VectorJoinForm.birch"
    return birch::value(x_prime_);
  });
}

#line 18 "src/form/VectorJoinForm.birch"
template<class Value>
#line 18 "src/form/VectorJoinForm.birch"
#line 18 "src/form/VectorJoinForm.birch"
libbirch::DefaultArray<Value,1> VectorJoinForm_<Value>::peek() {
  #line 19 "src/form/VectorJoinForm.birch"
  return transform(x, [=](const Expression<Value>& x_prime_) {
    #line 20 "src/form/VectorJoinForm.birch"
    return birch::peek(x_prime_);
  });
}

#line 24 "src/form/VectorJoinForm.birch"
template<class Value>
#line 24 "src/form/VectorJoinForm.birch"
#line 24 "src/form/VectorJoinForm.birch"
libbirch::DefaultArray<Value,1> VectorJoinForm_<Value>::eval() {
  #line 25 "src/form/VectorJoinForm.birch"
  return transform(x, [=](const Expression<Value>& x_prime_) {
    #line 26 "src/form/VectorJoinForm.birch"
    return birch::eval(x_prime_);
  });
}

#line 30 "src/form/VectorJoinForm.birch"
template<class Value>
#line 30 "src/form/VectorJoinForm.birch"
#line 30 "src/form/VectorJoinForm.birch"
libbirch::DefaultArray<Value,1> VectorJoinForm_<Value>::reval() {
  #line 31 "src/form/VectorJoinForm.birch"
  return transform(x, [=](const Expression<Value>& x_prime_) {
    #line 32 "src/form/VectorJoinForm.birch"
    return birch::reval(x_prime_);
  });
}

#line 36 "src/form/VectorJoinForm.birch"
template<class Value>
#line 36 "src/form/VectorJoinForm.birch"
#line 36 "src/form/VectorJoinForm.birch"
void VectorJoinForm_<Value>::grad(const libbirch::DefaultArray<Value,1>& d) {
  #line 37 "src/form/VectorJoinForm.birch"
  for_each(x, d, [=](const Expression<Value>& x_prime_, const Value& d_prime_) {
    #line 38 "src/form/VectorJoinForm.birch"
    birch::grad(x_prime_, d_prime_);
  });
}

#line 42 "src/form/VectorJoinForm.birch"
template<class Value>
#line 42 "src/form/VectorJoinForm.birch"
#line 42 "src/form/VectorJoinForm.birch"
void VectorJoinForm_<Value>::label(const Integer& gen) {
  #line 43 "src/form/VectorJoinForm.birch"
  for_each(x, [=](const Expression<Value>& x_prime_) {
    #line 43 "src/form/VectorJoinForm.birch"
    birch::label(x_prime_, gen);
  });
}

#line 46 "src/form/VectorJoinForm.birch"
template<class Value>
#line 46 "src/form/VectorJoinForm.birch"
#line 46 "src/form/VectorJoinForm.birch"
void VectorJoinForm_<Value>::constant(const Integer& gen) {
  #line 47 "src/form/VectorJoinForm.birch"
  for_each(x, [=](const Expression<Value>& x_prime_) {
    #line 47 "src/form/VectorJoinForm.birch"
    birch::constant(x_prime_, gen);
  });
}

#line 50 "src/form/VectorJoinForm.birch"
template<class Value>
#line 50 "src/form/VectorJoinForm.birch"
#line 50 "src/form/VectorJoinForm.birch"
void VectorJoinForm_<Value>::constant() {
  #line 51 "src/form/VectorJoinForm.birch"
  for_each(x, [=](const Expression<Value>& x_prime_) {
    #line 51 "src/form/VectorJoinForm.birch"
    birch::constant(x_prime_);
  });
}

#line 6 "src/iterator/ArrayIterator.birch"
template<class Type>
#line 6 "src/iterator/ArrayIterator.birch"
ArrayIterator_<Type>::ArrayIterator_(const Array<Type>& o) :
    #line 6 "src/iterator/ArrayIterator.birch"
    base_type_(),
    #line 10 "src/iterator/ArrayIterator.birch"
    o(o),
    #line 15 "src/iterator/ArrayIterator.birch"
    i(Integer(0)) {
  //
}

#line 20 "src/iterator/ArrayIterator.birch"
template<class Type>
#line 20 "src/iterator/ArrayIterator.birch"
#line 20 "src/iterator/ArrayIterator.birch"
Boolean ArrayIterator_<Type>::hasNext() {
  #line 21 "src/iterator/ArrayIterator.birch"
  return i < o->size();
}

#line 27 "src/iterator/ArrayIterator.birch"
template<class Type>
#line 27 "src/iterator/ArrayIterator.birch"
#line 27 "src/iterator/ArrayIterator.birch"
Type ArrayIterator_<Type>::next() {
  #line 28 "src/iterator/ArrayIterator.birch"
  i = i + Integer(1);
  #line 29 "src/iterator/ArrayIterator.birch"
  return o->get(i);
}

#line 4 "src/iterator/EmptyIterator.birch"
template<class Type>
#line 4 "src/iterator/EmptyIterator.birch"
EmptyIterator_<Type>::EmptyIterator_() :
    #line 4 "src/iterator/EmptyIterator.birch"
    base_type_() {
  //
}

#line 8 "src/iterator/EmptyIterator.birch"
template<class Type>
#line 8 "src/iterator/EmptyIterator.birch"
#line 8 "src/iterator/EmptyIterator.birch"
Boolean EmptyIterator_<Type>::hasNext() {
  #line 9 "src/iterator/EmptyIterator.birch"
  return false;
}

#line 15 "src/iterator/EmptyIterator.birch"
template<class Type>
#line 15 "src/iterator/EmptyIterator.birch"
#line 15 "src/iterator/EmptyIterator.birch"
Type EmptyIterator_<Type>::next() {
  #line 16 "src/iterator/EmptyIterator.birch"
  assert(false);
  #line 17 "src/iterator/EmptyIterator.birch"
  return construct<Type>();
}

#line 4 "src/iterator/Iterator.birch"
template<class Type>
#line 4 "src/iterator/Iterator.birch"
Iterator_<Type>::Iterator_() :
    #line 4 "src/iterator/Iterator.birch"
    base_type_() {
  //
}

#line 6 "src/iterator/ListIterator.birch"
template<class Type>
#line 6 "src/iterator/ListIterator.birch"
ListIterator_<Type>::ListIterator_(const std::optional<ListNode<Type>>& head) :
    #line 6 "src/iterator/ListIterator.birch"
    base_type_(),
    #line 10 "src/iterator/ListIterator.birch"
    node(head) {
  //
}

#line 15 "src/iterator/ListIterator.birch"
template<class Type>
#line 15 "src/iterator/ListIterator.birch"
#line 15 "src/iterator/ListIterator.birch"
Boolean ListIterator_<Type>::hasNext() {
  #line 16 "src/iterator/ListIterator.birch"
  return node.has_value();
}

#line 22 "src/iterator/ListIterator.birch"
template<class Type>
#line 22 "src/iterator/ListIterator.birch"
#line 22 "src/iterator/ListIterator.birch"
Type ListIterator_<Type>::next() {
  auto x = node.value()->x;
  #line 24 "src/iterator/ListIterator.birch"
  node = node.value()->next;
  #line 25 "src/iterator/ListIterator.birch"
  return x;
}

#line 7 "src/iterator/MatrixBufferIterator.birch"
template<class Type>
#line 7 "src/iterator/MatrixBufferIterator.birch"
MatrixBufferIterator_<Type>::MatrixBufferIterator_(const libbirch::DefaultArray<Type,2>& values) :
    #line 7 "src/iterator/MatrixBufferIterator.birch"
    base_type_(),
    #line 11 "src/iterator/MatrixBufferIterator.birch"
    values(values),
    #line 16 "src/iterator/MatrixBufferIterator.birch"
    i(Integer(0)) {
  //
}

#line 21 "src/iterator/MatrixBufferIterator.birch"
template<class Type>
#line 21 "src/iterator/MatrixBufferIterator.birch"
#line 21 "src/iterator/MatrixBufferIterator.birch"
Boolean MatrixBufferIterator_<Type>::hasNext() {
  #line 22 "src/iterator/MatrixBufferIterator.birch"
  return i < rows(this->values);
}

#line 28 "src/iterator/MatrixBufferIterator.birch"
template<class Type>
#line 28 "src/iterator/MatrixBufferIterator.birch"
#line 28 "src/iterator/MatrixBufferIterator.birch"
Buffer MatrixBufferIterator_<Type>::next() {
  #line 29 "src/iterator/MatrixBufferIterator.birch"
  this->i = this->i + Integer(1);
  #line 30 "src/iterator/MatrixBufferIterator.birch"
  return make_buffer(row(this->values, this->i));
}

#line 7 "src/iterator/RaggedArrayElementIterator.birch"
template<class Type>
#line 7 "src/iterator/RaggedArrayElementIterator.birch"
RaggedArrayElementIterator_<Type>::RaggedArrayElementIterator_(const RaggedArray<Type>& o, const Integer& i) :
    #line 7 "src/iterator/RaggedArrayElementIterator.birch"
    base_type_(),
    #line 12 "src/iterator/RaggedArrayElementIterator.birch"
    o(o),
    #line 17 "src/iterator/RaggedArrayElementIterator.birch"
    i(i),
    #line 22 "src/iterator/RaggedArrayElementIterator.birch"
    j(Integer(0)) {
  //
}

#line 27 "src/iterator/RaggedArrayElementIterator.birch"
template<class Type>
#line 27 "src/iterator/RaggedArrayElementIterator.birch"
#line 27 "src/iterator/RaggedArrayElementIterator.birch"
Boolean RaggedArrayElementIterator_<Type>::hasNext() {
  #line 28 "src/iterator/RaggedArrayElementIterator.birch"
  return j < o->size(i);
}

#line 34 "src/iterator/RaggedArrayElementIterator.birch"
template<class Type>
#line 34 "src/iterator/RaggedArrayElementIterator.birch"
#line 34 "src/iterator/RaggedArrayElementIterator.birch"
Type RaggedArrayElementIterator_<Type>::next() {
  #line 35 "src/iterator/RaggedArrayElementIterator.birch"
  j = j + Integer(1);
  #line 36 "src/iterator/RaggedArrayElementIterator.birch"
  return o->get(i, j);
}

#line 6 "src/iterator/RaggedArrayRowIterator.birch"
template<class Type>
#line 6 "src/iterator/RaggedArrayRowIterator.birch"
RaggedArrayRowIterator_<Type>::RaggedArrayRowIterator_(const RaggedArray<Type>& o) :
    #line 6 "src/iterator/RaggedArrayRowIterator.birch"
    base_type_(),
    #line 11 "src/iterator/RaggedArrayRowIterator.birch"
    o(o),
    #line 16 "src/iterator/RaggedArrayRowIterator.birch"
    i(Integer(0)) {
  //
}

#line 21 "src/iterator/RaggedArrayRowIterator.birch"
template<class Type>
#line 21 "src/iterator/RaggedArrayRowIterator.birch"
#line 21 "src/iterator/RaggedArrayRowIterator.birch"
Boolean RaggedArrayRowIterator_<Type>::hasNext() {
  #line 22 "src/iterator/RaggedArrayRowIterator.birch"
  return i < o->size();
}

#line 28 "src/iterator/RaggedArrayRowIterator.birch"
template<class Type>
#line 28 "src/iterator/RaggedArrayRowIterator.birch"
#line 28 "src/iterator/RaggedArrayRowIterator.birch"
libbirch::DefaultArray<Type,1> RaggedArrayRowIterator_<Type>::next() {
  #line 29 "src/iterator/RaggedArrayRowIterator.birch"
  i = i + Integer(1);
  #line 30 "src/iterator/RaggedArrayRowIterator.birch"
  return o->get(i);
}

#line 6 "src/iterator/ScalarBufferIterator.birch"
template<class Type>
#line 6 "src/iterator/ScalarBufferIterator.birch"
ScalarBufferIterator_<Type>::ScalarBufferIterator_(const Type& x) :
    #line 6 "src/iterator/ScalarBufferIterator.birch"
    base_type_(),
    #line 10 "src/iterator/ScalarBufferIterator.birch"
    x(x) {
  //
}

#line 12 "src/iterator/ScalarBufferIterator.birch"
template<class Type>
#line 12 "src/iterator/ScalarBufferIterator.birch"
#line 12 "src/iterator/ScalarBufferIterator.birch"
Boolean ScalarBufferIterator_<Type>::hasNext() {
  #line 13 "src/iterator/ScalarBufferIterator.birch"
  return this->x.has_value();
}

#line 16 "src/iterator/ScalarBufferIterator.birch"
template<class Type>
#line 16 "src/iterator/ScalarBufferIterator.birch"
#line 16 "src/iterator/ScalarBufferIterator.birch"
Buffer ScalarBufferIterator_<Type>::next() {
  auto x_prime_ = this->x.value();
  #line 18 "src/iterator/ScalarBufferIterator.birch"
  this->x = std::nullopt;
  #line 19 "src/iterator/ScalarBufferIterator.birch"
  return make_buffer(x_prime_);
}

#line 6 "src/iterator/StackIterator.birch"
template<class Type>
#line 6 "src/iterator/StackIterator.birch"
StackIterator_<Type>::StackIterator_(const std::optional<StackNode<Type>>& head) :
    #line 6 "src/iterator/StackIterator.birch"
    base_type_(),
    #line 10 "src/iterator/StackIterator.birch"
    node(head) {
  //
}

#line 15 "src/iterator/StackIterator.birch"
template<class Type>
#line 15 "src/iterator/StackIterator.birch"
#line 15 "src/iterator/StackIterator.birch"
Boolean StackIterator_<Type>::hasNext() {
  #line 16 "src/iterator/StackIterator.birch"
  return node.has_value();
}

#line 22 "src/iterator/StackIterator.birch"
template<class Type>
#line 22 "src/iterator/StackIterator.birch"
#line 22 "src/iterator/StackIterator.birch"
Type StackIterator_<Type>::next() {
  auto x = node.value()->x;
  #line 24 "src/iterator/StackIterator.birch"
  node = node.value()->next;
  #line 25 "src/iterator/StackIterator.birch"
  return x;
}

#line 6 "src/iterator/TapeIterator.birch"
template<class Type>
#line 6 "src/iterator/TapeIterator.birch"
TapeIterator_<Type>::TapeIterator_(const std::optional<TapeNode<Type>>& head) :
    #line 6 "src/iterator/TapeIterator.birch"
    base_type_(),
    #line 10 "src/iterator/TapeIterator.birch"
    node(head) {
  //
}

#line 15 "src/iterator/TapeIterator.birch"
template<class Type>
#line 15 "src/iterator/TapeIterator.birch"
#line 15 "src/iterator/TapeIterator.birch"
Boolean TapeIterator_<Type>::hasNext() {
  #line 16 "src/iterator/TapeIterator.birch"
  return node.has_value();
}

#line 22 "src/iterator/TapeIterator.birch"
template<class Type>
#line 22 "src/iterator/TapeIterator.birch"
#line 22 "src/iterator/TapeIterator.birch"
Type TapeIterator_<Type>::next() {
  auto x = node.value()->x;
  #line 24 "src/iterator/TapeIterator.birch"
  node = node.value()->next;
  #line 25 "src/iterator/TapeIterator.birch"
  return x;
}

#line 7 "src/iterator/VectorBufferIterator.birch"
template<class Type>
#line 7 "src/iterator/VectorBufferIterator.birch"
VectorBufferIterator_<Type>::VectorBufferIterator_(const libbirch::DefaultArray<Type,1>& values) :
    #line 7 "src/iterator/VectorBufferIterator.birch"
    base_type_(),
    #line 11 "src/iterator/VectorBufferIterator.birch"
    values(values),
    #line 16 "src/iterator/VectorBufferIterator.birch"
    i(Integer(0)) {
  //
}

#line 21 "src/iterator/VectorBufferIterator.birch"
template<class Type>
#line 21 "src/iterator/VectorBufferIterator.birch"
#line 21 "src/iterator/VectorBufferIterator.birch"
Boolean VectorBufferIterator_<Type>::hasNext() {
  #line 22 "src/iterator/VectorBufferIterator.birch"
  return i < length(values);
}

#line 28 "src/iterator/VectorBufferIterator.birch"
template<class Type>
#line 28 "src/iterator/VectorBufferIterator.birch"
#line 28 "src/iterator/VectorBufferIterator.birch"
Buffer VectorBufferIterator_<Type>::next() {
  #line 29 "src/iterator/VectorBufferIterator.birch"
  this->i = this->i + Integer(1);
  #line 30 "src/iterator/VectorBufferIterator.birch"
  return make_buffer(this->values(this->i));
}

#line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
MatrixGaussianDistribution_<Arg1, Arg2, Arg3>::MatrixGaussianDistribution_(const Arg1& M, const Arg2& U, const Arg3& V) :
    #line 1 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
    M(M),
    #line 14 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
    U(U),
    #line 19 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
    V(V) {
  //
}

#line 21 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 21 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
#line 21 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
Boolean MatrixGaussianDistribution_<Arg1, Arg2, Arg3>::supportsLazy() {
  #line 22 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  return true;
}

#line 25 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 25 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
#line 25 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
libbirch::DefaultArray<Real,2> MatrixGaussianDistribution_<Arg1, Arg2, Arg3>::simulate() {
  #line 26 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  return simulate_matrix_gaussian(value(M), value(U), value(V));
}

#line 29 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 29 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
#line 29 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,2>> MatrixGaussianDistribution_<Arg1, Arg2, Arg3>::simulateLazy() {
  #line 30 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  return simulate_matrix_gaussian(peek(M), peek(U), peek(V));
}

#line 33 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 33 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
#line 33 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
Real MatrixGaussianDistribution_<Arg1, Arg2, Arg3>::logpdf(const libbirch::DefaultArray<Real,2>& X) {
  #line 34 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  return logpdf_matrix_gaussian(X, value(M), value(U), value(V));
}

#line 37 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 37 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
#line 37 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
std::optional<Expression<Real>> MatrixGaussianDistribution_<Arg1, Arg2, Arg3>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 38 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  return box(logpdf_matrix_gaussian(X, M, U, V));
}

#line 41 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 41 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
#line 41 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
Boolean MatrixGaussianDistribution_<Arg1, Arg2, Arg3>::isMatrixGaussian() {
  #line 42 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  return true;
}

#line 45 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 45 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
#line 45 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>>> MatrixGaussianDistribution_<Arg1, Arg2, Arg3>::getMatrixGaussian() {
  #line 47 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  return box(M, U, V);
}

#line 50 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 50 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
#line 50 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
void MatrixGaussianDistribution_<Arg1, Arg2, Arg3>::write(const Buffer& buffer) {
  #line 51 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  buffer->set(String("class"), String("MatrixGaussian"));
  #line 52 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  buffer->set(String("M"), value(M));
  #line 53 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  buffer->set(String("U"), value(U));
  #line 54 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  buffer->set(String("V"), value(V));
}

#line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
LinearMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::LinearMatrixNormalInverseWishartMatrixGaussianDistribution_(const Arg1& A, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& C, const Arg7& U) :
    #line 5 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    base_type_(),
    #line 11 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    A(A),
    #line 16 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    N(N),
    #line 21 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    \u039b(\u039b),
    #line 26 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    \u03a8(\u03a8),
    #line 31 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    k(k),
    #line 36 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    C(C),
    #line 41 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    U(U) {
  //
}

#line 43 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 43 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 43 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
Boolean LinearMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::supportsLazy() {
  #line 44 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return true;
}

#line 47 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 47 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 47 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
libbirch::DefaultArray<Real,2> LinearMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::simulate() {
  #line 48 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return simulate_scaled_matrix_normal_inverse_wishart_matrix_gaussian(value(A), value(N), value(\u039b), value(\u03a8), value(k), value(U)) + value(C);
}

#line 53 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 53 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 53 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,2>> LinearMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::simulateLazy() {
  #line 54 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return simulate_scaled_matrix_normal_inverse_wishart_matrix_gaussian(peek(A), peek(N), peek(\u039b), peek(\u03a8), peek(k), peek(U)) + peek(C);
}

#line 58 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 58 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 58 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
Real LinearMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::logpdf(const libbirch::DefaultArray<Real,2>& X) {
  #line 59 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return logpdf_scaled_matrix_normal_inverse_wishart_matrix_gaussian(X - value(C), value(A), value(N), value(\u039b), value(\u03a8), value(k), value(U));
}

#line 64 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 64 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 64 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<Expression<Real>> LinearMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 65 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return box(logpdf_scaled_matrix_normal_inverse_wishart_matrix_gaussian(box(X - C), A, N, \u039b, \u03a8, k, U));
}

#line 69 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 69 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 69 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<Delay> LinearMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::update(const libbirch::DefaultArray<Real,2>& X) {
  #line 70 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return update_scaled_matrix_normal_inverse_wishart_matrix_gaussian(X - value(C), value(A), value(N), value(\u039b), value(\u03a8), value(k), value(U));
}

#line 75 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 75 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 75 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<Delay> LinearMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::updateLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 76 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return update_scaled_matrix_normal_inverse_wishart_matrix_gaussian(box(X - C), A, N, \u039b, \u03a8, k, U);
}

#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::MatrixNormalInverseWishartDistribution_(const Arg1& N, const Arg2& \u039b, const Arg3& \u03a8, const Arg4& k) :
    #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
    N(N),
    #line 14 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
    \u039b(\u039b),
    #line 19 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
    \u03a8(\u03a8),
    #line 24 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
    k(k) {
  //
}

#line 26 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 26 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 26 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
Boolean MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::supportsLazy() {
  #line 27 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return true;
}

#line 30 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 30 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 30 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
libbirch::DefaultArray<Real,2> MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::simulate() {
  #line 31 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return simulate_matrix_normal_inverse_wishart(value(N), value(\u039b), value(\u03a8), value(k));
}

#line 35 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 35 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 35 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
std::optional<libbirch::DefaultArray<Real,2>> MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::simulateLazy() {
  #line 36 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return simulate_matrix_normal_inverse_wishart(peek(N), peek(\u039b), peek(\u03a8), peek(k));
}

#line 40 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 40 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 40 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
Real MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::logpdf(const libbirch::DefaultArray<Real,2>& X) {
  #line 41 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return logpdf_matrix_normal_inverse_wishart(X, value(N), value(\u039b), value(\u03a8), value(k));
}

#line 45 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 45 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 45 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
std::optional<Expression<Real>> MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 46 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return box(logpdf_matrix_normal_inverse_wishart(X, N, \u039b, \u03a8, k));
}

#line 49 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 49 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 49 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
std::optional<Delay> MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::update(const libbirch::DefaultArray<Real,2>& X) {
  #line 50 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return update_matrix_normal_inverse_wishart(X, value(N), value(\u039b), value(\u03a8), value(k));
}

#line 54 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 54 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 54 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
std::optional<Delay> MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::updateLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 55 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return update_matrix_normal_inverse_wishart(X, N, \u039b, \u03a8, k);
}

#line 59 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 59 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 59 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
Boolean MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::isMatrixNormalInverseWishart() {
  #line 60 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return true;
}

#line 63 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 63 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 63 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>>> MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::getMatrixNormalInverseWishart() {
  #line 65 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return box(N, \u039b, \u03a8, k);
}

#line 68 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 68 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
#line 68 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
void MatrixNormalInverseWishartDistribution_<Arg1, Arg2, Arg3, Arg4>::write(const Buffer& buffer) {
  #line 69 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  buffer->set(String("class"), String("MatrixNormalInverseWishart"));
  #line 70 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  buffer->set(String("N"), value(N));
  #line 71 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  buffer->set(String("Λ"), value(\u039b));
  #line 72 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  buffer->set(String("Ψ"), value(\u03a8));
  #line 73 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  buffer->set(String("k"), value(k));
}

#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
MatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::MatrixNormalInverseWishartMatrixGaussianDistribution_(const Arg1& N, const Arg2& \u039b, const Arg3& \u03a8, const Arg4& k, const Arg5& U) :
    #line 4 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    N(N),
    #line 14 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    \u039b(\u039b),
    #line 19 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    \u03a8(\u03a8),
    #line 24 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    k(k),
    #line 29 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    U(U) {
  //
}

#line 31 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 31 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 31 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
Boolean MatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::supportsLazy() {
  #line 32 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return true;
}

#line 35 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 35 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 35 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
libbirch::DefaultArray<Real,2> MatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulate() {
  #line 36 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return simulate_matrix_normal_inverse_wishart_matrix_gaussian(value(N), value(\u039b), value(\u03a8), value(k), value(U));
}

#line 40 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 40 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 40 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,2>> MatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulateLazy() {
  #line 41 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return simulate_matrix_normal_inverse_wishart_matrix_gaussian(peek(N), peek(\u039b), peek(\u03a8), peek(k), peek(U));
}

#line 45 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 45 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 45 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
Real MatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdf(const libbirch::DefaultArray<Real,2>& X) {
  #line 46 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return logpdf_matrix_normal_inverse_wishart_matrix_gaussian(X, value(N), value(\u039b), value(\u03a8), value(k), value(U));
}

#line 50 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 50 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 50 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<Expression<Real>> MatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 51 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return box(logpdf_matrix_normal_inverse_wishart_matrix_gaussian(X, N, \u039b, \u03a8, k, U));
}

#line 55 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 55 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 55 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<Delay> MatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::update(const libbirch::DefaultArray<Real,2>& X) {
  #line 56 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return update_matrix_normal_inverse_wishart_matrix_gaussian(X, value(N), value(\u039b), value(\u03a8), value(k), value(U));
}

#line 60 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 60 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 60 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<Delay> MatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::updateLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 61 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return update_matrix_normal_inverse_wishart_matrix_gaussian(X, N, \u039b, \u03a8, k, U);
}

#line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_(const Arg1& A, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& U) :
    #line 5 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    base_type_(),
    #line 11 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    A(A),
    #line 16 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    N(N),
    #line 21 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    \u039b(\u039b),
    #line 26 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    \u03a8(\u03a8),
    #line 31 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    k(k),
    #line 36 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    U(U) {
  //
}

#line 38 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 38 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 38 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
Boolean ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::supportsLazy() {
  #line 39 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return true;
}

#line 42 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 42 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 42 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
libbirch::DefaultArray<Real,2> ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::simulate() {
  #line 43 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return simulate_scaled_matrix_normal_inverse_wishart_matrix_gaussian(value(A), value(N), value(\u039b), value(\u03a8), value(k), value(U));
}

#line 47 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 47 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 47 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,2>> ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::simulateLazy() {
  #line 48 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return simulate_scaled_matrix_normal_inverse_wishart_matrix_gaussian(peek(A), peek(N), peek(\u039b), peek(\u03a8), peek(k), peek(U));
}

#line 52 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 52 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 52 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
Real ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::logpdf(const libbirch::DefaultArray<Real,2>& X) {
  #line 53 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return logpdf_scaled_matrix_normal_inverse_wishart_matrix_gaussian(X, value(A), value(N), value(\u039b), value(\u03a8), value(k), value(U));
}

#line 57 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 57 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 57 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<Expression<Real>> ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 58 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return box(logpdf_scaled_matrix_normal_inverse_wishart_matrix_gaussian(X, A, N, \u039b, \u03a8, k, U));
}

#line 62 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 62 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 62 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<Delay> ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::update(const libbirch::DefaultArray<Real,2>& X) {
  #line 63 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return update_scaled_matrix_normal_inverse_wishart_matrix_gaussian(X, value(A), value(N), value(\u039b), value(\u03a8), value(k), value(U));
}

#line 67 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 67 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
#line 67 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
std::optional<Delay> ScaledMatrixNormalInverseWishartMatrixGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::updateLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 68 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return update_scaled_matrix_normal_inverse_wishart_matrix_gaussian(X, A, N, \u039b, \u03a8, k, U);
}

#line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
InverseWishartDistribution_<Arg1, Arg2>::InverseWishartDistribution_(const Arg1& \u03a8, const Arg2& k) :
    #line 30 "src/distribution/matrix_other/InverseWishartDistribution.birch"
    base_type_(),
    #line 35 "src/distribution/matrix_other/InverseWishartDistribution.birch"
    \u03a8(\u03a8),
    #line 40 "src/distribution/matrix_other/InverseWishartDistribution.birch"
    k(k) {
  //
}

#line 42 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 42 "src/distribution/matrix_other/InverseWishartDistribution.birch"
#line 42 "src/distribution/matrix_other/InverseWishartDistribution.birch"
Boolean InverseWishartDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 43 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  return true;
}

#line 46 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 46 "src/distribution/matrix_other/InverseWishartDistribution.birch"
#line 46 "src/distribution/matrix_other/InverseWishartDistribution.birch"
libbirch::DefaultArray<Real,2> InverseWishartDistribution_<Arg1, Arg2>::simulate() {
  #line 47 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  return simulate_inverse_wishart(value(\u03a8), value(k));
}

#line 50 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 50 "src/distribution/matrix_other/InverseWishartDistribution.birch"
#line 50 "src/distribution/matrix_other/InverseWishartDistribution.birch"
std::optional<libbirch::DefaultArray<Real,2>> InverseWishartDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 51 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  return simulate_inverse_wishart(peek(\u03a8), peek(k));
}

#line 54 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 54 "src/distribution/matrix_other/InverseWishartDistribution.birch"
#line 54 "src/distribution/matrix_other/InverseWishartDistribution.birch"
Real InverseWishartDistribution_<Arg1, Arg2>::logpdf(const libbirch::DefaultArray<Real,2>& X) {
  #line 55 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  return logpdf_inverse_wishart(X, value(\u03a8), value(k));
}

#line 58 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 58 "src/distribution/matrix_other/InverseWishartDistribution.birch"
#line 58 "src/distribution/matrix_other/InverseWishartDistribution.birch"
std::optional<Expression<Real>> InverseWishartDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 59 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  return box(logpdf_inverse_wishart(X, \u03a8, k));
}

#line 62 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 62 "src/distribution/matrix_other/InverseWishartDistribution.birch"
#line 62 "src/distribution/matrix_other/InverseWishartDistribution.birch"
Boolean InverseWishartDistribution_<Arg1, Arg2>::isInverseWishart() {
  #line 63 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  return true;
}

#line 66 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 66 "src/distribution/matrix_other/InverseWishartDistribution.birch"
#line 66 "src/distribution/matrix_other/InverseWishartDistribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>>> InverseWishartDistribution_<Arg1, Arg2>::getInverseWishart() {
  #line 68 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  return box(\u03a8, k);
}

#line 71 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 71 "src/distribution/matrix_other/InverseWishartDistribution.birch"
#line 71 "src/distribution/matrix_other/InverseWishartDistribution.birch"
void InverseWishartDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 72 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  buffer->set(String("class"), String("InverseWishart"));
  #line 73 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  buffer->set(String("Ψ"), value(\u03a8));
  #line 74 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  buffer->set(String("k"), value(k));
}

#line 4 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/matrix_other/WishartDistribution.birch"
WishartDistribution_<Arg1, Arg2>::WishartDistribution_(const Arg1& \u03a8, const Arg2& k) :
    #line 4 "src/distribution/matrix_other/WishartDistribution.birch"
    base_type_(),
    #line 8 "src/distribution/matrix_other/WishartDistribution.birch"
    \u03a8(\u03a8),
    #line 13 "src/distribution/matrix_other/WishartDistribution.birch"
    k(k) {
  //
}

#line 15 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 15 "src/distribution/matrix_other/WishartDistribution.birch"
#line 15 "src/distribution/matrix_other/WishartDistribution.birch"
Boolean WishartDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 16 "src/distribution/matrix_other/WishartDistribution.birch"
  return true;
}

#line 19 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 19 "src/distribution/matrix_other/WishartDistribution.birch"
#line 19 "src/distribution/matrix_other/WishartDistribution.birch"
libbirch::DefaultArray<Real,2> WishartDistribution_<Arg1, Arg2>::simulate() {
  #line 20 "src/distribution/matrix_other/WishartDistribution.birch"
  return simulate_wishart(value(\u03a8), value(k));
}

#line 23 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 23 "src/distribution/matrix_other/WishartDistribution.birch"
#line 23 "src/distribution/matrix_other/WishartDistribution.birch"
std::optional<libbirch::DefaultArray<Real,2>> WishartDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 24 "src/distribution/matrix_other/WishartDistribution.birch"
  return simulate_wishart(peek(\u03a8), peek(k));
}

#line 27 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 27 "src/distribution/matrix_other/WishartDistribution.birch"
#line 27 "src/distribution/matrix_other/WishartDistribution.birch"
Real WishartDistribution_<Arg1, Arg2>::logpdf(const libbirch::DefaultArray<Real,2>& X) {
  #line 28 "src/distribution/matrix_other/WishartDistribution.birch"
  return logpdf_wishart(X, value(\u03a8), value(k));
}

#line 31 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 31 "src/distribution/matrix_other/WishartDistribution.birch"
#line 31 "src/distribution/matrix_other/WishartDistribution.birch"
std::optional<Expression<Real>> WishartDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,2>>& X) {
  #line 32 "src/distribution/matrix_other/WishartDistribution.birch"
  return box(logpdf_wishart(X, \u03a8, k));
}

#line 35 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 35 "src/distribution/matrix_other/WishartDistribution.birch"
#line 35 "src/distribution/matrix_other/WishartDistribution.birch"
void WishartDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 36 "src/distribution/matrix_other/WishartDistribution.birch"
  buffer->set(String("class"), String("Wishart"));
  #line 37 "src/distribution/matrix_other/WishartDistribution.birch"
  buffer->set(String("Ψ"), value(\u03a8));
  #line 38 "src/distribution/matrix_other/WishartDistribution.birch"
  buffer->set(String("k"), value(k));
}

#line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
DirichletMultinomialDistribution_<Arg1, Arg2>::DirichletMultinomialDistribution_(const Arg1& n, const Arg2& \u03b1) :
    #line 4 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    n(n),
    #line 14 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    \u03b1(\u03b1) {
  //
}

#line 16 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 16 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
#line 16 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
Boolean DirichletMultinomialDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 17 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  return false;
}

#line 20 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 20 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
#line 20 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
libbirch::DefaultArray<Integer,1> DirichletMultinomialDistribution_<Arg1, Arg2>::simulate() {
  #line 21 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  return simulate_dirichlet_multinomial(value(n), value(\u03b1));
}

#line 24 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 24 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
#line 24 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
Real DirichletMultinomialDistribution_<Arg1, Arg2>::logpdf(const libbirch::DefaultArray<Integer,1>& x) {
  #line 25 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  return logpdf_dirichlet_multinomial(x, value(n), value(\u03b1));
}

#line 28 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 28 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
#line 28 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
std::optional<Delay> DirichletMultinomialDistribution_<Arg1, Arg2>::update(const libbirch::DefaultArray<Integer,1>& x) {
  #line 29 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  return update_dirichlet_multinomial(x, value(n), value(\u03b1));
}

#line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
MultinomialDistribution_<Arg1, Arg2>::MultinomialDistribution_(const Arg1& n, const Arg2& \u03c1) :
    #line 4 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    n(n),
    #line 14 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
    \u03c1(\u03c1) {
  //
}

#line 16 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 16 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
#line 16 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
Boolean MultinomialDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 17 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  return false;
}

#line 20 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 20 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
#line 20 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
libbirch::DefaultArray<Integer,1> MultinomialDistribution_<Arg1, Arg2>::simulate() {
  #line 21 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  return simulate_multinomial(value(n), value(\u03c1));
}

#line 24 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 24 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
#line 24 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
Real MultinomialDistribution_<Arg1, Arg2>::logpdf(const libbirch::DefaultArray<Integer,1>& x) {
  #line 25 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  return logpdf_multinomial(x, value(n), value(\u03c1));
}

#line 28 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 28 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
#line 28 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
void MultinomialDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 29 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  buffer->set(String("class"), String("Multinomial"));
  #line 30 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  buffer->set(String("n"), value(n));
  #line 31 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  buffer->set(String("ρ"), value(\u03c1));
}

#line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::LinearMultivariateGaussianMultivariateGaussianDistribution_(const Arg1& A, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& c, const Arg5& S) :
    #line 4 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
    base_type_(),
    #line 10 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
    A(A),
    #line 15 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
    \u03bc(\u03bc),
    #line 20 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
    \u03a3(\u03a3),
    #line 25 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
    c(c),
    #line 30 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
    S(S) {
  //
}

#line 32 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 32 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 32 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
Boolean LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::supportsLazy() {
  #line 33 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  return true;
}

#line 36 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 36 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 36 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulate() {
  #line 37 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  return simulate_scaled_multivariate_gaussian_multivariate_gaussian(value(A), value(\u03bc), value(\u03a3), value(S)) + value(c);
}

#line 41 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 41 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 41 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulateLazy() {
  #line 42 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  return simulate_scaled_multivariate_gaussian_multivariate_gaussian(peek(A), peek(\u03bc), peek(\u03a3), peek(S)) + peek(c);
}

#line 46 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 46 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 46 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
Real LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 47 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  return logpdf_scaled_multivariate_gaussian_multivariate_gaussian(x - value(c), value(A), value(\u03bc), value(\u03a3), value(S));
}

#line 51 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 51 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 51 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<Expression<Real>> LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 52 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  return box(logpdf_scaled_multivariate_gaussian_multivariate_gaussian(box(x - c), A, \u03bc, \u03a3, S));
}

#line 56 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 56 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 56 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<Delay> LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::update(const libbirch::DefaultArray<Real,1>& x) {
  #line 57 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  return update_scaled_multivariate_gaussian_multivariate_gaussian(x - value(c), value(A), value(\u03bc), value(\u03a3), value(S));
}

#line 61 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 61 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 61 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<Delay> LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 62 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  return update_scaled_multivariate_gaussian_multivariate_gaussian(box(x - c), A, \u03bc, \u03a3, S);
}

#line 66 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 66 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 66 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
Boolean LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::isMultivariateGaussian() {
  #line 67 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  return true;
}

#line 70 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 70 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 70 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>>> LinearMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::getMultivariateGaussian() {
  #line 72 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  return box(A * \u03bc + c, outer(A * \u03a3, A) + S);
}

#line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
MultivariateGaussianDistribution_<Arg1, Arg2>::MultivariateGaussianDistribution_(const Arg1& \u03bc, const Arg2& \u03a3) :
    #line 1 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
    \u03bc(\u03bc),
    #line 14 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
    \u03a3(\u03a3) {
  //
}

#line 16 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 16 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
#line 16 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
Boolean MultivariateGaussianDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 17 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  return true;
}

#line 20 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 20 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
#line 20 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> MultivariateGaussianDistribution_<Arg1, Arg2>::simulate() {
  #line 21 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  return simulate_multivariate_gaussian(value(\u03bc), value(\u03a3));
}

#line 24 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 24 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
#line 24 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> MultivariateGaussianDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 25 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  return simulate_multivariate_gaussian(peek(\u03bc), peek(\u03a3));
}

#line 28 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 28 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
#line 28 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
Real MultivariateGaussianDistribution_<Arg1, Arg2>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 29 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  return logpdf_multivariate_gaussian(x, value(\u03bc), value(\u03a3));
}

#line 32 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 32 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
#line 32 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
std::optional<Expression<Real>> MultivariateGaussianDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 33 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  return box(logpdf_multivariate_gaussian(x, \u03bc, \u03a3));
}

#line 36 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 36 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
#line 36 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
Boolean MultivariateGaussianDistribution_<Arg1, Arg2>::isMultivariateGaussian() {
  #line 37 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  return true;
}

#line 40 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 40 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
#line 40 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>>> MultivariateGaussianDistribution_<Arg1, Arg2>::getMultivariateGaussian() {
  #line 42 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  return box(\u03bc, \u03a3);
}

#line 45 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 45 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
#line 45 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
void MultivariateGaussianDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 46 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  buffer->set(String("class"), String("MultivariateGaussian"));
  #line 47 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  buffer->set(String("μ"), value(\u03bc));
  #line 48 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  buffer->set(String("Σ"), value(\u03a3));
}

#line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>::MultivariateGaussianMultivariateGaussianDistribution_(const Arg1& \u03bc, const Arg2& \u03a3, const Arg3& S) :
    #line 4 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
    \u03bc(\u03bc),
    #line 14 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
    \u03a3(\u03a3),
    #line 19 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
    S(S) {
  //
}

#line 21 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 21 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
#line 21 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
Boolean MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>::supportsLazy() {
  #line 22 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return true;
}

#line 25 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 25 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
#line 25 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>::simulate() {
  #line 26 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return simulate_multivariate_gaussian_multivariate_gaussian(value(\u03bc), value(\u03a3), value(S));
}

#line 30 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 30 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
#line 30 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>::simulateLazy() {
  #line 31 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return simulate_multivariate_gaussian_multivariate_gaussian(peek(\u03bc), peek(\u03a3), peek(S));
}

#line 35 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 35 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
#line 35 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
Real MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 36 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return logpdf_multivariate_gaussian_multivariate_gaussian(x, value(\u03bc), value(\u03a3), value(S));
}

#line 40 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 40 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
#line 40 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<Expression<Real>> MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 41 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return box(logpdf_multivariate_gaussian_multivariate_gaussian(x, \u03bc, \u03a3, S));
}

#line 45 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 45 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
#line 45 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<Delay> MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>::update(const libbirch::DefaultArray<Real,1>& x) {
  #line 46 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return update_multivariate_gaussian_multivariate_gaussian(x, value(\u03bc), value(\u03a3), value(S));
}

#line 50 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 50 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
#line 50 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<Delay> MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>::updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 51 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return update_multivariate_gaussian_multivariate_gaussian(x, \u03bc, \u03a3, S);
}

#line 54 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 54 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
#line 54 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
Boolean MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>::isMultivariateGaussian() {
  #line 55 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return true;
}

#line 58 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 58 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
#line 58 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>>> MultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3>::getMultivariateGaussian() {
  #line 60 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return box(\u03bc, \u03a3 + S);
}

#line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>::ScaledMultivariateGaussianMultivariateGaussianDistribution_(const Arg1& A, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& S) :
    #line 4 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
    A(A),
    #line 14 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
    \u03bc(\u03bc),
    #line 19 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
    \u03a3(\u03a3),
    #line 24 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
    S(S) {
  //
}

#line 26 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 26 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 26 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
Boolean ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>::supportsLazy() {
  #line 27 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return true;
}

#line 30 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 30 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 30 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>::simulate() {
  #line 31 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return simulate_scaled_multivariate_gaussian_multivariate_gaussian(value(A), value(\u03bc), value(\u03a3), value(S));
}

#line 35 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 35 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 35 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>::simulateLazy() {
  #line 36 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return simulate_scaled_multivariate_gaussian_multivariate_gaussian(peek(A), peek(\u03bc), peek(\u03a3), peek(S));
}

#line 40 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 40 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 40 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
Real ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 41 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return logpdf_scaled_multivariate_gaussian_multivariate_gaussian(x, value(A), value(\u03bc), value(\u03a3), value(S));
}

#line 45 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 45 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 45 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<Expression<Real>> ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 46 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return box(logpdf_scaled_multivariate_gaussian_multivariate_gaussian(x, A, \u03bc, \u03a3, S));
}

#line 50 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 50 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 50 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<Delay> ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>::update(const libbirch::DefaultArray<Real,1>& x) {
  #line 51 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return update_scaled_multivariate_gaussian_multivariate_gaussian(x, value(A), value(\u03bc), value(\u03a3), value(S));
}

#line 55 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 55 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 55 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<Delay> ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>::updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 56 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return update_scaled_multivariate_gaussian_multivariate_gaussian(x, A, \u03bc, \u03a3, S);
}

#line 60 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 60 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 60 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
Boolean ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>::isMultivariateGaussian() {
  #line 61 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return true;
}

#line 64 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 64 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
#line 64 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>>> ScaledMultivariateGaussianMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4>::getMultivariateGaussian() {
  #line 66 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return box(A * \u03bc, outer(A * \u03a3, A) + S);
}

#line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_(const Arg1& A, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& \u03a3) :
    #line 5 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    base_type_(),
    #line 11 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    A(A),
    #line 16 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bd(\u03bd),
    #line 21 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u039b(\u039b),
    #line 26 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03b1(\u03b1),
    #line 31 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03b2(\u03b2),
    #line 36 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    c(c),
    #line 41 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3(\u03a3) {
  //
}

#line 43 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 43 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 43 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Boolean LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::supportsLazy() {
  #line 44 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return true;
}

#line 47 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 47 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 47 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::simulate() {
  #line 48 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return simulate_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(value(A), value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2), value(\u03a3)) + value(c);
}

#line 53 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 53 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 53 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::simulateLazy() {
  #line 54 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return simulate_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(peek(A), peek(\u03bd), peek(\u039b), peek(\u03b1), peek(\u03b2), peek(\u03a3)) + peek(c);
}

#line 58 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 58 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 58 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Real LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 59 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return logpdf_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(x - value(c), value(A), value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2), value(\u03a3));
}

#line 64 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 64 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 64 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<Expression<Real>> LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 65 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return box(logpdf_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(box(x - c), A, \u03bd, \u039b, \u03b1, \u03b2, \u03a3));
}

#line 69 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 69 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 69 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<Delay> LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::update(const libbirch::DefaultArray<Real,1>& x) {
  #line 70 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return update_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(x - value(c), value(A), value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2), value(\u03a3));
}

#line 75 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 75 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 75 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<Delay> LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 76 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return update_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(box(x - c), A, \u03bd, \u039b, \u03b1, \u03b2, \u03a3);
}

#line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::MultivariateNormalInverseGammaDistribution_(const Arg1& \u03bd, const Arg2& \u039b, const Arg3& \u03b1, const Arg4& \u03b2) :
    #line 35 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    base_type_(),
    #line 40 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    \u03bd(\u03bd),
    #line 45 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    \u039b(\u039b),
    #line 50 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    \u03b1(\u03b1),
    #line 55 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    \u03b2(\u03b2) {
  //
}

#line 57 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 57 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 57 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
Boolean MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::supportsLazy() {
  #line 58 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return true;
}

#line 61 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 61 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 61 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
libbirch::DefaultArray<Real,1> MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::simulate() {
  #line 62 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return simulate_multivariate_normal_inverse_gamma(value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2));
}

#line 66 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 66 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 66 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::simulateLazy() {
  #line 67 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return simulate_multivariate_normal_inverse_gamma(peek(\u03bd), peek(\u039b), peek(\u03b1), peek(\u03b2));
}

#line 71 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 71 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 71 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
Real MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 72 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return logpdf_multivariate_normal_inverse_gamma(x, value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2));
}

#line 76 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 76 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 76 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
std::optional<Expression<Real>> MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 77 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return box(logpdf_multivariate_normal_inverse_gamma(x, \u03bd, \u039b, \u03b1, \u03b2));
}

#line 80 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 80 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 80 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
std::optional<Delay> MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::update(const libbirch::DefaultArray<Real,1>& x) {
  #line 81 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return update_multivariate_normal_inverse_gamma(x, value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2));
}

#line 85 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 85 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 85 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
std::optional<Delay> MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 86 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return update_multivariate_normal_inverse_gamma(x, \u03bd, \u039b, \u03b1, \u03b2);
}

#line 89 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 89 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 89 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
Boolean MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::isMultivariateNormalInverseGamma() {
  #line 90 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return true;
}

#line 93 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 93 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 93 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
std::optional<std::tuple<Expression<libbirch::DefaultArray<Real,1>>, Expression<libbirch::DefaultArray<Real,2>>, Expression<Real>, Expression<Real>>> MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::getMultivariateNormalInverseGamma() {
  #line 95 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return box(\u03bd, \u039b, \u03b1, \u03b2);
}

#line 98 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 98 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
#line 98 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
void MultivariateNormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::write(const Buffer& buffer) {
  #line 99 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  buffer->set(String("class"), String("MultivariateNormalInverseGamma"));
  #line 100 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  buffer->set(String("ν"), value(\u03bd));
  #line 101 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  buffer->set(String("Λ"), value(\u039b));
  #line 102 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  buffer->set(String("α"), value(\u03b1));
  #line 103 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  buffer->set(String("β"), value(\u03b2));
}

#line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
MultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::MultivariateNormalInverseGammaMultivariateGaussianDistribution_(const Arg1& \u03bd, const Arg2& \u039b, const Arg3& \u03b1, const Arg4& \u03b2, const Arg5& \u03a3) :
    #line 4 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    base_type_(),
    #line 10 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bd(\u03bd),
    #line 15 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u039b(\u039b),
    #line 20 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03b1(\u03b1),
    #line 25 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03b2(\u03b2),
    #line 30 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3(\u03a3) {
  //
}

#line 32 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 32 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 32 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Boolean MultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::supportsLazy() {
  #line 33 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return true;
}

#line 36 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 36 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 36 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> MultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulate() {
  #line 37 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return simulate_multivariate_normal_inverse_gamma_multivariate_gaussian(value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2), value(\u03a3));
}

#line 41 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 41 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 41 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> MultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulateLazy() {
  #line 42 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return simulate_multivariate_normal_inverse_gamma_multivariate_gaussian(peek(\u03bd), peek(\u039b), peek(\u03b1), peek(\u03b2), peek(\u03a3));
}

#line 46 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 46 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 46 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Real MultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 47 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return logpdf_multivariate_normal_inverse_gamma_multivariate_gaussian(x, value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2), value(\u03a3));
}

#line 51 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 51 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 51 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<Expression<Real>> MultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 52 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return box(logpdf_multivariate_normal_inverse_gamma_multivariate_gaussian(x, \u03bd, \u039b, \u03b1, \u03b2, \u03a3));
}

#line 56 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 56 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 56 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<Delay> MultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::update(const libbirch::DefaultArray<Real,1>& x) {
  #line 57 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return update_multivariate_normal_inverse_gamma_multivariate_gaussian(x, value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2), value(\u03a3));
}

#line 61 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 61 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 61 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<Delay> MultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 62 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return update_multivariate_normal_inverse_gamma_multivariate_gaussian(x, \u03bd, \u039b, \u03b1, \u03b2, \u03a3);
}

#line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_(const Arg1& A, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& \u03a3) :
    #line 5 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    base_type_(),
    #line 11 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    A(A),
    #line 16 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bd(\u03bd),
    #line 21 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u039b(\u039b),
    #line 26 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03b1(\u03b1),
    #line 31 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03b2(\u03b2),
    #line 36 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3(\u03a3) {
  //
}

#line 38 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 38 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 38 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Boolean ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::supportsLazy() {
  #line 39 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return true;
}

#line 42 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 42 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 42 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::simulate() {
  #line 43 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return simulate_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(value(A), value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2), value(\u03a3));
}

#line 47 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 47 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 47 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::simulateLazy() {
  #line 48 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return simulate_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(peek(A), peek(\u03bd), peek(\u039b), peek(\u03b1), peek(\u03b2), peek(\u03a3));
}

#line 52 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 52 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 52 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
Real ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 53 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return logpdf_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(x, value(A), value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2), value(\u03a3));
}

#line 57 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 57 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 57 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<Expression<Real>> ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 58 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return box(logpdf_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(x, A, \u03bd, \u039b, \u03b1, \u03b2, \u03a3));
}

#line 62 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 62 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 62 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<Delay> ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::update(const libbirch::DefaultArray<Real,1>& x) {
  #line 63 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return update_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(x, value(A), value(\u03bd), value(\u039b), value(\u03b1), value(\u03b2), value(\u03a3));
}

#line 67 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 67 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
#line 67 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
std::optional<Delay> ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 68 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return update_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(x, A, \u03bd, \u039b, \u03b1, \u03b2, \u03a3);
}

#line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_(const Arg1& a, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& c) :
    #line 5 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    base_type_(),
    #line 11 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    a(a),
    #line 16 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    N(N),
    #line 21 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u039b(\u039b),
    #line 26 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u03a8(\u03a8),
    #line 31 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    k(k),
    #line 36 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    c(c) {
  //
}

#line 38 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 38 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 38 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
Boolean LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::supportsLazy() {
  #line 39 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return true;
}

#line 42 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 42 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 42 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::simulate() {
  #line 43 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return simulate_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(value(a), value(N), value(\u039b), value(\u03a8), value(k)) + value(c);
}

#line 47 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 47 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 47 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::simulateLazy() {
  #line 48 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return simulate_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(peek(a), peek(N), peek(\u039b), peek(\u03a8), peek(k)) + peek(c);
}

#line 52 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 52 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 52 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
Real LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 53 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return logpdf_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(x - value(c), value(a), value(N), value(\u039b), value(\u03a8), value(k));
}

#line 57 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 57 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 57 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
std::optional<Expression<Real>> LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 58 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return box(logpdf_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(box(x - c), a, N, \u039b, \u03a8, k));
}

#line 62 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 62 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 62 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
std::optional<Delay> LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::update(const libbirch::DefaultArray<Real,1>& x) {
  #line 63 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return update_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(x - value(c), value(a), value(N), value(\u039b), value(\u03a8), value(k));
}

#line 67 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 67 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 67 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
std::optional<Delay> LinearMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>::updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 68 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return update_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(box(x - c), a, N, \u039b, \u03a8, k);
}

#line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_(const Arg1& a, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k) :
    #line 5 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    base_type_(),
    #line 11 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    a(a),
    #line 16 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    N(N),
    #line 21 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u039b(\u039b),
    #line 26 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u03a8(\u03a8),
    #line 31 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    k(k) {
  //
}

#line 33 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 33 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 33 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
Boolean ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::supportsLazy() {
  #line 34 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return true;
}

#line 37 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 37 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 37 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
libbirch::DefaultArray<Real,1> ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulate() {
  #line 38 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return simulate_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(value(a), value(N), value(\u039b), value(\u03a8), value(k));
}

#line 42 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 42 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 42 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulateLazy() {
  #line 43 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return simulate_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(peek(a), peek(N), peek(\u039b), peek(\u03a8), peek(k));
}

#line 47 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 47 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 47 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
Real ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 48 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return logpdf_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(x, value(a), value(N), value(\u039b), value(\u03a8), value(k));
}

#line 52 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 52 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 52 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
std::optional<Expression<Real>> ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 53 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return box(logpdf_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(x, a, N, \u039b, \u03a8, k));
}

#line 57 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 57 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 57 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
std::optional<Delay> ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::update(const libbirch::DefaultArray<Real,1>& x) {
  #line 58 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return update_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(x, value(a), value(N), value(\u039b), value(\u03a8), value(k));
}

#line 62 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 62 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
#line 62 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
std::optional<Delay> ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::updateLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 63 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return update_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(x, a, N, \u039b, \u03a8, k);
}

#line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
DirichletDistribution_<Arg>::DirichletDistribution_(const Arg& \u03b1) :
    #line 4 "src/distribution/multivariate_other/DirichletDistribution.birch"
    base_type_(),
    #line 8 "src/distribution/multivariate_other/DirichletDistribution.birch"
    \u03b1(\u03b1) {
  //
}

#line 10 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 10 "src/distribution/multivariate_other/DirichletDistribution.birch"
#line 10 "src/distribution/multivariate_other/DirichletDistribution.birch"
Boolean DirichletDistribution_<Arg>::supportsLazy() {
  #line 11 "src/distribution/multivariate_other/DirichletDistribution.birch"
  return false;
}

#line 14 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 14 "src/distribution/multivariate_other/DirichletDistribution.birch"
#line 14 "src/distribution/multivariate_other/DirichletDistribution.birch"
libbirch::DefaultArray<Real,1> DirichletDistribution_<Arg>::simulate() {
  #line 15 "src/distribution/multivariate_other/DirichletDistribution.birch"
  return simulate_dirichlet(value(\u03b1));
}

#line 18 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 18 "src/distribution/multivariate_other/DirichletDistribution.birch"
#line 18 "src/distribution/multivariate_other/DirichletDistribution.birch"
Real DirichletDistribution_<Arg>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 19 "src/distribution/multivariate_other/DirichletDistribution.birch"
  return logpdf_dirichlet(x, value(\u03b1));
}

#line 22 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 22 "src/distribution/multivariate_other/DirichletDistribution.birch"
#line 22 "src/distribution/multivariate_other/DirichletDistribution.birch"
Boolean DirichletDistribution_<Arg>::isDirichlet() {
  #line 23 "src/distribution/multivariate_other/DirichletDistribution.birch"
  return true;
}

#line 26 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 26 "src/distribution/multivariate_other/DirichletDistribution.birch"
#line 26 "src/distribution/multivariate_other/DirichletDistribution.birch"
std::optional<Expression<libbirch::DefaultArray<Real,1>>> DirichletDistribution_<Arg>::getDirichlet() {
  #line 27 "src/distribution/multivariate_other/DirichletDistribution.birch"
  return box(\u03b1);
}

#line 30 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 30 "src/distribution/multivariate_other/DirichletDistribution.birch"
#line 30 "src/distribution/multivariate_other/DirichletDistribution.birch"
void DirichletDistribution_<Arg>::write(const Buffer& buffer) {
  #line 31 "src/distribution/multivariate_other/DirichletDistribution.birch"
  buffer->set(String("class"), String("Dirichlet"));
  #line 32 "src/distribution/multivariate_other/DirichletDistribution.birch"
  buffer->set(String("α"), value(\u03b1));
}

#line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
RestaurantDistribution_<Arg1, Arg2>::RestaurantDistribution_(const Arg1& \u03b1, const Arg2& \u03b8) :
    #line 8 "src/distribution/multivariate_other/RestaurantDistribution.birch"
    base_type_(),
    #line 12 "src/distribution/multivariate_other/RestaurantDistribution.birch"
    \u03b1(\u03b1),
    #line 17 "src/distribution/multivariate_other/RestaurantDistribution.birch"
    \u03b8(\u03b8),
    #line 22 "src/distribution/multivariate_other/RestaurantDistribution.birch"
    n(),
    #line 27 "src/distribution/multivariate_other/RestaurantDistribution.birch"
    N(Integer(0)) {
  //
}

#line 29 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 29 "src/distribution/multivariate_other/RestaurantDistribution.birch"
#line 29 "src/distribution/multivariate_other/RestaurantDistribution.birch"
Boolean RestaurantDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 30 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  return false;
}

#line 33 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 33 "src/distribution/multivariate_other/RestaurantDistribution.birch"
#line 33 "src/distribution/multivariate_other/RestaurantDistribution.birch"
libbirch::DefaultArray<Real,1> RestaurantDistribution_<Arg1, Arg2>::simulate() {
  #line 34 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  assert(false);
  #line 35 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  return vector(0.0, Integer(0));
}

#line 38 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 38 "src/distribution/multivariate_other/RestaurantDistribution.birch"
#line 38 "src/distribution/multivariate_other/RestaurantDistribution.birch"
std::optional<libbirch::DefaultArray<Real,1>> RestaurantDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 39 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  assert(false);
  #line 40 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  return vector(0.0, Integer(0));
}

#line 43 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 43 "src/distribution/multivariate_other/RestaurantDistribution.birch"
#line 43 "src/distribution/multivariate_other/RestaurantDistribution.birch"
Real RestaurantDistribution_<Arg1, Arg2>::logpdf(const libbirch::DefaultArray<Real,1>& x) {
  #line 44 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  assert(false);
  #line 45 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  return 0.0;
}

#line 48 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 48 "src/distribution/multivariate_other/RestaurantDistribution.birch"
#line 48 "src/distribution/multivariate_other/RestaurantDistribution.birch"
std::optional<Expression<Real>> RestaurantDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<libbirch::DefaultArray<Real,1>>& x) {
  #line 49 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  assert(false);
  #line 50 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  return box(0.0);
}

#line 53 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 53 "src/distribution/multivariate_other/RestaurantDistribution.birch"
#line 53 "src/distribution/multivariate_other/RestaurantDistribution.birch"
Boolean RestaurantDistribution_<Arg1, Arg2>::isRestaurant() {
  #line 54 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  return true;
}

#line 57 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 57 "src/distribution/multivariate_other/RestaurantDistribution.birch"
#line 57 "src/distribution/multivariate_other/RestaurantDistribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>, Expression<libbirch::DefaultArray<Integer,1>>, Expression<Integer>>> RestaurantDistribution_<Arg1, Arg2>::getRestaurant() {
  #line 59 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  return box(\u03b1, \u03b8, n, N);
}

#line 62 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 62 "src/distribution/multivariate_other/RestaurantDistribution.birch"
#line 62 "src/distribution/multivariate_other/RestaurantDistribution.birch"
void RestaurantDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 63 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  buffer->set(String("class"), String("Restaurant"));
  #line 64 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  buffer->set(String("α"), value(\u03b1));
  #line 65 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  buffer->set(String("θ"), value(\u03b8));
  #line 66 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  buffer->set(String("n"), value(n));
}

#line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
BernoulliDistribution_<Arg>::BernoulliDistribution_(const Arg& \u03c1) :
    #line 4 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
    base_type_(),
    #line 8 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
    \u03c1(\u03c1) {
  //
}

#line 10 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 10 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
#line 10 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
Boolean BernoulliDistribution_<Arg>::supportsLazy() {
  #line 11 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  return true;
}

#line 14 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 14 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
#line 14 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
Boolean BernoulliDistribution_<Arg>::simulate() {
  #line 15 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  return simulate_bernoulli(value(\u03c1));
}

#line 18 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 18 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
#line 18 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
std::optional<Boolean> BernoulliDistribution_<Arg>::simulateLazy() {
  #line 19 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  return simulate_bernoulli(peek(\u03c1));
}

#line 22 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 22 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
#line 22 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
Real BernoulliDistribution_<Arg>::logpdf(const Boolean& x) {
  #line 23 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  return logpdf_bernoulli(x, value(\u03c1));
}

#line 26 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 26 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
#line 26 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
std::optional<Expression<Real>> BernoulliDistribution_<Arg>::logpdfLazy(const Expression<Boolean>& x) {
  #line 27 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  return box(logpdf_bernoulli(x, \u03c1));
}

#line 30 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 30 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
#line 30 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
void BernoulliDistribution_<Arg>::write(const Buffer& buffer) {
  #line 31 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  buffer->set(String("class"), String("Bernoulli"));
  #line 32 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  buffer->set(String("ρ"), value(\u03c1));
}

#line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
BetaBernoulliDistribution_<Arg1, Arg2>::BetaBernoulliDistribution_(const Arg1& \u03b1, const Arg2& \u03b2) :
    #line 4 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
    base_type_(),
    #line 8 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
    \u03b1(\u03b1),
    #line 13 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
    \u03b2(\u03b2) {
  //
}

#line 15 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 15 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
#line 15 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
Boolean BetaBernoulliDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 16 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  return true;
}

#line 19 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 19 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
#line 19 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
Boolean BetaBernoulliDistribution_<Arg1, Arg2>::simulate() {
  #line 20 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  return simulate_beta_bernoulli(value(\u03b1), value(\u03b2));
}

#line 23 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 23 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
#line 23 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
std::optional<Boolean> BetaBernoulliDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 24 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  return simulate_beta_bernoulli(peek(\u03b1), peek(\u03b2));
}

#line 27 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 27 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
#line 27 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
Real BetaBernoulliDistribution_<Arg1, Arg2>::logpdf(const Boolean& x) {
  #line 28 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  return logpdf_beta_bernoulli(x, value(\u03b1), value(\u03b2));
}

#line 31 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 31 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
#line 31 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
std::optional<Expression<Real>> BetaBernoulliDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<Boolean>& x) {
  #line 32 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  return box(logpdf_beta_bernoulli(x, \u03b1, \u03b2));
}

#line 35 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 35 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
#line 35 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
std::optional<Delay> BetaBernoulliDistribution_<Arg1, Arg2>::update(const Boolean& x) {
  #line 36 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  return update_beta_bernoulli(x, value(\u03b1), value(\u03b2));
}

#line 39 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 39 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
#line 39 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
std::optional<Delay> BetaBernoulliDistribution_<Arg1, Arg2>::updateLazy(const Expression<Boolean>& x) {
  #line 40 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  return update_beta_bernoulli(x, \u03b1, \u03b2);
}

#line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
BetaBinomialDistribution_<Arg1, Arg2, Arg3>::BetaBinomialDistribution_(const Arg1& n, const Arg2& \u03b1, const Arg3& \u03b2) :
    #line 4 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
    n(n),
    #line 14 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
    \u03b1(\u03b1),
    #line 19 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
    \u03b2(\u03b2) {
  //
}

#line 21 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 21 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 21 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
Boolean BetaBinomialDistribution_<Arg1, Arg2, Arg3>::supportsLazy() {
  #line 22 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return true;
}

#line 25 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 25 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 25 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
Integer BetaBinomialDistribution_<Arg1, Arg2, Arg3>::simulate() {
  #line 26 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return simulate_beta_binomial(value(n), value(\u03b1), value(\u03b2));
}

#line 29 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 29 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 29 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
std::optional<Integer> BetaBinomialDistribution_<Arg1, Arg2, Arg3>::simulateLazy() {
  #line 30 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return simulate_beta_binomial(peek(n), peek(\u03b1), peek(\u03b2));
}

#line 33 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 33 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 33 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
Real BetaBinomialDistribution_<Arg1, Arg2, Arg3>::logpdf(const Integer& x) {
  #line 34 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return logpdf_beta_binomial(x, value(n), value(\u03b1), value(\u03b2));
}

#line 37 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 37 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 37 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
std::optional<Expression<Real>> BetaBinomialDistribution_<Arg1, Arg2, Arg3>::logpdfLazy(const Expression<Integer>& x) {
  #line 38 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return box(logpdf_beta_binomial(x, n, \u03b1, \u03b2));
}

#line 41 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 41 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 41 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
std::optional<Delay> BetaBinomialDistribution_<Arg1, Arg2, Arg3>::update(const Integer& x) {
  #line 42 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return update_beta_binomial(x, value(n), value(\u03b1), value(\u03b2));
}

#line 45 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 45 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 45 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
std::optional<Delay> BetaBinomialDistribution_<Arg1, Arg2, Arg3>::updateLazy(const Expression<Integer>& x) {
  #line 46 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return update_beta_binomial(x, n, \u03b1, \u03b2);
}

#line 49 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 49 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 49 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
std::optional<Real> BetaBinomialDistribution_<Arg1, Arg2, Arg3>::cdf(const Integer& x) {
  #line 50 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return cdf_beta_binomial(x, value(n), value(\u03b1), value(\u03b2));
}

#line 53 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 53 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 53 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
std::optional<Integer> BetaBinomialDistribution_<Arg1, Arg2, Arg3>::lower() {
  #line 54 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return Integer(0);
}

#line 57 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 57 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
#line 57 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
std::optional<Integer> BetaBinomialDistribution_<Arg1, Arg2, Arg3>::upper() {
  #line 58 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return value(n);
}

#line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
BetaNegativeBinomialDistribution_<Arg1, Arg2, Arg3>::BetaNegativeBinomialDistribution_(const Arg1& k, const Arg2& \u03b1, const Arg3& \u03b2) :
    #line 4 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
    k(k),
    #line 14 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
    \u03b1(\u03b1),
    #line 19 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
    \u03b2(\u03b2) {
  //
}

#line 21 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 21 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
#line 21 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
Boolean BetaNegativeBinomialDistribution_<Arg1, Arg2, Arg3>::supportsLazy() {
  #line 22 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return true;
}

#line 25 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 25 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
#line 25 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
Integer BetaNegativeBinomialDistribution_<Arg1, Arg2, Arg3>::simulate() {
  #line 26 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return simulate_beta_negative_binomial(value(k), value(\u03b1), value(\u03b2));
}

#line 29 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 29 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
#line 29 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
std::optional<Integer> BetaNegativeBinomialDistribution_<Arg1, Arg2, Arg3>::simulateLazy() {
  #line 30 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return simulate_beta_negative_binomial(peek(k), peek(\u03b1), peek(\u03b2));
}

#line 33 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 33 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
#line 33 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
Real BetaNegativeBinomialDistribution_<Arg1, Arg2, Arg3>::logpdf(const Integer& x) {
  #line 34 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return logpdf_beta_negative_binomial(x, value(k), value(\u03b1), value(\u03b2));
}

#line 37 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 37 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
#line 37 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
std::optional<Expression<Real>> BetaNegativeBinomialDistribution_<Arg1, Arg2, Arg3>::logpdfLazy(const Expression<Integer>& x) {
  #line 38 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return box(logpdf_beta_negative_binomial(x, k, \u03b1, \u03b2));
}

#line 41 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 41 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
#line 41 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
std::optional<Delay> BetaNegativeBinomialDistribution_<Arg1, Arg2, Arg3>::update(const Integer& x) {
  #line 42 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return update_beta_negative_binomial(x, value(k), value(\u03b1), value(\u03b2));
}

#line 45 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 45 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
#line 45 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
std::optional<Delay> BetaNegativeBinomialDistribution_<Arg1, Arg2, Arg3>::updateLazy(const Expression<Integer>& x) {
  #line 46 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return update_beta_negative_binomial(x, k, \u03b1, \u03b2);
}

#line 49 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 49 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
#line 49 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
std::optional<Integer> BetaNegativeBinomialDistribution_<Arg1, Arg2, Arg3>::lower() {
  #line 50 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return Integer(0);
}

#line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
BinomialDistribution_<Arg1, Arg2>::BinomialDistribution_(const Arg1& n, const Arg2& \u03c1) :
    #line 8 "src/distribution/scalar_discrete/BinomialDistribution.birch"
    base_type_(),
    #line 13 "src/distribution/scalar_discrete/BinomialDistribution.birch"
    n(n),
    #line 18 "src/distribution/scalar_discrete/BinomialDistribution.birch"
    \u03c1(\u03c1) {
  //
}

#line 20 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 20 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 20 "src/distribution/scalar_discrete/BinomialDistribution.birch"
Boolean BinomialDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 21 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return true;
}

#line 24 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 24 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 24 "src/distribution/scalar_discrete/BinomialDistribution.birch"
Integer BinomialDistribution_<Arg1, Arg2>::simulate() {
  #line 25 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return simulate_binomial(value(n), value(\u03c1));
}

#line 28 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 28 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 28 "src/distribution/scalar_discrete/BinomialDistribution.birch"
std::optional<Integer> BinomialDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 29 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return simulate_binomial(peek(n), peek(\u03c1));
}

#line 32 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 32 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 32 "src/distribution/scalar_discrete/BinomialDistribution.birch"
Real BinomialDistribution_<Arg1, Arg2>::logpdf(const Integer& x) {
  #line 33 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return logpdf_binomial(x, value(n), value(\u03c1));
}

#line 36 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 36 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 36 "src/distribution/scalar_discrete/BinomialDistribution.birch"
std::optional<Expression<Real>> BinomialDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<Integer>& x) {
  #line 37 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return box(logpdf_binomial(x, n, \u03c1));
}

#line 40 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 40 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 40 "src/distribution/scalar_discrete/BinomialDistribution.birch"
std::optional<Real> BinomialDistribution_<Arg1, Arg2>::cdf(const Integer& x) {
  #line 41 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return cdf_binomial(x, value(n), value(\u03c1));
}

#line 44 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 44 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 44 "src/distribution/scalar_discrete/BinomialDistribution.birch"
std::optional<Integer> BinomialDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 45 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return quantile_binomial(P, value(n), value(\u03c1));
}

#line 48 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 48 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 48 "src/distribution/scalar_discrete/BinomialDistribution.birch"
std::optional<Integer> BinomialDistribution_<Arg1, Arg2>::lower() {
  #line 49 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return Integer(0);
}

#line 52 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 52 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 52 "src/distribution/scalar_discrete/BinomialDistribution.birch"
std::optional<Integer> BinomialDistribution_<Arg1, Arg2>::upper() {
  #line 53 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return value(n);
}

#line 56 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 56 "src/distribution/scalar_discrete/BinomialDistribution.birch"
#line 56 "src/distribution/scalar_discrete/BinomialDistribution.birch"
void BinomialDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 57 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  buffer->set(String("class"), String("Binomial"));
  #line 58 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  buffer->set(String("n"), value(n));
  #line 59 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  buffer->set(String("ρ"), value(\u03c1));
}

#line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
CategoricalDistribution_<Arg>::CategoricalDistribution_(const Arg& \u03c1) :
    #line 6 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    base_type_(),
    #line 11 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
    \u03c1(\u03c1) {
  //
}

#line 13 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 13 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 13 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Boolean CategoricalDistribution_<Arg>::supportsLazy() {
  #line 14 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return false;
}

#line 17 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 17 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 17 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Integer CategoricalDistribution_<Arg>::simulate() {
  #line 18 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return simulate_categorical(value(\u03c1));
}

#line 21 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 21 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 21 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
Real CategoricalDistribution_<Arg>::logpdf(const Integer& x) {
  #line 22 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return logpdf_categorical(x, value(\u03c1));
}

#line 25 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 25 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 25 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
std::optional<Real> CategoricalDistribution_<Arg>::cdf(const Integer& x) {
  #line 26 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return cdf_categorical(x, value(\u03c1));
}

#line 29 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 29 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 29 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
std::optional<Integer> CategoricalDistribution_<Arg>::quantile(const Real& P) {
  #line 30 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return quantile_categorical(P, value(\u03c1));
}

#line 33 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 33 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 33 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
std::optional<Integer> CategoricalDistribution_<Arg>::lower() {
  #line 34 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return if_then_else(length(eval(\u03c1)) > Integer(0), Integer(1), Integer(0));
}

#line 37 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 37 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 37 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
std::optional<Integer> CategoricalDistribution_<Arg>::upper() {
  #line 38 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return length(eval(\u03c1));
}

#line 41 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 41 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
#line 41 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
void CategoricalDistribution_<Arg>::write(const Buffer& buffer) {
  #line 42 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  buffer->set(String("class"), String("Categorical"));
  #line 43 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  buffer->set(String("ρ"), value(\u03c1));
}

#line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::ConwayMaxwellPoissonDistribution_(const Arg1& \u03bb, const Arg2& \u03bd, const Arg3& n) :
    #line 1 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
    base_type_(),
    #line 19 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
    \u03bb(\u03bb),
    #line 24 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
    \u03bd(\u03bd),
    #line 29 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
    n(n) {
  //
}

#line 31 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 31 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 31 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
Boolean ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::supportsLazy() {
  #line 32 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return true;
}

#line 35 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 35 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 35 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
Integer ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::simulate() {
  #line 36 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return simulate_conway_maxwell_poisson(value(\u03bb), value(\u03bd), value(n));
}

#line 39 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 39 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 39 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
std::optional<Integer> ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::simulateLazy() {
  #line 40 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return simulate_conway_maxwell_poisson(peek(\u03bb), peek(\u03bd), peek(n));
}

#line 43 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 43 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 43 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
Real ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::logpdf(const Integer& x) {
  #line 44 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return logpdf_conway_maxwell_poisson(x, value(\u03bb), value(\u03bd), value(n));
}

#line 47 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 47 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 47 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
std::optional<Expression<Real>> ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::logpdfLazy(const Expression<Integer>& x) {
  #line 48 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return box(logpdf_conway_maxwell_poisson(x, \u03bb, \u03bd, n));
}

#line 51 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 51 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 51 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
std::optional<Real> ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::cdf(const Integer& x) {
  #line 52 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return cdf_conway_maxwell_poisson(x, value(\u03bb), value(\u03bd), value(n));
}

#line 55 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 55 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 55 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
std::optional<Integer> ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::quantile(const Real& P) {
  #line 56 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return quantile_conway_maxwell_poisson(P, value(\u03bb), value(\u03bd), value(n));
}

#line 59 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 59 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 59 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
std::optional<Integer> ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::lower() {
  #line 60 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return Integer(0);
}

#line 63 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 63 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 63 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
std::optional<Integer> ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::upper() {
  #line 64 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return value(n);
}

#line 67 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 67 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
#line 67 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
void ConwayMaxwellPoissonDistribution_<Arg1, Arg2, Arg3>::write(const Buffer& buffer) {
  #line 68 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  buffer->set(String("class"), String("ConwayMaxwellPoisson"));
  #line 69 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  buffer->set(String("λ"), value(\u03bb));
  #line 70 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  buffer->set(String("ν"), value(\u03bd));
  #line 71 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  buffer->set(String("n"), value(n));
}

#line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
DirichletCategoricalDistribution_<Arg>::DirichletCategoricalDistribution_(const Arg& \u03b1) :
    #line 4 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
    base_type_(),
    #line 8 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
    \u03b1(\u03b1) {
  //
}

#line 10 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 10 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 10 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
Boolean DirichletCategoricalDistribution_<Arg>::supportsLazy() {
  #line 11 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  return false;
}

#line 14 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 14 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 14 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
Integer DirichletCategoricalDistribution_<Arg>::simulate() {
  #line 15 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  return simulate_dirichlet_categorical(value(\u03b1));
}

#line 18 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 18 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 18 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
Real DirichletCategoricalDistribution_<Arg>::logpdf(const Integer& x) {
  #line 19 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  return logpdf_dirichlet_categorical(x, value(\u03b1));
}

#line 22 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 22 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 22 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
std::optional<Delay> DirichletCategoricalDistribution_<Arg>::update(const Integer& x) {
  #line 23 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  return update_dirichlet_categorical(x, value(\u03b1));
}

#line 26 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 26 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 26 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
std::optional<Integer> DirichletCategoricalDistribution_<Arg>::lower() {
  #line 27 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  return Integer(1);
}

#line 30 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 30 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
#line 30 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
std::optional<Integer> DirichletCategoricalDistribution_<Arg>::upper() {
  #line 31 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  return rows(eval(\u03b1));
}

#line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
GammaPoissonDistribution_<Arg1, Arg2>::GammaPoissonDistribution_(const Arg1& k, const Arg2& \u03b8) :
    #line 8 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
    base_type_(),
    #line 13 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
    k(k),
    #line 18 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
    \u03b8(\u03b8) {
  //
}

#line 20 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 20 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 20 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Boolean GammaPoissonDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 21 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return true;
}

#line 24 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 24 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 24 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Integer GammaPoissonDistribution_<Arg1, Arg2>::simulate() {
  #line 25 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return simulate_gamma_poisson(value(k), value(\u03b8));
}

#line 28 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 28 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 28 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
std::optional<Integer> GammaPoissonDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 29 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return simulate_gamma_poisson(peek(k), peek(\u03b8));
}

#line 32 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 32 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 32 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
Real GammaPoissonDistribution_<Arg1, Arg2>::logpdf(const Integer& x) {
  #line 33 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return logpdf_gamma_poisson(x, value(k), value(\u03b8));
}

#line 36 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 36 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 36 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
std::optional<Expression<Real>> GammaPoissonDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<Integer>& x) {
  #line 37 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return box(logpdf_gamma_poisson(x, k, \u03b8));
}

#line 40 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 40 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 40 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
std::optional<Delay> GammaPoissonDistribution_<Arg1, Arg2>::update(const Integer& x) {
  #line 41 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return update_gamma_poisson(x, value(k), value(\u03b8));
}

#line 44 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 44 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 44 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
std::optional<Delay> GammaPoissonDistribution_<Arg1, Arg2>::updateLazy(const Expression<Integer>& x) {
  #line 45 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return update_gamma_poisson(x, k, \u03b8);
}

#line 48 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 48 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 48 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
std::optional<Real> GammaPoissonDistribution_<Arg1, Arg2>::cdf(const Integer& x) {
  #line 49 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return cdf_gamma_poisson(x, value(k), value(\u03b8));
}

#line 52 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 52 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 52 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
std::optional<Integer> GammaPoissonDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 53 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return quantile_gamma_poisson(P, value(k), value(\u03b8));
}

#line 56 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 56 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
#line 56 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
std::optional<Integer> GammaPoissonDistribution_<Arg1, Arg2>::lower() {
  #line 57 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return Integer(0);
}

#line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
NegativeBinomialDistribution_<Arg1, Arg2>::NegativeBinomialDistribution_(const Arg1& k, const Arg2& \u03c1) :
    #line 8 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
    base_type_(),
    #line 13 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
    k(k),
    #line 18 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
    \u03c1(\u03c1) {
  //
}

#line 20 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 20 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 20 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
Boolean NegativeBinomialDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 21 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  return true;
}

#line 24 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 24 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 24 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
Integer NegativeBinomialDistribution_<Arg1, Arg2>::simulate() {
  #line 25 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  return simulate_negative_binomial(value(k), value(\u03c1));
}

#line 28 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 28 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 28 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
std::optional<Integer> NegativeBinomialDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 29 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  return simulate_negative_binomial(peek(k), peek(\u03c1));
}

#line 32 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 32 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 32 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
Real NegativeBinomialDistribution_<Arg1, Arg2>::logpdf(const Integer& x) {
  #line 33 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  return logpdf_negative_binomial(x, value(k), value(\u03c1));
}

#line 36 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 36 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 36 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
std::optional<Expression<Real>> NegativeBinomialDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<Integer>& x) {
  #line 37 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  return box(logpdf_negative_binomial(x, k, \u03c1));
}

#line 40 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 40 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 40 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
std::optional<Real> NegativeBinomialDistribution_<Arg1, Arg2>::cdf(const Integer& x) {
  #line 41 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  return cdf_negative_binomial(x, value(k), value(\u03c1));
}

#line 44 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 44 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 44 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
std::optional<Integer> NegativeBinomialDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 45 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  return quantile_negative_binomial(P, value(k), value(\u03c1));
}

#line 48 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 48 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 48 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
std::optional<Integer> NegativeBinomialDistribution_<Arg1, Arg2>::lower() {
  #line 49 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  return Integer(0);
}

#line 52 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 52 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
#line 52 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
void NegativeBinomialDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 53 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  buffer->set(String("class"), String("NegativeBinomial"));
  #line 54 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  buffer->set(String("k"), value(k));
  #line 55 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  buffer->set(String("ρ"), value(\u03c1));
}

#line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
PoissonDistribution_<Arg>::PoissonDistribution_(const Arg& \u03bb) :
    #line 8 "src/distribution/scalar_discrete/PoissonDistribution.birch"
    base_type_(),
    #line 12 "src/distribution/scalar_discrete/PoissonDistribution.birch"
    \u03bb(\u03bb) {
  //
}

#line 14 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 14 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 14 "src/distribution/scalar_discrete/PoissonDistribution.birch"
Boolean PoissonDistribution_<Arg>::supportsLazy() {
  #line 15 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  return true;
}

#line 18 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 18 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 18 "src/distribution/scalar_discrete/PoissonDistribution.birch"
Integer PoissonDistribution_<Arg>::simulate() {
  #line 19 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  return simulate_poisson(value(\u03bb));
}

#line 22 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 22 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 22 "src/distribution/scalar_discrete/PoissonDistribution.birch"
std::optional<Integer> PoissonDistribution_<Arg>::simulateLazy() {
  #line 23 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  return simulate_poisson(peek(\u03bb));
}

#line 26 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 26 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 26 "src/distribution/scalar_discrete/PoissonDistribution.birch"
Real PoissonDistribution_<Arg>::logpdf(const Integer& x) {
  #line 27 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  return logpdf_poisson(x, value(\u03bb));
}

#line 30 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 30 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 30 "src/distribution/scalar_discrete/PoissonDistribution.birch"
std::optional<Expression<Real>> PoissonDistribution_<Arg>::logpdfLazy(const Expression<Integer>& x) {
  #line 31 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  return box(logpdf_poisson(x, \u03bb));
}

#line 34 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 34 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 34 "src/distribution/scalar_discrete/PoissonDistribution.birch"
std::optional<Real> PoissonDistribution_<Arg>::cdf(const Integer& x) {
  #line 35 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  return cdf_poisson(x, value(\u03bb));
}

#line 38 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 38 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 38 "src/distribution/scalar_discrete/PoissonDistribution.birch"
std::optional<Integer> PoissonDistribution_<Arg>::quantile(const Real& P) {
  #line 39 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  return quantile_poisson(P, value(\u03bb));
}

#line 42 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 42 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 42 "src/distribution/scalar_discrete/PoissonDistribution.birch"
std::optional<Integer> PoissonDistribution_<Arg>::lower() {
  #line 43 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  return Integer(0);
}

#line 46 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 46 "src/distribution/scalar_discrete/PoissonDistribution.birch"
#line 46 "src/distribution/scalar_discrete/PoissonDistribution.birch"
void PoissonDistribution_<Arg>::write(const Buffer& buffer) {
  #line 47 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  buffer->set(String("class"), String("Poisson"));
  #line 48 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  buffer->set(String("λ"), value(\u03bb));
}

#line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
RestaurantCategoricalDistribution_<Arg1, Arg2, Arg3, Arg4>::RestaurantCategoricalDistribution_(const Arg1& \u03b1, const Arg2& \u03b8, const Arg3& n, const Arg4& N) :
    #line 4 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
    \u03b1(\u03b1),
    #line 14 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
    \u03b8(\u03b8),
    #line 19 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
    n(n),
    #line 24 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
    N(N) {
  //
}

#line 26 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 26 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
#line 26 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
Boolean RestaurantCategoricalDistribution_<Arg1, Arg2, Arg3, Arg4>::supportsLazy() {
  #line 27 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  return false;
}

#line 30 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 30 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
#line 30 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
Integer RestaurantCategoricalDistribution_<Arg1, Arg2, Arg3, Arg4>::simulate() {
  #line 31 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  return simulate_restaurant_categorical(value(\u03b1), value(\u03b8), value(n), value(N));
}

#line 35 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 35 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
#line 35 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
Real RestaurantCategoricalDistribution_<Arg1, Arg2, Arg3, Arg4>::logpdf(const Integer& x) {
  #line 36 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  return logpdf_restaurant_categorical(x, value(\u03b1), value(\u03b8), value(n), value(N));
}

#line 40 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 40 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
#line 40 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
std::optional<Delay> RestaurantCategoricalDistribution_<Arg1, Arg2, Arg3, Arg4>::update(const Integer& x) {
  #line 41 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  return update_restaurant_categorical(x, value(\u03b1), value(\u03b8), value(n), value(N));
}

#line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::ScaledGammaPoissonDistribution_(const Arg1& a, const Arg2& k, const Arg3& \u03b8) :
    #line 4 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    a(a),
    #line 14 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    k(k),
    #line 19 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    \u03b8(\u03b8) {
  //
}

#line 21 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 21 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 21 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
Boolean ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::supportsLazy() {
  #line 22 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return true;
}

#line 25 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 25 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 25 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
Integer ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::simulate() {
  #line 26 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return simulate_gamma_poisson(value(k), value(a) * value(\u03b8));
}

#line 29 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 29 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 29 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
std::optional<Integer> ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::simulateLazy() {
  #line 30 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return simulate_gamma_poisson(peek(k), peek(a) * peek(\u03b8));
}

#line 33 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 33 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 33 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
Real ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::logpdf(const Integer& x) {
  #line 34 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return logpdf_gamma_poisson(x, value(k), value(a) * value(\u03b8));
}

#line 37 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 37 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 37 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
std::optional<Expression<Real>> ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::logpdfLazy(const Expression<Integer>& x) {
  #line 38 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return box(logpdf_gamma_poisson(x, k, a * \u03b8));
}

#line 41 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 41 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 41 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
std::optional<Delay> ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::update(const Integer& x) {
  #line 42 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return update_scaled_gamma_poisson(x, value(a), value(k), value(\u03b8));
}

#line 45 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 45 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 45 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
std::optional<Delay> ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::updateLazy(const Expression<Integer>& x) {
  #line 46 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return update_scaled_gamma_poisson(x, a, k, \u03b8);
}

#line 49 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 49 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 49 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
std::optional<Real> ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::cdf(const Integer& x) {
  #line 50 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return cdf_gamma_poisson(x, value(k), value(a) * value(\u03b8));
}

#line 53 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 53 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 53 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
std::optional<Integer> ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::quantile(const Real& P) {
  #line 54 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return quantile_gamma_poisson(P, value(k), value(a) * value(\u03b8));
}

#line 57 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 57 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
#line 57 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
std::optional<Integer> ScaledGammaPoissonDistribution_<Arg1, Arg2, Arg3>::lower() {
  #line 58 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return Integer(0);
}

#line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
UniformIntegerDistribution_<Arg1, Arg2>::UniformIntegerDistribution_(const Arg1& l, const Arg2& u) :
    #line 8 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
    base_type_(),
    #line 13 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
    l(l),
    #line 18 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
    u(u) {
  //
}

#line 20 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 20 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 20 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Boolean UniformIntegerDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 21 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  return false;
}

#line 24 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 24 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 24 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Integer UniformIntegerDistribution_<Arg1, Arg2>::simulate() {
  #line 25 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  return simulate_uniform_int(value(l), value(u));
}

#line 28 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 28 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 28 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
Real UniformIntegerDistribution_<Arg1, Arg2>::logpdf(const Integer& x) {
  #line 29 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  return logpdf_uniform_int(x, value(l), value(u));
}

#line 32 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 32 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 32 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
std::optional<Real> UniformIntegerDistribution_<Arg1, Arg2>::cdf(const Integer& x) {
  #line 33 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  return cdf_uniform_int(x, value(l), value(u));
}

#line 36 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 36 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 36 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
std::optional<Integer> UniformIntegerDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 37 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  return quantile_uniform_int(P, value(l), value(u));
}

#line 40 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 40 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 40 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
std::optional<Integer> UniformIntegerDistribution_<Arg1, Arg2>::lower() {
  #line 41 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  return value(l);
}

#line 44 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 44 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 44 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
std::optional<Integer> UniformIntegerDistribution_<Arg1, Arg2>::upper() {
  #line 45 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  return value(u);
}

#line 48 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 48 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
#line 48 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
void UniformIntegerDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 49 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  buffer->set(String("class"), String("UniformInteger"));
  #line 50 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  buffer->set(String("l"), value(l));
  #line 51 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  buffer->set(String("u"), value(u));
}

#line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
DeltaDistribution_<Arg>::DeltaDistribution_(const Arg& \u03bc) :
    #line 7 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
    base_type_(),
    #line 11 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
    \u03bc(\u03bc) {
  //
}

#line 13 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 13 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
#line 13 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
Integer DeltaDistribution_<Arg>::simulate() {
  #line 14 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  return value(\u03bc);
}

#line 17 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 17 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
#line 17 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
Real DeltaDistribution_<Arg>::logpdf(const Integer& x) {
  #line 18 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  return if_then_else(x == value(\u03bc), 0.0, -(std::numeric_limits<double>::infinity()));
}

#line 21 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 21 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
#line 21 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
std::optional<Integer> DeltaDistribution_<Arg>::lower() {
  #line 22 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  return value(\u03bc);
}

#line 25 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 25 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
#line 25 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
std::optional<Integer> DeltaDistribution_<Arg>::upper() {
  #line 26 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  return value(\u03bc);
}

#line 29 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 29 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
#line 29 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
void DeltaDistribution_<Arg>::write(const Buffer& buffer) {
  #line 30 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  buffer->set(String("class"), String("Delta"));
  #line 31 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  buffer->set(String("μ"), value(\u03bc));
}

#line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
GaussianDistribution_<Arg1, Arg2>::GaussianDistribution_(const Arg1& \u03bc, const Arg2& \u03c32) :
    #line 3 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
    base_type_(),
    #line 12 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
    \u03bc(\u03bc),
    #line 17 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
    \u03c32(\u03c32) {
  //
}

#line 19 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 19 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 19 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
Boolean GaussianDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 20 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return true;
}

#line 23 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 23 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 23 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
Real GaussianDistribution_<Arg1, Arg2>::simulate() {
  #line 24 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return simulate_gaussian(value(\u03bc), value(\u03c32));
}

#line 27 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 27 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 27 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
std::optional<Real> GaussianDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 28 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return simulate_gaussian(peek(\u03bc), peek(\u03c32));
}

#line 31 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 31 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 31 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
Real GaussianDistribution_<Arg1, Arg2>::logpdf(const Real& x) {
  #line 32 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return logpdf_gaussian(x, value(\u03bc), value(\u03c32));
}

#line 35 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 35 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 35 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
std::optional<Expression<Real>> GaussianDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<Real>& x) {
  #line 36 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return box(logpdf_gaussian(x, \u03bc, \u03c32));
}

#line 39 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 39 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 39 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
std::optional<Real> GaussianDistribution_<Arg1, Arg2>::cdf(const Real& x) {
  #line 40 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return cdf_gaussian(x, value(\u03bc), value(\u03c32));
}

#line 43 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 43 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 43 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
std::optional<Real> GaussianDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 44 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return quantile_gaussian(P, value(\u03bc), value(\u03c32));
}

#line 47 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 47 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 47 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
Boolean GaussianDistribution_<Arg1, Arg2>::isGaussian() {
  #line 48 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return true;
}

#line 51 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 51 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 51 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> GaussianDistribution_<Arg1, Arg2>::getGaussian() {
  #line 52 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return box(\u03bc, \u03c32);
}

#line 55 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 55 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
#line 55 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
void GaussianDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 56 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  buffer->set(String("class"), String("Gaussian"));
  #line 57 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  buffer->set(String("μ"), value(\u03bc));
  #line 58 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  buffer->set(String("σ2"), value(\u03c32));
}

#line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::GaussianGaussianDistribution_(const Arg1& \u03bc, const Arg2& \u03c32, const Arg3& s2) :
    #line 4 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
    \u03bc(\u03bc),
    #line 14 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
    \u03c32(\u03c32),
    #line 19 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
    s2(s2) {
  //
}

#line 21 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 21 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 21 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
Boolean GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::supportsLazy() {
  #line 22 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return true;
}

#line 25 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 25 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 25 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
Real GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::simulate() {
  #line 26 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return simulate_gaussian_gaussian(value(\u03bc), value(\u03c32), value(s2));
}

#line 29 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 29 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 29 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
std::optional<Real> GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::simulateLazy() {
  #line 30 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return simulate_gaussian_gaussian(peek(\u03bc), peek(\u03c32), peek(s2));
}

#line 33 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 33 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 33 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
Real GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::logpdf(const Real& x) {
  #line 34 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return logpdf_gaussian_gaussian(x, value(\u03bc), value(\u03c32), value(s2));
}

#line 37 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 37 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 37 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
std::optional<Expression<Real>> GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::logpdfLazy(const Expression<Real>& x) {
  #line 38 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return box(logpdf_gaussian_gaussian(x, \u03bc, \u03c32, s2));
}

#line 41 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 41 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 41 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
std::optional<Delay> GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::update(const Real& x) {
  #line 42 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return update_gaussian_gaussian(x, value(\u03bc), value(\u03c32), value(s2));
}

#line 45 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 45 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 45 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
std::optional<Delay> GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::updateLazy(const Expression<Real>& x) {
  #line 46 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return update_gaussian_gaussian(x, \u03bc, \u03c32, s2);
}

#line 49 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 49 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 49 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
std::optional<Real> GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::cdf(const Real& x) {
  #line 50 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return cdf_gaussian_gaussian(x, value(\u03bc), value(\u03c32), value(s2));
}

#line 53 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 53 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 53 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
std::optional<Real> GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::quantile(const Real& P) {
  #line 54 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return quantile_gaussian_gaussian(P, value(\u03bc), value(\u03c32), value(s2));
}

#line 57 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 57 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 57 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
Boolean GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::isGaussian() {
  #line 58 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return true;
}

#line 61 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 61 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
#line 61 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> GaussianGaussianDistribution_<Arg1, Arg2, Arg3>::getGaussian() {
  #line 62 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return box(\u03bc, \u03c32 + s2);
}

#line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::LinearGaussianGaussianDistribution_(const Arg1& a, const Arg2& \u03bc, const Arg3& \u03c32, const Arg4& c, const Arg5& s2) :
    #line 4 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
    a(a),
    #line 14 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
    \u03bc(\u03bc),
    #line 19 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
    \u03c32(\u03c32),
    #line 24 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
    c(c),
    #line 29 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
    s2(s2) {
  //
}

#line 31 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 31 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 31 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
Boolean LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::supportsLazy() {
  #line 32 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return true;
}

#line 35 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 35 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 35 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
Real LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulate() {
  #line 36 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return simulate_linear_gaussian_gaussian(value(a), value(\u03bc), value(\u03c32), value(c), value(s2));
}

#line 40 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 40 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 40 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
std::optional<Real> LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulateLazy() {
  #line 41 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return simulate_linear_gaussian_gaussian(peek(a), peek(\u03bc), peek(\u03c32), peek(c), peek(s2));
}

#line 45 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 45 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 45 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
Real LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdf(const Real& x) {
  #line 46 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return logpdf_linear_gaussian_gaussian(x, value(a), value(\u03bc), value(\u03c32), value(c), value(s2));
}

#line 50 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 50 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 50 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
std::optional<Expression<Real>> LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdfLazy(const Expression<Real>& x) {
  #line 51 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return box(logpdf_linear_gaussian_gaussian(x, a, \u03bc, \u03c32, c, s2));
}

#line 54 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 54 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 54 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
std::optional<Delay> LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::update(const Real& x) {
  #line 55 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return update_linear_gaussian_gaussian(x, value(a), value(\u03bc), value(\u03c32), value(c), value(s2));
}

#line 59 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 59 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 59 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
std::optional<Delay> LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::updateLazy(const Expression<Real>& x) {
  #line 60 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return update_linear_gaussian_gaussian(x, a, \u03bc, \u03c32, c, s2);
}

#line 63 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 63 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 63 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
std::optional<Real> LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::cdf(const Real& x) {
  #line 64 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return cdf_linear_gaussian_gaussian(x, value(a), value(\u03bc), value(\u03c32), value(c), value(s2));
}

#line 68 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 68 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 68 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
std::optional<Real> LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::quantile(const Real& P) {
  #line 69 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return quantile_linear_gaussian_gaussian(P, value(a), value(\u03bc), value(\u03c32), value(c), value(s2));
}

#line 73 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 73 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 73 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
Boolean LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::isGaussian() {
  #line 74 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return true;
}

#line 77 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 77 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
#line 77 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> LinearGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::getGaussian() {
  #line 78 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return box(a * \u03bc + c, a * a * \u03c32 + s2);
}

#line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::LinearMultivariateGaussianGaussianDistribution_(const Arg1& a, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& c, const Arg5& s2) :
    #line 4 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
    a(a),
    #line 14 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
    \u03bc(\u03bc),
    #line 19 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
    \u03a3(\u03a3),
    #line 24 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
    c(c),
    #line 29 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
    s2(s2) {
  //
}

#line 31 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 31 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 31 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
Boolean LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::supportsLazy() {
  #line 32 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return true;
}

#line 35 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 35 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 35 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
Real LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulate() {
  #line 36 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return simulate_linear_multivariate_gaussian_gaussian(value(a), value(\u03bc), value(\u03a3), value(c), value(s2));
}

#line 40 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 40 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 40 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
std::optional<Real> LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulateLazy() {
  #line 41 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return simulate_linear_multivariate_gaussian_gaussian(peek(a), peek(\u03bc), peek(\u03a3), peek(c), peek(s2));
}

#line 45 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 45 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 45 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
Real LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdf(const Real& x) {
  #line 46 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return logpdf_linear_multivariate_gaussian_gaussian(x, value(a), value(\u03bc), value(\u03a3), value(c), value(s2));
}

#line 50 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 50 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 50 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
std::optional<Expression<Real>> LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdfLazy(const Expression<Real>& x) {
  #line 51 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return box(logpdf_linear_multivariate_gaussian_gaussian(x, a, \u03bc, \u03a3, c, s2));
}

#line 55 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 55 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 55 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
std::optional<Delay> LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::update(const Real& x) {
  #line 56 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return update_linear_multivariate_gaussian_gaussian(x, value(a), value(\u03bc), value(\u03a3), value(c), value(s2));
}

#line 60 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 60 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 60 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
std::optional<Delay> LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::updateLazy(const Expression<Real>& x) {
  #line 61 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return update_linear_multivariate_gaussian_gaussian(x, a, \u03bc, \u03a3, c, s2);
}

#line 64 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 64 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 64 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
std::optional<Real> LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::cdf(const Real& x) {
  #line 65 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return cdf_linear_multivariate_gaussian_gaussian(x, value(a), value(\u03bc), value(\u03a3), value(c), value(s2));
}

#line 69 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 69 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 69 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
std::optional<Real> LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::quantile(const Real& P) {
  #line 70 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return quantile_linear_multivariate_gaussian_gaussian(P, value(a), value(\u03bc), value(\u03a3), value(c), value(s2));
}

#line 74 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 74 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 74 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
Boolean LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::isGaussian() {
  #line 75 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return true;
}

#line 78 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 78 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
#line 78 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> LinearMultivariateGaussianGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::getGaussian() {
  #line 79 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return box(dot(a, \u03bc) + c, dot(a, \u03a3 * a) + s2);
}

#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::LinearMultivariateNormalInverseGammaGaussianDistribution_(const Arg1& a, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& s2) :
    #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    base_type_(),
    #line 10 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    a(a),
    #line 15 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u03bd(\u03bd),
    #line 20 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u039b(\u039b),
    #line 25 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u03b1(\u03b1),
    #line 30 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u03b2(\u03b2),
    #line 35 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    c(c),
    #line 40 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    s2(s2) {
  //
}

#line 42 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 42 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 42 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Boolean LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::supportsLazy() {
  #line 43 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return true;
}

#line 46 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 46 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 46 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Real LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::simulate() {
  #line 47 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return simulate_linear_multivariate_normal_inverse_gamma_gaussian(value(a), value(\u03bd), value(\u039b), value(c), value(\u03b1), value(\u03b2), value(s2));
}

#line 52 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 52 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 52 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
std::optional<Real> LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::simulateLazy() {
  #line 53 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return simulate_linear_multivariate_normal_inverse_gamma_gaussian(peek(a), peek(\u03bd), peek(\u039b), peek(c), peek(\u03b1), peek(\u03b2), peek(s2));
}

#line 58 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 58 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 58 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
Real LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::logpdf(const Real& x) {
  #line 59 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return logpdf_linear_multivariate_normal_inverse_gamma_gaussian(x, value(a), value(\u03bd), value(\u039b), value(c), value(\u03b1), value(\u03b2), value(s2));
}

#line 64 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 64 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 64 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
std::optional<Expression<Real>> LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::logpdfLazy(const Expression<Real>& x) {
  #line 65 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return box(logpdf_linear_multivariate_normal_inverse_gamma_gaussian(x, a, \u03bd, \u039b, c, \u03b1, \u03b2, s2));
}

#line 69 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 69 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 69 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
std::optional<Delay> LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::update(const Real& x) {
  #line 70 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return update_linear_multivariate_normal_inverse_gamma_gaussian(x, value(a), value(\u03bd), value(\u039b), value(c), value(\u03b1), value(\u03b2), value(s2));
}

#line 75 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 75 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 75 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
std::optional<Delay> LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::updateLazy(const Expression<Real>& x) {
  #line 76 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return update_linear_multivariate_normal_inverse_gamma_gaussian(x, a, \u03bd, \u039b, c, \u03b1, \u03b2, s2);
}

#line 80 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 80 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 80 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
std::optional<Real> LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::cdf(const Real& x) {
  #line 81 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return cdf_linear_multivariate_normal_inverse_gamma_gaussian(x, value(a), value(\u03bd), value(\u039b), value(c), value(\u03b1), value(\u03b2), value(s2));
}

#line 85 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 85 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
#line 85 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
std::optional<Real> LinearMultivariateNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::quantile(const Real& P) {
  #line 86 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return quantile_linear_multivariate_normal_inverse_gamma_gaussian(P, value(a), value(\u03bd), value(\u039b), value(c), value(\u03b1), value(\u03b2), value(s2));
}

#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::LinearNormalInverseGammaGaussianDistribution_(const Arg1& a, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& s2) :
    #line 4 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    base_type_(),
    #line 10 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    a(a),
    #line 15 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03bd(\u03bd),
    #line 20 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03bb(\u03bb),
    #line 25 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03b1(\u03b1),
    #line 30 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03b2(\u03b2),
    #line 35 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    c(c),
    #line 40 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    s2(s2) {
  //
}

#line 42 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 42 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 42 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Boolean LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::supportsLazy() {
  #line 43 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return true;
}

#line 46 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 46 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 46 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Real LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::simulate() {
  #line 47 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return simulate_linear_normal_inverse_gamma_gaussian(value(a), value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2), value(c), value(s2));
}

#line 51 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 51 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 51 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
std::optional<Real> LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::simulateLazy() {
  #line 52 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return simulate_linear_normal_inverse_gamma_gaussian(peek(a), peek(\u03bd), peek(\u03bb), peek(\u03b1), peek(\u03b2), peek(c), peek(s2));
}

#line 56 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 56 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 56 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
Real LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::logpdf(const Real& x) {
  #line 57 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return logpdf_linear_normal_inverse_gamma_gaussian(x, value(a), value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2), value(c), value(s2));
}

#line 61 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 61 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 61 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
std::optional<Expression<Real>> LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::logpdfLazy(const Expression<Real>& x) {
  #line 62 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return box(logpdf_linear_normal_inverse_gamma_gaussian(x, a, \u03bd, \u03bb, \u03b1, \u03b2, c, s2));
}

#line 66 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 66 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 66 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
std::optional<Delay> LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::update(const Real& x) {
  #line 67 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return update_linear_normal_inverse_gamma_gaussian(x, value(a), value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2), value(c), value(s2));
}

#line 71 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 71 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 71 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
std::optional<Delay> LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::updateLazy(const Expression<Real>& x) {
  #line 72 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return update_linear_normal_inverse_gamma_gaussian(x, a, \u03bd, \u03bb, \u03b1, \u03b2, c, s2);
}

#line 76 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 76 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 76 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
std::optional<Real> LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::cdf(const Real& x) {
  #line 77 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return cdf_linear_normal_inverse_gamma_gaussian(x, value(a), value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2), value(c), value(s2));
}

#line 81 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 81 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
#line 81 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
std::optional<Real> LinearNormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>::quantile(const Real& P) {
  #line 82 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return quantile_linear_normal_inverse_gamma_gaussian(P, value(a), value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2), value(c), value(s2));
}

#line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::NormalInverseGammaDistribution_(const Arg1& \u03bd, const Arg2& \u03bb, const Arg3& \u03b1, const Arg4& \u03b2) :
    #line 28 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    base_type_(),
    #line 33 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    \u03bd(\u03bd),
    #line 38 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    \u03bb(\u03bb),
    #line 43 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    \u03b1(\u03b1),
    #line 48 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    \u03b2(\u03b2) {
  //
}

#line 50 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 50 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 50 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Boolean NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::supportsLazy() {
  #line 51 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return true;
}

#line 54 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 54 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 54 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Real NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::simulate() {
  #line 55 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return simulate_normal_inverse_gamma(value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2));
}

#line 59 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 59 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 59 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
std::optional<Real> NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::simulateLazy() {
  #line 60 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return simulate_normal_inverse_gamma(peek(\u03bd), peek(\u03bb), peek(\u03b1), peek(\u03b2));
}

#line 63 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 63 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 63 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Real NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::logpdf(const Real& x) {
  #line 64 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return logpdf_normal_inverse_gamma(x, value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2));
}

#line 68 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 68 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 68 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
std::optional<Expression<Real>> NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::logpdfLazy(const Expression<Real>& x) {
  #line 69 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return box(logpdf_normal_inverse_gamma(x, \u03bd, \u03bb, \u03b1, \u03b2));
}

#line 72 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 72 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 72 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
std::optional<Delay> NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::update(const Real& x) {
  #line 73 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return update_normal_inverse_gamma(x, value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2));
}

#line 77 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 77 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 77 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
std::optional<Delay> NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::updateLazy(const Expression<Real>& x) {
  #line 78 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return update_normal_inverse_gamma(x, \u03bd, \u03bb, \u03b1, \u03b2);
}

#line 81 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 81 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 81 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
std::optional<Real> NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::cdf(const Real& x) {
  #line 82 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return cdf_normal_inverse_gamma(x, value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2));
}

#line 86 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 86 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 86 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
std::optional<Real> NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::quantile(const Real& P) {
  #line 87 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return quantile_normal_inverse_gamma(P, value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2));
}

#line 91 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 91 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 91 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
Boolean NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::isNormalInverseGamma() {
  #line 92 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return true;
}

#line 95 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 95 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 95 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>, Expression<Real>, Expression<Real>>> NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::getNormalInverseGamma() {
  #line 97 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return box(\u03bd, \u03bb, \u03b1, \u03b2);
}

#line 100 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 100 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
#line 100 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
void NormalInverseGammaDistribution_<Arg1, Arg2, Arg3, Arg4>::write(const Buffer& buffer) {
  #line 101 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  buffer->set(String("class"), String("NormalInverseGamma"));
  #line 102 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  buffer->set(String("ν"), value(\u03bd));
  #line 103 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  buffer->set(String("λ"), value(\u03bb));
  #line 104 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  buffer->set(String("α"), value(\u03b1));
  #line 105 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  buffer->set(String("β"), value(\u03b2));
}

#line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::NormalInverseGammaGaussianDistribution_(const Arg1& \u03bd, const Arg2& \u03bb, const Arg3& \u03b1, const Arg4& \u03b2, const Arg5& s2) :
    #line 4 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    \u03bd(\u03bd),
    #line 14 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    \u03bb(\u03bb),
    #line 19 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    \u03b1(\u03b1),
    #line 24 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    \u03b2(\u03b2),
    #line 29 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    s2(s2) {
  //
}

#line 31 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 31 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 31 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
Boolean NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::supportsLazy() {
  #line 32 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return true;
}

#line 35 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 35 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 35 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
Real NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulate() {
  #line 36 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return simulate_normal_inverse_gamma_gaussian(value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2), value(s2));
}

#line 40 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 40 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 40 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
std::optional<Real> NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::simulateLazy() {
  #line 41 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return simulate_normal_inverse_gamma_gaussian(peek(\u03bd), peek(\u03bb), peek(\u03b1), peek(\u03b2), peek(s2));
}

#line 45 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 45 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 45 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
Real NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdf(const Real& x) {
  #line 46 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return logpdf_normal_inverse_gamma_gaussian(x, value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2), value(s2));
}

#line 50 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 50 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 50 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
std::optional<Expression<Real>> NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::logpdfLazy(const Expression<Real>& x) {
  #line 51 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return box(logpdf_normal_inverse_gamma_gaussian(x, \u03bd, \u03bb, \u03b1, \u03b2, s2));
}

#line 54 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 54 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 54 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
std::optional<Delay> NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::update(const Real& x) {
  #line 55 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return update_normal_inverse_gamma_gaussian(x, value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2), value(s2));
}

#line 59 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 59 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 59 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
std::optional<Delay> NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::updateLazy(const Expression<Real>& x) {
  #line 60 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return update_normal_inverse_gamma_gaussian(x, \u03bd, \u03bb, \u03b1, \u03b2, s2);
}

#line 63 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 63 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 63 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
std::optional<Real> NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::cdf(const Real& x) {
  #line 64 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return cdf_normal_inverse_gamma_gaussian(x, value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2), value(s2));
}

#line 68 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 68 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
#line 68 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
std::optional<Real> NormalInverseGammaGaussianDistribution_<Arg1, Arg2, Arg3, Arg4, Arg5>::quantile(const Real& P) {
  #line 69 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return quantile_normal_inverse_gamma_gaussian(P, value(\u03bd), value(\u03bb), value(\u03b1), value(\u03b2), value(s2));
}

#line 8 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/BetaDistribution.birch"
BetaDistribution_<Arg1, Arg2>::BetaDistribution_(const Arg1& \u03b1, const Arg2& \u03b2) :
    #line 8 "src/distribution/scalar_other/BetaDistribution.birch"
    base_type_(),
    #line 12 "src/distribution/scalar_other/BetaDistribution.birch"
    \u03b1(\u03b1),
    #line 17 "src/distribution/scalar_other/BetaDistribution.birch"
    \u03b2(\u03b2) {
  //
}

#line 19 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 19 "src/distribution/scalar_other/BetaDistribution.birch"
#line 19 "src/distribution/scalar_other/BetaDistribution.birch"
Boolean BetaDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 20 "src/distribution/scalar_other/BetaDistribution.birch"
  return true;
}

#line 23 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 23 "src/distribution/scalar_other/BetaDistribution.birch"
#line 23 "src/distribution/scalar_other/BetaDistribution.birch"
Real BetaDistribution_<Arg1, Arg2>::simulate() {
  #line 24 "src/distribution/scalar_other/BetaDistribution.birch"
  return simulate_beta(value(\u03b1), value(\u03b2));
}

#line 27 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 27 "src/distribution/scalar_other/BetaDistribution.birch"
#line 27 "src/distribution/scalar_other/BetaDistribution.birch"
std::optional<Real> BetaDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 28 "src/distribution/scalar_other/BetaDistribution.birch"
  return simulate_beta(peek(\u03b1), peek(\u03b2));
}

#line 31 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 31 "src/distribution/scalar_other/BetaDistribution.birch"
#line 31 "src/distribution/scalar_other/BetaDistribution.birch"
Real BetaDistribution_<Arg1, Arg2>::logpdf(const Real& x) {
  #line 32 "src/distribution/scalar_other/BetaDistribution.birch"
  return logpdf_beta(x, value(\u03b1), value(\u03b2));
}

#line 35 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 35 "src/distribution/scalar_other/BetaDistribution.birch"
#line 35 "src/distribution/scalar_other/BetaDistribution.birch"
std::optional<Expression<Real>> BetaDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<Real>& x) {
  #line 36 "src/distribution/scalar_other/BetaDistribution.birch"
  return box(logpdf_beta(x, \u03b1, \u03b2));
}

#line 39 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 39 "src/distribution/scalar_other/BetaDistribution.birch"
#line 39 "src/distribution/scalar_other/BetaDistribution.birch"
std::optional<Real> BetaDistribution_<Arg1, Arg2>::cdf(const Real& x) {
  #line 40 "src/distribution/scalar_other/BetaDistribution.birch"
  return cdf_beta(x, value(\u03b1), value(\u03b2));
}

#line 43 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 43 "src/distribution/scalar_other/BetaDistribution.birch"
#line 43 "src/distribution/scalar_other/BetaDistribution.birch"
std::optional<Real> BetaDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 44 "src/distribution/scalar_other/BetaDistribution.birch"
  return quantile_beta(P, value(\u03b1), value(\u03b2));
}

#line 47 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 47 "src/distribution/scalar_other/BetaDistribution.birch"
#line 47 "src/distribution/scalar_other/BetaDistribution.birch"
std::optional<Real> BetaDistribution_<Arg1, Arg2>::lower() {
  #line 48 "src/distribution/scalar_other/BetaDistribution.birch"
  return 0.0;
}

#line 51 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 51 "src/distribution/scalar_other/BetaDistribution.birch"
#line 51 "src/distribution/scalar_other/BetaDistribution.birch"
std::optional<Real> BetaDistribution_<Arg1, Arg2>::upper() {
  #line 52 "src/distribution/scalar_other/BetaDistribution.birch"
  return 1.0;
}

#line 55 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 55 "src/distribution/scalar_other/BetaDistribution.birch"
#line 55 "src/distribution/scalar_other/BetaDistribution.birch"
Boolean BetaDistribution_<Arg1, Arg2>::isBeta() {
  #line 56 "src/distribution/scalar_other/BetaDistribution.birch"
  return true;
}

#line 59 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 59 "src/distribution/scalar_other/BetaDistribution.birch"
#line 59 "src/distribution/scalar_other/BetaDistribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> BetaDistribution_<Arg1, Arg2>::getBeta() {
  #line 60 "src/distribution/scalar_other/BetaDistribution.birch"
  return box(\u03b1, \u03b2);
}

#line 63 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 63 "src/distribution/scalar_other/BetaDistribution.birch"
#line 63 "src/distribution/scalar_other/BetaDistribution.birch"
void BetaDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 64 "src/distribution/scalar_other/BetaDistribution.birch"
  buffer->set(String("class"), String("Beta"));
  #line 65 "src/distribution/scalar_other/BetaDistribution.birch"
  buffer->set(String("α"), value(\u03b1));
  #line 66 "src/distribution/scalar_other/BetaDistribution.birch"
  buffer->set(String("β"), value(\u03b2));
}

#line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
ExponentialDistribution_<Arg1>::ExponentialDistribution_(const Arg1& \u03bb) :
    #line 8 "src/distribution/scalar_other/ExponentialDistribution.birch"
    base_type_(),
    #line 12 "src/distribution/scalar_other/ExponentialDistribution.birch"
    \u03bb(\u03bb) {
  //
}

#line 14 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 14 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 14 "src/distribution/scalar_other/ExponentialDistribution.birch"
Boolean ExponentialDistribution_<Arg1>::supportsLazy() {
  #line 15 "src/distribution/scalar_other/ExponentialDistribution.birch"
  return true;
}

#line 18 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 18 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 18 "src/distribution/scalar_other/ExponentialDistribution.birch"
Real ExponentialDistribution_<Arg1>::simulate() {
  #line 19 "src/distribution/scalar_other/ExponentialDistribution.birch"
  return simulate_exponential(value(\u03bb));
}

#line 22 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 22 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 22 "src/distribution/scalar_other/ExponentialDistribution.birch"
std::optional<Real> ExponentialDistribution_<Arg1>::simulateLazy() {
  #line 23 "src/distribution/scalar_other/ExponentialDistribution.birch"
  return simulate_exponential(peek(\u03bb));
}

#line 26 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 26 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 26 "src/distribution/scalar_other/ExponentialDistribution.birch"
Real ExponentialDistribution_<Arg1>::logpdf(const Real& x) {
  #line 27 "src/distribution/scalar_other/ExponentialDistribution.birch"
  return logpdf_exponential(x, value(\u03bb));
}

#line 30 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 30 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 30 "src/distribution/scalar_other/ExponentialDistribution.birch"
std::optional<Expression<Real>> ExponentialDistribution_<Arg1>::logpdfLazy(const Expression<Real>& x) {
  #line 31 "src/distribution/scalar_other/ExponentialDistribution.birch"
  return box(logpdf_exponential(x, \u03bb));
}

#line 34 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 34 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 34 "src/distribution/scalar_other/ExponentialDistribution.birch"
std::optional<Real> ExponentialDistribution_<Arg1>::cdf(const Real& x) {
  #line 35 "src/distribution/scalar_other/ExponentialDistribution.birch"
  return cdf_exponential(x, value(\u03bb));
}

#line 38 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 38 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 38 "src/distribution/scalar_other/ExponentialDistribution.birch"
std::optional<Real> ExponentialDistribution_<Arg1>::quantile(const Real& P) {
  #line 39 "src/distribution/scalar_other/ExponentialDistribution.birch"
  return quantile_exponential(P, value(\u03bb));
}

#line 42 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 42 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 42 "src/distribution/scalar_other/ExponentialDistribution.birch"
std::optional<Real> ExponentialDistribution_<Arg1>::lower() {
  #line 43 "src/distribution/scalar_other/ExponentialDistribution.birch"
  return 0.0;
}

#line 46 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1>
#line 46 "src/distribution/scalar_other/ExponentialDistribution.birch"
#line 46 "src/distribution/scalar_other/ExponentialDistribution.birch"
void ExponentialDistribution_<Arg1>::write(const Buffer& buffer) {
  #line 47 "src/distribution/scalar_other/ExponentialDistribution.birch"
  buffer->set(String("class"), String("Exponential"));
  #line 48 "src/distribution/scalar_other/ExponentialDistribution.birch"
  buffer->set(String("λ"), value(\u03bb));
}

#line 8 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/GammaDistribution.birch"
GammaDistribution_<Arg1, Arg2>::GammaDistribution_(const Arg1& k, const Arg2& \u03b8) :
    #line 8 "src/distribution/scalar_other/GammaDistribution.birch"
    base_type_(),
    #line 12 "src/distribution/scalar_other/GammaDistribution.birch"
    k(k),
    #line 17 "src/distribution/scalar_other/GammaDistribution.birch"
    \u03b8(\u03b8) {
  //
}

#line 19 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 19 "src/distribution/scalar_other/GammaDistribution.birch"
#line 19 "src/distribution/scalar_other/GammaDistribution.birch"
Boolean GammaDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 20 "src/distribution/scalar_other/GammaDistribution.birch"
  return true;
}

#line 23 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 23 "src/distribution/scalar_other/GammaDistribution.birch"
#line 23 "src/distribution/scalar_other/GammaDistribution.birch"
Real GammaDistribution_<Arg1, Arg2>::simulate() {
  #line 24 "src/distribution/scalar_other/GammaDistribution.birch"
  return simulate_gamma(value(k), value(\u03b8));
}

#line 27 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 27 "src/distribution/scalar_other/GammaDistribution.birch"
#line 27 "src/distribution/scalar_other/GammaDistribution.birch"
std::optional<Real> GammaDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 28 "src/distribution/scalar_other/GammaDistribution.birch"
  return simulate_gamma(peek(k), peek(\u03b8));
}

#line 31 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 31 "src/distribution/scalar_other/GammaDistribution.birch"
#line 31 "src/distribution/scalar_other/GammaDistribution.birch"
Real GammaDistribution_<Arg1, Arg2>::logpdf(const Real& x) {
  #line 32 "src/distribution/scalar_other/GammaDistribution.birch"
  return logpdf_gamma(x, value(k), value(\u03b8));
}

#line 35 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 35 "src/distribution/scalar_other/GammaDistribution.birch"
#line 35 "src/distribution/scalar_other/GammaDistribution.birch"
std::optional<Expression<Real>> GammaDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<Real>& x) {
  #line 36 "src/distribution/scalar_other/GammaDistribution.birch"
  return box(logpdf_gamma(x, k, \u03b8));
}

#line 39 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 39 "src/distribution/scalar_other/GammaDistribution.birch"
#line 39 "src/distribution/scalar_other/GammaDistribution.birch"
std::optional<Real> GammaDistribution_<Arg1, Arg2>::cdf(const Real& x) {
  #line 40 "src/distribution/scalar_other/GammaDistribution.birch"
  return cdf_gamma(x, value(k), value(\u03b8));
}

#line 43 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 43 "src/distribution/scalar_other/GammaDistribution.birch"
#line 43 "src/distribution/scalar_other/GammaDistribution.birch"
std::optional<Real> GammaDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 44 "src/distribution/scalar_other/GammaDistribution.birch"
  return quantile_gamma(P, value(k), value(\u03b8));
}

#line 47 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 47 "src/distribution/scalar_other/GammaDistribution.birch"
#line 47 "src/distribution/scalar_other/GammaDistribution.birch"
std::optional<Real> GammaDistribution_<Arg1, Arg2>::lower() {
  #line 48 "src/distribution/scalar_other/GammaDistribution.birch"
  return 0.0;
}

#line 51 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 51 "src/distribution/scalar_other/GammaDistribution.birch"
#line 51 "src/distribution/scalar_other/GammaDistribution.birch"
Boolean GammaDistribution_<Arg1, Arg2>::isGamma() {
  #line 52 "src/distribution/scalar_other/GammaDistribution.birch"
  return true;
}

#line 55 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 55 "src/distribution/scalar_other/GammaDistribution.birch"
#line 55 "src/distribution/scalar_other/GammaDistribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> GammaDistribution_<Arg1, Arg2>::getGamma() {
  #line 56 "src/distribution/scalar_other/GammaDistribution.birch"
  return box(k, \u03b8);
}

#line 59 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 59 "src/distribution/scalar_other/GammaDistribution.birch"
#line 59 "src/distribution/scalar_other/GammaDistribution.birch"
void GammaDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 60 "src/distribution/scalar_other/GammaDistribution.birch"
  buffer->set(String("class"), String("Gamma"));
  #line 61 "src/distribution/scalar_other/GammaDistribution.birch"
  buffer->set(String("k"), value(k));
  #line 62 "src/distribution/scalar_other/GammaDistribution.birch"
  buffer->set(String("θ"), value(\u03b8));
}

#line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
GammaExponentialDistribution_<Arg1, Arg2>::GammaExponentialDistribution_(const Arg1& k, const Arg2& \u03b8) :
    #line 8 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
    base_type_(),
    #line 12 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
    k(k),
    #line 17 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
    \u03b8(\u03b8) {
  //
}

#line 19 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 19 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 19 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
Boolean GammaExponentialDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 20 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return true;
}

#line 23 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 23 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 23 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
Real GammaExponentialDistribution_<Arg1, Arg2>::simulate() {
  #line 24 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return simulate_lomax(1.0 / value(\u03b8), value(k));
}

#line 27 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 27 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 27 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
std::optional<Real> GammaExponentialDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 28 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return simulate_lomax(1.0 / peek(\u03b8), peek(k));
}

#line 31 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 31 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 31 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
Real GammaExponentialDistribution_<Arg1, Arg2>::logpdf(const Real& x) {
  #line 32 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return logpdf_lomax(x, 1.0 / value(\u03b8), value(k));
}

#line 35 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 35 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 35 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
std::optional<Expression<Real>> GammaExponentialDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<Real>& x) {
  #line 36 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return box(logpdf_lomax(x, 1.0 / \u03b8, k));
}

#line 39 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 39 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 39 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
std::optional<Delay> GammaExponentialDistribution_<Arg1, Arg2>::update(const Real& x) {
  #line 40 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return update_gamma_exponential(x, value(k), value(\u03b8));
}

#line 43 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 43 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 43 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
std::optional<Delay> GammaExponentialDistribution_<Arg1, Arg2>::updateLazy(const Expression<Real>& x) {
  #line 44 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return update_gamma_exponential(x, k, \u03b8);
}

#line 47 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 47 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 47 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
std::optional<Real> GammaExponentialDistribution_<Arg1, Arg2>::cdf(const Real& x) {
  #line 48 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return cdf_lomax(x, 1.0 / value(\u03b8), value(k));
}

#line 51 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 51 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 51 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
std::optional<Real> GammaExponentialDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 52 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return quantile_lomax(P, 1.0 / value(\u03b8), value(k));
}

#line 55 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 55 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
#line 55 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
std::optional<Real> GammaExponentialDistribution_<Arg1, Arg2>::lower() {
  #line 56 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return 0.0;
}

#line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
InverseGammaDistribution_<Arg1, Arg2>::InverseGammaDistribution_(const Arg1& \u03b1, const Arg2& \u03b2) :
    #line 8 "src/distribution/scalar_other/InverseGammaDistribution.birch"
    base_type_(),
    #line 12 "src/distribution/scalar_other/InverseGammaDistribution.birch"
    \u03b1(\u03b1),
    #line 17 "src/distribution/scalar_other/InverseGammaDistribution.birch"
    \u03b2(\u03b2) {
  //
}

#line 19 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 19 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 19 "src/distribution/scalar_other/InverseGammaDistribution.birch"
Boolean InverseGammaDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 20 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return true;
}

#line 23 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 23 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 23 "src/distribution/scalar_other/InverseGammaDistribution.birch"
Real InverseGammaDistribution_<Arg1, Arg2>::simulate() {
  #line 24 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return simulate_inverse_gamma(value(\u03b1), value(\u03b2));
}

#line 27 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 27 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 27 "src/distribution/scalar_other/InverseGammaDistribution.birch"
std::optional<Real> InverseGammaDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 28 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return simulate_inverse_gamma(peek(\u03b1), peek(\u03b2));
}

#line 31 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 31 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 31 "src/distribution/scalar_other/InverseGammaDistribution.birch"
Real InverseGammaDistribution_<Arg1, Arg2>::logpdf(const Real& x) {
  #line 32 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return logpdf_inverse_gamma(x, value(\u03b1), value(\u03b2));
}

#line 35 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 35 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 35 "src/distribution/scalar_other/InverseGammaDistribution.birch"
std::optional<Expression<Real>> InverseGammaDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<Real>& x) {
  #line 36 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return box(logpdf_inverse_gamma(x, \u03b1, \u03b2));
}

#line 39 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 39 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 39 "src/distribution/scalar_other/InverseGammaDistribution.birch"
std::optional<Real> InverseGammaDistribution_<Arg1, Arg2>::cdf(const Real& x) {
  #line 40 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return cdf_inverse_gamma(x, value(\u03b1), value(\u03b2));
}

#line 43 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 43 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 43 "src/distribution/scalar_other/InverseGammaDistribution.birch"
std::optional<Real> InverseGammaDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 44 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return quantile_inverse_gamma(P, value(\u03b1), value(\u03b2));
}

#line 47 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 47 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 47 "src/distribution/scalar_other/InverseGammaDistribution.birch"
std::optional<Real> InverseGammaDistribution_<Arg1, Arg2>::lower() {
  #line 48 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return 0.0;
}

#line 51 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 51 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 51 "src/distribution/scalar_other/InverseGammaDistribution.birch"
Boolean InverseGammaDistribution_<Arg1, Arg2>::isInverseGamma() {
  #line 52 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return true;
}

#line 55 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 55 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 55 "src/distribution/scalar_other/InverseGammaDistribution.birch"
std::optional<std::tuple<Expression<Real>, Expression<Real>>> InverseGammaDistribution_<Arg1, Arg2>::getInverseGamma() {
  #line 56 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return box(\u03b1, \u03b2);
}

#line 59 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 59 "src/distribution/scalar_other/InverseGammaDistribution.birch"
#line 59 "src/distribution/scalar_other/InverseGammaDistribution.birch"
void InverseGammaDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 60 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  buffer->set(String("class"), String("InverseGamma"));
  #line 61 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  buffer->set(String("α"), value(\u03b1));
  #line 62 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  buffer->set(String("β"), value(\u03b2));
}

#line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>::InverseGammaGammaDistribution_(const Arg1& k, const Arg2& \u03b1, const Arg3& \u03b2) :
    #line 4 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
    k(k),
    #line 14 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
    \u03b1(\u03b1),
    #line 19 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
    \u03b2(\u03b2) {
  //
}

#line 21 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 21 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 21 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
Boolean InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>::supportsLazy() {
  #line 22 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return true;
}

#line 25 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 25 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 25 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
Real InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>::simulate() {
  #line 26 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return simulate_inverse_gamma_gamma(value(k), value(\u03b1), value(\u03b2));
}

#line 29 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 29 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 29 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
std::optional<Real> InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>::simulateLazy() {
  #line 30 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return simulate_inverse_gamma_gamma(peek(k), peek(\u03b1), peek(\u03b2));
}

#line 33 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 33 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 33 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
Real InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>::logpdf(const Real& x) {
  #line 34 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return logpdf_inverse_gamma_gamma(x, value(k), value(\u03b1), value(\u03b2));
}

#line 37 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 37 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 37 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
std::optional<Expression<Real>> InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>::logpdfLazy(const Expression<Real>& x) {
  #line 38 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return box(logpdf_inverse_gamma_gamma(x, k, \u03b1, \u03b2));
}

#line 41 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 41 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 41 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
std::optional<Delay> InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>::update(const Real& x) {
  #line 42 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return update_inverse_gamma_gamma(x, value(k), value(\u03b1), value(\u03b2));
}

#line 45 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 45 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 45 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
std::optional<Delay> InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>::updateLazy(const Expression<Real>& x) {
  #line 46 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return update_inverse_gamma_gamma(x, k, \u03b1, \u03b2);
}

#line 49 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 49 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 49 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
std::optional<Real> InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>::cdf(const Real& x) {
  #line 50 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return cdf_inverse_gamma_gamma(x, value(k), value(\u03b1), value(\u03b2));
}

#line 53 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 53 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
#line 53 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
std::optional<Real> InverseGammaGammaDistribution_<Arg1, Arg2, Arg3>::lower() {
  #line 54 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return 0.0;
}

#line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::ScaledGammaExponentialDistribution_(const Arg1& a, const Arg2& k, const Arg3& \u03b8) :
    #line 4 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
    base_type_(),
    #line 9 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
    a(a),
    #line 14 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
    k(k),
    #line 19 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
    \u03b8(\u03b8) {
  //
}

#line 21 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 21 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 21 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
Boolean ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::supportsLazy() {
  #line 22 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return true;
}

#line 25 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 25 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 25 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
Real ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::simulate() {
  #line 26 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return simulate_lomax(1.0 / (value(a) * value(\u03b8)), value(k));
}

#line 29 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 29 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 29 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
std::optional<Real> ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::simulateLazy() {
  #line 30 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return simulate_lomax(1.0 / (peek(a) * peek(\u03b8)), peek(k));
}

#line 33 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 33 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 33 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
Real ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::logpdf(const Real& x) {
  #line 34 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return logpdf_lomax(x, 1.0 / (value(a) * value(\u03b8)), value(k));
}

#line 37 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 37 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 37 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
std::optional<Expression<Real>> ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::logpdfLazy(const Expression<Real>& x) {
  #line 38 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return box(logpdf_lomax(x, 1.0 / (a * \u03b8), k));
}

#line 41 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 41 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 41 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
std::optional<Delay> ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::update(const Real& x) {
  #line 42 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return update_scaled_gamma_exponential(x, value(a), value(k), value(\u03b8));
}

#line 45 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 45 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 45 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
std::optional<Delay> ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::updateLazy(const Expression<Real>& x) {
  #line 46 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return update_scaled_gamma_exponential(x, a, k, \u03b8);
}

#line 49 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 49 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 49 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
std::optional<Real> ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::cdf(const Real& x) {
  #line 50 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return cdf_lomax(x, 1.0 / (value(a) * value(\u03b8)), value(k));
}

#line 53 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 53 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 53 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
std::optional<Real> ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::quantile(const Real& P) {
  #line 54 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return quantile_lomax(P, 1.0 / (value(a) * value(\u03b8)), value(k));
}

#line 57 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 57 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
#line 57 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
std::optional<Real> ScaledGammaExponentialDistribution_<Arg1, Arg2, Arg3>::lower() {
  #line 58 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return 0.0;
}

#line 8 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 8 "src/distribution/scalar_other/StudentDistribution.birch"
StudentDistribution_<Arg1, Arg2, Arg3>::StudentDistribution_(const Arg1& \u03bd, const Arg2& \u03bc, const Arg3& \u03c32) :
    #line 8 "src/distribution/scalar_other/StudentDistribution.birch"
    base_type_(),
    #line 13 "src/distribution/scalar_other/StudentDistribution.birch"
    \u03bd(\u03bd),
    #line 18 "src/distribution/scalar_other/StudentDistribution.birch"
    \u03bc(\u03bc),
    #line 23 "src/distribution/scalar_other/StudentDistribution.birch"
    \u03c32(\u03c32) {
  //
}

#line 25 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 25 "src/distribution/scalar_other/StudentDistribution.birch"
#line 25 "src/distribution/scalar_other/StudentDistribution.birch"
Boolean StudentDistribution_<Arg1, Arg2, Arg3>::supportsLazy() {
  #line 26 "src/distribution/scalar_other/StudentDistribution.birch"
  return true;
}

#line 29 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 29 "src/distribution/scalar_other/StudentDistribution.birch"
#line 29 "src/distribution/scalar_other/StudentDistribution.birch"
Real StudentDistribution_<Arg1, Arg2, Arg3>::simulate() {
  #line 30 "src/distribution/scalar_other/StudentDistribution.birch"
  return simulate_student_t(value(\u03bd), value(\u03bc), value(\u03c32));
}

#line 33 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 33 "src/distribution/scalar_other/StudentDistribution.birch"
#line 33 "src/distribution/scalar_other/StudentDistribution.birch"
std::optional<Real> StudentDistribution_<Arg1, Arg2, Arg3>::simulateLazy() {
  #line 34 "src/distribution/scalar_other/StudentDistribution.birch"
  return simulate_student_t(peek(\u03bd), peek(\u03bc), peek(\u03c32));
}

#line 37 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 37 "src/distribution/scalar_other/StudentDistribution.birch"
#line 37 "src/distribution/scalar_other/StudentDistribution.birch"
Real StudentDistribution_<Arg1, Arg2, Arg3>::logpdf(const Real& x) {
  #line 38 "src/distribution/scalar_other/StudentDistribution.birch"
  return logpdf_student_t(x, value(\u03bd), value(\u03bc), value(\u03c32));
}

#line 41 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 41 "src/distribution/scalar_other/StudentDistribution.birch"
#line 41 "src/distribution/scalar_other/StudentDistribution.birch"
std::optional<Expression<Real>> StudentDistribution_<Arg1, Arg2, Arg3>::logpdfLazy(const Expression<Real>& x) {
  #line 42 "src/distribution/scalar_other/StudentDistribution.birch"
  return box(logpdf_student_t(x, \u03bd, \u03bc, \u03c32));
}

#line 45 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 45 "src/distribution/scalar_other/StudentDistribution.birch"
#line 45 "src/distribution/scalar_other/StudentDistribution.birch"
std::optional<Real> StudentDistribution_<Arg1, Arg2, Arg3>::cdf(const Real& x) {
  #line 46 "src/distribution/scalar_other/StudentDistribution.birch"
  return cdf_student_t(x, value(\u03bd), value(\u03bc), value(\u03c32));
}

#line 49 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 49 "src/distribution/scalar_other/StudentDistribution.birch"
#line 49 "src/distribution/scalar_other/StudentDistribution.birch"
std::optional<Real> StudentDistribution_<Arg1, Arg2, Arg3>::quantile(const Real& P) {
  #line 50 "src/distribution/scalar_other/StudentDistribution.birch"
  return quantile_student_t(P, value(\u03bd), value(\u03bc), value(\u03c32));
}

#line 53 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 53 "src/distribution/scalar_other/StudentDistribution.birch"
#line 53 "src/distribution/scalar_other/StudentDistribution.birch"
void StudentDistribution_<Arg1, Arg2, Arg3>::write(const Buffer& buffer) {
  #line 54 "src/distribution/scalar_other/StudentDistribution.birch"
  buffer->set(String("class"), String("Student"));
  #line 55 "src/distribution/scalar_other/StudentDistribution.birch"
  buffer->set(String("ν"), value(\u03bd));
  #line 56 "src/distribution/scalar_other/StudentDistribution.birch"
  buffer->set(String("μ"), value(\u03bc));
  #line 57 "src/distribution/scalar_other/StudentDistribution.birch"
  buffer->set(String("σ2"), value(\u03c32));
}

#line 8 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/UniformDistribution.birch"
UniformDistribution_<Arg1, Arg2>::UniformDistribution_(const Arg1& l, const Arg2& u) :
    #line 8 "src/distribution/scalar_other/UniformDistribution.birch"
    base_type_(),
    #line 12 "src/distribution/scalar_other/UniformDistribution.birch"
    l(l),
    #line 17 "src/distribution/scalar_other/UniformDistribution.birch"
    u(u) {
  //
}

#line 19 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 19 "src/distribution/scalar_other/UniformDistribution.birch"
#line 19 "src/distribution/scalar_other/UniformDistribution.birch"
Boolean UniformDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 20 "src/distribution/scalar_other/UniformDistribution.birch"
  return false;
}

#line 23 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 23 "src/distribution/scalar_other/UniformDistribution.birch"
#line 23 "src/distribution/scalar_other/UniformDistribution.birch"
Real UniformDistribution_<Arg1, Arg2>::simulate() {
  #line 24 "src/distribution/scalar_other/UniformDistribution.birch"
  return simulate_uniform(value(l), value(u));
}

#line 27 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 27 "src/distribution/scalar_other/UniformDistribution.birch"
#line 27 "src/distribution/scalar_other/UniformDistribution.birch"
Real UniformDistribution_<Arg1, Arg2>::logpdf(const Real& x) {
  #line 28 "src/distribution/scalar_other/UniformDistribution.birch"
  return logpdf_uniform(x, value(l), value(u));
}

#line 31 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 31 "src/distribution/scalar_other/UniformDistribution.birch"
#line 31 "src/distribution/scalar_other/UniformDistribution.birch"
std::optional<Real> UniformDistribution_<Arg1, Arg2>::cdf(const Real& x) {
  #line 32 "src/distribution/scalar_other/UniformDistribution.birch"
  return cdf_uniform(x, value(l), value(u));
}

#line 35 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 35 "src/distribution/scalar_other/UniformDistribution.birch"
#line 35 "src/distribution/scalar_other/UniformDistribution.birch"
std::optional<Real> UniformDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 36 "src/distribution/scalar_other/UniformDistribution.birch"
  return quantile_uniform(P, value(l), value(u));
}

#line 39 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 39 "src/distribution/scalar_other/UniformDistribution.birch"
#line 39 "src/distribution/scalar_other/UniformDistribution.birch"
std::optional<Real> UniformDistribution_<Arg1, Arg2>::lower() {
  #line 40 "src/distribution/scalar_other/UniformDistribution.birch"
  return value(l);
}

#line 43 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 43 "src/distribution/scalar_other/UniformDistribution.birch"
#line 43 "src/distribution/scalar_other/UniformDistribution.birch"
std::optional<Real> UniformDistribution_<Arg1, Arg2>::upper() {
  #line 44 "src/distribution/scalar_other/UniformDistribution.birch"
  return value(u);
}

#line 47 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 47 "src/distribution/scalar_other/UniformDistribution.birch"
#line 47 "src/distribution/scalar_other/UniformDistribution.birch"
void UniformDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 48 "src/distribution/scalar_other/UniformDistribution.birch"
  buffer->set(String("class"), String("Uniform"));
  #line 49 "src/distribution/scalar_other/UniformDistribution.birch"
  buffer->set(String("l"), value(l));
  #line 50 "src/distribution/scalar_other/UniformDistribution.birch"
  buffer->set(String("u"), value(u));
}

#line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
WeibullDistribution_<Arg1, Arg2>::WeibullDistribution_(const Arg1& k, const Arg2& \u03bb) :
    #line 8 "src/distribution/scalar_other/WeibullDistribution.birch"
    base_type_(),
    #line 12 "src/distribution/scalar_other/WeibullDistribution.birch"
    k(k),
    #line 17 "src/distribution/scalar_other/WeibullDistribution.birch"
    \u03bb(\u03bb) {
  //
}

#line 19 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 19 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 19 "src/distribution/scalar_other/WeibullDistribution.birch"
Boolean WeibullDistribution_<Arg1, Arg2>::supportsLazy() {
  #line 20 "src/distribution/scalar_other/WeibullDistribution.birch"
  return true;
}

#line 23 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 23 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 23 "src/distribution/scalar_other/WeibullDistribution.birch"
Real WeibullDistribution_<Arg1, Arg2>::simulate() {
  #line 24 "src/distribution/scalar_other/WeibullDistribution.birch"
  return simulate_weibull(value(k), value(\u03bb));
}

#line 27 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 27 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 27 "src/distribution/scalar_other/WeibullDistribution.birch"
std::optional<Real> WeibullDistribution_<Arg1, Arg2>::simulateLazy() {
  #line 28 "src/distribution/scalar_other/WeibullDistribution.birch"
  return simulate_weibull(peek(k), peek(\u03bb));
}

#line 31 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 31 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 31 "src/distribution/scalar_other/WeibullDistribution.birch"
Real WeibullDistribution_<Arg1, Arg2>::logpdf(const Real& x) {
  #line 32 "src/distribution/scalar_other/WeibullDistribution.birch"
  return logpdf_weibull(x, value(k), value(\u03bb));
}

#line 35 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 35 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 35 "src/distribution/scalar_other/WeibullDistribution.birch"
std::optional<Expression<Real>> WeibullDistribution_<Arg1, Arg2>::logpdfLazy(const Expression<Real>& x) {
  #line 36 "src/distribution/scalar_other/WeibullDistribution.birch"
  return box(logpdf_weibull(x, k, \u03bb));
}

#line 39 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 39 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 39 "src/distribution/scalar_other/WeibullDistribution.birch"
std::optional<Real> WeibullDistribution_<Arg1, Arg2>::cdf(const Real& x) {
  #line 40 "src/distribution/scalar_other/WeibullDistribution.birch"
  return cdf_weibull(x, value(k), value(\u03bb));
}

#line 43 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 43 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 43 "src/distribution/scalar_other/WeibullDistribution.birch"
std::optional<Real> WeibullDistribution_<Arg1, Arg2>::quantile(const Real& P) {
  #line 44 "src/distribution/scalar_other/WeibullDistribution.birch"
  return quantile_weibull(P, value(k), value(\u03bb));
}

#line 47 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 47 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 47 "src/distribution/scalar_other/WeibullDistribution.birch"
std::optional<Real> WeibullDistribution_<Arg1, Arg2>::lower() {
  #line 48 "src/distribution/scalar_other/WeibullDistribution.birch"
  return 0.0;
}

#line 51 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 51 "src/distribution/scalar_other/WeibullDistribution.birch"
#line 51 "src/distribution/scalar_other/WeibullDistribution.birch"
void WeibullDistribution_<Arg1, Arg2>::write(const Buffer& buffer) {
  #line 52 "src/distribution/scalar_other/WeibullDistribution.birch"
  buffer->set(String("class"), String("Weibull"));
  #line 53 "src/distribution/scalar_other/WeibullDistribution.birch"
  buffer->set(String("k"), value(k));
  #line 54 "src/distribution/scalar_other/WeibullDistribution.birch"
  buffer->set(String("λ"), value(\u03bb));
}

#line 2 "src/form/binary_matrix/CholSolve.birch"
#line 2 "src/form/binary_matrix/CholSolve.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_matrix/CholSolve.birch"
decltype(auto) CholSolveFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_matrix/CholSolve.birch"
  return cholsolve(l, r);
}

#line 6 "src/form/binary_matrix/CholSolve.birch"
#line 6 "src/form/binary_matrix/CholSolve.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_matrix/CholSolve.birch"
decltype(auto) CholSolveFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  auto x = cholsolve(l, d);
  auto y = cholsolve(l, r);
  #line 10 "src/form/binary_matrix/CholSolve.birch"
  return std::make_tuple(outer(-(x), y), x);
}

#line 2 "src/form/binary_matrix/Diagonal.birch"
#line 2 "src/form/binary_matrix/Diagonal.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_matrix/Diagonal.birch"
decltype(auto) DiagonalFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_matrix/Diagonal.birch"
  return diagonal(l, r);
}

#line 6 "src/form/binary_matrix/Diagonal.birch"
#line 6 "src/form/binary_matrix/Diagonal.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_matrix/Diagonal.birch"
decltype(auto) DiagonalFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_matrix/Diagonal.birch"
  return std::make_tuple(trace(d), 0.0);
}

#line 2 "src/form/binary_matrix/Dot.birch"
#line 2 "src/form/binary_matrix/Dot.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_matrix/Dot.birch"
decltype(auto) DotFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_matrix/Dot.birch"
  return dot(l, r);
}

#line 6 "src/form/binary_matrix/Dot.birch"
#line 6 "src/form/binary_matrix/Dot.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_matrix/Dot.birch"
decltype(auto) DotFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_matrix/Dot.birch"
  return std::make_tuple(r * d, outer(l, d));
}

#line 2 "src/form/binary_matrix/Inner.birch"
#line 2 "src/form/binary_matrix/Inner.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_matrix/Inner.birch"
decltype(auto) InnerFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_matrix/Inner.birch"
  return inner(l, r);
}

#line 6 "src/form/binary_matrix/Inner.birch"
#line 6 "src/form/binary_matrix/Inner.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_matrix/Inner.birch"
decltype(auto) InnerFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_matrix/Inner.birch"
  return std::make_tuple(outer(r, d), l * d);
}

#line 2 "src/form/binary_matrix/MatrixPack.birch"
#line 2 "src/form/binary_matrix/MatrixPack.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_matrix/MatrixPack.birch"
decltype(auto) MatrixPackFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_matrix/MatrixPack.birch"
  return pack(l, r);
}

#line 6 "src/form/binary_matrix/MatrixPack.birch"
#line 6 "src/form/binary_matrix/MatrixPack.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_matrix/MatrixPack.birch"
decltype(auto) MatrixPackFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_matrix/MatrixPack.birch"
  return std::make_tuple(d(Integer(1), Integer(1)), d(Integer(1), Integer(2)));
}

#line 11 "src/form/binary_matrix/MatrixPack.birch"
#line 11 "src/form/binary_matrix/MatrixPack.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 11 "src/form/binary_matrix/MatrixPack.birch"
decltype(auto) MatrixPackFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const libbirch::DefaultArray<RightValue,1>& r) {
  #line 13 "src/form/binary_matrix/MatrixPack.birch"
  return std::make_tuple(column(d, Integer(1)), column(d, Integer(2)));
}

#line 16 "src/form/binary_matrix/MatrixPack.birch"
#line 16 "src/form/binary_matrix/MatrixPack.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 16 "src/form/binary_matrix/MatrixPack.birch"
decltype(auto) MatrixPackFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,2>& l, const libbirch::DefaultArray<RightValue,2>& r) {
  auto r1 = rows(l);
  auto c1 = columns(l);
  auto c2 = columns(r);
  #line 21 "src/form/binary_matrix/MatrixPack.birch"
  return std::make_tuple(d(libbirch::make_range(Integer(1), r1), libbirch::make_range(Integer(1), c1)), d(libbirch::make_range(Integer(1), r1), libbirch::make_range((c1 + Integer(1)), (c1 + c2))));
}

#line 2 "src/form/binary_matrix/MatrixStack.birch"
#line 2 "src/form/binary_matrix/MatrixStack.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_matrix/MatrixStack.birch"
decltype(auto) MatrixStackFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_matrix/MatrixStack.birch"
  return stack(l, r);
}

#line 6 "src/form/binary_matrix/MatrixStack.birch"
#line 6 "src/form/binary_matrix/MatrixStack.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_matrix/MatrixStack.birch"
decltype(auto) MatrixStackFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_matrix/MatrixStack.birch"
  return std::make_tuple(d(Integer(1)), d(Integer(2)));
}

#line 11 "src/form/binary_matrix/MatrixStack.birch"
#line 11 "src/form/binary_matrix/MatrixStack.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 11 "src/form/binary_matrix/MatrixStack.birch"
decltype(auto) MatrixStackFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const RightValue& r) {
  #line 13 "src/form/binary_matrix/MatrixStack.birch"
  return std::make_tuple(d(libbirch::make_range(Integer(1), (length(d) - Integer(1)))), d(length(d)));
}

#line 16 "src/form/binary_matrix/MatrixStack.birch"
#line 16 "src/form/binary_matrix/MatrixStack.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 16 "src/form/binary_matrix/MatrixStack.birch"
decltype(auto) MatrixStackFunction_::grad(const Gradient& d, const LeftValue& l, const libbirch::DefaultArray<RightValue,1>& r) {
  #line 18 "src/form/binary_matrix/MatrixStack.birch"
  return std::make_tuple(d(Integer(1)), d(libbirch::make_range(Integer(2), length(d))));
}

#line 21 "src/form/binary_matrix/MatrixStack.birch"
#line 21 "src/form/binary_matrix/MatrixStack.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 21 "src/form/binary_matrix/MatrixStack.birch"
decltype(auto) MatrixStackFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const libbirch::DefaultArray<RightValue,1>& r) {
  #line 23 "src/form/binary_matrix/MatrixStack.birch"
  return std::make_tuple(d(libbirch::make_range(Integer(1), length(l))), d(libbirch::make_range((length(l) + Integer(1)), length(d))));
}

#line 26 "src/form/binary_matrix/MatrixStack.birch"
#line 26 "src/form/binary_matrix/MatrixStack.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 26 "src/form/binary_matrix/MatrixStack.birch"
decltype(auto) MatrixStackFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,2>& l, const libbirch::DefaultArray<RightValue,2>& r) {
  auto r1 = rows(l);
  auto r2 = rows(r);
  auto c1 = columns(l);
  #line 31 "src/form/binary_matrix/MatrixStack.birch"
  return std::make_tuple(d(libbirch::make_range(Integer(1), r1), libbirch::make_range(Integer(1), c1)), d(libbirch::make_range((r1 + Integer(1)), (r1 + r2)), libbirch::make_range(Integer(1), c1)));
}

#line 2 "src/form/binary_matrix/Outer.birch"
#line 2 "src/form/binary_matrix/Outer.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_matrix/Outer.birch"
decltype(auto) OuterFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_matrix/Outer.birch"
  return outer(l, r);
}

#line 6 "src/form/binary_matrix/Outer.birch"
#line 6 "src/form/binary_matrix/Outer.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_matrix/Outer.birch"
decltype(auto) OuterFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_matrix/Outer.birch"
  return std::make_tuple(d * r, inner(d, l));
}

#line 2 "src/form/binary_matrix/Solve.birch"
#line 2 "src/form/binary_matrix/Solve.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_matrix/Solve.birch"
decltype(auto) SolveFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_matrix/Solve.birch"
  return solve(l, r);
}

#line 6 "src/form/binary_matrix/Solve.birch"
#line 6 "src/form/binary_matrix/Solve.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_matrix/Solve.birch"
decltype(auto) SolveFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  auto dl = outer(solve(transpose(l), -(d)), solve(l, r));
  auto dr = solve(transpose(l), d);
  #line 10 "src/form/binary_matrix/Solve.birch"
  return std::make_tuple(dl, dr);
}

#line 2 "src/form/binary_other/Add.birch"
#line 2 "src/form/binary_other/Add.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_other/Add.birch"
decltype(auto) AddFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_other/Add.birch"
  return l + r;
}

#line 6 "src/form/binary_other/Add.birch"
#line 6 "src/form/binary_other/Add.birch"
template<class Gradient>
#line 6 "src/form/binary_other/Add.birch"
decltype(auto) AddFunction_::grad(const Gradient& d) {
  #line 7 "src/form/binary_other/Add.birch"
  return std::make_tuple(d, d);
}

#line 2 "src/form/binary_other/CopySign.birch"
#line 2 "src/form/binary_other/CopySign.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_other/CopySign.birch"
decltype(auto) CopySignFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_other/CopySign.birch"
  return copysign(l, r);
}

#line 6 "src/form/binary_other/CopySign.birch"
#line 6 "src/form/binary_other/CopySign.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_other/CopySign.birch"
decltype(auto) CopySignFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_other/CopySign.birch"
  return std::make_tuple(if_then_else(copysign(l, r) == l, d, -(d)), 0.0);
}

#line 2 "src/form/binary_other/Divide.birch"
#line 2 "src/form/binary_other/Divide.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_other/Divide.birch"
decltype(auto) DivideFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_other/Divide.birch"
  return l / r;
}

#line 6 "src/form/binary_other/Divide.birch"
#line 6 "src/form/binary_other/Divide.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_other/Divide.birch"
decltype(auto) DivideFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_other/Divide.birch"
  return std::make_tuple(d / r, -(trace(inner(d, l))) / pow(r, 2.0));
}

#line 2 "src/form/binary_other/LessThan.birch"
#line 2 "src/form/binary_other/LessThan.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_other/LessThan.birch"
decltype(auto) LessThanFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_other/LessThan.birch"
  return l < r;
}

#line 6 "src/form/binary_other/LessThan.birch"
#line 6 "src/form/binary_other/LessThan.birch"
template<class Gradient>
#line 6 "src/form/binary_other/LessThan.birch"
decltype(auto) LessThanFunction_::grad(const Gradient& d) {
  #line 7 "src/form/binary_other/LessThan.birch"
  return std::make_tuple(0.0, 0.0);
}

#line 2 "src/form/binary_other/LogBeta.birch"
#line 2 "src/form/binary_other/LogBeta.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_other/LogBeta.birch"
decltype(auto) LogBetaFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_other/LogBeta.birch"
  return lbeta(l, r);
}

#line 6 "src/form/binary_other/LogBeta.birch"
#line 6 "src/form/binary_other/LogBeta.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_other/LogBeta.birch"
decltype(auto) LogBetaFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_other/LogBeta.birch"
  return std::make_tuple(d * (digamma(l) + digamma(l + r)), d * (digamma(r) + digamma(l + r)));
}

#line 2 "src/form/binary_other/LogChoose.birch"
#line 2 "src/form/binary_other/LogChoose.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_other/LogChoose.birch"
decltype(auto) LogChooseFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_other/LogChoose.birch"
  return lchoose(l, r);
}

#line 6 "src/form/binary_other/LogChoose.birch"
#line 6 "src/form/binary_other/LogChoose.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_other/LogChoose.birch"
decltype(auto) LogChooseFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_other/LogChoose.birch"
  return std::make_tuple(0.0, 0.0);
}

#line 2 "src/form/binary_other/LogGammaP.birch"
#line 2 "src/form/binary_other/LogGammaP.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_other/LogGammaP.birch"
decltype(auto) LogGammaPFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_other/LogGammaP.birch"
  return lgamma(l, r);
}

#line 6 "src/form/binary_other/LogGammaP.birch"
#line 6 "src/form/binary_other/LogGammaP.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_other/LogGammaP.birch"
decltype(auto) LogGammaPFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  auto y = 0.0;
  #line 9 "src/form/binary_other/LogGammaP.birch"
  for (auto i = Integer(1); i <= r; ++i) {
    #line 10 "src/form/binary_other/LogGammaP.birch"
    y = y + digamma(l + 0.5 * (Integer(1) - i));
  }
  #line 12 "src/form/binary_other/LogGammaP.birch"
  return std::make_tuple(d * y, 0.0);
}

#line 2 "src/form/binary_other/Multiply.birch"
#line 2 "src/form/binary_other/Multiply.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_other/Multiply.birch"
decltype(auto) MultiplyFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_other/Multiply.birch"
  return l * r;
}

#line 6 "src/form/binary_other/Multiply.birch"
#line 6 "src/form/binary_other/Multiply.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_other/Multiply.birch"
decltype(auto) MultiplyFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  #line 8 "src/form/binary_other/Multiply.birch"
  return std::make_tuple(trace(outer(d, r)), trace(inner(l, d)));
}

#line 11 "src/form/binary_other/Multiply.birch"
#line 11 "src/form/binary_other/Multiply.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 11 "src/form/binary_other/Multiply.birch"
decltype(auto) MultiplyFunction_::grad(const Gradient& d, const LeftValue& l, const libbirch::DefaultArray<RightValue,1>& r) {
  #line 13 "src/form/binary_other/Multiply.birch"
  return std::make_tuple(trace(outer(d, r)), inner(l, d));
}

#line 16 "src/form/binary_other/Multiply.birch"
#line 16 "src/form/binary_other/Multiply.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 16 "src/form/binary_other/Multiply.birch"
decltype(auto) MultiplyFunction_::grad(const Gradient& d, const LeftValue& l, const libbirch::DefaultArray<RightValue,2>& r) {
  #line 18 "src/form/binary_other/Multiply.birch"
  return std::make_tuple(trace(outer(d, r)), inner(l, d));
}

#line 21 "src/form/binary_other/Multiply.birch"
#line 21 "src/form/binary_other/Multiply.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 21 "src/form/binary_other/Multiply.birch"
decltype(auto) MultiplyFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const RightValue& r) {
  #line 23 "src/form/binary_other/Multiply.birch"
  return std::make_tuple(outer(d, r), trace(inner(l, d)));
}

#line 26 "src/form/binary_other/Multiply.birch"
#line 26 "src/form/binary_other/Multiply.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 26 "src/form/binary_other/Multiply.birch"
decltype(auto) MultiplyFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const libbirch::DefaultArray<RightValue,1>& r) {
  #line 28 "src/form/binary_other/Multiply.birch"
  return std::make_tuple(outer(d, r), inner(l, d));
}

#line 31 "src/form/binary_other/Multiply.birch"
#line 31 "src/form/binary_other/Multiply.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 31 "src/form/binary_other/Multiply.birch"
decltype(auto) MultiplyFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,1>& l, const libbirch::DefaultArray<RightValue,2>& r) {
  #line 33 "src/form/binary_other/Multiply.birch"
  return std::make_tuple(outer(d, r), inner(l, d));
}

#line 36 "src/form/binary_other/Multiply.birch"
#line 36 "src/form/binary_other/Multiply.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 36 "src/form/binary_other/Multiply.birch"
decltype(auto) MultiplyFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,2>& l, const RightValue& r) {
  #line 38 "src/form/binary_other/Multiply.birch"
  return std::make_tuple(outer(d, r), trace(inner(l, d)));
}

#line 41 "src/form/binary_other/Multiply.birch"
#line 41 "src/form/binary_other/Multiply.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 41 "src/form/binary_other/Multiply.birch"
decltype(auto) MultiplyFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,2>& l, const libbirch::DefaultArray<RightValue,1>& r) {
  #line 43 "src/form/binary_other/Multiply.birch"
  return std::make_tuple(outer(d, r), inner(l, d));
}

#line 46 "src/form/binary_other/Multiply.birch"
#line 46 "src/form/binary_other/Multiply.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 46 "src/form/binary_other/Multiply.birch"
decltype(auto) MultiplyFunction_::grad(const Gradient& d, const libbirch::DefaultArray<LeftValue,2>& l, const libbirch::DefaultArray<RightValue,2>& r) {
  #line 48 "src/form/binary_other/Multiply.birch"
  return std::make_tuple(outer(d, r), inner(l, d));
}

#line 2 "src/form/binary_other/Pow.birch"
#line 2 "src/form/binary_other/Pow.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_other/Pow.birch"
decltype(auto) PowFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_other/Pow.birch"
  return pow(l, r);
}

#line 6 "src/form/binary_other/Pow.birch"
#line 6 "src/form/binary_other/Pow.birch"
template<class Gradient,class LeftValue,class RightValue>
#line 6 "src/form/binary_other/Pow.birch"
decltype(auto) PowFunction_::grad(const Gradient& d, const LeftValue& l, const RightValue& r) {
  auto dl = d * r * pow(l, r - 1.0);
  auto dr = 0.0;
  #line 10 "src/form/binary_other/Pow.birch"
  if (l > 0.0) {
    #line 11 "src/form/binary_other/Pow.birch"
    dr = d * pow(l, r) * log(l);
  }
  #line 13 "src/form/binary_other/Pow.birch"
  return std::make_tuple(dl, dr);
}

#line 2 "src/form/binary_other/Subtract.birch"
#line 2 "src/form/binary_other/Subtract.birch"
template<class LeftValue,class RightValue>
#line 2 "src/form/binary_other/Subtract.birch"
decltype(auto) SubtractFunction_::compute(const LeftValue& l, const RightValue& r) {
  #line 3 "src/form/binary_other/Subtract.birch"
  return l - r;
}

#line 6 "src/form/binary_other/Subtract.birch"
#line 6 "src/form/binary_other/Subtract.birch"
template<class Gradient>
#line 6 "src/form/binary_other/Subtract.birch"
decltype(auto) SubtractFunction_::grad(const Gradient& d) {
  #line 7 "src/form/binary_other/Subtract.birch"
  return std::make_tuple(d, -(d));
}

#line 2 "src/form/ternary/IfThenElse.birch"
#line 2 "src/form/ternary/IfThenElse.birch"
template<class LeftValue,class MiddleValue,class RightValue>
#line 2 "src/form/ternary/IfThenElse.birch"
decltype(auto) IfThenElseFunction_::compute(const LeftValue& l, const MiddleValue& m, const RightValue& r) {
  #line 3 "src/form/ternary/IfThenElse.birch"
  return if_then_else(l, m, r);
}

#line 6 "src/form/ternary/IfThenElse.birch"
#line 6 "src/form/ternary/IfThenElse.birch"
template<class Gradient,class LeftValue,class MiddleValue,class RightValue>
#line 6 "src/form/ternary/IfThenElse.birch"
decltype(auto) IfThenElseFunction_::grad(const Gradient& d, const LeftValue& l, const MiddleValue& m, const RightValue& r) {
  #line 8 "src/form/ternary/IfThenElse.birch"
  return std::make_tuple(0.0, if_then_else(l, d, 0.0), if_then_else(l, 0.0, d));
}

#line 2 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
#line 2 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
template<class Arg1,class Arg2,class Arg3>
#line 2 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
decltype(auto) LogZConwayMaxwellPoissonFunction_::compute(const Arg1& \u03bb, const Arg2& \u03bd, const Arg3& n) {
  #line 3 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  return logz_conway_maxwell_poisson(\u03bb, \u03bd, n);
}

#line 6 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
#line 6 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
template<class Gradient,class Arg1,class Arg2,class Arg3>
#line 6 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
decltype(auto) LogZConwayMaxwellPoissonFunction_::grad(const Gradient& d, const Arg1& \u03bb, const Arg2& \u03bd, const Arg3& n) {
  auto z = vector_lambda([=](const Integer& x) {
    #line 9 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
    return log(x) + x * log(\u03bb) - \u03bd * lgamma(x + Integer(1));
  }, n);
  auto dl = d * exp(log(\u03bb) - log_sum_exp(z));
  auto Z = 0.0;
  #line 14 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  for (auto x = Integer(1); x <= n; ++x) {
    #line 15 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
    Z = Z + lgamma(x + Integer(1)) * exp(x * log(\u03bb) - \u03bd * lgamma(x + Integer(1)));
  }
  auto dm = -(d) / Z;
  #line 19 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  return std::make_tuple(dl, dm, 0.0);
}

#line 2 "src/form/unary_matrix/CholInv.birch"
#line 2 "src/form/unary_matrix/CholInv.birch"
template<class Value>
#line 2 "src/form/unary_matrix/CholInv.birch"
decltype(auto) CholInvFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_matrix/CholInv.birch"
  return cholinv(m);
}

#line 6 "src/form/unary_matrix/CholInv.birch"
#line 6 "src/form/unary_matrix/CholInv.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_matrix/CholInv.birch"
decltype(auto) CholInvFunction_::grad(const Gradient& d, const Value& m) {
  auto m1 = cholinv(m);
  #line 8 "src/form/unary_matrix/CholInv.birch"
  return -(m1) * d * m1;
}

#line 2 "src/form/unary_matrix/Inv.birch"
#line 2 "src/form/unary_matrix/Inv.birch"
template<class Value>
#line 2 "src/form/unary_matrix/Inv.birch"
decltype(auto) InvFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_matrix/Inv.birch"
  return inv(m);
}

#line 6 "src/form/unary_matrix/Inv.birch"
#line 6 "src/form/unary_matrix/Inv.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_matrix/Inv.birch"
decltype(auto) InvFunction_::grad(const Gradient& d, const Value& m) {
  auto m1 = transpose(inv(m));
  #line 8 "src/form/unary_matrix/Inv.birch"
  return -(m1) * d * m1;
}

#line 2 "src/form/unary_matrix/LCholDet.birch"
#line 2 "src/form/unary_matrix/LCholDet.birch"
template<class Value>
#line 2 "src/form/unary_matrix/LCholDet.birch"
decltype(auto) LCholDetFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_matrix/LCholDet.birch"
  return lcholdet(m);
}

#line 6 "src/form/unary_matrix/LCholDet.birch"
#line 6 "src/form/unary_matrix/LCholDet.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_matrix/LCholDet.birch"
decltype(auto) LCholDetFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_matrix/LCholDet.birch"
  return d * cholinv(m);
}

#line 2 "src/form/unary_matrix/LDet.birch"
#line 2 "src/form/unary_matrix/LDet.birch"
template<class Value>
#line 2 "src/form/unary_matrix/LDet.birch"
decltype(auto) LDetFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_matrix/LDet.birch"
  return ldet(m);
}

#line 6 "src/form/unary_matrix/LDet.birch"
#line 6 "src/form/unary_matrix/LDet.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_matrix/LDet.birch"
decltype(auto) LDetFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_matrix/LDet.birch"
  return d * inv(transpose(m));
}

#line 1 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 1 "src/form/unary_matrix/Matrix.birch"
MatrixFunction_<Value>::MatrixFunction_() {
  //
}

#line 2 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 2 "src/form/unary_matrix/Matrix.birch"
template<class MiddleValue>
#line 2 "src/form/unary_matrix/Matrix.birch"
decltype(auto) MatrixFunction_<Value>::compute(const MiddleValue& m) {
  #line 3 "src/form/unary_matrix/Matrix.birch"
  return matrix<Value>(m);
}

#line 6 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 6 "src/form/unary_matrix/Matrix.birch"
template<class Gradient,class MiddleValue>
#line 6 "src/form/unary_matrix/Matrix.birch"
decltype(auto) MatrixFunction_<Value>::grad(const Gradient& d, const MiddleValue& m) {
  #line 7 "src/form/unary_matrix/Matrix.birch"
  return grad_matrix<Value>(d, m);
}

#line 2 "src/form/unary_matrix/Trace.birch"
#line 2 "src/form/unary_matrix/Trace.birch"
template<class Value>
#line 2 "src/form/unary_matrix/Trace.birch"
decltype(auto) TraceFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_matrix/Trace.birch"
  return trace(m);
}

#line 6 "src/form/unary_matrix/Trace.birch"
#line 6 "src/form/unary_matrix/Trace.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_matrix/Trace.birch"
decltype(auto) TraceFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_matrix/Trace.birch"
  return d;
}

#line 10 "src/form/unary_matrix/Trace.birch"
#line 10 "src/form/unary_matrix/Trace.birch"
template<class Gradient,class Value>
#line 10 "src/form/unary_matrix/Trace.birch"
decltype(auto) TraceFunction_::grad(const Gradient& d, const libbirch::DefaultArray<Value,1>& m) {
  #line 11 "src/form/unary_matrix/Trace.birch"
  return vector(d, birch::length(m));
}

#line 14 "src/form/unary_matrix/Trace.birch"
#line 14 "src/form/unary_matrix/Trace.birch"
template<class Gradient,class Value>
#line 14 "src/form/unary_matrix/Trace.birch"
decltype(auto) TraceFunction_::grad(const Gradient& d, const libbirch::DefaultArray<Value,2>& m) {
  #line 15 "src/form/unary_matrix/Trace.birch"
  return diagonal(d, birch::rows(m));
}

#line 2 "src/form/unary_matrix/Transpose.birch"
#line 2 "src/form/unary_matrix/Transpose.birch"
template<class Value>
#line 2 "src/form/unary_matrix/Transpose.birch"
decltype(auto) TransposeFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_matrix/Transpose.birch"
  return transpose(m);
}

#line 6 "src/form/unary_matrix/Transpose.birch"
#line 6 "src/form/unary_matrix/Transpose.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_matrix/Transpose.birch"
decltype(auto) TransposeFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_matrix/Transpose.birch"
  return transpose(d);
}

#line 1 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 1 "src/form/unary_matrix/Vector.birch"
VectorFunction_<Value>::VectorFunction_() {
  //
}

#line 2 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 2 "src/form/unary_matrix/Vector.birch"
template<class MiddleValue>
#line 2 "src/form/unary_matrix/Vector.birch"
decltype(auto) VectorFunction_<Value>::compute(const MiddleValue& m) {
  #line 3 "src/form/unary_matrix/Vector.birch"
  return vector<Value>(m);
}

#line 6 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 6 "src/form/unary_matrix/Vector.birch"
template<class Gradient,class MiddleValue>
#line 6 "src/form/unary_matrix/Vector.birch"
decltype(auto) VectorFunction_<Value>::grad(const Gradient& d, const MiddleValue& m) {
  #line 7 "src/form/unary_matrix/Vector.birch"
  return grad_vector(d, m);
}

#line 2 "src/form/unary_other/Abs.birch"
#line 2 "src/form/unary_other/Abs.birch"
template<class Value>
#line 2 "src/form/unary_other/Abs.birch"
decltype(auto) AbsFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Abs.birch"
  return abs(m);
}

#line 6 "src/form/unary_other/Abs.birch"
#line 6 "src/form/unary_other/Abs.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Abs.birch"
decltype(auto) AbsFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Abs.birch"
  return copysign(d, m);
}

#line 2 "src/form/unary_other/Acos.birch"
#line 2 "src/form/unary_other/Acos.birch"
template<class Value>
#line 2 "src/form/unary_other/Acos.birch"
decltype(auto) AcosFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Acos.birch"
  return acos(m);
}

#line 6 "src/form/unary_other/Acos.birch"
#line 6 "src/form/unary_other/Acos.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Acos.birch"
decltype(auto) AcosFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Acos.birch"
  return -(d) / sqrt(1.0 - pow(m, 2.0));
}

#line 2 "src/form/unary_other/Asin.birch"
#line 2 "src/form/unary_other/Asin.birch"
template<class Value>
#line 2 "src/form/unary_other/Asin.birch"
decltype(auto) AsinFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Asin.birch"
  return asin(m);
}

#line 6 "src/form/unary_other/Asin.birch"
#line 6 "src/form/unary_other/Asin.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Asin.birch"
decltype(auto) AsinFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Asin.birch"
  return d / sqrt(1.0 - pow(m, 2.0));
}

#line 2 "src/form/unary_other/Atan.birch"
#line 2 "src/form/unary_other/Atan.birch"
template<class Value>
#line 2 "src/form/unary_other/Atan.birch"
decltype(auto) AtanFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Atan.birch"
  return atan(m);
}

#line 6 "src/form/unary_other/Atan.birch"
#line 6 "src/form/unary_other/Atan.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Atan.birch"
decltype(auto) AtanFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Atan.birch"
  return d / (1.0 + pow(m, 2.0));
}

#line 2 "src/form/unary_other/Cos.birch"
#line 2 "src/form/unary_other/Cos.birch"
template<class Value>
#line 2 "src/form/unary_other/Cos.birch"
decltype(auto) CosFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Cos.birch"
  return cos(m);
}

#line 6 "src/form/unary_other/Cos.birch"
#line 6 "src/form/unary_other/Cos.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Cos.birch"
decltype(auto) CosFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Cos.birch"
  return -(d) * sin(m);
}

#line 2 "src/form/unary_other/Cosh.birch"
#line 2 "src/form/unary_other/Cosh.birch"
template<class Value>
#line 2 "src/form/unary_other/Cosh.birch"
decltype(auto) CoshFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Cosh.birch"
  return cosh(m);
}

#line 6 "src/form/unary_other/Cosh.birch"
#line 6 "src/form/unary_other/Cosh.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Cosh.birch"
decltype(auto) CoshFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Cosh.birch"
  return -(d) * sinh(m);
}

#line 2 "src/form/unary_other/Exp.birch"
#line 2 "src/form/unary_other/Exp.birch"
template<class Value>
#line 2 "src/form/unary_other/Exp.birch"
decltype(auto) ExpFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Exp.birch"
  return exp(m);
}

#line 6 "src/form/unary_other/Exp.birch"
#line 6 "src/form/unary_other/Exp.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Exp.birch"
decltype(auto) ExpFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Exp.birch"
  return d * exp(m);
}

#line 2 "src/form/unary_other/Expm1.birch"
#line 2 "src/form/unary_other/Expm1.birch"
template<class Value>
#line 2 "src/form/unary_other/Expm1.birch"
decltype(auto) Expm1Function_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Expm1.birch"
  return expm1(m);
}

#line 6 "src/form/unary_other/Expm1.birch"
#line 6 "src/form/unary_other/Expm1.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Expm1.birch"
decltype(auto) Expm1Function_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Expm1.birch"
  return d * exp(m);
}

#line 2 "src/form/unary_other/Log.birch"
#line 2 "src/form/unary_other/Log.birch"
template<class Value>
#line 2 "src/form/unary_other/Log.birch"
decltype(auto) LogFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Log.birch"
  return log(m);
}

#line 6 "src/form/unary_other/Log.birch"
#line 6 "src/form/unary_other/Log.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Log.birch"
decltype(auto) LogFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Log.birch"
  return d / m;
}

#line 2 "src/form/unary_other/Log1p.birch"
#line 2 "src/form/unary_other/Log1p.birch"
template<class Value>
#line 2 "src/form/unary_other/Log1p.birch"
decltype(auto) Log1pFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Log1p.birch"
  return log1p(m);
}

#line 6 "src/form/unary_other/Log1p.birch"
#line 6 "src/form/unary_other/Log1p.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Log1p.birch"
decltype(auto) Log1pFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Log1p.birch"
  return d / (1.0 + m);
}

#line 2 "src/form/unary_other/LogGamma.birch"
#line 2 "src/form/unary_other/LogGamma.birch"
template<class Value>
#line 2 "src/form/unary_other/LogGamma.birch"
decltype(auto) LogGammaFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/LogGamma.birch"
  return lgamma(m);
}

#line 6 "src/form/unary_other/LogGamma.birch"
#line 6 "src/form/unary_other/LogGamma.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/LogGamma.birch"
decltype(auto) LogGammaFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/LogGamma.birch"
  return d * digamma(m);
}

#line 2 "src/form/unary_other/Negate.birch"
#line 2 "src/form/unary_other/Negate.birch"
template<class Value>
#line 2 "src/form/unary_other/Negate.birch"
decltype(auto) NegateFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Negate.birch"
  return -(m);
}

#line 6 "src/form/unary_other/Negate.birch"
#line 6 "src/form/unary_other/Negate.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Negate.birch"
decltype(auto) NegateFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Negate.birch"
  return -(d);
}

#line 2 "src/form/unary_other/Rectify.birch"
#line 2 "src/form/unary_other/Rectify.birch"
template<class Value>
#line 2 "src/form/unary_other/Rectify.birch"
decltype(auto) RectifyFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Rectify.birch"
  return rectify(m);
}

#line 6 "src/form/unary_other/Rectify.birch"
#line 6 "src/form/unary_other/Rectify.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Rectify.birch"
decltype(auto) RectifyFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Rectify.birch"
  return transform(d, m, [=](const Real& d, const Real& m) -> Real {
    #line 8 "src/form/unary_other/Rectify.birch"
    return if_then_else(m > 0.0, d, 0.0);
  });
}

#line 1 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 1 "src/form/unary_other/Scalar.birch"
ScalarFunction_<Value>::ScalarFunction_() {
  //
}

#line 2 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 2 "src/form/unary_other/Scalar.birch"
template<class MiddleValue>
#line 2 "src/form/unary_other/Scalar.birch"
decltype(auto) ScalarFunction_<Value>::compute(const MiddleValue& m) {
  #line 3 "src/form/unary_other/Scalar.birch"
  return scalar<Value>(m);
}

#line 6 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 6 "src/form/unary_other/Scalar.birch"
template<class Gradient,class MiddleValue>
#line 6 "src/form/unary_other/Scalar.birch"
decltype(auto) ScalarFunction_<Value>::grad(const Gradient& d, const MiddleValue& m) {
  #line 7 "src/form/unary_other/Scalar.birch"
  return grad_scalar(d, m);
}

#line 2 "src/form/unary_other/Sin.birch"
#line 2 "src/form/unary_other/Sin.birch"
template<class Value>
#line 2 "src/form/unary_other/Sin.birch"
decltype(auto) SinFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Sin.birch"
  return sin(m);
}

#line 6 "src/form/unary_other/Sin.birch"
#line 6 "src/form/unary_other/Sin.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Sin.birch"
decltype(auto) SinFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Sin.birch"
  return d * cos(m);
}

#line 2 "src/form/unary_other/Sinh.birch"
#line 2 "src/form/unary_other/Sinh.birch"
template<class Value>
#line 2 "src/form/unary_other/Sinh.birch"
decltype(auto) SinhFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Sinh.birch"
  return sinh(m);
}

#line 6 "src/form/unary_other/Sinh.birch"
#line 6 "src/form/unary_other/Sinh.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Sinh.birch"
decltype(auto) SinhFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Sinh.birch"
  return d * cosh(m);
}

#line 2 "src/form/unary_other/Sqrt.birch"
#line 2 "src/form/unary_other/Sqrt.birch"
template<class Value>
#line 2 "src/form/unary_other/Sqrt.birch"
decltype(auto) SqrtFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Sqrt.birch"
  return sqrt(m);
}

#line 6 "src/form/unary_other/Sqrt.birch"
#line 6 "src/form/unary_other/Sqrt.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Sqrt.birch"
decltype(auto) SqrtFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Sqrt.birch"
  return d * 0.5 / sqrt(m);
}

#line 2 "src/form/unary_other/Sum.birch"
#line 2 "src/form/unary_other/Sum.birch"
template<class Value>
#line 2 "src/form/unary_other/Sum.birch"
decltype(auto) SumFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Sum.birch"
  return sum(m);
}

#line 6 "src/form/unary_other/Sum.birch"
#line 6 "src/form/unary_other/Sum.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Sum.birch"
decltype(auto) SumFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Sum.birch"
  return d;
}

#line 10 "src/form/unary_other/Sum.birch"
#line 10 "src/form/unary_other/Sum.birch"
template<class Gradient,class Value>
#line 10 "src/form/unary_other/Sum.birch"
decltype(auto) SumFunction_::grad(const Gradient& d, const libbirch::DefaultArray<Value,1>& m) {
  #line 11 "src/form/unary_other/Sum.birch"
  return vector(d, length(m));
}

#line 14 "src/form/unary_other/Sum.birch"
#line 14 "src/form/unary_other/Sum.birch"
template<class Gradient,class Value>
#line 14 "src/form/unary_other/Sum.birch"
decltype(auto) SumFunction_::grad(const Gradient& d, const libbirch::DefaultArray<Value,2>& m) {
  #line 15 "src/form/unary_other/Sum.birch"
  return vector(d, rows(m), columns(m));
}

#line 2 "src/form/unary_other/Tan.birch"
#line 2 "src/form/unary_other/Tan.birch"
template<class Value>
#line 2 "src/form/unary_other/Tan.birch"
decltype(auto) TanFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Tan.birch"
  return tan(m);
}

#line 6 "src/form/unary_other/Tan.birch"
#line 6 "src/form/unary_other/Tan.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Tan.birch"
decltype(auto) TanFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Tan.birch"
  return d * (1.0 + pow(tan(m), 2.0));
}

#line 2 "src/form/unary_other/Tanh.birch"
#line 2 "src/form/unary_other/Tanh.birch"
template<class Value>
#line 2 "src/form/unary_other/Tanh.birch"
decltype(auto) TanhFunction_::compute(const Value& m) {
  #line 3 "src/form/unary_other/Tanh.birch"
  return tanh(m);
}

#line 6 "src/form/unary_other/Tanh.birch"
#line 6 "src/form/unary_other/Tanh.birch"
template<class Gradient,class Value>
#line 6 "src/form/unary_other/Tanh.birch"
decltype(auto) TanhFunction_::grad(const Gradient& d, const Value& m) {
  #line 7 "src/form/unary_other/Tanh.birch"
  return d * (1.0 + pow(tanh(m), 2.0));
}

#line 61 "src/basic/Boolean.birch"
template<class Gradient>
#line 61 "src/basic/Boolean.birch"
inline void grad(const Boolean& x, const Gradient& d) {
}

#line 75 "src/basic/Integer.birch"
template<class Gradient>
#line 75 "src/basic/Integer.birch"
inline void grad(const Integer& x, const Gradient& d) {
}

#line 102 "src/basic/Real.birch"
template<class Gradient>
#line 102 "src/basic/Real.birch"
inline void grad(const Real& x, const Gradient& d) {
}

#line 91 "src/basic/String.birch"
template<class Type>
#line 91 "src/basic/String.birch"
inline String to_string(const libbirch::DefaultArray<Type,1>& x) {
  String result;
  #line 93 "src/basic/String.birch"

  std::stringstream buf;
    #line 96 "src/basic/String.birch"
  for (auto i = Integer(1); i <= length(x); ++i) {
    auto value = x(i);
    #line 98 "src/basic/String.birch"

    if (i > 1) {
      buf << ' ';
    }
    buf << to_string(value);
      }
  #line 105 "src/basic/String.birch"

  result = buf.str();
    #line 108 "src/basic/String.birch"
  return result;
}

#line 114 "src/basic/String.birch"
template<class Type>
#line 114 "src/basic/String.birch"
inline String to_string(const libbirch::DefaultArray<Type,2>& X) {
  String result;
  #line 116 "src/basic/String.birch"

  std::stringstream buf;
    #line 119 "src/basic/String.birch"
  for (auto i = Integer(1); i <= rows(X); ++i) {
    #line 120 "src/basic/String.birch"

    if (i > 1) {
      buf << '\n';
    }
        #line 125 "src/basic/String.birch"
    for (auto j = Integer(1); j <= columns(X); ++j) {
      auto value = X(i, j);
      #line 127 "src/basic/String.birch"

      if (j > 1) {
        buf << ' ';
      }
      buf << to_string(value);
          }
  }
  #line 135 "src/basic/String.birch"

  result = buf.str();
    #line 138 "src/basic/String.birch"
  return result;
}

#line 146 "src/basic/String.birch"
template<class Value>
#line 146 "src/basic/String.birch"
inline Value from_string(const String& s) {
  #line 147 "src/basic/String.birch"

  if (std::is_same<Value,Real>::value) {
    return static_cast<Value>(std::stod(s));
  } else if (std::is_same<Value,Integer>::value) {
    return static_cast<Value>(std::stoll(s));
  } else if (s == "true" || s == "True") {
    return true;
  } else {
    return false;
  }
  }

#line 4 "src/basic/matrix.birch"
template<class Type>
#line 4 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,2> row(const libbirch::DefaultArray<Type,1>& x) {
  #line 5 "src/basic/matrix.birch"
  return matrix_lambda([=](const Integer& i, const Integer& j) {
    #line 5 "src/basic/matrix.birch"
    return x(j);
  }, Integer(1), length(x));
}

#line 12 "src/basic/matrix.birch"
template<class Type>
#line 12 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,1> row(const libbirch::DefaultArray<Type,2>& X, const Integer& i) {
  #line 13 "src/basic/matrix.birch"
  return X(i, libbirch::make_range(Integer(1), columns(X)));
}

#line 19 "src/basic/matrix.birch"
template<class Type>
#line 19 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,2> column(const libbirch::DefaultArray<Type,1>& x) {
  #line 20 "src/basic/matrix.birch"
  return matrix_lambda([=](const Integer& i, const Integer& j) {
    #line 20 "src/basic/matrix.birch"
    return x(i);
  }, Integer(1), length(x));
}

#line 27 "src/basic/matrix.birch"
template<class Type>
#line 27 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,1> column(const libbirch::DefaultArray<Type,2>& X, const Integer& j) {
  #line 28 "src/basic/matrix.birch"
  return X(libbirch::make_range(Integer(1), rows(X)), j);
}

#line 37 "src/basic/matrix.birch"
template<class Type>
#line 37 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,1> vec(const libbirch::DefaultArray<Type,2>& X) {
  auto R = rows(X);
  auto C = columns(X);
  #line 40 "src/basic/matrix.birch"
  return vector_lambda([=](const Integer& i) -> Type {
    #line 41 "src/basic/matrix.birch"
    return X(mod(i - Integer(1), R) + Integer(1), (i - Integer(1)) / R + Integer(1));
  }, R * C);
}

#line 52 "src/basic/matrix.birch"
template<class Type>
#line 52 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,2> mat(const libbirch::DefaultArray<Type,1>& x, const Integer& columns) {
  #line 53 "src/basic/matrix.birch"
  assert(mod(length(x), columns) == Integer(0));
  auto R = length(x) / columns;
  auto C = columns;
  #line 56 "src/basic/matrix.birch"
  return matrix_lambda([=](const Integer& i, const Integer& j) -> Type {
    #line 57 "src/basic/matrix.birch"
    return x((j - Integer(1)) * R + i);
  }, R, C);
}

#line 83 "src/basic/matrix.birch"
template<class Lambda>
#line 83 "src/basic/matrix.birch"
inline decltype(auto) matrix_lambda(const Lambda& \u03bb, const Integer& rows, const Integer& columns) {
  #line 84 "src/basic/matrix.birch"

  /* wrap \u03bb in another lambda function to translate 0-based serial (row-wise)
   * indices into 1-based row/column indices */
  return libbirch::make_array_from_lambda(libbirch::make_shape(rows,
      columns), [&](int64_t i) { return \u03bb(i/columns + 1, i%columns + 1); });
  }

#line 99 "src/basic/matrix.birch"
template<class Type>
#line 99 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,2> matrix(const Type& x, const Integer& rows, const Integer& columns) {
  #line 100 "src/basic/matrix.birch"

  return libbirch::make_array_from_value<Type>(libbirch::make_shape(rows,
      columns), x);
  }

#line 106 "src/basic/matrix.birch"
template<class Type>
#line 106 "src/basic/matrix.birch"
inline Integer rows(const libbirch::DefaultArray<Type,2>& x) {
  #line 107 "src/basic/matrix.birch"

  return x.rows();
  }

#line 112 "src/basic/matrix.birch"
template<class Type>
#line 112 "src/basic/matrix.birch"
inline Integer columns(const libbirch::DefaultArray<Type,2>& x) {
  #line 113 "src/basic/matrix.birch"

  return x.cols();
  }

#line 118 "src/basic/matrix.birch"
template<class Type>
#line 118 "src/basic/matrix.birch"
inline Integer length(const libbirch::DefaultArray<Type,2>& x) {
  #line 119 "src/basic/matrix.birch"
  return rows(x);
}

#line 122 "src/basic/matrix.birch"
template<class Type>
#line 122 "src/basic/matrix.birch"
inline Integer size(const libbirch::DefaultArray<Type,2>& x) {
  #line 123 "src/basic/matrix.birch"
  return rows(x) * columns(x);
}

#line 126 "src/basic/matrix.birch"
template<class Type>
#line 126 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,2> value(const libbirch::DefaultArray<Type,2>& x) {
  #line 127 "src/basic/matrix.birch"
  return x;
}

#line 130 "src/basic/matrix.birch"
template<class Type>
#line 130 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,2> peek(const libbirch::DefaultArray<Type,2>& x) {
  #line 131 "src/basic/matrix.birch"
  return x;
}

#line 134 "src/basic/matrix.birch"
template<class Type>
#line 134 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,2> eval(const libbirch::DefaultArray<Type,2>& x) {
  #line 135 "src/basic/matrix.birch"
  return x;
}

#line 138 "src/basic/matrix.birch"
template<class Type>
#line 138 "src/basic/matrix.birch"
inline libbirch::DefaultArray<Type,2> reval(const libbirch::DefaultArray<Type,2>& x) {
  #line 139 "src/basic/matrix.birch"
  return x;
}

#line 142 "src/basic/matrix.birch"
template<class Type,class Gradient>
#line 142 "src/basic/matrix.birch"
inline void grad(const libbirch::DefaultArray<Type,2>& x, const Gradient& d) {
}

#line 146 "src/basic/matrix.birch"
template<class Type,class Gradient>
#line 146 "src/basic/matrix.birch"
inline void grad(const libbirch::DefaultArray<Type,2>& x, const Gradient& d, const Integer& i, const Integer& j) {
}

#line 150 "src/basic/matrix.birch"
template<class Type>
#line 150 "src/basic/matrix.birch"
inline void label(const libbirch::DefaultArray<Type,2>& x, const Integer& gen) {
}

#line 154 "src/basic/matrix.birch"
template<class Type>
#line 154 "src/basic/matrix.birch"
inline void constant(const libbirch::DefaultArray<Type,2>& x, const Integer& gen) {
}

#line 158 "src/basic/matrix.birch"
template<class Type>
#line 158 "src/basic/matrix.birch"
inline void constant(const libbirch::DefaultArray<Type,2>& x) {
}

#line 13 "src/basic/vector.birch"
template<class Lambda>
#line 13 "src/basic/vector.birch"
inline decltype(auto) vector_lambda(const Lambda& \u03bb, const Integer& length) {
  #line 14 "src/basic/vector.birch"

  /* wrap \u03bb in another lambda function to translate 0-based indices into
   * 1-based indices */
  return libbirch::make_array_from_lambda(libbirch::make_shape(length),
        [&](int64_t i) { return \u03bb(i + 1); });
  }

#line 25 "src/basic/vector.birch"
template<class Type>
#line 25 "src/basic/vector.birch"
inline libbirch::DefaultArray<Type,1> vector(const Type& x, const Integer& length) {
  #line 26 "src/basic/vector.birch"

  return libbirch::make_array_from_value<Type>(libbirch::make_shape(length),
      x);
  }

#line 32 "src/basic/vector.birch"
template<class Type>
#line 32 "src/basic/vector.birch"
inline Integer rows(const libbirch::DefaultArray<Type,1>& x) {
  #line 33 "src/basic/vector.birch"

  return x.rows();
  }

#line 38 "src/basic/vector.birch"
template<class Type>
#line 38 "src/basic/vector.birch"
inline Integer columns(const libbirch::DefaultArray<Type,1>& x) {
  #line 39 "src/basic/vector.birch"
  return Integer(1);
}

#line 42 "src/basic/vector.birch"
template<class Type>
#line 42 "src/basic/vector.birch"
inline Integer length(const libbirch::DefaultArray<Type,1>& x) {
  #line 43 "src/basic/vector.birch"
  return rows(x);
}

#line 46 "src/basic/vector.birch"
template<class Type>
#line 46 "src/basic/vector.birch"
inline Integer size(const libbirch::DefaultArray<Type,1>& x) {
  #line 47 "src/basic/vector.birch"
  return rows(x) * columns(x);
}

#line 50 "src/basic/vector.birch"
template<class Type>
#line 50 "src/basic/vector.birch"
inline libbirch::DefaultArray<Type,1> value(const libbirch::DefaultArray<Type,1>& x) {
  #line 51 "src/basic/vector.birch"
  return x;
}

#line 54 "src/basic/vector.birch"
template<class Type>
#line 54 "src/basic/vector.birch"
inline libbirch::DefaultArray<Type,1> peek(const libbirch::DefaultArray<Type,1>& x) {
  #line 55 "src/basic/vector.birch"
  return x;
}

#line 58 "src/basic/vector.birch"
template<class Type>
#line 58 "src/basic/vector.birch"
inline libbirch::DefaultArray<Type,1> eval(const libbirch::DefaultArray<Type,1>& x) {
  #line 59 "src/basic/vector.birch"
  return x;
}

#line 62 "src/basic/vector.birch"
template<class Type>
#line 62 "src/basic/vector.birch"
inline libbirch::DefaultArray<Type,1> reval(const libbirch::DefaultArray<Type,1>& x) {
  #line 63 "src/basic/vector.birch"
  return x;
}

#line 66 "src/basic/vector.birch"
template<class Type,class Gradient>
#line 66 "src/basic/vector.birch"
inline void grad(const libbirch::DefaultArray<Type,1>& x, const Gradient& d) {
}

#line 70 "src/basic/vector.birch"
template<class Type,class Gradient>
#line 70 "src/basic/vector.birch"
inline void grad(const libbirch::DefaultArray<Type,1>& x, const Gradient& d, const Integer& i) {
}

#line 74 "src/basic/vector.birch"
template<class Type>
#line 74 "src/basic/vector.birch"
inline void label(const libbirch::DefaultArray<Type,1>& x, const Integer& gen) {
}

#line 78 "src/basic/vector.birch"
template<class Type>
#line 78 "src/basic/vector.birch"
inline void constant(const libbirch::DefaultArray<Type,1>& x, const Integer& gen) {
}

#line 82 "src/basic/vector.birch"
template<class Type>
#line 82 "src/basic/vector.birch"
inline void constant(const libbirch::DefaultArray<Type,1>& x) {
}

#line 995 "src/data/Buffer.birch"
template<class Type>
#line 995 "src/data/Buffer.birch"
inline Buffer make_buffer(const Type& root) {
  Buffer o;
  #line 997 "src/data/Buffer.birch"
  o->set(root);
  #line 998 "src/data/Buffer.birch"
  return o;
}

#line 1007 "src/data/Buffer.birch"
template<class Type>
#line 1007 "src/data/Buffer.birch"
inline Buffer make_buffer(const Integer& t, const Type& root) {
  Buffer o;
  #line 1009 "src/data/Buffer.birch"
  o->set(t, root);
  #line 1010 "src/data/Buffer.birch"
  return o;
}

#line 500 "src/event/Handler.birch"
template<class Value>
#line 500 "src/event/Handler.birch"
inline Integer argSize(const Tape<Random<Value>>& o) {
  auto n = Integer(0);
  auto i = Integer(1);
  #line 503 "src/event/Handler.birch"
  while (i <= o->size()) {
    #line 504 "src/event/Handler.birch"
    if (!(o(i)->isConstant())) {
      #line 505 "src/event/Handler.birch"
      n = n + o(i)->size();
      #line 506 "src/event/Handler.birch"
      i = i + Integer(1);
    } else {
      #line 508 "src/event/Handler.birch"
      o->erase(i);
    }
  }
  #line 511 "src/event/Handler.birch"
  return n;
}

#line 518 "src/event/Handler.birch"
template<class Value>
#line 518 "src/event/Handler.birch"
inline Integer argSizeReverse(const Tape<Random<Value>>& o) {
  auto n = Integer(0);
  auto i = o->size();
  #line 522 "src/event/Handler.birch"
  while (i >= Integer(1)) {
    #line 523 "src/event/Handler.birch"
    if (!(o(i)->isConstant())) {
      #line 524 "src/event/Handler.birch"
      n = n + o(i)->size();
    } else {
      #line 526 "src/event/Handler.birch"
      o->erase(i);
    }
    #line 528 "src/event/Handler.birch"
    i = i - Integer(1);
  }
  #line 530 "src/event/Handler.birch"
  return n;
}

#line 65 "src/event/handle.birch"
template<class Left,class Right>
#line 65 "src/event/handle.birch"
inline Left handle_assume(const Left& x, const Right& p) {
  #line 66 "src/event/handle.birch"
  return get_handler()->handleAssume<typename Right::value_type::Value_>(x, p);
}

#line 83 "src/event/handle.birch"
template<class Right>
#line 83 "src/event/handle.birch"
inline Random<typename Right::value_type::Value_> handle_assume(const Right& p) {
  #line 84 "src/event/handle.birch"
  return handle_assume(make_random<typename Right::value_type::Value_>(), p);
}

#line 100 "src/event/handle.birch"
template<class Left,class Right>
#line 100 "src/event/handle.birch"
inline Left handle_simulate(const Left& x, const Right& p) {
  #line 101 "src/event/handle.birch"
  return get_handler()->handleSimulate<typename Right::value_type::Value_>(x, p);
}

#line 116 "src/event/handle.birch"
template<class Right>
#line 116 "src/event/handle.birch"
inline typename Right::value_type::Value_ handle_simulate(const Right& p) {
  #line 117 "src/event/handle.birch"
  return get_handler()->handleSimulate<typename Right::value_type::Value_>(p);
}

#line 133 "src/event/handle.birch"
template<class Left,class Right>
#line 133 "src/event/handle.birch"
inline Left handle_observe(const Left& x, const Right& p) {
  #line 134 "src/event/handle.birch"
  return get_handler()->handleObserve<typename Right::value_type::Value_>(x, p);
}

#line 147 "src/event/handle.birch"
template<class Arg>
#line 147 "src/event/handle.birch"
inline void handle_factor(const Arg& w) {
  #line 148 "src/event/handle.birch"
  get_handler()->handleFactor<Arg>(w);
}

#line 156 "src/event/handle.birch"
template<class Value>
#line 156 "src/event/handle.birch"
inline void handle_delay_simulate(const Random<Value>& x) {
  #line 157 "src/event/handle.birch"
  get_handler()->handleDelaySimulate<Value>(x);
}

#line 165 "src/event/handle.birch"
template<class Value>
#line 165 "src/event/handle.birch"
inline void handle_delay_observe(const Random<Value>& x) {
  #line 166 "src/event/handle.birch"
  get_handler()->handleDelayObserve<Value>(x);
}

#line 177 "src/event/handle.birch"
template<class Left,class Right>
#line 177 "src/event/handle.birch"
inline Delay handle_prune(const Left& p, const Right& x) {
  #line 178 "src/event/handle.birch"
  return get_handler()->handleDelayPrune<typename Left::value_type::Value_>(p, x);
}

#line 229 "src/expression/Expression.birch"
template<class Type>
#line 229 "src/expression/Expression.birch"
inline Integer length(const Type& x) {
  #line 230 "src/expression/Expression.birch"
  return x->length();
}

#line 233 "src/expression/Expression.birch"
template<class Type>
#line 233 "src/expression/Expression.birch"
inline Integer size(const Type& x) {
  #line 234 "src/expression/Expression.birch"
  return x->size();
}

#line 237 "src/expression/Expression.birch"
template<class Type>
#line 237 "src/expression/Expression.birch"
inline Integer rows(const Type& x) {
  #line 238 "src/expression/Expression.birch"
  return x->rows();
}

#line 241 "src/expression/Expression.birch"
template<class Type>
#line 241 "src/expression/Expression.birch"
inline Integer columns(const Type& x) {
  #line 242 "src/expression/Expression.birch"
  return x->columns();
}

#line 245 "src/expression/Expression.birch"
template<class Type>
#line 245 "src/expression/Expression.birch"
inline decltype(auto) value(const Type& x) {
  #line 246 "src/expression/Expression.birch"
  return x->value();
}

#line 249 "src/expression/Expression.birch"
template<class Type>
#line 249 "src/expression/Expression.birch"
inline decltype(auto) peek(const Type& x) {
  #line 250 "src/expression/Expression.birch"
  return x->peek();
}

#line 253 "src/expression/Expression.birch"
template<class Type>
#line 253 "src/expression/Expression.birch"
inline decltype(auto) eval(const Type& x) {
  #line 254 "src/expression/Expression.birch"
  return x->eval();
}

#line 257 "src/expression/Expression.birch"
template<class Type>
#line 257 "src/expression/Expression.birch"
inline decltype(auto) reval(const Type& x) {
  #line 258 "src/expression/Expression.birch"
  return x->reval();
}

#line 261 "src/expression/Expression.birch"
template<class Type,class Gradient>
#line 261 "src/expression/Expression.birch"
inline void grad(const Type& x, const Gradient& d) {
  #line 262 "src/expression/Expression.birch"
  x->grad(d);
}

#line 265 "src/expression/Expression.birch"
template<class Type,class Gradient>
#line 265 "src/expression/Expression.birch"
inline void grad(const Type& x, const Gradient& d, const Integer& i) {
  #line 266 "src/expression/Expression.birch"
  x->grad(d, i);
}

#line 269 "src/expression/Expression.birch"
template<class Type,class Gradient>
#line 269 "src/expression/Expression.birch"
inline void grad(const Type& x, const Gradient& d, const Integer& i, const Integer& j) {
  #line 270 "src/expression/Expression.birch"
  x->grad(d, i, j);
}

#line 273 "src/expression/Expression.birch"
template<class Type>
#line 273 "src/expression/Expression.birch"
inline void label(const Type& x, const Integer& gen) {
  #line 274 "src/expression/Expression.birch"
  x->label(gen);
}

#line 277 "src/expression/Expression.birch"
template<class Type>
#line 277 "src/expression/Expression.birch"
inline void constant(const Type& x, const Integer& gen) {
  #line 278 "src/expression/Expression.birch"
  x->constant(gen);
}

#line 281 "src/expression/Expression.birch"
template<class Type>
#line 281 "src/expression/Expression.birch"
inline void constant(const Type& x) {
  #line 282 "src/expression/Expression.birch"
  x->constant();
}

#line 35 "src/expression/MatrixSplitExpression.birch"
template<class Form>
#line 35 "src/expression/MatrixSplitExpression.birch"
inline decltype(auto) matrix_split(const Form& f) {
  #line 36 "src/expression/MatrixSplitExpression.birch"
  return matrix_split(eval(f), f);
}

#line 45 "src/expression/MatrixSplitExpression.birch"
template<class Value,class Form>
#line 45 "src/expression/MatrixSplitExpression.birch"
inline decltype(auto) matrix_split(const Value& x, const Form& f) {
  #line 46 "src/expression/MatrixSplitExpression.birch"
  return construct<MatrixSplitExpression<Value, Form>>(x, f);
}

#line 353 "src/expression/Random.birch"
template<class Value>
#line 353 "src/expression/Random.birch"
inline Random<Value> make_random() {
  Random<Value> o;
  #line 355 "src/expression/Random.birch"
  return o;
}

#line 363 "src/expression/Random.birch"
template<class Value>
#line 363 "src/expression/Random.birch"
inline Random<Value> make_random(const Value& x) {
  Random<Value> o;
  #line 365 "src/expression/Random.birch"
  o = x;
  #line 366 "src/expression/Random.birch"
  return o;
}

#line 374 "src/expression/Random.birch"
template<class Value>
#line 374 "src/expression/Random.birch"
inline Random<Value> make_random(const std::optional<Value>& x) {
  Random<Value> o;
  #line 376 "src/expression/Random.birch"
  if (x.has_value()) {
    #line 377 "src/expression/Random.birch"
    o = x.value();
  }
  #line 379 "src/expression/Random.birch"
  return o;
}

#line 387 "src/expression/Random.birch"
template<class Value>
#line 387 "src/expression/Random.birch"
inline Random<Value> make_random(const Random<Value>& x) {
  #line 388 "src/expression/Random.birch"
  return x;
}

#line 37 "src/expression/VectorSplitExpression.birch"
template<class Form>
#line 37 "src/expression/VectorSplitExpression.birch"
inline decltype(auto) vector_split(const Form& f) {
  #line 38 "src/expression/VectorSplitExpression.birch"
  return vector_split(eval(f), f);
}

#line 47 "src/expression/VectorSplitExpression.birch"
template<class Value,class Form>
#line 47 "src/expression/VectorSplitExpression.birch"
inline decltype(auto) vector_split(const Value& x, const Form& f) {
  #line 48 "src/expression/VectorSplitExpression.birch"
  return construct<VectorSplitExpression<Value, Form>>(x, f);
}

#line 4 "src/expression/box.birch"
template<class Value>
#line 4 "src/expression/box.birch"
inline Expression<Value> box(const Expression<Value>& x) {
  #line 5 "src/expression/box.birch"
  return x;
}

#line 11 "src/expression/box.birch"
template<class Value,class Form>
#line 11 "src/expression/box.birch"
inline Expression<Value> box(const BoxedForm<Value, Form>& x) {
  #line 12 "src/expression/box.birch"
  return x;
}

#line 18 "src/expression/box.birch"
template<class Value>
#line 18 "src/expression/box.birch"
inline Expression<Value> box(const BoxedValue<Value>& x) {
  #line 19 "src/expression/box.birch"
  return x;
}

#line 25 "src/expression/box.birch"
template<class Value,class Form>
#line 25 "src/expression/box.birch"
inline Expression<libbirch::DefaultArray<Value,2>> box(const MatrixSplitExpression<libbirch::DefaultArray<Value,2>, Form>& x) {
  #line 27 "src/expression/box.birch"
  return x;
}

#line 33 "src/expression/box.birch"
template<class Value>
#line 33 "src/expression/box.birch"
inline Expression<Value> box(const Random<Value>& x) {
  #line 34 "src/expression/box.birch"
  return x;
}

#line 40 "src/expression/box.birch"
template<class Value,class Form>
#line 40 "src/expression/box.birch"
inline Expression<libbirch::DefaultArray<Value,1>> box(const VectorSplitExpression<libbirch::DefaultArray<Value,1>, Form>& x) {
  #line 42 "src/expression/box.birch"
  return x;
}

#line 69 "src/expression/box.birch"
template<class Type>
#line 69 "src/expression/box.birch"
inline Expression<libbirch::DefaultArray<Type,1>> box(const libbirch::DefaultArray<Type,1>& x) {
  #line 70 "src/expression/box.birch"
  return construct<BoxedValue<libbirch::DefaultArray<Type,1>>>(x);
}

#line 76 "src/expression/box.birch"
template<class Type>
#line 76 "src/expression/box.birch"
inline Expression<libbirch::DefaultArray<Type,2>> box(const libbirch::DefaultArray<Type,2>& x) {
  #line 77 "src/expression/box.birch"
  return construct<BoxedValue<libbirch::DefaultArray<Type,2>>>(x);
}

#line 85 "src/expression/box.birch"
template<class Form>
#line 85 "src/expression/box.birch"
inline decltype(auto) box(const Form& f) {
  #line 86 "src/expression/box.birch"
  return box_form(f->eval(), f);
}

#line 88 "src/expression/box.birch"
template<class Value,class Form>
#line 88 "src/expression/box.birch"
inline Expression<Value> box_form(const Value& x, const Form& f) {
  #line 89 "src/expression/box.birch"
  return construct<BoxedForm<Value, Form>>(x, f);
}

#line 95 "src/expression/box.birch"
template<class Value1,class Value2>
#line 95 "src/expression/box.birch"
inline decltype(auto) box(const std::tuple<Value1, Value2>& x) {
  auto n4519_ = x;
  auto x1 = std::get<0>(n4519_);
  auto x2 = std::get<1>(n4519_);
  #line 97 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2));
}

#line 103 "src/expression/box.birch"
template<class Value1,class Value2,class Value3>
#line 103 "src/expression/box.birch"
inline decltype(auto) box(const std::tuple<Value1, Value2, Value3>& x) {
  auto n4520_ = x;
  auto x1 = std::get<0>(n4520_);
  auto x2 = std::get<1>(n4520_);
  auto x3 = std::get<2>(n4520_);
  #line 105 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3));
}

#line 111 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4>
#line 111 "src/expression/box.birch"
inline decltype(auto) box(const std::tuple<Value1, Value2, Value3, Value4>& x) {
  auto n4521_ = x;
  auto x1 = std::get<0>(n4521_);
  auto x2 = std::get<1>(n4521_);
  auto x3 = std::get<2>(n4521_);
  auto x4 = std::get<3>(n4521_);
  #line 113 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3), box(x4));
}

#line 119 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5>
#line 119 "src/expression/box.birch"
inline decltype(auto) box(const std::tuple<Value1, Value2, Value3, Value4, Value5>& x) {
  auto n4522_ = x;
  auto x1 = std::get<0>(n4522_);
  auto x2 = std::get<1>(n4522_);
  auto x3 = std::get<2>(n4522_);
  auto x4 = std::get<3>(n4522_);
  auto x5 = std::get<4>(n4522_);
  #line 121 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3), box(x4), box(x5));
}

#line 127 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6>
#line 127 "src/expression/box.birch"
inline decltype(auto) box(const std::tuple<Value1, Value2, Value3, Value4, Value5, Value6>& x) {
  auto n4523_ = x;
  auto x1 = std::get<0>(n4523_);
  auto x2 = std::get<1>(n4523_);
  auto x3 = std::get<2>(n4523_);
  auto x4 = std::get<3>(n4523_);
  auto x5 = std::get<4>(n4523_);
  auto x6 = std::get<5>(n4523_);
  #line 129 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3), box(x4), box(x5), box(x6));
}

#line 135 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6,class Value7>
#line 135 "src/expression/box.birch"
inline decltype(auto) box(const std::tuple<Value1, Value2, Value3, Value4, Value5, Value6, Value7>& x) {
  auto n4524_ = x;
  auto x1 = std::get<0>(n4524_);
  auto x2 = std::get<1>(n4524_);
  auto x3 = std::get<2>(n4524_);
  auto x4 = std::get<3>(n4524_);
  auto x5 = std::get<4>(n4524_);
  auto x6 = std::get<5>(n4524_);
  auto x7 = std::get<6>(n4524_);
  #line 137 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3), box(x4), box(x5), box(x6), box(x7));
}

#line 143 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6,class Value7,class Value8>
#line 143 "src/expression/box.birch"
inline decltype(auto) box(const std::tuple<Value1, Value2, Value3, Value4, Value5, Value6, Value7, Value8>& x) {
  auto n4525_ = x;
  auto x1 = std::get<0>(n4525_);
  auto x2 = std::get<1>(n4525_);
  auto x3 = std::get<2>(n4525_);
  auto x4 = std::get<3>(n4525_);
  auto x5 = std::get<4>(n4525_);
  auto x6 = std::get<5>(n4525_);
  auto x7 = std::get<6>(n4525_);
  auto x8 = std::get<7>(n4525_);
  #line 145 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3), box(x4), box(x5), box(x6), box(x7), box(x8));
}

#line 151 "src/expression/box.birch"
template<class Value1,class Value2>
#line 151 "src/expression/box.birch"
inline decltype(auto) box(const Value1& x1, const Value2& x2) {
  #line 152 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2));
}

#line 158 "src/expression/box.birch"
template<class Value1,class Value2,class Value3>
#line 158 "src/expression/box.birch"
inline decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3) {
  #line 159 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3));
}

#line 165 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4>
#line 165 "src/expression/box.birch"
inline decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3, const Value4& x4) {
  #line 166 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3), box(x4));
}

#line 172 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5>
#line 172 "src/expression/box.birch"
inline decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3, const Value4& x4, const Value5& x5) {
  #line 173 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3), box(x4), box(x5));
}

#line 179 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6>
#line 179 "src/expression/box.birch"
inline decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3, const Value4& x4, const Value5& x5, const Value6& x6) {
  #line 180 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3), box(x4), box(x5), box(x6));
}

#line 186 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6,class Value7>
#line 186 "src/expression/box.birch"
inline decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3, const Value4& x4, const Value5& x5, const Value6& x6, const Value7& x7) {
  #line 187 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3), box(x4), box(x5), box(x6), box(x7));
}

#line 193 "src/expression/box.birch"
template<class Value1,class Value2,class Value3,class Value4,class Value5,class Value6,class Value7,class Value8>
#line 193 "src/expression/box.birch"
inline decltype(auto) box(const Value1& x1, const Value2& x2, const Value3& x3, const Value4& x4, const Value5& x5, const Value6& x6, const Value7& x7, const Value8& x8) {
  #line 194 "src/expression/box.birch"
  return std::make_tuple(box(x1), box(x2), box(x3), box(x4), box(x5), box(x6), box(x7), box(x8));
}

#line 62 "src/form/MatrixElement.birch"
template<class Value,class Form,class Middle,class Right>
#line 62 "src/form/MatrixElement.birch"
inline decltype(auto) element(const MatrixSplitExpression<Value, Form>& x, const Middle& i, const Right& j) {
  #line 63 "src/form/MatrixElement.birch"
  return construct<MatrixElement<MatrixSplitExpression<Value, Form>, Middle, Right>>(x, i, j);
}

#line 69 "src/form/MatrixElement.birch"
template<class Left,class Middle,class Right>
#line 69 "src/form/MatrixElement.birch"
inline decltype(auto) element(const Left& x, const Middle& i, const Right& j) {
  #line 70 "src/form/MatrixElement.birch"
  return element(matrix_split(x), i, j);
}

#line 58 "src/form/MatrixJoinForm.birch"
template<class Value>
#line 58 "src/form/MatrixJoinForm.birch"
inline decltype(auto) join(const libbirch::DefaultArray<Expression<Value>,2>& x) {
  #line 59 "src/form/MatrixJoinForm.birch"
  return box(construct<MatrixJoinForm<Value>>(x));
}

#line 54 "src/form/VectorElement.birch"
template<class Value,class Form,class Right>
#line 54 "src/form/VectorElement.birch"
inline decltype(auto) element(const VectorSplitExpression<Value, Form>& x, const Right& i) {
  #line 55 "src/form/VectorElement.birch"
  return construct<VectorElement<VectorSplitExpression<Value, Form>, Right>>(x, i);
}

#line 61 "src/form/VectorElement.birch"
template<class Left,class Right>
#line 61 "src/form/VectorElement.birch"
inline decltype(auto) element(const Left& x, const Right& i) {
  #line 62 "src/form/VectorElement.birch"
  return element(vector_split(x), i);
}

#line 58 "src/form/VectorJoinForm.birch"
template<class Value>
#line 58 "src/form/VectorJoinForm.birch"
inline decltype(auto) join(const libbirch::DefaultArray<Expression<Value>,1>& x) {
  #line 59 "src/form/VectorJoinForm.birch"
  return box(construct<VectorJoinForm<Value>>(x));
}

#line 66 "src/math/eigen.birch"
template<class Type>
#line 66 "src/math/eigen.birch"
inline libbirch::DefaultArray<Type,1> hadamard(const libbirch::DefaultArray<Type,1>& x, const libbirch::DefaultArray<Type,1>& y) {
  #line 67 "src/math/eigen.birch"

  return x.toEigen().cwiseProduct(y.toEigen());
  }

#line 75 "src/math/eigen.birch"
template<class Type>
#line 75 "src/math/eigen.birch"
inline libbirch::DefaultArray<Type,2> hadamard(const libbirch::DefaultArray<Type,2>& X, const libbirch::DefaultArray<Type,2>& Y) {
  #line 76 "src/math/eigen.birch"

  return X.toEigen().cwiseProduct(Y.toEigen());
  }

#line 8 "src/primitive/iterator.birch"
template<class Type,class Lambda>
#line 8 "src/primitive/iterator.birch"
inline Type reduce(const Iterator<Type>& iter, const Type& init, const Lambda& op) {
  auto result = init;
  #line 11 "src/primitive/iterator.birch"
  while (iter->hasNext()) {
    #line 12 "src/primitive/iterator.birch"
    result = op(result, iter->next());
  }
  #line 14 "src/primitive/iterator.birch"
  return result;
}

#line 20 "src/primitive/iterator.birch"
template<class Type>
#line 20 "src/primitive/iterator.birch"
inline Type max(const Iterator<Type>& iter) {
  #line 21 "src/primitive/iterator.birch"
  assert(iter->hasNext());
  #line 22 "src/primitive/iterator.birch"
  return reduce(iter, iter->next(), [=](const Real& x, const Real& y) -> Real {
    #line 23 "src/primitive/iterator.birch"
    return max(x, y);
  });
}

#line 30 "src/primitive/iterator.birch"
template<class Type>
#line 30 "src/primitive/iterator.birch"
inline Type min(const Iterator<Type>& iter) {
  #line 31 "src/primitive/iterator.birch"
  assert(iter->hasNext());
  #line 32 "src/primitive/iterator.birch"
  return reduce(iter, iter->next(), [=](const Type& x, const Type& y) -> Type {
    #line 33 "src/primitive/iterator.birch"
    return min(x, y);
  });
}

#line 9 "src/primitive/primitive.birch"
template<class Type,class Lambda>
#line 9 "src/primitive/primitive.birch"
inline void for_each(const libbirch::DefaultArray<Type,1>& x, const Lambda& f) {
  #line 10 "src/primitive/primitive.birch"
  for (auto i = Integer(1); i <= length(x); ++i) {
    #line 11 "src/primitive/primitive.birch"
    f(x(i));
  }
}

#line 23 "src/primitive/primitive.birch"
template<class Type,class Lambda>
#line 23 "src/primitive/primitive.birch"
inline void for_each(const libbirch::DefaultArray<Type,2>& X, const Lambda& f) {
  #line 24 "src/primitive/primitive.birch"
  for (auto i = Integer(1); i <= rows(X); ++i) {
    #line 25 "src/primitive/primitive.birch"
    for (auto j = Integer(1); j <= columns(X); ++j) {
      #line 26 "src/primitive/primitive.birch"
      f(X(i, j));
    }
  }
}

#line 38 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Lambda>
#line 38 "src/primitive/primitive.birch"
inline void for_each(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const Lambda& f) {
  #line 39 "src/primitive/primitive.birch"
  assert(length(x) == length(y));
  #line 40 "src/primitive/primitive.birch"
  for (auto i = Integer(1); i <= length(x); ++i) {
    #line 41 "src/primitive/primitive.birch"
    f(x(i), y(i));
  }
}

#line 52 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Lambda>
#line 52 "src/primitive/primitive.birch"
inline void for_each(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const Lambda& f) {
  #line 54 "src/primitive/primitive.birch"
  assert(rows(X) == rows(Y));
  #line 55 "src/primitive/primitive.birch"
  assert(columns(X) == columns(Y));
  #line 56 "src/primitive/primitive.birch"
  for (auto i = Integer(1); i <= rows(X); ++i) {
    #line 57 "src/primitive/primitive.birch"
    for (auto j = Integer(1); j <= columns(X); ++j) {
      #line 58 "src/primitive/primitive.birch"
      f(X(i, j), Y(i, j));
    }
  }
}

#line 71 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Lambda>
#line 71 "src/primitive/primitive.birch"
inline void for_each(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const libbirch::DefaultArray<Input3,1>& z, const Lambda& f) {
  #line 73 "src/primitive/primitive.birch"
  assert(length(x) == length(y));
  #line 74 "src/primitive/primitive.birch"
  assert(length(y) == length(z));
  #line 75 "src/primitive/primitive.birch"
  for (auto i = Integer(1); i <= length(x); ++i) {
    #line 76 "src/primitive/primitive.birch"
    f(x(i), y(i), z(i));
  }
}

#line 88 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Lambda>
#line 88 "src/primitive/primitive.birch"
inline void for_each(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const libbirch::DefaultArray<Input3,2>& Z, const Lambda& f) {
  #line 90 "src/primitive/primitive.birch"
  assert(rows(X) == rows(Y));
  #line 91 "src/primitive/primitive.birch"
  assert(rows(Y) == rows(Z));
  #line 92 "src/primitive/primitive.birch"
  assert(columns(X) == columns(Y));
  #line 93 "src/primitive/primitive.birch"
  assert(columns(Y) == columns(Z));
  #line 94 "src/primitive/primitive.birch"
  for (auto i = Integer(1); i <= rows(X); ++i) {
    #line 95 "src/primitive/primitive.birch"
    for (auto j = Integer(1); j <= columns(X); ++j) {
      #line 96 "src/primitive/primitive.birch"
      f(X(i, j), Y(i, j), Z(i, j));
    }
  }
}

#line 107 "src/primitive/primitive.birch"
template<class Input,class Lambda>
#line 107 "src/primitive/primitive.birch"
inline decltype(auto) transform(const libbirch::DefaultArray<Input,1>& x, const Lambda& f) {
  #line 108 "src/primitive/primitive.birch"
  return vector_lambda([=](const Integer& i) {
    #line 108 "src/primitive/primitive.birch"
    return f(x(i));
  }, length(x));
}

#line 117 "src/primitive/primitive.birch"
template<class Input,class Lambda>
#line 117 "src/primitive/primitive.birch"
inline decltype(auto) transform(const libbirch::DefaultArray<Input,2>& X, const Lambda& f) {
  #line 118 "src/primitive/primitive.birch"
  return matrix_lambda([=](const Integer& i, const Integer& j) {
    #line 118 "src/primitive/primitive.birch"
    return f(X(i, j));
  }, rows(X), columns(X));
}

#line 129 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Lambda>
#line 129 "src/primitive/primitive.birch"
inline decltype(auto) transform(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const Lambda& f) {
  #line 131 "src/primitive/primitive.birch"
  assert(length(x) == length(y));
  #line 132 "src/primitive/primitive.birch"
  return vector_lambda([=](const Integer& i) {
    #line 132 "src/primitive/primitive.birch"
    return f(x(i), y(i));
  }, length(x));
}

#line 142 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Lambda>
#line 142 "src/primitive/primitive.birch"
inline decltype(auto) transform(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const Lambda& f) {
  #line 144 "src/primitive/primitive.birch"
  assert(rows(X) == rows(Y));
  #line 145 "src/primitive/primitive.birch"
  assert(columns(X) == columns(Y));
  #line 146 "src/primitive/primitive.birch"
  return matrix_lambda([=](const Integer& i, const Integer& j) {
    #line 146 "src/primitive/primitive.birch"
    return f(X(i, j), Y(i, j));
  }, rows(X), columns(X));
}

#line 158 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Lambda>
#line 158 "src/primitive/primitive.birch"
inline decltype(auto) transform(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const libbirch::DefaultArray<Input3,1>& z, const Lambda& f) {
  #line 160 "src/primitive/primitive.birch"
  assert(length(x) == length(y));
  #line 161 "src/primitive/primitive.birch"
  assert(length(x) == length(z));
  #line 162 "src/primitive/primitive.birch"
  return vector_lambda([=](const Integer& i) {
    #line 162 "src/primitive/primitive.birch"
    return f(x(i), y(i), z(i));
  }, length(x));
}

#line 174 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Lambda>
#line 174 "src/primitive/primitive.birch"
inline decltype(auto) transform(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const libbirch::DefaultArray<Input3,2>& Z, const Lambda& f) {
  #line 176 "src/primitive/primitive.birch"
  assert(rows(X) == rows(Y));
  #line 177 "src/primitive/primitive.birch"
  assert(rows(X) == rows(Z));
  #line 178 "src/primitive/primitive.birch"
  assert(columns(X) == columns(Y));
  #line 179 "src/primitive/primitive.birch"
  assert(columns(Y) == columns(Z));
  #line 180 "src/primitive/primitive.birch"
  return matrix_lambda([=](const Integer& i, const Integer& j) {
    #line 181 "src/primitive/primitive.birch"
    return f(X(i, j), Y(i, j), Z(i, j));
  }, rows(X), columns(X));
}

#line 192 "src/primitive/primitive.birch"
template<class Type,class Output,class Lambda>
#line 192 "src/primitive/primitive.birch"
inline Output reduce(const libbirch::DefaultArray<Type,1>& x, const Output& init, const Lambda& op) {
  #line 194 "src/primitive/primitive.birch"

  #ifdef __cpp_lib_parallel_algorithm
  return std::reduce(x.begin(), x.end(), init, op);
  #else
  return std::accumulate(x.begin(), x.end(), init, [&](auto x, auto y) {
        return op(x, y);
      });
  #endif
  }

#line 212 "src/primitive/primitive.birch"
template<class Type,class Output,class Lambda>
#line 212 "src/primitive/primitive.birch"
inline Output reduce(const libbirch::DefaultArray<Type,2>& X, const Output& init, const Lambda& op) {
  Type result;
  #line 215 "src/primitive/primitive.birch"

  #ifdef __cpp_lib_parallel_algorithm
  return std::reduce(X.begin(), X.end(), init, op);
  #else
  return std::accumulate(X.begin(), X.end(), init, [&](auto x, auto y) {
        return op(x, y);
      });
  #endif
  }

#line 234 "src/primitive/primitive.birch"
template<class Input,class Output,class Lambda1,class Lambda2>
#line 234 "src/primitive/primitive.birch"
inline Output transform_reduce(const libbirch::DefaultArray<Input,1>& x, const Output& init, const Lambda1& op1, const Lambda2& op2) {
  auto r = init;
  #line 237 "src/primitive/primitive.birch"
  for (auto n = Integer(1); n <= length(x); ++n) {
    #line 238 "src/primitive/primitive.birch"
    r = op1(r, op2(x(n)));
  }
  #line 240 "src/primitive/primitive.birch"
  return r;
}

#line 251 "src/primitive/primitive.birch"
template<class Input,class Output,class Lambda1,class Lambda2>
#line 251 "src/primitive/primitive.birch"
inline Output transform_reduce(const libbirch::DefaultArray<Input,2>& X, const Output& init, const Lambda1& op1, const Lambda2& op2) {
  auto r = init;
  #line 254 "src/primitive/primitive.birch"
  for (auto i = Integer(1); i <= rows(X); ++i) {
    #line 255 "src/primitive/primitive.birch"
    for (auto j = Integer(1); j <= columns(X); ++j) {
      #line 256 "src/primitive/primitive.birch"
      r = op1(r, op2(X(i, j)));
    }
  }
  #line 259 "src/primitive/primitive.birch"
  return r;
}

#line 271 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Output,class Lambda1,class Lambda2>
#line 271 "src/primitive/primitive.birch"
inline Output transform_reduce(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const Output& init, const Lambda1& op1, const Lambda2& op2) {
  #line 273 "src/primitive/primitive.birch"
  assert(length(x) == length(y));
  auto r = init;
  #line 275 "src/primitive/primitive.birch"
  for (auto n = Integer(1); n <= length(x); ++n) {
    #line 276 "src/primitive/primitive.birch"
    r = op1(r, op2(x(n), y(n)));
  }
  #line 278 "src/primitive/primitive.birch"
  return r;
}

#line 290 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Output,class Lambda1,class Lambda2>
#line 290 "src/primitive/primitive.birch"
inline Output transform_reduce(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const Output& init, const Lambda1& op1, const Lambda2& op2) {
  #line 292 "src/primitive/primitive.birch"
  assert(rows(X) == rows(Y));
  #line 293 "src/primitive/primitive.birch"
  assert(columns(X) == columns(Y));
  auto r = init;
  #line 295 "src/primitive/primitive.birch"
  for (auto i = Integer(1); i <= rows(X); ++i) {
    #line 296 "src/primitive/primitive.birch"
    for (auto j = Integer(1); j <= columns(X); ++j) {
      #line 297 "src/primitive/primitive.birch"
      r = op1(r, op2(X(i, j), Y(i, j)));
    }
  }
  #line 300 "src/primitive/primitive.birch"
  return r;
}

#line 313 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Output,class Lambda1,class Lambda2>
#line 313 "src/primitive/primitive.birch"
inline Output transform_reduce(const libbirch::DefaultArray<Input1,1>& x, const libbirch::DefaultArray<Input2,1>& y, const libbirch::DefaultArray<Input3,1>& z, const Output& init, const Lambda1& op1, const Lambda2& op2) {
  #line 316 "src/primitive/primitive.birch"
  assert(length(x) == length(y));
  #line 317 "src/primitive/primitive.birch"
  assert(length(x) == length(z));
  auto r = init;
  #line 319 "src/primitive/primitive.birch"
  for (auto n = Integer(1); n <= length(x); ++n) {
    #line 320 "src/primitive/primitive.birch"
    r = op1(r, op2(x(n), y(n), z(n)));
  }
  #line 322 "src/primitive/primitive.birch"
  return r;
}

#line 335 "src/primitive/primitive.birch"
template<class Input1,class Input2,class Input3,class Output,class Lambda1,class Lambda2>
#line 335 "src/primitive/primitive.birch"
inline Output transform_reduce(const libbirch::DefaultArray<Input1,2>& X, const libbirch::DefaultArray<Input2,2>& Y, const libbirch::DefaultArray<Input3,2>& Z, const Output& init, const Lambda1& op1, const Lambda2& op2) {
  #line 338 "src/primitive/primitive.birch"
  assert(rows(X) == rows(Y));
  #line 339 "src/primitive/primitive.birch"
  assert(columns(X) == columns(Y));
  #line 340 "src/primitive/primitive.birch"
  assert(rows(X) == rows(Z));
  #line 341 "src/primitive/primitive.birch"
  assert(columns(X) == columns(Z));
  auto r = init;
  #line 343 "src/primitive/primitive.birch"
  for (auto i = Integer(1); i <= rows(X); ++i) {
    #line 344 "src/primitive/primitive.birch"
    for (auto j = Integer(1); j <= columns(X); ++j) {
      #line 345 "src/primitive/primitive.birch"
      r = op1(r, op2(X(i, j), Y(i, j), Z(i, j)));
    }
  }
  #line 348 "src/primitive/primitive.birch"
  return r;
}

#line 357 "src/primitive/primitive.birch"
template<class Type,class Lambda>
#line 357 "src/primitive/primitive.birch"
inline libbirch::DefaultArray<Type,1> inclusive_scan(const libbirch::DefaultArray<Type,1>& x, const Type& init, const Lambda& op) {
  libbirch::DefaultArray<Type,1> y(libbirch::make_shape(length(x)));
  #line 360 "src/primitive/primitive.birch"
  if (length(x) > Integer(0)) {
    #line 361 "src/primitive/primitive.birch"

    #ifdef __cpp_lib_parallel_algorithm
    std::inclusive_scan(x.begin(), x.end(), y.begin(), init, op);
    #else
        #line 366 "src/primitive/primitive.birch"
    y(Integer(1)) = op(init, x(Integer(1)));
    #line 367 "src/primitive/primitive.birch"
    for (auto i = Integer(2); i <= length(x); ++i) {
      #line 368 "src/primitive/primitive.birch"
      y(i) = op(y(i - Integer(1)), x(i - Integer(1)));
    }
    #line 370 "src/primitive/primitive.birch"

    #endif
      }
  #line 374 "src/primitive/primitive.birch"
  return y;
}

#line 380 "src/primitive/primitive.birch"
template<class Type>
#line 380 "src/primitive/primitive.birch"
inline libbirch::DefaultArray<Type,1> inclusive_scan(const libbirch::DefaultArray<Type,1>& x) {
  #line 381 "src/primitive/primitive.birch"
  return inclusive_scan(x, 0.0, [=](const Type& x, const Type& y) -> Type {
    #line 381 "src/primitive/primitive.birch"
    return x + y;
  });
}

#line 391 "src/primitive/primitive.birch"
template<class Type,class Lambda>
#line 391 "src/primitive/primitive.birch"
inline libbirch::DefaultArray<Type,1> exclusive_scan(const libbirch::DefaultArray<Type,1>& x, const Type& init, const Lambda& op) {
  libbirch::DefaultArray<Type,1> y(libbirch::make_shape(length(x)));
  #line 394 "src/primitive/primitive.birch"
  if (length(x) > Integer(0)) {
    #line 395 "src/primitive/primitive.birch"

    #ifdef __cpp_lib_parallel_algorithm
    std::exclusive_scan(x.begin(), x.end(), y.begin(), init, op);
    #else
        #line 400 "src/primitive/primitive.birch"
    y(Integer(1)) = init;
    #line 401 "src/primitive/primitive.birch"
    for (auto i = Integer(2); i <= length(x); ++i) {
      #line 402 "src/primitive/primitive.birch"
      y(i) = op(y(i - Integer(1)), x(i - Integer(1)));
    }
    #line 404 "src/primitive/primitive.birch"

    #endif
      }
  #line 408 "src/primitive/primitive.birch"
  return y;
}

#line 414 "src/primitive/primitive.birch"
template<class Type>
#line 414 "src/primitive/primitive.birch"
inline libbirch::DefaultArray<Type,1> exclusive_scan(const libbirch::DefaultArray<Type,1>& x) {
  #line 415 "src/primitive/primitive.birch"
  return exclusive_scan(x, 0.0, [=](const Type& x, const Type& y) -> Type {
    #line 415 "src/primitive/primitive.birch"
    return x + y;
  });
}

#line 424 "src/primitive/primitive.birch"
template<class Type,class Lambda>
#line 424 "src/primitive/primitive.birch"
inline libbirch::DefaultArray<Type,1> adjacent_difference(const libbirch::DefaultArray<Type,1>& x, const Lambda& op) {
  libbirch::DefaultArray<Type,1> y(libbirch::make_shape(length(x)));
  #line 426 "src/primitive/primitive.birch"

  std::adjacent_difference(x.begin(), x.end(), y.begin(),
      [&](auto x, auto y) { return op(x, y); });
    #line 430 "src/primitive/primitive.birch"
  return y;
}

#line 436 "src/primitive/primitive.birch"
template<class Type>
#line 436 "src/primitive/primitive.birch"
inline Boolean is_sorted(const libbirch::DefaultArray<Type,1>& x) {
  #line 437 "src/primitive/primitive.birch"

  return std::is_sorted(x.begin(), x.end());
  }

#line 449 "src/primitive/primitive.birch"
template<class Type>
#line 449 "src/primitive/primitive.birch"
inline libbirch::DefaultArray<Type,1> sort(const libbirch::DefaultArray<Type,1>& x) {
  auto y = x;
  #line 451 "src/primitive/primitive.birch"

  std::sort(y.begin(), y.end());
    #line 454 "src/primitive/primitive.birch"
  return y;
}

#line 465 "src/primitive/primitive.birch"
template<class Type>
#line 465 "src/primitive/primitive.birch"
inline libbirch::DefaultArray<Integer,1> sort_index(const libbirch::DefaultArray<Type,1>& x) {
  auto a = iota(Integer(1), length(x));
  #line 467 "src/primitive/primitive.birch"

  std::sort(a.begin(), a.end(), [=](Integer i, Integer j) {
      return x(i - 1) < x(j - 1);
    });
    #line 472 "src/primitive/primitive.birch"
  return a;
}

#line 483 "src/primitive/primitive.birch"
template<class Type>
#line 483 "src/primitive/primitive.birch"
inline libbirch::DefaultArray<Type,1> gather(const libbirch::DefaultArray<Integer,1>& a, const libbirch::DefaultArray<Type,1>& x) {
  #line 484 "src/primitive/primitive.birch"
  return vector_lambda([=](const Integer& i) -> Type {
    #line 484 "src/primitive/primitive.birch"
    return x(a(i));
  }, length(a));
}

#line 497 "src/primitive/primitive.birch"
template<class Type>
#line 497 "src/primitive/primitive.birch"
inline libbirch::DefaultArray<Type,1> scatter(const libbirch::DefaultArray<Integer,1>& a, const libbirch::DefaultArray<Type,1>& x) {
  auto N = length(a);
  libbirch::DefaultArray<Type,1> y(libbirch::make_shape(N));
  #line 500 "src/primitive/primitive.birch"
  for (auto n = Integer(1); n <= N; ++n) {
    #line 501 "src/primitive/primitive.birch"
    y(a(n)) = x(n);
  }
  #line 503 "src/primitive/primitive.birch"
  return y;
}

#line 512 "src/primitive/primitive.birch"
template<class Type>
#line 512 "src/primitive/primitive.birch"
inline libbirch::DefaultArray<Real,1> iota(const Type& x, const Integer& length) {
  #line 513 "src/primitive/primitive.birch"
  return vector_lambda([=](const Integer& i) -> Type {
    #line 513 "src/primitive/primitive.birch"
    return x + (i - Integer(1));
  }, length);
}

#line 519 "src/primitive/primitive.birch"
template<class Type>
#line 519 "src/primitive/primitive.birch"
inline Type max(const libbirch::DefaultArray<Type,1>& x) {
  #line 520 "src/primitive/primitive.birch"
  assert(length(x) > Integer(0));
  #line 521 "src/primitive/primitive.birch"
  if (length(x) == Integer(1)) {
    #line 522 "src/primitive/primitive.birch"
    return x(Integer(1));
  } else {
    #line 524 "src/primitive/primitive.birch"
    return reduce(x(libbirch::make_range(Integer(2), length(x))), x(Integer(1)), [=](const Type& x, const Type& y) {
      #line 525 "src/primitive/primitive.birch"
      return max(x, y);
    });
  }
}

#line 533 "src/primitive/primitive.birch"
template<class Type>
#line 533 "src/primitive/primitive.birch"
inline Type min(const libbirch::DefaultArray<Type,1>& x) {
  #line 534 "src/primitive/primitive.birch"
  assert(length(x) > Integer(0));
  #line 535 "src/primitive/primitive.birch"
  if (length(x) == Integer(1)) {
    #line 536 "src/primitive/primitive.birch"
    return x(Integer(1));
  } else {
    #line 538 "src/primitive/primitive.birch"
    return reduce(x(libbirch::make_range(Integer(2), length(x))), x(Integer(1)), [=](const Type& x, const Type& y) -> Type {
      #line 539 "src/primitive/primitive.birch"
      return min(x, y);
    });
  }
}

#line 8 "src/utility/construct.birch"
template<class Type>
#line 8 "src/utility/construct.birch"
inline Type construct() {
  #line 9 "src/utility/construct.birch"

  return Type();
  }

#line 23 "src/utility/construct.birch"
template<class Type,class Arg1>
#line 23 "src/utility/construct.birch"
inline Type construct(const Arg1& arg1) {
  #line 24 "src/utility/construct.birch"

  return Type(std::in_place, arg1);
  }

#line 38 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2>
#line 38 "src/utility/construct.birch"
inline Type construct(const Arg1& arg1, const Arg2& arg2) {
  #line 39 "src/utility/construct.birch"

  return Type(std::in_place, arg1, arg2);
  }

#line 53 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3>
#line 53 "src/utility/construct.birch"
inline Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3) {
  #line 55 "src/utility/construct.birch"

  return Type(std::in_place, arg1, arg2, arg3);
  }

#line 69 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3,class Arg4>
#line 69 "src/utility/construct.birch"
inline Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3, const Arg4& arg4) {
  #line 71 "src/utility/construct.birch"

  return Type(std::in_place, arg1, arg2, arg3, arg4);
  }

#line 85 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 85 "src/utility/construct.birch"
inline Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3, const Arg4& arg4, const Arg5& arg5) {
  #line 87 "src/utility/construct.birch"

  return Type(std::in_place, arg1, arg2, arg3, arg4, arg5);
  }

#line 101 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 101 "src/utility/construct.birch"
inline Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3, const Arg4& arg4, const Arg5& arg5, const Arg6& arg6) {
  #line 103 "src/utility/construct.birch"

  return Type(std::in_place, arg1, arg2, arg3, arg4, arg5, arg6);
  }

#line 117 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 117 "src/utility/construct.birch"
inline Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3, const Arg4& arg4, const Arg5& arg5, const Arg6& arg6, const Arg7& arg7) {
  #line 120 "src/utility/construct.birch"

  return Type(std::in_place, arg1, arg2, arg3, arg4, arg5, arg6, arg7);
  }

#line 134 "src/utility/construct.birch"
template<class Type,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7,class Arg8>
#line 134 "src/utility/construct.birch"
inline Type construct(const Arg1& arg1, const Arg2& arg2, const Arg3& arg3, const Arg4& arg4, const Arg5& arg5, const Arg6& arg6, const Arg7& arg7, const Arg8& arg8) {
  #line 137 "src/utility/construct.birch"

  return Type(std::in_place, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
  }

#line 4 "src/utility/copy.birch"
template<class Type>
#line 4 "src/utility/copy.birch"
inline void bridge(const Type& o) {
  #line 5 "src/utility/copy.birch"

  o.bridge();
  }

#line 14 "src/utility/copy.birch"
template<class Type>
#line 14 "src/utility/copy.birch"
inline Type copy(const Type& o) {
  #line 15 "src/utility/copy.birch"

  return o.copy();
  }

#line 27 "src/utility/copy.birch"
template<class Type>
#line 27 "src/utility/copy.birch"
inline libbirch::DefaultArray<Type,1> copy(const Type& o, const Integer& length) {
  #line 28 "src/utility/copy.birch"
  return vector_lambda([=](const Integer& i) {
    #line 28 "src/utility/copy.birch"
    return copy(o);
  }, length);
}

#line 12 "src/utility/make.birch"
template<class Type>
#line 12 "src/utility/make.birch"
inline std::optional<Type> make() {
  #line 13 "src/utility/make.birch"

  return libbirch::make_optional<Type>();
  }

#line 29 "src/utility/make.birch"
template<class Type>
#line 29 "src/utility/make.birch"
inline std::optional<Type> make(const String& name) {
  std::optional<Object> result;
  String symbol = String("make_") + name + String("_");
  #line 32 "src/utility/make.birch"

  using make_t = Object_*();
  void* addr = dlsym(RTLD_DEFAULT, symbol.c_str());
  if (addr) {
    result = Object(reinterpret_cast<make_t*>(addr)());
  }
    #line 39 "src/utility/make.birch"
  return libbirch::cast<Type>(result);
}

#line 86 "src/utility/make.birch"
template<class Type>
#line 86 "src/utility/make.birch"
inline std::optional<Type> make(const Buffer& buffer) {
  std::optional<Type> result;
  auto className = buffer->get<String>(String("class"));
  #line 89 "src/utility/make.birch"
  if (className.has_value()) {
    #line 90 "src/utility/make.birch"
    result = make<Type>(className.value());
  } else {
    #line 92 "src/utility/make.birch"
    result = make<Type>();
  }
  #line 94 "src/utility/make.birch"
  if (result.has_value()) {
    #line 95 "src/utility/make.birch"
    result.value()->read(buffer);
  }
  #line 97 "src/utility/make.birch"
  return result;
}

#line 61 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 61 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,2>> MatrixGaussian(const Arg1& M, const Arg2& U, const Arg3& V) {
  #line 63 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  return construct<MatrixGaussianDistribution<Arg1, Arg2, Arg3>>(M, U, V);
}

#line 97 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 97 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
inline decltype(auto) logpdf_matrix_gaussian(const Arg1& X, const Arg2& M, const Arg3& U, const Arg4& V) {
  auto n = rows(X);
  auto p = columns(X);
  #line 101 "src/distribution/matrix_gaussian/MatrixGaussianDistribution.birch"
  return -(0.5) * (trace(cholsolve(V, transpose(X - M)) * cholsolve(U, X - M)) + n * p * log(2.0 * \u03c0) + n * lcholdet(V) + p * lcholdet(U));
}

#line 81 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 81 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
inline decltype(auto) LinearMatrixNormalInverseWishartMatrixGaussian(const Arg1& A, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& C, const Arg7& U) {
  #line 84 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return construct<LinearMatrixNormalInverseWishartMatrixGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>>(A, N, \u039b, \u03a8, k, C, U);
}

#line 91 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 91 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,2>> MatrixGaussian(const Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,2>>>, Arg2>& M, const Arg3& U, const Random<libbirch::DefaultArray<Real,2>>& V) {
  #line 94 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  M->l->r->prune();
  #line 95 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  if (M->l->r->hasMatrixNormalInverseWishart() && M->l->r->isGrandChildOf(V)) {
    auto n4526_ = M->l->r->getMatrixNormalInverseWishart();
    auto N = std::get<0>(n4526_);
    auto \u039b = std::get<1>(n4526_);
    auto \u03b1 = std::get<2>(n4526_);
    auto \u03b2 = std::get<3>(n4526_);
    auto p = LinearMatrixNormalInverseWishartMatrixGaussian(M->l->l, N, \u039b, \u03b1, \u03b2, M->r, U);
    #line 99 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    M->l->r->graft(p, Integer(1));
    #line 100 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    V->graftReplace(p, Integer(2));
    #line 101 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    M->l->r->join(V);
    #line 102 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    return p;
  } else {
    #line 104 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    V->prune();
    #line 105 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    if (V->hasInverseWishart()) {
      auto n4527_ = V->getInverseWishart();
      auto \u03a8 = std::get<0>(n4527_);
      auto k = std::get<1>(n4527_);
      auto \u039b = cholinv(U);
      auto p = MatrixNormalInverseWishart(\u039b * M, \u039b, \u03a8, k);
      #line 109 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
      V->graft(p);
      #line 110 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
      return p;
    } else {
      #line 112 "src/distribution/matrix_normal_inverse_wishart/LinearMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
      return construct<MatrixGaussianDistribution<Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,2>>>, Arg2>, Arg3, Random<libbirch::DefaultArray<Real,2>>>>(M, U, V);
    }
  }
}

#line 77 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 77 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
inline decltype(auto) MatrixNormalInverseWishart(const Arg1& N, const Arg2& \u039b, const Arg3& \u03a8, const Arg4& k) {
  #line 79 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return construct<MatrixNormalInverseWishartDistribution<Arg1, Arg2, Arg3, Arg4>>(N, \u039b, \u03a8, k);
}

#line 86 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 86 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,2>> MatrixGaussian(const Arg1& M, const Arg2& U, const Random<libbirch::DefaultArray<Real,2>>& V) {
  #line 88 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  V->prune();
  #line 89 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  if (V->hasInverseWishart()) {
    auto n4528_ = V->getInverseWishart();
    auto \u03a8 = std::get<0>(n4528_);
    auto k = std::get<1>(n4528_);
    auto \u039b = cholinv(U);
    auto p = MatrixNormalInverseWishart(\u039b * M, \u039b, \u03a8, k);
    #line 93 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
    V->graft(p);
    #line 94 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
    return p;
  } else {
    #line 96 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
    return construct<MatrixGaussianDistribution<Arg1, Arg2, Random<libbirch::DefaultArray<Real,2>>>>(M, U, V);
  }
}

#line 126 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 126 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
inline decltype(auto) logpdf_matrix_normal_inverse_wishart(const Arg1& X, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k) {
  auto \u03a3 = cholinv(\u039b);
  auto M = \u03a3 * N;
  #line 130 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return logpdf_matrix_t(X, k, M, \u03a3, \u03a8);
}

#line 144 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 144 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
inline decltype(auto) update_matrix_normal_inverse_wishart(const Arg1& X, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k) {
  auto n = rows(X);
  auto M = cholsolve(\u039b, N);
  auto \u03a8_prime_ = \u03a8 + inner(X - M, \u039b * (X - M));
  auto k_prime_ = k + n;
  #line 150 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartDistribution.birch"
  return InverseWishart(\u03a8_prime_, k_prime_);
}

#line 66 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 66 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
inline decltype(auto) MatrixNormalInverseWishartMatrixGaussian(const Arg1& N, const Arg2& \u039b, const Arg3& \u03a8, const Arg4& k, const Arg5& U) {
  #line 68 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return construct<MatrixNormalInverseWishartMatrixGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5>>(N, \u039b, \u03a8, k, U);
}

#line 75 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg>
#line 75 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,2>> MatrixGaussian(const Random<libbirch::DefaultArray<Real,2>>& M, const Arg& U, const Random<libbirch::DefaultArray<Real,2>>& V) {
  #line 77 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  M->prune();
  #line 78 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  if (M->hasMatrixNormalInverseWishart() && M->isGrandChildOf(V)) {
    auto n4529_ = M->getMatrixNormalInverseWishart();
    auto N = std::get<0>(n4529_);
    auto \u039b = std::get<1>(n4529_);
    auto \u03a8 = std::get<2>(n4529_);
    auto k = std::get<3>(n4529_);
    auto p = MatrixNormalInverseWishartMatrixGaussian(N, \u039b, \u03a8, k, U);
    #line 81 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    M->graft(p, Integer(1));
    #line 82 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    V->graftReplace(p, Integer(2));
    #line 83 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    M->join(V);
    #line 84 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    return p;
  } else {
    #line 86 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    V->prune();
    #line 87 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    if (V->hasInverseWishart()) {
      auto n4530_ = V->getInverseWishart();
      auto \u03a8 = std::get<0>(n4530_);
      auto k = std::get<1>(n4530_);
      auto \u039b = cholinv(U);
      auto p = MatrixNormalInverseWishart(\u039b * M, \u039b, \u03a8, k);
      #line 91 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
      V->graft(p);
      #line 92 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
      return p;
    } else {
      #line 94 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
      return construct<MatrixGaussianDistribution<Random<libbirch::DefaultArray<Real,2>>, Arg, Random<libbirch::DefaultArray<Real,2>>>>(M, U, V);
    }
  }
}

#line 129 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 129 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
inline decltype(auto) logpdf_matrix_normal_inverse_wishart_matrix_gaussian(const Arg1& X, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& U) {
  auto S = cholinv(\u039b);
  auto M = S * N;
  auto \u03a3 = U + S;
  #line 134 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return logpdf_matrix_t(X, k, M, \u03a3, \u03a8);
}

#line 150 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 150 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
inline decltype(auto) update_matrix_normal_inverse_wishart_matrix_gaussian(const Arg1& X, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& U) {
  auto n = rows(X);
  auto L = cholinv(U);
  auto Z = L * X;
  auto \u039b_prime_ = \u039b + L;
  auto N_prime_ = N + Z;
  auto M = cholsolve(\u039b, N);
  auto M_prime_ = cholsolve(\u039b_prime_, N_prime_);
  auto \u03a8_prime_ = \u03a8 + inner(X, Z) + inner(M, N) - inner(M_prime_, N_prime_);
  auto k_prime_ = k + n;
  #line 161 "src/distribution/matrix_normal_inverse_wishart/MatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return MatrixNormalInverseWishart(N_prime_, \u039b_prime_, \u03a8_prime_, k_prime_);
}

#line 73 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 73 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
inline decltype(auto) ScaledMatrixNormalInverseWishartMatrixGaussian(const Arg1& A, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& U) {
  #line 75 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return construct<ScaledMatrixNormalInverseWishartMatrixGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>>(A, N, \u039b, \u03a8, k, U);
}

#line 82 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 82 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,2>> MatrixGaussian(const Multiply<Arg1, Random<libbirch::DefaultArray<Real,2>>>& M, const Arg2& U, const Random<libbirch::DefaultArray<Real,2>>& V) {
  #line 84 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  M->prune();
  #line 85 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  if (M->hasMatrixNormalInverseWishart() && M->isGrandChildOf(V)) {
    auto n4531_ = M->getMatrixNormalInverseWishart();
    auto N = std::get<0>(n4531_);
    auto \u039b = std::get<1>(n4531_);
    auto \u03a8 = std::get<2>(n4531_);
    auto k = std::get<3>(n4531_);
    auto p = ScaledMatrixNormalInverseWishartMatrixGaussian(M->l, N, \u039b, \u03a8, k, U);
    #line 89 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    M->graft(p, Integer(1));
    #line 90 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    V->graftReplace(p, Integer(2));
    #line 91 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    M->join(V);
    #line 92 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    return p;
  } else {
    #line 94 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    V->prune();
    #line 95 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
    if (V->hasInverseWishart()) {
      auto n4532_ = V->getInverseWishart();
      auto \u03a8 = std::get<0>(n4532_);
      auto k = std::get<1>(n4532_);
      auto \u039b = cholinv(U);
      auto p = MatrixNormalInverseWishart(\u039b * M, \u039b, \u03a8, k);
      #line 99 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
      V->graft(p);
      #line 100 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
      return p;
    } else {
      #line 102 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
      return construct<MatrixGaussianDistribution<Multiply<Arg1, Random<libbirch::DefaultArray<Real,2>>>, Arg2, Random<libbirch::DefaultArray<Real,2>>>>(M, U, V);
    }
  }
}

#line 142 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 142 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
inline decltype(auto) logpdf_scaled_matrix_normal_inverse_wishart_matrix_gaussian(const Arg1& X, const Arg2& A, const Arg3& N, const Arg4& \u039b, const Arg5& \u03a8, const Arg6& k, const Arg7& U) {
  auto S = cholinv(\u039b);
  auto M = A * S * N;
  auto \u03a3 = U + outer(A * S, A);
  #line 148 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return logpdf_matrix_t(X, k, M, \u03a3, \u03a8);
}

#line 165 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 165 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
inline decltype(auto) update_scaled_matrix_normal_inverse_wishart_matrix_gaussian(const Arg1& X, const Arg2& A, const Arg3& N, const Arg4& \u039b, const Arg5& \u03a8, const Arg6& k, const Arg7& U) {
  auto n = rows(X);
  auto L = cholinv(U);
  auto Z = L * X;
  auto \u039b_prime_ = \u039b + inner(A, L * A);
  auto N_prime_ = N + inner(A, Z);
  auto M = cholsolve(\u039b, N);
  auto M_prime_ = cholsolve(\u039b_prime_, N_prime_);
  auto \u03a8_prime_ = \u03a8 + inner(X, Z) + inner(M, N) - inner(M_prime_, N_prime_);
  auto k_prime_ = k + n;
  #line 177 "src/distribution/matrix_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMatrixGaussianDistribution.birch"
  return MatrixNormalInverseWishart(N_prime_, \u039b_prime_, \u03a8_prime_, k_prime_);
}

#line 81 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2>
#line 81 "src/distribution/matrix_other/InverseWishartDistribution.birch"
inline decltype(auto) InverseWishart(const Arg1& \u03a8, const Arg2& k) {
  #line 82 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  return construct<InverseWishartDistribution<Arg1, Arg2>>(\u03a8, k);
}

#line 104 "src/distribution/matrix_other/InverseWishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 104 "src/distribution/matrix_other/InverseWishartDistribution.birch"
inline decltype(auto) logpdf_inverse_wishart(const Arg1& X, const Arg2& \u03a8, const Arg3& k) {
  auto p = columns(X);
  auto l = k + p + 1.0;
  #line 107 "src/distribution/matrix_other/InverseWishartDistribution.birch"
  return -(0.5) * (l + p + 1.0) * lcholdet(X) - 0.5 * trace(cholsolve(X, \u03a8)) - 0.5 * l * (p * log(2.0) - lcholdet(\u03a8)) - lgamma(0.5 * l, p);
}

#line 26 "src/distribution/matrix_other/MatrixTDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 26 "src/distribution/matrix_other/MatrixTDistribution.birch"
inline decltype(auto) logpdf_matrix_t(const Arg1& X, const Arg2& k, const Arg3& M, const Arg4& U, const Arg5& V) {
  auto n = rows(X);
  auto p = columns(X);
  auto a = 0.5 * (k + p + n - 1.0);
  auto b = 0.5 * (k + n - 1.0);
  auto Z = X - M;
  #line 33 "src/distribution/matrix_other/MatrixTDistribution.birch"
  return lgamma(a, n) - 0.5 * p * n * log(\u03c0) - lgamma(b, n) - 0.5 * p * lcholdet(U) - 0.5 * n * lcholdet(V) - a * ldet(identity(n) + cholsolve(U, Z) * cholsolve(V, transpose(Z)));
}

#line 45 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2>
#line 45 "src/distribution/matrix_other/WishartDistribution.birch"
inline decltype(auto) Wishart(const Arg1& \u03a8, const Arg2& k) {
  #line 46 "src/distribution/matrix_other/WishartDistribution.birch"
  return construct<WishartDistribution<Arg1, Arg2>>(\u03a8, k);
}

#line 82 "src/distribution/matrix_other/WishartDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 82 "src/distribution/matrix_other/WishartDistribution.birch"
inline decltype(auto) logpdf_wishart(const Arg1& X, const Arg2& \u03a8, const Arg3& k) {
  auto p = columns(X);
  auto l = k + p + 1.0;
  #line 85 "src/distribution/matrix_other/WishartDistribution.birch"
  return 0.5 * k * lcholdet(X) - 0.5 * trace(cholsolve(\u03a8, X)) - 0.5 * l * (p * log(2.0) + lcholdet(\u03a8)) - lgamma(0.5 * l, p);
}

#line 33 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 33 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
inline decltype(auto) DirichletMultinomial(const Arg1& n, const Arg2& \u03b1) {
  #line 34 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  return construct<DirichletMultinomialDistribution<Arg1, Arg2>>(n, \u03b1);
}

#line 40 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
template<class Arg>
#line 40 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
inline Distribution<libbirch::DefaultArray<Integer,1>> Multinomial(const Arg& n, const Random<libbirch::DefaultArray<Real,1>>& \u03c1) {
  #line 42 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  \u03c1->prune();
  #line 43 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
  if (\u03c1->hasDirichlet()) {
    auto \u03b1 = \u03c1->getDirichlet();
    auto p = DirichletMultinomial(n, \u03b1);
    #line 46 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    \u03c1->graft(p);
    #line 47 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    return p;
  } else {
    #line 49 "src/distribution/multivariate_discrete/DirichletMultinomialDistribution.birch"
    return construct<MultinomialDistribution<Arg, Random<libbirch::DefaultArray<Real,1>>>>(n, \u03c1);
  }
}

#line 38 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 38 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
inline decltype(auto) Multinomial(const Arg1& n, const Arg2& \u03c1) {
  #line 39 "src/distribution/multivariate_discrete/MultinomialDistribution.birch"
  return construct<MultinomialDistribution<Arg1, Arg2>>(n, \u03c1);
}

#line 76 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 76 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
inline decltype(auto) LinearMultivariateGaussianMultivariateGaussian(const Arg1& A, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& c, const Arg5& S) {
  #line 78 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  return construct<LinearMultivariateGaussianMultivariateGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5>>(A, \u03bc, \u03a3, c, S);
}

#line 85 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 85 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Arg3& \u03a3) {
  #line 87 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  \u03bc->l->r->prune();
  #line 88 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
  if (\u03bc->l->r->hasMultivariateGaussian()) {
    auto n4533_ = \u03bc->l->r->getMultivariateGaussian();
    auto m = std::get<0>(n4533_);
    auto S = std::get<1>(n4533_);
    auto p = LinearMultivariateGaussianMultivariateGaussian(\u03bc->l->l, m, S, \u03bc->r, \u03a3);
    #line 92 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
    \u03bc->l->r->graft(p);
    #line 93 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 95 "src/distribution/multivariate_gaussian/LinearMultivariateGaussianMultivariateGaussianDistribution.birch"
    return construct<MultivariateGaussianDistribution<Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>, Arg3>>(\u03bc, \u03a3);
  }
}

#line 55 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 55 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
inline decltype(auto) MultivariateGaussian(const Arg1& \u03bc, const Arg2& \u03a3) {
  #line 56 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  return construct<MultivariateGaussianDistribution<Arg1, Arg2>>(\u03bc, \u03a3);
}

#line 84 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 84 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
inline decltype(auto) logpdf_multivariate_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03a3) {
  auto n = length(x);
  #line 87 "src/distribution/multivariate_gaussian/MultivariateGaussianDistribution.birch"
  return -(0.5) * (dot(x - \u03bc, cholsolve(\u03a3, x - \u03bc)) + n * log(2.0 * \u03c0) + lcholdet(\u03a3));
}

#line 64 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 64 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
inline decltype(auto) MultivariateGaussianMultivariateGaussian(const Arg1& \u03bc, const Arg2& \u03a3, const Arg3& S) {
  #line 66 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return construct<MultivariateGaussianMultivariateGaussianDistribution<Arg1, Arg2, Arg3>>(\u03bc, \u03a3, S);
}

#line 73 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg>
#line 73 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Random<libbirch::DefaultArray<Real,1>>& \u03bc, const Arg& \u03a3) {
  #line 75 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  \u03bc->prune();
  #line 76 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  if (\u03bc->hasMultivariateGaussian()) {
    auto n4534_ = \u03bc->getMultivariateGaussian();
    auto m = std::get<0>(n4534_);
    auto S = std::get<1>(n4534_);
    auto p = MultivariateGaussianMultivariateGaussian(m, S, \u03a3);
    #line 79 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
    \u03bc->graft(p);
    #line 80 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 82 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
    return construct<MultivariateGaussianDistribution<Random<libbirch::DefaultArray<Real,1>>, Arg>>(\u03bc, \u03a3);
  }
}

#line 109 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 109 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
inline decltype(auto) logpdf_multivariate_gaussian_multivariate_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& S) {
  #line 111 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return logpdf_multivariate_gaussian(x, \u03bc, \u03a3 + S);
}

#line 125 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 125 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
inline decltype(auto) update_multivariate_gaussian_multivariate_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& S) {
  auto K_prime_ = transpose(cholsolve(\u03a3 + S, \u03a3));
  auto \u03bc_prime_ = \u03bc + K_prime_ * (x - \u03bc);
  auto \u03a3_prime_ = \u03a3 - K_prime_ * \u03a3;
  #line 130 "src/distribution/multivariate_gaussian/MultivariateGaussianMultivariateGaussianDistribution.birch"
  return MultivariateGaussian(\u03bc_prime_, \u03a3_prime_);
}

#line 70 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 70 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
inline decltype(auto) ScaledMultivariateGaussianMultivariateGaussian(const Arg1& A, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& S) {
  #line 72 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return construct<ScaledMultivariateGaussianMultivariateGaussianDistribution<Arg1, Arg2, Arg3, Arg4>>(A, \u03bc, \u03a3, S);
}

#line 79 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 79 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Arg2& \u03a3) {
  #line 81 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  \u03bc->r->prune();
  #line 82 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  if (\u03bc->r->hasMultivariateGaussian()) {
    auto n4535_ = \u03bc->r->getMultivariateGaussian();
    auto m = std::get<0>(n4535_);
    auto S = std::get<1>(n4535_);
    auto p = ScaledMultivariateGaussianMultivariateGaussian(\u03bc->l, m, S, \u03a3);
    #line 85 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
    \u03bc->r->graft(p);
    #line 86 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 88 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
    return construct<MultivariateGaussianDistribution<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>>(\u03bc, \u03a3);
  }
}

#line 118 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 118 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
inline decltype(auto) logpdf_scaled_multivariate_gaussian_multivariate_gaussian(const Arg1& x, const Arg2& A, const Arg3& \u03bc, const Arg4& \u03a3, const Arg5& S) {
  #line 120 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return logpdf_multivariate_gaussian(x, A * \u03bc, outer(A * \u03a3, A) + S);
}

#line 135 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 135 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
inline decltype(auto) update_scaled_multivariate_gaussian_multivariate_gaussian(const Arg1& x, const Arg2& A, const Arg3& \u03bc, const Arg4& \u03a3, const Arg5& S) {
  auto K_prime_ = outer(\u03a3, cholsolve(outer(A * \u03a3, A) + S, A));
  auto \u03bc_prime_ = \u03bc + K_prime_ * (x - A * \u03bc);
  auto \u03a3_prime_ = \u03a3 - K_prime_ * A * \u03a3;
  #line 140 "src/distribution/multivariate_gaussian/ScaledMultivariateGaussianMultivariateGaussianDistribution.birch"
  return MultivariateGaussian(\u03bc_prime_, \u03a3_prime_);
}

#line 81 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 81 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline decltype(auto) LinearMultivariateNormalInverseGammaMultivariateGaussian(const Arg1& A, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& \u03a3) {
  #line 84 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return construct<LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>>(A, \u03bd, \u039b, \u03b1, \u03b2, c, \u03a3);
}

#line 91 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 91 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Multiply<Arg3, Random<Real>>& \u03a3) {
  #line 94 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  \u03bc->l->r->prune();
  #line 95 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  if (\u03bc->l->r->hasMultivariateNormalInverseGamma() && \u03bc->l->r->isGrandChildOf(\u03a3->r)) {
    auto n4536_ = \u03bc->l->r->getMultivariateNormalInverseGamma();
    auto \u03bd = std::get<0>(n4536_);
    auto \u039b = std::get<1>(n4536_);
    auto \u03b1 = std::get<2>(n4536_);
    auto \u03b2 = std::get<3>(n4536_);
    auto p = LinearMultivariateNormalInverseGammaMultivariateGaussian(\u03bc->l->l, \u03bd, \u039b, \u03b1, \u03b2, \u03bc->r, \u03a3->l);
    #line 99 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->l->r->graft(p, Integer(1));
    #line 100 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->r->graftReplace(p, Integer(2));
    #line 101 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->l->r->join(\u03a3->r);
    #line 102 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 104 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->r->prune();
    #line 105 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    if (\u03a3->r->hasInverseGamma()) {
      auto n4537_ = \u03a3->r->getInverseGamma();
      auto \u03b1 = std::get<0>(n4537_);
      auto \u03b2 = std::get<1>(n4537_);
      auto p = MultivariateNormalInverseGamma(\u03bc, \u03a3->l, \u03b1, \u03b2);
      #line 108 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      \u03a3->r->graft(p);
      #line 109 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return p;
    } else {
      #line 111 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return construct<MultivariateGaussianDistribution<Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>, Multiply<Arg3, Random<Real>>>>(\u03bc, \u03a3);
    }
  }
}

#line 120 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 120 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Multiply<Random<Real>, Arg3>& \u03a3) {
  #line 123 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  \u03bc->l->r->prune();
  #line 124 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  if (\u03bc->l->r->hasMultivariateNormalInverseGamma() && \u03bc->l->r->isGrandChildOf(\u03a3->l)) {
    auto n4538_ = \u03bc->l->r->getMultivariateNormalInverseGamma();
    auto \u03bd = std::get<0>(n4538_);
    auto \u039b = std::get<1>(n4538_);
    auto \u03b1 = std::get<2>(n4538_);
    auto \u03b2 = std::get<3>(n4538_);
    auto p = LinearMultivariateNormalInverseGammaMultivariateGaussian(\u03bc->l->l, \u03bd, \u039b, \u03b1, \u03b2, \u03bc->r, \u03a3->r);
    #line 128 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->l->r->graft(p, Integer(1));
    #line 129 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->r->graftReplace(p, Integer(2));
    #line 130 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->l->r->join(\u03a3->r);
    #line 131 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 132 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    if (\u03a3->l->hasInverseGamma()) {
      #line 133 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      \u03a3->l->prune();
      auto n4539_ = \u03a3->l->getInverseGamma();
      auto \u03b1 = std::get<0>(n4539_);
      auto \u03b2 = std::get<1>(n4539_);
      auto p = MultivariateNormalInverseGamma(\u03bc, \u03a3->r, \u03b1, \u03b2);
      #line 136 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      \u03a3->l->graft(p);
      #line 137 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return p;
    } else {
      #line 139 "src/distribution/multivariate_normal_inverse_gamma/LinearMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return construct<MultivariateGaussianDistribution<Add<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>, Multiply<Random<Real>, Arg3>>>(\u03bc, \u03a3);
    }
  }
}

#line 107 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 107 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
inline decltype(auto) MultivariateNormalInverseGamma(const Arg1& \u03bd, const Arg2& \u039b, const Arg3& \u03b1, const Arg4& \u03b2) {
  #line 109 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return construct<MultivariateNormalInverseGammaDistribution<Arg1, Arg2, Arg3, Arg4>>(\u03bd, \u039b, \u03b1, \u03b2);
}

#line 116 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 116 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Arg1& \u03bc, const Multiply<Arg2, Random<Real>>& \u03a3) {
  #line 118 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  \u03a3->r->prune();
  #line 119 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  if (\u03a3->r->hasInverseGamma()) {
    auto n4540_ = \u03a3->r->getInverseGamma();
    auto \u03b1 = std::get<0>(n4540_);
    auto \u03b2 = std::get<1>(n4540_);
    auto \u039b = cholinv(\u03a3->l);
    auto p = MultivariateNormalInverseGamma(\u039b * \u03bc, \u039b, \u03b1, \u03b2);
    #line 123 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    \u03a3->r->graft(p);
    #line 124 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    return p;
  } else {
    #line 126 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    return construct<MultivariateGaussianDistribution<Arg1, Multiply<Arg2, Random<Real>>>>(\u03bc, \u03a3);
  }
}

#line 134 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 134 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Arg1& \u03bc, const Multiply<Random<Real>, Arg2>& \u03a3) {
  #line 136 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  \u03a3->l->prune();
  #line 137 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  if (\u03a3->l->hasInverseGamma()) {
    auto n4541_ = \u03a3->l->getInverseGamma();
    auto \u03b1 = std::get<0>(n4541_);
    auto \u03b2 = std::get<1>(n4541_);
    auto \u039b = cholinv(\u03a3->r);
    auto p = MultivariateNormalInverseGamma(\u039b * \u03bc, \u039b, \u03b1, \u03b2);
    #line 141 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    \u03a3->l->graft(p);
    #line 142 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    return p;
  } else {
    #line 144 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
    return construct<MultivariateGaussianDistribution<Arg1, Multiply<Random<Real>, Arg2>>>(\u03bc, \u03a3);
  }
}

#line 174 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 174 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
inline decltype(auto) logpdf_multivariate_normal_inverse_gamma(const Arg1& x, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2) {
  auto \u03a3 = cholinv(\u039b);
  #line 177 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return logpdf_multivariate_t(x, 2.0 * \u03b1, \u03a3 * \u03bd, 2.0 * \u03b2 * \u03a3);
}

#line 192 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 192 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
inline decltype(auto) update_multivariate_normal_inverse_gamma(const Arg1& x, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2) {
  auto n = length(x);
  auto \u03bc = cholsolve(\u039b, \u03bd);
  #line 196 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaDistribution.birch"
  return InverseGamma(\u03b1 + 0.5 * n, \u03b2 + 0.5 * dot(x - \u03bc, \u039b * (x - \u03bc)));
}

#line 67 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 67 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline decltype(auto) MultivariateNormalInverseGammaMultivariateGaussian(const Arg1& \u03bd, const Arg2& \u039b, const Arg3& \u03b1, const Arg4& \u03b2, const Arg5& \u03a3) {
  #line 69 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return construct<MultivariateNormalInverseGammaMultivariateGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5>>(\u03bd, \u039b, \u03b1, \u03b2, \u03a3);
}

#line 76 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg>
#line 76 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Random<libbirch::DefaultArray<Real,1>>& \u03bc, const Multiply<Arg, Random<Real>>& \u03a3) {
  #line 78 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  \u03bc->prune();
  #line 79 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  if (\u03bc->hasMultivariateNormalInverseGamma() && \u03bc->isGrandChildOf(\u03a3->r)) {
    auto n4542_ = \u03bc->getMultivariateNormalInverseGamma();
    auto \u03bd = std::get<0>(n4542_);
    auto \u039b = std::get<1>(n4542_);
    auto \u03b1 = std::get<2>(n4542_);
    auto \u03b2 = std::get<3>(n4542_);
    auto p = MultivariateNormalInverseGammaMultivariateGaussian(\u03bd, \u039b, \u03b1, \u03b2, \u03a3->l);
    #line 83 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->graft(p, Integer(1));
    #line 84 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->r->graftReplace(p, Integer(2));
    #line 85 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->join(\u03a3->r);
    #line 86 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 88 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->r->prune();
    #line 89 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    if (\u03a3->r->hasInverseGamma()) {
      auto n4543_ = \u03a3->r->getInverseGamma();
      auto \u03b1 = std::get<0>(n4543_);
      auto \u03b2 = std::get<1>(n4543_);
      auto \u039b = cholinv(\u03a3->l);
      auto p = MultivariateNormalInverseGamma(\u039b * \u03bc, \u039b, \u03b1, \u03b2);
      #line 93 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      \u03a3->r->graft(p);
      #line 94 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return p;
    } else {
      #line 96 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return construct<MultivariateGaussianDistribution<Random<libbirch::DefaultArray<Real,1>>, Multiply<Arg, Random<Real>>>>(\u03bc, \u03a3);
    }
  }
}

#line 105 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg>
#line 105 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Random<libbirch::DefaultArray<Real,1>>& \u03bc, const Multiply<Random<Real>, Arg>& \u03a3) {
  #line 107 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  \u03bc->prune();
  #line 108 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  if (\u03bc->hasMultivariateNormalInverseGamma() && \u03bc->isGrandChildOf(\u03a3->l)) {
    auto n4544_ = \u03bc->getMultivariateNormalInverseGamma();
    auto \u03bd = std::get<0>(n4544_);
    auto \u039b = std::get<1>(n4544_);
    auto \u03b1 = std::get<2>(n4544_);
    auto \u03b2 = std::get<3>(n4544_);
    auto p = MultivariateNormalInverseGammaMultivariateGaussian(\u03bd, \u039b, \u03b1, \u03b2, \u03a3->r);
    #line 112 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->graft(p, Integer(1));
    #line 113 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->l->graftReplace(p, Integer(2));
    #line 114 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->join(\u03a3->l);
    #line 115 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 117 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->l->prune();
    #line 118 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    if (\u03a3->l->hasInverseGamma()) {
      auto n4545_ = \u03a3->l->getInverseGamma();
      auto \u03b1 = std::get<0>(n4545_);
      auto \u03b2 = std::get<1>(n4545_);
      auto \u039b = cholinv(\u03a3->r);
      auto p = MultivariateNormalInverseGamma(\u039b * \u03bc, \u039b, \u03b1, \u03b2);
      #line 122 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      \u03a3->l->graft(p);
      #line 123 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return p;
    } else {
      #line 125 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return construct<MultivariateGaussianDistribution<Random<libbirch::DefaultArray<Real,1>>, Multiply<Arg, Random<Real>>>>(\u03bc, \u03a3);
    }
  }
}

#line 161 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 161 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline decltype(auto) logpdf_multivariate_normal_inverse_gamma_multivariate_gaussian(const Arg1& x, const Arg2& \u03bd, const Arg3& \u039b, const Arg5& \u03b1, const Arg6& \u03b2, const Arg4& \u03a3) {
  auto S = cholinv(\u039b);
  auto \u03bc = S * \u03bd;
  #line 166 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return logpdf_multivariate_t(x, 2.0 * \u03b1, \u03bc, 2.0 * \u03b2 * (\u03a3 + S));
}

#line 182 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 182 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline decltype(auto) update_multivariate_normal_inverse_gamma_multivariate_gaussian(const Arg1& x, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& \u03a3) {
  auto n = length(x);
  auto L = cholinv(\u03a3);
  auto z = L * x;
  auto \u039b_prime_ = \u039b + L;
  auto \u03bd_prime_ = \u03bd + z;
  auto \u03bc = cholsolve(\u039b, \u03bd);
  auto \u03bc_prime_ = cholsolve(\u039b_prime_, \u03bd_prime_);
  auto \u03b1_prime_ = \u03b1 + 0.5 * n;
  auto \u03b2_prime_ = \u03b2 + 0.5 * dot(x, z) + 0.5 * dot(\u03bc, \u03bd) - 0.5 * dot(\u03bc_prime_, \u03bd_prime_);
  #line 194 "src/distribution/multivariate_normal_inverse_gamma/MultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return MultivariateNormalInverseGamma(\u03bd_prime_, \u039b_prime_, \u03b1_prime_, \u03b2_prime_);
}

#line 73 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 73 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline decltype(auto) ScaledMultivariateNormalInverseGammaMultivariateGaussian(const Arg1& A, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& \u03a3) {
  #line 76 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return construct<ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>>(A, \u03bd, \u039b, \u03b1, \u03b2, \u03a3);
}

#line 83 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 83 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Multiply<Arg2, Random<Real>>& \u03a3) {
  #line 85 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  \u03bc->r->prune();
  #line 86 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  if (\u03bc->r->hasMultivariateNormalInverseGamma() && \u03bc->r->isGrandChildOf(\u03a3->r)) {
    auto n4546_ = \u03bc->r->getMultivariateNormalInverseGamma();
    auto \u03bd = std::get<0>(n4546_);
    auto \u039b = std::get<1>(n4546_);
    auto \u03b1 = std::get<2>(n4546_);
    auto \u03b2 = std::get<3>(n4546_);
    auto p = ScaledMultivariateNormalInverseGammaMultivariateGaussian(\u03bc->l, \u03bd, \u039b, \u03b1, \u03b2, \u03a3->l);
    #line 90 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->r->graft(p, Integer(1));
    #line 91 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->r->graftReplace(p, Integer(2));
    #line 92 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->r->join(\u03a3->r);
    #line 93 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 95 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->r->prune();
    #line 96 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    if (\u03a3->r->hasInverseGamma()) {
      auto n4547_ = \u03a3->r->getInverseGamma();
      auto \u03b1 = std::get<0>(n4547_);
      auto \u03b2 = std::get<1>(n4547_);
      auto p = MultivariateNormalInverseGamma(\u03bc, \u03a3->l, \u03b1, \u03b2);
      #line 99 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      \u03a3->r->graft(p);
      #line 100 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return p;
    } else {
      #line 102 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return construct<MultivariateGaussianDistribution<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Multiply<Arg2, Random<Real>>>>(\u03bc, \u03a3);
    }
  }
}

#line 111 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 111 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Multiply<Random<Real>, Arg2>& \u03a3) {
  #line 113 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  \u03bc->r->prune();
  #line 114 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  if (\u03bc->r->hasMultivariateNormalInverseGamma() && \u03bc->r->isGrandChildOf(\u03a3->l)) {
    auto n4548_ = \u03bc->r->getMultivariateNormalInverseGamma();
    auto \u03bd = std::get<0>(n4548_);
    auto \u039b = std::get<1>(n4548_);
    auto \u03b1 = std::get<2>(n4548_);
    auto \u03b2 = std::get<3>(n4548_);
    auto p = ScaledMultivariateNormalInverseGammaMultivariateGaussian(\u03bc->l, \u03bd, \u039b, \u03b1, \u03b2, \u03a3->r);
    #line 118 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->r->graft(p, Integer(1));
    #line 119 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->l->graftReplace(p, Integer(2));
    #line 120 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03bc->r->join(\u03a3->l);
    #line 121 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 123 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    \u03a3->l->prune();
    #line 124 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
    if (\u03a3->l->hasInverseGamma()) {
      auto n4549_ = \u03a3->l->getInverseGamma();
      auto \u03b1 = std::get<0>(n4549_);
      auto \u03b2 = std::get<1>(n4549_);
      auto p = MultivariateNormalInverseGamma(\u03bc, \u03a3->r, \u03b1, \u03b2);
      #line 127 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      \u03a3->l->graft(p);
      #line 128 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return p;
    } else {
      #line 130 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
      return construct<MultivariateGaussianDistribution<Multiply<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Multiply<Arg2, Random<Real>>>>(\u03bc, \u03a3);
    }
  }
}

#line 170 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 170 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline decltype(auto) logpdf_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(const Arg1& x, const Arg2& A, const Arg3& \u03bd, const Arg4& \u039b, const Arg5& \u03b1, const Arg6& \u03b2, const Arg7& \u03a3) {
  auto S = cholinv(\u039b);
  auto \u03bc = S * \u03bd;
  #line 175 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return logpdf_multivariate_t(x, 2.0 * \u03b1, A * \u03bc, 2.0 * \u03b2 * (\u03a3 + outer(A * S, A)));
}

#line 193 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 193 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
inline decltype(auto) update_scaled_multivariate_normal_inverse_gamma_multivariate_gaussian(const Arg1& x, const Arg2& A, const Arg3& \u03bd, const Arg4& \u039b, const Arg5& \u03b1, const Arg6& \u03b2, const Arg7& \u03a3) {
  auto n = length(x);
  auto L = cholinv(\u03a3);
  auto z = L * x;
  auto \u039b_prime_ = \u039b + inner(A, L * A);
  auto \u03bd_prime_ = \u03bd + inner(A, z);
  auto \u03bc = cholsolve(\u039b, \u03bd);
  auto \u03bc_prime_ = cholsolve(\u039b_prime_, \u03bd_prime_);
  auto \u03b1_prime_ = \u03b1 + 0.5 * n;
  auto \u03b2_prime_ = \u03b2 + 0.5 * (dot(x, z) + dot(\u03bc, \u03bd) - dot(\u03bc_prime_, \u03bd_prime_));
  #line 205 "src/distribution/multivariate_normal_inverse_gamma/ScaledMultivariateNormalInverseGammaMultivariateGaussianDistribution.birch"
  return MultivariateNormalInverseGamma(\u03bd_prime_, \u039b_prime_, \u03b1_prime_, \u03b2_prime_);
}

#line 73 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 73 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
inline decltype(auto) LinearMatrixNormalInverseWishartMultivariateGaussian(const Arg1& a, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k, const Arg6& c) {
  #line 75 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return construct<LinearMatrixNormalInverseWishartMultivariateGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6>>(a, N, \u039b, \u03a8, k, c);
}

#line 82 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 82 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,2>>>, Arg2>& \u03bc, const Random<libbirch::DefaultArray<Real,2>>& \u03a3) {
  #line 85 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  \u03bc->l->r->prune();
  #line 86 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  if (\u03bc->l->r->hasMatrixNormalInverseWishart() && \u03bc->l->r->isGrandChildOf(\u03a3)) {
    auto n4550_ = \u03bc->l->r->getMatrixNormalInverseWishart();
    auto N = std::get<0>(n4550_);
    auto \u039b = std::get<1>(n4550_);
    auto \u03a8 = std::get<2>(n4550_);
    auto k = std::get<3>(n4550_);
    auto p = LinearMatrixNormalInverseWishartMultivariateGaussian(\u03bc->l->l, N, \u039b, \u03a8, k, \u03bc->r);
    #line 90 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u03bc->l->r->graft(p, Integer(1));
    #line 91 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u03a3->graftReplace(p, Integer(2));
    #line 92 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u03bc->l->r->join(\u03a3);
    #line 93 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 95 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u03a3->prune();
    #line 100 "src/distribution/multivariate_normal_inverse_wishart/LinearMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    return construct<MultivariateGaussianDistribution<Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,2>>>, Arg2>, Random<libbirch::DefaultArray<Real,2>>>>(\u03bc, \u03a3);
  }
}

#line 68 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 68 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
inline decltype(auto) ScaledMatrixNormalInverseWishartMultivariateGaussian(const Arg1& a, const Arg2& N, const Arg3& \u039b, const Arg4& \u03a8, const Arg5& k) {
  #line 70 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return construct<ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5>>(a, N, \u039b, \u03a8, k);
}

#line 77 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg>
#line 77 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
inline Distribution<libbirch::DefaultArray<Real,1>> MultivariateGaussian(const Dot<Arg, Random<libbirch::DefaultArray<Real,2>>>& \u03bc, const Random<libbirch::DefaultArray<Real,2>>& \u03a3) {
  #line 79 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  \u03bc->r->prune();
  #line 80 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  if (\u03bc->r->hasMatrixNormalInverseWishart() && \u03bc->r->isGrandChildOf(\u03a3)) {
    auto n4551_ = \u03bc->r->getMatrixNormalInverseWishart();
    auto N = std::get<0>(n4551_);
    auto \u039b = std::get<1>(n4551_);
    auto \u03a8 = std::get<2>(n4551_);
    auto k = std::get<3>(n4551_);
    auto p = ScaledMatrixNormalInverseWishartMultivariateGaussian(\u03bc->l, N, \u039b, \u03a8, k);
    #line 84 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u03bc->r->graft(p, Integer(1));
    #line 85 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u03a3->graftReplace(p, Integer(2));
    #line 86 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u03bc->r->join(\u03a3);
    #line 87 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    return p;
  } else {
    #line 89 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    \u03a3->prune();
    #line 94 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
    return construct<MultivariateGaussianDistribution<Dot<Arg, Random<libbirch::DefaultArray<Real,2>>>, Random<libbirch::DefaultArray<Real,2>>>>(\u03bc, \u03a3);
  }
}

#line 131 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 131 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
inline decltype(auto) logpdf_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(const Arg1& x, const Arg2& a, const Arg3& N, const Arg4& \u039b, const Arg5& \u03a8, const Arg6& k) {
  auto S = cholinv(\u039b);
  auto M = S * N;
  auto \u03c32 = 1.0 + dot(a, S * a);
  #line 137 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return logpdf_multivariate_t(x, k, dot(a, M), \u03c32 * \u03a8);
}

#line 153 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 153 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
inline decltype(auto) update_scaled_matrix_normal_inverse_wishart_multivariate_gaussian(const Arg1& x, const Arg2& a, const Arg3& N, const Arg4& \u039b, const Arg5& \u03a8, const Arg6& k) {
  auto \u039b_prime_ = \u039b + outer(a);
  auto N_prime_ = N + outer(a, x);
  auto M = cholsolve(\u039b, N);
  auto M_prime_ = cholsolve(\u039b_prime_, N_prime_);
  auto \u03a8_prime_ = \u03a8 + outer(x) + inner(M, N) - inner(M_prime_, N_prime_);
  auto k_prime_ = k + 1.0;
  #line 162 "src/distribution/multivariate_normal_inverse_wishart/ScaledMatrixNormalInverseWishartMultivariateGaussianDistribution.birch"
  return MatrixNormalInverseWishart(N_prime_, \u039b_prime_, \u03a8_prime_, k_prime_);
}

#line 39 "src/distribution/multivariate_other/DirichletDistribution.birch"
template<class Arg>
#line 39 "src/distribution/multivariate_other/DirichletDistribution.birch"
inline decltype(auto) Dirichlet(const Arg& \u03b1) {
  #line 40 "src/distribution/multivariate_other/DirichletDistribution.birch"
  return construct<DirichletDistribution<Arg>>(\u03b1);
}

#line 23 "src/distribution/multivariate_other/MultivariateTDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 23 "src/distribution/multivariate_other/MultivariateTDistribution.birch"
inline decltype(auto) logpdf_multivariate_t(const Arg1& x, const Arg2& k, const Arg3& \u03bc, const Arg4& U) {
  auto p = length(x);
  auto a = 0.5 * (k + p);
  auto b = 0.5 * k;
  auto z = x - \u03bc;
  #line 29 "src/distribution/multivariate_other/MultivariateTDistribution.birch"
  return lgamma(a) - 0.5 * p * log(\u03c0) - lgamma(b) - 0.5 * lcholdet(U) - a * log1p(dot(z, cholsolve(U, z)));
}

#line 73 "src/distribution/multivariate_other/RestaurantDistribution.birch"
template<class Arg1,class Arg2>
#line 73 "src/distribution/multivariate_other/RestaurantDistribution.birch"
inline decltype(auto) Restaurant(const Arg1& \u03b1, const Arg2& \u03b8) {
  #line 74 "src/distribution/multivariate_other/RestaurantDistribution.birch"
  return construct<RestaurantDistribution<Arg1, Arg2>>(\u03b1, \u03b8);
}

#line 39 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg>
#line 39 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
inline decltype(auto) Bernoulli(const Arg& \u03c1) {
  #line 40 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  return construct<BernoulliDistribution<Arg>>(\u03c1);
}

#line 63 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 63 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
inline decltype(auto) logpdf_bernoulli(const Arg1& x, const Arg2& \u03c1) {
  #line 64 "src/distribution/scalar_boolean/BernoulliDistribution.birch"
  return if_then_else(x, log(\u03c1), log1p(-(\u03c1)));
}

#line 44 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2>
#line 44 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
inline decltype(auto) BetaBernoulli(const Arg1& \u03b1, const Arg2& \u03b2) {
  #line 45 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  return construct<BetaBernoulliDistribution<Arg1, Arg2>>(\u03b1, \u03b2);
}

#line 84 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 84 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
inline decltype(auto) logpdf_beta_bernoulli(const Arg1& x, const Arg2& \u03b1, const Arg3& \u03b2) {
  #line 85 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  return if_then_else(x, log(\u03b1), log(\u03b2)) - log(\u03b1 + \u03b2);
}

#line 97 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 97 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
inline decltype(auto) update_beta_bernoulli(const Arg1& x, const Arg2& \u03b1, const Arg3& \u03b2) {
  #line 98 "src/distribution/scalar_boolean/BetaBernoulliDistribution.birch"
  return Beta(\u03b1 + x, \u03b2 + 1.0 - x);
}

#line 62 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 62 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
inline decltype(auto) BetaBinomial(const Arg1& n, const Arg2& \u03b1, const Arg3& \u03b2) {
  #line 63 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return construct<BetaBinomialDistribution<Arg1, Arg2, Arg3>>(n, \u03b1, \u03b2);
}

#line 69 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1>
#line 69 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
inline Distribution<Integer> Binomial(const Arg1& n, const Random<Real>& \u03c1) {
  #line 70 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  \u03c1->prune();
  #line 71 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  if (\u03c1->hasBeta()) {
    auto n4552_ = \u03c1->getBeta();
    auto \u03b1 = std::get<0>(n4552_);
    auto \u03b2 = std::get<1>(n4552_);
    auto p = BetaBinomial(n, \u03b1, \u03b2);
    #line 74 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
    \u03c1->graft(p);
    #line 75 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
    return p;
  } else {
    #line 77 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
    return construct<BinomialDistribution<Arg1, Random<Real>>>(n, \u03c1);
  }
}

#line 105 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 105 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
inline decltype(auto) logpdf_beta_binomial(const Arg1& x, const Arg2& n, const Arg3& \u03b1, const Arg4& \u03b2) {
  #line 107 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return lbeta(x + \u03b1, n - x + \u03b2) - lbeta(\u03b1, \u03b2) + lchoose(n, x);
}

#line 120 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 120 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
inline decltype(auto) update_beta_binomial(const Arg1& x, const Arg2& n, const Arg3& \u03b1, const Arg4& \u03b2) {
  #line 122 "src/distribution/scalar_discrete/BetaBinomialDistribution.birch"
  return Beta(\u03b1 + x, \u03b2 + n - x);
}

#line 54 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 54 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
inline decltype(auto) BetaNegativeBinomial(const Arg1& k, const Arg2& \u03b1, const Arg3& \u03b2) {
  #line 55 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return construct<BetaNegativeBinomialDistribution<Arg1, Arg2, Arg3>>(k, \u03b1, \u03b2);
}

#line 61 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1>
#line 61 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
inline Distribution<Integer> NegativeBinomial(const Arg1& k, const Random<Real>& \u03c1) {
  #line 62 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  \u03c1->prune();
  #line 63 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  if (\u03c1->hasBeta()) {
    auto n4553_ = \u03c1->getBeta();
    auto \u03b1 = std::get<0>(n4553_);
    auto \u03b2 = std::get<1>(n4553_);
    auto p = BetaNegativeBinomial(k, \u03b1, \u03b2);
    #line 66 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
    \u03c1->graft(p);
    #line 67 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
    return p;
  } else {
    #line 69 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
    return construct<NegativeBinomialDistribution<Arg1, Random<Real>>>(k, \u03c1);
  }
}

#line 97 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 97 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
inline decltype(auto) logpdf_beta_negative_binomial(const Arg1& x, const Arg2& k, const Arg3& \u03b1, const Arg4& \u03b2) {
  #line 99 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return lbeta(\u03b1 + k, \u03b2 + x) - lbeta(\u03b1, \u03b2) + lchoose(x + k - Integer(1), x);
}

#line 112 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 112 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
inline decltype(auto) update_beta_negative_binomial(const Arg1& x, const Arg2& k, const Arg3& \u03b1, const Arg4& \u03b2) {
  #line 114 "src/distribution/scalar_discrete/BetaNegativeBinomialDistribution.birch"
  return Beta(\u03b1 + k, \u03b2 + x);
}

#line 66 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 66 "src/distribution/scalar_discrete/BinomialDistribution.birch"
inline decltype(auto) Binomial(const Arg1& n, const Arg2& \u03c1) {
  #line 67 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return construct<BinomialDistribution<Arg1, Arg2>>(n, \u03c1);
}

#line 93 "src/distribution/scalar_discrete/BinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 93 "src/distribution/scalar_discrete/BinomialDistribution.birch"
inline decltype(auto) logpdf_binomial(const Arg1& x, const Arg2& n, const Arg3& \u03c1) {
  #line 94 "src/distribution/scalar_discrete/BinomialDistribution.birch"
  return x * log(\u03c1) + (n - x) * log1p(-(\u03c1)) + lchoose(n, x);
}

#line 50 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
template<class Arg>
#line 50 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
inline decltype(auto) Categorical(const Arg& \u03c1) {
  #line 51 "src/distribution/scalar_discrete/CategoricalDistribution.birch"
  return construct<CategoricalDistribution<Arg>>(\u03c1);
}

#line 78 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 78 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
inline decltype(auto) ConwayMaxwellPoisson(const Arg1& \u03bb, const Arg2& \u03bd, const Arg3& n) {
  #line 79 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return construct<ConwayMaxwellPoissonDistribution<Arg1, Arg2, Arg3>>(\u03bb, \u03bd, n);
}

#line 105 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 105 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
inline decltype(auto) logpdf_conway_maxwell_poisson(const Arg1& x, const Arg2& \u03bb, const Arg3& \u03bd, const Arg4& n) {
  #line 107 "src/distribution/scalar_discrete/ConwayMaxwellPoissonDistribution.birch"
  return x * log(\u03bb) - \u03bd * lgamma(x + Integer(1)) - logz_conway_maxwell_poisson(\u03bb, \u03bd, n);
}

#line 35 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg>
#line 35 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
inline decltype(auto) DirichletCategorical(const Arg& \u03b1) {
  #line 36 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  return construct<DirichletCategoricalDistribution<Arg>>(\u03b1);
}

#line 94 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
template<class Arg1,class Arg2>
#line 94 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
inline decltype(auto) update_dirichlet_categorical(const Arg1& x, const Arg2& \u03b1) {
  auto \u03b1_prime_ = \u03b1;
  #line 96 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  \u03b1_prime_(x) = \u03b1_prime_(x) + 1.0;
  #line 97 "src/distribution/scalar_discrete/DirichletCategoricalDistribution.birch"
  return Dirichlet(\u03b1_prime_);
}

#line 61 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 61 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
inline decltype(auto) GammaPoisson(const Arg1& k, const Arg2& \u03b8) {
  #line 62 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return construct<GammaPoissonDistribution<Arg1, Arg2>>(k, \u03b8);
}

#line 104 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 104 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
inline decltype(auto) logpdf_gamma_poisson(const Arg1& x, const Arg2& k, const Arg3& \u03b8) {
  auto \u03c1 = 1.0 / (\u03b8 + 1.0);
  #line 106 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return k * log(\u03c1) + x * log1p(-(\u03c1)) + lchoose(x + k - Integer(1), x);
}

#line 118 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 118 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
inline decltype(auto) update_gamma_poisson(const Arg1& x, const Arg2& k, const Arg3& \u03b8) {
  #line 119 "src/distribution/scalar_discrete/GammaPoissonDistribution.birch"
  return Gamma(k + x, \u03b8 / (\u03b8 + 1.0));
}

#line 4 "src/distribution/scalar_discrete/GeometricDistribution.birch"
template<class Arg>
#line 4 "src/distribution/scalar_discrete/GeometricDistribution.birch"
inline decltype(auto) Geometric(const Arg& \u03c1) {
  #line 5 "src/distribution/scalar_discrete/GeometricDistribution.birch"
  return NegativeBinomial(Integer(1), \u03c1);
}

#line 62 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2>
#line 62 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
inline decltype(auto) NegativeBinomial(const Arg1& k, const Arg2& \u03c1) {
  #line 63 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  return construct<NegativeBinomialDistribution<Arg1, Arg2>>(k, \u03c1);
}

#line 91 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 91 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
inline decltype(auto) logpdf_negative_binomial(const Arg1& x, const Arg2& k, const Arg3& \u03c1) {
  #line 92 "src/distribution/scalar_discrete/NegativeBinomialDistribution.birch"
  return k * log(\u03c1) + x * log1p(-(\u03c1)) + lchoose(x + k - Integer(1), x);
}

#line 55 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg>
#line 55 "src/distribution/scalar_discrete/PoissonDistribution.birch"
inline Distribution<Integer> Poisson(const Arg& \u03bb) {
  #line 56 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  return construct<PoissonDistribution<Arg>>(\u03bb);
}

#line 83 "src/distribution/scalar_discrete/PoissonDistribution.birch"
template<class Arg1,class Arg2>
#line 83 "src/distribution/scalar_discrete/PoissonDistribution.birch"
inline decltype(auto) logpdf_poisson(const Arg1& x, const Arg2& \u03bb) {
  #line 84 "src/distribution/scalar_discrete/PoissonDistribution.birch"
  return x * log(\u03bb) - \u03bb - lgamma(x + Integer(1));
}

#line 46 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 46 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
inline decltype(auto) RestaurantCategorical(const Arg1& \u03b1, const Arg2& \u03b8, const Arg3& n, const Arg4& N) {
  #line 48 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  return construct<RestaurantCategoricalDistribution<Arg1, Arg2, Arg3, Arg4>>(\u03b1, \u03b8, n, N);
}

#line 118 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 118 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
inline decltype(auto) update_restaurant_categorical(const Arg1& x, const Arg2& \u03b1, const Arg3& \u03b8, const Arg4& n, const Arg5& N) {
  auto K = length(n);
  #line 121 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  assert(x <= K + Integer(1));
  auto n_prime_ = vector_lambda([=](const Integer& i) -> Integer {
    #line 123 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
    return n(x) + (x == i);
  }, if_then_else(x == K + Integer(1), K + Integer(1), K));
  auto N_prime_ = N + Integer(1);
  #line 126 "src/distribution/scalar_discrete/RestaurantCategoricalDistribution.birch"
  return RestaurantCategorical(\u03b1, \u03b8, n_prime_, N_prime_);
}

#line 62 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 62 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
inline decltype(auto) ScaledGammaPoisson(const Arg1& a, const Arg2& k, const Arg3& \u03b8) {
  #line 63 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return construct<ScaledGammaPoissonDistribution<Arg1, Arg2, Arg3>>(a, k, \u03b8);
}

#line 69 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg>
#line 69 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
inline Distribution<Integer> Poisson(const Multiply<Arg, Random<Real>>& \u03bb) {
  #line 71 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  \u03bb->r->prune();
  #line 72 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  if (\u03bb->r->hasGamma()) {
    auto n4554_ = \u03bb->r->getGamma();
    auto k = std::get<0>(n4554_);
    auto \u03b8 = std::get<1>(n4554_);
    auto p = ScaledGammaPoisson(\u03bb->l, k, \u03b8);
    #line 75 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    \u03bb->r->graft(p);
    #line 76 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    return p;
  } else {
    #line 78 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    return construct<PoissonDistribution<Multiply<Arg, Random<Real>>>>(\u03bb);
  }
}

#line 85 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg>
#line 85 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
inline Distribution<Integer> Poisson(const Multiply<Random<Real>, Arg>& \u03bb) {
  #line 87 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  \u03bb->l->prune();
  #line 88 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  if (\u03bb->l->hasGamma()) {
    auto n4555_ = \u03bb->l->getGamma();
    auto k = std::get<0>(n4555_);
    auto \u03b8 = std::get<1>(n4555_);
    auto p = ScaledGammaPoisson(\u03bb->r, k, \u03b8);
    #line 91 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    \u03bb->l->graft(p);
    #line 92 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    return p;
  } else {
    #line 94 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
    return construct<PoissonDistribution<Multiply<Random<Real>, Arg>>>(\u03bb);
  }
}

#line 133 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 133 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
inline decltype(auto) update_scaled_gamma_poisson(const Arg1& x, const Arg2& a, const Arg3& k, const Arg4& \u03b8) {
  #line 135 "src/distribution/scalar_discrete/ScaledGammaPoissonDistribution.birch"
  return Gamma(k + x, \u03b8 / (a * \u03b8 + 1.0));
}

#line 58 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
template<class Arg1,class Arg2>
#line 58 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
inline decltype(auto) UniformInteger(const Arg1& l, const Arg2& u) {
  #line 59 "src/distribution/scalar_discrete/UniformIntegerDistribution.birch"
  return construct<UniformIntegerDistribution<Arg1, Arg2>>(l, u);
}

#line 113 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
template<class Arg>
#line 113 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
inline decltype(auto) Delta(const Add<Random<Integer>, Arg>& \u03bc) {
  auto r = handle_assume(Delta(\u03bc->r));
  #line 115 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  return Delta(\u03bc->l + r);
}

#line 121 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
template<class Arg>
#line 121 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
inline decltype(auto) Delta(const Add<Arg, Random<Integer>>& \u03bc) {
  auto l = handle_assume(Delta(\u03bc->l));
  #line 123 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  return Delta(l + \u03bc->r);
}

#line 129 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
template<class Arg1,class Arg2>
#line 129 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
inline decltype(auto) Delta(const Add<Arg1, Arg2>& \u03bc) {
  auto l = handle_assume(Delta(\u03bc->l));
  auto r = handle_assume(Delta(\u03bc->r));
  #line 132 "src/distribution/scalar_enumerate/AddDiscreteDeltaDistribution.birch"
  return Delta(l + r);
}

#line 38 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
template<class Arg>
#line 38 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
inline Distribution<Integer> Delta(const Arg& \u03bc) {
  #line 39 "src/distribution/scalar_enumerate/DeltaDistribution.birch"
  return construct<DeltaDistribution<Arg>>(\u03bc);
}

#line 62 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
template<class Arg>
#line 62 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
inline decltype(auto) Delta(const Negate<Arg>& \u03bc) {
  auto m = handle_assume(Delta(\u03bc->m));
  #line 64 "src/distribution/scalar_enumerate/NegateDiscreteDeltaDistribution.birch"
  return Delta(-(m));
}

#line 111 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
template<class Arg>
#line 111 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
inline decltype(auto) Delta(const Subtract<Random<Integer>, Arg>& \u03bc) {
  auto r = handle_assume(Delta(\u03bc->r));
  #line 113 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  return Delta(\u03bc->l - r);
}

#line 119 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
template<class Arg>
#line 119 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
inline decltype(auto) Delta(const Subtract<Arg, Random<Integer>>& \u03bc) {
  auto l = handle_assume(Delta(\u03bc->l));
  #line 121 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  return Delta(l - \u03bc->r);
}

#line 127 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
template<class Arg1,class Arg2>
#line 127 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
inline decltype(auto) Delta(const Subtract<Arg1, Arg2>& \u03bc) {
  auto l = handle_assume(Delta(\u03bc->l));
  auto r = handle_assume(Delta(\u03bc->r));
  #line 130 "src/distribution/scalar_enumerate/SubtractDiscreteDeltaDistribution.birch"
  return Delta(l - r);
}

#line 65 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 65 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
inline decltype(auto) Gaussian(const Arg1& \u03bc, const Arg2& \u03c32) {
  #line 66 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return construct<GaussianDistribution<Arg1, Arg2>>(\u03bc, \u03c32);
}

#line 95 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 95 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
inline decltype(auto) logpdf_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03c32) {
  #line 96 "src/distribution/scalar_gaussian/GaussianDistribution.birch"
  return -(0.5) * (pow(x - \u03bc, 2.0) / \u03c32 + log(2.0 * \u03c0 * \u03c32));
}

#line 66 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 66 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
inline decltype(auto) GaussianGaussian(const Arg1& \u03bc, const Arg2& \u03c32, const Arg3& s2) {
  #line 67 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return construct<GaussianGaussianDistribution<Arg1, Arg2, Arg3>>(\u03bc, \u03c32, s2);
}

#line 73 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg>
#line 73 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Random<Real>& \u03bc, const Arg& \u03c32) {
  #line 74 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  \u03bc->prune();
  #line 75 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  if (\u03bc->hasGaussian()) {
    auto n4556_ = \u03bc->getGaussian();
    auto m = std::get<0>(n4556_);
    auto s2 = std::get<1>(n4556_);
    auto p = GaussianGaussian(m, s2, \u03c32);
    #line 78 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
    \u03bc->graft(p);
    #line 79 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
    return p;
  } else {
    #line 81 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
    return construct<GaussianDistribution<Random<Real>, Arg>>(\u03bc, \u03c32);
  }
}

#line 107 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 107 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
inline decltype(auto) logpdf_gaussian_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03c32, const Arg4& s2) {
  #line 109 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return logpdf_gaussian(x, \u03bc, \u03c32 + s2);
}

#line 152 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 152 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
inline decltype(auto) update_gaussian_gaussian(const Arg1& x, const Arg2& \u03bc, const Arg3& \u03c32, const Arg4& s2) {
  auto k_prime_ = \u03c32 / (\u03c32 + s2);
  auto \u03bc_prime_ = \u03bc + k_prime_ * (x - \u03bc);
  auto \u03c32_prime_ = \u03c32 - k_prime_ * \u03c32;
  #line 157 "src/distribution/scalar_gaussian/GaussianGaussianDistribution.birch"
  return Gaussian(\u03bc_prime_, \u03c32_prime_);
}

#line 82 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 82 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
inline decltype(auto) LinearGaussianGaussian(const Arg1& a, const Arg2& \u03bc, const Arg3& \u03c32, const Arg4& c, const Arg5& s2) {
  #line 84 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return construct<LinearGaussianGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5>>(a, \u03bc, \u03c32, c, s2);
}

#line 91 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 91 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Add<Multiply<Arg1, Random<Real>>, Arg2>& \u03bc, const Arg3& \u03c32) {
  #line 93 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  \u03bc->l->r->prune();
  #line 94 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  if (\u03bc->l->r->hasGaussian()) {
    auto n4557_ = \u03bc->l->r->getGaussian();
    auto m = std::get<0>(n4557_);
    auto s2 = std::get<1>(n4557_);
    auto p = LinearGaussianGaussian(\u03bc->l->l, m, s2, \u03bc->r, \u03c32);
    #line 97 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
    \u03bc->l->r->graft(p);
    #line 98 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
    return p;
  } else {
    #line 100 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
    return construct<GaussianDistribution<Add<Multiply<Arg1, Random<Real>>, Arg2>, Arg3>>(\u03bc, \u03c32);
  }
}

#line 108 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 108 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Multiply<Arg1, Random<Real>>& \u03bc, const Arg2& \u03c32) {
  #line 110 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return Gaussian(\u03bc + 0.0, \u03c32);
}

#line 141 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 141 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
inline decltype(auto) logpdf_linear_gaussian_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bc, const Arg4& \u03c32, const Arg5& c, const Arg6& s2) {
  #line 143 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return logpdf_gaussian(x, a * \u03bc + c, a * a * \u03c32 + s2);
}

#line 195 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 195 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
inline decltype(auto) update_linear_gaussian_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bc, const Arg4& \u03c32, const Arg5& c, const Arg6& s2) {
  auto k_prime_ = \u03c32 * a / (a * a * \u03c32 + s2);
  auto \u03bc_prime_ = \u03bc + k_prime_ * (x - a * \u03bc - c);
  auto \u03c32_prime_ = \u03c32 - k_prime_ * a * \u03c32;
  #line 200 "src/distribution/scalar_gaussian/LinearGaussianGaussianDistribution.birch"
  return Gaussian(\u03bc_prime_, \u03c32_prime_);
}

#line 83 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 83 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
inline decltype(auto) LinearMultivariateGaussianGaussian(const Arg1& a, const Arg2& \u03bc, const Arg3& \u03a3, const Arg4& c, const Arg5& s2) {
  #line 85 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return construct<LinearMultivariateGaussianGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5>>(a, \u03bc, \u03a3, c, s2);
}

#line 92 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 92 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Arg3& \u03c32) {
  #line 94 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  \u03bc->l->r->prune();
  #line 95 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  if (\u03bc->l->r->hasMultivariateGaussian()) {
    auto n4558_ = \u03bc->l->r->getMultivariateGaussian();
    auto m = std::get<0>(n4558_);
    auto S = std::get<1>(n4558_);
    auto p = LinearMultivariateGaussianGaussian(\u03bc->l->l, m, S, \u03bc->r, \u03c32);
    #line 98 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
    \u03bc->l->r->graft(p);
    #line 99 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
    return p;
  } else {
    #line 101 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
    return construct<GaussianDistribution<Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>, Arg3>>(\u03bc, \u03c32);
  }
}

#line 109 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 109 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Arg2& \u03c32) {
  #line 111 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return Gaussian(\u03bc + 0.0, \u03c32);
}

#line 142 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 142 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
inline decltype(auto) logpdf_linear_multivariate_gaussian_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bc, const Arg4& \u03a3, const Arg5& c, const Arg6& s2) {
  #line 144 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return logpdf_gaussian(x, dot(a, \u03bc) + c, dot(a, \u03a3 * a) + s2);
}

#line 197 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 197 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
inline decltype(auto) update_linear_multivariate_gaussian_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bc, const Arg4& \u03a3, const Arg5& c, const Arg6& s2) {
  auto k_prime_ = \u03a3 * a / (dot(a, \u03a3 * a) + s2);
  auto \u03bc_prime_ = \u03bc + k_prime_ * (x - dot(a, \u03bc) - c);
  auto \u03a3_prime_ = \u03a3 - outer(k_prime_, a) * \u03a3;
  #line 202 "src/distribution/scalar_gaussian/LinearMultivariateGaussianGaussianDistribution.birch"
  return MultivariateGaussian(\u03bc_prime_, \u03a3_prime_);
}

#line 92 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 92 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline decltype(auto) LinearMultivariateNormalInverseGammaGaussian(const Arg1& a, const Arg2& \u03bd, const Arg3& \u039b, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& s2) {
  #line 95 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return construct<LinearMultivariateNormalInverseGammaGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>>(a, \u03bd, \u039b, \u03b1, \u03b2, c, s2);
}

#line 102 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 102 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Random<Real>& \u03c32) {
  #line 104 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc, 1.0 * \u03c32);
}

#line 110 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 110 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Multiply<Random<Real>, Arg3>& \u03c32) {
  #line 112 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc, \u03c32->r * \u03c32->l);
}

#line 118 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 118 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Multiply<Arg3, Random<Real>>& \u03c32) {
  #line 120 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  \u03bc->l->r->prune();
  #line 121 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  if (\u03bc->l->r->hasMultivariateNormalInverseGamma() && \u03bc->l->r->isGrandChildOf(\u03c32->r)) {
    auto n4559_ = \u03bc->l->r->getMultivariateNormalInverseGamma();
    auto \u03bd = std::get<0>(n4559_);
    auto \u039b = std::get<1>(n4559_);
    auto \u03b1 = std::get<2>(n4559_);
    auto \u03b2 = std::get<3>(n4559_);
    auto p = LinearMultivariateNormalInverseGammaGaussian(\u03bc->l->l, \u03bd, \u039b, \u03b1, \u03b2, \u03bc->r, \u03c32->l);
    #line 125 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u03bc->l->r->graft(p, Integer(1));
    #line 126 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u03c32->r->graftReplace(p, Integer(2));
    #line 127 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u03bc->l->r->join(\u03c32->r);
    #line 128 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    return p;
  } else {
    #line 130 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u03c32->r->prune();
    #line 131 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    if (\u03c32->r->hasInverseGamma()) {
      auto n4560_ = \u03c32->r->getInverseGamma();
      auto \u03b1 = std::get<0>(n4560_);
      auto \u03b2 = std::get<1>(n4560_);
      auto p = NormalInverseGamma(\u03bc / \u03c32->l, 1.0 / \u03c32->l, \u03b1, \u03b2);
      #line 134 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
      \u03c32->r->graft(p);
      #line 135 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
      return p;
    } else {
      #line 136 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
      if (\u03bc->l->r->hasMultivariateGaussian()) {
        auto n4561_ = \u03bc->l->r->getMultivariateGaussian();
        auto m = std::get<0>(n4561_);
        auto S = std::get<1>(n4561_);
        auto p = LinearMultivariateGaussianGaussian(\u03bc->l->l, m, S, \u03bc->r, \u03c32);
        #line 139 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
        \u03bc->l->r->graft(p);
        #line 140 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
        return p;
      } else {
        #line 142 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
        return construct<GaussianDistribution<Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>, Multiply<Arg3, Random<Real>>>>(\u03bc, \u03c32);
      }
    }
  }
}

#line 151 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 151 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32) {
  #line 153 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  \u03bc->l->r->prune();
  #line 154 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  if (\u03bc->l->r->hasMultivariateNormalInverseGamma() && \u03bc->l->r->isGrandChildOf(\u03c32->r)) {
    auto n4562_ = \u03bc->l->r->getMultivariateNormalInverseGamma();
    auto \u03bd = std::get<0>(n4562_);
    auto \u039b = std::get<1>(n4562_);
    auto \u03b1 = std::get<2>(n4562_);
    auto \u03b2 = std::get<3>(n4562_);
    auto p = LinearMultivariateNormalInverseGammaGaussian(\u03bc->l->l, \u03bd, \u039b, \u03b1, \u03b2, \u03bc->r, \u03c32->l);
    #line 158 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u03bc->l->r->graft(p, Integer(1));
    #line 159 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u03c32->r->graftReplace(p, Integer(2));
    #line 160 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    \u03bc->l->r->join(\u03c32->r);
    #line 161 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    return p;
  } else {
    #line 162 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
    if (\u03bc->l->r->hasMultivariateNormalInverseGamma() && \u03bc->l->r->isGrandChildOf(\u03c32->l)) {
      auto n4563_ = \u03bc->l->r->getMultivariateNormalInverseGamma();
      auto \u03bd = std::get<0>(n4563_);
      auto \u039b = std::get<1>(n4563_);
      auto \u03b1 = std::get<2>(n4563_);
      auto \u03b2 = std::get<3>(n4563_);
      auto p = LinearMultivariateNormalInverseGammaGaussian(\u03bc->l->l, \u03bd, \u039b, \u03b1, \u03b2, \u03bc->r, \u03c32->r);
      #line 167 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
      \u03bc->l->r->graft(p, Integer(1));
      #line 168 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
      \u03c32->l->graftReplace(p, Integer(2));
      #line 169 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
      \u03bc->l->r->join(\u03c32->l);
      #line 170 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
      return p;
    } else {
      #line 172 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
      \u03c32->l->prune();
      #line 173 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
      \u03c32->r->prune();
      #line 174 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
      if (\u03c32->l->hasInverseGamma()) {
        auto n4564_ = \u03c32->l->getInverseGamma();
        auto \u03b1 = std::get<0>(n4564_);
        auto \u03b2 = std::get<1>(n4564_);
        auto p = NormalInverseGamma(\u03bc / \u03c32->r, 1.0 / \u03c32->r, \u03b1, \u03b2);
        #line 177 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
        \u03c32->l->graft(p);
        #line 178 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
        return p;
      } else {
        #line 179 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
        if (\u03c32->r->hasInverseGamma()) {
          auto n4565_ = \u03c32->r->getInverseGamma();
          auto \u03b1 = std::get<0>(n4565_);
          auto \u03b2 = std::get<1>(n4565_);
          auto p = NormalInverseGamma(\u03bc / \u03c32->l, 1.0 / \u03c32->l, \u03b1, \u03b2);
          #line 182 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
          \u03c32->r->graft(p);
          #line 183 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
          return p;
        } else {
          #line 184 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
          if (\u03bc->l->r->hasMultivariateGaussian()) {
            auto n4566_ = \u03bc->l->r->getMultivariateGaussian();
            auto m = std::get<0>(n4566_);
            auto S = std::get<1>(n4566_);
            auto p = LinearMultivariateGaussianGaussian(\u03bc->l->l, m, S, \u03bc->r, \u03c32);
            #line 187 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
            \u03bc->l->r->graft(p);
            #line 188 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
            return p;
          } else {
            #line 190 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
            return construct<GaussianDistribution<Add<Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>, Arg2>, Multiply<Random<Real>, Random<Real>>>>(\u03bc, \u03c32);
          }
        }
      }
    }
  }
}

#line 199 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 199 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Dot<Arg, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Random<Real>& \u03c32) {
  #line 201 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc + 0.0, \u03c32);
}

#line 207 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 207 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Multiply<Arg2, Random<Real>>& \u03c32) {
  #line 209 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc + 0.0, \u03c32);
}

#line 215 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 215 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Dot<Arg1, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Multiply<Random<Real>, Arg2>& \u03c32) {
  #line 217 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc + 0.0, \u03c32);
}

#line 223 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 223 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Dot<Arg, Random<libbirch::DefaultArray<Real,1>>>& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32) {
  #line 225 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc + 0.0, \u03c32);
}

#line 261 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7,class Arg8>
#line 261 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline decltype(auto) logpdf_linear_multivariate_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bd, const Arg4& \u039b, const Arg5& c, const Arg6& \u03b1, const Arg7& \u03b2, const Arg8& s2) {
  auto S = cholinv(\u039b);
  auto \u03bc = S * \u03bd;
  #line 266 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return logpdf_student_t(x, 2.0 * \u03b1, dot(a, \u03bc) + c, (\u03b2 / \u03b1) * (s2 + dot(a, S * a)));
}

#line 284 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7,class Arg8>
#line 284 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
inline decltype(auto) update_linear_multivariate_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bd, const Arg4& \u039b, const Arg5& c, const Arg6& \u03b1, const Arg7& \u03b2, const Arg8& s2) {
  auto l = 1.0 / s2;
  auto y = x - c;
  auto z = l * y;
  auto \u039b_prime_ = \u039b + outer(a, l * a);
  auto \u03bd_prime_ = \u03bd + a * z;
  auto \u03bc = cholsolve(\u039b, \u03bd);
  auto \u03bc_prime_ = cholsolve(\u039b_prime_, \u03bd_prime_);
  auto \u03b1_prime_ = \u03b1 + 0.5;
  auto \u03b2_prime_ = \u03b2 + 0.5 * (y * z + dot(\u03bc, \u03bd) - dot(\u03bc_prime_, \u03bd_prime_));
  #line 296 "src/distribution/scalar_normal_inverse_gamma/LinearMultivariateNormalInverseGammaGaussianDistribution.birch"
  return MultivariateNormalInverseGamma(\u03bd_prime_, \u039b_prime_, \u03b1_prime_, \u03b2_prime_);
}

#line 87 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7>
#line 87 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline decltype(auto) LinearNormalInverseGammaGaussian(const Arg1& a, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& c, const Arg7& s2) {
  #line 89 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return construct<LinearNormalInverseGammaGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Arg7>>(a, \u03bd, \u03bb, \u03b1, \u03b2, c, s2);
}

#line 96 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 96 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Add<Multiply<Arg1, Random<Real>>, Arg2>& \u03bc, const Random<Real>& \u03c32) {
  #line 98 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc, 1.0 * \u03c32);
}

#line 104 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 104 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Add<Multiply<Arg1, Random<Real>>, Arg2>& \u03bc, const Multiply<Random<Real>, Arg3>& \u03c32) {
  #line 106 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc, \u03c32->r * \u03c32->l);
}

#line 112 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 112 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Add<Multiply<Arg1, Random<Real>>, Arg2>& \u03bc, const Multiply<Arg3, Random<Real>>& \u03c32) {
  #line 114 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  \u03bc->l->r->prune();
  #line 115 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  if (\u03bc->l->r->hasNormalInverseGamma() && \u03bc->l->r->isGrandChildOf(\u03c32->r)) {
    auto n4567_ = \u03bc->l->r->getNormalInverseGamma();
    auto \u03bd = std::get<0>(n4567_);
    auto \u03bb = std::get<1>(n4567_);
    auto \u03b1 = std::get<2>(n4567_);
    auto \u03b2 = std::get<3>(n4567_);
    auto p = LinearNormalInverseGammaGaussian(\u03bc->l->l, \u03bd, \u03bb, \u03b1, \u03b2, \u03bc->r, \u03c32->l);
    #line 118 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03bc->l->r->graft(p, Integer(1));
    #line 119 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03c32->r->graftReplace(p, Integer(2));
    #line 120 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03bc->l->r->join(\u03c32->r);
    #line 121 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    return p;
  } else {
    #line 123 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03c32->r->prune();
    #line 124 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    if (\u03c32->r->hasInverseGamma()) {
      auto n4568_ = \u03c32->r->getInverseGamma();
      auto \u03b1 = std::get<0>(n4568_);
      auto \u03b2 = std::get<1>(n4568_);
      auto p = NormalInverseGamma(\u03bc / \u03c32->l, 1.0 / \u03c32->l, \u03b1, \u03b2);
      #line 127 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
      \u03c32->r->graft(p);
      #line 128 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
      return p;
    } else {
      #line 129 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
      if (\u03bc->l->r->hasGaussian()) {
        auto n4569_ = \u03bc->l->r->getGaussian();
        auto m = std::get<0>(n4569_);
        auto s2 = std::get<1>(n4569_);
        auto p = LinearGaussianGaussian(\u03bc->l->l, m, s2, \u03bc->r, \u03c32);
        #line 132 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
        \u03bc->l->r->graft(p);
        #line 133 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
        return p;
      } else {
        #line 135 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
        return construct<GaussianDistribution<Add<Multiply<Arg1, Random<Real>>, Arg2>, Multiply<Arg3, Random<Real>>>>(\u03bc, \u03c32);
      }
    }
  }
}

#line 144 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 144 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Add<Multiply<Arg1, Random<Real>>, Arg2>& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32) {
  #line 146 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  \u03bc->l->r->prune();
  #line 147 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  if (\u03bc->l->r->hasNormalInverseGamma() && \u03bc->l->r->isGrandChildOf(\u03c32->r)) {
    auto n4570_ = \u03bc->l->r->getNormalInverseGamma();
    auto \u03bd = std::get<0>(n4570_);
    auto \u03bb = std::get<1>(n4570_);
    auto \u03b1 = std::get<2>(n4570_);
    auto \u03b2 = std::get<3>(n4570_);
    auto p = LinearNormalInverseGammaGaussian(\u03bc->l->l, \u03bd, \u03bb, \u03b1, \u03b2, \u03bc->r, \u03c32->l);
    #line 150 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03bc->l->r->graft(p, Integer(1));
    #line 151 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03c32->r->graftReplace(p, Integer(2));
    #line 152 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    \u03bc->l->r->join(\u03c32->r);
    #line 153 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    return p;
  } else {
    #line 154 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
    if (\u03bc->l->r->hasNormalInverseGamma() && \u03bc->l->r->isGrandChildOf(\u03c32->l)) {
      auto n4571_ = \u03bc->l->r->getNormalInverseGamma();
      auto \u03bd = std::get<0>(n4571_);
      auto \u03bb = std::get<1>(n4571_);
      auto \u03b1 = std::get<2>(n4571_);
      auto \u03b2 = std::get<3>(n4571_);
      auto p = LinearNormalInverseGammaGaussian(\u03bc->l->l, \u03bd, \u03bb, \u03b1, \u03b2, \u03bc->r, \u03c32->r);
      #line 157 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
      \u03bc->l->r->graft(p, Integer(1));
      #line 158 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
      \u03c32->l->graftReplace(p, Integer(2));
      #line 159 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
      \u03bc->l->r->join(\u03c32->l);
      #line 160 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
      return p;
    } else {
      #line 162 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
      \u03c32->l->prune();
      #line 163 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
      \u03c32->r->prune();
      #line 164 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
      if (\u03c32->l->hasInverseGamma()) {
        auto n4572_ = \u03c32->l->getInverseGamma();
        auto \u03b1 = std::get<0>(n4572_);
        auto \u03b2 = std::get<1>(n4572_);
        auto p = NormalInverseGamma(\u03bc / \u03c32->r, 1.0 / \u03c32->r, \u03b1, \u03b2);
        #line 167 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
        \u03c32->l->graft(p);
        #line 168 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
        return p;
      } else {
        #line 169 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
        if (\u03c32->r->hasInverseGamma()) {
          auto n4573_ = \u03c32->r->getInverseGamma();
          auto \u03b1 = std::get<0>(n4573_);
          auto \u03b2 = std::get<1>(n4573_);
          auto p = NormalInverseGamma(\u03bc / \u03c32->l, 1.0 / \u03c32->l, \u03b1, \u03b2);
          #line 172 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
          \u03c32->r->graft(p);
          #line 173 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
          return p;
        } else {
          #line 174 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
          if (\u03bc->l->r->hasGaussian()) {
            auto n4574_ = \u03bc->l->r->getGaussian();
            auto m = std::get<0>(n4574_);
            auto s2 = std::get<1>(n4574_);
            auto p = LinearGaussianGaussian(\u03bc->l->l, m, s2, \u03bc->r, \u03c32);
            #line 177 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
            \u03bc->l->r->graft(p);
            #line 178 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
            return p;
          } else {
            #line 180 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
            return construct<GaussianDistribution<Add<Multiply<Arg1, Random<Real>>, Arg2>, Multiply<Random<Real>, Random<Real>>>>(\u03bc, \u03c32);
          }
        }
      }
    }
  }
}

#line 189 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 189 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Multiply<Arg, Random<Real>>& \u03bc, const Random<Real>& \u03c32) {
  #line 191 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc + 0.0, \u03c32);
}

#line 197 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 197 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Multiply<Arg1, Random<Real>>& \u03bc, const Multiply<Arg2, Random<Real>>& \u03c32) {
  #line 199 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc + 0.0, \u03c32);
}

#line 205 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2>
#line 205 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Multiply<Arg1, Random<Real>>& \u03bc, const Multiply<Random<Real>, Arg2>& \u03c32) {
  #line 207 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc + 0.0, \u03c32);
}

#line 213 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 213 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Multiply<Arg, Random<Real>>& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32) {
  #line 215 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc + 0.0, \u03c32);
}

#line 249 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7,class Arg8>
#line 249 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline decltype(auto) logpdf_linear_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bd, const Arg4& \u03bb, const Arg5& \u03b1, const Arg6& \u03b2, const Arg7& c, const Arg8& s2) {
  #line 252 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return logpdf_student_t(x, 2.0 * \u03b1, a * \u03bd / \u03bb + c, (\u03b2 / \u03b1) * (s2 + a * a / \u03bb));
}

#line 270 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6,class Arg7,class Arg8>
#line 270 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
inline decltype(auto) update_linear_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& a, const Arg3& \u03bd, const Arg4& \u03bb, const Arg5& \u03b1, const Arg6& \u03b2, const Arg7& c, const Arg8& s2) {
  auto l = 1.0 / s2;
  auto y = x - c;
  auto z = l * y;
  auto \u03bb_prime_ = \u03bb + a * l * a;
  auto \u03bd_prime_ = \u03bd + a * z;
  auto \u03b1_prime_ = \u03b1 + 0.5;
  auto \u03b2_prime_ = \u03b2 + 0.5 * (y * z + \u03bd * \u03bd / \u03bb - \u03bd_prime_ * \u03bd_prime_ / \u03bb_prime_);
  #line 280 "src/distribution/scalar_normal_inverse_gamma/LinearNormalInverseGammaGaussianDistribution.birch"
  return NormalInverseGamma(\u03bd_prime_, \u03bb_prime_, \u03b1_prime_, \u03b2_prime_);
}

#line 109 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 109 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
inline decltype(auto) NormalInverseGamma(const Arg1& \u03bd, const Arg2& \u03bb, const Arg3& \u03b1, const Arg4& \u03b2) {
  #line 111 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return construct<NormalInverseGammaDistribution<Arg1, Arg2, Arg3, Arg4>>(\u03bd, \u03bb, \u03b1, \u03b2);
}

#line 118 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg>
#line 118 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
inline Distribution<Real> Gaussian(const Arg& \u03bc, const Random<Real>& \u03c32) {
  #line 119 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  \u03c32->prune();
  #line 120 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  if (\u03c32->hasInverseGamma()) {
    auto n4575_ = \u03c32->getInverseGamma();
    auto \u03b1 = std::get<0>(n4575_);
    auto \u03b2 = std::get<1>(n4575_);
    auto p = NormalInverseGamma(\u03bc, 1.0, \u03b1, \u03b2);
    #line 123 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    \u03c32->graft(p);
    #line 124 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    return p;
  } else {
    #line 126 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    return construct<GaussianDistribution<Arg, Random<Real>>>(\u03bc, \u03c32);
  }
}

#line 133 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 133 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
inline Distribution<Real> Gaussian(const Arg1& \u03bc, const Multiply<Arg2, Random<Real>>& \u03c32) {
  #line 135 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  \u03c32->r->prune();
  #line 136 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  if (\u03c32->r->hasInverseGamma()) {
    auto n4576_ = \u03c32->r->getInverseGamma();
    auto \u03b1 = std::get<0>(n4576_);
    auto \u03b2 = std::get<1>(n4576_);
    auto p = NormalInverseGamma(\u03bc / \u03c32->l, 1.0 / \u03c32->l, \u03b1, \u03b2);
    #line 139 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    \u03c32->r->graft(p);
    #line 140 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    return p;
  } else {
    #line 142 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    return construct<GaussianDistribution<Arg1, Multiply<Arg2, Random<Real>>>>(\u03bc, \u03c32);
  }
}

#line 150 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 150 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
inline Distribution<Real> Gaussian(const Arg1& \u03bc, const Multiply<Random<Real>, Arg2>& \u03c32) {
  #line 152 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  \u03c32->l->prune();
  #line 153 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  if (\u03c32->l->hasInverseGamma()) {
    auto n4577_ = \u03c32->l->getInverseGamma();
    auto \u03b1 = std::get<0>(n4577_);
    auto \u03b2 = std::get<1>(n4577_);
    auto p = NormalInverseGamma(\u03bc / \u03c32->r, 1.0 / \u03c32->r, \u03b1, \u03b2);
    #line 156 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    \u03c32->l->graft(p);
    #line 157 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    return p;
  } else {
    #line 159 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    return construct<GaussianDistribution<Arg1, Multiply<Random<Real>, Arg2>>>(\u03bc, \u03c32);
  }
}

#line 167 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg>
#line 167 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
inline Distribution<Real> Gaussian(const Arg& \u03bc, const Multiply<Random<Real>, Random<Real>>& \u03c32) {
  #line 169 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  \u03c32->l->prune();
  #line 170 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  \u03c32->r->prune();
  #line 171 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  if (\u03c32->l->hasInverseGamma()) {
    auto n4578_ = \u03c32->l->getInverseGamma();
    auto \u03b1 = std::get<0>(n4578_);
    auto \u03b2 = std::get<1>(n4578_);
    auto p = NormalInverseGamma(\u03bc / \u03c32->r, 1.0 / \u03c32->r, \u03b1, \u03b2);
    #line 174 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    \u03c32->l->graft(p);
    #line 175 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    return p;
  } else {
    #line 176 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
    if (\u03c32->r->hasInverseGamma()) {
      auto n4579_ = \u03c32->r->getInverseGamma();
      auto \u03b1 = std::get<0>(n4579_);
      auto \u03b2 = std::get<1>(n4579_);
      auto p = NormalInverseGamma(\u03bc / \u03c32->l, 1.0 / \u03c32->l, \u03b1, \u03b2);
      #line 179 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
      \u03c32->r->graft(p);
      #line 180 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
      return p;
    } else {
      #line 182 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
      return construct<GaussianDistribution<Arg, Multiply<Random<Real>, Random<Real>>>>(\u03bc, \u03c32);
    }
  }
}

#line 211 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 211 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
inline decltype(auto) logpdf_normal_inverse_gamma(const Arg1& x, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2) {
  #line 213 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return logpdf_student_t(x, 2.0 * \u03b1, \u03bd / \u03bb, \u03b2 / (\u03b1 * \u03bb));
}

#line 228 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 228 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
inline decltype(auto) update_normal_inverse_gamma(const Arg1& x, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2) {
  #line 230 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaDistribution.birch"
  return InverseGamma(\u03b1 + 0.5, \u03b2 + 0.5 * pow(x - \u03bd / \u03bb, 2.0) * \u03bb);
}

#line 74 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
#line 74 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
inline decltype(auto) NormalInverseGammaGaussian(const Arg1& \u03bd, const Arg2& \u03bb, const Arg3& \u03b1, const Arg4& \u03b2, const Arg5& s2) {
  #line 76 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return construct<NormalInverseGammaGaussianDistribution<Arg1, Arg2, Arg3, Arg4, Arg5>>(\u03bd, \u03bb, \u03b1, \u03b2, s2);
}

#line 90 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 90 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Random<Real>& \u03bc, const Multiply<Random<Real>, Arg>& \u03c32) {
  #line 92 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return Gaussian(\u03bc, \u03c32->r * \u03c32->l);
}

#line 98 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg>
#line 98 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
inline Distribution<Real> Gaussian(const Random<Real>& \u03bc, const Multiply<Arg, Random<Real>>& \u03c32) {
  #line 100 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  \u03bc->prune();
  #line 101 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  if (\u03bc->hasNormalInverseGamma() && \u03bc->isGrandChildOf(\u03c32->r)) {
    auto n4580_ = \u03bc->getNormalInverseGamma();
    auto \u03bd = std::get<0>(n4580_);
    auto \u03bb = std::get<1>(n4580_);
    auto \u03b1 = std::get<2>(n4580_);
    auto \u03b2 = std::get<3>(n4580_);
    auto p = NormalInverseGammaGaussian(\u03bd, \u03bb, \u03b1, \u03b2, \u03c32->l);
    #line 104 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    \u03bc->graft(p, Integer(1));
    #line 105 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    \u03c32->r->graftReplace(p, Integer(2));
    #line 106 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    \u03bc->join(\u03c32->r);
    #line 107 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    return p;
  } else {
    #line 109 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    \u03c32->r->prune();
    #line 110 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
    if (\u03c32->r->hasInverseGamma()) {
      auto n4581_ = \u03c32->r->getInverseGamma();
      auto \u03b1 = std::get<0>(n4581_);
      auto \u03b2 = std::get<1>(n4581_);
      auto p = NormalInverseGamma(\u03bc / \u03c32->l, 1.0 / \u03c32->l, \u03b1, \u03b2);
      #line 113 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
      \u03c32->r->graft(p);
      #line 114 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
      return p;
    } else {
      #line 115 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
      if (\u03bc->hasGaussian()) {
        auto n4582_ = \u03bc->getGaussian();
        auto m = std::get<0>(n4582_);
        auto s2 = std::get<1>(n4582_);
        auto p = GaussianGaussian(m, s2, \u03c32);
        #line 118 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
        \u03bc->graft(p);
        #line 119 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
        return p;
      } else {
        #line 121 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
        return construct<GaussianDistribution<Random<Real>, Multiply<Arg, Random<Real>>>>(\u03bc, \u03c32);
      }
    }
  }
}

#line 198 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 198 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
inline decltype(auto) logpdf_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& s2) {
  #line 200 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return logpdf_student_t(x, 2.0 * \u03b1, \u03bd / \u03bb, (\u03b2 / \u03b1) * (s2 + 1.0 / \u03bb));
}

#line 216 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5,class Arg6>
#line 216 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
inline decltype(auto) update_normal_inverse_gamma_gaussian(const Arg1& x, const Arg2& \u03bd, const Arg3& \u03bb, const Arg4& \u03b1, const Arg5& \u03b2, const Arg6& s2) {
  auto l = 1.0 / s2;
  auto z = l * x;
  auto \u03bb_prime_ = \u03bb + l;
  auto \u03bd_prime_ = \u03bd + z;
  auto \u03b1_prime_ = \u03b1 + 0.5;
  auto \u03b2_prime_ = \u03b2 + 0.5 * (x * z + \u03bd * \u03bd / \u03bb - \u03bd_prime_ * \u03bd_prime_ / \u03bb_prime_);
  #line 224 "src/distribution/scalar_normal_inverse_gamma/NormalInverseGammaGaussianDistribution.birch"
  return NormalInverseGamma(\u03bd_prime_, \u03bb_prime_, \u03b1_prime_, \u03b2_prime_);
}

#line 73 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2>
#line 73 "src/distribution/scalar_other/BetaDistribution.birch"
inline decltype(auto) Beta(const Arg1& \u03b1, const Arg2& \u03b2) {
  #line 74 "src/distribution/scalar_other/BetaDistribution.birch"
  return construct<BetaDistribution<Arg1, Arg2>>(\u03b1, \u03b2);
}

#line 100 "src/distribution/scalar_other/BetaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 100 "src/distribution/scalar_other/BetaDistribution.birch"
inline decltype(auto) logpdf_beta(const Arg1& x, const Arg2& \u03b1, const Arg3& \u03b2) {
  #line 101 "src/distribution/scalar_other/BetaDistribution.birch"
  return (\u03b1 - 1.0) * log(x) + (\u03b2 - 1.0) * log1p(-(x)) - lbeta(\u03b1, \u03b2);
}

#line 25 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
template<class Arg1,class Arg2>
#line 25 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
inline decltype(auto) logpdf_chi_squared(const Arg1& x, const Arg2& \u03bd) {
  auto k = 0.5 * \u03bd;
  #line 27 "src/distribution/scalar_other/ChiSquaredDistribution.birch"
  return (k - 1.0) * log(x) - 0.5 * x - lgamma(k) - k * log(2.0);
}

#line 55 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg>
#line 55 "src/distribution/scalar_other/ExponentialDistribution.birch"
inline decltype(auto) Exponential(const Arg& \u03bb) {
  #line 56 "src/distribution/scalar_other/ExponentialDistribution.birch"
  return construct<ExponentialDistribution<Arg>>(\u03bb);
}

#line 79 "src/distribution/scalar_other/ExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 79 "src/distribution/scalar_other/ExponentialDistribution.birch"
inline decltype(auto) logpdf_exponential(const Arg1& x, const Arg2& \u03bb) {
  #line 80 "src/distribution/scalar_other/ExponentialDistribution.birch"
  return if_then_else(x < 0.0, -(std::numeric_limits<double>::infinity()), log(\u03bb) - \u03bb * x);
}

#line 69 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2>
#line 69 "src/distribution/scalar_other/GammaDistribution.birch"
inline decltype(auto) Gamma(const Arg1& k, const Arg2& \u03b8) {
  #line 70 "src/distribution/scalar_other/GammaDistribution.birch"
  return construct<GammaDistribution<Arg1, Arg2>>(k, \u03b8);
}

#line 96 "src/distribution/scalar_other/GammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 96 "src/distribution/scalar_other/GammaDistribution.birch"
inline decltype(auto) logpdf_gamma(const Arg1& x, const Arg2& k, const Arg3& \u03b8) {
  #line 97 "src/distribution/scalar_other/GammaDistribution.birch"
  return if_then_else(x < 0.0, -(std::numeric_limits<double>::infinity()), (k - 1.0) * log(x) - x / \u03b8 - lgamma(k) - k * log(\u03b8));
}

#line 60 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2>
#line 60 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
inline decltype(auto) GammaExponential(const Arg1& k, const Arg2& \u03b8) {
  #line 61 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return construct<GammaExponentialDistribution<Arg1, Arg2>>(k, \u03b8);
}

#line 101 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 101 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
inline decltype(auto) logpdf_lomax(const Arg1& x, const Arg2& \u03bb, const Arg3& \u03b1) {
  #line 102 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return if_then_else(x < 0.0, -(std::numeric_limits<double>::infinity()), log(\u03b1) - log(\u03bb) - (\u03b1 + 1.0) * log1p(x / \u03bb));
}

#line 116 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 116 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
inline decltype(auto) update_gamma_exponential(const Arg1& x, const Arg2& k, const Arg3& \u03b8) {
  #line 117 "src/distribution/scalar_other/GammaExponentialDistribution.birch"
  return Gamma(k + 1.0, \u03b8 / (1.0 + x * \u03b8));
}

#line 69 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2>
#line 69 "src/distribution/scalar_other/InverseGammaDistribution.birch"
inline decltype(auto) InverseGamma(const Arg1& \u03b1, const Arg2& \u03b2) {
  #line 70 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return construct<InverseGammaDistribution<Arg1, Arg2>>(\u03b1, \u03b2);
}

#line 92 "src/distribution/scalar_other/InverseGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 92 "src/distribution/scalar_other/InverseGammaDistribution.birch"
inline decltype(auto) logpdf_inverse_gamma(const Arg1& x, const Arg2& \u03b1, const Arg3& \u03b2) {
  #line 93 "src/distribution/scalar_other/InverseGammaDistribution.birch"
  return if_then_else(x < 0.0, -(std::numeric_limits<double>::infinity()), \u03b1 * log(\u03b2) - (\u03b1 + 1.0) * log(x) - \u03b2 / x - lgamma(\u03b1));
}

#line 58 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 58 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
inline decltype(auto) InverseGammaGamma(const Arg1& k, const Arg2& \u03b1, const Arg3& \u03b2) {
  #line 59 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return construct<InverseGammaGammaDistribution<Arg1, Arg2, Arg3>>(k, \u03b1, \u03b2);
}

#line 65 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg>
#line 65 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
inline Distribution<Real> Gamma(const Arg& k, const Random<Real>& \u03b8) {
  #line 66 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  \u03b8->prune();
  #line 67 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  if (\u03b8->hasInverseGamma()) {
    auto n4583_ = \u03b8->getInverseGamma();
    auto \u03b1 = std::get<0>(n4583_);
    auto \u03b2 = std::get<1>(n4583_);
    auto p = InverseGammaGamma(k, \u03b1, \u03b2);
    #line 70 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
    \u03b8->graft(p);
    #line 71 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
    return p;
  } else {
    #line 73 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
    return construct<GammaDistribution<Arg, Random<Real>>>(k, \u03b8);
  }
}

#line 98 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 98 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
inline decltype(auto) logpdf_inverse_gamma_gamma(const Arg1& x, const Arg2& k, const Arg3& \u03b1, const Arg4& \u03b2) {
  #line 100 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return (k - Integer(1)) * log(x) + \u03b1 * log(\u03b2) - (\u03b1 + k) * log(\u03b2 + x) - lbeta(\u03b1, k);
}

#line 114 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 114 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
inline decltype(auto) update_inverse_gamma_gamma(const Arg1& x, const Arg2& k, const Arg3& \u03b1, const Arg4& \u03b2) {
  #line 116 "src/distribution/scalar_other/InverseGammaGammaDistribution.birch"
  return InverseGamma(\u03b1 + k, \u03b2 + x);
}

#line 62 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 62 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
inline decltype(auto) ScaledGammaExponential(const Arg1& a, const Arg2& k, const Arg3& \u03b8) {
  #line 63 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return construct<ScaledGammaExponentialDistribution<Arg1, Arg2, Arg3>>(a, k, \u03b8);
}

#line 69 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg>
#line 69 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
inline Distribution<Real> Exponential(const Multiply<Arg, Random<Real>>& \u03bb) {
  #line 71 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  \u03bb->r->prune();
  #line 72 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  if (\u03bb->r->hasGamma()) {
    auto n4584_ = \u03bb->r->getGamma();
    auto k = std::get<0>(n4584_);
    auto \u03b8 = std::get<1>(n4584_);
    auto p = ScaledGammaExponential(\u03bb->l, k, \u03b8);
    #line 75 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
    \u03bb->r->graft(p);
    #line 76 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
    return p;
  } else {
    #line 78 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
    return construct<ExponentialDistribution<Multiply<Arg, Random<Real>>>>(\u03bb);
  }
}

#line 85 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg>
#line 85 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
inline Distribution<Real> Exponential(const Multiply<Random<Real>, Arg>& \u03bb) {
  #line 87 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  \u03bb->l->prune();
  #line 88 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  if (\u03bb->l->hasGamma()) {
    auto n4585_ = \u03bb->l->getGamma();
    auto k = std::get<0>(n4585_);
    auto \u03b8 = std::get<1>(n4585_);
    auto p = ScaledGammaExponential(\u03bb->r, k, \u03b8);
    #line 91 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
    \u03bb->l->graft(p);
    #line 92 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
    return p;
  } else {
    #line 94 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
    return construct<ExponentialDistribution<Multiply<Random<Real>, Arg>>>(\u03bb);
  }
}

#line 133 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 133 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
inline decltype(auto) update_scaled_gamma_exponential(const Arg1& x, const Arg2& a, const Arg3& k, const Arg4& \u03b8) {
  #line 135 "src/distribution/scalar_other/ScaledGammaExponentialDistribution.birch"
  return Gamma(k + 1.0, \u03b8 / (1.0 + x * a * \u03b8));
}

#line 64 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 64 "src/distribution/scalar_other/StudentDistribution.birch"
inline decltype(auto) Student(const Arg1& \u03bd, const Arg2& \u03bc, const Arg3& \u03c32) {
  #line 65 "src/distribution/scalar_other/StudentDistribution.birch"
  return construct<StudentDistribution<Arg1, Arg2, Arg3>>(\u03bd, \u03bc, \u03c32);
}

#line 71 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg>
#line 71 "src/distribution/scalar_other/StudentDistribution.birch"
inline decltype(auto) Student(const Arg& \u03bd) {
  #line 72 "src/distribution/scalar_other/StudentDistribution.birch"
  return Student(\u03bd, 0.0, 1.0);
}

#line 106 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2>
#line 106 "src/distribution/scalar_other/StudentDistribution.birch"
inline decltype(auto) logpdf_student_t(const Arg1& x, const Arg2& k) {
  auto a = 0.5 * (k + 1.0);
  auto b = 0.5 * k;
  #line 109 "src/distribution/scalar_other/StudentDistribution.birch"
  return lgamma(a) - lgamma(b) - 0.5 * log(\u03c0 * k) - a * log1p(x * x / k);
}

#line 122 "src/distribution/scalar_other/StudentDistribution.birch"
template<class Arg1,class Arg2,class Arg3,class Arg4>
#line 122 "src/distribution/scalar_other/StudentDistribution.birch"
inline decltype(auto) logpdf_student_t(const Arg1& x, const Arg2& k, const Arg3& \u03bc, const Arg4& v) {
  auto z = (x - \u03bc) / sqrt(v);
  auto a = 0.5 * (k + 1.0);
  auto b = 0.5 * k;
  #line 127 "src/distribution/scalar_other/StudentDistribution.birch"
  return lgamma(a) - lgamma(b) - 0.5 * log(\u03c0 * k * v) - a * log1p(z * z / k);
}

#line 57 "src/distribution/scalar_other/UniformDistribution.birch"
template<class Arg1,class Arg2>
#line 57 "src/distribution/scalar_other/UniformDistribution.birch"
inline decltype(auto) Uniform(const Arg1& l, const Arg2& u) {
  #line 58 "src/distribution/scalar_other/UniformDistribution.birch"
  return construct<UniformDistribution<Arg1, Arg2>>(l, u);
}

#line 61 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2>
#line 61 "src/distribution/scalar_other/WeibullDistribution.birch"
inline decltype(auto) Weibull(const Arg1& k, const Arg2& \u03bb) {
  #line 62 "src/distribution/scalar_other/WeibullDistribution.birch"
  return construct<WeibullDistribution<Arg1, Arg2>>(k, \u03bb);
}

#line 88 "src/distribution/scalar_other/WeibullDistribution.birch"
template<class Arg1,class Arg2,class Arg3>
#line 88 "src/distribution/scalar_other/WeibullDistribution.birch"
inline decltype(auto) logpdf_weibull(const Arg1& x, const Arg2& k, const Arg3& \u03bb) {
  #line 89 "src/distribution/scalar_other/WeibullDistribution.birch"
  return if_then_else(x < 0.0, -(std::numeric_limits<double>::infinity()), log(k) + (k - 1.0) * log(x) - k * log(\u03bb) - pow(x / \u03bb, k));
}

#line 19 "src/form/binary_matrix/CholSolve.birch"
template<class Left,class Right>
#line 19 "src/form/binary_matrix/CholSolve.birch"
inline decltype(auto) cholsolve(const Left& l, const Right& r) {
  #line 20 "src/form/binary_matrix/CholSolve.birch"
  return construct<CholSolve<Left, Right>>(l, r);
}

#line 19 "src/form/binary_matrix/Diagonal.birch"
template<class Left,class Right>
#line 19 "src/form/binary_matrix/Diagonal.birch"
inline decltype(auto) diagonal(const Left& l, const Right& r) {
  #line 20 "src/form/binary_matrix/Diagonal.birch"
  return construct<Diagonal<Left, Right>>(l, r);
}

#line 16 "src/form/binary_matrix/Dot.birch"
template<class Left,class Right>
#line 16 "src/form/binary_matrix/Dot.birch"
inline decltype(auto) dot(const Left& l, const Right& r) {
  #line 17 "src/form/binary_matrix/Dot.birch"
  return construct<Dot<Left, Right>>(l, r);
}

#line 23 "src/form/binary_matrix/Dot.birch"
template<class Middle>
#line 23 "src/form/binary_matrix/Dot.birch"
inline decltype(auto) dot(const Middle& m) {
  #line 24 "src/form/binary_matrix/Dot.birch"
  return dot(m, m);
}

#line 16 "src/form/binary_matrix/Inner.birch"
template<class Left,class Right>
#line 16 "src/form/binary_matrix/Inner.birch"
inline decltype(auto) inner(const Left& l, const Right& r) {
  #line 17 "src/form/binary_matrix/Inner.birch"
  return construct<Inner<Left, Right>>(l, r);
}

#line 23 "src/form/binary_matrix/Inner.birch"
template<class Middle>
#line 23 "src/form/binary_matrix/Inner.birch"
inline decltype(auto) inner(const Middle& m) {
  #line 24 "src/form/binary_matrix/Inner.birch"
  return inner(m, m);
}

#line 30 "src/form/binary_matrix/MatrixPack.birch"
template<class Left,class Right>
#line 30 "src/form/binary_matrix/MatrixPack.birch"
inline decltype(auto) pack(const Left& l, const Right& r) {
  #line 31 "src/form/binary_matrix/MatrixPack.birch"
  return construct<MatrixPack<Left, Right>>(l, r);
}

#line 58 "src/form/binary_matrix/MatrixPack.birch"
template<class Type>
#line 58 "src/form/binary_matrix/MatrixPack.birch"
inline libbirch::DefaultArray<Type,2> pack(const libbirch::DefaultArray<Type,1>& x, const libbirch::DefaultArray<Type,1>& y) {
  #line 59 "src/form/binary_matrix/MatrixPack.birch"
  assert(length(x) == length(y));
  #line 60 "src/form/binary_matrix/MatrixPack.birch"
  return matrix_lambda([=](const Integer& i, const Integer& j) -> Type {
    #line 61 "src/form/binary_matrix/MatrixPack.birch"
    if (j == Integer(1)) {
      #line 62 "src/form/binary_matrix/MatrixPack.birch"
      return x(i);
    } else {
      #line 64 "src/form/binary_matrix/MatrixPack.birch"
      return y(i);
    }
  }, length(x), Integer(2));
}

#line 73 "src/form/binary_matrix/MatrixPack.birch"
template<class Type>
#line 73 "src/form/binary_matrix/MatrixPack.birch"
inline libbirch::DefaultArray<Type,2> pack(const libbirch::DefaultArray<Type,2>& l, const libbirch::DefaultArray<Type,2>& r) {
  #line 74 "src/form/binary_matrix/MatrixPack.birch"
  assert(rows(l) == rows(r));
  auto R = rows(l);
  auto C1 = columns(l);
  auto C2 = columns(r);
  #line 78 "src/form/binary_matrix/MatrixPack.birch"
  return matrix_lambda([=](const Integer& i, const Integer& j) -> Type {
    #line 79 "src/form/binary_matrix/MatrixPack.birch"
    if (j <= C1) {
      #line 80 "src/form/binary_matrix/MatrixPack.birch"
      return l(i, j);
    } else {
      #line 82 "src/form/binary_matrix/MatrixPack.birch"
      return r(i, j - C1);
    }
  }, R, C1 + C2);
}

#line 40 "src/form/binary_matrix/MatrixStack.birch"
template<class Left,class Right>
#line 40 "src/form/binary_matrix/MatrixStack.birch"
inline decltype(auto) stack(const Left& l, const Right& r) {
  #line 41 "src/form/binary_matrix/MatrixStack.birch"
  return construct<MatrixStack<Left, Right>>(l, r);
}

#line 72 "src/form/binary_matrix/MatrixStack.birch"
template<class Type>
#line 72 "src/form/binary_matrix/MatrixStack.birch"
inline libbirch::DefaultArray<Type,1> stack(const libbirch::DefaultArray<Type,1>& x, const libbirch::DefaultArray<Type,1>& y) {
  auto l1 = length(x);
  auto l2 = length(y);
  #line 75 "src/form/binary_matrix/MatrixStack.birch"
  return vector_lambda([=](const Integer& i) -> Type {
    #line 76 "src/form/binary_matrix/MatrixStack.birch"
    if (i <= l1) {
      #line 77 "src/form/binary_matrix/MatrixStack.birch"
      return x(i);
    } else {
      #line 79 "src/form/binary_matrix/MatrixStack.birch"
      return y(i - l1);
    }
  }, l1 + l2);
}

#line 88 "src/form/binary_matrix/MatrixStack.birch"
template<class Type>
#line 88 "src/form/binary_matrix/MatrixStack.birch"
inline libbirch::DefaultArray<Type,1> stack(const libbirch::DefaultArray<Type,1>& x, const Type& y) {
  auto l1 = length(x);
  #line 90 "src/form/binary_matrix/MatrixStack.birch"
  return vector_lambda([=](const Integer& i) -> Type {
    #line 91 "src/form/binary_matrix/MatrixStack.birch"
    if (i <= l1) {
      #line 92 "src/form/binary_matrix/MatrixStack.birch"
      return x(i);
    } else {
      #line 94 "src/form/binary_matrix/MatrixStack.birch"
      return y;
    }
  }, l1 + Integer(1));
}

#line 103 "src/form/binary_matrix/MatrixStack.birch"
template<class Type>
#line 103 "src/form/binary_matrix/MatrixStack.birch"
inline libbirch::DefaultArray<Type,1> stack(const Type& x, const libbirch::DefaultArray<Type,1>& y) {
  auto l2 = length(y);
  #line 105 "src/form/binary_matrix/MatrixStack.birch"
  return vector_lambda([=](const Integer& i) -> Type {
    #line 106 "src/form/binary_matrix/MatrixStack.birch"
    if (i <= Integer(1)) {
      #line 107 "src/form/binary_matrix/MatrixStack.birch"
      return x;
    } else {
      #line 109 "src/form/binary_matrix/MatrixStack.birch"
      return y(i - Integer(1));
    }
  }, Integer(1) + l2);
}

#line 118 "src/form/binary_matrix/MatrixStack.birch"
template<class Type>
#line 118 "src/form/binary_matrix/MatrixStack.birch"
inline libbirch::DefaultArray<Type,2> stack(const libbirch::DefaultArray<Type,2>& X, const libbirch::DefaultArray<Type,2>& Y) {
  #line 119 "src/form/binary_matrix/MatrixStack.birch"
  assert(columns(X) == columns(Y));
  auto R1 = rows(X);
  auto R2 = rows(Y);
  auto C = columns(X);
  #line 123 "src/form/binary_matrix/MatrixStack.birch"
  return matrix_lambda([=](const Integer& i, const Integer& j) -> Type {
    #line 124 "src/form/binary_matrix/MatrixStack.birch"
    if (i <= R1) {
      #line 125 "src/form/binary_matrix/MatrixStack.birch"
      return X(i, j);
    } else {
      #line 127 "src/form/binary_matrix/MatrixStack.birch"
      return Y(i - R1, j);
    }
  }, R1 + R2, C);
}

#line 16 "src/form/binary_matrix/Outer.birch"
template<class Left,class Right>
#line 16 "src/form/binary_matrix/Outer.birch"
inline decltype(auto) outer(const Left& l, const Right& r) {
  #line 17 "src/form/binary_matrix/Outer.birch"
  return construct<Outer<Left, Right>>(l, r);
}

#line 23 "src/form/binary_matrix/Outer.birch"
template<class Middle>
#line 23 "src/form/binary_matrix/Outer.birch"
inline decltype(auto) outer(const Middle& m) {
  #line 24 "src/form/binary_matrix/Outer.birch"
  return outer(m, m);
}

#line 18 "src/form/binary_matrix/Solve.birch"
template<class Left,class Right>
#line 18 "src/form/binary_matrix/Solve.birch"
inline decltype(auto) solve(const Left& l, const Right& r) {
  #line 19 "src/form/binary_matrix/Solve.birch"
  return construct<Solve<Left, Right>>(l, r);
}

#line 16 "src/form/binary_other/CopySign.birch"
template<class Left,class Right>
#line 16 "src/form/binary_other/CopySign.birch"
inline decltype(auto) copysign(const Left& l, const Right& r) {
  #line 17 "src/form/binary_other/CopySign.birch"
  return construct<CopySign<Left, Right>>(l, r);
}

#line 16 "src/form/binary_other/LogBeta.birch"
template<class Left,class Right>
#line 16 "src/form/binary_other/LogBeta.birch"
inline decltype(auto) lbeta(const Left& l, const Right& r) {
  #line 17 "src/form/binary_other/LogBeta.birch"
  return construct<LogBeta<Left, Right>>(l, r);
}

#line 16 "src/form/binary_other/LogChoose.birch"
template<class Left,class Right>
#line 16 "src/form/binary_other/LogChoose.birch"
inline decltype(auto) lchoose(const Left& l, const Right& r) {
  #line 17 "src/form/binary_other/LogChoose.birch"
  return construct<LogChoose<Left, Right>>(l, r);
}

#line 20 "src/form/binary_other/LogGammaP.birch"
template<class Left,class Right>
#line 20 "src/form/binary_other/LogGammaP.birch"
inline decltype(auto) lgamma(const Left& l, const Right& r) {
  #line 21 "src/form/binary_other/LogGammaP.birch"
  return construct<LogGammaP<Left, Right>>(l, r);
}

#line 21 "src/form/binary_other/Pow.birch"
template<class Left,class Right>
#line 21 "src/form/binary_other/Pow.birch"
inline decltype(auto) pow(const Left& l, const Right& r) {
  #line 22 "src/form/binary_other/Pow.birch"
  return construct<Pow<Left, Right>>(l, r);
}

#line 16 "src/form/ternary/IfThenElse.birch"
template<class Left,class Middle,class Right>
#line 16 "src/form/ternary/IfThenElse.birch"
inline decltype(auto) if_then_else(const Left& l, const Middle& m, const Right& r) {
  #line 17 "src/form/ternary/IfThenElse.birch"
  return construct<IfThenElse<Left, Middle, Right>>(l, m, r);
}

#line 23 "src/form/ternary/IfThenElse.birch"
template<class Type>
#line 23 "src/form/ternary/IfThenElse.birch"
inline Type if_then_else(const Boolean& cond, const Type& x, const Type& y) {
  #line 24 "src/form/ternary/IfThenElse.birch"
  if (cond) {
    #line 25 "src/form/ternary/IfThenElse.birch"
    return x;
  } else {
    #line 27 "src/form/ternary/IfThenElse.birch"
    return y;
  }
}

#line 29 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
template<class Arg1,class Arg2,class Arg3>
#line 29 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
inline decltype(auto) logz_conway_maxwell_poisson(const Arg1& \u03bb, const Arg2& \u03bd, const Arg3& n) {
  #line 30 "src/form/ternary/LogZConwayMaxwellPoisson.birch"
  return construct<LogZConwayMaxwellPoisson<Arg1, Arg2, Arg3>>(\u03bb, \u03bd, n);
}

#line 17 "src/form/unary_matrix/CholInv.birch"
template<class Middle>
#line 17 "src/form/unary_matrix/CholInv.birch"
inline decltype(auto) cholinv(const Middle& m) {
  #line 18 "src/form/unary_matrix/CholInv.birch"
  return construct<CholInv<Middle>>(m);
}

#line 16 "src/form/unary_matrix/Inv.birch"
template<class Middle>
#line 16 "src/form/unary_matrix/Inv.birch"
inline decltype(auto) inv(const Middle& m) {
  #line 17 "src/form/unary_matrix/Inv.birch"
  return construct<Inv<Middle>>(m);
}

#line 16 "src/form/unary_matrix/LCholDet.birch"
template<class Middle>
#line 16 "src/form/unary_matrix/LCholDet.birch"
inline decltype(auto) lcholdet(const Middle& m) {
  #line 17 "src/form/unary_matrix/LCholDet.birch"
  return construct<LCholDet<Middle>>(m);
}

#line 15 "src/form/unary_matrix/LDet.birch"
template<class Middle>
#line 15 "src/form/unary_matrix/LDet.birch"
inline decltype(auto) ldet(const Middle& m) {
  #line 16 "src/form/unary_matrix/LDet.birch"
  return construct<LDet<Middle>>(m);
}

#line 15 "src/form/unary_matrix/Matrix.birch"
template<class Value,class Middle>
#line 15 "src/form/unary_matrix/Matrix.birch"
inline decltype(auto) matrix(const Middle& m) {
  #line 16 "src/form/unary_matrix/Matrix.birch"
  return construct<Matrix<Value, Middle>>(m);
}

#line 22 "src/form/unary_matrix/Matrix.birch"
template<class Value,class Type>
#line 22 "src/form/unary_matrix/Matrix.birch"
inline libbirch::DefaultArray<Value,2> matrix(const libbirch::DefaultArray<Type,1>& m) {
  #line 23 "src/form/unary_matrix/Matrix.birch"
  return matrix<Value>(mat(m, Integer(1)));
}

#line 29 "src/form/unary_matrix/Matrix.birch"
template<class Value,class Type>
#line 29 "src/form/unary_matrix/Matrix.birch"
inline libbirch::DefaultArray<Value,2> matrix(const libbirch::DefaultArray<Type,2>& m) {
  #line 30 "src/form/unary_matrix/Matrix.birch"
  return transform(m, [=](const Type& x) -> Value {
    #line 30 "src/form/unary_matrix/Matrix.birch"
    return scalar<Value>(x);
  });
}

#line 36 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 36 "src/form/unary_matrix/Matrix.birch"
inline libbirch::DefaultArray<Value,2> matrix(const Boolean& m) {
  #line 37 "src/form/unary_matrix/Matrix.birch"
  return libbirch::make_array_from_sequence({ { scalar<Value>(m) } });
}

#line 43 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 43 "src/form/unary_matrix/Matrix.birch"
inline libbirch::DefaultArray<Value,2> matrix(const Integer& m) {
  #line 44 "src/form/unary_matrix/Matrix.birch"
  return libbirch::make_array_from_sequence({ { scalar<Value>(m) } });
}

#line 50 "src/form/unary_matrix/Matrix.birch"
template<class Value>
#line 50 "src/form/unary_matrix/Matrix.birch"
inline libbirch::DefaultArray<Value,2> matrix(const Real& m) {
  #line 51 "src/form/unary_matrix/Matrix.birch"
  return libbirch::make_array_from_sequence({ { scalar<Value>(m) } });
}

#line 57 "src/form/unary_matrix/Matrix.birch"
template<class Gradient,class Value>
#line 57 "src/form/unary_matrix/Matrix.birch"
inline decltype(auto) grad_matrix(const libbirch::DefaultArray<Gradient,2>& d, const Value& m) {
  #line 58 "src/form/unary_matrix/Matrix.birch"
  return scalar<Value>(d);
}

#line 64 "src/form/unary_matrix/Matrix.birch"
template<class Gradient,class Value>
#line 64 "src/form/unary_matrix/Matrix.birch"
inline decltype(auto) grad_matrix(const libbirch::DefaultArray<Gradient,2>& d, const libbirch::DefaultArray<Value,1>& m) {
  #line 65 "src/form/unary_matrix/Matrix.birch"
  return vector<Value>(d);
}

#line 71 "src/form/unary_matrix/Matrix.birch"
template<class Gradient,class Value>
#line 71 "src/form/unary_matrix/Matrix.birch"
inline decltype(auto) grad_matrix(const libbirch::DefaultArray<Gradient,2>& d, const libbirch::DefaultArray<Value,2>& m) {
  #line 72 "src/form/unary_matrix/Matrix.birch"
  return matrix<Value>(d);
}

#line 23 "src/form/unary_matrix/Trace.birch"
template<class Middle>
#line 23 "src/form/unary_matrix/Trace.birch"
inline decltype(auto) trace(const Middle& m) {
  #line 24 "src/form/unary_matrix/Trace.birch"
  return construct<Trace<Middle>>(m);
}

#line 15 "src/form/unary_matrix/Transpose.birch"
template<class Middle>
#line 15 "src/form/unary_matrix/Transpose.birch"
inline decltype(auto) transpose(const Middle& m) {
  #line 16 "src/form/unary_matrix/Transpose.birch"
  return construct<Transpose<Middle>>(m);
}

#line 15 "src/form/unary_matrix/Vector.birch"
template<class Value,class Middle>
#line 15 "src/form/unary_matrix/Vector.birch"
inline decltype(auto) vector(const Middle& m) {
  #line 16 "src/form/unary_matrix/Vector.birch"
  return construct<Vector<Value, Middle>>(m);
}

#line 22 "src/form/unary_matrix/Vector.birch"
template<class Value,class Type>
#line 22 "src/form/unary_matrix/Vector.birch"
inline libbirch::DefaultArray<Value,1> vector(const libbirch::DefaultArray<Type,1>& m) {
  #line 23 "src/form/unary_matrix/Vector.birch"
  return transform(m, [=](const Type& x) -> Value {
    #line 23 "src/form/unary_matrix/Vector.birch"
    return scalar<Value>(x);
  });
}

#line 29 "src/form/unary_matrix/Vector.birch"
template<class Value,class Type>
#line 29 "src/form/unary_matrix/Vector.birch"
inline libbirch::DefaultArray<Value,1> vector(const libbirch::DefaultArray<Type,2>& m) {
  #line 30 "src/form/unary_matrix/Vector.birch"
  assert(columns(m) == Integer(1));
  #line 31 "src/form/unary_matrix/Vector.birch"
  return vector<Value>(m(libbirch::make_range(Integer(1), rows(m)), Integer(1)));
}

#line 37 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 37 "src/form/unary_matrix/Vector.birch"
inline libbirch::DefaultArray<Value,1> vector(const Boolean& m) {
  #line 38 "src/form/unary_matrix/Vector.birch"
  return libbirch::make_array_from_sequence({ scalar<Value>(m) });
}

#line 44 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 44 "src/form/unary_matrix/Vector.birch"
inline libbirch::DefaultArray<Value,1> vector(const Integer& m) {
  #line 45 "src/form/unary_matrix/Vector.birch"
  return libbirch::make_array_from_sequence({ scalar<Value>(m) });
}

#line 51 "src/form/unary_matrix/Vector.birch"
template<class Value>
#line 51 "src/form/unary_matrix/Vector.birch"
inline libbirch::DefaultArray<Value,1> vector(const Real& m) {
  #line 52 "src/form/unary_matrix/Vector.birch"
  return libbirch::make_array_from_sequence({ scalar<Value>(m) });
}

#line 58 "src/form/unary_matrix/Vector.birch"
template<class Gradient,class Value>
#line 58 "src/form/unary_matrix/Vector.birch"
inline decltype(auto) grad_vector(const libbirch::DefaultArray<Gradient,1>& d, const Value& m) {
  #line 59 "src/form/unary_matrix/Vector.birch"
  return scalar<Value>(d);
}

#line 65 "src/form/unary_matrix/Vector.birch"
template<class Gradient,class Value>
#line 65 "src/form/unary_matrix/Vector.birch"
inline decltype(auto) grad_vector(const libbirch::DefaultArray<Gradient,1>& d, const libbirch::DefaultArray<Value,1>& m) {
  #line 66 "src/form/unary_matrix/Vector.birch"
  return vector<Value>(d);
}

#line 72 "src/form/unary_matrix/Vector.birch"
template<class Gradient,class Value>
#line 72 "src/form/unary_matrix/Vector.birch"
inline decltype(auto) grad_vector(const libbirch::DefaultArray<Gradient,1>& d, const libbirch::DefaultArray<Value,2>& m) {
  #line 73 "src/form/unary_matrix/Vector.birch"
  return matrix<Value>(d);
}

#line 15 "src/form/unary_other/Abs.birch"
template<class Middle>
#line 15 "src/form/unary_other/Abs.birch"
inline decltype(auto) abs(const Middle& m) {
  #line 16 "src/form/unary_other/Abs.birch"
  return construct<Abs<Middle>>(m);
}

#line 15 "src/form/unary_other/Acos.birch"
template<class Middle>
#line 15 "src/form/unary_other/Acos.birch"
inline decltype(auto) acos(const Middle& m) {
  #line 16 "src/form/unary_other/Acos.birch"
  return construct<Acos<Middle>>(m);
}

#line 15 "src/form/unary_other/Asin.birch"
template<class Middle>
#line 15 "src/form/unary_other/Asin.birch"
inline decltype(auto) asin(const Middle& m) {
  #line 16 "src/form/unary_other/Asin.birch"
  return construct<Asin<Middle>>(m);
}

#line 15 "src/form/unary_other/Atan.birch"
template<class Middle>
#line 15 "src/form/unary_other/Atan.birch"
inline decltype(auto) atan(const Middle& m) {
  #line 16 "src/form/unary_other/Atan.birch"
  return construct<Atan<Middle>>(m);
}

#line 16 "src/form/unary_other/Cos.birch"
template<class Middle>
#line 16 "src/form/unary_other/Cos.birch"
inline decltype(auto) cos(const Middle& m) {
  #line 17 "src/form/unary_other/Cos.birch"
  return construct<Cos<Middle>>(m);
}

#line 15 "src/form/unary_other/Cosh.birch"
template<class Middle>
#line 15 "src/form/unary_other/Cosh.birch"
inline decltype(auto) cosh(const Middle& m) {
  #line 16 "src/form/unary_other/Cosh.birch"
  return construct<Cosh<Middle>>(m);
}

#line 15 "src/form/unary_other/Exp.birch"
template<class Middle>
#line 15 "src/form/unary_other/Exp.birch"
inline decltype(auto) exp(const Middle& m) {
  #line 16 "src/form/unary_other/Exp.birch"
  return construct<Exp<Middle>>(m);
}

#line 15 "src/form/unary_other/Expm1.birch"
template<class Middle>
#line 15 "src/form/unary_other/Expm1.birch"
inline decltype(auto) expm1(const Middle& m) {
  #line 16 "src/form/unary_other/Expm1.birch"
  return construct<Expm1<Middle>>(m);
}

#line 15 "src/form/unary_other/Log.birch"
template<class Middle>
#line 15 "src/form/unary_other/Log.birch"
inline decltype(auto) log(const Middle& m) {
  #line 16 "src/form/unary_other/Log.birch"
  return construct<Log<Middle>>(m);
}

#line 15 "src/form/unary_other/Log1p.birch"
template<class Middle>
#line 15 "src/form/unary_other/Log1p.birch"
inline decltype(auto) log1p(const Middle& m) {
  #line 16 "src/form/unary_other/Log1p.birch"
  return construct<Log1p<Middle>>(m);
}

#line 15 "src/form/unary_other/LogGamma.birch"
template<class Middle>
#line 15 "src/form/unary_other/LogGamma.birch"
inline decltype(auto) lgamma(const Middle& m) {
  #line 16 "src/form/unary_other/LogGamma.birch"
  return construct<LogGamma<Middle>>(m);
}

#line 17 "src/form/unary_other/Rectify.birch"
template<class Middle>
#line 17 "src/form/unary_other/Rectify.birch"
inline decltype(auto) rectify(const Middle& m) {
  #line 18 "src/form/unary_other/Rectify.birch"
  return construct<Rectify<Middle>>(m);
}

#line 15 "src/form/unary_other/Scalar.birch"
template<class Value,class Middle>
#line 15 "src/form/unary_other/Scalar.birch"
inline decltype(auto) scalar(const Middle& m) {
  #line 16 "src/form/unary_other/Scalar.birch"
  return construct<Scalar<Value, Middle>>(m);
}

#line 22 "src/form/unary_other/Scalar.birch"
template<class Value,class Type>
#line 22 "src/form/unary_other/Scalar.birch"
inline Value scalar(const libbirch::DefaultArray<Type,1>& m) {
  #line 23 "src/form/unary_other/Scalar.birch"
  assert(length(m) == Integer(1));
  #line 24 "src/form/unary_other/Scalar.birch"
  return scalar<Value>(m(Integer(1)));
}

#line 30 "src/form/unary_other/Scalar.birch"
template<class Value,class Type>
#line 30 "src/form/unary_other/Scalar.birch"
inline Value scalar(const libbirch::DefaultArray<Type,2>& m) {
  #line 31 "src/form/unary_other/Scalar.birch"
  assert(rows(m) == Integer(1) && columns(m) == Integer(1));
  #line 32 "src/form/unary_other/Scalar.birch"
  return scalar<Value>(m(Integer(1), Integer(1)));
}

#line 38 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 38 "src/form/unary_other/Scalar.birch"
inline Value scalar(const Boolean& m) {
  #line 39 "src/form/unary_other/Scalar.birch"

  return static_cast<Value>(m);
  }

#line 47 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 47 "src/form/unary_other/Scalar.birch"
inline Value scalar(const Integer& m) {
  #line 48 "src/form/unary_other/Scalar.birch"

  return static_cast<Value>(m);
  }

#line 56 "src/form/unary_other/Scalar.birch"
template<class Value>
#line 56 "src/form/unary_other/Scalar.birch"
inline Value scalar(const Real& m) {
  #line 57 "src/form/unary_other/Scalar.birch"

  return static_cast<Value>(m);
  }

#line 65 "src/form/unary_other/Scalar.birch"
template<class Gradient,class Value>
#line 65 "src/form/unary_other/Scalar.birch"
inline decltype(auto) grad_scalar(const Gradient& d, const Value& m) {
  #line 66 "src/form/unary_other/Scalar.birch"
  return scalar<Value>(d);
}

#line 72 "src/form/unary_other/Scalar.birch"
template<class Gradient,class Value>
#line 72 "src/form/unary_other/Scalar.birch"
inline decltype(auto) grad_scalar(const Gradient& d, const libbirch::DefaultArray<Value,1>& m) {
  #line 73 "src/form/unary_other/Scalar.birch"
  return vector<Value>(d);
}

#line 79 "src/form/unary_other/Scalar.birch"
template<class Gradient,class Value>
#line 79 "src/form/unary_other/Scalar.birch"
inline decltype(auto) grad_scalar(const Gradient& d, const libbirch::DefaultArray<Value,2>& m) {
  #line 80 "src/form/unary_other/Scalar.birch"
  return matrix<Value>(d);
}

#line 15 "src/form/unary_other/Sin.birch"
template<class Middle>
#line 15 "src/form/unary_other/Sin.birch"
inline decltype(auto) sin(const Middle& m) {
  #line 16 "src/form/unary_other/Sin.birch"
  return construct<Sin<Middle>>(m);
}

#line 15 "src/form/unary_other/Sinh.birch"
template<class Middle>
#line 15 "src/form/unary_other/Sinh.birch"
inline decltype(auto) sinh(const Middle& m) {
  #line 16 "src/form/unary_other/Sinh.birch"
  return construct<Sinh<Middle>>(m);
}

#line 15 "src/form/unary_other/Sqrt.birch"
template<class Middle>
#line 15 "src/form/unary_other/Sqrt.birch"
inline decltype(auto) sqrt(const Middle& m) {
  #line 16 "src/form/unary_other/Sqrt.birch"
  return construct<Sqrt<Middle>>(m);
}

#line 23 "src/form/unary_other/Sum.birch"
template<class Middle>
#line 23 "src/form/unary_other/Sum.birch"
inline decltype(auto) sum(const Middle& m) {
  #line 24 "src/form/unary_other/Sum.birch"
  return construct<Sum<Middle>>(m);
}

#line 15 "src/form/unary_other/Tan.birch"
template<class Middle>
#line 15 "src/form/unary_other/Tan.birch"
inline decltype(auto) tan(const Middle& m) {
  #line 16 "src/form/unary_other/Tan.birch"
  return construct<Tan<Middle>>(m);
}

#line 15 "src/form/unary_other/Tanh.birch"
template<class Middle>
#line 15 "src/form/unary_other/Tanh.birch"
inline decltype(auto) tanh(const Middle& m) {
  #line 16 "src/form/unary_other/Tanh.birch"
  return construct<Tanh<Middle>>(m);
}

#line 40 "src/test/basic/test_basic_array.birch"
template<class Container>
#line 40 "src/test/basic/test_basic_array.birch"
inline Boolean check_container(const Container& o, const libbirch::DefaultArray<Integer,1>& values) {
  auto result = true;
  #line 44 "src/test/basic/test_basic_array.birch"
  if (o->size() != length(values)) {
    #line 45 "src/test/basic/test_basic_array.birch"
    stderr_->print(String("incorrect total size\n"));
    #line 46 "src/test/basic/test_basic_array.birch"
    result = false;
  }
  #line 50 "src/test/basic/test_basic_array.birch"
  for (auto i = Integer(1); i <= o->size(); ++i) {
    #line 51 "src/test/basic/test_basic_array.birch"
    if (o->get(i) != values(i)) {
      #line 52 "src/test/basic/test_basic_array.birch"
      stderr_->print(String("incorrect value\n"));
      #line 53 "src/test/basic/test_basic_array.birch"
      result = false;
    }
  }
  auto iter = o->walk();
  auto i = Integer(0);
  #line 60 "src/test/basic/test_basic_array.birch"
  while (iter->hasNext()) {
    #line 61 "src/test/basic/test_basic_array.birch"
    i = i + Integer(1);
    #line 62 "src/test/basic/test_basic_array.birch"
    if (iter->next() != values(i)) {
      #line 63 "src/test/basic/test_basic_array.birch"
      stderr_->print(String("incorrect value\n"));
      #line 64 "src/test/basic/test_basic_array.birch"
      result = false;
    }
  }
  #line 68 "src/test/basic/test_basic_array.birch"
  return result;
}

#line 36 "src/test/basic/test_basic_stack.birch"
template<class Container>
#line 36 "src/test/basic/test_basic_stack.birch"
inline Boolean check_stack(const Container& o, const libbirch::DefaultArray<Integer,1>& values) {
  auto result = true;
  #line 40 "src/test/basic/test_basic_stack.birch"
  if (o->size() != length(values)) {
    #line 41 "src/test/basic/test_basic_stack.birch"
    stderr_->print(String("incorrect total size\n"));
    #line 42 "src/test/basic/test_basic_stack.birch"
    result = false;
  }
  auto iter = o->walk();
  auto i = Integer(0);
  #line 48 "src/test/basic/test_basic_stack.birch"
  while (iter->hasNext()) {
    #line 49 "src/test/basic/test_basic_stack.birch"
    i = i + Integer(1);
    #line 50 "src/test/basic/test_basic_stack.birch"
    if (iter->next() != values(i)) {
      #line 51 "src/test/basic/test_basic_stack.birch"
      stderr_->print(String("incorrect value\n"));
      #line 52 "src/test/basic/test_basic_stack.birch"
      result = false;
    }
  }
  #line 56 "src/test/basic/test_basic_stack.birch"
  return result;
}

#line 15 "src/form/binary_other/Add.birch"
template<class Left,class Right>
#line 15 "src/form/binary_other/Add.birch"
inline Add<Left, Right> operator+(const Left& l, const Right& r) {
  #line 16 "src/form/binary_other/Add.birch"
  return construct<Add<Left, Right>>(l, r);
}

#line 16 "src/form/binary_other/Divide.birch"
template<class Left,class Right>
#line 16 "src/form/binary_other/Divide.birch"
inline Divide<Left, Right> operator/(const Left& l, const Right& r) {
  #line 17 "src/form/binary_other/Divide.birch"
  return construct<Divide<Left, Right>>(l, r);
}

#line 15 "src/form/binary_other/LessThan.birch"
template<class Left,class Right>
#line 15 "src/form/binary_other/LessThan.birch"
inline decltype(auto) operator<(const Left& l, const Right& r) {
  #line 16 "src/form/binary_other/LessThan.birch"
  return construct<LessThan<Left, Right>>(l, r);
}

#line 56 "src/form/binary_other/Multiply.birch"
template<class Left,class Right>
#line 56 "src/form/binary_other/Multiply.birch"
inline Multiply<Left, Right> operator*(const Left& l, const Right& r) {
  #line 57 "src/form/binary_other/Multiply.birch"
  return construct<Multiply<Left, Right>>(l, r);
}

#line 15 "src/form/binary_other/Subtract.birch"
template<class Left,class Right>
#line 15 "src/form/binary_other/Subtract.birch"
inline Subtract<Left, Right> operator-(const Left& l, const Right& r) {
  #line 16 "src/form/binary_other/Subtract.birch"
  return construct<Subtract<Left, Right>>(l, r);
}

#line 24 "src/math/eigen.birch"
template<class Type>
#line 24 "src/math/eigen.birch"
libbirch::DefaultArray<Type,1> operator+(const libbirch::DefaultArray<Type,1>& x) {
  #line 25 "src/math/eigen.birch"
  return x;
}

#line 31 "src/math/eigen.birch"
template<class Type>
#line 31 "src/math/eigen.birch"
libbirch::DefaultArray<Type,2> operator+(const libbirch::DefaultArray<Type,2>& X) {
  #line 32 "src/math/eigen.birch"
  return X;
}

#line 15 "src/form/unary_other/Negate.birch"
template<class Middle>
#line 15 "src/form/unary_other/Negate.birch"
decltype(auto) operator-(const Middle& m) {
  #line 16 "src/form/unary_other/Negate.birch"
  return construct<Negate<Middle>>(m);
}

}

#endif
